#!/bin/bash
# description: Init script rb-workers
# chkconfig: 2345 96 14
# description: daemon control script
#

lockfile="/var/lock/rb-workers.lck"
locked=0
subsysfile="/var/lock/subsys/rb-workers"

mkdir -p /var/www/rb-rails/tmp/delayed_job
chown -R webui:webui /var/www/rb-rails/tmp/delayed_job
touch /var/www/rb-rails/log/production.log
touch /var/www/rb-rails/log/delayed_job.log
chown webui:webui /var/www/rb-rails/log/production.log /var/www/rb-rails/log/delayed_job.log

if [ -f $lockfile ]; then
    PIDLOCK=$(<$lockfile)
    if [ "x$PIDLOCK" != "x" ]; then
        if [ ! -f /proc/$PIDLOCK/cmdline ]; then
            echo "The init script rb-workers looks locked but the 'lock proccess' is not running. Unblocking it!!"
            rm -f $lockfile
            PIDLOCK=""
            locked=0
        else
            strings /proc/$PIDLOCK/cmdline | grep -q "rb_workers"
            if [ $? -ne 0 ]; then
                echo "The init script rb-workers looks locked but the 'lock proccess' is not running. Unblocking it!!"
                rm -f $lockfile
                locked=0
                PIDLOCK=""
            elif [ "x$1" == "xstatus" ]; then
                locked=1
            else
                echo "The lock file $lockfile exist (PID: $PIDLOCK) ... exiting!"
                exit 0
            fi
        fi
    elif [ "x$1" == "xstatus" ]; then
        locked=1
    fi
fi

if [ "x$1" != "xstatus" ]; then
	trap "rm -f $lockfile" 0 2 5 15
	echo $$ > $lockfile
fi

# RHEL rc script library
. /etc/rc.d/init.d/functions
source /etc/profile
rvm gemset use web &>/dev/null

USER=webui

RETVAL=0

function f_start() {
    PID0=$(pidof delayed_job.0)
    PID1=$(pidof delayed_job.1)
    if [ "x$PID0" == "x" -o "x$PID1" == "x" ]; then
        [ "x$PID0" != "x" -o "x$PID1" != "x" ] && f_stop
        rm -f /var/www/rb-rails/tmp/pids/delayed_job*.pid
        pushd /var/www/rb-rails/ &>/dev/null
        bundle exec rake redBorder:start_worker
        popd &>/dev/null

        PID0=$(pidof delayed_job.0)
        PID1=$(pidof delayed_job.1)
        if [ "x$PID0" != "x" ]; then
            renice -n 15 -p $PID0 &>/dev/null
            ionice -c 3 -p $PID0 &>/dev/null
        fi
        if [ "x$PID1" != "x" ]; then
            renice -n 15 -p $PID1 &>/dev/null
            ionice -c 3 -p $PID1 &>/dev/null
        fi

	touch $subsysfile
    else
        f_status
    fi
    PID0=$(pidof delayed_job.0)
    PID1=$(pidof delayed_job.1)
    ret=1

    [ "x$PID0" != "x" -a "x$PID1" != "x" ] && ret=0

    return $ret
}

function f_stop() {
    RETVAL=0
    PIDS=`pidof delayed_job.0 delayed_job.1 delayed_job.2 delayed_job.3`
    if [ "x$PIDS" != "x" ];then
        pushd /var/www/rb-rails/ &>/dev/null
        bundle exec rake redBorder:stop_worker
        popd &>/dev/null
        sleep 4
        for PID in $PIDS; do
            if [ -f /proc/$PID/cmdline ];then
                strings /proc/$PID/cmdline | grep -q "^delayed_job\."
                if [ $? -eq 0 ]; then
                    echo -n "* Force stop workers (killing $PID ...) "
                    kill -9 $PID &>/dev/null
                    echo_success
                    echo
        	    fi
            fi
        done
    else
        echo "rb-workers already stopped"
    fi
    rm -f /var/www/rb-rails/tmp/pids/delayed_job*.pid
    rm -f $subsysfile
}

function f_clear_jobs() {
    RETVAL=0
    PIDS=`pidof delayed_job.0 delayed_job.1 delayed_job.2 delayed_job.3`
    if [ "x$PIDS" != "x" ];then
        pushd /var/www/rb-rails/ &>/dev/null
        bundle exec rake redBorder:clear_jobs
        popd &>/dev/null
        sleep 4
    else
        echo "rb-workers is not running on this node"
    fi
}

function f_stop_all() {
    RETVAL=0
    PIDS=`pidof delayed_job.0 delayed_job.1 delayed_job.2 delayed_job.3`
    if [ "x$PIDS" != "x" ];then
        pushd /var/www/rb-rails/ &>/dev/null
        bundle exec rake redBorder:stop_worker
        bundle exec rake redBorder:clear_jobs
        popd &>/dev/null
        sleep 4
        for PID in $PIDS; do
            if [ -f /proc/$PID/cmdline ];then
                strings /proc/$PID/cmdline | grep -q "^delayed_job\."
                if [ $? -eq 0 ]; then
                    echo -n "* Force stop workers (killing $PID ...) "
                    kill -9 $PID &>/dev/null
                    echo_success
                    echo
        	    fi
            fi
        done
    else
        echo "rb-workers already stopped"
    fi
    rm -f /var/www/rb-rails/tmp/pids/delayed_job*.pid
    rm -f $subsysfile
}

function f_status() {
        if [ $locked -eq 0 ]; then
	    echo -n "rb-workers: "
	else
	    echo -n "rb-workers (blocked): "
	fi

	PID=`pidof delayed_job.0 delayed_job.1 delayed_job.2 delayed_job.3`
	if [ "x$PID" != "x" ];then
		echo -n "running (pids: $PID)"
		RETVAL=0
        echo_success
        echo
	else
		PIDS=$(ls /opt/rb/var/www/rb-rails/tmp/pids/*.pid 2>/dev/null) # This is legacy, not NG
		if [ "x$PIDS" != "x" ]; then
			PIDONLY=""
			for n in $PIDS; do
				value=$(<$n)
				if [ -f /proc/$value/cmdline ]; then
					strings /proc/$value/cmdline|grep -q "^delayed"
					if [ $? -ne 0 ]; then
						RETVAL=3
					else
						PIDONLY="$PIDONLY $value"
					fi
				else
					RETVAL=3
				fi
			done
			if [ $RETVAL -eq 0 ]; then
				echo -n "running (pids:$PIDONLY)"
                RETVAL=0
                echo_success
                echo
			else
				echo "not running propertly (pids found:$PIDONLY)"
				RETVAL=3
			fi
		else
			echo "not running"
			RETVAL=3
		fi
	fi
}


case "$1" in
    start)
	    f_start
            RETVAL=$?
        ;;
    stop)
	    f_stop
	    ;;
    clearjobs)
	    f_clear_jobs
	    ;;
    stopall)
	    f_stop_all
	    ;;
    restart)
	    f_stop
	    sleep 1
	    f_start
            ;;
    status)
        f_status
        ;;
    *)
        echo "Usage $0 {start|stop|stopall|clearjobs|status|restart}"
        exit 1
        ;;
esac

[ $locked -eq 1 ] && RETVAL=0
if [ "x$1" != "xstatus" ]; then
	rm -f $lockfile
fi
exit $RETVAL

## vim:ts=4:sw=4:expandtab:ai:nowrap:formatoptions=croqln:
