/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

$(document).on('click', 'div.user', function() {
  var element = $(this),
    id = element.data('user-id'),
    parent = element.parents("tr"),
    name = $('span', element).text(),
    imgUrl = $('img', element).attr("src"),
    foundElement = ($('tr[data-userid="' + id + '"]').length !== 0);

  parent.remove();
  if (foundElement) return false;

  var added = $('article.dashboard-form form #users_table tbody').append('\
    <tr data-userid="' + id + '" data-username="' + name + '"> \
      <td><img class="avatar-small" width="16px" height="16px" src="' + imgUrl + '"></img> ' + name + '<input type="hidden" name="dashboard[collaborators][][user_id]" value="' + id +'" /></td> \
      <td><input type="checkbox" name="dashboard[collaborators][][can_edit]" /></td> \
      <td data-action="delete" class="row-right last"><a class="btn" href="#"><i class="fa fa-trash"></i></a></td> \
    </tr> \
  ');

  added.find('tr:last-child td.icons i.fa-trash').on('click', function () {
    $(this).parents("tr").remove();
    return false;
  });

  return false;
});

var rbWidgetReloadInterval;

function toggleWidgetReloadInterval(autoreload) {
  var ref = $('.widget_reload a');
  var rbReloadingSecs = autoreload + 1;

  if (rbWidgetReloadInterval === undefined) {
    rbWidgetReloadInterval = setInterval(function () {
      ref.html('<i class="fa fa-refresh"></i>' + --rbReloadingSecs);
      if (rbReloadingSecs === 0) {
        redBorder.timeNow = redBorder.timeMachine || new Date();
        rbReloadingSecs = autoreload;
        $('.widget-toolbar a.reload i').addClass('fa-spin')
        //$('.widget-toolbar a.reload').click();
        $('.widget').each(function(index){
          if (redBorder.widgets.active["widget_" + $(this).data('widget-id')].reload !== undefined){
            var that = this;
            setTimeout(function() {
              $(that).addClass('reloading');
              redBorder.widgets.active["widget_" + $(that).data('widget-id')].reload();
            }, 1000 * index);
          }
        });
      }
    }, 1000);
  } else {
    ref.text("Stopped");
    rbReloadingSecs = autoreload;
    clearTimeout(rbWidgetReloadInterval);
    rbWidgetReloadInterval = undefined;
  }
}

function lockDashboard(nosave) {
  var dashboard_id = $("#dashboard").data("dashboard-id");
  if (dashboard_id === undefined) return;
  rbDashboardLocked = true;

  if ($('div#dashboard').hasClass('dashboard-absolute')) {
    $('#dashboard-bg img').resizable('disable');
    $('#dashboard-bg').draggable('disable');
    $('.widget').resizable({ disabled: true }).draggable({ disabled: true });
  } else {
    redBorder.widgets.gridster.disable();
    redBorder.widgets.gridster.disable_resize();
  }

  $('#dashboard').attr('data-locked', 'true');

  if(nosave === undefined || nosave === false){
    setLockedStatus('true');
  }
  $('.widget_unlock').show();
  $('.widget_lock').hide();
}

function unlockDashboard(nosave) {
  var dashboard_id = $("#dashboard").data("dashboard-id");
  if (dashboard_id === undefined) return;
  rbDashboardLocked = false;

  if ($('div#dashboard').hasClass('dashboard-absolute')) {
    $('#dashboard-bg img').resizable('enable');
    $('#dashboard-bg').draggable('enable');
    $('.widget').resizable('enable').draggable('enable');
  } else {
    redBorder.widgets.gridster.enable();
    redBorder.widgets.gridster.enable_resize();
  }

  $('#dashboard').attr('data-locked', 'false');

  if(nosave === undefined || nosave === false){
    setLockedStatus('false');
  }
  $('.widget_unlock').hide();
  $('.widget_lock').show();
}

function setLockedStatus(status) {
  var dashboard_id = $("#dashboard").data("dashboard-id");
  if (dashboard_id === undefined) return;

  $.ajax({
    url: "/dashboard/" + dashboard_id + "/set_locked_status",
    type: 'POST',
    dataType: 'script',
    data: {
      locked: status,
    }
  });
}

// Clean the current widgets when change the dashboard.
$(document).on('click', '.dashboard-name a', function () {
  redBorder.widgets.active = {};
});

function initializeDashboard(locked_status, max_rows) {
  // Create the container for dashboards vars
  redBorder.dashboard = {};

  if (!$('div#dashboard').hasClass('dashboard-columns')) {
    redBorder.dashboard.layout = 'absolute';

    $('#dashboard-bg img').resizable({
      aspectRatio: true,
      minHeight: 50,
      minWidth: 50,
      stop: function () {
        var dashboard_id = $("#dashboard").data("dashboard-id");
        $.ajax({
          url: "/dashboard/" + dashboard_id + "/set_background_position",
          type: 'POST',
          dataType: 'script',
          data: {
            width: $(this).css("width"),
            height: $(this).css("height")
          }
        });
      }
    });

    $('#dashboard-bg').draggable({
      stop: function () {
        var dashboard_id = $("#dashboard").data("dashboard-id");
        $.ajax({
          url: "/dashboard/" + dashboard_id + "/set_background_position",
          type: 'POST',
          dataType: 'script',
          data: {
            x: $(this).css("left"),
            y: $(this).css("top")
          }
        });
      }
    });
  } else {
    redBorder.dashboard.layout = 'columns';

    redBorder.widgets.gridster = $('div#dashboard.dashboard-columns').gridster({
      widget_selector: ".widget",
      widget_margins: [15, 15],
      widget_base_dimensions: [($(window).width()/8)-22, 112],
      max_cols: 8,
      min_rows: 2,
      max_rows: max_rows || 15,
      resize: {
        enabled: true,
        start: function(e, ui, $widget){
          redBorder.widgets.active["widget_"+$widget.attr('data-widget-id')].hide();
        },
        stop: function(e, ui, $widget){
          redBorder.widgets.active["widget_"+$widget.attr('data-widget-id')].resize();
          redBorder.widgets.active["widget_"+$widget.attr('data-widget-id')].show();

          // Save the new positions
          set_grid_positions();
        }
      },
      draggable: {
        stop: function(e, ui){
          // Save the current positions
          set_grid_positions();
        }
      }

    }).data('gridster');
  }

  //$(window).resize();
  if (locked_status) lockDashboard(true);
  else unlockDashboard(true);

  if($('.ui-wrapper').css("height") === "0px") {
    $('.ui-wrapper, .ui-wrapper img').css("height", "auto");
  }
}


function set_grid_positions(){
  var positions = [];
  if (rbDashboardLocked == true) return;

  $('div#dashboard.dashboard-columns .widget').each(function () {
    var position_x = $(this).attr("data-col"),
        position_y = $(this).attr("data-row"),
        position_width = $(this).attr("data-sizex"),
        position_height = $(this).attr("data-sizey"),
        widget_id = $(this).attr("data-widget-id")

    positions.push({
      id: widget_id,
      position_x: position_x,
      position_y: position_y,
      position_width: position_width,
      position_height: position_height
    });
  });

  var dashboard_id = $('#dashboard').data('dashboard-id');

  $.ajax({
    url: "/dashboard/"+ dashboard_id +"/widgets/set_position/",
    type: 'POST',
    dataType: 'script',
    data: {
      data: JSON.stringify(positions)
    }
  }).fail(function(data){
    show_alert("error", I18n.t('widget_privileges'));
    lockDashboard(true);
  });
}

// Listener to delete the current image
$(document).on("click", "#remove_current_image", function(event){
  event.preventDefault();
  $(".dashboard-img").html('<div class="control-group file optional dashboard_image"><label class="file optional control-label" for="dashboard_image">Background Image (Max. 10 MB)</label><div class="controls"><a class="file-input-wrapper btn btn-default  file optional btn">Search for a file to add<input class="file optional btn" id="dashboard_image" name="dashboard[image]" title="' + I18n.t('search_file') + '" type="file" data-bfi-disabled="" style="left: -50px; top: -7px;"></a><input name="dashboard[delete_image]" style="display:none" value="true"></div></div>');
});

// Listener to delete user from colaborative table
$(document).on("click", "article.dashboard-form table#users_table td[data-action=delete] a", function(event){
  event.preventDefault();
  var element = $(this),
  parent = element.parents("tr");

  parent.remove();
});

// Listener to delete domain from colaborative table
$(document).on("click", "article.dashboard-form table#sensors_table td[data-action=delete] a", function(event){
  event.preventDefault();
  var element = $(this),
  parent = element.parents("tr");

  parent.remove();
});
