/*!   
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.expandableTables = {}
redBorder.expandableTable = function(opts) {

  // Defalt Values
  opts = opts || {};
  opts.table = opts.table || "expandable-table";

  // Variables
  var $table = $("." + opts.table),
      $roots = $table.find("tr"),
      $childs;


  // Helpers
  var helpers = {
    initRoots: function () {
      $roots.addClass("expandable");
      $roots.each(function(i){ $(this).attr("data-expandable-root", i) });
    },
    addExpandableTableClass: function() {
      $table.addClass("expandable-table");
    },
    setExpandIcon: function() {
      var icon = "<i class='expand-icon fa fa-play'>&nbsp;</i>";
      $roots.find("td:first-child").prepend(icon);
    },
    reloadTable: function() {
      $table = $("." + opts.table);
      $roots = $table.find("tr.expandable");
    },
    getChilds: function($row) {
      if($row == undefined) return;
      helpers.reloadTable();
      var rootIndex = $row.data("expandable-root");
      return $table.find("tr[data-expandable-parent=" + rootIndex +"]");
    }
  }

  // Functions

  var setCallbacks = function () {
    $(document).on("click", ".expand-icon", function(e){
      var self = $(this),
          $row = self.parent().parent();

      var generateChilds = function(){
        self.switchClass('fa-play', 'fa-spinner fa-pulse');
        $.ajax({
          url: opts.url,
          type: "GET",
          dataType: "html",
          data: {
            value: $row.data("expand-category")
          }
        }).done(function(html){
          self.switchClass('fa-spinner fa-pulse', 'fa-play');
          var $html = $(html).filter(function(i) { return $(this).is("tr") });
          $html.find("td").addClass("not-expandable");
          $html.each(function(i){ 
            $(this).attr("data-expandable-parent", $row.data("expandable-root"));
          });
          $row.addClass("already-expanded");
          $row.after($html);
          self.toggleClass('icon-rotated');
          if (opts.callback != undefined)
            opts.callback();
        });
      }

      if (self.hasClass('icon-rotated')) {
        helpers.getChilds($row).hide();
        self.removeClass('icon-rotated');
      } else {
        // is Not Expanded
        if ($row.hasClass("already-expanded")){
          self.addClass('icon-rotated');
          helpers.getChilds($row).show();
        } else {
          generateChilds();
        }
      }
    });
  }

  var destroyCallbacks = function() {
    $(document).off("click", ".expand-icon");
  }

  var initialize = function() {
    helpers.addExpandableTableClass();
    helpers.initRoots();
    helpers.setExpandIcon();
    // make sure old listener are deleted
    destroyCallbacks();
    setCallbacks();
  }

  // Initialize
  initialize();
}