
/*!
 * Copyright (c) 2024 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

document.addEventListener('DOMContentLoaded', function () {
    var buttons = document.getElementsByClassName('view-details-btn');

    for (var i = 0; i < buttons.length; i++) {
        buttons[i].addEventListener('click', function () {
            var rule = this.getAttribute('data-rule');

            redBorder.modals.showModalWith(
                'Rule',
                rule,
                redBorder.modals.dismissButton('Back'),
                '#default_modal'
            );
        });
    }
});

function escapeSelector(selector) {
    return selector.replace(/([.*+?^${}()|[\]\\])/g, '\\$1');
}

function populateTable(data) {
    var tbody = document.getElementById('technique-table-body');
    if (tbody) {
        tbody.innerHTML = '';

        for (var i = 0; i < data.length; i++) {
            var category = data[i];
            var totalRows = category.techniques.length;
            var totalSubtechniques = 0;

            for (var j = 0; j < category.techniques.length; j++) {
                totalSubtechniques += category.techniques[j].subtechniques.length;
            }

            var categoryRow = document.createElement('tr');
            categoryRow.innerHTML = '<td rowspan="' + (totalRows + totalSubtechniques + 1) + '"><b>' + category.tactic + '</b></td>';
            tbody.appendChild(categoryRow);

            for (var k = 0; k < category.techniques.length; k++) {
                var technique = category.techniques[k];
                var techniqueRow = document.createElement('tr');
                techniqueRow.className = technique.technique_id;
                var techniqueHasAssignedRule = technique.rule_ids.indexOf(window.ruleId) > -1;
                var hasSubtechniques = technique.subtechniques.length > 0;
                techniqueRow.innerHTML = '<td>' +
                    (hasSubtechniques ?
                        '<button class="expand-icon fa fa-chevron-right" onclick="toggleSubtechniques(\'' + technique.technique_id + '\')"></button>' +
                        '<span class="expand-button" onclick="toggleSubtechniques(\'' + technique.technique_id + '\')"> ' + technique.technique_name + ' (ID: ' + technique.technique_id + ')</span>'
                        :
                        technique.technique_name + ' (ID: ' + technique.technique_id + ')') +
                    '</td>' +
                    '<td></td>' +
                    '<td> <i style="color:' + (techniqueHasAssignedRule ? 'green;' : 'red;') + '" class="fa ' + (techniqueHasAssignedRule ? 'fa-check deassign-button' : 'fa-close assign-button') + '"></i> </td>' +
                    '<td>' +
                        '<button class="btn btn-mini assign-tech-rule-btn-mitre" onclick="' + (!techniqueHasAssignedRule ? 'assignTask' : 'deassignTask') + '(\'' + technique.technique_id + '\')">' +
                            '<i class="fa ' + (!techniqueHasAssignedRule ? 'fa-plus deassign-button' : 'fa-trash assign-button') + '"></i>' +
                        '</button>' +
                    '</td>';

                tbody.appendChild(techniqueRow);

                for (var l = 0; l < technique.subtechniques.length; l++) {
                    var subtechnique = technique.subtechniques[l];
                    var subtechniqueRow = document.createElement('tr');
                    var subtechniqueHasAssignedRule = subtechnique.rule_ids.indexOf(window.ruleId) > -1;
                    subtechniqueRow.classList.add('subtechnique-row', technique.technique_id, subtechnique.subtechnique_id);
                    subtechniqueRow.innerHTML = '<td></td>' +
                        '<td><a href="#' + subtechnique.subtechnique_id + '">' + subtechnique.subtechnique_name + ' (ID: ' + subtechnique.subtechnique_id + ')</a></td>' +
                        '<td> <i style="color:' + (subtechniqueHasAssignedRule ? 'green;' : 'red;') + '" class="fa ' + (subtechniqueHasAssignedRule ? 'fa-check deassign-button' : 'fa-close assign-button') + '"></i> </td>' +
                        '<td>' +
                            '<button class="btn btn-mini assign-tech-rule-btn-mitre" onclick="' + (!subtechniqueHasAssignedRule ? 'assignTask' : 'deassignTask') + '(\'' + subtechnique.subtechnique_id + '\')">' +
                                '<i class="fa ' + (!subtechniqueHasAssignedRule ? 'fa-plus deassign-button' : 'fa-trash assign-button') + '"></i>' +
                            '</button>' +
                        '</td>';
                    tbody.appendChild(subtechniqueRow);
                }
            }
        }
    }
}

function toggleSubtechniques(id) {
    var escapedId = escapeSelector(id);
    var rows = document.querySelectorAll('.subtechnique-row.' + escapedId);
    var icon = document.querySelector('.expand-icon[onclick="toggleSubtechniques(\'' + id + '\')"]');

    for (var i = 0; i < rows.length; i++) {
        var row = rows[i];
        if (row.style.display === 'none' || row.style.display === '') {
            row.style.display = 'table-row';
            if (icon) {
                icon.classList.remove('fa-chevron-right');
                icon.classList.add('fa-chevron-down');
            }
        } else {
            row.style.display = 'none';
            if (icon) {
                icon.classList.remove('fa-chevron-down');
                icon.classList.add('fa-chevron-right');
            }
        }
    }
}

function updateUIAfterToggle(element, action) {
    var icon = element.querySelector('i.fa');
    var button = element.querySelector('button.btn-mini.assign-tech-rule-btn-mitre');

    var newMitreId = element.className.split(" ");
    newMitreId = newMitreId[newMitreId.length - 1];

    if (action === 'assign') {
        icon.classList.replace('fa-close', 'fa-check');
        icon.style.color = 'green';
        button.querySelector('i').classList.replace('fa-plus', 'fa-trash');
        button.setAttribute('onclick', 'deassignTask(\'' + newMitreId + '\')');
    } else if (action === 'deassign') {
        icon.classList.replace('fa-check', 'fa-close');
        icon.style.color = 'red';
        button.querySelector('i').classList.replace('fa-trash', 'fa-plus');
        button.setAttribute('onclick', 'assignTask(\'' + newMitreId + '\')');
    }
}

function deassignTask(id) {
    $.ajax({
        type: 'POST',
        url: '/mitre/' + window.tacticId + '/deassign_rule_from_mitre',
        data: { rule_id: window.ruleId, mitre_id: id },
        success: function (response) {
            show_alert("success", "Mitre rule has been saved");
            updateUIAfterToggle(document.getElementsByClassName(id)[0], "deassign");
        },
        error: function (xhr, status, error) {
            alert('Error assigning rule ' + id + ': ' + error);
        }
    });
}

function assignTask(id) {
    // Autoassign the root technique if is a subtechnique
    if (id.includes('.')) {
        const technique = id.split('.')[0];
        return assignTask(technique).then(function() {
            return assignTaskAjaxCall(id);
        });
    }
    return assignTaskAjaxCall(id);
}

function assignTaskAjaxCall(id) {
    return $.ajax({
        type: 'POST',
        url: '/mitre/' + window.tacticId + '/assign_rule_to_mitre',
        data: { rule_id: window.ruleId, mitre_id: id },
        success: function (response) {
            show_alert("success", "Mitre rule has been saved");
            updateUIAfterToggle(document.getElementsByClassName(id)[0], "assign");
        },
        error: function (xhr, status, error) {
            alert('Error assigning rule ' + id + ': ' + error);
        }
    });
}

function mitreToTable(mitre_tactic_id) {
    fetch('/mitre/' + mitre_tactic_id + '/json_mitre_techniques')
        .then(function (response) { return response.json(); })
        .then(function (data) { populateTable(data.sort()); })
        .catch(function (error) { console.error('Error fetching data:', error); });
}

$(document).ready(function () {
    $('#assignRuleModal').one('hidden.bs.modal', function () {
        $('#modal-rule-id').val('');
        $('#modal-rule-id').modal('hide');
    });

    $('.config-rule-btn-mitre').on('click', function (e) {
        e.preventDefault();
        window.ruleId = $(this).data('rule-id');
        redBorder.modals.showModalWith(
            'Mitre technique assignation',
            window.mitre_html_modal,
            redBorder.modals.dismissButton('Back'),
            '#assignRuleModal'
        );
    });

    $('.deassign-rule-btn-mitre').on('click', function (e) {
        e.preventDefault();
        window.ruleId = $(this).data('rule-id');
        $.ajax({
            type: 'POST',
            url: '/mitre/' + window.tacticId + '/deassign_tactic_from_rule',
            data: { rule_id: window.ruleId, mitre_tactic_id: window.tacticId  },
            success: function (response) {
                show_alert("success", response.message);
                setTimeout(function(){
                    location.reload();
                },1e3)
            },
            error: function (xhr, status, error) {
                show_alert("error", error);
            }
        });
    });

    $('.assign-rule-btn-mitre').on('click', function (e) {
        e.preventDefault();
        window.ruleId = $(this).data('rule-id');
        $.ajax({
            type: 'POST',
            url: '/mitre/' + window.tacticId + '/assign_tactic_to_rule',
            data: { rule_id: window.ruleId, mitre_tactic_id: window.tacticId  },
            success: function (response) {
                show_alert("success", response.message);
                setTimeout(function(){
                    location.reload();
                },1e3)
            },
            error: function (xhr, status, error) {
                show_alert("error", error);
            }
        });
    });
});