
/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

 redBorder.animatedHeatmaps = {}

 redBorder.animatedHeatmap = function (map, opts) {

  /* Times as Miliseconds */
  var start_time = new Date(opts['start_time']*1000).setSeconds(0);
  var end_time = new Date(opts['end_time']*1000).setSeconds(0);
  var time_step = opts['step'];
  var current_time = start_time;

  var heatmap = map;
  var granularity = opts['granularity'];
  var aggregation = opts['aggregation'];
  var buffer = [];

  var is_loading = false;
  var iteration = 0;
  var time_iteration = 0;
  var force_stop = false;
  var pause = false;
  var speed = 1;

  function stop(){
    force_stop = true;
    buffer = [];
  }

  function end_animation(){
    iteration = 0;
    time_iteration = 0;
    current_time = start_time;
    heatmap.setOptions({draggable: true, panControl: true});
    $('.fa-pause').addClass("fa-play");
    $('.fa-pause').removeClass("fa-pause");
  }

  function update_time(time){
    var d = new Date(time);
    var dd = d.getDate();
    var mm = d.getMonth()+1;
    var hh = d.getHours();
    var mmm = d.getMinutes();

    if (dd < 10) dd = '0'+ dd;
    if (mm < 10) mm = '0'+ mm;
    if (hh < 10) hh = '0'+ hh;
    if (mmm < 10) mmm = '0'+ mmm;

    var progress = (100 * time_iteration / Math.floor((end_time - start_time) / time_step));

    if (progress < 80){
      $('.bol').css({"left":""+ progress +"%"});
    } else {
      $('.bol').css({"left":""+ Math.floor(progress) +"%"});
    }

    var date = ""+ d.getFullYear() + "-" + mm + "-" + dd + " " + hh + ":" + mmm + "";
    $('.time_box').html("<i class='fa fa-clock-o'></i>"+ "  " + date);
  }

  function finish_with_no_data(flag){
    if (!pause){
      time_iteration++;
      if (flag == undefined){
        var empty = [];
        heatmap.reloadData_dynamic(empty, true);
      }
      if ((end_time - current_time) <= time_step){
        end_animation();
      } else{
      current_time = parseInt(current_time) + parseInt(time_step);
      update_time(current_time);
      setTimeout(finish_with_no_data, 40000/ (speed * Math.floor((end_time - start_time) / time_step)), true);
      }
    }
  }

  function start_animated_heatmap(){
    time_iteration++;
    if(!pause){
      update_time(current_time);

      if ( buffer[iteration][0].timestamp == new Date(current_time).toISOString()){
        heatmap.reloadData_dynamic(buffer[iteration][0].result, true);
        iteration++;
      } else{
        var empty = [];
        heatmap.reloadData_dynamic(empty, true);
      }

      if(iteration < buffer.length){
      current_time = parseInt(current_time) + parseInt(time_step);
      if (!force_stop) setTimeout(start_animated_heatmap, 40000/ (speed * Math.floor((end_time - start_time) / time_step)));
      } else{
        if ((end_time - current_time) > time_step){
          finish_with_no_data();
        } else{
        end_animation();
        }
      }
    }
  }

  function loading(num, status){
    if(status == undefined){
      $('.loading').html(I18n.t('javascript.loading') + Math.round(num) + "%");
    } else if (status){
      $('.loading').html(I18n.t('javascript.loading') + "0%");
    } else{
      $('.button_gmap').hide();
      $('.animated_player').show();
    }
  }

  function receive_data(data){
    if (data.length > 0) buffer.push(data);
    loading( (100 * iteration++) / ((end_time - start_time)/time_step) );

    var current_start = (start_time + iteration * time_step)/1000;
    var current_end = (start_time + (iteration+1) * time_step)/1000;

    if (1000*(current_start) < end_time){
      if(!force_stop) request_data( current_start, current_end);
    }
    else{
      loading(0, false);
      heatmap.setMax(Math.max.apply(Math, buffer.map(function(x){return Math.max.apply(Math, x[0].result.map(function(y){return y[aggregation]}))})));
      iteration = 0;
      time_iteration = 0;
      current_time = start_time;
    }
  }

  $(document).on('click', '.fa-pause', function() {
    if(force_stop) return;
    pause = true;
    $('.fa-pause').addClass("fa-play");
    $('.fa-pause').removeClass("fa-pause");
  });

  $(document).on('click', '.fa-forward', function() {
    if(force_stop) return;
    if ($('.acx').hasClass('fa-play')) return;
    speed = speed * 2;
    if(speed > 8) speed = 8;
    $('.speed').html("x"+speed);
  });
  $(document).on('click', '.fa-backward', function() {
    if(force_stop) return;
    if ($('.acx').hasClass('fa-play')) return;
    speed = speed / 2;
    if(speed < 1) speed = 1;
    $('.speed').html("x"+speed);
  });

  $(document).on('click', '.fa-stop', function() {
    if(force_stop) return;
    pause = true;
    $('.acx').removeClass("fa-pause");
    $('.acx').removeClass("fa-play");
    $('.acx').addClass("fa-play");
    iteration = 0;
    time_iteration = 0;
    current_time = start_time;
    update_time(current_time);
    heatmap.setOptions({draggable: true, panControl: true});
    heatmap.reloadData_dynamic(buffer[0][0].result, true);
  });

  $(document).on('click', '.fa-play', function() {
    if(force_stop) return;
    pause = false;
    heatmap.setOptions({draggable: false, panControl: false});
    $('.fa-play').addClass("fa-pause");
    $('.fa-play').removeClass("fa-play");
    if (iteration >= buffer.length && ((end_time - current_time) > time_step)){
      finish_with_no_data();
    } else{
      start_animated_heatmap();
    }
  });


  function animated_heatmap(){
    force_stop = false;
    loading(0, true);
    iteration = 0;
    time_iteration = 0;
    request_data(start_time/1000, (start_time + iteration * time_step)/1000);
  }

  function request_data(start,end){
    var url =  "/flow/get_points?start_time="+start+"&end_time="+end+"&granularity="+granularity+"&aggregation="+aggregation+"";
    $.ajax({
          url: url,
          remote: false,
          dataType: 'json',
          type: 'GET',
        }).done(function (data) {
          receive_data(data);
        });
  }

  $(document).on('click', '.gmap_button', function() {
    if(force_stop) return;
    $('.button_gmap').removeClass('gmap_button');
    $('.button_gmap').html("<i class='fa fa-spinner fa-pulse'></i>&nbsp;<span class='loading'></span>")
    animated_heatmap();
  });

  return{
    stop: stop
  }

}
