/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Helps the interaction with forms
 */

redBorder.forms = {

  /*
   * Create a hidden input that activate when a select has or hasn't a
   * value.
   */
  addDependentInput: function(selector){
    // Add a listener on the selector
    $(selector).on('change', function(event){
      var child = $($(this).attr('data-dependent-child'));
      var show_on = $(this).attr('data-show-on');
      var hide_on = $(this).attr('data-hide-on');
      var wrapper = $(this).parents('.form-group.select');
      var selected = $(this).val();

      // Show the child if agree the values.
      if(((show_on !== undefined && selected === show_on) || (hide_on !== undefined && selected !== hide_on)) && !child.hasClass('showed')){
        wrapper.animate({
          width: "-=51.002%"
        }, 200, function(n){
          wrapper.css('float', 'left');
          wrapper.css('width', '49%')
          child.fadeIn(100);
          child.addClass('showed');
        });
      } else if (((show_on !== undefined && selected !== show_on) || (hide_on !== undefined && selected === hide_on)) && child.hasClass('showed')){
        child.removeClass('showed');
        child.fadeOut(100, function(n){
          wrapper.css('float', 'none');
          wrapper.animate({
            width: "+=51.002%"
          }, 200, function(n){
            wrapper.css('width', '100%')
          });
        });
      }
    });

    // Check now the elements and show them without animation
    $(selector).trigger('change');
  },

  /*
   * Limit the granularity on widget forms based on the range
   */
  limitGranularity: function(range_selector, granularity_selector){

    /*
     *  Return the granularities based on a range
     */
    function get_granularity_list(range) {
      switch(range) {
        case "last_1_minute":
          ret = {
            "minute": I18n.t('javascript.1_minute')
          }
          break;
        case "last_2_minutes":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes')
          }
          break;
        case "last_5_minute":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes'),
            "pt5m": I18n.t('javascript.5_minutes')
          }
          break;
        case "last_15_minute":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes'),
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes')
          }
          break;
        case "last_30_minute":
        case "last_1_hour":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes'),
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes'),
            "thirty_minute": I18n.t('javascript.30_minutes')
          };
          break;
        case "last_2_hour":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes'),
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes'),
            "thirty_minute":
            I18n.t('javascript.30_minutes'),
            "hour": I18n.t('javascript.1_hour')
          };
          break;
        case "last_3_hour":
          ret = {
            "minute": I18n.t('javascript.1_minute'),
            "pt2m": I18n.t('javascript.2_minutes'),
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes'),
            "thirty_minute": I18n.t('javascript.30_minutes'),
            "hour": I18n.t('javascript.1_hour')
          };
          break;
        case "last_12_hour":
          ret = {
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes'),
            "thirty_minute": I18n.t('javascript.30_minutes'),
            "hour": I18n.t('javascript.1_hour'),
            "pt2h": I18n.t('javascript.2_hours')
          };
          break;
        case "last_24":
          ret = {
            "pt5m": I18n.t('javascript.5_minutes'),
            "fifteen_minute": I18n.t('javascript.15_minutes'),
            "thirty_minute": I18n.t('javascript.30_minutes'),
            "hour": I18n.t('javascript.1_hour'),
            "pt2h": I18n.t('javascript.2_hours'),
            "pt8h": I18n.t('javascript.8_hours')
          };
          break;
        case "last_week":
          ret = {
            "hour": I18n.t('javascript.1_hour'),
            "pt2h": I18n.t('javascript.2_hours'),
            "pt8h": I18n.t('javascript.8_hours'),
            "pt24h": I18n.t('javascript.1_day')
          };
          break;
        case 'custom':
          ret = {
            'minute': I18n.t('javascript.1_minute'),
            'pt2m': I18n.t('javascript.2_minutes'),
            'pt5m': I18n.t('javascript.5_minutes'),
            'fifteen_minute': I18n.t('javascript.15_minutes'),
            'thirty_minute': I18n.t('javascript.30_minutes'),
            'hour': I18n.t('javascript.1_hour'),
            'pt2h': I18n.t('javascript.2_hours'),
            'pt8h': I18n.t('javascript.8_hours'),
            'pt24h': I18n.t('javascript.1_day'),
            'pt120h': I18n.t('javascript.5_days'),
            'pt168h': I18n.t('javascript.1_week'),
            'pt720h': I18n.t('javascript.1_month')
          };
          break;
        default:
          ret = {
            "pt8h": I18n.t('javascript.8_hours'),
            "pt24h": I18n.t('javascript.1_day'),
            "pt120h": I18n.t('javascript.5_days'),
            "pt168h": I18n.t('javascript.1_week'),
            "pt720h": I18n.t('javascript.1_month')
          };
      }
      return ret;
    }

    $(range_selector).on('change', function(event){
      var range = $(this).val();
      var granularity = $(granularity_selector).val();
      var granularities = get_granularity_list(range);
      var options = '';
      var selected = '';

      for(var item in granularities){
        if(item === granularity){
          selected = ' selected="selected"'
        } else {
          selected = ''
        }

        options += "<option" + selected + " value='" + item + "'>"+ granularities[item] +"</option>"
      }

      $(granularity_selector).empty().append(options);
    });

    // Trigger the change
    $(range_selector).trigger('change');
  },

  /*
   * Load the aggregation on widget forms based on the selected module
   */
  loadAggregation: function(module_selector, aggregation_selector){

    $(module_selector).on('change', function(event){
      var module_active = $(this).val(),
          aggregation = $(aggregation_selector).val(),
          aggregations, options = '', selected = '';

      if (module_active === 'monitor') {
        aggregations = redBorder.modules[module_active].tabs;
      } else {
        aggregations = redBorder.modules[module_active].aggregations
      }


      for(var item in aggregations){
        if(item === aggregation){
          selected = ' selected="selected"'
        } else {
          selected = ''
        }

        var aggregation_translate = aggregations[item].name;
        if (aggregations[item].name.startsWith("tr.")) {
          aggregation_translate = I18n.t(aggregation_translate.slice(3)); //Slice 3 to remove 'tr.' from the translations 
        }
        options += "<option" + selected + " value='" + item + "'>"+ aggregation_translate +"</option>"
      }

      $(aggregation_selector).empty().append(options);
    });

    // Trigger the change
    $(module_selector).trigger('change');
  },

  /*
   * Load the filters on widget forms based on the selected module
   */
  loadFilters: function(module_selector, filter_selector, count){
    $(module_selector).on('change', function(event){
      var module_active = $(this).val();
      search_hash = redBorder['search_hash_' + module_active];

      // Load rules
      var data = $('[data-serie-position="' + count +'"] #widget_filters').val(),
        filters = undefined;

      if (data !== "" && data !== undefined) {
        filters = JSON.stringify({
          match_all: true,
          items: JSON.parse(data)
        });
      }

      $('[data-serie-position="' + count +'"] #search .rules').html("");
      redBorder.forms.loadRules(filters, module_active, null, count);
    });

    // Trigger the change
    $(module_selector).trigger('change');
  },

  /* Add a listener to the color selector to change the background */
  listenColorTo: function(selector){
    $('.color-item').on('mouseenter', function(event){
      // Get the value
      var color = $(this).attr('data-color');
      // Set the background
      $(selector).css('background-color', color);
    });

    $('.color-item').on('mouseleave', function(event){
      // Set the background
      $(selector).css('background-color', '');
    });
  },

  /* Load the current filter and create the form to add them */
  loadRules: function(filters, product_type, units, count){
    var where = '';
    window['widget_product_type_' + product_type](units);

    rule.searchUI({
      cssClass: "submit-search",
      buttonTitle: '' + I18n.t('javascript.submit_filter')
    }, function() {
      if (count !== undefined) where = '[data-serie-position="' + count +'"]'

      if (filters === undefined) {
        rule.add(where + ' #search .rules');
        rule.add(where + ' #search .rules');
      } else {
        rule.build(filters, where + ' #search .rules');
        rule.add(where + ' #search .rules');
        rule.add(where + ' #search .rules');
      }
    });
  },

  /* Set a sub-options div to show when data-check selector is active.
     @param el : .sub-options JQuery element
   */
  subCheckboxOptions: function(el){
    el.each(function() {
      var option = $(this);
      var dependent = $('#' + option.data('check'));

      if(dependent.is(':checked')){
        option.fadeIn(200);
      }

      // Add a listener
      $(document).on('change', dependent, function(){
        if(dependent.is(':checked')){
          option.fadeIn(200);
        } else {
          option.fadeOut(200);
          // Uncheck all childs
          option.find('input').each(function(index){
            // Set value for chef properties
            if($(this).attr('type') == 'checkbox'){
              $(this).prop('checked', false);
            } else if ($(this).attr('type') == 'text' || $(this).attr('type') == 'password') {
              $(this).val('');
            }
          });
        }
      });
    })
  },

  /* Check if the given array of CSS selectors are filled. It add errors to
     elements that are not filled.
     @param elements : Array of String with CSS paths
     @return boolean indicating if exists an error.
   */
  checkAllFilled: function(elements){
    var pass = true;
    var first = true;
    var mustFilled = false;

    $.each(elements, function(index, value) {
      var el = $('input[name="' + value + '"]');
      if (el.val() != ''){
        mustFilled = true;
      }
    });

    $.each(elements, function(index, value) {
      var el = $('input[name="' + value + '"]');
      if (el.val() == '' && mustFilled){
        el.parents('.form-group').addClass('error');
        pass = false;
      }
    });

    return pass;
  },

  initDateRange: function ($selector, options) {
    $selector.dateRangeSlider({
      bounds: {
        min: options.bounds.min,
        max: options.bounds.max
      },
      defaultValues: {
        min: options.defaults.min,
        max: options.defaults.max
      },
      arrows:false,
      formatter:function(val){
        var days = val.getDate(),
          month = val.getMonth();
        return days + " " + MONTHS[month];
      },
      scales: [{
        next: function(value){
          var next = new Date(value);
          return new Date(next.setMonth(value.getMonth() + 1));
        },
        label: function(value){
          return MONTHS[value.getMonth()];
        },
        format: function(tickContainer, tickStart, tickEnd){
          tickContainer.addClass("myCustomClass");
        },
      }],
      step: {
        days: 1
      },
      wheelMode: "scroll",
      wheelSpeed: 0.1
    });
  }
}
