/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Overview widgets
 */

redBorder.overview = {}

// Set a listener to change between access points and zones
$(document).on('click', '.toggle-aps', function(event) {
  event.preventDefault();

  if (redBorder.overview.current === undefined) { return; }

  redBorder.overview.toggleAps($(this));
});

// Change the range
$(document).on('click', '.overview-time a', function(event) {
  event.preventDefault();
  // Update URL
  var path = updateURLParameter(window.location.href, 'range', $(this).data('range'));
  if ($(this).data('floor')) {
    path = updateURLParameter(path, 'floor_uuid', $(this).data('floor'));
  }
  Turbolinks.visit(path);
});

redBorder.overview.toggleAps = function($button, nocenter) {
  // Wait query of APs
  if (redBorder.overview.current.aps === undefined) {
    setTimeout(toggleAps($button), 200);
    return;
  }

  if (nocenter === undefined) { nocenter = true; }

  // Toggle elements
  if ($button.data('aps')){
    redBorder.overview.current.imap.removeAccessPoints();
    if (redBorder.overview.current.domains.length > 0) {
      // Remove access points
      redBorder.overview.current.imap.addDomains(
        redBorder.overview.current.domains);
    }
    // Change button
    $button.data('aps', false);
    $button.html('<i class="fa fa-wifi"></i>' + I18n.t('javascript.show_aps'));
  } else {
    // Remove previous
    redBorder.overview.current.imap.removeDomains();
    // Add access points
    if (redBorder.overview.current.aps.length > 0) {
      // Add Access points
      redBorder.overview.current.imap.addAccessPoints(
        redBorder.overview.current.aps, nocenter);
    }
    // Change button
    $button.data('aps', true);
    $button.html('<i class="fa fa-map-marker"></i>' + I18n.t('javascript.show_domains'));
  }
}

// Autoreload functionality
redBorder.overview.start_reload = function() {
  // Clear old interval
  clearInterval(redBorder.overview.reload_interval);
  // Add a new interval
  redBorder.overview.reload_interval =
    setInterval(function(){
      // Update time
      redBorder.timeNow = redBorder.timeMachine || new Date();

      // Check if we are in the view
      if ($("section.overview").length === 0) {
        clearInterval(redBorder.overview.reload_interval);
        return;
      }

      // Set the counter to number of widgets
      redBorder.overview.current.reload_count =
        Object.keys(redBorder.overview.current.widgets).length;
      $('.overview-reload').removeClass('hidden');

      for (var w in redBorder.overview.current.widgets) {
        if (!redBorder.overview.current.widgets.hasOwnProperty(w)) {
          //The current property is not a direct property of p
          continue;
        }
        // Reload
        redBorder.overview.current.widgets[w].reload();
      }

      // Update current interval
      var end_time = redBorder.timeNow,
          start_time = new Date((end_time.getTime() - redBorder.overview.interval_time))
      $('.overview-current-interval span').html(
        format_time(start_time, { bold: true }) + ' - ' +
        format_time(end_time, { bold: true }));

      // Reload access points
      var id = $("section.overview").data('sensor-id');

      $.ajax({
        url: '/sensors/' + id + '/overview_aps',
        type: 'GET',
        dataType: 'json'
      }).then(function (data) {
        // If we will calculate mean, return data
        if (data.query) {
          // Store new aps
          redBorder.overview.current.aps = data.aps;
          // Display it
          if ($('.toggle-aps').length === 0 || $('.toggle-aps').data('aps')) {
            redBorder.overview.current.imap.updateAccessPoints(data.aps);
          }
        }
      });
    }, redBorder.overview.reload_time);
}

redBorder.overview.reloaded = function() {
  // Decrease
  --redBorder.overview.current.reload_count;

  if (redBorder.overview.current.reload_count === 0){
    $('.overview-reload').addClass('hidden');
  }
}

redBorder.overview.onChangeFloor = function(floor) {
  // Sensor to filter
  var sensor;
  // Clear reload
  clearInterval(redBorder.overview.reload_interval);
  // Update time
  redBorder.timeNow = redBorder.timeMachine || new Date();
  // Set the counter to number of widgets
  redBorder.overview.current.reload_count =
    Object.keys(redBorder.overview.current.widgets).length;
  $('.overview-reload').removeClass('hidden');
  // Update filters
  if (floor === 'all') {
    sensor = {
      id: redBorder.overview.current.sensor.id,
      name: redBorder.overview.current.sensor.name,
      domain_type_name: redBorder.overview.current.sensor.domain_type_name,
      uuid: redBorder.overview.current.sensor.uuid
    }
  } else {
    sensor = {
      id: floor.id,
      name: floor.name,
      domain_type_name: floor.domain_type_name,
      uuid: floor.uuid
    }
  }

  for (var widget in redBorder.overview.current.widgets) {
    if (redBorder.overview.current.widgets.hasOwnProperty(widget)) {
      // Update
      redBorder.overview.current.widgets[widget].updateSensor(
        sensor.id,
        sensor.name,
        sensor.domain_type_name,
        sensor.uuid
      )
      // Force reload
      redBorder.overview.current.widgets[widget].reload();
    }
  }
  // Start reload
  redBorder.overview.start_reload();
}

redBorder.overview.onClickDomain = function(id) {
  // Update the URL
  var path = '/dashboard/sensors/' + id + '/overview';
  Turbolinks.visit(path)
}

// Widgets for this view
redBorder.overview.widgets = {}

/*
 * Utils methods for overview widgets
 */
redBorder.overview.widgets.utils = {
  /*
   * Get the filters
   */
  getFilters: function(root, sensor_name, sensor_uuid, sensor_type, predefined_filters) {
    var filters,
        next
        predefined_filters = predefined_filters || [];
    // Fill filters
    if (root) {
      filters = {};
      next = 0;
    } else {
      filters = {
        0: {
          column: sensor_type + '_uuid',
          value: sensor_uuid,
          operator: 'is'
        }
      }
      // Set the next filters
      next = 1;
    }

    for(var i = 0, l = predefined_filters.length; i < l; i++) {
      filters[next] = predefined_filters[i];
      next++;
    }

    // Return filters
    return filters;
  },
  /*
   * Get interval length
   */
  intervalLength: function(range) {
    var time;

    switch (range) {
      case 'last_1_hour':
        time = (1).hours();
        break;
      case 'last_36':
        time = (36).hours();
        break;
      default:
        time = (12).hours();
        break;
    }

    return time;
  },
  /*
   * Stub data to 0
   */
  stubData: function() {
    return 0;
  },
  /*
   * Check if we need to ignore the load of the widget based on domain_type
   */
  isIgnored: function(sensor_name, sensor_type, ignore_domain_types) {
    return sensor_name !== 'root' && ignore_domain_types.length > 0 &&
             $.inArray(sensor_type, ignore_domain_types) >= 0;
  },
  /*
   * Return the EndTime
   */
  getEndTimeSeconds: function(range, step) {
    // Store
    end_time_secs = parseInt(redBorder.timeNow.getTime());

    // Round for graphs
    var minute = (1).minutes();
    end_time_secs = new Date(Math.floor(end_time_secs / minute) * minute);
    if (step <= (5).minutes()) {
      end_time_secs = new Date(end_time_secs.getTime() - step);
    }

    return parseInt(end_time_secs.getTime() / 1000);
  },
  /*
   * Get the domain for graphs
   */
  getDomain: function(range, step) {
    return [new Date(this.getStartTimeSeconds(range, step) * 1000),
            new Date(this.getEndTimeSeconds(range, step) * 1000)];
  },
  /*
   * Return startTime based on range
   */
  getStartTimeSeconds: function(range, step) {
    // Store
    start_time_secs =
      parseInt((redBorder.timeNow.getTime() -
        redBorder.overview.widgets.utils.intervalLength(range)));

    if (step <= (5).minutes()){
      coef = step;
    } else {
      coef = (1).minutes();
    }

    start_time_secs = new Date(Math.ceil(start_time_secs / coef) * coef);

    // Return in seconds
    return parseInt(start_time_secs.getTime() / 1000);
  },
  /*
   * Return more_info url
   */
  getMoreInfoURL: function(more_info_url, root, range, aggregation, more_info_filters, sensor_name, sensor_uuid, sensor_type, predefined_filters) {
    var params;
    // Include filters
    if (more_info_filters) {
      var filters = redBorder.overview.widgets.utils.getFilters(
        root, sensor_name, sensor_uuid, sensor_type, predefined_filters);
      var preffix = (more_info_url.indexOf("?") > -1) ? "&" : "?";
      params = preffix + "range=" + range + "&aggregation=" + aggregation +
               "&filters=" +  encodeURIComponent(JSON.stringify(filters)) +
               "&aggregation_action=replace";
    } else {
      params = "";
    }

    return more_info_url + params;
  },
  /*
   * Round a value to 1 decimal
   */
  round1: function(value) {
    return (Math.round(value * 10) / 10);
  },
  /*
   * Get the granularity
   */
  getGranularity: function(mean, isGraph, range) {
    var gran;

    if (mean || isGraph) {
      switch (range) {
        case 'last_1_hour':
          gran = 'pt5m';
          break;
        default:
          gran = 'hour';
          break;
      }
    } else {
      gran = 'all';
    }

    return gran;
  },
  /*
   * Get the step of the graph
   */
  getStep: function(range) {
    var step;
    switch (range) {
      case 'last_1_hour':
        step = 300;
        break;
      default:
        step = 3600;
    }
    return step * 1000;
  }
}



/*
 * Value widget
 */
redBorder.overview.widgets.value = function(container, opts) {
  var $container = $(container),
      $moreinfo = $(container).find('.owidget-more'),
      container = container,
      aggregation = opts.aggregation,
      aggregation_name = opts.aggregation_name || opts.aggregation,
      range = opts.range,
      module_name = opts.module_name,
      sensor_name = opts.sensor_name,
      sensor_uuid = opts.sensor_uuid,
      sensor_type = opts.sensor_type,
      ignore_domain_types = opts.ignore_domain_types || [],
      more_info_url = opts.more_info_url,
      more_info_filters = (opts.more_info_filters === true || opts.more_info_filters === undefined),
      predefined_filters = (opts.predefined_filters) || [],
      root = opts.root,
      mean = (opts.mean === true),
      reloaded = false,
      total, // Last total value (used in drawComparative)
      url,
      start_time_secs,
      end_time_secs,
      id = Math.random().toString(36).slice(2),// ID of the graph
      graph, // Graph element
      step;

  initialize();

  /*
   * Initialize some data like URL
   */
  function initialize() {
    // Create the URL. After we need to change :start_time and :end_time with
    // the proper values
    url = '/' + module_name + '/tops/get_serie/raw';
    // Update more info URL
    if ($moreinfo.length > 0 && more_info_url !== undefined) {
      // Update more info URL
      $moreinfo.attr('href',
        redBorder.overview.widgets.utils.getMoreInfoURL(
          more_info_url, root, range, aggregation, more_info_filters,
          sensor_name, sensor_uuid, sensor_type, predefined_filters));
    } else if ($moreinfo.length > 0) {
      $moreinfo.remove();
    }
    // Add containers
    appendContainers();
    // Load step
    step = redBorder.overview.widgets.utils.getStep(range);
    // Init graph
    initGraph();
  }

  /*
   * Create containers needed to show the data
   */
  function appendContainers() {
    $container.addClass('owidget-value');
    var html = '<div class="text-center">' +
                 '<span class="owidget-total">-</span>' +
                 '<span class="owidget-units"></span>' +
               '</div>' +
               '<div id="owidget-graph-' + id + '" class="owidget-graph"></div>';
    $container.append(html);
  }

  /*
   * Initialize the graph
   */
  function initGraph() {
    // Load secs
    end_time_secs = redBorder.overview.widgets.utils.getEndTimeSeconds(range, step);
    start_time_secs = redBorder.overview.widgets.utils.getStartTimeSeconds(range, step);

    graph = new RB.Serie({
      container: 'owidget-graph-' + id,
      autoscaling: true,
      step: step,
      domain: [start_time_secs, end_time_secs],
      margin: { top: 0, left: 0, right: 0, bottom: 0 },
      zoomable: false,
      autorender: false,
      units: aggregation_name,
      predict_area: false,
      hide_axises: true,
      tooltip: false
    });
  }

  /*
   * Call getData and Draw methods
   */
  function render() {
    if (redBorder.overview.widgets.utils.isIgnored(
        sensor_name, sensor_type, ignore_domain_types)) {
      draw(redBorder.overview.widgets.utils.stubData());
    } else {
      getData(false).then(draw, function(){
        if (reloaded) { redBorder.overview.reloaded(); }
        console.log('Error retrieving data');
      });
      // Draw graph
      getData(true).then(drawGraph, function() {
        console.log('Error retrieving graph data');
      })
    }
  }

  /*
   * Fetch data from the Manager. This method return a promise object
   */
  function getData(isGraph) {
    var filters = redBorder.overview.widgets.utils.getFilters(
      opts.root, sensor_name, sensor_uuid, sensor_type, predefined_filters);
    // Load secs
    end_time_secs = redBorder.overview.widgets.utils.getEndTimeSeconds(range, step);
    start_time_secs = redBorder.overview.widgets.utils.getStartTimeSeconds(range, step);
    // Set the data
    return $.ajax({
      url: url,
      type: 'GET',
      dataType: 'json',
      data: {
        granularity: redBorder.overview.widgets.utils.getGranularity(mean, isGraph, range),
        aggregation: aggregation,
        w: true,
        filters: JSON.stringify(filters),
        end_time: end_time_secs,
        start_time: start_time_secs
      }
    }).then(function (data) {
      // If we will calculate mean, return data
      if (mean || isGraph) return $.Deferred().resolve(data);
      // Else, perform some transformations
      if (data.length === 0) return $.Deferred().resolve(0);
      var time = end_time_secs - start_time_secs;
      data = redBorder.widgets.util.checkPerSecondAggregationTotal(
        data, aggregation, time);
      return $.Deferred().resolve(data[0].event.events);
    });
  }

  /*
   * Draw the widget
   */
  function draw(data) {
    var $data = $container.find('.owidget-total'),
        $units = $container.find('.owidget-units');

    // Get the data
    result = processData(data);

    // Store total
    total = result;

    // Update
    $data.html(result.value);
    $units.html(result.units);

    // Update reload count
    if (reloaded) { redBorder.overview.reloaded(); }
  }

  /*
   * Draw the graph
   */
  function drawGraph(data) {
    if (reloaded) graph.remove('Total');
    graph.add({
      data: data,
      name: 'Total',
      color: RB.Common.palette[0],
      type: 'line'
    });
    // Start
    graph.startRender();
  }
  /*
   * Reload the widget
   */
  function reload() {
    reloaded = true;
    // Update graph too
    graph.forceDomain(redBorder.overview.widgets.utils.getDomain(range, step), true);
    // Render the data
    render();
  }

  /*
   * Process the data to get the total.
   */
  function processData(data) {
    var multiplier, value, prefix, units, meanData, initial;
    // Store total!
    initial = data;

    if (mean) {
      meanData = getMeanData();

      var l = data.length,
          sum = 0;

      // Calc mean based on current data
      for(var i = 0; i < l; i++){
        sum += data[i].event.events;
      }

      // Round total to first decimal
      initial = redBorder.overview.widgets.utils.round1(sum / meanData.count);
    }

    // Get values
    multiplier = redBorder.widgets.util.getMultiplier(initial),
    value = Math.round(initial * multiplier.num * 100) / 100,
    prefix = multiplier.prefix;

    if (mean) {
      units = prefix + aggregation_name + ' / ' + meanData.suffix;
    } else {
      units = prefix + aggregation_name;
    }

    // Return an object
    return {
      value: value,
      units: units,
      initial: initial,
      multiplier: multiplier
    }
  }

  /*
   * Update sensor to filters
   */
  function updateSensor(id, name, type_name, uuid) {
    sensor_name = name;
    sensor_type = type_name;
    sensor_uuid = uuid;
  }

  /*
   * Get the number of intervals based on the range
   */
  function getMeanData() {
    var count, suffix;

    switch (range) {
      case 'last_1_hour':
        // For last hour, the step is 5 minutes
        count = 12;
        suffix = 'min';
        break;
      case 'last_36':
        count = 36;
        suffix = 'hour';
        break;
      default:
        count = 12;
        suffix = 'hour';
        break;
    }

    return {
      count: count,
      suffix: suffix
    }
  }

  // Return
  return {
    render: render,
    reload: reload,
    updateSensor: updateSensor
  }
}

// GMaps Places
  function initGmapPlaces(placesSearchBox) {
    var map = redBorder.overview.current.map.getMap();
    // Bias the SearchBox results towards current map's viewport.
    map.addListener('bounds_changed', function() {
      placesSearchBox.setBounds(map.getBounds());
    });

    // Listen for the event fired when the user selects a prediction and retrieve
    // more details for that place.
    placesSearchBox.addListener('places_changed', function() {
      var places = placesSearchBox.getPlaces();

      if (places.length === 0) {
        return;
      }

      // For each place, get the icon, name and location.
      var bounds = new google.maps.LatLngBounds();

      // For each place, get the icon, name and location.
      places.forEach(function(place) {
        if (place.geometry.viewport) {
          // Only geocodes have viewport.
          bounds.union(place.geometry.viewport);
        } else {
          bounds.extend(place.geometry.location);
        }
      });
      map.fitBounds(bounds);
    });
  }

// SearchBox for OverView
redBorder.overview.SearchBox = function (root, searchType) {
  // Init variables
  var input = document.getElementById("overview-search"),
    placesSearchBox,
    domainsSearchBox;

  function disablePlacesSearch() {
    google.maps.event.clearListeners(input, "focus");
    google.maps.event.clearListeners(input, "blur");
    google.maps.event.clearListeners(input, "keydown");
  }


  if (searchType == "places") {
    placesSearchBox = new google.maps.places.SearchBox(input);
    initGmapPlaces(placesSearchBox);
  } else {
    // Domains search by default
    domainsSearchBox = new redBorder.domainsSearchBox(input, root, function(sensor) {
      Turbolinks.visit("/dashboard/sensors/" + sensor + "/overview");
    });
  }

  // Remove Listener if set (avoid Turbolinks problems)
  $(document).off("click", ".overview-search .input-group-btn .dropdown-menu a");
  // Listener on change search type
  $(document).on("click", ".overview-search .input-group-btn .dropdown-menu a", function() {
    // Init variables
    var $btn = $(".overview-search button .search-at"),
      searchAt = $(this).data("search");

    event.preventDefault();

    // Replace Button Text
    $btn.text($(this).text());

    // Store Search Type at Session
    $.post("/dashboard/overview/store_search_type", { search_type: searchAt });

    // Option Change Control
    if (searchAt == "places") {
      placesSearchBox = new google.maps.places.SearchBox(input);
      initGmapPlaces(placesSearchBox);
      if (domainsSearchBox !== undefined) {
        domainsSearchBox.hide();
      }
    } else if (searchAt == "domains") {
      domainsSearchBox = new redBorder.domainsSearchBox(input, root, function(sensor) {
        Turbolinks.visit("/dashboard/sensors/" + sensor + "/overview");
      });
      disablePlacesSearch();
    }
  });
};
