
/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.threeDimensionsFloors = function (container, data) {

  var $container = $(container);

  drawFloors(data, $container);
  initHandlers(data);

  /*
  * Function that draws the floors over a container ( $selector ). Receives an array with the floor data and the container element.
  * Also sets up necessary handlers for each floor
  * @param floorData  An array with the floor data
  * @param $selector  Container in which the floor maps will be drawn
  * */
  function drawFloors(floorData, $selector){
    $selector.append('<div class="label"></div>');
    var top_level = 0;

    for (var i = 0; i < floorData.length; i++){
      var floor = floorData[i],
          level_n = 0,
          $level_selector;

      if (floor.level !== undefined){
        level_n = floor.level;
        if (level_n > top_level) top_level = level_n;
      }

      if ( ($level_selector = $selector.find('.level[data-level="' + level_n + '"]')).length == 0){   // level container is unexistant, create it
        $selector.append('<div class="level" data-level="' + level_n + '"></div>');
        $level_selector = $selector.find('.level[data-level="' + level_n + '"]');
      }

      $level_selector.append('<a class="block-ui-but" href="/sensors/' + floor.id +'/view"><div class="floor" style="bottom: ' + floor.sw[1][0] + 'px; left: ' + floor.sw[0][0] + 'px; transform-origin: bottom left; transform: rotate(' + floor.rotation + 'deg)"' + '' +
                             'data-id="' + floor.id + '" data-parent="' + floor.parent + '" data-title="' + floor.name + '" data-up="' + floor.access_points.up + '" data-down="' + floor.access_points.down + '" ' +
                             'data-unknown="' + floor.access_points.unknown + '" data-unallocated="' + floor.access_points.unallocated + '" + data-bottom="' + floor.sw[1][0]+ '" + data-left="' + floor.sw[0][0] + '" >' +
                              '<img src="' + floor.image_url + '" alt="" width="' + floor.width + '" height="' + floor.height + '" />' +
                              drawFloorsAccessPoints(floor) +
                              '</div></a>');

    }

    $("#access_points_3d_container .floor").hover(function(){
      $(this).css('z-index', '1');
      $("#access_points_3d_container .floor").not(this).addClass("unfocused");
      $('.access_points_circles').not($(this).parent().find('.access_points_circles')).addClass("unfocused");
      showFloorInfoWindow('.info', { title: $(this).data('title'), content: { up: $(this).data('up'), down: $(this).data('down'), unallocated: $(this).data('unallocated'), unknown: $(this).data('unknown') } });
    }, function(){
      $(this).css('z-index', '0');
      $("#access_points_3d_container .floor").not(this).removeClass("unfocused");
      $('.access_points_circles').not($(this).parent().find('.access_points_circles')).removeClass("unfocused");
      hideFloorInfoWindow('.info');
    });

    $(".access_points_circles").hover(function(){
      showApInfoWindow('.info-ap', { title: $(this).data('name'), content: {ip: $(this).data('ip-address'), mac: $(this).data('mac-address') } });
    }, function(){
      hideApInfoWindow('.info-ap');
    });
    $container.parent().css("margin-top", top_level * 100 + "px");
    scatterLevels();
  }

  /*
  * Draws the access points of a floor over said floor in it's corresponding positions.
  * @param floor  The data for the containing floor
  * */
  function drawFloorsAccessPoints(floor){
    var aps = floor.access_points.aps,
        result = '';
    for (var i = 0; i < aps.length; i++){
      var ap = aps[i],
          status;

      if (ap.status === 1) status = 'up';
      else if (ap.status === 2) status = 'down';
      else status = 'unknown';

      var marker = ' \
      <i data-name="' + ap.name + '" data-ip-address="' + ap.ip_address + '\
      " data-mac-address="' + ap.mac_address + '" class="access_points_circles ap-' + status + '" data-name="' + ap.name + '"style=" \
      position: absolute; top: ' + ap.y + 'px; \
      left: ' + ap.x + 'px; text-shadow: 0 0 5px dark' + status + ';"></i>';
      result = result + marker;
    }
    return result;
  }

  /*
  * In order to present the user with a 3D effect floor maps should be scattered and rotated accordingly.
  * Each new level adds an additional 40 px separation.
  * Rotation is user defined via GUI controls.
  * @param rotation  User defined rotation as an integer expressed on degrees.
  * */
  function scatterLevels(rotation){
    $(".level").each( function( index ){
      var $floor = $(this).find(".floor");
      var level = $(this).data("level");
      $floor.each(function (index) {
        if (rotation !== undefined){
          if (rotation == -90 || rotation == 270){
            $(this).css("bottom", parseFloat( $(this).data("bottom")) - 40 * level );
            $(this).css("left", parseFloat( $(this).data("left")) + 40 * level );
            $('.top-domain img').css("box-shadow", "-5px -5px 15px rgba(33,33,33,0.5)");
            $('.top-domain').css("box-shadow", "-20px -20px 100px rgba(22,22,22,0.5)");
          } else if (rotation == 180 || rotation == -180){
            $(this).css("bottom", parseFloat( $(this).data("bottom")) - 40 * level );
            $(this).css("left", parseFloat( $(this).data("left")) - 40 * level );
            $('.top-domain img').css("box-shadow", "5px -5px 15px rgba(33,33,33,0.5)");
            $('.top-domain').css("box-shadow", "20px -20px 100px rgba(22,22,22,0.5)");
          } else if (rotation == -270 || rotation == 90){
            $(this).css("bottom", parseFloat( $(this).data("bottom")) + 40 * level );
            $(this).css("left", parseFloat( $(this).data("left")) - 40 * level );
            $('.top-domain img').css("box-shadow", "5px 5px 15px rgba(33,33,33,0.5)");
            $('.top-domain').css("box-shadow", "20px 20px 100px rgba(22,22,22,0.5)");
          } else{
            $(this).css("left", parseFloat( $(this).data("left") ) + 40*level );
            $(this).css("bottom", parseFloat( $(this).data("bottom") ) + 40*level );
            $('.top-domain img').css("box-shadow", "-5px 5px 15px rgba(33,33,33,0.5)");
            $('.top-domain').css("box-shadow", "-20px 20px 100px rgba(22,22,22,0.5)");
          }
        } else{
          $(this).css("left", parseFloat( $(this).data("left") ) + 40*level );
          $(this).css("bottom", parseFloat( $(this).data("bottom") ) + 40*level );
        }
      });
    });
  }

  /*
  * Initializes handlers for user events such as clicking or contextual menus
  * */
  function initHandlers(){
    $(':not(.context-menu)').bind("click", function (event) {
      $('.context-menu').remove();
    });

    $('#access_points_3d_container .floor').bind("contextmenu", function(event) {
      event.preventDefault();
      $('.context-menu').remove();
      $('<div class="context-menu">' +
        '<a class="block-ui-but" href="/sensors/' + $(this).data("id") + '/view"><li>' + I18n.t('javascript.go_floor') + '</li></a>' +
        '<a class="block-ui-but" href="/sensors/' + $(this).data("parent") +'/view"><li>' + I18n.t('javascript.go_domain') + '</li></a>' +
      '</div>')
          .appendTo('body')
          .css({
            position: "absolute",
            top: event.pageY + 'px',
            left: event.pageX + 'px'
          });
    });

    $('.rotate-left').bind("click", function(event){
      rotation = $container.data('rotation');
      if (rotation == -270) rotation = 0;
      else rotation = rotation - 90;
      $container.data('rotation', rotation);
      $container.css("transform", "rotate("+ rotation +"deg)");
      scatterLevels(rotation);
    });
    $('.rotate-right').bind("click", function(event){
      rotation = $container.data('rotation');
      if (rotation == 270) rotation = 0;
      else rotation = rotation + 90;
      $container.data('rotation', rotation);
      $container.css("transform", "rotate("+ rotation +"deg)");
      scatterLevels(rotation);
    });
  }

  /*
  * Shows and fills an info window with the floor data when the user hovers over any floor.
  * @param selector  CSS selector for the info window container
  * @param data      Floor data used to fill the info window
  * */
  function showFloorInfoWindow(selector, data){
    $info = $(selector);
    $info.find('.title').text(data.title);
    $info.find('[data-holder="up"]').text(data.content.up);
    $info.find('[data-holder="down"]').text(data.content.down);
    $info.find('[data-holder="unallocated"]').text(data.content.unallocated);
    $info.find('[data-holder="unknown"]').text(data.content.unknown);
    $info.fadeIn(100)
  }

  /*
  * Hides previously described info window
  * */
  function hideFloorInfoWindow(selector){
    $(selector).hide();
  }

  /*
  * Shows and fills an info window with an access point information when the user hovers over it
  * @param selector  CSS selector for the info window container
  * @param data      Access Point data used to fill the info window
  * */
  function showApInfoWindow(selector, data){
    $info = $(selector);
    $info.find('.title').text(data.title);
    $info.find('[data-holder="ip-address"]').text(data.content.ip);
    $info.find('[data-holder="mac-address"]').text(data.content.mac);
    $info.fadeIn(100)
  }

  /*
  * Hides the previously described AP info window
  * */
  function hideApInfoWindow(selector){
    $(selector).hide();
  }


};
