redBorder.widgets.graphs.combination = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      stepSecs = widget.graph.step / 1000,
      reloaded = true,
      units = isNaN(parseFloat(widget.property.reference)) ? redBorder.widgets.unitsHandler(widget) : '%';

  graph = new RB.Serie({
    container: container,
    autoscaling: true,
    step: widget.graph.step,
    domain: [(Math.floor(widget.wtype.start_time_secs/60)*60 + stepSecs), (Math.floor(widget.wtype.end_time_secs/60)*60 - stepSecs)],
    timezone: widget.graph.tzoffset,
    margin: { top: 0, left: 0, right: 0, bottom: 10 },
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: units,
    predict_area: false
  });

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    getData();
  }

  function getData() {
    wtype.queryList().then(function (data) {
      process(data);
    }, function () {
      redBorder.widgets.util.reloadDone($container.parents('.widget'))
    });
  }

  function process(list) {
    var serie = [],
        n = list.length;

    list.forEach(function (element) {
      wtype
        .queryTimeserie(element)
        .then(function (serieData) {
          serie = combine(serieData, element, serie);
          n -= 1;
          if (n == 0) { draw(serie); }
        });
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function combine(data, element, serie) {
    var weight = parseFloat(element.weight)
    var offset = parseFloat(element.offset || 0)
    var reference = parseFloat(element.reference)

    data = data.map(function (e) {
      var mixed = serie.filter(function (obj) { return obj.timestamp === e.timestamp; })[0]
      var referenced_events = e.event.events;
      if ( !isNaN(parseFloat(element.reference)) ) {
        referenced_events = (100 * (e.event.events - offset) / (reference - offset));
      }
      if (mixed !== undefined) {
        serie = serie.map(function(s) {
          if(s.timestamp === e.timestamp) {
            s.event.events = (weight * referenced_events) + mixed.event.events
          }
          return s;
        })
      } else {
        serie = serie.concat({
          timestamp: e.timestamp,
          event: { events: (weight * referenced_events) }
        })
      }
    });

    return serie;
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    // Delete data if we come from reload event
    if(reloaded){
      reloadGraph();
    }

    if (data.length > 0) {
      graph.add({
        data: data,
        name: 'Total'
      });
      graph.startRender();
    }

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reloadGraph() {
    // Update the domain
    var diffSecs = widget.wtype.end_time_secs - widget.wtype.start_time_secs,
        minute = (1).minutes(),
        coef, nStartTime, nEndTime;

    if (widget.graph.step <= (5).minutes()){
      coef = widget.graph.step;
    } else {
      coef = minute;
    }

    nEndTime = new Date(Math.floor(redBorder.timeNow.getTime()/minute)*minute);
    if (widget.graph.step !== undefined && widget.graph.step <= (5).minutes()) {
      nEndTime = new Date(nEndTime.getTime() - widget.graph.step);
    }

    nStartTime = new Date(Math.ceil((((redBorder.timeNow.getTime()/1000) - diffSecs)*1000)/coef)*coef);

    graph.forceDomain([nStartTime, nEndTime], true);

    // Delete all series
    graph.remove('Total');
  }

  function reload() {
    reloaded = true;
    // Draw new series
    getData();
  }

  // Trigger an event when the widget is reloaded
  function reloadDone(){
    // Create the event
    $container.parents('.widget').trigger('widget-reload');
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', ($container.parents('.widget').width()) + 'px');
    graph.resize();
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('svg').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('svg').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
