redBorder.widgets.graphs.combination_box = function (container, widget) {
  var wtype, $container = $('#' + container);
  var total = 0;

  // Color range
  var colors = ["#5DB262", "#7BC255", "#99CF3F", "#B2D62A", "#D4D62A",
                "#E7D820", "#E7B820", "#E78920", "#D95A1D", "#C61E33"];
  if (widget.property.invert_colors === '1') colors = colors.reverse();

  var domain = [0, parseFloat(widget.property.box_max)*0.11,
                parseFloat(widget.property.box_max)*0.22, parseFloat(widget.property.box_max)*0.33,
                parseFloat(widget.property.box_max)*0.44, parseFloat(widget.property.box_max)*0.55,
                parseFloat(widget.property.box_max)*0.66, parseFloat(widget.property.box_max)*0.77,
                parseFloat(widget.property.box_max)*0.88, parseFloat(widget.property.box_max)];
  var scale = d3.scale.linear().domain(domain).range(colors).clamp(true);

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    total = 0;
    wtype.queryList().then(function (data) {
      process(data);
    }, function () {
      redBorder.widgets.util.reloadDone($container.parents('.widget'))
    });
  }

  function add(num) {
    total += num;
    draw(total);
  }

  function process(list) {
    var serie = [],
        n = list.length;

    list.forEach(function (element) {
      wtype
        .queryTotal(element)
        .then(function (total) {
          var n = total;
          var offset = parseFloat(element.offset || 0);
          var reference = parseFloat(element.reference);

          if ( !isNaN(reference) ) {
            n = RB.util.transform(total, offset, reference);
          }
          add(n);
        });
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(total) {
    var scaled = RB.util.scale(total);
    var text = scaled.scaledNum.toFixed(2) + ' ' + scaled.prefix;

    if (widget.property.always_show_quantity === '0') {
      $container.hover(function () { $container.find('.box-value').html(text) },
                       function () { $container.find('.box-value').html('') });
    } else {
      $container.find('.box-value').html(text);
    }
    fitText();
    // Scale functions
    $container.css({ 'background-color': scale(scaled.num) });
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload() {
    render();
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }
    $container.css('width', $container.parents('.widget').width() + 'px');
    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    fitText();
  }

  function fitText() {
    var w = parseInt($container.css('width'));
    var n = $container.find('.box-value').text().length;
    $container.find('.box-value').css('font-size', w / n + 'px');
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100);
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
