redBorder.widgets.graphs.horizontal_bar = function (container, widget) {
  var wtype,
      $container = $('#' + container);

  var colors = redBorder.utils.colors;

  var palette;
  if (colors[widget.property.column] !== undefined) {
    palette = colors[widget.property.column];
  }

  var graph = new RB.Bars({
      container: container,
      max: widget.property.top_num,
      color: widget.property.color,
      palette: palette,
      shared_colors: false,
      showBiggestAsBarTotal: true,
      units: redBorder.widgets.unitsHandler(widget)
    });

  var _scale = d3.scale.linear().domain([100,900]).range([0,130]);
  var MAX = {
    COLUMNS: {
      1: 10,
      2: 20,
      3: 50,
      4: 80,
      5: 110
    },
    WIDTH: function(n) {
      return Math.floor(_scale(parseInt(n)));
    }
  };

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    $.when(wtype.queryList(), wtype.queryTotal()).then( draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'))
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(tops, total) {
    tops.forEach(function(top) {
        if (widget.property.product_type === "location") {
            var locationName = redBorder.widgets.locations && redBorder.widgets.locations[top.name] ? redBorder.widgets.locations[top.name] : top.name;
            top.name = locationName;
        }
    });
    graph.setTotal(total, false);
    graph.setData(tops);
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
}

  function reload(){
    $.when(wtype.queryList(), wtype.queryTotal()).then(function(tops, total){
      render(tops, total);
    }, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('width', $container.parents('.widget').width() + 'px');
    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    graph.resize();
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('svg').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('svg').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  function formatTopData(_data) {
    data = _data;

    var cap = 150;
    switch(typeof widget.property.position_width) {
      case "number":
        cap = MAX.COLUMNS[widget.property.position_width] || cap;
        break;
      case "string":
        cap = MAX.WIDTH(widget.property.position_width) || cap;
        break;
    };
    // Cap secondary names
    data.forEach(function(element) {
      if (element.name && element.name.length >= cap) {
        element.name = element.name.substring(0, cap - 3) + '...';
      }
    });

    return data;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
