redBorder.widgets.graphs.image = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      $image = $container.find('img'),
      keep_aspect = false;

  if (widget.property.keep_aspect_ratio === "1"){
    keep_aspect = true;
  }

  // The widget manager calls this method when the graph should start
  function init(wtype) {
    if (keep_aspect){
      calculateAspectRatio();
    }
  }

  function render(){
    if (keep_aspect){
      calculateAspectRatio();
    }
    $container.find('img').load(this.draw);
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw() {
    if (keep_aspect){
      calculateAspectRatio();
    }

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload() { }

  function calculateAspectRatio(){
    var $img = $container.find('img');
    var height_c = $container.height();
    var width_c = $container.width();
    var height_r = $img[0].naturalHeight;
    var width_r = $img[0].naturalWidth;

    // Get the ratios
    var rc = height_c/width_c;
    var rr = height_r/width_r;

    // Clear classes of container
    $container.removeClass('keep_aspect_height');
    $container.removeClass('keep_aspect_width');
    $img.css('margin-top', '0');

    if(rr > rc){
      // The limit is on height
      $container.addClass('keep_aspect_height');
    } else if(rr < rc){
      // The limit is on width
      $container.addClass('keep_aspect_width');
      // We must center the img
      $img.css('margin-top', ((height_c - parseInt(width_c*rr))/2) + 'px' );
    }

  }

  function resize() {
    height = 0;
    if(widget.name.length !== 0) {
      height = 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', $container.parents('.widget').width() + 'px');

    if (keep_aspect){
      calculateAspectRatio();
    }
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
