redBorder.widgets.graphs.incident_table = function (container, widget) {

  var $container = $('#' + container),
      range = widget.property.range,
      priorities = widget.property.priorities,
      status = widget.property.status,
      domains = widget.property.domains,
      users = widget.property.users,
      sources = widget.property.sources;

  // Entry point for the graph
  function init(wtype) {
    render();
  }

  function render(){
    getData();
  }

  function getData(){
    var url = "/incidents/get_incidents_widget_data/?type=table";;

    $.ajax({
      url: url,
      type: 'GET',
      dataType: 'json',
      data: {
        range: range,
        priorities: priorities,
        status: status,
        domains: domains,
        users: users,
        sources: sources
      }
    }).done(function (data) {
      draw(data);
    }).fail(function () {
      // Forbidden!
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    }).always(function () {

    });
  }

  function draw(data) {
    $container.html("");

    if (!data || data.length === 0) {
      $container.append("<div class='center'><p>" + I18n.t('javascript.no_data') + "</p></div>");
      redBorder.widgets.util.reloadDone($container.parents('.widget'));
      return;
    }

    $container.append("<table>");
    var $table = $("table", $container);
    $table.addClass("table table-striped table-condensed table-hover");

    var columns = ['ID', 'Name', 'Priority', 'Created At']
    $table.append("<thead><tr></tr></thead>");
    var $theadRow = $("table thead tr", $container);

    columns.forEach(function (column) {
      $("<th>").text(column).appendTo($theadRow);
    });

    $table.append("<tbody>");
    var $tbody = $("table tbody", $container);

    data.forEach(function (incident, i) {
      var $tr = $("<tr>").appendTo($tbody);
      var nameLink = $("<a>")
          .attr("href", "/incidents/" + incident.name.id)
          .text(incident.name.name || 'N/A')
          .css({"font-weight": "bold"});

      $("<td>").text(incident.name.id || 0).appendTo($tr);
      $("<td>").append(nameLink).appendTo($tr);
      $("<td>").append($("<span>").text(incident.name.priority_name || 'N/A')
                                  .css({
                                    "font-weight": "bold",
                                    "border-radius": "5px",
                                    padding: "3px 5px",
                                    "color": incident.name.priority_color || "black",
                                    "background-color": incident.name.priority_bgcolor || "white"
                                  })
      ).appendTo($tr);
      $("<td>").text(formatDate(incident.name.created_at)).appendTo($tr);
    });

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function formatDate(dateString) {
    var date = new Date(dateString);
    return date.toLocaleDateString() + " " + date.toLocaleTimeString();
  }

  function reload() {
    getData();
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }
    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', $container.parents('.widget').width() + 'px');
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('table').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('table').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return undefined;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
