redBorder.widgets.graphs.map = function (container, widget) {
  var graph,
      $container = $('#' + container),
      wtype;

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    wtype.queryList({ limit: widget.property.points_number }).then( draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'))
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    try {
      graph = redBorder.map({
        container: container,
        data: data,
        aggregation: widget.property.aggregation,
        scrollwheel: false,
        searchBox: false,
        clustered: true,
        mirrorControls: true,
        type: widget.property.map_type,
        hideUI: widget.parent_type === "Report"
      });
    } catch (err) {
      console.error("Couldn't load widget " + widget.id + ". " + err);
    }
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload(){
    wtype.queryList({ limit: widget.property.points_number }).then(function(data){
      graph.removeMarkers();
      graph.addMarkers(data);
      redBorder.widgets.util.reloadDone($container.parents('.widget'));
    }, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  function resize() {
    var height = (widget.name.length !== 0) ? 43 : 0;
    var w = $container.parents('.widget').width();
    var h = $container.parents('.widget').height() - height;
    $container.css('height', h + 'px');
    $container.css('width', w + 'px');
    graph.resize(w, h);
  }

  // Hide the main element when resizing
  function hide() {
    $container.css('opacity', 0);
  }

  // Show the main element
  function show() {
    $container.css('opacity', 1);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
