redBorder.widgets.graphs.multiserie = function (container, widget) {
  var graph, fillWith, wtype,
      range = widget.property.range,
      widget_type = widget.property.widget_type,
      aggregation = widget.property.aggregation,
      product_type = widget.property.product_type,
      $container = $('#' + container),
      stepSecs = widget.graph.step / 1000,
      reloaded = false,
      elements = [],
      colors = redBorder.utils.colors,
      current_item = 0,
      rendered = false,
      deleted = false,
      units = isNaN(parseFloat(widget.property.reference)) ? redBorder.widgets.unitsHandler(widget) : '%';

  // REFACTOR: This check doesnt belong here
  if (widget_type === 'monitor') fillWith = 'previous';

  var ylimits = [];

  if(!isNaN(parseFloat(widget.property.top_reference))){
    ylimits.push(parseFloat(widget.property.top_reference));
  }

  if(!isNaN(parseFloat(widget.property.bottom_reference))){
    ylimits.push(parseFloat(widget.property.bottom_reference));
  }

  graph = new RB.Serie({
    container: container,
    autoscaling: true,
    step: widget.graph.step,
    domain: [(widget.wtype.start_time_secs + stepSecs), (widget.wtype.end_time_secs - stepSecs)],
    timezone: widget.graph.tzoffset,
    margin: { top: 5, left: 0, right: 0, bottom: 10 },
    zoomable: false,
    fillWith: fillWith,
    legend: widget.parent_type === "Report",
    autorender: widget.parent_type === "Report",
    shared_colors: true,
    genericXAxis: (function () {
      if (widget_type == 'compare') {
        return true;
      } else {
        return false;
      }
    })(),
    show_value_always: widget.property.show_value_always == "1",
    tooltip_date_format: (function() {
      if (widget_type == 'compare') {
        switch (range) {
          case 'pt24h':
            return I18n.t('javascript.minute') + ' %M';
          case 'pt168h':
            return I18n.t('javascript.hour') + ' %H:%M';
          case 'pt720h':
            return '%A, %H:%M';
          case 'pt4320h':
            return I18n.t('javascript.day') + ' %d - %H:%M';
        };
      }
    })(),
    units: units,
    predict_area: false,
    yLimits: ylimits,
    discrete_y_axis: widget.property.discrete_y_axis === "1"
  });

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    wtype.queryList().then(function (data) {
      process(wtype, data);
    }, function () {
      redBorder.widgets.util.reloadDone($container.parents('.widget'))
    });
  }

  function reload() {
    reloaded = true;
    deleted = false;

    wtype.queryList().then(function (data) {
      process(wtype, data);
    });
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    var w = $container.parents('.widget').width();
    var h = $container.parents('.widget').height() - height;
    $container.css('height', h + 'px');
    $container.css('width', w + 'px');

    var legend_height = Math.ceil(graph.elements.length/Math.floor(w/120));
    graph.resize(w, h, legend_height);
  }

  // Update the domain to the new data
  function updateDomain(){
    // Update the domain if we need. On compare widgets, the domain is always
    // the same
    if (widget_type != 'compare') {
      var diffSecs = (widget.wtype.end_time_secs - widget.wtype.start_time_secs) * 1000;

      // Domain will be converted to the correct based on the step of the widget
      if (widget.property.range == 'custom') {
        nStartTime = new Date(widget.wtype.start_time_secs * 1000)
        nEndTime = new Date(widget.wtype.end_time_secs * 1000)
      } else {
        nEndTime = new Date(redBorder.timeNow.getTime());
        nStartTime = new Date(redBorder.timeNow.getTime() - diffSecs);
      }

      graph.forceDomain([nStartTime, nEndTime], false);
    }
  }

  // Gets the list and queries the timeseries associated
  // If list returns nothing, we call Timeserie without params to
  // get the total timeserie and draws it as Total.
  // Else we query all the series needed and draw them.
  function process(wtype, list) {
    if (list === false) {
      wtype.queryTimeserie().then(function (data) {
        drawAsTotal(data);
      });
    } else {
      if (widget.property.show_total === "1") {
        wtype.queryTimeserie(undefined, true).then(function (data) {
          drawAsTotal(data);
        });
      }
      // Get the number of items
      current_item = list.length - 1;

      list.forEach(function (element) {
        var opts = element.value || element.opts || element.name || 'N/A';
        var name = element.name || 'N/A';
        var w = $container.parents('.widget').width();
        var legend_height = Math.ceil(list.length/Math.floor(w/120));

        wtype
          .queryTimeserie(opts)
          .then(function (serieData) {
            draw(serieData, name, legend_height);
          });
      });
      redBorder.widgets.util.reloadDone($container.parents('.widget'));
    }
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data, name, legend_height) {
    // Check if we must to delete the graphs
    if (reloaded) {
      updateDomain();
      reloaded = false;
    }

    if (!deleted) {
      deleteGraphs();
      deleted = true;
    }

    var color;
    if (colors[widget.property.column] !== undefined) {
      color = colors[widget.property.column][name];
    }

    if ( !isNaN(parseFloat(widget.property.reference)) ) {
      var offset = parseFloat(widget.property.offset || 0);
      var reference = parseFloat(widget.property.reference);

      // Print the reference over the widget
      //$container.find('.widget-reference').empty();
      //$container.prepend('<div class="widget-reference"><h5>With ' + reference + ' ' +
      //  redBorder.widgets.unitsHandler(widget) + ' as reference</h5></div>');
      data = data.map(function (e) {
        return {
          timestamp: e.timestamp,
          event: { events: 100 * (e.event.events - offset) / (reference - offset) }
        }
      });
    }

    if (widget.property.product_type == "location" && widget.property.widget_type != 'compare') {
      if (redBorder.widgets.locations && redBorder.widgets.locations[name]) {
        name = redBorder.widgets.locations[name];
      }
    }
    // Save the created vars
    elements.push(name);

    graph.add({
      data: data,
      name: name,
      type: widget.property.serie_type,
      color: color,
      legend: (legend_height !== Infinity)
    });

    if(current_item === 0 && !rendered){
      // Reload the domain
      graph.startRender(legend_height);
      rendered = true;
    }

    // Increase
    current_item--;
  }

  // Draws the serie as a total, so that serie wont be stacked and
  // will be displayed as grey.
  function drawAsTotal(data) {
    // Check if we must to delete the graphs
    if (!deleted){
      deleteGraphs();
      deleted = true;
    }

    graph.add({
      data: data,
      name: 'Total',
      color: '#ccc',
      type: 'line',
      dashed: true,
      stroke: 1
    })
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  // Delete all graphs on elements var and total if it exists.
  function deleteGraphs() {
    if (widget.property.show_total === "1") {
      graph.remove('Total');
    }

    elements.forEach(function(d){
      graph.remove(d);
    });

    elements = [];
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('svg').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('svg').fadeIn(100);
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
