redBorder.widgets.graphs.pie = function (container, widget) {
  var graph,
      wtype,
      colors = redBorder.utils.colors,
      $container = $('#' + container);

  graph = new RB.Pie({
    container: container,
    max: widget.property.top_num,
    full_legend: widget.property.full_legend == '1',
    show_highest_percen: widget.property.show_highest_percen == '1'
  });

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    $.when(wtype.queryList(), wtype.queryTotal(), wtype.getColors()).then( draw, function () {
      redBorder.widgets.util.reloadDone($container.parents('.widget'))
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(tops, total, _colors) {
    var data;

    // Set the colors
    if (_colors !== undefined){
      _colors.forEach(function (e) {
        colors[widget.property.column] = {};
        colors[widget.property.column][e.name] = e.bg_color;
      });
    }

    // Sets the total of the pie graph
    // We check the number of elements because
    // there are some problems with N/A
    if (tops.length < widget.property.elements) {
      total = 0;
      tops.forEach(function(d){
        total += d.events;
      })
    }

    // Only set the total with aggregations that make sense
    if (['bps', 'bytes', 'pkts', 'pps', 'flows', 'fps'].indexOf(widget.property.aggregation) !== -1) {
      graph.total = total;
    }

    // Process the tops input to add the color
    data = tops.map(function (d) {
      var color;

      if (colors[widget.property.column] !== undefined) {
        color = colors[widget.property.column][d.name];
      } else {
        var name = d['secondary_name'];

        if (colors[widget.property.column] !== undefined &&
            colors[widget.property.column][name] !== undefined) {
          color = colors[widget.property.column][name];
        }
      }

      return {
        name: widget.property.product_type == "location" ? (redBorder.widgets.locations && redBorder.widgets.locations[d.name] ? redBorder.widgets.locations[d.name] : d.name) : d.name,
        events: d.events,
        color: color
      };
    });

    graph.setData(data);
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload() {
    $.when(wtype.queryList(), wtype.queryTotal(), wtype.getColors()).then(draw);
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    var h = $container.parents('.widget').height() - height;
    var w = $container.parents('.widget').width();
    $container.css('height', h + 'px');
    $container.css('width', w + 'px');
    graph.resize(w, $container.height());
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('svg').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('svg').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
