redBorder.widgets.graphs.redfish = function (container, widget) {
  var columns, wtype,
      aggregation,
      product_type,
      $container = $('#' + container);

  columns = widget.property.stream_columns.filter(function (column) {
    return column !== "";
  });

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    wtype.queryTable(columns).then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    if (data.length !== 0) {
      var product_type = widget.property.product_type
      var aggregation = widget.property.aggregation
      var aggregation_name = redBorder.modules[product_type].aggregations[aggregation].name;

      $container.empty();
      $container.append("<table>");
      $("table", $container).append("<thead>");
      $("table", $container).append("<tbody>");
      $("table", $container).attr("class", "table table-striped table-condensed table-hover");
      $("table thead", $container).append("<tr>");

      columns.forEach(function (d) {
        //Translating from init.rb
        var column_name = redBorder.modules[product_type].columns[d].columns[d]
        if (column_name.startsWith("tr.")) {
          column_name = I18n.t(column_name.slice(3))
        }
        $("table thead tr", $container).append($("<th>").text(column_name));
      });
      $("table thead tr", $container).append($("<th>").text(aggregation_name));

      data.forEach(function (d, i) {
        if (i < widget.property.elements) {
          var multiplier = redBorder.widgets.util.getMultiplier(d[aggregation]);
          var $tr = $("<tr>").appendTo($("table tbody", $container));

          columns.forEach(function (column) {
            var name = d[column],
                elementclass = '';
            if (name === undefined || name === '' || name === null) name = 'N/A';
            if (column === 'wan_ip_name' || column === 'lan_ip_name' ||
                column === 'src_name' || column === 'dst_name' ||
                column === 'public_ip_name') {
              elementclass = 'auto-resolv';
            }
            $("<td>").text(name).attr('class', elementclass).appendTo($tr);
          })
          var n = (Math.round(d[aggregation] * multiplier.num * 100) / 100) || '';
          var unit = multiplier.prefix || '';
          $("<td>").text(n + " " + unit).attr('class', 'last').appendTo($tr);
        }
      });
      if (window.redBorder.resolve_dns === true) resolve_dns();
    } else {
      $container.append("<div class='center'><p>No data to display</p></div>");
    }

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload() {
    $container.html("");
    wtype.queryTable(columns).then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('width', $container.parents('.widget').width() + 'px');
    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('table').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('table').fadeIn(100);
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
