redBorder.widgets.graphs.serie = function (container, widget) {
  var graph,
      wtype,
      $container = $('#' + container),
      stepSecs = widget.graph.step / 1000,
      movingPoints = 10,
      reloaded = false,
      units = isNaN(parseFloat(widget.property.reference)) ? redBorder.widgets.unitsHandler(widget) : '%';

  var ylimits = [];

  if(!isNaN(parseFloat(widget.property.top_reference))){
    ylimits.push(parseFloat(widget.property.top_reference));
  }

  if(!isNaN(parseFloat(widget.property.bottom_reference))){
    ylimits.push(parseFloat(widget.property.bottom_reference));
  }

  graph = new RB.Serie({
    container: container,
    autoscaling: true,
    step: widget.graph.step,
    domain: [(Math.floor(widget.wtype.start_time_secs/60)*60 + stepSecs), (Math.floor(widget.wtype.end_time_secs/60)*60 - stepSecs)],
    timezone: widget.graph.tzoffset,
    margin: { top: 0, left: 0, right: 0, bottom: 10 },
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: units,
    predict_area: false,
    yLimits: ylimits,
    discrete_y_axis: widget.property.discrete_y_axis === "1"
  });

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    getData();
  }

  function getData() {
    wtype.queryTimeserie().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });

    if (widget.property.analytics.indexOf('previous_period') !== -1) {
      previousPeriod(wtype);
    }

    if (widget.property.analytics.indexOf('day_over_day') !== -1) {
      dayOverDay(wtype);
    }
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    // Delete data if we come from reload event
    if(reloaded){
      reloadGraph();
    }
    if ( !isNaN(parseFloat(widget.property.reference)) ) {
      var offset = parseFloat(widget.property.offset || 0);
      var reference = parseFloat(widget.property.reference);

      // Print the reference over the widget
      //$container.find('.widget-reference').empty();
      //$container.prepend('<div class="widget-reference"><h5>With ' + reference + ' ' +
      //  redBorder.widgets.unitsHandler(widget) + ' as reference</h5></div>');
      data = data.map(function (e) {
        var relative = 100 * (e.event.events - offset) / (reference - offset);
        return {
          timestamp: e.timestamp,
          event: { events: relative < 0 ? 0 : relative }
        }
      });
    }

    graph.add({
      data: data,
      name: 'Total',
      color: widget.property.color,
      type: widget.property.serie_type
    });

    if (data.length > 0) {
      if (widget.property.analytics.indexOf('moving_average') !== -1) {
        wtype.queryTimeserie({
          // Load 7 points more
          start_time: new Date(graph.domain[0].getTime() - movingPoints*widget.graph.step),
          end_time: graph.domain[1]
        }).then(drawMovingAverage);
      }

      if (widget.property.analytics.indexOf('average') !== -1) {
        graph.add({
          data: average(data),
          name: 'Average',
          type: 'line',
          dashed: true
        })
      }
    }

    graph.startRender();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function drawMovingAverage(data){
    graph.add({
      data: movingAverage(data),
      name: 'Moving Average',
      type: 'line',
      dashed: true
    })
  }

  function movingAverage(data) {
    var sortedData, average = [];

    if (data.length === 0) { return average; }
    // Sort by date
    sortedData = data.sort(function (a, b) {
      var a2 = new Date(a.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3")),
          b2 = new Date(b.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3"));

      return a2.getTime() - b2.getTime();
    });

    // Fill empty points
    sortedData = redBorder.utils.query.fillEmptyPoints(sortedData, widget.graph.step, new Date(graph.domain[0].getTime() - movingPoints*widget.graph.step), graph.domain[1]);

    // Build the moving average
    sortedData.forEach(function (point, index) {
      // Ignore the first N points defined by movingAverage
      if(index < movingPoints){
        return;
      }

      var pointAverage = 0;

      // Construct the average taking the previous {averagePoints} points
      for (var i = 0; i < movingPoints; i++) {
        // We take the points with index less than zero as zeros
        if ((index - i - 1) >= 0) {
          pointAverage += sortedData[index - i - 1].event.events;
        }
      }

      // Divide between the number of elements to get the average
      pointAverage /= movingPoints;

      // Add to the average serie
      average.push({
        event: {
          events: pointAverage
        },
        timestamp: point.timestamp
      });
    });

    return average;
  }

  function average(data) {
    var average = [],
        averageValue = 0;

    if (data.length === 0) { return average; }
    data = redBorder.utils.query.fillEmptyPoints(data, widget.graph.step, graph.domain[0], graph.domain[1]);

    // Sum each value
    data.forEach(function (point) {
      averageValue += point.event.events;
    });

    // Divide by number of elemengs
    averageValue /= data.length;

    // Generate the correct object
    data.forEach(function (point) {
      average.push({
        event: {
          events: averageValue
        },
        timestamp: point.timestamp
      });
    });

    return average;
  }

  function previousPeriod(wtype) {
    var lastRange = redBorder.widgets.util.lastRange(widget.property.range);

    wtype.queryTimeserie({
      start_time: lastRange[0],
      end_time: lastRange[1] - widget.graph.step/1000
    }).then(function (data) {
      // The range width in seconds
      var rangeWidth = lastRange[1] - lastRange[0],
          movedData;

      // We move the points retrieved so they can fit above
      // the real data to show the difference
      movedData = data.map(function (point) {
        var date = new Date(point.timestamp.replace(/(\d*)-(\d*)-(\d*)/, "$1/$2/$3"));
        return {
          event: {
            events: point.event.events
          },
          timestamp: new Date(date.getTime() + rangeWidth * 1000)
        }
      });

      graph.add({
        name: 'Last range',
        data: movedData,
        type: widget.property.serie_type
      });

      graph.startRender();
    });
  }

  function dayOverDay(wtype) {
    var lastDay = redBorder.widgets.util.lastDay(widget.property.range);

    wtype.queryTimeserie({
      start_time: lastDay[0],
      end_time: lastDay[1] - widget.graph.step/1000
    }).then(function (data) {
      // The range width in seconds
      var rangeWidth = (widget.wtype.start_time_secs + (Math.round(redBorder.timeNow.getTime()/1000)
                          - widget.wtype.end_time_secs)) - lastDay[0],
          minute = 60 * 1000,
          movedData;

      // If data is empty, we add a 0 to show the serie
      if(data.length == 0){
        data.push({
          event: {
            events: 0
          },
          timestamp: new Date(lastDay[0] * 1000)
        });
      }

      // We move the points retrieved so they can fit above
      // the real data to show the difference
      movedData = data.map(function (point) {
        var date = new Date(point.timestamp);
        return {
          event: {
            events: point.event.events
          },
          timestamp: new Date(Math.floor((date.getTime() + rangeWidth * 1000) / minute) * minute).toString()
        }
      });

      graph.add({
        name: 'Day over day',
        data: movedData,
        type: widget.property.serie_type
      });

      graph.startRender();
    });
  }

  function reloadGraph() {
    // Update the domain
    var diffSecs = widget.wtype.end_time_secs - widget.wtype.start_time_secs,
        minute = (1).minutes(),
        coef, nStartTime, nEndTime;

    if (widget.graph.step <= (5).minutes()){
      coef = widget.graph.step;
    } else {
      coef = minute;
    }

    if (widget.property.range == 'custom') {
      nStartTime = new Date(widget.wtype.start_time_secs * 1000)
      nEndTime = new Date(widget.wtype.end_time_secs * 1000)
    } else {
      nEndTime = new Date(Math.floor(redBorder.timeNow.getTime()/minute)*minute);
      if (widget.graph.step !== undefined && widget.graph.step <= (5).minutes()) {
        nEndTime = new Date(nEndTime.getTime() - widget.graph.step);
      }

      nStartTime = new Date(Math.ceil((((redBorder.timeNow.getTime()/1000) - diffSecs)*1000)/coef)*coef);
    }
    graph.forceDomain([nStartTime, nEndTime], true);

    // Delete all series
    graph.remove('Total');
    if (widget.property.analytics.indexOf('moving_average') !== -1) {
      graph.remove('Moving Average');
    }
    if (widget.property.analytics.indexOf('average') !== -1) {
      graph.remove('Average');
    }
    if (widget.property.analytics.indexOf('previous_period') !== -1) {
      graph.remove('Last range');
    }
    if (widget.property.analytics.indexOf('day_over_day') !== -1) {
      graph.remove('Day over day');
    }
  }

  function reload() {
    reloaded = true;
    // Draw new series
    getData();
  }

  // Trigger an event when the widget is reloaded
  function reloadDone(){
    // Create the event
    $container.parents('.widget').trigger('widget-reload');
  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', ($container.parents('.widget').width()) + 'px');
    graph.resize();
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('svg').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('svg').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
