redBorder.widgets.graphs.table = function (container, widget) {
  var $container = $('#' + container);
  var wtype;

  // Entry point for the graph
  function init(_wtype) {
    wtype = _wtype;
  }

  function render(){
    wtype.queryList().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    var product_type = widget.property.product_type,
        aggregation = widget.property.aggregation,
        aggregationData, unit, aggregation_name;

    $container.html("");
    if (aggregation === undefined){
      // We are on unique. There isn't aggregation.
      aggregation_name = 'Total';
    } else {
      aggregationData = redBorder.modules[product_type].aggregations[aggregation];
      unit = aggregationData.unit;
      aggregation_name = aggregationData.name;
    }

    if (data.length !== 0) {
      var columnText = widget.property.product_type === 'location' 
          ? I18n.t('location2.' + widget.property.column)
          : widget.property.column;
      $container.append("<table>");
      $("table", $container).append("<thead>");
      $("table", $container).append("<tbody>");
      $("table", $container).attr("class", "table table-striped table-condensed table-hover");
      $("table thead", $container).append("<tr>");
      $("table thead tr", $container).append($("<th>").text(columnText));
      $("table thead tr", $container).append($("<th>").attr("class", "last").text(aggregation));

      data.forEach(function (d, i) {

        var matchingLocation = redBorder.widgets.locations && redBorder.widgets.locations[d.name];
        if (matchingLocation) {
          d.name = matchingLocation;
        } else {
          d.name = widget.property.product_type == "location" ? (redBorder.widgets.locations && redBorder.widgets.locations[d.name] ? redBorder.widgets.locations[d.name] : d.name) : d.name;
        }

        events  = d.events;
        var name = d.name,
            multiplier = redBorder.widgets.util.getMultiplier(events),
            column = widget.property.column, elementclass = '';

        if (column === 'wan_ip_name' || column === 'lan_ip_name' ||
            column === 'dst_name' || column === 'src_name' ||
            column === 'public_ip_name') {
          elementclass = 'auto-resolv';
        }

        if (name === undefined || name === '' || name === null) name = 'N/A';

        if (i < widget.property.elements) {
          var $tr = $("<tr>").appendTo($("table tbody", $container));
          $("<td>").text(name).attr('class', elementclass).appendTo($tr);
          $("<td>").text((Math.round(events * multiplier.num * 100) / 100) +
            " " + multiplier.prefix).attr("class", "last").appendTo($tr);
        }
      });
      if (window.redBorder.resolve_dns === true) resolve_dns();
    } else {
      $container.append("<div class='center'><p>" + I18n.t('javascript.no_data') +"</p></div>");
    }

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function reload() {
    $container.html("");
    wtype.queryList().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });

  }

  function resize() {
    var height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', $container.parents('.widget').width() + 'px');
  }

  // Hide the main element when resizing
  function hide() {
    $container.find('table').fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.find('table').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return undefined;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
