redBorder.widgets.types.compare = function (widget, data) {
  var product_type = widget.property.product_type,
      aggregation = widget.property.aggregation,
      elements = widget.property.elements,
      range = widget.property.range,
      domainTime = widget.wtype.domain_time,
      domainTime, startDate, startDateAtHour,
      startDateSecs, startDateAtHourSecs;

  // We dont want the show total option activated, lets disable it
  widget.property.show_total = false;

  // Init the times to query on server
  function loadTimes(){
    // Starts dates. We need startDateAtHour if the compare widget is
    // using a range of 24 hours. If not, we use startDate.
    var date_time = redBorder.timeNow, day;
    $.each(widget.property.filters, function (index, value) {
      if (value['column'] == 'end_time')
        date_time = value['value'] * 1000;
    })

    startDate = new Date(date_time);
    startDate.setHours(0,0,0,0);
    startDateAtHour = new Date(date_time);
    if(range !== "pt24h") {
      startDateAtHour.setHours(0,0,0,0);
    } else {
      startDateAtHour.setMinutes(0,0,0);
    }
    if (range === "pt4320h") {
      day = startDate.getDate() || 30;
      if( day !== 1 ) startDate.setHours(-24 * (day - 1));
    } else if (range === "pt720h") {
      day = startDate.getDay() || 7;
      if( day !== 1 ) startDate.setHours(-24 * (day - 1));
    }

    // Converted to secs
    startDateSecs = parseInt(startDate.getTime() / 1000);
    startDateAtHourSecs = parseInt(startDateAtHour.getTime() / 1000);
  }

  // We will create a list with every element being the start date
  // of that serie, so when queryTimeserie gets called with that date
  // we just need to sum it the domainTime to create the interval that
  // we must query
  function queryList(opts) {
    var timeAux, name, list = [];

    // Load the times
    loadTimes();

    for (var i = 0; i < elements; i += 1) {
      if (range === "pt24h") {
        timeAux = new Date((startDateAtHourSecs - i * domainTime) * 1000);
      } else {
        timeAux = new Date((startDateSecs - i * domainTime) * 1000);
      }

      switch (range) {
        case 'pt24h':
          name = timeAux.getHours() + ':00 h - ' + timeAux.getDate() + '/' + (timeAux.getMonth() + 1);
          break;
        case 'pt168h':
          name = getWeekDayName(timeAux) + ' - ' + timeAux.getDate() + '/' + (timeAux.getMonth() + 1);
          break;
        case 'pt720h':
          name = I18n.t('javascript.week') + ' ' +  getWeekNumber(timeAux) + ' - ' + timeAux.getDate() + '/' + (timeAux.getMonth() + 1);
          break;
        case 'pt4320h':
          name = getMonthName(timeAux);
          break;
      }

      list.push({
        name: name,
        opts: timeAux
      })
    }

    return $.Deferred().resolve(list);
  }

  // Do the query to get a compare serie. The param is the start
  // date from the interval that we must query.
  function queryTimeserie(dateString) {
    if (data === undefined){
      var date = new Date(dateString), opts, dateToDiff
        endTime = new Date(date.getTime() + domainTime * 1000).toISOString();

      if (range === "pt24h") {
        dateToDiffSecs = startDateAtHourSecs;
      } else {
        dateToDiffSecs = startDateSecs;
      }

      if (range === "pt4320h") {
        var month = date.getMonth(),
            endTimeDate, dateDiff;

        date.setDate(1);

        dateDiff = new Date(startDateSecs * 1000);
        dateDiff.setDate(1);
        dateToDiffSecs = dateDiff.getTime() / 1000;
        endTimeDate = new Date(date);

        if (month + 1 <= 12) month = month + 1;
        else month = 1;

        endTimeDate.setMonth(month);
        endTime = endTimeDate.toISOString();
      }

      opts = {
        start_time_serie: date.toISOString(),
        end_time_serie: endTime,
        value: date.toISOString(),
        diff: (dateToDiffSecs - date.getTime() / 1000),
        comparative: true
      };

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_serie/' + range,
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone
        },
        graphOpts: opts
      });
    } else if (data.timeserie !== undefined){
      return $.Deferred().resolve(data.timeserie);
    } else {
      return $.Deferred().resolve(data.timeseries[dateString.getTime()/1000]);
    }
  }

  // Unimplemented
  function queryTotal(opts) {
    return $.Deferred().resolve(0);
  }

  // Gives the week number for a given Date object
  function getWeekNumber(d) {
      // Copy date so don't modify original
      d = new Date(+d);
      d.setHours(0,0,0);
      // Set to nearest Thursday: current date + 4 - current day number
      // Make Sunday's day number 7
      d.setDate(d.getDate() + 4 - (d.getDay()||7));
      // Get first day of year
      var yearStart = new Date(d.getFullYear(),0,1);
      // Calculate full weeks to nearest Thursday
      var weekNo = Math.ceil(( ( (d - yearStart) / 86400000) + 1)/7)
      // Return array of year and week number
      return weekNo;
  }

  function getWeekDayName(d) {
      var days = [
        'Sunday', 'Monday', 'Tuesday', 'Wednesday',
        'Thursday', 'Friday', 'Saturday'
      ];

      var day = d.getDay();
      return days[day];
  }

  function getMonthName(d) {
    var months = [
      'January', 'February', 'March', 'April', 'May',
      'June', 'July', 'August', 'September', 'October',
      'November', 'December'
    ];

    var monthNum = d.getMonth();
    return months[monthNum];
  }

  return {
    queryList: queryList,
    queryTimeserie: queryTimeserie,
    queryTotal: queryTotal
  }
}
