redBorder.widgets.types.raw = function (widget, data) {
  var product_type = widget.property.product_type;

  // Do the query to get the data
  // Return a deferred object
  function queryTimeserie(opts) {
    if (data === undefined || data.timeserie === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_serie/raw',
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          granularity: widget.property.granularity
        },
        graphOpts: opts
      });
    } else {
      return $.Deferred().resolve(data.timeserie);
    }
  }

  // Gets the total data from that interval
  function queryTotal(opts) {
    if (data === undefined || data.total === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_serie/raw',
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          granularity: 'all'
        },
        graphOpts: opts
      }).then(function (data) {
        if (data.length === 0) return 0;
        var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
        data = redBorder.widgets.util.checkPerSecondAggregationTotal(data, widget.property.aggregation, time);
        return data[0].event.events;
      });
    } else {
      if (data.length === 0){
        return $.Deferred().resolve(0);
      }

      // Check the aggregation
      var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
      if (data[0] === undefined){
        // Graph Ball
        data_tmp = data.total;
      } else {
        data_tmp = data;
      }
      data = redBorder.widgets.util.checkPerSecondAggregationTotal(data_tmp, widget.property.aggregation, time);

      return $.Deferred().resolve(data[0].event.events);
    }
  }

  // Gets the raw table
  function queryTable(columns) {
    if(data === undefined || data.raw_table === undefined) {
      return $.ajax({
        url: '/' + widget.property.product_type + '/tops/get_raw_table',
        type: 'GET',
        dataType: 'json',
        data: {
          start_time: widget.wtype.start_time,
          end_time: widget.wtype.end_time,
          range: widget.property.range,
          granularity: 'all',
          aggregation: widget.property.aggregation,
          columns: columns,
          w: true,
          filters: JSON.stringify(widget.property.filters),
          limit: widget.property.elements
        }
      });
    } else {
      return $.Deferred().resolve(data.raw_table);
    }
  }

  return {
    queryTable: queryTable,
    queryTimeserie: queryTimeserie,
    queryTotal: queryTotal
  }
}
