redBorder.widgets.types.top = function (widget, data) {
  var product_type = widget.property.product_type,
      aggregation = widget.property.aggregation,
      client_profile = widget.property.client_profile,
      column = widget.property.column,
      elements = widget.property.elements,
      column_name;

  if ($.inArray(String(column), ["src_name", "dst_name", "lan_ip_name", "wan_ip_name", "public_ip_name"]) != -1) {
    column_name = column;
    column = column.replace("_name", "");
  };

  // Do the query to get the tops data
  function queryList(opts) {
    if (data === undefined || data.list === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_tops/' + column,
        typeOpts: {
          aggregation: widget.property.aggregation,
          client_profile: widget.property.client_profile,
          timezone: widget.property.timezone,
          limit: elements
        },
        graphOpts: opts
      }).then(function (data) {

        // Temporal, for Location widgets
        if (widget.property.product_type === "location") {
          redBorder.widgets.locations = data.locations;
          data = data.data;
        }

        // Check the aggregation
        var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
        // data = redBorder.widgets.util.checkPerSecondAggregationTops(data, widget.property.aggregation, time);

        return redBorder.widgets.util.parseTopData(data, aggregation, column, elements, column_name);
      });
    } else {
      var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
      // parsedData = redBorder.widgets.util.checkPerSecondAggregationTops(data.list, widget.property.aggregation, time);
      parsedData = data.list;

      return $.Deferred().resolve(redBorder.widgets.util.parseTopData(parsedData, aggregation, column, elements, column_name));
    }
  }

  // Gets the total data from that interval
  //
  // @param [string] value Column to get the data
  // @param [boolean] total Indicate if this is a total query. In this case,
  //    the query don't apply filters
  //
  function queryTimeserie(value, total) {
    if (data === undefined) {
      var opts = {},
          ignoreFilters = false;

      if (value !== undefined) {
        opts = { value: value }
      }

      if (total === true) {
        ignoreFilters = true;
      }

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_serie/' + column,
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          granularity: widget.property.granularity,
          client_profile: widget.property.client_profile,
        },
        graphOpts: opts,
        ignoreFilters: ignoreFilters
      });
    } else if (data.timeserie !== undefined) {
      return $.Deferred().resolve(data.timeserie);
    } else if (data.total_timeserie !== undefined && total === true){
      return $.Deferred().resolve(data.total_timeserie);
    } else {
      return $.Deferred().resolve(data.timeseries[value]);
    }
  }

  // Gets the total data from that interval
  function queryTotal(opts) {
    if (data === undefined || data.total === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/tops/get_serie/' + column,
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          client_profile: widget.property.client_profile,
          granularity: 'all'
        },
        graphOpts: opts
      }).then(function (data) {
        if (data.length === 0) return 0;

        // Check the aggregation
        var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
        data = redBorder.widgets.util.checkPerSecondAggregationTotal(data, widget.property.aggregation, time);

        return data[0].event.events;
      });
    } else {
      if (data.length === 0){
        return $.Deferred().resolve(0);
      }
      // Check the aggregation
      var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
      parsedData = redBorder.widgets.util.checkPerSecondAggregationTotal(data.total, widget.property.aggregation, time);

      return $.Deferred().resolve(parsedData[0].event.events);
    }
  }

  // It queries the web app if it should get the colors from priorities
  // or darklist scores. Returns a deferred object.
  function getColors() {
    if (data === undefined || data.colors === undefined) {
      var d = new $.Deferred();
      return d.resolve([]);
    } else {
      if (widget.property.column === "priority" ||
          widget.property.column === "darklist_score_name" ||
          widget.property.column === "influence" ||
          widget.property.column === "client_rssi" ||
          widget.property.column === "action") {
        var d = new $.Deferred();
        return d.resolve([]);
      } else {
        return $.Deferred().resolve(data.colors);
      }
    }
  }

  return {
    queryList: queryList,
    queryTimeserie: queryTimeserie,
    queryTotal: queryTotal,
    getColors: getColors
  }
}
