redBorder.widgets.types.unique = function (widget, data) {
  var product_type = widget.property.product_type,
      column = widget.property.column,
      group_by = widget.property.group_by || undefined,
      elements = widget.property.elements;

  // Gets the total data from that interval
  function queryList(opts) {
    if (group_by == 'none') {
      return $.Deferred().resolve(false);
    } else if(data === undefined || data.list === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/' + column + '/load_cardinality_table',
        typeOpts: {
          timezone: widget.property.timezone,
          group_by: group_by,
          limit: elements
        },
        graphOpts: opts
      }).then(function (data) {
        return redBorder.widgets.util.parseTopData(data, 'result', group_by, elements);
      });
    } else {
      return $.Deferred().resolve(redBorder.widgets.util.parseTopData(data.list, 'result', column, elements));
    }
  }

  // Do the query to get the tops data
  function queryTimeserie(value) {
    if (data === undefined || (data.timeserie === undefined && data.timeseries === undefined)) {
      opts_tmp = {
        timezone: widget.property.timezone,
        granularity: widget.property.granularity,
        group_by: group_by || column
      }
      if (widget.property.widget_type === 'single_unique')
        opts_tmp['group_by'] = 'none'

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/cardinality/get_serie/' + column,
        typeOpts: opts_tmp,
        graphOpts: {
          granularity: widget.property.granularity,
          range: widget.property.range,
          value: value
        }
      });
    } else if (data.timeserie !== undefined) {
      return $.Deferred().resolve(data.timeserie);
    } else {
      return $.Deferred().resolve(data.timeseries[value]);
    }
  }

  // Gets the total data from that interval
  function queryTotal(opts) {
    if (data === undefined || data.total === undefined) {
      opts_tmp = {
        timezone: widget.property.timezone,
        granularity: 'all',
        group_by: column
      }
      if (widget.property.widget_type === 'single_unique')
        opts_tmp['group_by'] = 'none'

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/' + product_type + '/cardinality/get_serie/' + column,
        typeOpts: opts_tmp,
        graphOpts: opts
      }).then(function (data) {
        if (data.length === 0) return 0;
        var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
        data = redBorder.widgets.util.checkPerSecondAggregationTotal(data, widget.property.aggregation, time);
        return data[0].event.events;
      });
    } else {
      if (data.length === 0){
        return $.Deferred().resolve(0);
      }

      // Check the aggregation
      var time = widget.wtype.end_time_secs - widget.wtype.start_time_secs;
      data = redBorder.widgets.util.checkPerSecondAggregationTotal(data, widget.property.aggregation, time);

      return $.Deferred().resolve(data.total[0].event.events);
    }
  }

  // It queries the web app if it should get the colors from priorities
  // or darklist scores. Returns a deferred object.
  function getColors() {
    if (data === undefined || data.colors === undefined) {
      var d = new $.Deferred();
      return d.resolve([]);
    } else {
      if (widget.property.column === "priority" ||
          widget.property.column === "darklist_score_name" ||
          widget.property.column === "influence" ||
          widget.property.column === "client_rssi" ||
          widget.property.column === "action") {
        var d = new $.Deferred();
        return d.resolve([]);
      } else {
        return $.Deferred().resolve(data.colors);
      }
    }
  }

  return {
    queryList: queryList,
    queryTimeserie: queryTimeserie,
    queryTotal: queryTotal,
    getColors: getColors
  }
}
