/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * This JS create the form to edit/create/destroy/save namespace rules
 */

redBorder.namespace = {}

redBorder.namespace.rulesBuilder = function(rulesJSON, tiers, container, options) {
  var rulesJSON = rulesJSON,
      rules = [],
      tiers = JSON.parse(tiers),
      container = container,
      $container = $(container),
      disabledAllInputs = '',
      disabledAll = false;

  if (options.disabled !== undefined && options.disabled){
    disabledAllInputs = ' disabled="disabled" ';
    disabledAll = true;
  }

  // Print rules
  initialize();

  /*
   * Initialize rule objects
   */
  function initialize() {
    var rules_tmp = JSON.parse(rulesJSON);
    for(var i = 0; i < rules_tmp.length; i++){
      rules.push(new NamespaceRule(rules_tmp[i]));
    }
    // Add rule button
    //html = buildAddRuleButton();
    // Build HTML
    html = buildRules();

    // Add to the container
    $container.html(html);
    $container.sortable({
      itemSelector: '.namespace-rule',
      handle: '.namespace-rule i.fa-arrows',
      onDrop: function($item, container, _super, event) {
        $item.removeClass("dragged").removeAttr("style")
        $("body").removeClass("dragging")
        // Iterate over rules to update number
        new_rules = []
        $container.find('.namespace-rule').each(function(index){
          // Last position
          var last_position = $(this).data('rule-position'),
              rule = rules[last_position];
          new_rules.push(rule);
          // Update title and position
          $(this).data('rule-position', index);
          $(this).attr('data-rule-position', index);
          $(this).find('.nsr-title-wrap').html(buildRuleTitle(rule, index));
        });
        // Apply it
        rules = new_rules
      }
    });
    // Create the callbacks
    createCallbacks();
  }

  /*
   * Create multiple callbacks and listener
   */
  function createCallbacks(){
    // Create a new rule. We need to create this callback even disabled is true
    // because this button maybe external to the library
    $(document).on('click', '.nsr-add-rule', function(event){
      event.preventDefault();
      if(disabledAll) { return; }
      addRule();
    });

    // Don't create more callbacks
    if(disabledAll) { return; }

    $(document).on('click', '.delete-rule', function(event) {
      var $rulesInput = $('#namespace_rules');
      var $rule       = $(this).closest('.namespace-rule');

      var rulePos = $rule.data('rule-position');
      rules.splice(rulePos, 1);
      $rulesInput.attr('value', JSON.stringify(rules));

      // Redraw rules
      var html = buildRules();
      $container.html(html);
    });

    // Change the type of a rule
    $(document).on('change', '.namespace-rule select.nsr-type', function(event){
      var $el = $(this),
          $nsrule = $el.parents('.namespace-rule'),
          value = $el.val(),
          position = $nsrule.data('rule-position');

      // Hide/Show replicants
      if(value == 'load'){
        // Show replicants
        $nsrule.find('.nsr-replicants').removeClass('hidden');
      } else {
        $nsrule.find('.nsr-replicants').addClass('hidden');
      }
      // Update the rule and the summary
      rules[position].type = value;
      $nsrule.find('.nsr-title-wrap').html(buildRuleTitle(rules[position], position));
      $nsrule.find('.nsr-summary').html(summaryFromRule(rules[position]));
    });

    $(document).on('change', '.namespace-rule select.nsr-time-type', function(event){
      var $el = $(this),
          $nsrule = $el.parents('.namespace-rule'),
          value = $el.val(),
          position = $nsrule.data('rule-position');

      // Hide/Show replicants
      if(value == 'ByInterval'){
        // Show replicants
        $nsrule.find('.nsr-interval').removeClass('hidden');
        $nsrule.find('.nsr-period').addClass('hidden');
      } else if(value == 'ByPeriod') {
        $nsrule.find('.nsr-interval').addClass('hidden');
        $nsrule.find('.nsr-period').removeClass('hidden');
        // Calculate period
        var multiplier = $nsrule.find('select.nsr-multiplier').val();
        var number = $nsrule.find('input.nsr-number-period').val();
        // Update the rule
        if (multiplier === 'H') {
          rules[position].period = 'PT' + number + multiplier;
          console.log("PERIOD PT ==> " +  rules[position].period + "\n\n");

        } else {
          rules[position].period = 'P' + number + multiplier;
          console.log("PERIOD P ==> " +  rules[position].period + "\n\n");

        }
      } else {
        $nsrule.find('.nsr-interval').addClass('hidden');
        $nsrule.find('.nsr-period').addClass('hidden');
        var number = $nsrule.find('input.nsr-number-period').val('1');
      }
      // Update the rule and the summary
      rules[position].time_type = value;
      $nsrule.find('.nsr-summary').html(summaryFromRule(rules[position]));
    });

    $(document).on('input', '.namespace-rule input.nsr-number-period', function(event){
      var $el = $(this),
          $nsrule = $el.parents('.namespace-rule'),
          multiplier = $nsrule.find('select.nsr-multiplier').val(),
          value = $el.val(),
          position = $nsrule.data('rule-position');

      var time_period_field = document.getElementsByName('time_period');
      var regex_time_period_field = new RegExp("^([1-9]{1}[0-9]{0,9})$");

      for(var i = 0; i < time_period_field.length; i++){
        if(regex_time_period_field.test(time_period_field[i].value)){
          time_period_field[i].setCustomValidity('');
          
        }
        else{
          time_period_field[i].setCustomValidity('Invalid input');
          time_period_field[i].reportValidity();
        }
      }
      // Update the rule and the summary
      if (multiplier === 'H') {
        rules[position].period = 'PT' + value + multiplier;
      } else {
        rules[position].period = 'P' + value + multiplier;
      }
      $nsrule.find('.nsr-summary').html(summaryFromRule(rules[position]));
    });

    $(document).on('input', '.namespace-rule select.nsr-multiplier', function(event){
      var $el = $(this),
          $nsrule = $el.parents('.namespace-rule'),
          multiplier = $el.val(),
          value = $nsrule.find('input.nsr-number-period').val(),
          position = $nsrule.data('rule-position');

      // Update the rule and the summary
      if (multiplier === 'H') {
        rules[position].period = 'PT' + value + multiplier;
      } else {
        rules[position].period = 'P' + value + multiplier;
      }
      $nsrule.find('.nsr-summary').html(summaryFromRule(rules[position]));
    });

    $(document).on('click', '.namespace-rule .nsr-new-tier', function(event){
      event.preventDefault();
      var $el = $(this),
          $replicants = $(this).siblings('.nsr-replicants-group'),
          $nsrule = $el.parents('.namespace-rule'),
          position = $nsrule.data('rule-position');

      if($replicants.find('.nsr-replicant-selector').length > 0){
        return;
      } else {
        $replicants.append(buildTierSelector(rules[position]));
      }
    });
    // Replicants Callbacks
    replicantsCallback();
  }

  function replicantsCallback() {
    $(document).on('change', '.namespace-rule .nsr-tier-selector', function(event){
      var $el = $(this),
          $replicants = $(this).parents('.nsr-replicants-group'),
          $nsrule = $el.parents('.namespace-rule'),
          position = $nsrule.data('rule-position'),
          rule = rules[position],
          tier = $el.val();

      if(tier != undefined && tier != null && tier != ''){
        $el.parent().remove();
        replicants = rule.replicants || {};
        replicants[tier] = 1
        rule.replicants = replicants
        $replicants.append(buildReplicant(tier, 1));
        $nsrule.find('.nsr-summary').html(summaryFromRule(rule));
      }
    });

    $(document).on('click', '.namespace-rule .nsr-replicant-button a', function(event){
      event.preventDefault();

      var $el = $(this),
          $replicant = $el.parents('.nsr-replicant'),
          $nsrule = $el.parents('.namespace-rule'),
          $blocks = $replicant.find('.nsr-replicant-container'),
          position = $nsrule.data('rule-position'),
          rule = rules[position];

      var count = $replicant.find('.nsr-replicant-block').length;

      if (count < 4){
        $blocks.append(buildBlock());
        rule.addReplicant($replicant.data('tier-name'));
        $nsrule.find('.nsr-summary').html(summaryFromRule(rule));
      }
    });

    // Add callback to blocks
    $(document).on('click', '.namespace-rule .nsr-replicant .nsr-replicant-block', function(event){
      var $el = $(this),
          $replicant = $el.parents('.nsr-replicant'),
          $nsrule = $el.parents('.namespace-rule'),
          $blocks = $replicant.find('.nsr-replicant-container'),
          position = $nsrule.data('rule-position'),
          rule = rules[position];

      // Get the number
      var count = $('.nsr-replicant-block').length - 1;
      // Remove!
      $el.remove();
      rule.removeReplicant($replicant.data('tier-name'));
      $nsrule.find('.nsr-summary').html(summaryFromRule(rule));
    });
  }

  /*
   * Destroy the callbacks generated by the library
   */
  function destroyCallbacks() {
    $(document).off('click', '.nsr-add-rule');
    $(document).off('click', '.delete-rule');
    $(document).off('change', '.namespace-rule select.nsr-type');
    $(document).off('change', '.namespace-rule select.nsr-time-type');
    $(document).off('input', '.namespace-rule input.nsr-number-period');
    $(document).off('input', '.namespace-rule select.nsr-multiplier');
    $(document).off('click', '.namespace-rule .nsr-new-tier');
    $(document).off('change', '.namespace-rule .nsr-tier-selector');
    $(document).off('click', '.namespace-rule .nsr-replicant-button a');
    $(document).off('click', '.namespace-rule .nsr-replicant .nsr-replicant-block');
  }

  /*
   * Create the button to add a new rule
   *
   * @return [String] HTML Code
   */
  function buildAddRuleButton() {
    return '<div class="row">' +
             '<div class="col-md-12 margin-bm">' +
               '<a href="#" class="btn btn-success nsr-add-rule">' +
                 '<i class="fa fa-plus"></i>' + I18n.t('add_rule') + '</a>' +
              '</div>' +
            '</div>';
  }

  /*
   * Create the form to edit all rules given
   *
   * @return [String] HTML Code
   */
  function buildRules() {
    rules_html = []
    for(var i = 0; i < rules.length; i++){
      rules_html.push(buildRule(rules[i], i));
    }
    // Return the HTML
    return rules_html.join('');
  }

  /*
   * Add a new Rule to the form
   *
   */
  function addRule() {
    rule = new NamespaceRule({ type: 'loadByPeriod', tieredReplicants: { _default_tier: 1 }, period: 'P1D' });
    rules.push(rule);
    html = buildRule(rule, rules.length - 1);
    $container.append(html);
    $container.sortable("refresh");
  }

  /*
   * Create the form to edit a rule
   *
   * @param [Object] Rule to build
   * @param [Integet] Number of the current rule
   *
   * @return [String] HTML Code
   */
  function buildRule(rule, number) {
    html ='<div class="namespace-rule" data-rule-position="' + number + '">' +
            buildRuleTitle(rule, number) +
            '<div class="nsr-form">' +
              '<div class="row">' +
                '<div class="col-md-8">' +
                  '<div class="row">' +
                    buildRuleTypeSelector(rule) +
                    buildRuleTimeTypeSelector(rule) +
                    '<div class="col-md-4">' +
                      buildRuleTimeIntervalInput(rule) +
                      buildRuleTimePeriodInput(rule) +
                    '</div>' +
                    buildRuleReplicants(rule) +
                  '</div>' +
                '</div>' +
                '<div class="col-md-4 nsr-summary margin-tl">' +
                  summaryFromRule(rule) +
                '</div>' +
              '</div>' +
            '</div>' +
          '</div>';
    return html;
  }

  /*
   * Create the title of a rule
   *
   * @param [NamespaceRule] rule Rule to build
   * @param [Integer] Number of the current rule
   *
   * @return [String] HTML Code
   */
  function buildRuleTitle(rule, number) {
    return '<div class="row nsr-title-wrap">' +
      '<div class="col-md-8">' +
        '<h4 class="nsr-title"><i class="fa fa-arrows"></i> ' +
        capitalize(rule.type) + ' rule #' + number + '<span class="delete-rule"><i class="fa fa-trash"></i>' + I18n.t('delete') + '</span>' +
        '</h4>' +
      '</div><div class="col-md-4">' +
        '<h4 class="nsr-summary-title text-right">' + I18n.t('summary') + '</h4>' +
      '</div>' +
    '</div>';
  }

  /*
   * Create the input to change the rule type
   *
   * @param [NamespaceRule] rule Rule to build
   *
   * @return [String] HTML Code
   */
  function buildRuleTypeSelector(rule) {
    var load_selected = rule.isLoad() ? 'selected="selected"' : '';
    var drop_selected = rule.isDrop() ? 'selected="selected"' : '';
    return '<div class="col-md-2">' +
             '<div class="form-group">' +
                '<label>' + I18n.t('type') + '</label>' +
                '<select ' + disabledAllInputs + ' name="type" class="nsr-type form-control select">' +
                  '<option value="load" '+ load_selected +'>' + I18n.t('load2') + '</option>' +
                  '<option value="drop" '+ drop_selected +'>' + I18n.t('drop') + '</option>' +
                '</select>' +
              '</div>' +
            '</div>';
  }

  /*
   * Create the input to change the rule time type
   *
   * @param [Object] Rule to build
   *
   * @return [String] HTML Code
   */
  function buildRuleTimeTypeSelector(rule) {
    var period_selected = rule.isPeriod() ? 'selected="selected"' : '';
    var interval_selected = rule.isInterval() ? 'selected="selected"' : '';
    var forever_selected = rule.isForever() ? 'selected="selected"' : '';
    return '<div class="col-md-2">' +
             '<div class="form-group">' +
                '<label>' + I18n.t('time_mode') + '</label>' +
                '<select ' + disabledAllInputs + ' name="time_type" class="nsr-time-type form-control select">' +
                  '<option value="ByPeriod" '+ period_selected +'>' + I18n.t('period') + '</option>' +
                  //'<option value="ByInterval" '+ interval_selected +'>Interval</option>' +
                  '<option value="Forever" '+ forever_selected +'>' + I18n.t('forever') + '</option>' +
                '</select>' +
              '</div>' +
            '</div>';
  }

  /*
   * Create the input to change the rule time with time mode is Interval
   *
   * @param [NamespaceRule] Rule to build
   *
   * @return [String] HTML Code
   */
  function buildRuleTimeIntervalInput(rule) {
    var hidden = rule.isInterval() ? '' : 'hidden';
    return '<div class="form-group nsr-interval ' + hidden + '">' +
              '<label>' + I18n.t('time_interval') + '</label>' +
              '<input ' + disabledAllInputs + ' name="start_time" class="form-control string nsr-start-time" />' +
              '<input ' + disabledAllInputs + ' name="end_time" class="form-control string nsr-end-time" />' +
            '</div>';
  }

  /*
   * Create the input to change the rule time with time mode is Period
   *
   * @param [NamespaceRule] Rule to build
   *
   * @return [String] HTML Code
   */
  function buildRuleTimePeriodInput(rule) {
    var hidden = rule.isPeriod() ? '' : 'hidden', parsed, hourActive = '',
        dayActive = '', monthActive = '', yearActive = '', value = 1;
    // Build the period

    if (rule.isPeriod() && rule.period.indexOf('T') > -1) {
      parsed = /PT([0-9]+).*/.exec(rule.period);
      value = parsed[1];
      hourActive = 'selected="selected"';
    } else if (rule.isPeriod()){
      parsed = /P([0-9]+)([A-Z]{1})/.exec(rule.period);
      value = parsed[1];
      switch(parsed[2]) {
        case 'D':
          dayActive = 'selected="selected"';
          break;
        case 'M':
          monthActive = 'selected="selected"';
          break;
        case 'Y':
          yearActive = 'selected="selected"';
          break;
      }
    } else {
      dayActive = 'selected="selected"';
    }

    return '<div class="form-group nsr-period ' + hidden + '">' +
              '<label>' + I18n.t('time_period') + '</label>' +
              '<div>' +
                '<input ' + disabledAllInputs + ' name="time_period" class="form-control string nsr-number-period"' +
                  ' type="number" min="1" value="' + value + '"required/>' +
                '<select ' + disabledAllInputs + ' name="time_multiplier" class="form-control nsr-multiplier">' +
                  '<option value="H" ' + hourActive + '>' + I18n.t('hour') + '</option>' +
                  '<option value="D" ' + dayActive + '>' + I18n.t('day') + '</option>' +
                  '<option value="M" ' + monthActive + '>' + I18n.t('month') + '</option>' +
                  '<option value="Y" ' + yearActive + '>' + I18n.t('year') + '</option>' +
                '</select>' +
              '</div>' +
            '</div>';
  }

  /*
   * Create the input to set the number of replicants
   *
   * @param [NamespaceRule] rule Rule to build
   *
   * @return [String] HTML Code
   */
  function buildRuleReplicants(rule) {
    var hidden = rule.isDrop() ? 'hidden' : '';
    var disabledButtons = disabledAll ? 'disabled' : ''
    html = '<div class="col-md-4 nsr-replicants ' + hidden + '">' +
             '<div class="form-group">' +
                '<label>' + I18n.t('replicants') + '</label>' +
                '<a href="#" class="nsr-new-tier ' + disabledButtons + '">' +
                  '<i class="fa fa-plus"></i>' + I18n.t('add_tier') +
                '</a>' +
                '<div class="nsr-replicants-group">';
    // Build replicants
    for(var tier in rule.replicants){
      html += buildReplicant(tier, rule.replicants[tier]);
    }
    // Close tags adn return
    html += '</div></div></div>';
    return html;
  }

  /*
   * Create an input to set replicants in a Tier
   *
   * @param [String] tier Name of the Tier
   * @param [Integer] number Number of replicants in this Tier
   *
   * @return [String] HTML Code
   */
  function buildReplicant(tier, number){
    var disabledButtons = disabledAll ? 'disabled' : ''
    html ='<div class="nsr-replicant clearfix ' + disabledButtons + '" data-tier-name="' + tier + '">' +
            '<div class="nsr-replicant-container">';
    // Add the number of replicas
    for(var i = 0; i < number; i++){
      html += buildBlock();
    }

    // Close element
    html += '</div>' +
            '<div class="nsr-replicant-button">' +
              '<a href="#" class="btn btn-small btn-default ' + disabledButtons + '">' +
                '<i class="fa fa-plus"></i> ' + tierName(tier) +
              '</a>' +
            '</div>' +
          '</div>';
    // Return the HTML!
    return html;
  }

  /*
   * Build a block for a replicant
   *
   * @return [String] HTML Code
   */
  function buildBlock() {
    return '<div class="nsr-replicant-block"></div>';
  }

  /*
   * Create a selectorfor a new tier.
   *
   * @param [NamespaceRule] rule Rule to build
   *
   * @return [String] HTML Code
   */
  function buildTierSelector(rule) {
    html ='<div class="nsr-replicant-selector">' +
            '<select name="nsr-tier-selector" class="form-control nsr-tier-selector">' +
              '<option>' + I18n.t('select_tier') + '</option>';
    // Delete used tiers
    var used_tiers = [];
    for(var rep in rule.replicants){
      used_tiers.push(tierName(rep));
    }
    // Get only unused tiers
    tiers = $(tiers).not(used_tiers).get();
    // Build options
    for(var i = 0; i < tiers.length; i++){
      html += '<option value="' + tiers[i] + '">' + tiers[i] + '</option>';
    }
    // Close tags
    html += '</select>' +
          '</div>';

    return html;
  }

  /*
   * Create a Summary based on a given Rule
   *
   * @param [NamespaceRule] rule Rule to get the summary
   *
   * @return [String] Summary of the rule
   */
  function summaryFromRule(rule) {
    text = I18n.t('this_rule_html') + I18n.t(rule.type + '2') + I18n.t('the_data_html');

    if(rule.isInterval()){
      text += I18n.t('of_html') + rule.period + I18n.t('interval_html');
    } else if(rule.isPeriod()){
      // Read from pattern
      period = readTimePeriod(rule.period);
      aux = period.split(' ');
      text += I18n.t('during_html') + aux[0] + ' ' + I18n.t(aux[1]) + '</b>';
    } else {
      text += I18n.t('forever_html');
    }

    // We don't need more info about a Drop rule
    if (rule.isDrop()){
      return text;
    }

    // Check replicants
    rep = []
    for(var tier in rule.replicants){
      if (rule.replicants[tier] > 0){
        rep.push('<b>' + tierName(tier) + I18n.t('tier_html') + rule.replicants[tier] + ' replicas)');
      }
    }
    // replace last "," with "and"
    if (rep.length == 0){
      // No replicants!!
      text = I18n.t('no_replica_html');
      return text;
    } else if (rep.length > 1) {
      var last_el = rep.pop();
      rep[rep.length - 1] = rep[rep.length - 1] + ' and ' + last_el;
    }

    text += I18n.t('data_stored');
    text += rep.join(', ');
    return text;
  }

  /*
   * Return default tier name instead of '_default_tier'
   *
   * @param [String] tier Tier name
   *
   * @return [String] Tier name
   */
  function tierName(tier){
    if (tier == '_default_tier'){
      return 'default';
    } else {
      return tier;
    }
  }

  /*
   * Read the time from a ISO8601 period
   *
   * @param [String] time String with a ISO8601 pattern
   *
   * @return [String] comprensible string with time
   */
  function readTimePeriod(time){
    if (time.indexOf('T') > -1) {
      var data = /PT([0-9]+).*/.exec(time),
          
          multiplier = 'H';
          if (data )  
            var number = data[1];
    } else {
      var data = /P([0-9]+)([A-Z]{1})/.exec(time)
        
      if (data ){    
        var number = data[1],
        multiplier = data[2];
      }
    }
    // Check the multiplier
    humanized_mutiplier = ''
    if(multiplier == 'D'){
      humanized_mutiplier = 'day';
    } else if (multiplier == 'M') {
      humanized_mutiplier = 'month';
    } else if (multiplier == 'H') {
      humanized_mutiplier = 'hour';
    } else {
      humanized_mutiplier = 'year';
    }
    // Check pluralization
    if (number > 1){
      humanized_mutiplier += 's';
    }
    return number + ' ' + humanized_mutiplier;
  }

  /*
   * Capitalize a word
   *
   * @param [String] str Word
   *
   * @return [String] String capitalized
   */
  function capitalize(str) {
    return str.charAt(0).toUpperCase() + str.slice(1);
  }

  /*
   * Create a Summary of a rule.
   *
   * @param [String] type Type of the rule
   * @param [String] type_time Time mode selected
   * @param [String] time value of the time
   * @param [Object] replicants Replicants for this rule
   *
   * @return [String] Summary of the rule
   */
  function summary(type, type_time, time, replicants) {
    // Base data object
    data = { tieredReplicants: replicants };
    data.type = type + type_time;

    if (type_time == 'ByInterval') {
      data.interval = time;
    } else if (type_time == 'ByPeriod') {
      data.period = time;
    }
    // Build the summary based on rule
    return summaryFromRule(new NamespaceRule(data));
  }

  /*
   * Return rules
   *
   * @return [Array (NamespaceRule)] all rules
   */
  function getRules(){
    var formatted_rules = []
    for(var i = 0; i < rules.length; i++){
      formatted_rules.push(rules[i].asJSON());
    }
    return formatted_rules;
  }

  /*
   * Return rules in JSON format
   *
   * @return [Array (NamespaceRule)] all rules
   */
  function getRulesJSON(){
    // Get JSON
    return JSON.stringify(rules);
  }

  // Returned interface
  return {
    addRule: addRule,
    getRules: getRules,
    getRulesJSON: getRulesJSON,
    destroyCallbacks: destroyCallbacks
  }
}

// Rule Object!

/*
 * Constructor of NamespaceRule.
 *
 * @param [Object] data Rule associated data.
 * @return [NamespaceRule] Instance of Namespace Rule
 */
function NamespaceRule(data){
  this.type = data.type.substring(0,4);
  this.time_type = data.type.substring(4,data.type.length);
  this.replicants = data.tieredReplicants;

  if (this.isPeriod()) {
    this.period = data.period;
  } else if (this.isInterval()){
    this.interval = data.interval;
  }
}

// Check the type of the Rule
// --------------------------------------------

NamespaceRule.prototype.isLoad = function() {
  return this.type === 'load'
}

NamespaceRule.prototype.isDrop = function() {
  return this.type === 'drop'
}

NamespaceRule.prototype.isPeriod = function() {
  return this.time_type === 'ByPeriod'
}

NamespaceRule.prototype.isInterval = function() {
  return this.time_type === 'ByInterval'
}

NamespaceRule.prototype.isForever = function() {
  return this.time_type === 'Forever'
}

/*
 * Return Tiem value. If it's a forever rule, this method will return an
 * empty String
 *
 * @return [String] time value
 */
NamespaceRule.prototype.time = function() {
  if(this.isInterval()) {
    return this.interval;
  } else if (this.isPeriod()){
    return this.period;
  } else {
    return '';
  }
}

/*
 * Add a replicant to a tier
 *
 * @param [String] tier Name of the tier
 */
NamespaceRule.prototype.addReplicant = function(tier) {
  if (this.replicants[tier] !== undefined){
    this.replicants[tier] += 1
  } else {
    this.replicants[tier] = 1
  }
}

/*
 * Remove a replicant to a tier
 *
 * @param [String] tier Name of the tier
 */
NamespaceRule.prototype.removeReplicant = function(tier) {
  if (this.replicants[tier] !== undefined && this.replicants[tier] > 1){
    this.replicants[tier] -= 1
  } else if (this.replicants[tier] !== undefined) {
    delete this.replicants[tier]
  }
}

/*
 * Convert current Rule as JSON
 *
 * @return [Object] Javascript Object
 */
NamespaceRule.prototype.asJSON = function() {
  var self = this;
  var formatted = {
    type: self.type + self.time_type
  }

  if(self.isInterval()) {
    formatted.interval = self.interval;
  } else if (self.isPeriod()){
    formatted.period = self.period;
  }

  // Get Tiers
  if(self.isLoad()){
    formatted.tieredReplicants = self.replicants;
  }
  // Return JSON
  return formatted;
}

/*
 * Convert current Rule to JSON
 *
 * @return [String] JSON string
 */
NamespaceRule.prototype.toJSON = function() {
  // Return JSON
  return JSON.stringify(this.asJSON());
}
