/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder      = redBorder || {};
redBorder.util = redBorder.util || {};

redBorder.util.fitBody = function () {
  var $filters = $('.navbar .filter-dropdown');
  var initialMargin = 75;

  $('body').css({
    'margin-top': $filters.height() + initialMargin + 'px'
  });
}

function resultsDropdown (selector, html) {
  $search_result = $(selector)
  $search_result.empty();
  $search_result.append(html);
  $search_result.show();
}

function noticeFilterExpansion () {
  $.ajax({
    url: '/filters/expanded',
    data: {
      simple: $('.navbar > .filter-dropdown').hasClass('show'),
      advanced: !$('.navbar .advanced').hasClass('hide')
    }
  });
}

function autocompleteSensors(search, callback) {
  $.ajax({
    url: '/sensors/search',
    data: {
      search: search
    },
    dataType: 'json',
    error: function () {},
    success: callback
  });
}

$(document).ready(function () {
  $('.global-column-filter').trigger('change');
});

// Listener for the global filter open button
$(document).on('click', '.navbar #global-filters', function (event) {
  event.preventDefault();
  $('.navbar > .filter-dropdown').toggleClass('show');
  noticeFilterExpansion();
  redBorder.util.fitBody();
});

// Listener for the global filter close button
$(document).on('click', '.navbar .filter-dropdown .cls', function (event) {
  $('.navbar > .filter-dropdown').removeClass('show');
  noticeFilterExpansion();
  redBorder.util.fitBody();
});

// Listener for the show advanced button
$(document).on('click', '.navbar #show-advanced', function (event) {
  event.preventDefault();
  $('.navbar .advanced').toggleClass('hide');
  noticeFilterExpansion();
  redBorder.util.fitBody();
});

// Listeners for the domain result dropdowns
$(document).on('click', '.navbar .sensor-result', function (event) {
  event.preventDefault();
  var $input  = $('.navbar .global-sensor_name-filter');
  var $sensor = $(this).find('.sensor_name-result');

  $input.val(
    $sensor.text().trim()
  );
  $('.navbar .global-sensor_name-hidden-filter').val( $sensor.data('uuid') );
  $('#global-sensor_name-results').hide();
});

// Listener for the domain search input
$(document).on('keydown', '.navbar .global-sensor_name-filter', function (event) {
  var selector = "#global-sensor_name-results";
  var $search_result = $(selector);
  var input = $(this).val();
  var autocompleteCallback = function (data) {
      var html = "";
      data.forEach(function (sensor) {
        var source   = $("#global-sensor_name-results-template").html();
        var template = Handlebars.compile(source);
        var context  = {
          path: sensor.path,
          uuid: sensor.uuid
        };
        html += template(context);
      });
      resultsDropdown(selector, html);
    };

  if (event.keyCode == 27) {
    // ESC key pressed
    $search_result.hide();
  } else if (event.keyCode == 13) {
    // ENTER key pressed => do NOT submit the form
    event.preventDefault();
    autocompleteSensors( input, autocompleteCallback );
    return false;
  } else if (input.length >= 2) {
    autocompleteSensors( input, autocompleteCallback );
  }
  return;
});

$(document).keydown(function(e){
  if (e.which == 13 && $('.filter-dropdown.show').length &&
      !$('.global-sensor_name-filter').is(':focus') &&
      !$('#global-range-selector').is(':focus')) {
    event.preventDefault();
    $('form#global-filter-form').submit();
    blockWebUI();
  }
});

$(document).on('change', 'form#global-filter-form span.filter > select', function (event) {
  event.preventDefault();
  $('form#global-filter-form').submit();
  blockWebUI();
});


// Listener for the module specific filters
$(document).on('change', '.navbar .global-column-filter', function (event) {
  if ( $(this).find(':selected').val() === '' ) {
    $(this).parent('.key-value-pair').find('.modules').html('');
  } else {
    var source   = $('#labels-template').html();
    var template = Handlebars.compile(source);
    var context  = { labels: $(this).find(':selected').data('modules').split(',') };
    var html     = template(context);
    $(this).parent('.key-value-pair').find('.modules').html("in " + html);
  }
});

$(document).on('click', '.navbar .advanced .remove-row', function (event) {
  var $parent = $(this).parent('.key-value-pair');
  $parent.find('.global-column-filter').val('');
  $parent.find('.global-value-filter').val('');

  // Dont allow to remove row if there are only one. Just clean it
  if($('.navbar .advanced .remove-row').length > 1){
    $parent.remove();
  }
});

// Listener for clear the sensor input
$(document).on('click', '.navbar #global-filter-form .clear-sensor', function(event) {
  $('input.global-sensor_name-filter').val('');
  $('ul#global-sensor_name-results').hide();
});

$(document).ready(redBorder.util.fitBody);
