/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

$(document).on('change', '#overlay_maps input.toggle_enabled', function () {
  var csrf = $('meta[name="csrf-token"]').attr('content');
  var overlay_map_id = $(this).parent('td').data('id');
  $.ajax({
    url: "/overlay_maps/"+ overlay_map_id,
    type: "PATCH",
    dataType: "json",
    data: {overlay_map: {enabled: $(this).is(':checked') === true},
           authenticity_token: csrf}
  });
});

function add_image_to_rectangle(sw_lat, sw_lng, ne_lat, ne_lng, srcImage) {

  var swBound = new google.maps.LatLng(sw_lat, sw_lng);
  var neBound = new google.maps.LatLng(ne_lat, ne_lng);
  var bounds = new google.maps.LatLngBounds(swBound, neBound);

  overlay = new google.maps.GroundOverlay(srcImage, bounds);
  overlay.setMap(map);

}

// Show the new coordinates for the rectangle in an info window.

/** @this {google.maps.Rectangle} */
function showNewRect(event) {
  if (overlay) {overlay.setMap(null)}
  var ne = rectangle.getBounds().getNorthEast();
  var sw = rectangle.getBounds().getSouthWest();

  $('input#overlay_map_ne').attr('value', ne.lat().toFixed(3) + ',' + ne.lng().toFixed(3));
  $('input#overlay_map_sw').attr('value', sw.lat().toFixed(3) + ',' + sw.lng().toFixed(3));

  add_image_to_rectangle(sw.lat(), sw.lng(), ne.lat(), ne.lng(), srcImage)

  var contentString = I18n.t('location_html') +
      I18n.t('new_ne') + ne.lat().toFixed(3) + ', ' + ne.lng().toFixed(3) + '<br>' +
      I18n.t('new_sw') + sw.lat().toFixed(3) + ', ' + sw.lng().toFixed(3);

  // Set the info window's content and position.
  infoWindow.setContent(contentString);
  infoWindow.setPosition(ne);

  infoWindow.open(map);
}

function overlay_map_context_menu(){

  //  create the ContextMenuOptions object
  var contextMenuOptions={};
  contextMenuOptions.classNames={menu:'context_menu', menuSeparator:'context_menu_separator'};

  //  create an array of ContextMenuItem objects
  var menuItems=[];
  menuItems.push({className:'context_menu_item', eventName:'zoom_in_click', label:'Zoom in'});
  menuItems.push({className:'context_menu_item', eventName:'zoom_out_click', label:'Zoom out'});
  //  a menuItem with no properties will be rendered as a separator
  menuItems.push({});
  menuItems.push({className:'context_menu_item', eventName:'center_map_click', label:'Center map here'});
  menuItems.push({});
  menuItems.push({className:'context_menu_item', eventName:'add_location_in_click', label:'Add Location'});

  contextMenuOptions.menuItems=menuItems;

  //  create the ContextMenu object
  var contextMenu=new ContextMenu(map, contextMenuOptions);

  //  display the ContextMenu on a Map right click
  google.maps.event.addListener(map, 'rightclick', function(mouseEvent){
    contextMenu.show(mouseEvent.latLng);
  });

  //  listen for the ContextMenu 'menu_item_selected' event
  google.maps.event.addListener(contextMenu, 'menu_item_selected', function(latLng, eventName){
    //  latLng is the position of the ContextMenu
    //  eventName is the eventName defined for the clicked ContextMenuItem in the ContextMenuOptions
    switch(eventName){
      case 'add_location_in_click':
        add_overlay_map(latLng.lat() + 1, latLng.lng() - 1, latLng.lat() - 1, latLng.lng() + 1, srcImage, map.getZoom(), map.getDiv().id)
        break;
      case 'zoom_in_click':
        map.setZoom(map.getZoom()+1);
        break;
      case 'zoom_out_click':
        map.setZoom(map.getZoom()-1);
        break;
      case 'center_map_click':
        map.panTo(latLng);
        break;
    }
  });

}

function add_overlay_map(sw_lat, sw_lng, ne_lat, ne_lng, srcImage, zoom, map_container) {

  map_container = map_container || 'map_container';

  srcImage = srcImage || null;
  var mapOptions = {
    center: new google.maps.LatLng(ne_lat, ne_lng),
    zoom: zoom
  };
  map = new google.maps.Map(document.getElementById(map_container),
      mapOptions);

  var bounds = new google.maps.LatLngBounds(
      new google.maps.LatLng(sw_lat, sw_lng),
      new google.maps.LatLng(ne_lat, ne_lng)
  );

  // Define the rectangle and set its editable property to true.
  rectangle = new google.maps.Rectangle({
    bounds: bounds,
    editable: true,
    strokeWeight: 1,
    fillOpacity: 0.1,
    draggable: true
  });

  rectangle.setMap(map);

  map.fitBounds(bounds);

  $('input#overlay_map_ne').attr('value', ne_lat.toFixed(3) + ',' + ne_lng.toFixed(3));
  $('input#overlay_map_sw').attr('value', sw_lat.toFixed(3) + ',' + sw_lng.toFixed(3));

  // Add an event listener on the rectangle.
  google.maps.event.addListener(rectangle, 'bounds_changed', showNewRect);

  // Define an info window on the map.
  infoWindow = new google.maps.InfoWindow();
  add_image_to_rectangle(sw_lat, sw_lng, ne_lat, ne_lng, srcImage);

  overlay_map_context_menu();
  // places_search_box();
}

function places_search_box(){
  $('.row.map').append('<input id="pac-input" class="controls" type="text" placeholder="' + I18n.t('search_box') + '">')

//   var input = (document.getElementById('pac-input'));
//   map.controls[google.maps.ControlPosition.TOP_LEFT].push(input);
//   var searchBox = new google.maps.places.SearchBox((input));

//   google.maps.event.addListener(searchBox, 'places_changed', function() {
//     var places = searchBox.getPlaces();

//     var bounds = new google.maps.LatLngBounds();
//     for (var i = 0, place; place = places[i]; i++) {
//       bounds.extend(place.geometry.location);
//     }

//     map.fitBounds(bounds);
//   });
}
