/*!
 * Copyright (c) 2024 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

//= require i18n

$(document).on('shown.bs.modal', '#edit-playbook-assignment-rule-modal', function () {
  showConditionType();
  const propertiesDataElement = document.getElementById("properties-data");
  var properties;
  if (propertiesDataElement) {
    properties = JSON.parse(propertiesDataElement.textContent);
  }
  
  const subpropertiesDataElement = document.getElementById("sub-properties-data");
  var subProperties;
  if (subpropertiesDataElement) {
    subProperties = JSON.parse(subpropertiesDataElement.textContent);
  }
  
  var base_condition_template = document.querySelector('.condition-row');

  if (base_condition_template) {
    base_condition_template = base_condition_template.cloneNode(true)
  } else {
    // Creates a new condition row if there isn't
    base_condition_template = document.createElement('div');
    base_condition_template.classList.add('row', 'condition-row');
    base_condition_template.innerHTML = '<input type="hidden" name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][id]" value="">' +
    '<input type="hidden" name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][_destroy]" class="destroy_condition" value="0">' +
    '<div class="col-md-5 property-column">' +
      '<label for="property">' + I18n.t('javascript.property') + '</label>' +
      '<select name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][property]" class="form-control" id="playbook_assignment_rule_playbook_assignment_rules_conditions_attributes_0_property" required="required">' +
        '<option value="">' + I18n.t('javascript.choose_property') + '</option>' +
        properties.map(function(property) {
          return '<option value="' + property[1] + '">' + property[0] + '</option>'}).join('') +
      '</select>' +
    '</div>' +
    '<div class="col-md-3 sub-property-column" style="display:none;">' +
      '<label for="sub_property">' + I18n.t('javascript.sub_property') + '</label>' +
      '<select name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][sub_property]" class="form-control" id="playbook_assignment_rule_playbook_assignment_rules_conditions_attributes_0_sub_property" required="required">' +
        '<option value="">' + I18n.t('javascript.choose_sub_property') + '</option>' +
      '</select>' +
    '</div>' +
    '<div class="col-md-3">' +
      '<label for="operation">' + I18n.t('javascript.operation') + '</label>' +
      '<select name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][operation]" class="form-control" required="required">' +
        '<option value="">' + I18n.t('javascript.choose_operator') + '</option>' +
        '<option value="=">' + I18n.t('javascript.operation_eq') + '</option>' +
        '<option value="!=">' + I18n.t('javascript.operation_not_eq') + '</option>' +
        '<option value=">">' + I18n.t('javascript.operation_greater') + '</option>' +
        '<option value="<">' + I18n.t('javascript.operation_less') + '</option>' +
      '</select>' +
    '</div>' +
    '<div class="col-md-3 value-column">' +
      '<label for="value">' + I18n.t('javascript.value') + '</label>' +
      '<select name="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes][0][value]" class="form-control" id="playbook_assignment_rule_playbook_assignment_rules_conditions_attributes_0_value" required="required">' +
        '<option value="">' + I18n.t('javascript.choose_value') + '</option>' +
      '</select>' +
    '</div>' +
    '<div class="col-md-1">' +
      '<button type="button" class="btn remove-condition">' +
        '<i class="fa fa-trash"></i>' +
      '</button>' +
    '</div>'
  }
  
  initializeConditionHandlers(base_condition_template, subProperties);
});

$(document).on('shown.bs.modal', '#new-playbook-assignment-rule-modal', function () {
  showConditionType();
  const base_condition_template = document.querySelector('.condition-row').cloneNode(true);

  const subpropertiesDataElement = document.getElementById("sub-properties-data");
  var subProperties;
  if (subpropertiesDataElement) {
    subProperties = JSON.parse(subpropertiesDataElement.textContent);
  }

  initializeConditionHandlers(base_condition_template, subProperties);
});

function initializeConditionHandlers(base_condition_template, subProperties) {
  // Add remove function and load subproperties to all condition rows that exists
  document.querySelectorAll('.remove-condition').forEach(function(button) {
    assignRemoveFunction(button);
    loadSubproperties(button, subProperties);
  });

  // Creates a new row of conditions
  $(document).off('click', '#add-condition-btn').on('click', '#add-condition-btn', function (e) {
    const base_container = document.getElementById('conditions-container');
    const currentIndex = base_container.children.length;
    var new_condition_row = base_condition_template.cloneNode(true);
    
    new_condition_row.querySelectorAll('select, input').forEach(function(el) {
      el.value = '';
      el.name = el.name.replace(/\[\d+\]/g, '[' + currentIndex + ']');
      if (el.name.includes("[_destroy]")) {
        el.value = '0';
      }
    });

    assignRemoveFunction(new_condition_row.querySelector('.remove-condition'));
    loadSubproperties(new_condition_row.querySelector('.property-column'), subProperties);
    
    base_container.appendChild(new_condition_row)
    
    showConditionType();
  });

  // Deletes a condition row
  $(document).off('click', '.remove-condition').on('click', '.remove-condition', function (e) {
    showConditionType();
  });
}

function assignRemoveFunction(removeButton) {
  removeButton.addEventListener('click', function() {
    const conditionRow = this.closest('.condition-row');
    conditionRow.querySelectorAll('input, select').forEach(function(el) {
      el.removeAttribute('required');
    });

    const destroyField = conditionRow.querySelector('.destroy_condition');
    if (destroyField) {
      destroyField.value = '1';
    }
    conditionRow.style.display = 'none';
    conditionRow.classList.add('destroy');
  });
}

function loadSubproperties(element, subProperties) {
  const condition_row = element.closest('.condition-row');
  const propertyColumn = condition_row.querySelector('.property-column');
  const propertySelect = condition_row.querySelector('select[name^="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes]"][name*="[property]"]');
  const subPropertyColumn = condition_row.querySelector('.sub-property-column');
  const subPropertySelect = condition_row.querySelector('select[name^="playbook_assignment_rule[playbook_assignment_rules_conditions_attributes]"][name*="[sub_property]"]');
  const valueColumn = condition_row.querySelector('.value-column');

  propertySelect.onchange = function() { handlePropertyChange(true, propertySelect, subPropertySelect, propertyColumn, subPropertyColumn, subProperties); }
  subPropertySelect.onchange = function() { handleSubPropertyChange(true, subPropertySelect, valueColumn); }
  handlePropertyChange(false, propertySelect, subPropertySelect, propertyColumn, subPropertyColumn, subProperties);
  handleSubPropertyChange(false, subPropertySelect, valueColumn);
}

function handlePropertyChange(onchange, propertySelect, subPropertySelect, propertyColumn, subPropertyColumn, subProperties) {
  const selectedProperty = propertySelect.value;
  
  if (onchange) {
    subPropertySelect.innerHTML = '';
  }
  if (selectedProperty in subProperties) {
    subPropertyColumn.style.display = 'block';
    propertyColumn.classList.remove('col-md-5');
    propertyColumn.classList.add('col-md-2');
    subPropertySelect.setAttribute('required', 'required');
    
    subProperties[selectedProperty].forEach(function(subProperty) {
      const exists = Array.from(subPropertySelect.options).some(function(option) {return option.value === subProperty[1];}); 
      if (!exists) {
        const option = new Option(subProperty[0], subProperty[1]);
        subPropertySelect.add(option);
      }
    });
  
  } else {
    subPropertyColumn.style.display = 'none';
    propertyColumn.classList.remove('col-md-2');
    propertyColumn.classList.add('col-md-5');
    subPropertySelect.removeAttribute('required');
  }
}

function handleSubPropertyChange(onchange, subPropertySelect, valueColumn) {
  const dropdownDataElement = document.getElementById("dropdown-data");
  var dropdownValues;
  if (dropdownDataElement) {
    dropdownValues = JSON.parse(dropdownDataElement.textContent);
  }
  const selectedSubProperty = subPropertySelect.value;

  const valueInputSelect = valueColumn.querySelector('select, input');

  const currentName = valueInputSelect.name;
  const currentValue = valueInputSelect.value;

  valueInputSelect.remove();

  if (selectedSubProperty in dropdownValues) {
    const dropdown = document.createElement('select');
    dropdown.name = currentName;
    dropdown.className = 'form-control';
    subPropertySelect.setAttribute('required', 'required');

    dropdownValues[selectedSubProperty].forEach(function(option) {
      const opt = new Option(option[1], option[0]);
      dropdown.add(opt);
    });
    
    if (!onchange) {
      dropdown.value = currentValue;
    }

    valueColumn.appendChild(dropdown);
  } else {
    const textField = document.createElement('input');
    textField.type = 'text';
    textField.name = currentName;
    textField.className = 'form-control';
    textField.setAttribute('required', 'required');

    if (!onchange) {
      textField.value = currentValue;
    }

    valueColumn.appendChild(textField);
  }
}

function showConditionType() {
  const visibleConditions = Array.from(document.querySelectorAll('.condition-row')).filter(function(element) {
    return !element.classList.contains('destroy');
  });
  const conditionType = document.getElementById('condition-type');

  if (visibleConditions.length <= 1) {
    conditionType.style.display = 'none';
    toggleRadioButtonsRequired(false);
  } else {
    conditionType.style.display = '';
    toggleRadioButtonsRequired(true);
  }
}

function toggleRadioButtonsRequired(isRequired) {
  const radioButtons = document.querySelectorAll('input[type="radio"]');
  radioButtons.forEach(function(radio) {
    if (isRequired) {
      radio.setAttribute('required', 'required');
    } else {
      radio.removeAttribute('required');
    }
  });
}
