/*!    
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

// Place all the behaviors and hooks related to the matching controller here.
// All this logic will automatically be available in application.js.
// You can use CoffeeScript in this file: http://coffeescript.org/

String.prototype.capitalize = function() {
  return this.charAt(0).toUpperCase() + this.slice(1);
}

// this function is used by report_blocks and other libraries.
function heatMap(options) {
  var data = options.data;

  if (data.length === 0) return;

  var myLatlng = new google.maps.LatLng(34.7100,-40.8678);
  var bounds = new google.maps.LatLngBounds();
  var mapData = {max: 1, data: []};

  data = data.filter(function (d) {
    return (d.client_latlong !== undefined &&
      d.client_latlong !== null &&
      d.client_latlong.split(",").length === 2);
  });

  if (data.length > 0) {
    mapData.max = data[0][options.aggregation];
  }

  data.forEach(function (element) {
    var res = element.client_latlong.split(",");

    bounds.extend(new google.maps.LatLng(res[0], res[1]));

    mapData.data.push({
      lat: parseFloat(res[0]),
      lng: parseFloat(res[1]),
      count: element[options.aggregation]
    });
  });

  var myOptions = {
    zoom: 3,
    center: myLatlng,
    mapTypeId: (options.mapType === "satellite" ? google.maps.MapTypeId.HYBRID : google.maps.MapTypeId.ROADMAP),
    disableDefaultUI: false,
    scrollwheel: false,
    draggable: false,
    navigationControl: false,
    zoomControlOptions: {
      style: google.maps.ZoomControlStyle.SMALL
    },
    mapTypeControl: false,
    scaleControl: false,
    disableDoubleClickZoom: true
  };

  var map = new google.maps.Map(document.getElementById(options.container), myOptions);
  map.fitBounds(bounds);

  var heatmap = new HeatmapOverlay(map, {
      "radius": 15,
      "visible": true, 
      "opacity": 60,
      "logscale": true
  });

  heatmap.rb_markers = {};

  google.maps.event.addListenerOnce(map, "idle", function() {
    heatmap.setDataSet(mapData);
  });

  google.maps.event.addListener(map, "dragend", function() {
    heatmap.draw();
  });

  return heatmap;
}

function display_report_block(options, data, user) {
  // Parse JSON options
  var options = JSON.parse(options);
  options['user'] = user;
  var data = JSON.parse(data);

  if (options.property.product_type === "sensors") {
    displaySensors(options);
  } else if (options.property.product_type == "monitor") {
    displayTops(options, data);
  } else {
    switch (options.property.graph_type) {
      case "pie":
        displayPie(options, data);
        break;
      case "horbars":
        if (options.property.report_block_type === "Cardinality") displayCardinalityHorizontalBars(options, data);
        else displayHorizontalBars(options, data);
        break;
      case "verbars":
        if (options.property.report_block_type === "Cardinality") displayCardinalityVerticalBars(options, data);
        else displayVerticalBars(options, data);
        break;
      case "area":
      case "stacked":
      case "line":
      case "bars":
      case "sbars":
        if (options.property.report_block_type === "Top") displayTops(options, data);
        else if (options.property.report_block_type === "Raw") displayRaw(options, data);
        else if (options.property.report_block_type === "Cardinality") displayCardinality(options, data);
        else displayCompare(options, data);
        break;
      case "table":
        displayTable(options, data);
        break;
      case "ball":
        displayBall(options, data);
        break;
      case "value":
        displayValue(options, data);
        break;
      case "map":
        if (options.property.map_column === "coordinates_map") displayCoordinatesMap(options, data);
        else displayMap(options, data);
        break;
      case "streams":
        displayStreams(options, data);
        break;
      default:
        break;
    }
  }

}

function widget_display_graph_units(options) {
  var aggregation = options.property.aggregation,
      product_type = options.property.product_type;

  if (options.property.product_type == "flow" || options.property.product_type == "ips") {
    return redBorder.modules[product_type].aggregations[aggregation].unit
  } else if (options.property.product_type == "monitor") {
    switch (options.property.category) {
      case 'avg_bytes_per_wire_packet': return 'B/p';
      case 'wire_bits_per_sec_realtime': return 'bps';
      case 'deleted_sessions_per_second': return 's/s';
      case 'tcp_sessions_closed_per_second': return 's/s';
      case 'alerts_per_second': return 'eps';
      case 'system_temp': return 'º';
      case 'cpu': 
      case 'disk':
      case 'disk_aggregated':
      case 'disk_raw':
      case 'pkt_drop_percent':
      case 'memory_buffer':
      case 'memory_cache':
      case 'memory':
      case 'patmatch_percent':
      case 'snort_cpu':
        return '%';
    }
  } else {
    return '';
  }
}

function displayRaw(options, data) {
  // Create the graph
  var graph = new RB.Serie({
    container: "widget_placeholder_" + options.id,
    autoscaling: true,
    step: options.step,
    domain: [options.start_time_secs, options.end_time_secs],
    timezone: options.tzoffset,
    hide_axises: (options.property.hide_axises === "1"),
    zoomable: false,
    autorender: false,
    height: options.property.height,
    width: options.width,
    shared_colors: true,
    units: (function() {
      var aggregation = options.property.aggregation,
          product_type = options.property.product_type;

      return redBorder.modules[product_type].aggregations[aggregation].unit
    })()
  });

  graph.add({data: data, name: "Total", color: options.property.color, type: options.property.graph_type})
  graph.startRender();

}

function displayTops(options, data) {
  // Create the graph
  var graph = new RB.Serie({
    container: "widget_placeholder_" + options.id,
    autoscaling: true,
    step: options.step,
    domain: [options.start_time_secs, options.end_time_secs],
    timezone: options.tzoffset,
    zoomable: false,
    autorender: false,
    hide_axises: false,
    shared_colors: true,
    width: options.width,
    height: options.property.height,
    hide_axises: (options.property.hide_axises === "1"),
    units: (function() {
      var aggregation = options.property.aggregation,
          product_type = options.property.product_type;

      if(product_type == 'monitor') aggregation = 'monitors';
      return redBorder.modules[product_type].aggregations[aggregation].unit
    })()
  });

  if(options.property.show_total === "1" && options.property.product_type != "monitor") {
    graph.add({data: data["total_serie"], name: "Total", color: "#ccc", type: "line", dashed: true, stroke: 1})
  }

  var colors = {};

  if (options.property.column === "priority" || options.property.column === "darklist_score_name") {
    colors["critical"] = "#ff4643";
    colors["very high"] = "#d24643";
    colors["high"] = "#aa4643";
    colors["medium"] = "#d9c03e";
    colors["low"] = "#89a54e";
    colors["very low"] = "#30BF93";
    colors["unknown"] = "#919191";
    colors["clean"] = "#5eca5e";
  } else if (options.property.column === "action") {
    colors.drop = "#FF0000";
    colors.sdrop = "#FF0000";
    colors.pass = "#00FF00";
    colors.alert = "#ffab2e";
  }

  for(var i = 0; (i < options.property.top_num && i < data["series"].length); i += 1) {
    graph.add({data: data["series"][i][0]["data"], name: data["series"][i][0]["secondary_name"], randomId: true, type: options.property.graph_type, color: (colors[data["series"][i][0]["name"]] || undefined) })
  }

  graph.startRender();
}

function displayPie(options, data) {
  var graph = new RB.Pie({
    container: "widget_placeholder_" + options.id,
    max: options.property.top_num,
    showQuantityOnLegend: true,
    showHighestPercen: true,
    width: options.width,
    height: options.property.height
  });

  var total = data.total;

  var what = options.property.aggregation,
      colors = {};

  function drawPie(datos, colors) {
    data_pie = datos.map(function (d) {
      var map = {
        name: d[0]["secondary_name"],
        events: d[0]["data"][0]["event"]["events"]
      };

      if (colors[d[0]["secondary_name"]] !== undefined) {
        map.color = colors[d[0]["secondary_name"]];
      }

      return map;
    });

    graph.total = total;
    graph.setData(data_pie);

  }

  if (options.property.column === "priority" || options.property.column === "darklist_score_name") {
    colors["critical"] = "#ff4643";
    colors["very high"] = "#d24643";
    colors["high"] = "#aa4643";
    colors["medium"] = "#d9c03e";
    colors["low"] = "#89a54e";
    colors["very low"] = "#30BF93";
    colors["unknown"] = "#919191";
    colors["clean"] = "#5eca5e";

    drawPie(data.series, colors);

  } else if(options.property.column === "action") { 
    colors.drop = "#FF0000";
    colors.sdrop = "#FF0000";
    colors.pass = "#00FF00";
    colors.alert = "#ffab2e";

    drawPie(data.series, colors);

  } else {
    drawPie(data.series, {});
  }

}

function displayHorizontalBars(options, data) {
  if (data.total !== 0) {
    var total = data.total,
        what = options.property.aggregation;

    data_bars = data.series.map(function (d) {
      var value = d[0]["data"][0]["event"]["events"];

      return {
        name: d[0]["name"],
        events: value
      };
    });

    var graph = new RB.Bars({
      container: "widget_placeholder_" + options.id,
      data: data_bars,
      max: options.property.top_num,
      color: options.property.color,
      total: total,
      showBiggestAsBarTotal: true,
      units: options.property.aggregation,
      width: options.width
    })

  } else {
    $('#widget_placeholder_' + options.id).html("<div class='widget_no_data'>No data to display</div>");
  }
}

function displayVerticalBars(options, data) {
  if (data.total !== 0) {
    var total = data.total,
        what = options.property.aggregation;

    data_bars = data.series.map(function (d) {
      var value = d[0]["data"][0]["event"]["events"];

      return {
        name: d[0]["name"],
        events: value
      };
    });

    var graph = new RB.VerticalBars({
      container: "widget_placeholder_" + options.id,
      data: data_bars,
      color: options.property.color,
      units: options.property.aggregation,
      height: options.property.height,
      width: options.width
    });

  } else {
    $('#widget_placeholder_' + options.id).html("<div class='widget_no_data'>No data to display</div>");
  }
}

function displayCompare(options, data) {
  var graph = new RB.Serie({
    container: "widget_placeholder_" + options.id,
    autoscaling: true,
    step: options.step,
    //domain: [domain_start, (domain_start + domain_time - options.step/1e3)],
    hide_axises: (options.property.hide_axises === "1"),
    zoomable: false,
    ignoreLastZeros: 0,
    shared_colors: true,
    genericXAxis: true,
    height: options.property.height,
    width: options.width
  });

  for(var i = 0; (i < options.property.top_num && i < data["series"].length); i += 1) {
    graph.add({data: data["series"][i][0]["data"], name: data["series"][i][0]["name"], randomId: true, type: options.property.graph_type })
  }
}

function displayCoordinatesMap(options, data) {
  $widget = $("#widget_placeholder_" + options.id);

  $widget.css("width", options.width);
  $widget.css("min-height", options.width / 2);
  if (options.property.show_name === "1") {
    $widget.parents(".widget_content").css("padding-top", 25);
  }

  try {
    var map = redBorder.heatMap({
      container: "widget_placeholder_" + options.id,
      data: data,
      tab: false,
      aggregation: options.aggregation,
      scrollwheel: false,
      mapType: options.property.map_type
    }).initialize();

    if(options.property.map_type === google.maps.MapTypeId.SATELLITE){
      map.getMap().setTilt(45);
    }

    $widget.trigger("mapLoad");
  } catch (err) {
    console.log("Couldn't load widget " + options.id + ". " + err);
  }
}

function displayMap(options, data) {
  var $widget = $("#widget_placeholder_" + options.id);

  $widget.css("width", options.width);
  $widget.css("min-height", options.width / 2);
  if (options.property.show_name === "1") {
    $widget.parents(".widget_content").css("padding-top", 25);
  }

  try {
    redBorder.map({
      container: "widget_placeholder_" + options.id,
      data: data,
      aggregation: options.aggregation,
      scrollwheel: false,
      clustered: true,
      mapType: options.property.map_type
    });
  } catch (err) {
    console.error("Couldn't load report block " + options.id + ". " + err);
  }

  $widget.trigger("mapLoad");
}

function displayCardinality(options, data) {
  // Create the graph
  var graph = new RB.Serie({
    container: "widget_placeholder_" + options.id,
    autoscaling: true,
    step: options.step,
    //domain: [options.start_time_secs, options.end_time_secs],
    hide_axises: (options.property.hide_axises === "1"),
    timezone: options.tzoffset,
    zoomable: false,
    shared_colors: true,
    autorender: false,
    height: options.property.height,
    width: options.width
  });

  if (options.property.cardinality_group_by === undefined || options.property.cardinality_group_by === "none") {
    graph.add({data: data["total"], name: "Cardinality", type: options.property.graph_type, color: options.property.color})
    graph.startRender();
  } else {
    if (options.property.show_total === "1") {
      graph.add({data: data["total"], name: "Cardinality", color: "#ccc", type: "line", dashed: true, stroke: 1})
    }

    for (var i = 0; (i < options.property.top_num && i < data["series"].length); i += 1) {
      if(data["series"][i][0]["data"].length > 1) {
        graph.add({data: data["series"][i][0]["data"], name: data["series"][i][0]["secondary_name"], randomId: true, type: options.property.graph_type })
      }
      
      graph.startRender();
    }
  }
}

function displayCardinalityHorizontalBars(options, data) {
  if (data["series"][0].length !== 0) {
    data_series = data["series"][0].map(function (d) {
      return {
        name: d["secondary_name"],
        events: d.result
      };
    });

    var graph = new RB.Bars({
      container: "widget_placeholder_" + options.id,
      data: data_series,
      max: options.property.top_num,
      color: options.property.color,
      showBiggestAsBarTotal: true,
      showAbsoluteNumbers: true,
      width: options.width
    });
  } else {
    $('#widget_placeholder_' + options.id).html("<div class='widget_no_data'>No data to display</div>");
  }
}

function displayCardinalityVerticalBars(options, data) {
  if (data["series"][0].length !== 0) {
    data_series = data["series"][0].map(function (d) {
      return {
        name: d["secondary_name"],
        events: d.result
      };
    });

    var graph = new RB.VerticalBars({
      container: "widget_placeholder_" + options.id,
      data: data_series,
      max: options.property.top_num,
      color: options.property.color,
      height: options.property.height,
      width: options.width
    });
  } else {
    $('#widget_placeholder_' + options.id).html("<div class='widget_no_data'>No data to display</div>");
  }
}

function displayTable(options, data) {
  var what = options.property.aggregation, $container;

  if (data.length !== 0) {
    data_table = data.map(function (d) {
      return {
        name: d["secondary_name"],
        events: d[what]
      };
    });

    $container = $("#widget_placeholder_" + options.id).addClass("block_table").append("<table>");

    $("table", $container).append("<thead>");
    $("table", $container).append("<tbody>");
    $("table", $container).attr("class", "table table-striped table-condensed table-hover");
    $("table thead", $container).append("<tr>");
    $("table thead tr", $container).append($("<th>").text("Element"));
    $("table thead tr", $container).append($("<th>").attr("class", "last").text(what.capitalize()));

    data_table.forEach(function (d, i) {
      var multiplier, multiplierPrefix, name,
        interval = new Date(options.end_time) - new Date(options.start_time);

      if (d.events < 1e3) {
        multiplier = 1;
        multiplierPrefix = "";
      } else if (d.events >= 1e3 && d.events < 1e6) {
        multiplier = 1e-3;
        multiplierPrefix = "K";
      } else if (d.events >= 1e6 && d.events < 1e9) {
        multiplier = 1e-6;
        multiplierPrefix = "M";
      } else if (d.events >= 1e9 && d.events < 1e12) {
        multiplier = 1e-9;
        multiplierPrefix = "G";
      } else if (d.events >= 1e12) {
        multiplier = 1e-12;
        multiplierPrefix = "T";
      }

      name = d.name;
      if (name === undefined || name === "" || name === null) {
        name = "N/A";
      }

      if (i < options.property.top_num) {
        var $tr = $("<tr>").appendTo($("table tbody", $container));
        $("<td>").text(name).appendTo($tr);
        $("<td>").text((Math.round(d.events * multiplier * 100) / 100) + " " + multiplierPrefix).attr("class", "last").appendTo($tr);
      }
    })
  } else {
    $("#widget_placeholder_" + options.id).append("<div class='center'><p>No data to display</p></div>");
  }
}

function displayBall(options, data) {
  var evt, interval = new Date(options.end_time) - new Date(options.start_time);

  if (data.length !== 0) {
    evt = data[0].event.events;
  } else {
    evt = 0;
  }

  var graph = new RB.Ball({
    container: "widget_placeholder_" + options.id,
    value: evt,
    max: options.property.ball_max,
    width: options.width,
    alwaysShowQuantity: true
  });
}

function displayValue(options, data) {
  var $container = $("#widget_placeholder_" + options.id),
    dato,
    reference = parseInt(options.property.reference),
    interval = new Date(options.end_time) - new Date(options.start_time);

  if (data.length !== 0) {
    dato = parseInt(data[0].event.events);
  } else {
    dato = 0;
  }

  if (isNaN(reference)) {
    var multiplier, multiplierPrefix;

    if (dato < 1e3) {
      multiplier = 1;
      multiplierPrefix = "";
    } else if (dato >= 1e3 && dato < 1e6) {
      multiplier = 1e-3;
      multiplierPrefix = "K";
    } else if (dato >= 1e6 && dato < 1e9) {
      multiplier = 1e-6;
      multiplierPrefix = "M";
    } else if (dato >= 1e9 && dato < 1e12) {
      multiplier = 1e-9;
      multiplierPrefix = "G";
    } else if (dato >= 1e12) {
      multiplier = 1e-12;
      multiplierPrefix = "T";
    }

    $container.html("<div>" + (Math.round(dato * multiplier * 100) / 100) + " " + multiplierPrefix + "</div>");
  } else {
    $container.html("<div>" + Math.round((dato / reference) * 100) + " %</div>");
  }

  var font_size = parseInt(options.width) / 5;

  $("div", $container).css("font-size", font_size.toString() + "px");
}

function displayStreams(options, data) {
  var columns = options.property.stream_columns.filter(function (column) {
    return column !== "";
  });

  var aggregation = options.property.aggregation,
    interval = new Date(options.end_time) - new Date(options.start_time);
  
  var graph = new RB.Sankey({
    container: "widget_placeholder_" + options.id,
    value: aggregation,
    linksName: aggregation,
    height: options.property.height,
    width: options.width
  });

  graph.add(data, columns);
}

function displaySensorsMap(options, data, reloading) {
  var $widget = $("#widget_placeholder_" + options.id);

  $widget.addClass("map");

  if ($('#dashboard').hasClass('dashboard-columns')) {
    $widget.css("min-height", $widget.width() / 1.5);
    if (options.property.show_name === "1") {
      $widget.parents(".widget_content").css("padding-top", 25);
    }
  } else {
    $widget.parents(".widget_content").css("padding-top", 25);
  }

  try {
    window.rbWidgets["id" + options.id] = redBorder.map({
      container: "widget_placeholder_" + options.id,
      data: data,
      aggregation: options.aggregation,
      scrollwheel: false,
      clustered: true,
      mapType: options.property.map_type
    });
  } catch (err) {
    console.error("Couldn't load report block " + widget.id + ". " + err);
  }

  $widget.trigger("mapLoad");
}