/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder = redBorder || {};
redBorder.SensorsTreeFilters = function(filterSource) {

  // Set Listeners

  // Clear selection
  $(document).off("click", "#remove-selected");
  $(document).on("click", "#remove-selected", function() {
    event.preventDefault();
    $("#sensors-tree-filters tr.sensor.selected").removeClass("selected");
    updateSelected();
  });

  // Filter Selected Sensors
  $(document).off("click", "#filter-selected");
  $(document).on("click", "#filter-selected", function() {
    event.preventDefault();
    // Get Selected Sensors IDs
    var selectedIds = $("#sensors-tree-filters tr.sensor.selected").map(function(i,d) {
      return $(d).data("tt-id");
    }).toArray();

    $.post("/sensors/tree_filters",
      { sensors_tree_filters: selectedIds, filter_source: filterSource }, function() {
      // Hide Modal
      $("#default_modal").modal("toggle");
      // Reload page using Turbolinks
      blockWebUI();
      Turbolinks.visit(document.location.pathname + document.location.search);
    });
  });

  // Select a Sensor
  $(document).off("click", "#sensors-tree-filters #tree tr.sensor");
  $(document).on("click", "#sensors-tree-filters #tree tr.sensor", function() {
    // Do not select sensor when trying to expand a branch
    if(event.target.tagName == "A"){
      return;
    }
    $(this).toggleClass("selected");
    updateSelected();
  });

  // Helper to Update Selected Counter
  function updateSelected() {
    var $filterSelected = $("#filter-selected");
    if ($(".sensor.selected").length > 0) {
      $filterSelected.removeClass("disabled");
      $filterSelected.text(I18n.t('tops.tab_content.selected') + " (" + $(".sensor.selected").length + ")");
    } else {
      $filterSelected.addClass("disabled");
      $filterSelected.text(I18n.t('tops.tab_content.selected'));
    }
  }

  // Tree Table

  $("#tree").treetable({
    expandable: true,
    onNodeCollapse: function() {
      var sensor = this;
    },
    onNodeExpand: function() {
      var sensor = this;
      var rows = $("#tree").find('tr[data-tt-parent-id='+ sensor.id +']');

      if(rows.length > 0){
        rows.show();
        return true;
      }

      $.ajax({
        async: false, // Must be false, otherwise loadBranch happens after showChildren?
        url: "/sensors/tree_filters/" + sensor.id,
      }).done(function(html) {
        var rows = $(html).filter("tr");
        $("#tree").treetable("loadBranch", sensor, rows);
      });
    }
  });

  if ($('#tree tbody tr[data-tt-parent-id=""]').length < 6){
    // Expand root domains
    $("#tree tbody > tr[data-type=domain]").each(function(){
      $("#tree").treetable("reveal", $(this).data("ttId"));
    });
  }
}
