/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

redBorder.utils.query = {
  /*
   * Fill empty timestamps points with 0 events. We need the data,
   * step, start time and end time to fill the array.
   *
   * @param step: in millis
   * @param startTime and endTime: in Date object
   */
  fillEmptyPoints: function(data, step, startTime, endTime){
    var filledData = [],
        timePoint;

    // Round start time to the step
    startTime = new Date(Math.floor(startTime.getTime()/step)*step);
    var sTime = new Date(data[0].timestamp);
    var eTime = new Date(data[data.length - 1].timestamp);
    // Sort data by timestamp
    if (sTime > eTime) data = data.reverse();

    if (data !== undefined && data.length >= 0){
      for(var i = 0; i < data.length; i++){
        timePoint = new Date(data[i].timestamp);
        while(startTime.getTime() < timePoint.getTime()){
          filledData.push({ event: {events: 0}, timestamp: new Date(startTime).toString() });
          startTime = new Date(startTime.getTime() + step);
        }

        filledData.push(data[i]);
        startTime = new Date(startTime.getTime() + step);
      }

      // Fill at end time
      while (startTime <= endTime){
        filledData.push({event: {events: 0}, timestamp: new Date(startTime).toString() });
        startTime = new Date(startTime.getTime() + step);
      }

    }

    return filledData;
  }
}
