DittoCode::HoldFile.if 'ENTERPRISE'
#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class AlertsController < ApplicationController

  before_action :require_super_administrative_privileges, except: [:dismiss]
  before_action :set_alert, only: [:destroy, :update,
      :enable, :disable]

  def index
    @all_alerts = Alert.all.order('enabled desc, updated_at desc')
                  .paginate(page: params[:page], per_page: current_user.per_page_count)
  end

  def new
    @alert = Alert.new
    render layout: false
  end

  def edit
    @alert = Alert.find params[:id]
    render layout: false
  end

  def create
    @alert = Alert.new(alert_params)

    respond_to do |format|
      if @alert.save
        format.html { redirect_to alerts_url, notice: "#{I18n.t('alert_success')}" }
      else
        format.html { redirect_to alerts_url, error: "#{I18n.t('alert_error')}" }
      end
    end
  end

  def update
    respond_to do |format|
      if @alert.update(alert_params)
        format.html { redirect_to alerts_url, notice: "#{I18n.t('alert_update')}" }
      else
        format.html { redirect_to alerts_url, error: "#{I18n.t('alert_error_update')}" }
      end
    end
  end

  def destroy
    @alert.destroy
    respond_to do |format|
      format.html { redirect_to alerts_url }
      format.json { head :no_content }
    end
  end

  def dismiss
    session[:dismissed_alerts] ||= []
    session[:dismissed_alerts] << params[:id]
    render js: {}
  end

  def disable
    @alert.update(enabled: false)
    render js: {}
  end

  def enable
    @alert.update(enabled: true)
    render js: {}
  end

  private

  def set_alert
    @alert = Alert.find params[:id].to_i
  end

  # Never trust parameters from the scary internet, only allow the white list through.
  def alert_params
    params[:alert].permit(:message, :level, :start_time, :end_time)
  end
end
