#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# This controller provide the basic methods to work with a Sensor controller.
# Every module has its own sensors so, the API controllers that work with them
# can be extended from this.
#
class Api::V1::BaseSensorsController < Api::V1::BaseController
  # Helper methods
  protected
  
  #
  # Set the parent of the tree to retrieve info about sensors.
  #
  def set_parent
    # Get the sensor by parent params or user top_domain
    @parent = Sensor.find_by_uuid(params[:parent_uuid] ||
      current_user.sensor.uuid)

    # Check permissions
    return error_404('The sensor doesn\'t exist') if @parent.nil?
    return error_401('You don\'t have enought privileges to do that.') unless
      current_user.can?(:read, @parent)
  end

  #
  # Set the domain based on params[:sensor_uuid] or take the sensor
  # of current user.
  #
  def set_domain
    # Get the sensor by parent params or user top_domain
    @domain = Sensor.find_by_uuid(params[:sensor_uuid] ||
      current_user.sensor.uuid)

    # Check permissions
    return error_404('The sensor doesn\'t exist') if @domain.nil?
    return error_401('You don\'t have enought privileges to do that.') unless
        current_user.can?(:read, @domain)
  end

  #
  # Set the domain based on params[:sensor_uuid] or return 404
  #
  def set_domain_only_param
    # Get the sensor by parent params or user top_domain
    @domain = Sensor.find_by_uuid(params[:sensor_uuid])

    # Check permissions
    return error_404('The sensor doesn\'t exist') if @domain.nil?
    return error_401('You don\'t have enought privileges to do that.') unless
        current_user.can?(:read, @domain)
  end

  #
  # Return a 422 error caused by an error when save a sensor. This method
  # concatenate the errors in the response
  #
  # == Parameters:
  # sensor::
  #   The sensor object that has errors
  #
  def error_on_sensor(sensor)
    error_422("The sensor can\'t be created due: "\
              "#{sensor.errors.full_messages.join(', ')}")
  end

  #
  # Check if the user has manage permissions on the parent domain. Parent must
  # be defined!
  #
  def manage_parent
    return error_401('You don\'t have enought privileges to do that.') unless
      current_user.can?(:manage, @parent)
  end

  #
  # Check if the user has manage permissions on the domain. Sensor must be
  # defined!
  #
  def manage_domain
    return error_401('You don\'t have enought privileges to do that.') unless
        current_user.can?(:manage, @domain)
  end

  #
  # Init a DelayedJob to wake up chef
  #
  def wakeup_chef
    Delayed::Job.enqueue(
      Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id),
      priority: 5
    ) unless DelayedJob.rb_wakeup_chef_cluster?
  end
end
