#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide an api to work with licenses
#
class Api::V1::LicensesController < Api::V1::BaseController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------

  # ----------------------------------------------
  # Actions
  # ----------------------------------------------

  #
  # Return the list of licenses.
  #
  # GET /api/v1/licenses
  # params:
  #       available_by_type (optional)
  #       auth_token
  #
  def index
    return error_422("You don\'t have privileges to do that.") unless
      current_user.admin?
    return error_400('The licensing mode is not global.') unless
      RedBorder::LICENSING_MODE == 'global'

    licenses = if params[:available_by_type].blank?
                 License.all
               else
                 License.available_by_type(params[:available_by_type])
               end

    if licenses.count.zero?
      render json: { message: 'There are no licenses available for this type of sensor.' }
    else
      render json: licenses.as_json(except: [:encoded_info, :signature,
                                             :created_at, :updated_at])
    end
  rescue StandardError
    error_500('There is an error trying to obtain this information.')
  end

  #
  # Return the information of a license.
  #
  # GET /api/v1/licenses/:id
  # params:
  #       auth_token
  #
  def show
    return error_422("You don\'t have privileges to do that.") unless
      current_user.admin?
    return error_400('The licensing mode is not global.') unless
      RedBorder::LICENSING_MODE == 'global'

    license = License.find_by(id: params[:id])
    if license
      render json: license.as_json(except: [:encoded_info, :signature,
                                            :created_at, :updated_at])
    else
      render json: { message: 'There is no license with that id.' }
    end
  rescue StandardError
    error_500('There is an error trying to obtain this information.')
  end
end
