DittoCode::HoldFile.if 'ENTERPRISE'

#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide API methods to work with organizations
#
class Api::V1::OrganizationsController < Api::V1::BaseSensorsController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------
  skip_before_action :authenticate_user_from_token!, :user_setup_api
  before_action :network_authentication
  before_action :set_organization, only: [:reach_bytes_limit]
  before_action :check_last_notification

  #
  # POST /api/v1/organizations/:uuid/reach_bytes_limit
  # Send an email when the organization has reached the limit.
  # The email will be sent to organization's admininstrator users.
  #
  def reach_bytes_limit
    admins = User.where(sensor_id: @organization.id, admin: true)
    if admins.any?
      OrganizationMailer.reach_bytes_limit(@organization, admins).deliver_now
      render json: { msg: 'email sent' }
    else
      render json: { msg: 'no recipients' }
    end
  end

  private

  def set_organization
    @organization = Sensor.find_by(uuid: params[:uuid], type: Sensor::DOMAIN, domain_type: Sensor::ORGANIZATION)
    return error_404 'Requested organization not found' unless @organization
  end

  def check_last_notification
    last_notification = @organization.last_organization_notification
    if last_notification.nil? || Time.at(last_notification).utc < Time.now.utc.beginning_of_day
      @organization.update(last_organization_notification: Time.now.to_i)
    else
      render json: { msg: 'email already sent today' } and return false
    end
  end
end
