#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Api::V1::RedborderObjectsController < Api::V1::BaseController
  # -------------------------------------------------------------
  # Before actions
  # -------------------------------------------------------------

  # Set the object class
  before_action :set_object_class, only: [:index]

  # -------------------------------------------------------------
  # Actions
  # -------------------------------------------------------------

  #
  # Return objects based on params[:value] and params[:type].
  #
  def index
    # Return objects
    @objects = @object_class.search(params[:value], current_user)

    # Render the response
    render json: {
      query: true,
      totalResultCount: @objects.count,
      objects: @objects.map { |o| { name: o.name, value: o.value } }
    }
  end

  private

  #
  # Check the type of the Object
  #
  def set_object_class
    @object_class = RedborderObject::OBJECT_CLASSES.find do |object_class|
      object_class.to_s == params[:type]
    end.try('constantize')

    return error_422("The object #{params[:type]} doesn't exist") if @object_class.nil?
  end
end
