#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide an api to work with rules.
#
class Api::V1::RulesController < Api::V1::BaseController
  # Set policies and queries
  before_action :set_policy, :set_query

  def active_rules
    @is_query[:rules][:category_id] = params[:category_id]
    @is_query[:rules][:group_id] = params[:group_id] if params[:group_id]
    @is_query[:rules][:family_id] = params[:family_id] if params[:family_id]
    @is_query[:rules][:id] = params[:id] if params[:id]

    count = Rule.active_custom_query(
      @policy,
      @query_tables,
      @is_query,
      @not_query,
      @msg
    ).count

    render json: { total: count }
  end

  private

  def set_policy
    @policy = Policy.find(params[:policy_id])
  end

  def set_query
    rule_db_version_ids = @policy.rule_db_versions.map(&:id)

    # Init query variables
    @is_query = {
      rules: {
        default_enabled: @policy.default_enabled_rules? ? true : [true, false],
        rule_db_version_id: rule_db_version_ids
      }
    }
    @not_query = []
    @query_tables = {
      rules: []
    }

    session[:rule][:filter].each do |k, v|
      v.each do |key, value|
        if k.to_sym == :factory_policy_id
          @query_tables[:rules] << :factory_policy_rules
          add_column(
            :factory_policy_rules,
            k.to_sym,
            key,
            value[:operator]
          )
        elsif [:flowbit_id, :flowbit_state_id].include?(k.to_sym)
          @query_tables[:rules] << :flowbit_rules
          add_column(
            :flowbit_rules,
            k.to_sym,
            key,
            value[:operator]
          )
        elsif [:category_id, :group_id, :family_id, :protocol, :class_type_id].include?(k.to_sym)
          @is_query[:rules] = {}
          add_column(
            :rules,
            k.to_sym,
            key,
            value[:operator]
          )
        elsif k.to_sym == :severity
          @query_tables[:rules] << { class_type: [:severity] }
          add_column(
            :severities,
            :id,
            key,
            value[:operator]
          )
        elsif k.to_sym == :source_id
          @query_tables[:rules] << :rule_source
          add_column(
            :rule_sources,
            :id,
            key,
            value[:operator]
          )
        elsif k.to_sym == :action_id
          @query_tables[:rules] << :policy_rules
          add_column(
            :rule_sources,
            k.to_sym,
            key,
            value[:operator]
          )
        elsif k.to_sym == :reference
          rb_reference_rule_ids =
            RbReferenceRule
            .where('url like ?', "%#{key.split.join('%')}%")
            .map(&:id)

          @query_tables[:rules] << :rb_reference_rules
          if value[:operator] == 'contains'
            @is_query[:rb_reference_rules] ||= {}
            if @is_query[:rb_reference_rules].has_key?(:id)
              @is_query[:rb_reference_rules][:id] += rb_reference_rule_ids
            else
              @is_query[:rb_reference_rules][:id] = rb_reference_rule_ids
            end
          else
             @not_query << {rb_reference_rules: {id: rb_reference_rule_ids}}
          end

        elsif [:is_new, :is_modified, :is_deprecated].include?(k.to_sym)
          key = key == '0' ? false : true
          add_column(
            :rules,
            k.to_sym,
            key,
            value[:operator]
          )
        end
      end
    end

    @query_tables = :rules if @query_tables.blank?
    @msg = params[:msg]
  end

  #
  # Add a column to the query hash.
  #
  # == Parameters:
  # column::
  #   A symbol with the column to add
  # group::
  #   A symbol with the group to search
  # value::
  #   The value to add
  # operator::
  #   String with the operator.
  #
  def add_column(column, group, value, operator)
    if operator == 'is'
      @is_query[column] ||= {}
      if @is_query[column].key?(group)
        @is_query[column][group] << value
      else
        @is_query[column][group] = [value]
      end
    else
      @not_query << { column => { group => value } }
    end
  end
end
