#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Api::V1::SessionsController < Devise::SessionsController
  # Stop authentication on create
  prepend_before_filter :require_no_authentication, only: [:create]

  # Before actions
  skip_before_filter :verify_authenticity_token, raise: false
  before_action :ensure_params_exist

  # Only JSON
  respond_to :json

  #
  # Login a user in the app based on username and password.
  #
  def create
    resource = User.find_for_database_authentication(
                 username: params[:user][:username]
               )
    return invalid_login_attempt unless resource

    if resource.valid_password?(params[:user][:password])
      sign_in('user', resource)

      # Check if we have the gcm id and store
      unless params[:user][:gcm_id].nil?
        resource.mobile_gcm_id = params[:user][:gcm_id]
        resource.save
      end

      # Load modules to show in mobile only are available
      modules = Dir.entries("#{Rails.root}/lib/modules/")
                .reject { |entry| ['.', '..'].include? entry }

      if File.exist? "#{Rails.root}/config/modules.yml"
        available_modules = YAML.load(File.read('config/modules.yml'))['modules']
        modules &= available_modules unless available_modules.include?('all')
      end

      render json: {
        query: true,
        auth_token: resource.authentication_token,
        email: resource.email,
        name: resource.name,
        login: resource.login,
        server_key: Setting.server_key,
        server_modules: modules
      }
      return
      last_sign_in_description = Notification.new(user: current_user, comment: "The last login acces from the user #{current_user.name} was at: #{current_user.last_sign_in_at.strftime('%a %b, %Y %I:%M %p')}")
      last_sign_in_description.save()
    end
    invalid_login_attempt
  end

  #
  # Sign out a user.
  #
  def destroy
    sign_out(resource_name)
  end

  protected

  def ensure_params_exist
    return unless params[:user][:username].blank? &&
                  params[:user][:password].blank?

    # Render an error
    error_401('Missing username or password parameter')
  end

  def invalid_login_attempt
    warden.custom_failure!
    # Render error
    error_401('Error with your login or password')
  end
end
