#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Api::V1::UsersController < Api::V1::BaseController
  # Init the user
  before_action :set_user, only: [:recreate_api_key]
  # Check permission
  before_action :need_superadmin, only: [:index]

  #
  # Show a list of user in JSON or CSV.
  #
  # @permission Super admin privileges
  #
  def index
    if params.include?(:sensor_id)
      # We return all children because this method need super admin privileges.
      # BE CAREFUL if we change this condition
      sensor_ids =
        Sensor
        .find(params[:sensor_id])
        .all_children_ids(true)
      users = User
              .where(user_params.except(:sensor_id))
              .where(sensor_id: sensor_ids)
    else
      users = User.where(user_params)
    end

    # Respond based on the format
    respond_to do |format|
      format.json do
        render json: {
          query: true,
          users: users.as_json(
                   only: [:name, :email, :id, :login, :admin,
                          :enabled, :authentication_token, :sensor_id]
                 )
        }
      end
      format.csv { render text: User.to_csv(users) }
    end
  end

  #
  # Recreate the API key of a user. We must check the permissions
  # of the users.
  #
  def recreate_api_key
    if @user.nil?
      error_404('The user doesn\'t exist.')
      return
    elsif current_user.eql?(@user) || current_user.super_admin?
      # Change the api key
      if @user.reset_authentication_token!
        res = { query: true, auth_token: @user.authentication_token }
      else
        error_500('The token can\'t be recreated now. Please retry.')
        return
      end
    else
      error_401('You don\'t have enought privileges to do that.')
      return
    end

    # Render the response
    render json: res, status: status
  end

  private

  #
  # Set the user based on params.id
  #
  def set_user
    # Use find_by_id to display an error and don't show a 500
    @user = params[:id].nil? ? nil : User.find_by_id(params[:id])
  end

  #
  # Params allowed to search users.
  #
  def user_params
    params.permit(:name, :email, :enabled, :login, :admin, :sensor_id)
  end
end
