#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class AuditsController < ApplicationController
  helper_method :sort_column, :sort_direction

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @audits = audits_list

    unless current_user.super_admin? || current_user.super_reader?
      @audits = @audits.where(user_id: current_user.id)
    end
    @page = params[:page]
    @audits = @audits.paginate(page: params[:page],
                               per_page: current_user.per_page_count)
  end

  def search
    @page = params[:page] || 1
    query_params = []
    values = {}

    if params[:search][:username].present?
      query_params << 'user_id = :user'
      values[:user] = params[:search][:username]
    end

    if params[:search][:resource].present?
      query_params << 'LOWER(auditable_type) = :resource'
      values[:resource] = params[:search][:resource].downcase
    end

    if params[:search][:action].present?
      query_params << 'LOWER(action) = :action'
      values[:action] = params[:search][:action].downcase
    end

    audits = Audited.audit_class.unscoped
                .where(query_params.join(' AND '), values).order(created_at: :desc)
    audits = audits.where(user_id: current_user.id) unless current_user.super_user?
    @audits = audits.paginate(page: @page,
                      per_page: current_user.per_page_count)
  end

  ## ======================================================================
  # => Private Methods
  ## ======================================================================

  private

  def users_list
    users = User.where('LOWER(name) LIKE :search',
                       search: "%#{params[:search].try(:downcase)}%")
    users.map(&:id)
  end

  def audits_list
    users_id = users_list
    Audited.audit_class.unscoped.order(
      sort_column + ' ' + sort_direction).where(
      'LOWER(comment) LIKE :search OR LOWER(action) LIKE :search OR
      LOWER(auditable_type) LIKE :search OR user_id IN (:users_id)',
      search: "%#{params[:search].try(:downcase)}%",
      users_id: users_id)
  end

  def sort_column
    params[:sort] ? params[:sort] : 'created_at'
  end

  def sort_direction
    %w(asc desc).include?(params[:direction]) ? params[:direction] : 'desc'
  end
end
