#######################################################################
# Copyright (c) 2021 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY] without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class CaptivePortalSessionsController < ApplicationController
    helper Redborder::ModulesHelper
    layout "#{Rails.root}/app/views/layouts/captive_portal_sessions"

    skip_before_action :authenticate_user!, raise: false

    #before_action :set_errors, only: [:new]
   
    before_action :check_permissions_and_set_sensor, only: [:new]

    def index 
        redirect_to new_captive_portal_session_path
    end

    def new
        cookies[:client_mac] = params[:client_mac] if params[:client_mac]
        @cp_session = CaptivePortalSession.new
    end

    def success
        @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]
        @cp_session = CaptivePortalSession.new
    end

    def preview
        @sensor = Sensor.find_by(:id => params[:id])
        @cp_session = CaptivePortalSession.new
    end

    def logout

    end

    def create
        if !captive_portal_session_params[:password]
		    captive_portal_session_params[:password] = cookies[:client_mac] if cookies[:client_mac]
        end

        if !captive_portal_session_params[:username] or captive_portal_session_params[:username].empty?
            params[:captive_portal_session][:username] = "#{(0...12).map { (65 + rand(26)).chr }.join.downcase}@anonymous.email"
        end
        @cp_session = CaptivePortalSession.new(captive_portal_session_params)

        respond_to do |format|
            if @cp_session.save
              format.html { render action: 'new' }
              format.json { render json: @cp_session.errors, status: :created }
            else
              format.html { render action: 'new' }
              format.json { render json: @cp_session.errors, status: :unprocessable_entity }
            end
          end
    end

    private
    def render_403
        respond_to do |format|
            format.html { render :file => "#{Rails.root}/public/403", :layout => false, :status => :not_found }
            format.xml  { head 403 }
            format.any  { head 403 }
        end
    end

    def set_sensor
        @sensor = Sensor.find(captive_portal_session_params[:sensor_id])  if captive_portal_session_params[:sensor_id]
    end

    def set_errors
        flash[:error] = params["error_message"] if params["error_message"]
    end

    def check_permissions_and_set_sensor
        flash[:error] = params["error_message"] if params["error_message"]
        ap_mac = params[:ap_mac] if params[:ap_mac]
        ap_mac = @cp_session.ap_mac if @cp_session and @cp_session.ap_mac
        if ap_mac
            Sensor.where(type: [Sensor::FLOW, Sensor::MERAKI]).each do |s|
                next unless (s.captive_portal_name.present? or s.captive_portal_logo_image_file_name.present? ) and s.access_points.count > 0
                access_point = s.access_points.where(:mac_address => ap_mac).first || nil
                @sensor = s if access_point
                break if @sensor
            end
        end
        render_403 unless @sensor
    end


    ############ PARAMS ##############
    def captive_portal_session_params
        params.require(:captive_portal_session).permit(:sensor_id, :username, :password, :switch_url, :client_mac, :ap_mac)
    end
    
end
