class IntegrationsController < ApplicationController
  # GET /integrations
  def index
    integrations = IntegrationService.get_integrations + IntegrationService.get_common_integrations
    @cards = integrations.map do |integration|
      card_data = I18n.t("integrations.cards.#{integration}", default: nil)

      if card_data
        {
          title: card_data[:title],
          description: card_data[:description],
          link: page_integrations_path(integration),
        }
      else
        {
          title: "#{integration} #{I18n.t("integrations.index.not_found")}",
          description: "#{integration} #{I18n.t("integrations.index.not_found_desc")}",
          link: "#",
        }
      end
    end
  end

  # GET /integrations/:page_name
  def show_page
    @integrations = {}
    Integration.all.each do |s|
      @integrations[s.name.to_sym] = s.value
    end

    @page_name = params[:page_name]

    # Construct the template path based on the locale
    template_path = "integrations/pages/#{@page_name}"

    if lookup_context.exists?(template_path)
      render template_path
    else
      # Handle the case where neither the localized nor the default page exists
      render file: "#{Rails.root}/public/404.html", status: :not_found
    end
  end

  # PATCH /integrations/:page_name
  def update
    integrations_params = (params[:integrations] || {}).dup.symbolize_keys
    errors = []

    integrations_params.each do |key, value|
      integration = Integration.find_or_initialize_by(name: key.to_s)
      integration.value = value

      unless integration.save
        errors << { name: integration.name, messages: integration.errors.full_messages }
      end
    end

    if errors.empty?
      respond_to do |format|
        format.html { redirect_to page_integrations_path(params[:page_name]), notice: I18n.t("integrations.common.success") }
      end
    else
      error_alert = errors.map { |e| "#{e[:name]} #{e[:messages].join(", ")}" }
      respond_to do |format|
        format.html { redirect_to page_integrations_path(params[:page_name]), alert: error_alert }
      end
    end
  end
end
