#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class LookupsController < ApplicationController
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :require_super_administrative_privileges,
                only: [
                  :new, :edit, :create, :update, :destroy
                ]
  before_action :require_super_reader_privileges, only: :index
  before_action :set_lookup, only: [:edit, :update, :destroy]

  ## ======================================================================
  # => Controller properties
  ## ======================================================================

  layout false, only: [:new, :edit]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  # GET /lookups
  # GET /lookups.json
  def index
    @lookups = Lookup.paginate(page: params[:page],
                               per_page: current_user.per_page_count)
  end

  # GET /lookups/new
  def new
    @lookup = Lookup.new
  end

  # GET /lookups/1/edit
  def edit
  end

  # POST /lookups
  # POST /lookups.json
  def create
    @lookup = Lookup.new(lookup_params)

    respond_to do |format|
      if @lookup.save
        @lookups = Lookup.paginate(page: params[:page],
                                   per_page: current_user.per_page_count)

        format.html do
          redirect_to lookups_path,
                      notice: "#{I18n.t('controllers.lookup_created')}"
        end
        format.js do
          flash[:notice] = "#{I18n.t('controllers.lookup_created')}"
        end
        format.json { head :no_content }
      else
        format.html { render action: 'new' }
        format.js do
          flash[:error] = @lookup.errors.first.map do |msg|
            # so it doesn't print the default translated error message, since it doesnt find it in the locale.
            # delete the if and try creating and lookup with an invalid url to see why it is not working.
            # it will return "value invalid url". value should not be printed
            if msg.to_s != 'value'
              "<li>#{msg}</li>"
            end
          end.join
        end
        format.json do
          render json: @lookup.errors, status: :unprocessable_entity
        end
      end
    end
  end

  # PATCH/PUT /lookups/1
  # PATCH/PUT /lookups/1.json
  def update
    respond_to do |format|
      if @lookup.update(lookup_params)
        @lookups =
          Lookup.paginate(
            page: params[:page],
            per_page: current_user.per_page_count
          )

        format.html do
          redirect_to lookups_path,
                      notice: "#{I18n.t('controllers.lookup_updated')}"
        end
        format.js { flash[:notice] = "#{I18n.t('controllers.lookup_updated')}" }
        format.json { head :no_content }
      else
        format.html { render action: 'edit' }
        format.js do
          flash[:error] = @lookup.errors.full_messages.map do |msg|
            "<li>#{msg}</li>"
          end.join
        end
        format.json do
          render json: @lookup.errors, status: :unprocessable_entity
        end
      end
    end
  end

  # DELETE /lookups/1
  # DELETE /lookups/1.json
  def destroy
    @lookup.destroy
    respond_to do |format|
      format.html { redirect_to lookups_url }
      format.json { head :no_content }
    end
  end

  def getaddrinfo
    ips = params[:ips]

    names = resolv_ips(ips)

    if names.length == 0
      render json: {}
    elsif names.length == 1
      render json: { name: names.first, names: names }
    else
      render json: { names: names }
    end
  end

  private

  #
  # => Use callbacks to share common setup or constraints between actions.
  #
  def set_lookup
    @lookup = Lookup.find(params[:id])
  end

  #
  # => Never trust parameters from the scary internet,
  #    only allow the white list through.
  #
  def lookup_params
    params.require(:lookup).permit(:title, :value)
  end

  def resolv_ips(ips)
    ips = [] unless ips.respond_to? 'each'

    names = {}
    ips.each do |ip|
      begin
        if names[ip].nil?
          ip_aux = IPAddr.new(ip.squish)

          name = Rails.cache
                 .fetch "rb-webui:ip:#{ip}", expires_in: 30.minutes do
            begin
              NetObject.find_by(value: ip).try('name') ||
              timeout(0.5) { Socket.getnameinfo(Socket.sockaddr_in(80, ip))[0] }
            rescue Timeout::Error
              ip_aux.to_s
            end
          end
        else
          name = ip
        end

        # inserting names
        names[ip] = name
      rescue
        names[ip] = ip
      end
    end

    names
  end
end
