#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ModelSourcesController < ApplicationController
  include ModelSourcesHelper
  
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_filter :require_administrative_privileges, except: [:index]

  before_filter :set_model_source, only: [:update]

  before_action :check_updating, only: [:index]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    respond_to do |format|
      format.html do
        @model_sources = ModelSource.all
      end
      format.js do
        @model_source = ModelSource.find(params[:rule_source_id])
      end
    end
  end

  def show
    
  end

  def update
    if @model_source.update(model_source_params)
      respond_to do |format|
        format.html { redirect_to model_sources_path, notice: 'Model Source has been updated' }
        format.json { render json: @model_source }
      end
    else
      respond_to do |format|
        format.html { redirect_to model_sources_path, alert: @model_source.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
        format.json { render json: @model_source.errors }
      end
    end
  end

  def check_model_job_status
    running = is_model_downloading?
  
    render json: { running: running }
  end

  def force_update
    if ModelSource.where(enable: true).exists?
      DelayedJob.force_model_update
      redirect_to model_sources_path, notice: "#{I18n.t('controllers.updating_models')}"
    else
      redirect_to model_sources_path, alert: "#{I18n.t('controllers.no_model_enabled')}"
    end
  end

  def check_update
    @model_sources = ModelSource.all 
  end

  private

  def check_updating
    unless DelayedJob.model_update && (DelayedJob.model_update.running? ||
      DelayedJob.model_update.run_at < (Time.now + 1.minute))
      ModelSource.all.update_all(updating: false)
    end
  end

  def set_model_source
    @model_source = ModelSource.find(params[:id])
  end

  def model_source_params
    params.require(:model_source).permit(
      :enable
    )
  end
end
