#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class OverlayMapsController < ApplicationController
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_filter :require_super_reader_privileges,
                except: [
                  :image, :original_image, :original_image_small
                ]

  before_filter :require_super_administrative_privileges,
                except: [
                  :image, :original_image, :original_image_small, :index
                ]

  before_action :set_overlay_map,
                only:
                  [:edit, :update, :destroy,
                   :image, :original_image,
                   :image_small, :original_image_small]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @overlay_maps =
      OverlayMap.paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
  end

  def new
    @overlay_map = OverlayMap.new
  end

  def create
    @overlay_map = OverlayMap.new(overlay_map_params)
    @overlay_map.sensor = Sensor.root

    if @overlay_map.save
      redirect_to edit_overlay_map_path(@overlay_map),
                  notice: "#{I18n.t('controllers.overlay_created')}"
    else
      render action: 'new'
    end
  end

  def edit
  end

  def update
    respond_to do |format|
      if @overlay_map.update(overlay_map_params)
        format.html do
          redirect_to overlay_maps_path,
                      notice: "#{I18n.t('controllers.overlay_updated')}"
        end
        format.js do
          flash[:notice] = "#{I18n.t('controllers.overlay_updated')}"
        end
        format.json { render json: { status: 200 } }
      else
        format.html { render action: 'edit' }
        format.js do
          flash[:error] = @overlay_map.errors.full_messages.map do |msg|
            "<li>#{msg}</li>"
          end.join
        end
        format.json do
          render json: @overlay_map.errors, status: :unprocessable_entity
        end
      end
    end
  end

  def destroy
    @overlay_map.destroy
    redirect_to overlay_maps_path
  end

  def image
    img = open(
            @overlay_map.image_uri,
            ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
          ).read
    render text: img, content_type: @overlay_map.image.content_type
  end

  def original_image
    img = open(
            @overlay_map.original_image_uri,
            ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
          ).read
    render text: img, content_type: @overlay_map.original_image.content_type
  end

  def image_small
    img = open(
            @overlay_map.image_uri_small,
            ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
          ).read
    render text: img, content_type: @overlay_map.image.content_type
  end

  def original_image_small
    img = open(
            @overlay_map.original_image_uri_small,
            ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
          ).read
    render text: img, content_type: @overlay_map.original_image.content_type
  end

  private

  def overlay_map_params
    params.require(:overlay_map).permit(:name, :ne, :sw, :enabled, :image)
  end

  def set_overlay_map
    @overlay_map = OverlayMap.find(params[:id])
  end
end
