#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class PasswordsController < Devise::PasswordsController
    
  layout 'login'
    
  # PUT /resource/password
  def update
    self.resource = resource_class.reset_password_by_token(resource_params)
    yield resource if block_given?

    if resource.errors.empty?
      resource.unlock_access! if unlockable?(resource)
      if Devise.sign_in_after_reset_password
        flash_message = resource.active_for_authentication? ? :updated : :updated_not_active
        set_flash_message(:notice, flash_message) if is_flashing_format?
        change_expired_date(resource)
        sign_in(resource_name, resource)
      else
        set_flash_message(:notice, :updated_not_active) if is_flashing_format?
      end
      respond_with resource, location: root_path
    else
      set_minimum_password_length
      respond_with resource
    end
  end

  def change_expired_date(resource)
    resource.update(pwd_changed_at: Time.now)
  end

  def change_force_password(resource)
    resource.update(force_pwd_change: false)
  end

  def reset
    @minimum_password_length = (Setting.find_by(name: 'minimum_password_length').try(:value) || 12).to_i
    self.resource = resource_class.new
    resource.reset_password_token = params[:reset_password_token]
  end

  def password_evaluation
    resource = User.find_by(reset_password_token: resource_params[:reset_password_token])
    minium_password_length = (Setting.find_by(name: 'minimum_password_length').try(:value) || 12).to_i

    if !resource.valid_password?(params[:user][:current_password])
      resource.errors.add(:current_password, "#{I18n.t('controllers.invalid_password')}")
    elsif (params[:user][:password].length < minium_password_length || params[:user][:password].length > 128)
      resource.errors.add(:password, "#{I18n.t('controllers.password_length')}")
    elsif !password_complexity
      resource.errors.add(:password, "#{I18n.t('controllers.password_format_error')}")
    elsif (params[:user][:password]) != (params[:user][:password_confirmation])
      resource.errors.add(:password, "#{I18n.t('controllers.diff_password')}")
    elsif password_change? and password_is_too_similar?
      resource.errors.add(:password, "#{I18n.t('controllers.similar_password')}")
    end

    if resource.errors.empty?
      resource.unlock_access! if unlockable?(resource)
      resource.update(password: resource_params[:password])
      if Devise.sign_in_after_reset_password
        flash_message = resource.active_for_authentication? ? :updated : :updated_not_active
        set_flash_message(:notice, flash_message) if is_flashing_format?
        change_expired_date(resource)
        change_force_password(resource)
        sign_out resource
        sign_in(resource_name, resource)
      else
        set_flash_message(:notice, :updated_not_active) if is_flashing_format?
      end
      redirect_to new_user_session_path
    else
      sign_out resource
      redirect_to reset_url(resource, reset_password_token: resource_params[:reset_password_token]), alert: resource.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join
    end
  end

  def password_complexity
    minium_password_length = (Setting.find_by(name: 'minimum_password_length').try(:value) || 12).to_i
    
    return true if params[:user][:password].blank? || params[:user][:password] =~ /^(?=.*[a-z])(?=.*[A-Z])(?=.*[0-9])(?=.*[ !"#\$%&'*()+,-.\/:;<=>?@\[\]\\^_`{}|~])[a-zA-Z0-9 !"#\$%&'*()+,-.\/.;<=>?@\[\]\\^_`{}|~]{#{minium_password_length},128}$/
    return false
  end

  def password_change?
    !params[:user][:password].blank? and !params[:user][:current_password].blank?
  end

  ## Returns true if you don't have to change the password, false otherwise
  def password_is_too_similar?
    resource = User.find_by(reset_password_token: resource_params[:reset_password_token])

    count_same_letter = 0
    params[:user][:current_password].split(//).each_with_index do |letter, index|
      if index < params[:user][:password].split(//).count
        count_same_letter += 1 if params[:user][:password].split(//)[index] == letter
      end
    end
    if count_same_letter >= 4
      return true # the password is too similar
    else
      return false
    end
  end

end
