#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class PluginsController < ApplicationController
  before_action :require_administrative_privileges
  before_action :set_plugin, only: [:update, :destroy, :updating, :check_is_applying]

  def index
    @plugins = Plugin.all.order(:name)
  end

  def new
    @plugin = Plugin.new
  end

  def create
    @plugin = Plugin.new(plugin_params)

    respond_to do |format|
      if @plugin.save
        format.html { redirect_to @plugin, notice: "#{I18n.t 'controllers.plugin_created'}" }
        format.json { render action: 'show', status: :created, location: @plugin }
      else
        format.html { render action: 'new' }
        format.json { render json: @plugin.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    respond_to do |format|
      if @plugin.update(plugin_params)
        format.html { redirect_to @plugin, notice: "#{I18n.t 'controllers.plugin_updated'}" }
        format.json { head :no_content }
      else
        format.html { render action: 'edit' }
        format.json { render json: @plugin.errors, status: :unprocessable_entity }
      end
    end
  end

  def check_is_applying
    respond_to do |format|
      format.json { render json: @plugin, status: 200 }
    end
  end

  def updating
      version = params[:version]
      enabled = params[:enabled]

      if @plugin.update(is_applying: true)
        Delayed::Job.enqueue(Redborder::Jobs::UpdatePluginJob.new(@plugin.id,
                                                                current_user.id,
                                                                version,
                                                                enabled),
                            priority: 10, run_at: Time.now)

        respond_to do |format|
          format.html { redirect_to @plugin, notice: "#{I18n.t 'controllers.plugin_updated'}" }
          format.json { head :no_content }
        end
      else
        respond_to do |format|
          format.html { render action: 'index' }
          format.json { render json: @plugin.errors, status: :unprocessable_entity }
        end
      end
  end

  def force_plugins_update
    DelayedJob.force_plugins_update(current_user.id, true)

    respond_to do |format|
      format.html { redirect_to jobs_url }
    end
  end

  def destroy
    @plugin.destroy
  end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_plugin
      @plugin = Plugin.find(params[:id])
    end

    # Never trust parameters from the scary internet, only allow the white list through.
    def plugin_params
      params.require(:plugin).permit(:name,:version,:enabled)
    end
end
