#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ProductsController < ApplicationController
  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @search_query = params[:fast_search].strip
    # Init the time
    @start_time = if params[:start_time]
                    in_time_machine { Time.zone.at(params[:start_time].to_i) }
                  else
                    in_time_machine { Time.zone.now.beginning_of_day }
                  end

    @end_time = if params[:end_time]
                  in_time_machine { Time.zone.at(params[:end_time].to_i) }
                else
                  in_time_machine { Time.zone.now.end_of_day }
                end

    # Init the results
    ips_result = []
    flow_result = []
    social_result = []

    # Get the data
    ips_result = ips_query if
      Redborder::Module.registered_modules.include? :ips
    flow_result = flow_query if
      Redborder::Module.registered_modules.include? :flow
    social_result = social_query if
      Redborder::Module.registered_modules.include? :social

    # Return results
    @results = ips_result + flow_result + social_result

    past_search if @results.blank?
  end

  protected

  #
  # Try to find data a week ago if the current day hasn't got any data for this
  # search.
  #
  def past_search
    index = 7

    while index > 0 && @results.blank?
      # Reduce the time
      @start_time -= 1.day
      @end_time -= 1.day

      # Init results
      ips_result = []
      flow_result = []
      social_result = []

      # Get the data
      ips_result = ips_query if
        Redborder::Module.registered_modules.include? :ips
      flow_result = flow_query if
        Redborder::Module.registered_modules.include? :flow
      social_result = social_query if
        Redborder::Module.registered_modules.include? :social

      # Return the results
      @results = ips_result + flow_result + social_result

      # Decrease the index for the next past search
      index -= 1
    end
  end

  # ----------------------------------------------------
  # Helper functions to get the data. This methods is based on global
  # variables
  # ----------------------------------------------------

  #
  # Return the data for a search on IPS data
  # TODO: move this method to IPS module
  #
  def ips_query
    Event
      .where(
        granularity: :all,
        start_time: @start_time,
        end_time: @end_time,
        query_type: :search,
        search_value: @search_query
      )
      .as_json
      .each { |x| x.merge!('product' => 'ips') }
  end

  #
  # Return the data for a search on Flow data
  # TODO: move this method to Flow module
  #
  def flow_query
    Traffic
      .where(
        granularity: :all,
        start_time: @start_time,
        end_time: @end_time,
        query_type: :search,
        search_value: @search_query
      )
      .as_json
      .each { |x| x.merge!('product' => 'flow') }
  end

  #
  # Return the data for a search on Social data
  # TODO: move this method to Social module
  #
  def social_query
    Social
      .where(
        granularity: :all,
        start_time: @start_time,
        end_time: @end_time,
        query_type: :search,
        search_value: @search_query
      )
      .as_json
      .each { |x| x.merge!('product' => 'social') }
  end
end
