#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class RegistrationsController < Devise::RegistrationsController
  include Redirections
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_filter :require_administrative_privileges, only: [:create, :new]
  before_filter :check_permission, only: :create
  skip_before_filter :require_no_authentication, only: [:create]
  before_action :set_minimum_password_length, only: [:edit]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def update
    method =
    if (RedBorder::CONFIG[:authentication_mode] == 'database') || (RedBorder::CONFIG[:authentication_mode] == 'saml')
      'update_with_password'
    else
      'update_attributes'
    end

    if password_change? and password_is_too_similar?
      resource.errors.add(:password, "#{I18n.t('controllers.similar_password')}")
    end

    if resource.errors.empty? && resource.send(method, user_params)
      resource.update(pwd_changed_at: Time.now)
      set_flash_message :notice, :updated
      redirect_to edit_user_registration_path
    else
      clean_up_passwords(resource)
      flash[:error] = @user.errors.full_messages
                      .map { |msg| "<li>#{msg}</li>" }.join
      render 'registrations/edit'
    end
  end

  def create
    begin
      @user = User.new(user_params)

      @user.admin = params[:user][:admin] == '1' ? true : false

      @user.pwd_changed_at = Time.now
      @user.remember_created_at = Time.now

      DittoCode::Exec.if 'COMMUNITY' do
        @user.sensor_id = Sensor.root.id
      end

      if @user.save
        @user.send_creation_instructions(params[:user][:password])
        flash[:notice] = "#{I18n.t('controllers.user_created')}"
        redirect_to users_path
      else
        clean_up_passwords @user
        flash[:error] = @user.errors.full_messages
                      .map { |msg| "<li>#{msg}</li>" }.join
        render 'users/new'
      end
    rescue Net::OpenTimeout
      redirect_and_error "#{I18n.t('controllers.email_error')}", redirect_to: users_path
    end
  end

  private

  def password_change?
    !params[:user][:password].blank? and !params[:user][:current_password].blank?
  end

  ## Returns true if you don't have to change the password, false otherwise
  def password_is_too_similar?
    resource = User.find_by(reset_password_token: resource_params[:reset_password_token])

    count_same_letter = 0
    params[:user][:current_password].split(//).each_with_index do |letter, index|
      if index < params[:user][:password].split(//).count
        count_same_letter += 1 if params[:user][:password].split(//)[index] == letter
      end
    end
    if count_same_letter >= 4
      return true # the password is too similar
    else
      return false
    end
  end


  DittoCode::Exec.if 'ENTERPRISE' do
    def check_permission
      return if current_user
                .domains_with_manage_permission
                .exists?(params[:user][:sensor_id].to_i)
      no_enough_privileges redirect_to: users_path
    end
  end

  DittoCode::Exec.if 'COMMUNITY' do
    def check_permission
      return if User.current_user.super_admin?
      no_enough_privileges redirect_to: users_path
    end
  end

  def user_params
    if current_user.super_admin?
      params.require(:user).permit(:name, :login, :email,
                                    :password, :password_confirmation,
                                    :current_password, :per_page_count,
                                    :root_page, :sensor_id, :admin,
                                    :letter_size, :preferred_locale,
                                    :force_pwd_change)
    else
      params.require(:user).permit(:name, :login, :email,
                                    :password, :password_confirmation,
                                    :current_password, :per_page_count,
                                    :root_page, :sensor_id, :letter_size, :preferred_locale)
    end
  end

  def set_minimum_password_length
    @minimum_password_length = (Setting.find_by(name: 'minimum_password_length').try(:value) || 12).to_i
  end
end
