#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Controller for Sensors Tree Filters
#
# This controller works with sensors_tree_filters.js library in order to apply
# filters from sensors tree modal
#
class SensorsTreeFiltersController < ApplicationController
  # Storage of the filters
  include Redborder::Filters::Storage::Session

  self.view_paths = 'app/views/sensors'

  SOURCES = Redborder::Module.registered_modules.map { |_, m| m[:data_source] }.compact.freeze

  #
  # GET /sensors/tree_filters
  #
  def index
    @sensors = Sensor.root_children.where(type: Sensor::DOMAIN)
    @source = params[:filter_source]
  end

  #
  # GET /sensors/tree_filters/:id
  #
  # Action for expanding a branch at Sensors' tree
  #
  def show
    @sensors = current_user.sensor.descendants.find_by(id: params[:id])
                           .children.where(type: Sensor::DOMAIN)
    render layout: false
  end

  #
  # POST /sensors/tree_filters/
  #
  # Add Selected Domain as Filters
  #
  def create
    filter_source = SOURCES.find do |source|
      source == params[:filter_source]
    end
    # init filter object
    if SOURCES.include?(filter_source)
      filter = Redborder::Filter.new(filter_source.constantize, load_filters)
      # get domains sensors to filter
      domains_to_filter = current_user.sensor.descendants.where(type: Sensor::DOMAIN,
                                                                id: params[:sensors_tree_filters])

      # add every filter
      domains_to_filter.each do |domain|
        filter.add("#{domain.domain_type_name(to_key: true)}_uuid", domain.uuid)
      end

      save_filters(filter.filters)
    end

    render nothing: true
  end
end
