#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ThreatIntelSourcesController < ApplicationController

  require 'uri'
  
  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :require_super_administrative_privileges,
                only: [
                  :new, :edit, :create, :update, :destroy
                ]

  before_action :set_threat_intel_source, only: [:edit, :update, :destroy]

  layout false, only: [:new, :edit]
  

  ## ======================================================================
  # => Actions
  ## ======================================================================
                
  def index

    sources = ThreatIntelSource.all

    @sources_ip = sources
    .order(created_at: :desc)
    .where('lower(threat_src_type) LIKE ?', 'ip')
    .paginate(page: params[:ip_page], per_page: current_user.per_page_count / 5)
  
    @sources_domain = sources
    .order(created_at: :desc)
    .where('lower(threat_src_type) LIKE ?', 'domain')
    .paginate(page: params[:domain_page], per_page: current_user.per_page_count / 5)

    @sources_sha_1 = sources
    .order(created_at: :desc)
    .where('lower(threat_src_type) LIKE ?', 'sha_1')
    .paginate(page: params[:sha_1_page], per_page: current_user.per_page_count / 5)

    @sources_sha_2 = sources
    .order(created_at: :desc)
    .where('lower(threat_src_type) LIKE ?', 'sha_2')
    .paginate(page: params[:sha_2_page], per_page: current_user.per_page_count / 5)

    @sources_url = sources
    .order(created_at: :desc)
    .where('lower(threat_src_type) LIKE ?', 'url')
    .paginate(page: params[:url_page], per_page: current_user.per_page_count / 5)
    
    @sources = [{'ip' => @sources_ip}, 
    {'domain' => @sources_domain}, 
    {'sha_1' => @sources_sha_1}, 
    {'sha_2' => @sources_sha_2},
    {'url' => @sources_url}]

    respond_to do |format|
      format.html
      format.js
    end
    # end
  end

  def show
    @threat_intel_source = ThreatIntelSource.find(params[:id])
  end

  def set_threat_intel_source
    @source = ThreatIntelSource.find(params[:id])
  end

  def threat_intel_source_params
    # Parse url into host port and path
    url = params[:threat_intel_source][:url]
    if url && !url.empty?
      uri = URI(params[:threat_intel_source][:url])
      params[:threat_intel_source][:host] = uri.host
      params[:threat_intel_source][:path] = uri.request_uri
      params[:threat_intel_source][:port] = uri.port
    end

    # if !params[:threat_intel_source][:password].empty?
  
    params
      .require(:threat_intel_source)
      .permit(:name, :description, :user, :host, :port, :url, :path, :anonymous, :enable, :threat_conn_types_id, :threat_src_type, :page, :search, :fast_search)
  end

  def new
    @threat_conn_types = ThreatConnType.all
    @source = ThreatIntelSource.new
    @source.threat_src_type = params[:threat_src_type] || 'ip'
    @source.enable = params[:enable] || true
    @url = ""
  end


  def edit
    @source = ThreatIntelSource.find(params[:id])
    if ThreatConnType.find_by_name('HTTP').id == @source.threat_conn_types_id
      uri = URI::HTTP.build(host: @source.host, port: @source.port, path: @source.path)
      url = uri.to_s
    elsif ThreatConnType.find_by_name('HTTPS').id == @source.threat_conn_types_id
      uri = URI::HTTPS.build(host: @source.host, port: @source.port, path: @source.path)
      url = uri.to_s
    end
    @url = url
  end

  def create
    @source = ThreatIntelSource.new(threat_intel_source_params)

    respond_to do |format|
      if @source.save
        begin
          if !@source.anonymous && !@source.user.blank?
            password = params[:threat_intel_source][:password]
            store_password_in_databag(password, @source.id)
          end
        rescue Net::HTTPFatalError => e
          Rails.logger.info 'INFO: Failed creation of password for databag passwords threat_intel_sources'
        end
        @sources = ThreatIntelSource.paginate(page: params[:page],
                      per_page: current_user.per_page_count)
        format.html do
          redirect_to threat_intel_sources_path,
                      notice: "#{I18n.t('controllers.threat_intel_source_created')}"
        end
        format.js { flash[:notice] = "#{I18n.t('controllers.threat_intel_source_created')}" }
        format.json { head :no_content }
      else
        format.html { render action: 'new' }
        format.js do
          flash[:error] = @source.errors.full_messages.map do
            |msg| "<li>#{msg}</li>"
          end.join
        end
        format.json do
          render json: @source.errors, status: :unprocessable_entity
        end
      end
    end
  end

  def update
    respond_to do |format|
      if @source.update(threat_intel_source_params)
        begin
          if !@source.anonymous && !@source.user.blank?
            password = params[:threat_intel_source][:password]
            store_password_in_databag(password, @source.id)
          end
        rescue Net::HTTPFatalError => e
          Rails.logger.info 'INFO: Failed creation of password for databag passwords threat_intel_sources'
        end
        @sources =
          ThreatIntelSource.paginate(
            page: params[:page],
            per_page: current_user.per_page_count
          )
        format.html do
          redirect_to threat_intel_sources_path,
                      notice: "#{I18n.t('controllers.threat_intel_source_updated')}"
        end
        format.js { flash[:notice] = "#{I18n.t('controllers.threat_intel_source_updated')}" }
        format.json { head :no_content }
      else
        format.html { render action: 'edit' }
        format.js do
          flash[:error] = @source.errors.full_messages.map do |msg|
            "<li>#{msg}</li>"
          end.join
        end
        format.json do
          render json: @source.errors, status: :unprocessable_entity
        end
      end
    end
  end

  def destroy
    delete_data_bag @source.id.to_s
    @source.destroy
    respond_to do |format|
      format.html { redirect_to threat_intel_sources_url }
      format.json { head :no_content }
    end
  end

  def store_password_in_databag(password, source_id)
    # If databag already exists
    begin
      data_bag = Chef::DataBagItem.load('passwords', 'threat_intel_sources')
      data_bag[source_id] = password
      data_bag.save
    rescue Errno::ECONNREFUSED, ArgumentError
      Rails.logger.error 'ERROR: when access databag: cannot contact erchef server'
    # Create databag if it doesnt exists
    rescue Net::HTTPServerException
      Rails.logger.info "INFO: Databag passwords threat_intel_sources doesn't exist. Creating..."
      data_bag = Chef::DataBagItem.new
      data_bag.data_bag 'passwords'
      data_bag['id'] = 'threat_intel_sources'
      data_bag[source_id] = password
      if data_bag.save
        Rails.logger.info 'INFO: Databag passwords threat_intel_sources created.'
      else
        Rails.logger.error 'ERROR: when access databag, cannot contact erchef server'
        raise Net::HTTPFatalError.new('ERROR: when access databag, cannot contact erchef server', nil)
      end
    end
  end

  def delete_data_bag(id)
    data_bag = Chef::DataBagItem.load('passwords', 'threat_intel_sources')
    if data_bag[id]
      data_bag.delete(id)
      data_bag.save
    end
  end

end