DittoCode::HoldFile.if 'ENTERPRISE'
#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
 
#
# Perform operations to Tiers
#
class TiersController < ApplicationController
  # Disable layout for this controller
  layout false
  
  # -------------------------------------------------------------
  # Before actions
  # -------------------------------------------------------------

  # This is only for super admins
  before_action :require_super_administrative_privileges
  # Init tier
  before_action :set_tier, only: [:save, :destroy, :move_to_tier,
                                  :confirm_destroy]
  # Init data to move a tier
  before_action :set_historical, only: [:move_to_tier]

  #
  # Render a modal to create a new Tier
  #
  def new
    @tier = Tier.new
  end

  #
  # Create a new Tier in database
  #
  def create
    @tier = Tier.new(new_tier_data.merge(user: current_user))
    @tier.save
  end

  #
  # Edit Tier both in database and chef values
  #
  def edit
    @tier = Tier.find(params[:id])
  end

  def update
    @tier = Tier.find(params[:id])
    @tier.update(new_tier_data)
  end

  #
  # Create a modal to confirm destroy
  #
  def confirm_destroy
    # Check the number of nodes inside this tier
    @historicals = []
    Tier.chef_historical_tiers_names.each do |k, v|
      next if v != @tier.name
      @historicals << k
    end
  end

  #
  # Destroy a Tier and move elements to another
  #
  def destroy
    @historicals = []
    # Destroy the element
    return unless @tier.destroy
    # Move elements
    return unless params[:destination_tier]
    @dst_tier = Tier.find_by(name: params[:destination_tier])
    Tier.chef_historical_tiers.each do |k, v|
      next if v.tier != @tier.name
      @historicals << v
    end
    # Move nodes
    move_nodes(@historicals, @dst_tier)
  end

  #
  # Move a node to a tier
  #
  def move_to_tier
    # Set new tier
    @node.update_tier(Tier.default_tier_name(@tier.name), @tier.memory)
    if @node.save
      render json: { query: true }
    else
      render json: { query: false, error: @node.errors }
    end
  end

  private

  #
  # Move multiple nodes to a tier
  #
  # == Parameters:
  # nodes::
  #   Array with nodes to move
  # dst::
  #   Destination Tier
  #
  def move_nodes(nodes, dst)
    nodes.each do |node|
      node.update_tier(Tier.default_tier_name(dst.name), dst.memory)
      node.save
    end
  end

  #
  # Initialize varaibles to move a node to a tier
  #
  def set_historical
    @node_name = params[:node]
    # Load node
    @node = DruidManager.new @node_name
  end

  #
  # Returns data from request to create a new tier
  #
  def new_tier_data
    params.require(:tier).permit(:name, :memory)
  end

  #
  # Initialize Tier
  #
  def set_tier
    @tier = Tier.find(params[:id])
  end
end
