#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Methods to help the UI of dashboards
#
module DashboardHelper
  def monitor_alarms(widget, user)
    @widgetid = widget.id || nil
    @tabName = Widget.where("id = #{@widgetid.to_i} AND property LIKE ?", "%category: redfish_%").map{|k| k.property[:category].to_sym}
    @widgetName = Widget.find(@widgetid).name
    @alarms = []
    @status = true
    all_alarms = if user.super_admin?
                   Alarm.all
                 else
                   user.alarms
                 end

    all_alarms.each do |a|
      property = Widget.where("id = #{@widgetid} AND property LIKE ?", "%filters: %").map{ |k|
           if k.property[:filters]
             condition = JSON.parse(a.condition)
             if condition.first and condition.first[1] and condition.first[1]["value"]
               k.property[:filters].include?(condition.first[1]["value"])
             end
           end
      }

      if property and property.first and @tabName and @tabName.first and a.monitor_tab.include? @tabName.first.to_s
        @alarms << a
      end
    end

    @alarms.each do |a|
      if !a.fired.nil? && (a.fired + a.interval) > Time.now
         @status = false
      end
    end
    widget['property']['status'] = @status;
  end

  def devicestatus_alarms(widget, user)
    @status = "error"
    @sensorid = widget.property['device_sensor'] || nil
    if !Sensor.find_by(id: @sensorid).nil?
      @sensorName = Sensor.find(@sensorid).name
      @alarms = []
      @status = true
      all_alarms = if user.super_admin?
                     Alarm.all
                   else
                     user.alarms
                   end

      all_alarms.each do |a|
        @alarms << a if a.condition.include? @sensorName.to_s
      end

      @alarms.each do |a|
        if !a.fired.nil? && (a.fired + a.interval) > Time.now
           @status = false
        end
      end
    end
    widget['property']['status'] = @status;
  end

  #
  # => Provides information from device sensors.
  #    Render the 'device_rendered' view on the widget content.
  #
  def device_rendered(widget)
    begin
      @sensorid = widget.property['device_sensor'] || nil
      @chassis_info = {}
      @processor_info = {}
      @pcie_info = {}
      @source = Redborder::DruidSource.get(source: 'monitor')
      @sensors = if @sensorid
                    Sensor.where(:id => @sensorid)
                 else
                    Sensor.where(:type => Sensor::DEVICE)
                 end

      @sensors.each do |sensor|

        @start_time = Time.now - 120
        @end_time = Time.now

        # retrieve the chassis information
        @chassis_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_chassisinfo' },
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )

        # retrieve the processor information
        @processor_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_processorinfo' },
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )

        # retrieve the pcie information
        @pcie_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_pcieinfo' },
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )

      end

      @status = MonitorController.new.render_to_string(:template => 'monitor/device_rendered.erb', :locals => {:sensors => @sensors, :chassis_info => @chassis_info, :processor_info => @processor_info, :pcie_info => @pcie_info}, :layout => false)
      widget['property']['status'] = @status;

    rescue StandardError => e
      @chassis_info = nil
      @processor_info = nil
      @pcie_info = nil
      Rails.logger.error e
    end
  end

  #
  # => Render a collection of widgets based on their graph_type
  #
  def render_widgets(opts = {})
    collection = opts.fetch(:collection, [])
    # If I use fetch here, the app breaks on send report job
    user = opts[:user] || User.current_user
    locals = opts.fetch(:locals, {})

    final_html = collection.each_with_object('') do |widget, html|
      widget_locals = locals.dup

      # The current type for the widget type will be the user time machine time
      unless user.time_machine_now.nil?
        widget.wtype.now = Time.at user.time_machine_now
      end

      widget.apply_global_filters(@global_filters.except(:sensor_name))
      widget_locals.merge!(widget: widget)
      graph_name = widget.graph.class.name.demodulize.underscore
      partial_file = "widgets/displays/#{graph_name}"

      if (widget.property['graph_type'] == "multiserie")
        monitor_alarms(widget, user)
      end

      if (widget.property['graph_type'] == "hardwarestatus")
        devicestatus_alarms(widget, user)
      end

      if (widget.property['graph_type'] == "hardwareinfo")
        device_rendered(widget)
      end

      begin
        html << render(partial: partial_file, locals: widget_locals)
      rescue StandardError
        html << render(partial: 'widgets/displays/default',
                       locals: widget_locals)
      end
    end

    final_html.html_safe
  end

  #
  # => Render a widget based on its graph_type
  #
  def render_widget(widget, user = current_user)
    render_widgets(collection: [widget], user: user)
  end

  # Render widgets for overview
  #
  def render_overview_widgets(user = current_user)
    # String to render
    html = ''
    user_modules = user.modules
    Redborder::Module.registered_modules.each do |_, prop|
      overview_widgets = prop[:overview_widgets]
      next unless overview_widgets.present? &&
                  user_modules.include?(prop[:path].downcase)
      # Iterate over widgets
      overview_widgets.sort_by { |w| w[:position] }.each do |w|
        next unless (w[:partial] && Setting.send("overview_#{w[:partial]}?"))
        if w[:name].start_with? "tr."
          w[:name] = I18n.t w[:name][3..-1]
        end
        if w[:info].to_s.start_with? "tr."
          html << render(partial: "overview/#{w[:partial]}", locals: { info: "#{I18n.t w[:info][3..-1]}" })
        else
          html << render(partial: "overview/#{w[:partial]}", locals: { info: w[:info] })
        end
      end
    end
    html.html_safe
  end

  #
  # => Return the collection of colours available for
  #    dashboard background
  #
  def dashboard_background_collection
    [
      # Monochromatic
      ["#{I18n.t('dashboard_helper.dark')}", '#111316'],
      ["#{I18n.t('dashboard_helper.dark_gray')}", '#C1C1C1'],
      ["#{I18n.t('dashboard_helper.light_gray')}", '#ededed'],
      ["#{I18n.t('dashboard_helper.white')}", '#F7F7F7'],
      # Blue
      ["#{I18n.t('dashboard_helper.universe')}", '#272F3C'],
      ["#{I18n.t('dashboard_helper.dark_blue')}", '#283F66'],
      ["#{I18n.t('dashboard_helper.light_blue')}", '#C9DCEC'],
      ["#{I18n.t('dashboard_helper.light_violet')}", '#EEE1F7'],
      # Green
      ["#{I18n.t('dashboard_helper.pool')}", '#183725'],
      # Red
      ["#{I18n.t('dashboard_helper.red')}", '#711C1C'],
      ["#{I18n.t('dashboard_helper.orange')}", '#FFD0AD'],
      ["#{I18n.t('dashboard_helper.light_red')}", '#ECD0C9']
    ]
  end

  #
  # => Return true if exists any global filter applied
  #
  def any_global_filter?(global_filters)
    return false unless global_filters.present?
    return false if global_filters.keys.size == 1 &&
                    global_filters[:advanced] &&
                    global_filters[:advanced].blank?
    true
  end

  def dashboard_url_encoded(dashboard_id)
    parameters = { id: dashboard_id }
    parameters.merge!(globals: session['global_filters']) if any_global_filter?(session['global_filters'])
    dashboard_url(parameters).to_s
  end
end
