# frozen_string_literal: true

#
# Module to help the generation of PDF files with Grover
#
module GroverHelper
  # Asset helper to include assets in the Grover HTML
  module GroverAssetsHelper
    def grover_stylesheet_include_inline(*sources)
      sources.collect do |source|
        if Rails.env.development?
          asset = Rails.application.assets.find_asset("#{source}.css")
          if asset
            "<style type='text/css'>#{asset.source}</style>"
          else
            Rails.logger.error "Asset not found: #{source}.css"
            "<!-- Asset not found: #{source}.css -->"
          end
        else
          asset_path = Rails.application.assets_manifest.assets["#{source}.css"]
          if asset_path && File.exist?(Rails.root.join('public', 'assets', asset_path))
            "<style type='text/css'>#{File.read(Rails.root.join('public', 'assets', asset_path))}</style>"
          else
            Rails.logger.error "Asset not found: #{source}.css"
            "<!-- Asset not found: #{source}.css -->"
          end
        end
      end.join("\n").html_safe
    end

    def grover_javascript_include_inline(*sources)
      sources.collect do |source|
        if Rails.env.development?
          asset = Rails.application.assets.find_asset("#{source}.js")
          if asset
            "<script type='text/javascript'>#{asset.source}</script>"
          else
            Rails.logger.error "Asset not found: #{source}.js"
            "<!-- Asset not found: #{source}.js -->"
          end
        else
          asset_path = Rails.application.assets_manifest.assets["#{source}.js"]
          if asset_path && File.exist?(Rails.root.join('public', 'assets', asset_path))
            "<script type='text/javascript'>#{File.read(Rails.root.join('public', 'assets', asset_path))}</script>"
          else
            Rails.logger.error "Asset not found: #{source}.js"
            "<!-- Asset not found: #{source}.js -->"
          end
        end
      end.join("\n").html_safe
    end
  end
end
