#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

# Helper for mails
#
module MailerHelper
  include TimeHelper

  #
  # Get the height of the bar based on the severity.
  #
  # @param alarm [Alarm] Alarm to get the severity
  # @return [Integer] Height of the bar
  #
  def height_from_severity(alarm)
    case alarm.severity.try(:name)
    when 'very low'
      20
    when 'low'
      50
    when 'medium'
      100
    when 'high'
      133
    when 'very high'
      166
    when 'extremely high'
      190
    when 'critical'
      200
    else
      100
    end
  end

  # Return the Current company stored in settings
  #
  # @return [String] company name
  #
  def company_name
    Setting.find_by(name: 'company').try(:value) || '-'
  end

  # Return the Current company web stored in settings
  #
  # @return [String] company web
  #
  def company_web
    Setting.find_by(name: 'webpage_url').try(:value) || '-'
  end

  def email_sender
    Setting.find_by(name: 'email').try(:value) || RedBorder::CONFIG[:mailer_sender]
  end
end
