#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Helps the generation of the range/month selector
#
module RangeHelper
  #
  # Return the name of the month based on current date
  #
  # == Parameters:
  # time::
  #   Time to show
  #
  # == Returns:
  # String with the name
  #
  def month_name(time)
    time.strftime('%^B')
  end

  #
  # Show a link to current month if it's necessary
  #
  # == Parameters:
  # selected::
  #   Selected month
  # now::
  #   Time now
  #
  def current_month(month, now)
    return '' if month.year == now.year && (month.month - now.month).abs <= 1
    if month.year == now.year
      link_to "#{I18n.t('javascript.months.' + month_name(now).downcase[0..2])} #{t 'current'}", url_for(month: now),
              class: 'month-current'
    else
      link_to "#{I18n.t('javascript.months.' + month_name(now).downcase[0..2])} #{t 'of'} #{now.year} #{t 'current'}",
              url_for(month: now), class: 'month-current'
    end
  end
end
