#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module ReportBlocksHelper
  include Redborder::Rbdruid

  def render_report_block (report, block, data, user)
    colors = ["#4572a7", "#aa4643", "#89a54e", "#80699b", "#3d96ae", "#db843d", "#92a8cd", "#a47d7c", "#b5ca92"]

    case block.report_type
    when ReportBlock::TYPE_TITLE
      content_tag :div, class: "report_block_container span#{block.width}", :"data-type" => ReportBlock::TYPE_TITLE do
        content_tag :h1, block.title, class: "text_align_#{block.text_alignment}"
      end
    when ReportBlock::TYPE_TEXT
      content_tag :div, class: "report_block_container span#{block.width}", :"data-type" => ReportBlock::TYPE_TEXT do
        content_tag :p, block.text, class: "text_align_#{block.text_alignment}"
      end
    when ReportBlock::TYPE_SEPARATOR
      content_tag :div, class: "report_block_container span#{block.width}", :"data-type" => ReportBlock::TYPE_SEPARATOR do
        content_tag :hr, nil, style: "background-color: #{block.color}"
      end
    when ReportBlock::TYPE_IMAGE
      content_tag :div, class: "report_block_container span#{block.width}", :"data-type" => ReportBlock::TYPE_IMAGE do
        image_tag data
      end
    when ReportBlock::TYPE_CUSTOM
      if block.report_block_type == 'Aps'
        # APs
        content_tag :div, class: "report_block_container span#{block.width}" do
          html = ""
          if block.show_name == "1"
            html += content_tag(:div, content_tag(:h3, block.name), class: "report_block_title")
          end

          html += content_tag :div, class: "report_block_table_aps" do
            elements = ''
            data.first.each do |item|
              elements += content_tag :table do
                table = content_tag :thead do
                  "<tr class='header'><th colspan='4'>" + I18n.t('report_blocks_helper.ap_address') + " #{item.first} <span>#{item.last["wireless_station_name"]}</span></th></tr>".html_safe
                end

                table += content_tag :tbody do
                  rows = "<tr class='subheader'><td>" + I18n.t('report_blocks_helper.band') + "</td><td>" + I18n.t('report_blocks_helper.clients') + "</td><td>Bytes</td><td>" + I18n.t('report_blocks_helper.packets') + "</td></tr>"

                  item.last['protocols'].each do |protocol, metrics|
                    rows += "<tr><td><b>#{band_name(protocol)}</b></td><td>#{round_metric(metrics['clients'])}</td><td>#{round_metric(metrics['bytes'])}</td><td>#{round_metric(metrics['pkts'])}</td></tr>"
                  end

                  rows.html_safe
                end

                table.html_safe

              end
            end
            elements.html_safe
          end

          html.html_safe
        end
      else
        options = JSON.parse block.to_json

        # Force the data of a setted time
        # [ TODO ] Angel. We need to refactor this.
        if !@start_time.nil? && !@end_time.nil? &&
           @end_time > @start_time

           options.merge!('end_time' => @end_time.to_s,
                          'start_time' => @start_time.to_s,
                          'start_time_secs' => @start_time.to_i,
                          'end_time_secs' => @end_time.to_i,
                          'granularity' => @granularity,
                          'step' => Redborder::Rbdruid::GRANULARITY_TYPE[@granularity][:step])
        end

        options.merge!(width: width_for_blocks(block.width, report.orientation_name, block.border == '1'))

        content_tag :div, class: "report_block_container span#{block.width}" do
          html = ""
          if block.show_name == "1"
            html += content_tag(:div, content_tag(:h3, block.name), class: "report_block_title")
          end

          placeholder_class = "report_block_placeholder"
          placeholder_class += " report_block_chart" if ["stacked", "area", "line", "bars", "sbars", "verbars"].include?(block.graph_type)
          placeholder_class += " report_block_streams" if block.graph_type == "streams"

          html += content_tag(:div, nil, id: "widget_placeholder_#{block.id}", class: "report_block_placeholder #{placeholder_class} #{block.border == '1' ? 'border' : ''}")

          if (%w(stacked area line bars sbars).include?(block.graph_type) && !%w(Raw Cardinality).include?(block.report_block_type)) ||
              (block.report_block_type == 'Cardinality' && !%w(ball horbars verbars).include?(block.graph_type) && block.cardinality_group_by.present? && block.cardinality_group_by != 'none')

            html += content_tag(:div, nil, id: "widget_leyend_#{block.id}", class: "widget_leyend") do
              leyend = ""
              data["series"].each_with_index do |datum, index|
                leyend += content_tag(:div, nil, class: "leyend_element") do
                  element = ""
                  element += content_tag(:div, nil, class: "leyend_square", style: "background-color: #{colors[index % colors.size]}")
                  element += "<span class='leyend_name'>#{datum[0]["name"] || "N/A"}</span>"
                  element.html_safe
                end
              end

              leyend.html_safe

            end
          end

          html += javascript_tag("display_report_block('" + escape_javascript(options.to_json.html_safe) + "', '" + escape_javascript(data.to_json.html_safe) + "', " + escape_javascript(user.to_json.html_safe) +");")
          html.html_safe

        end
      end
    end
  end

  def band_name(band)
    band_name = ""

    case band
    when "DOT11N5"
      band_name = "802.11n (5GHz)"
    when "DOT11N24"
      band_name = "802.11n (2.4GHz)"
    when "DOT11A"
      band_name = "802.11a"
    when "DOT11B"
      band_name = "802.11b"
    when "DOT11G"
      band_name = "802.11g"
    else
      band_name = band
    end

    band_name
  end

  def round_metric(quantity)
    if quantity > 1e12
      quantity = (quantity / 1e12).round(2)
      quantity = "#{quantity} T"
    elsif quantity > 1e9
      quantity = (quantity / 1e9).round(2)
      quantity = "#{quantity} G"
    elsif quantity > 1e6
      quantity = (quantity / 1e6).round(2)
      quantity = "#{quantity} M"
    elsif quantity > 1e3
      quantity = (quantity / 1e3).round(2)
      quantity = "#{quantity} K"
    else
      quantity = "#{quantity}"
    end

    quantity
  end

	def report_block_sensor_table_type_collection
    list = []
    menu_items_for(:report_block_sensor_table_type_menu) do |node|
      list << [node.caption, node.name]
    end
    list
  end

  private

  def width_for_blocks(width, orientation, border)
    width_ret = 250

    if orientation == 'Portrait'
      width_ret = (800/4)*width
    else
      width_ret = (1000/4)*width
    end

    width_ret -= 10 if border

    width_ret
  end

end
