#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module ReportsHelper
  def report_block_get_category(category)
    if category == "cpu"
      category.upcase
    else
      category.capitalize
    end
  end

  #
  # Generate the options for move page select when delete a page
  #
  # == Parameters:
  # report::
  #   Report to delete the page
  # page::
  #   Integer of page to delete
  #
  # == Returns:
  # An Array with format: ['Page X', X].
  #
  def move_pages(report, page)
    options = []
    for i in 1..report.number_pages do
      next if page.to_i == i
      options << [ I18n.t('reports_helper.page') + " " + i.to_s , i]
    end
    # Return options
    options
  end

  #
  # Set the HTML code for the monthly days selector
  #
  def set_monthly_days_selector(report)
    # The final code var
    code = "<ul class='days-selector monthly #{ "hidden" if @report.schedule_type == "weekly"}'>";

    if @report.schedule_days.nil? or @report.schedule_type == "weekly"
      days = [];
    else
      days = @report.schedule_days.split(",").map(&:to_i)
    end

    days_html = Array(1..27).map{ |day| "<li data-day='#{day}' class='#{ "active" if days.include?(day)}'>#{day}</li>"}
    days_html.push("<li data-day='28' class='last #{ "active" if days.include?(28)}'>" + I18n.t('reports_helper.last_day') +"</li>")

    code = "#{code}#{days_html.join("")}</ul>"
  end

  #
  # Set the HTML code for the weekly days selector
  #
  def set_days_selector(report)
    # The final code var
    code = "<ul class='days-selector weekly  #{ "hidden" if @report.schedule_type == "monthly"}'>"

    if @report.schedule_days.nil? or @report.schedule_type == "monthly"
      days = [];
    else
      days = @report.schedule_days.split(",").map(&:to_i)
    end

    if days.include? 0
      code = "#{code}<li data-day='0' class='active'>#{t 'dates.sunday'}</li>"
    else
      code = "#{code}<li data-day='0'>#{t 'dates.sunday'}</li>"
    end

    if days.include? 1
      code = "#{code}<li data-day='1' class='active'>#{t 'dates.monday'}</li>"
    else
      code = "#{code}<li data-day='1'>#{t 'dates.monday'}</li>"
    end

    if days.include? 2
      code = "#{code}<li data-day='2' class='active'>#{t 'dates.tuesday'}</li>"
    else
      code = "#{code}<li data-day='2'>#{t 'dates.tuesday'}</li>"
    end

    if days.include? 3
      code = "#{code}<li data-day='3' class='active'>#{t 'dates.wednesday'}</li>"
    else
      code = "#{code}<li data-day='3'>#{t 'dates.wednesday'}</li>"
    end

    if days.include? 4
      code = "#{code}<li data-day='4' class='active'>#{t 'dates.thursday'}</li>"
    else
      code = "#{code}<li data-day='4'>#{t 'dates.thursday'}</li>"
    end

    if days.include? 5
      code = "#{code}<li data-day='5' class='active'>#{t 'dates.friday'}</li>"
    else
      code = "#{code}<li data-day='5'>#{t 'dates.friday'}</li>"
    end

    if days.include? 6
      code = "#{code}<li data-day='6' class='active'>#{t 'dates.saturday'}</li>"
    else
      code = "#{code}<li data-day='6'>#{t 'dates.saturday'}</li>"
    end
  end
end
