#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module SettingsHelper
  def geoip_input_file(geoip_type)
    html = content_tag(:input, '', value: "file:///usr/share/GeoIP/#{geoip_type}.dat", type: 'text', class: 'input-xxlarge form-control')
    
    md5 = setting_value("geoip_#{geoip_type}_md5") || 'N/A'
    html += content_tag(:span, "MD5: " + md5, id: "geoip_#{geoip_type}_md5")
    html.html_safe
  end

  #
  # Return the value or false of a settings. This method preload all settings
  #
  # == Parameters:
  # setting::
  #   Symbol with setting to check
  # settings::
  #   Global Hash variable including all settings
  # column::
  #   Boolean if the setting is a columns. By default, columns are active
  #
  # == Returns:
  # Value or false
  #
  def setting_value(setting, column = false)
    return '1' if column && @settings[setting].nil?
    @settings[setting]
  end

  def render_overview_widget_checkboxes(f, user = current_user)
    checkboxes_collection = []
  
    Redborder::Module.registered_modules.each do |_, prop|
      next unless prop[:overview_widgets].present? && prop[:path].present? && user.modules.include?(prop[:path].downcase)
  
      prop[:overview_widgets].each do |w|
        next unless w[:partial]
        setting_name = "overview_#{w[:partial]}"
        checkbox_content = f.input(setting_name, as: :boolean, label: "#{w[:partial]}".humanize, wrapper: :inline_checkbox, input_html: { checked: setting_value(setting_name.to_sym) == "1" })
        checkboxes_collection << content_tag(:span, checkbox_content, class: "col-md-2")
      end
    end
    checkboxes_collection.join.html_safe
  end  
end
