#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Time helper
#
module TimeHelper
  #
  # Format the time with this style: DD/MM/YY at <strong>HH:MM</strong>
  # If time == nil, this functions return the formated time now or the
  # options undefinedString
  #
  # == Parameters:
  # time::
  #   Time object to display
  # options::
  #   Hash. Available options:
  #     - bold: Don't display time at bold
  #     - year: Don't display the year
  #     - undefinedString: String to display when time is nil
  #
  # == Returns:
  # Safe String
  #
  def format_time(time, options = {})
    # Set the format
    format = options[:year].nil? ? '%Y/%m/%d at' : '%m/%d at'
    format = if options[:bold]
               "#{format} <strong>%H:%M</strong>"
             else
               "#{format} %H:%M"
             end
    format = "#{format} %:z" if options[:timezone]

    if time.nil? || time.blank?
      # Set the undefined string
      undefined_string = options[:undefinedString] ||
                         Time.zone.now.strftime(format)
      undefined_string.html_safe
    else
      time.strftime(format).html_safe
    end
  end
end
