#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Helper to build users view.
#
module UsersHelper
  #
  # Return the name of the createor of a model. If the model don't respond
  # to user o user is undefined it will return a default string.
  #
  # == Parameters:
  # model::
  #   Model to fetch the name
  #
  # == Returns:
  # String with username or 'Administrator'
  #
  def user_name(model)
    return 'Administrator' unless model.respond_to?(:user) && model.user
    model.user.is_a?(User) ? model.user.name : model.user
  end

  #
  # Return the top domain of the user. If the Flow module isn't enabled,
  # the element hasn't got any link.
  #
  # == Parameters:
  # user::
  #   User object to render hiw top domain
  #
  # == Returns:
  # Safe string with a link or the name
  #
  def user_top_domain(user)
    # If the user can't click on the path name, we musn't showe it as button
    return user.sensor.try(:path_name) unless
      Redborder::Module.registered_modules.keys.include?(:flow)

    link_to user.sensor.try(:path_name), view_sensor_path(user.sensor),
            class: 'btn btn-small'
  end

  #
  # Return the gravatar of the creator of given model. If there aren't
  # any user associated to the mode. It returns default avatar.
  #
  # == Parameters:
  # model::
  #   Model to fetch the user avatar
  # size::
  #   Integer with size of user. 16 by default.
  #
  # == Returns:
  # String with HTML code of the iamge
  #
  def user_gravatar(model, size = 16)
    email = if model.respond_to?(:user) && model.user.is_a?(User)
              model.user.email
            else
              ''
            end
    # Get Gravatar
    get_gravatar_image(email, size, 'avatar-small')
  end
end
