#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
#
# => Create an input color selector based on colors or custom.
#
class ColorInput < SimpleForm::Inputs::Base
  #
  # => Render the HTML + javascript code for the input
  #
  def input(wrapper_options = nil)
    "#{@builder.hidden_field(attribute_name, input_html_options)} \
    <div class='clearfix'>#{create_color_selector}</div>#{javascript}".html_safe
  end

  #
  # => Create the boxes to select the color
  #
  def create_color_selector
    colors = ''
    selected = options[:default_color] || options[:collection].first.last
    selected = selected.downcase
    options[:collection].each do |color|
      colors += "<div class='color-item #{'selected' if selected == color.last.downcase }' title='#{color.first}' \
                style='background-color: #{color.last}; width: #{(100.to_f/options[:collection].size).round(6)}%' \
                data-color='#{color.last}'></div>"
      first = false
    end
    # Return the colors
    colors
  end

  #
  # => Set the Javascript code needed to use the input
  #
  def javascript
    "<script> \
      $(document).on('click', '##{@builder.options[:html][:id]} .#{object_name}_#{attribute_name} .color-item', function(event){ \
        event.preventDefault(); \
        $('##{@builder.options[:html][:id]} ##{object_name}_#{attribute_name}').val($(this).attr('data-color')); \
        $('##{@builder.options[:html][:id]} .#{object_name}_#{attribute_name} .color-item.selected').removeClass('selected'); \
        $(this).addClass('selected'); \
      }) \
    </script>"
  end
end