#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class DelayedJob < ApplicationRecord
  @@pid_path = "#{Rails.root}/tmp/pids"
  @@pid_file_0 = "#{Rails.root}/tmp/pids/delayed_job.0.pid"
  @@pid_file_1 = "#{Rails.root}/tmp/pids/delayed_job.1.pid"

  def name
    raw = handler.split("\n")
    /Redborder::Jobs::(?<name>.+)$/.match(raw[0])[:name]
  end

  def to_partial_path
    'jobs/job'
  end

  def self.process
    process = []

    process << Redborder::Process.new(`ps -o ruser,pid,%cpu,%mem,vsize,rss,tt,stat,start_time,etime,command -p #{pid_0} |grep delayed_job | grep -v grep`.chomp.strip) if pid_0
    process << Redborder::Process.new(`ps -o ruser,pid,%cpu,%mem,vsize,rss,tt,stat,start_time,etime,command -p #{pid_1} |grep delayed_job | grep -v grep`.chomp.strip) if pid_1

    process
  end

  def self.pid_0
    File.open(@@pid_file_0).read.to_i if File.exist?(@@pid_file_0)
  end

  def self.pid_1
    File.open(@@pid_file_1).read.to_i if File.exist?(@@pid_file_1)
  end

  def self.running?
    return true if (File.exist?(@@pid_file_0) ||
                    File.exist?(@@pid_file_1)) && !process.empty?
    false
  end

  def self.start
    `#{Rails.root}/script/delayed_job start -n 2 --pid-dir #{@@pid_path} RAILS_ENV=#{Rails.env}`
  end

  def self.stop
    `#{Rails.root}/script/delayed_job stop --pid-dir #{@@pid_path} RAILS_ENV=#{Rails.env}`
  end

  def self.restart
    `#{Rails.root}/script/delayed_job restart -n 2 --pid-dir #{@@pid_path} RAILS_ENV=#{Rails.env}`
  end

  def self.rule_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RuleUpdatedbJob%').first
  end

  def self.model_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ModelUpdatedbJob%').first
  end

  def self.check_plugins
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckPluginsJob%').first
  end

  def self.plugin_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdatePluginJob%').first
  end

  def self.custom_rule_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CustomRuleUpdatedbJob%').first
  end

  def self.apply_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ApplyUpdateJob%').first
  end

  def self.alarm
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckAlarmsJob%').first
  end

  def self.incidents_outliers
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::OutlierIncidentJob%').first
  end

  def self.incidents
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateIncidentsJob%').first
  end

  def self.report
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckReportsJob%').first
  end

  def self.geoip_update
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::GeoipUpdateJob%').first
  end

  def self.rb_wakeup_chef_cluster
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RbWakeupChefClusterJob%').first
  end

  def self.glpi_inventory_job
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::GlpiInventoryJob%').first
  end

  def self.threat_intel_source_job
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateThreatIntelJob%').first
  end

  def self.incident_to_syslog
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::IncidentToSyslogJob%').first
  end

  def self.rb_vulnerabilities_update_db
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RbVulnerabilitiesUpdateDBJob%').first
  end

  def self.scan_profile
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ScanProfileJob%').first
  end

  def self.check_corrupt_data
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckCorruptDataJob%').first
  end

  def self.update_content_incidents
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateContentIncidentsJob%').first
  end

  def self.delete_stored_jobs
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::DeleteStoredJobs%').first
  end

  def self.rule_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RuleUpdatedbJob%').present?
  end

  def self.model_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ModelUpdatedbJob%').present?
  end

  def self.custom_rule_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CustomRuleUpdatedbJob%').present?
  end

  def self.apply_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ApplyUpdateJob%').present?
  end

  def self.incidents_outliers?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::OutlierIncidentJob%').present?
  end

  def self.alarm?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckAlarmsJob%').present?
  end

  def self.incidents?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateIncidentsJob%').present?
  end

  def self.report?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckReportsJob%').present?
  end

  def self.geoip_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::GeoipUpdateJob%').present?
  end

  def self.rb_wakeup_chef_cluster?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RbWakeupChefClusterJob%').present?
  end

  def self.glpi_inventory_job?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::GlpiInventoryJob%').present?
  end

  def self.threat_intel_source_job?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateThreatIntelJob%').present?
  end

  def self.incident_to_syslog?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::IncidentToSyslogJob%').present?
  end

  def self.rb_vulnerabilities_update_db?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RbVulnerabilitiesUpdateDBJob%').present?
  end

  def self.scan_profile?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ScanProfileJob%').present?
  end

  def self.restart_chef_client?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RestartChefClientJob%').present?
  end

  def self.update_rpms?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateRPMSJob%').present?
  end

  def self.reassign_partitions?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::ReassignPartitionsJob%').present?
  end

  def self.up_pcp?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpPCPJob%').present?
  end

  def self.recovery_pcp?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::RecoveryPCPJob%').present?
  end

  def self.deleting_sensors?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::DeleteSensorJob%').present?
  end

  def self.check_corrupt_data?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckCorruptDataJob%').present?
  end

  def self.plugin_update?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdatePluginJob%').present?
  end

  def self.check_plugins?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::CheckPluginsJob%').present?
  end

  def self.update_content_incidents?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::UpdateContentIncidentsJob%').present?
  end

  def self.delete_stored_jobs?
    where('handler like ?',
          '%!ruby/struct:Redborder::Jobs::DeleteStoredJobs%').present?
  end

  # Check if a job is queued and running
  def running?
    locked_at.present?
  end

  def self.force_rule_update
    if rule_update? && !rule_update.running?
      rule_update.destroy!
      RuleSource.where(is_custom: false, enable: true).update_all(updating: true)
      Delayed::Job.enqueue(Redborder::Jobs::RuleUpdatedbJob.new(User.current_user.id, true),
                          priority: 10, run_at: Time.now + 5.seconds)
    elsif !self.rule_update?
      RuleSource.where(is_custom: false, enable: true).update_all(updating: true)
      Delayed::Job.enqueue(Redborder::Jobs::RuleUpdatedbJob.new(User.current_user.id, true),
                          priority: 10, run_at: Time.now + 5.seconds)
    end
  end

  def self.force_model_update
    if model_update? && !model_update.running?
      model_update.destroy!
      ModelSource.where(enable: true).update_all(updating: true)
      Delayed::Job.enqueue(Redborder::Jobs::ModelUpdatedbJob.new(User.current_user.id, true),
                          priority: 10, run_at: Time.now + 5.seconds)
    elsif !self.model_update?
      ModelSource.where(enable: true).update_all(updating: true)
      Delayed::Job.enqueue(Redborder::Jobs::ModelUpdatedbJob.new(User.current_user.id, true),
                          priority: 10, run_at: Time.now + 5.seconds)
    end
  end

  def self.force_plugins_update(user_id,force=false)
    user_id = User.first.id unless user_id
    if check_plugins? && !check_plugins.running?
      check_plugins.destroy!
      Delayed::Job.enqueue(Redborder::Jobs::CheckPluginsJob.new(user_id, force),
                          priority: 5, run_at: Time.now + 2.seconds)
    elsif !self.check_plugins
      Delayed::Job.enqueue(Redborder::Jobs::CheckPluginsJob.new(user_id, force),
                          priority: 5, run_at: Time.now + 2.seconds)
    end
  end

  #TODO: def and variable sharing same name :(
  def user_id
    /user_id:\s*(?<user_id>\d+)\n/i.match(self.handler)[:user_id].to_i
  rescue NoMethodError
    nil
  end

  def plugin_id
    /plugin_id:\s*(?<plugin_id>\d+)\n/i.match(self.handler)[:plugin_id].to_i
  rescue NoMethodError
    nil
  end

  def enabled
    /enabled:\s*(?<enabled>\d+)\n/i.match(self.handler)[:enabled]
  rescue NoMethodError
    nil
  end

  def version
    /version:\s*(?<version>\d+)\n/i.match(self.handler)[:version]
  rescue NoMethodError
    nil
  end



  # def self.force_geoip_databases_update
  #   if self.geoip_update? and !self.geoip_update.running?
  #     self.geoip_update.destroy!
  #     Delayed::Job.enqueue(Redborder::Jobs::GeoipUpdateJob.new(true),
  #                          priority: 10, run_at: Time.now + 5.seconds)
  #   elsif !self.geoip_update?
  #     Delayed::Job.enqueue(Redborder::Jobs::GeoipUpdateJob.new(true),
  #                          priority: 10, run_at: Time.now + 5.seconds)
  #   end
  # end

  #
  # Enqueue default jobs
  #
  def self.run_now
    unless alarm?
      Delayed::Job.enqueue(Redborder::Jobs::CheckAlarmsJob.new(true),
                          priority: 10, run_at: Time.now + 2.seconds)
    end
    unless incidents?
      Delayed::Job.enqueue(Redborder::Jobs::UpdateIncidentsJob.new(true),
                          priority: 10, run_at: Time.now + 2.seconds)
    end
    unless report?
      Delayed::Job.enqueue(Redborder::Jobs::CheckReportsJob.new(true),
                          priority: 10, run_at: Time.now + 2.seconds)
    end
    unless check_corrupt_data?
      Delayed::Job.enqueue(Redborder::Jobs::CheckCorruptDataJob.new(true),
                          priority: 5, run_at: Time.now + 2.seconds)
    end
    unless incidents_outliers?
      Delayed::Job.enqueue(Redborder::Jobs::OutlierIncidentJob.new(true),
                          priority: 5, run_at: Time.now + 2.seconds)
    end
    unless check_plugins?
      begin
        user_id = User.first.id
      rescue
        user_id = nil
        Rails.logger.warn("No User was found to get its id when enqueuing CheckPluginsJob, default 0 value assigned")
      end
      Delayed::Job.enqueue(Redborder::Jobs::CheckPluginsJob.new(user_id, false),
                          priority: 5, run_at: Time.now + 2.seconds)
    end
    unless rb_vulnerabilities_update_db?
      begin
        user_id = User.first.id
      rescue
        user_id = nil
        Rails.logger.warn("FIRST TRY: No User was found to get its id when enqueuing RbVulnerabilitiesUpdateDBJob")
      end
      Delayed::Job.enqueue(Redborder::Jobs::RbVulnerabilitiesUpdateDBJob.new(user_id, true),
                          priority: 5, run_at: Time.now + 2.seconds)
    end
    unless threat_intel_source_job?
      begin
        user_id = User.first.id
      rescue
        user_id = nil
        Rails.logger.warn("FIRST TRY: No User was found to get its id when enqueuing Threat Intel Source Job")
      end
      Delayed::Job.enqueue(Redborder::Jobs::UpdateThreatIntelJob.new(user_id, true),
                          priority: 10, run_at: Time.now + 2.seconds)
    end
    unless update_content_incidents?
      begin
        user_id = User.first.id
      rescue
        user_id = nil
        Rails.logger.warn("No User was found to get its id when enqueuing UpdateContentIncidentsJob, default 0 value assigned")
      end
      Delayed::Job.enqueue(Redborder::Jobs::UpdateContentIncidentsJob.new(true, user_id),
                          priority: 10, run_at: Time.now + 2.seconds)
    end

    unless delete_stored_jobs?
      begin
        user_id = User.first.id
      rescue
        user_id = nil
        Rails.logger.warn("No User was found to get its id when enqueuing DeleteStoredJobs, default 0 value assigned")
      end
      Delayed::Job.enqueue(Redborder::Jobs::DeleteStoredJobs.new(user_id, true),
                          priority: 10, run_at: 1.day.from_now.beginning_of_day)
    end
  end

  def host
    return nil if locked_by.nil?
    locked_by.match(/host:(?<host>[^\s]+)/)[:host]
  end

  def pid
    return nil if locked_by.nil?
    locked_by.match(/pid:(?<pid>\d+)/)[:pid]
  end
end