#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class DruidRules < ApplicationRecord
  # ==========================================================
  # Custom attributes
  # ==========================================================

  establish_connection :druid
  self.table_name = 'druid_rules'

  #
  # => Returns an array of info about where segments
  #    should land.
  #    [
  #     {"period"=>"P5000Y", "replicants"=>"1",
  #      "tier"=>"_default_tier", "type"=>"loadByPeriod"},
  #     ...
  #    ]
  #
  def self.payload
    JSON.parse first.payload
  end

  #
  # => Returns the configuration of hot and default tiers.
  #
  def self.rules
    ret = {
      hot: {
        period: :none,
        replicants: 1
      },
      default: {
        period: :forever,
        replicants: 1
      }
    }

    payload.each do |rule|
      if rule['tier'] == '_default_tier'
        ret[:default][:period] = (rule['period'].to_sym == :P5000Y) ? :forever : rule['period'].to_sym unless rule['period'].nil?
        ret[:default][:replicants] = rule['replicants'].to_i unless rule['replicants'].nil?
      elsif rule['tier'] == 'hot'
        ret[:hot][:period] = rule['period'].to_sym unless rule['period'].nil?
        ret[:hot][:replicants] = rule['replicants'].to_i unless rule['replicants'].nil?
      end
    end

    ret
  end

  #
  # => Config the hot and default tiers.
  #    rules -> { hot: {
  #                 period: :none,
  #                 replicants: 1
  #               },
  #               default: {
  #                 period: :forever,
  #                 replicants: 1
  #               }
  #             }
  #
  def self.rules=(rules)
    if rules['hot']['period'] == 'none' && rules['default']['period'] == 'forever'
      payload = [
        { period: :P5000Y, replicants: rules['default']['replicants'],
          tier: :'_default_tier', type: :loadByPeriod }
      ]
    elsif rules['hot']['period'] != 'none' && rules['default']['period'] == 'forever'
      payload = [
        { period: rules['hot']['period'],
          replicants: rules['hot']['replicants'], tier: :hot,
          type: :loadByPeriod },
        {period: :P5000Y, replicants: rules['hot']['replicants'],
          tier: :'_default_tier', type: :loadByPeriod }
      ]
    elsif rules['hot']['period'] == 'none' && rules['default']['period'] != 'forever'
      payload = [
        { period: rules['default']['period'],
          replicants: rules['default']['replicants'],
          tier: :'_default_tier', type: :loadByPeriod },
        { period: :P5000Y, type: :dropByPeriod }
      ]
    else
      payload = [
        { period: rules['hot']['period'],
          replicants: rules['hot']['replicants'], tier: :hot,
          type: :loadByPeriod },
        { period: rules['default']['period'],
          replicants: rules['default']['replicants'],
          tier: :'_default_tier', type: :loadByPeriod },
        { period: :P5000Y, type: :dropByPeriod }
      ]
    end

    rule = first
    rule.payload = payload.to_json
    rule.save
  end
end
