#######################################################################
# Copyright (c) 2016 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder #:nodoc:
  module Malware
    class Email

      #
      # Check if the given email is valid
      #
      # == Parameters
      # value::
      #   String with value
      #
      # == Returns:
      # Boolean true or false
      def self.valid?(value)
        value.match(/\A[^@\s]+@([^@\s]+\.)+[^@\s]+\z/i) != nil
      end

      def self.type
        :email
      end

      def self.add_blacklist(_, value)
        role = init_role
        # Values should not get duplicated
        return false if role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['blacklist'].
                        any? { |x| x['email'] == value[:value] }
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['blacklist'] << {
          email: value[:value], description: value[:description],
          type: 'email', list_type: 'blacklist'
        }
        role.save
      end

      def self.remove_blacklist(_, value)
        role = init_role
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['blacklist'].
          delete_if { |e| e['email'] == value[:value] }
        role.save
      end

      def self.add_whitelist(_, value)
        role = init_role
        # Values should not get duplicated
        return false if role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['whitelist'].
                        any? { |x| x['email'] == value[:value] }
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['whitelist'] << {
          email: value[:value], description: value[:description],
          type: 'email', list_type: 'whitelist'
        }
        role.save
      end

      def self.remove_whitelist(_, value)
        role = init_role
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['whitelist'].
          delete_if { |e| e['email'] == value[:value] }
        role.save
      end

      def self.blacklist(_ = nil)
        role = init_role
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['blacklist']
      end

      def self.whitelist(_ = nil)
        role = init_role
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['whitelist']
      end

      def self.move_from_whitelist(_, tuple)
        return false unless valid?(tuple[:value])
        remove_whitelist(nil, tuple)
        add_blacklist(nil, tuple)
      end

      def self.move_from_blacklist(_, tuple)
        return false unless valid?(tuple[:value])
        remove_blacklist(nil, tuple)
        add_whitelist(nil, tuple)
      end

      private

      def self.init_role
        role = ::Chef::Role.load('gateway-sensor')
        role.override_attributes['redborder'] ||= {}
        role.override_attributes['redborder']['mailrelay'] ||= {}
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf'] ||= {}
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender'] ||= {}
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['blacklist'] ||= []
        role.override_attributes['redborder']['mailrelay']['mail_dissection_conf']['sender']['whitelist'] ||= []
        role
      end

    end
  end
end
