#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class IncidentLink < ApplicationRecord
  # ==========================================================
  # Relations
  # ==========================================================
  
  belongs_to :incident
  belongs_to :linked_incident, class_name: 'Incident'

  # ================================================================
  # Callbacks
  # ================================================================
  
  after_create :create_reverse_link

  # ================================================================
  # Validations
  # ================================================================
  
  validate :prevent_self_link

  # ================================================================
  # Class Methods
  # ================================================================

  def self.link_exists?(incident1, incident2)
    exists?(incident: incident1, linked_incident: incident2) ||
    exists?(incident: incident2, linked_incident: incident1)
  end

  # ================================================================
  # Private methods
  # ================================================================

  private

  # Ensure that an incident can't be linked to itself
  def prevent_self_link
    errors.add(:linked_incident, "cannot be the same as the incident") if incident_id == linked_incident_id
  end

  # Create the reverse link if it doesn't already exist
  def create_reverse_link
    unless IncidentLink.exists?(incident: linked_incident, linked_incident: incident)
      IncidentLink.create(incident: linked_incident, linked_incident: incident)
    end
  end
end

