#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class LlmGenerationRecords < ApplicationRecord
  # ==========================================================
  # Relations
  # ==========================================================
  belongs_to :record, polymorphic: true

  # ==========================================================
  # Validations
  # ==========================================================
  validate :table_exists
  validate :field_name_valid_for_table

  validates :table_name, :field_name, :record_id, presence: true
  validates :record_id, uniqueness: { scope: [:table_name, :field_name, :record_type],
                                      message: "LLM generation record already exists for this field and record."
                            }

  private

  # Check if the table exists
  def table_exists
    unless ActiveRecord::Base.connection.data_source_exists?(table_name)
      errors.add(:table_name, "does not exist")
    end
  end

  # Check if the field_name exists in the given table
  def field_name_valid_for_table
    if table_exists
      unless ActiveRecord::Base.connection.columns(table_name).map(&:name).include?(field_name)
        errors.add(:field_name, "is not valid for table #{table_name}")
      end
    end
  end
end
