#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class Phase < ApplicationRecord
  belongs_to :playbook
  has_many :tasks, dependent: :destroy

  validates :name, presence: true
  validates :playbook, presence: true

  def self.current
    current = find_by(current: true)
  end

  def next_phase
    playbook.phases.order(:id).where("id > ?", self.id).first
  end

  def previous_phase
    playbook.phases.order(:id).where("id < ?", self.id).last
  end

  def set_as_current
    playbook.phases.update_all(current: false)
    update(current: true)
  end

  def last_phase?
    next_phase.nil?
  end
end