#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class PlaybookAssignmentRulesCondition < ApplicationRecord
  belongs_to :playbook_assignment_rule

  validates :property, presence: true
  validates :sub_property, presence: true
  validates :operation, presence: true
  validates :value, presence: true

  def execute(sub_property_value)
    return false if sub_property_value.nil?

    operations = operators
    safe_method = operations[operation]

    raise StandardError, "Invalid operator: #{operation}" unless safe_method

    begin
      if property == 'Incidents'
        if sub_property_value.respond_to?(safe_method)
          if sub_property == 'priority_id'
            return sub_property_value.public_send(safe_method, value.to_i)
          else
            return sub_property_value.public_send(safe_method, value.to_s)
          end
        end

      elsif property == 'Mitre'
        if safe_method.is_a?(Proc)
          return safe_method.call(sub_property_value, sub_property.to_s, value.to_s)
        end
      end

      raise StandardError, "Invalid operation on #{sub_property_value} (#{sub_property_value.class})"
    rescue StandardError => e
      Rails.logger.error "Error executing Playbook Assignment Rule Condition: #{e.message}"
      raise
    end
  end

  def operators
    operations = {
      '=' => :==,
      '>' => :>,
      '<' => :<,
      '!=' => :!=
    }

    if property == 'Mitre'
      operations = {
        '=' => lambda do |sub_property_value, sub_property, value|
          sub_property_value.any? do |item|
            item.is_a?(Hash) && (item[sub_property] == value)
          end
        end,

        '!=' => lambda do |sub_property_value, sub_property, value|
          sub_property_value.none? do |item|
            item.is_a?(Hash) && (item[sub_property] == value)
          end
        end
      }
    end
    operations
  end
end
