#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class PlaybookTemplate < ApplicationRecord
  belongs_to :user
  has_many :phase_templates, dependent: :destroy

  validates :user, presence: true
  validate :playbooks_limit, on: :create

  before_destroy :check_read_only
  before_destroy :disable_assignment_rules

  def set_as_default
    PlaybookTemplate.transaction do
      # Desactivar el playbook predeterminado anterior
      PlaybookTemplate.where(default: true).update_all(default: false)

      # Establecer el actual como predeterminado
      self.update!(default: true)
    end
  end

  private

  def check_read_only
    if self.read_only?
      errors.add(:base, "Cannot delete a read-only playbook template")
      throw(:abort)
    end
  end

  def playbooks_limit
    if PlaybookTemplate.count >= 25
      errors.add(:base, 'Cannot create more than 25 playbooks')
      throw(:abort)
    end
  end

  def disable_assignment_rules
    PlaybookAssignmentRule.where(playbook_templates_id: self.id).update_all(enabled: false)
  end
end
