#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
require 'socket'

# https://datatracker.ietf.org/doc/html/rfc5424
class Rfc5424SyslogMessage
  attr_accessor :facility, :severity, :version,
                :hostname, :app_name, :pid, :msg_id,
                :structured_data, :msg

  def initialize
    @facility = 16 # local0 value
    @severity = 6 # Informational
    @version = '1'
    @timestamp = Time.now.utc.strftime('%Y-%m-%dT%H:%M:%S.%LZ')
    @hostname = Socket.gethostname
    @app_name = 'redborder-ndr'
    @pid = Process.pid
    @msg_id = '-'
    @structured_data = []
    @msg = ''
  end

  def priority
    (@facility * 8) + @severity
  end

  def add_structured_data(structured_data)
    @structured_data.push(structured_data)
  end

  def sd
    @structured_data.count > 0 ? @structured_data.join : '-'
  end

  def to_s
    message = "<#{priority}>#{@version} #{@timestamp} #{@hostname} #{@app_name} #{@pid} #{@msg_id} #{sd}"
    message += " #{@msg}" if @msg && !@msg.empty?
    message
  end

end