#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class TaskTemplate < ApplicationRecord
  belongs_to :phase_template
  acts_as_list scope: :phase_template

  validate :tasks_per_phase_limit, on: :create

  before_destroy :check_read_only

  private

  def check_read_only
    if self.phase_template.playbook_template.read_only?
      errors.add(:base, "Cannot delete a task template from read-only playbook template")
      throw(:abort)
    end
  end

  def tasks_per_phase_limit
    if TaskTemplate.where(phase_template_id: self.phase_template).count >= 12
      errors.add(:base, 'Cannot create more than 12 tasks per phase')
      throw(:abort)
    end
  end
end
