#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ThreatIntelSource < ApplicationRecord

  validates :name, presence: { message: "#{I18n.t('controllers.name_required')}" }

  validates :host, presence: { message: "#{I18n.t('controllers.host_required')}" }
  validates :port, presence: { message: "#{I18n.t('controllers.port_required')}" }, numericality: { greater_than_or_equal_to: 0, less_than_or_equal_to: 65535}
  validates :path, presence: { message: "#{I18n.t('controllers.port_required')}" }

  validates :threat_conn_types_id, presence: { message: "#{I18n.t('controllers.conn_type_required')}" }
  validates :threat_src_type, presence: { message: "#{I18n.t('controllers.src_type_required')}" }

  attr_accessor :url
  validate :validate_url_format

  def validate_url_format
    if is_http && !url.blank?
      begin
        uri = URI.parse(url)
        unless uri.is_a?(URI::HTTP) || uri.is_a?(URI::HTTPS)
          errors.add(:url, "#{I18n.t('controllers.url_not_http')}")
        end
      rescue URI::InvalidURIError
        errors.add(:url, "#{I18n.t('controllers.url_not_found')}")
      end
    end
  end

  def full_host
    host
  end

  def is_http
    ThreatConnType.find_by_name('HTTP').id == threat_conn_types_id || ThreatConnType.find_by_name('HTTPS').id == threat_conn_types_id
  end
  

end