#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Image widgets
#
class WidgetGraph::Image < WidgetGraph::Base
  ALLOWED_TYPES = %w(image/jpg image/jpeg image/png).freeze

  widget_fields [:keep_aspect_ratio]

  widget_eval do
    storage_file :media, path: 'rb-webui/:class/:id'

    before_save :set_image_dimensions

    validates_attachment :media, content_type: { content_type: ALLOWED_TYPES },
                                  size: { in: 0..10.megabytes }

    #
    # Set the image dimensions when we don't keep the aspect ratio or the
    # widget is from report or use a column layout.
    #
    def set_image_dimensions
      return if keep_aspect_ratio != '1' || media.queued_for_write.empty? ||
                parent_type == 'Report' || parent.layout == 'columns'

      img = media.queued_for_write[:original]
      dimensions = Dimensions.dimensions(img.path)
      self.position_width = "#{dimensions[0]}px"
      self.position_height = "#{dimensions[1]}px"
    end
  end

  widget_after_initialize do
    self.keep_aspect_ratio  ||= true
  end
end
