#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Lines, Areas, Stacked Areas, Bars & Stacked Bars
#
class WidgetGraph::Serie < WidgetGraph::Base
  # ================================================================
  # Widget Properties
  # ================================================================

  SERIE_TYPES = {
    area: 'Area',
    line: "Line",
    bars: "Bars",
    sbars: "Stacked Bars",
    stacked: "Stacked Area"
  }

  display_name 'Serie'
  image 'serie.png'
  data [:timeserie]
  order 1

  widget_fields [:granularity, :color, :serie_type, :timezone, :reference, :offset,
                 :bottom_reference, :top_reference, :discrete_y_axis]
  widget_array_fields [:analytics]

  widget_validates :timezone, allow_blank: true, inclusion: {
    in: ['auto'] + ActiveSupport::TimeZone.all.map(&:name)
  }

  # TODO: validate aggregations and analytics

  widget_after_initialize do
    self.timezone   ||= 'auto'
    self.serie_type ||= 'line'
    self.analytics  ||= []

    self.discrete_y_axis = "0"
    if self.aggregation && self.product_type
      aggs = Redborder::Module.registered_modules[self.product_type.to_sym][:aggregations] rescue nil
      if aggs
        agg  = aggs[self.aggregation.to_sym]
        self.discrete_y_axis = "1" if agg && agg[:round] == true
      end
    end
  end

  def as_json(opts = {})
    obj_opts = { methods: [:step, :tzoffset] }
    super(obj_opts.merge(opts))
  end

  def tzoffset
    return 0 if @widget.timezone.nil?

    if @widget.timezone.to_sym == :auto
      Time.zone.now.utc_offset * 1000
    else
      ActiveSupport::TimeZone[@widget.timezone].now.utc_offset * 1000
    end
  end

  def step
    if !@widget.granularity.nil?
      GRANULARITY_TYPE[@widget.granularity.to_sym][:step] rescue 60_000
    else
      60_000
    end
  end
end
