#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::Combination < WidgetType::Base
  graphs [:combination_ball, :combination_box, :combination]
  widget_fields [:module, :aggregation, :formula, :weight, :offset, :reference]

  widget_validates :reference, numericality: true, allow_blank: true
  widget_validates :offset, numericality: true, allow_blank: true

  def start_time
    range = @widget.range || 'last_1_hour'
    time_range = TIME_RANGES.fetch(range.to_sym) { 1.hour }
    result = end_time - time_range
    result = result.beginning_of_hour if time_range >= 12.hours
    round_time result
  end

  def end_time
    round_time(Timecop.travel(now) { Time.now })
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeserie(user = @widget.parent.owner, options = {})
    series = list()
    timeseries = {}
    series[:list].each do |n, serie|
      source = Redborder::DruidSource.get(source: serie['mod'])
      timeseries["#{serie['mod']}_#{serie['agg']}"] = source.timeserie({
        aggregation: serie['agg'],
        granularity: @widget.property[:granularity],
        user: user,
        range: @widget.property[:range],
        start_time: options[:start_time] || start_time,
        end_time: options[:end_time] || end_time,
        include_all_namespaces: options[:include_all_namespaces] || false,
        filters: @widget.fetch_filters
      })
    end
    { timeserie: timeseries }
  end

  def list(user = @widget.parent.owner, options = {})
    { list: JSON.parse(@widget.property[:series]) }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def total(user = @widget.parent.owner, options = {})
    series = list()
    totals = {}
    series[:list].each do |n, serie|
      source = Redborder::DruidSource.get(source: serie['mod'])
      totals["#{serie['mod']}_#{serie['agg']}"] = source.timeserie({
        aggregation: serie['agg'],
        granularity: :all,
        user: user,
        range: @widget.property[:range],
        start_time: options[:start_time] || start_time,
        end_time: options[:end_time] || end_time,
        include_all_namespaces: options[:include_all_namespaces] || false,
        filters: @widget.fetch_filters
      })
    end
    { total: totals }
  end
end