#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::GroupedUnique < WidgetType::Base
  graphs [:multiserie, :pie, :horizontal_bar, :vertical_bar, :table]

  widget_fields [:range, :column, :group_by, :elements]

  widget_after_initialize do
    self.group_by = 'none' if group_by.to_s.empty?
    self.elements = 5 if elements.to_s.empty?
  end

  # Validate the number of elements
  widget_validate :grouped_unique_elements_num do
    self.elements = 2 if elements.to_i < 2
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def list(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.where(
      query_type: :group_by_cardinality,
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      cardinality_column: @widget.column,
      group_by: source::COLUMNS[@widget.group_by.to_sym][:columns].keys,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_start_time,
      sort: [:result, source::COLUMNS[@widget.group_by.to_sym][:columns].keys.first],
      granularity: :all,
      limit: elements,
      search: @widget.fetch_filters
    )

    {
      list: serie.map { |s| {
                              result: s.result,
                              @widget.group_by.to_sym => s.send(@widget.group_by),
                              secondary_name: (source::COLUMNS[@widget.group_by.to_sym][:complementary_columns] and @widget.group_by.to_sym != :sensor_name) ?
                                                      s.send(source::COLUMNS[@widget.group_by.to_sym][:complementary_columns].keys.first.to_s) : s.send(@widget.group_by)
                            }
                      }
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def total_timeserie(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.timeserie(
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: ovrr_granularity,
      aggregation: @widget.aggregation
    )

    { total_timeserie: serie }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeseries(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)
    elements = list(user, options)
    # Init timeseries
    timeseries = {}
    search = @widget.fetch_filters

    elements[:list].each do |e|
      value = e[@widget.group_by.to_sym]
      search[@widget.group_by.to_sym] = { value => { operator: :is } }

      serie = source.where(
        user: user,
        include_all_namespaces: options[:include_all_namespaces] || false,
        query_type: :cardinality,
        cardinality_column: @widget.column,
        range: @widget.range,
        start_time: ovrr_start_time,
        end_time: ovrr_end_time,
        granularity: ovrr_granularity,
        search: search.dup
      ).map do |e|
        {
          event: { events: e.result },
          timestamp: e.timestamp.to_s
        }
      end

      timeseries.merge!(value => serie)
    end

    {
      timeseries: timeseries
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def total(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.where(
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      query_type: :cardinality,
      cardinality_column: @widget.column,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: :all,
      search: @widget.fetch_filters
    ).map do |e|
      {
        event: { events: e.result },
        timestamp: e.timestamp.to_s
      }
    end

    {
      total: serie
    }
  end

  #
  # Fetch the color of severities. These no needs permissions.
  #
  # == Returns:
  # An array with severities
  #
  def colors
    {
      colors: Severity.all
    }
  end
end
