#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::Map < WidgetType::Base
  MAP_COLUMNS = [:src_map, :dst_map, :coordinates_map, :public_ip_map]

  graphs [:map]

  widget_fields [:range, :column, :aggregation]

  widget_validates :column, inclusion: { in: MAP_COLUMNS }

  widget_after_initialize do
    self.click_action = 'none'
    self.column ||= product_type.to_sym == :flow ? :public_ip_map : :src_map
    self.range ||= :last_1_hour
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def list(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    data = source.where(
      query_type: :group_by,
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: :all,
      sort: @widget.aggregation,
      group_by: source::COLUMNS[@widget.column.to_sym][:columns].keys,
      aggregation: @widget.aggregation,
      search: @widget.fetch_filters
    )

    # Fetch GeoIP info
    {
      list:
        Redborder::Geoip.latlong(
          data.collect(&source::COLUMNS[@widget.column.to_sym][:ips])
        )
    }
  end
end
