#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::Raw < WidgetType::Base
  graphs [:serie, :stream, :ball, :box, :value, :table_raw]
  widget_fields [:range, :aggregation, :client_profile]

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeserie(user = @widget.parent.owner, options = {})
    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    # [ TODO ] Angel. Add filters
    {
      timeserie:
        source.timeserie(
          range: @widget.range,
          user: user,
          include_all_namespaces: options[:include_all_namespaces] || false,
          start_time: options[:start_time] || start_time,
          end_time: options[:end_time] || end_time,
          granularity: options[:granularity] || @widget.granularity,
          aggregation: @widget.aggregation,
          search: @widget.fetch_filters
        )
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def raw_table(user = @widget.parent.owner, options = {})
    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity || :all

    {
      raw_table:
        source.where(
          query_type: :group_by,
          range: start_time ? :custom : @widget.range,
          user: user,
          include_all_namespaces: options[:include_all_namespaces] || false,
          start_time: ovrr_start_time,
          end_time: ovrr_end_time,
          granularity: ovrr_granularity,
          sort: source.conditions_sort(@widget.aggregation.to_sym),
          group_by: @widget.stream_columns,
          aggregation: @widget.aggregation,
          search: @widget.fetch_filters
        )
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def total(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    total = source.timeserie(
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: :all,
      aggregation: @widget.aggregation,
      search: @widget.fetch_filters
    )

    # [ TODO ] Angel. Add filters
    {
      total: total
    }
  end
end
