#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::SingleUnique < WidgetType::Base
  graphs [:serie, :ball, :value]

  widget_fields [:range, :column]

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeserie(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity
    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.where(
      query_type: :cardinality,
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      cardinality_column: @widget.column,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: ovrr_granularity,
      search: @widget.fetch_filters
    ).map do |e|
      {
        event: { events: e.result },
        timestamp: e.timestamp.to_s
      }
    end

    {
      timeserie: serie
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def total(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.where(
      query_type: :cardinality,
      user: user,
      include_all_namespaces: options[:include_all_namespaces] || false,
      cardinality_column: @widget.column,
      range: @widget.range,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: :all,
      search: @widget.fetch_filters
    ).map do |e|
      {
        event: { events: e.result },
        timestamp: e.timestamp.to_s
      }
    end

    {
      total: serie
    }
  end
end
