#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class IntegrationService
  AVAILABLE_INTEGRATIONS = Integrations::BaseStrategy.subclasses
  AVAILABLE_COMMON_INTEGRATIONS = Integrations::CommonStrategy.subclasses

  def initialize(integration_name)
    strategy_class = self.class.strategy_class_for(integration_name)
    @strategy = strategy_class.new
  end

  def execute(action, params)
    @strategy.execute(action, params)
  end

  def actions
    @strategy.actions
  end

  def self.get_integrations
    AVAILABLE_INTEGRATIONS.map(&:strategy_name)
  end

  def self.get_common_integrations
    AVAILABLE_COMMON_INTEGRATIONS.map(&:strategy_name)
  end

  def self.get_actions_by_integration(integration_name)
    strategy_class = strategy_class_for(integration_name)
    actions = strategy_class.new.actions
    actions.each_with_object({}) do |action, hash|
      hash[action] = strategy_class.action_params(action)
    end
  end

  def self.all_actions
    AVAILABLE_INTEGRATIONS.flat_map do |strategy_class|
      strategy_class.new.actions
    end.uniq
  end

  private

  def self.strategy_class_for(integration_name)
    available_strategy = AVAILABLE_INTEGRATIONS.find { |strategy| strategy.strategy_name == integration_name }
    raise "Unknown integration strategy: #{integration_name}" unless available_strategy

    available_strategy
  end
end
