#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
module Integrations
  class CommonStrategy
    def self.strategy_name
      const_get(:NAME)
    end
  end

  class ZabbixStrategy < CommonStrategy
    NAME = "zabbix"
  
    def saved_params
      {
        zabbix_url: fetch_integration_value("zabbix_url"),
        zabbix_user: fetch_integration_value("zabbix_user"),
        zabbix_password: fetch_integration_value("zabbix_password"),
        zabbix_token: fetch_integration_value("zabbix_token"),
        zabbix_update_periodicity: fetch_integration_value("zabbix_update_periodicity"),
        zabbix_update_time: fetch_integration_value("zabbix_update_time")
      }
    end
    private
  
    def fetch_integration_value(name)
      integration = Integration.find_by(name: name)
      integration ? integration.value : nil
    end
  end

  class GlpiStrategy < CommonStrategy
    NAME = "glpi"
  
    def saved_params
      {
        glpi_api_token: fetch_integration_value("glpi_api_token"),
        glpi_api_url: fetch_integration_value("glpi_api_url"),
        glpi_app_token: fetch_integration_value("glpi_app_token"),
        glpi_update_periodicity: fetch_integration_value("glpi_update_periodicity"),
        glpi_update_time: fetch_integration_value("glpi_update_time")
      }
    end
  
    private
  
    def fetch_integration_value(name)
      integration = Integration.find_by(name: name)
      integration ? integration.value : nil
    end
  end
  
  class SyslogStrategy < CommonStrategy
    NAME = "incidents_to_syslog"

    def saved_params
      {
        syslog_server: fetch_integration_value("syslog_server"),
        syslog_port: fetch_integration_value("syslog_port"),
        syslog_protocol: fetch_integration_value("syslog_protocol")
      }
    end

    private
    
    def fetch_integration_value(name)
      integration = Integration.find_by(name: name)
      integration ? integration.value : nil
    end
  end
end
