#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class UpdateApplicationDatabag

  def self.map_value(value)
    x, y = value.split(':').map(&:to_i)
    (x << 24) + y
  end

  #
  # Updates the applications databag on Chef.
  #
  # == Parameters:
  # items::
  #   A hash with the referenced list of applications to update.
  # remove::
  #   A boolean to add/update or remove the items from Chef.
  #
  def self.call(items, remove=false)
    backup_app = Chef::DataBagItem.load('rBglobal', 'applications')

    backup_app['id'] = 'applications'
    remove ? backup_app['list'].except!(*items.keys) : backup_app['list'].merge!(items)
    backup_app.save
  rescue Errno::ECONNREFUSED, ArgumentError
    Rails.logger.error "\nErrno::ECONNREFUSED - when access databag: cannot contact erchef server."
  rescue Net::HTTPServerException
    Rails.logger.info "Databag rBglobal applications doesn't exist. Creating..."
    backup_app = Chef::DataBagItem.new
    backup_app.data_bag 'rBglobal'
    backup_app['id'] = 'applications'
    # We add the one that are by default added in the data base
    items = {}
    ApplicationObject.all.each do |application|
      items[UpdateApplicationDatabag.map_value(application.value).to_s] = { name: application.name,
                                                  value: application.value,
                                                  value_int: UpdateApplicationDatabag.map_value(application.value) }
    end
    backup_app['list'] = items unless remove
    if backup_app.save
      Rails.logger.info 'Databag rBglobal applications created.'
    else
      Rails.logger.error "\nNet::HTTPServerException - when access databag, cannot contact erchef server."
      Rails.logger.info 'Removing databag rBglobal applications.'
    end
  end
end
