#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class UpdateObjectDatabag
  #
  # Updates the applications databag on Chef.
  #
  # == Parameters:
  # items::
  #   A hash with the referenced list of objects to update.
  # remove::
  #   A boolean to add/update or remove the items from Chef.
  #
  def self.call(items, object_class, databag_name, remove=false)
    return unless databag_name
    backup_obj = Chef::DataBagItem.load('rBglobal', databag_name)

    backup_obj['id'] = databag_name
    remove ? backup_obj['list'].except!(*items.keys) : backup_obj['list'].merge!(items)
    backup_obj.save
  rescue Errno::ECONNREFUSED, ArgumentError
    Rails.logger.error "\nErrno::ECONNREFUSED - when access databag: cannot contact erchef server."
  rescue Net::HTTPServerException
    Rails.logger.info "Databag rBglobal objects doesn't exist. Creating..."
    backup_obj = Chef::DataBagItem.new
    backup_obj.data_bag 'rBglobal'
    backup_obj['id'] = databag_name
    # We add the one that are by default added in the data base
    items = {}
    object_class.all.each do |object|
       items[object.name.to_s] = { name: object.name,
                                   value: object.value,
                                  }
    end
    backup_obj['list'] = items unless remove
    if backup_obj.save
      Rails.logger.info 'Databag rBglobal objects created.'
    else
      Rails.logger.error "\nNet::HTTPServerException - when access databag, cannot contact erchef server."
      Rails.logger.info 'Removing databag rBglobal objects.'
    end
  end
end
