#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

require 'rubygems'
gem 'countries'
require 'countries'

class Continent
	Data = YAML.load_file(File.join(File.dirname(__FILE__), 'data', 'continents.yml')) || {}
	Names = Data.map{|k,v| [v['name'],k]}.sort_by{|d| d[0]}
	NameIndex = Hash[*Names.flatten] 

	define_method :name do
		@data['name']
	end

	define_method :countries do
		@data['countries'].map{|c| Country.new(c)}
	end

  attr_reader :data

	def initialize(continent_data)
		@data = continent_data.is_a?(Hash) ? continent_data : Data[continent_data.to_s.upcase]
	end

	def valid?
		not(@data.nil? or @data.empty?)
	end

	class << self
		
		def new(continent_data)
			if continent_data.is_a?(Hash) || Data.keys.include?(continent_data.to_s.upcase)
				super
			end
		end

		def all(&blk)
			blk ||= Proc.new {|continent, data| [data['name'], continent]}
			Data.map &blk
		end

		alias :continents :all

		def search(query)
			continent = self.new(query.to_s.upcase)
			(continent && continent.valid?) ? continent : nil
		end

		def [](query)
			self.search(query)
		end

	end

end