/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

 /* ========================================================================
 *  SEARCH AP
 * ======================================================================== */

var SearchAPState;

SearchAPState = function() {

  function SearchAPState(selectData, callback) {
    var self = this;
    self.html = Handlebars.templates['search-rule']({});
    self.columns = selectData.ap_state_columns;

    //Translating from init.rb
    for(var i = 0; i < self.columns.length; i++) {
      if (self.columns[i].value.startsWith("tr")) {
        self.columns[i].value = I18n.t(self.columns[i].value.slice(3))
      }
    }

    self.operators = selectData.operators;

    self.selectData = selectData;
    self.organizations = selectData.organizations;

    self.init();
    return self;
  };

  SearchAPState.prototype = {

    init: function(callback) {
      var self = this;
      var width = "100%";

      self.columns_html = Handlebars.templates['select']({
        name: "column-select",
        placeholder: I18n.t('javascript.choose_query'),
        width: width,
        data: {
          value: self.columns
        }
      });

      self.operators_html = function($html, data) {
        var select = Handlebars.templates['select']({
          name: "operators-select",
          width: width,
          data: {
            value: data
          }
        });

        self.organizations_html = Handlebars.templates['select']({
          name: "organizations-select",
          width: width,
          multiple: false,
          data: self.organizations
        });

        $html.find('div.operator-select').html(select);
      };

      self.datetime_picker = function(that) {
        that.datetimepicker({
          timeFormat: 'HH:mm',
          dateFormat: 'yy-mm-dd',
          numberOfMonths: 1,
          separator: ' '
        });
      };

    },

    searchUI: function(data, callback) {
      var self = this;
      $('#content #title').after(Handlebars.templates['search'](data));

      $('.search-content-remove').on('click', function(e) {
        e.preventDefault();
        self.remove(this);
      });

      $('div.search-content-enable input').on('click', function() {
        if ($(this).is(':checked')) {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', false).css('opacity', 1);
        } else {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', true).css('opacity', 0.8);
        };

        $('.add_chosen').trigger("chosen:updated");
      });

      $('button.submit-search').on('click', function(e) {
        e.preventDefault();
        self.submit();
      });

      $('#content #title').on('click', 'a.reset-search-form', function(e) {
        e.preventDefault();

        $('.rules').empty();

        self.add();
        self.add();
        self.add();
      });

      if (callback && (typeof callback === "function")) {
        callback();
      };
    },

    add: function(that, newOptions, callback) {
      var self = this;
      var options = {};

      if (options && (typeof options === "object")) {
        options = newOptions;
      };

      var $html = $(self.html);
      $html.find('div.column-select').html(self.columns_html);
      self.operators_html($html, self.operators.text_input);

      if (that && (typeof that === "object")) {
        $(that).parents('.search-content-box').after($html);
      } else if (that && (typeof that === "string")) {
        $(that).append($html);
      } else {
        $('.rules').append($html);
      };

      $html.find('.add_chosen')
      .chosen({allow_single_deselect: false, width: '100%'})
      .change(function(event, data) {
        var value = $(this).val();
        var that = $(this).parents('.search-content-box');

        if (value === "start_time") {
          that.find('.value').html('<input id="time-'+ new Date().getTime() +'" class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text">');
          self.datetime_picker(that.find('.value input:first'))
          self.operators_html($html, self.operators.datetime);
        } else if (value === "end_time") {
          that.find('.value').html('<input id="time-'+ new Date().getTime() +'" class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text">');
          self.datetime_picker(that.find('.value input:first'))
          self.operators_html($html, self.operators.datetime);
        } else if (value === "organization") {
          that.find('.value').html(self.organizations_html);
          self.operators_html($html, self.operators.select);
        } else {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.text_input);
        };

        that.find('.add_chosen').chosen({
          allow_single_deselect: false,
          width: "100%"
        });

        if (callback && (typeof callback === "function")) {
          callback(that);
        };

        that.trigger('rbsearch:search:change');
      });

      $('.search-content-box:first')
      .find('.search-content-remove')
      .css('opacity', 1)
      .unbind('hover')
      .unbind('mouseover');

      return $html;
    },

    remove: function(that) {
      var self = this;
      if ($('.search-content-box').length > 2) {
        $(that).parents('.search-content-box').remove();
      } else if ($('.search-content-box').length == 2) {
        $(that).parents('.search-content-box').remove();

        $('.search-content-box:first')
        .find('.search-content-remove')
        .css('opacity', 0.4)
        .attr('title', I18n.t('javascript.you_must'));
      };
    },

    pack: function(that) {
      var self = this;
      var matchAll = false;

      if ($('#search select.global-match-setting').val() === "all") {
        matchAll = true;
      };

      var json = {
        match_all: matchAll,
        items: {}
      };

      var fields = '.search-content-box'
      if (that && (typeof that === "string")) {
        fields = that;
      }

      $(fields).each(function(index, item) {
        var enabled = $(item).find('.search-content-enable input').is(':checked');
        var column = $(item).find('.column-select select').val();
        var operator = $(item).find('.operator-select select').val();
        var value = $(item).find('.value input, .value select').val();

        if (enabled){
          if (column == "start_time" || column == "end_time"){
            if ($(item).find('.value input').datetimepicker("getDate") == null) {
              alert(I18n.t('javascript.valid_date'));
              throw I18n.t('javascript.valid_date')
            }
            else{
              value = $(item).find('.value input').datetimepicker("getDate").getTime() / 1000;
              //value = new Date(value).getTime() / 1000; it doesn't work in Safari
            }
          }

          if ((column !== "") || (value !== "")) {
            json.items[index] = {
              column: column,
              operator: operator,
              value: value,
              enabled: enabled
            };
          };
        }
      });

      return json;
    },

    // submit: function(callback, otherOptions) {
    //   var self = this;
    //   var search = self.pack();
    //   var csrf = $('meta[name="csrf-token"]').attr('content');
    //
    //   if (typeof otherOptions !== "object") {
    //     otherOptions = {};
    //   };
    //
    //   var update_url = function() {
    //     if (history && history.pushState) {
    //       history.pushState(null, document.title, '/filters/flow/advanced');
    //     };
    //   };
    //
    //   if (callback && (typeof callback === "function")) {
    //     callback(search, self);
    //   };
    //
    //   if (search && !$.isEmptyObject(search.items)) {
    //     if (otherOptions.search_id) {
    //       post_to_url('/filters/flow/advanced', {
    //         match_all: search.match_all,
    //         search: search.items,
    //         title: otherOptions.title,
    //         search_id: ""+otherOptions.search_id+"",
    //         authenticity_token: csrf
    //       });
    //     } else {
    //       post_to_url('/filters/flow/advanced', {
    //         match_all: search.match_all,
    //         search: search.items,
    //         authenticity_token: csrf
    //       });
    //     };
    //   } else {
    //     flash_message.push({type: 'error', message: "You cannot submit all empty search rules."});flash();
    //   };
    // },

    build: function(search, where) {
      var self = this;

      if (typeof search === "string") {
        var data = JSON.parse(search);
      } else if (typeof search === "object") {
        var data = search;
      } else {
        var data = {};
      };

      if (data.items) {
       var rules = data.items;
      } else if (data.search) {
        var rules = data.search;
      } else {
        var rules = {};
      };

      if (data.match_all && data.match_all === "false") {
       $('select.global-match-setting option[value="any"]').prop('selected', 'selected');
      } else {
        $('select.global-match-setting option[value="all"]').prop('selected', 'selected');
      };

      for (id in rules) {
        var item = rules[id];

        var rule = (where !== undefined ? self.add(where) : self.add(false, {}));

        var column = rule.find('div.column-select select');

        rule.bind('rbsearch:search:change', function() {
          var operator = $(this).find('div.operator-select select');
          var value = $(this).find('div.value select, div.value input');

          operator.find("option[value='"+item.operator+"']").prop('selected', 'selected');
          operator.trigger("chosen:updated");

          if (item.column === "start_time" || item.column === "end_time") {
            if ($.isNumeric(item.value)) {
              date = new Date(item.value*1000);
            } else {
              date = new Date(Date.parse(item.value, "yyyy-MM-dd HH:mm"))
            }
            date_string = $.datepicker.formatDate( "yy-mm-dd", date);
            date_hours = date.getHours();
            date_minutes = date.getMinutes();

            if (date_hours < 10) { date_hours = "0" + date_hours; }
            if (date_minutes < 10) {date_minutes = "0" + date_minutes; }

            value.prop('value', date_string + " " + date_hours + ":" + date_minutes);
          } else {
            if (operator.val() == "empty" || operator.val() == "not_empty") {
              $(this).find('.value').html('<input class="search-content-value" placeholder="" name="" type="text" disabled x-webkit-speech lang="en-US">');
            }else {
              value.prop('value', item.value);
            }

          }

          rule.unbind('rbsearch:search:change');
        });

        rule.attr('data-rule-id', id);

        column.find("option[value='"+item.column+"']").prop('selected','selected');
        column.trigger("chosen:updated").trigger('change');

        if (item.enabled === "false") {
          rule.find('div.search-content-enable input').prop('checked', false);
          rule.find('.value *, .operator-select *, .column-select *')
          .attr('disabled', true).css('opacity', 0.8);
        };
      };

      $('.rules .add_chosen').trigger("chosen:updated");
    },

  };

  return SearchAPState;
}();
