#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading AP State module'

Redborder::Module.register :ap_state do

  name 'Access Point'
  path 'ap_state'
  session_id 'ap_state'
  data_source 'ApState'
  customizable_tabs true
  selectable_widget true

  columns({
    sensor_uuid: {
      columns: { sensor_uuid: 'Sensor UUID' }
    },
    sensor_name: {
      columns: { sensor_name: 'Sensor Name' }
    },
    deployment: {
      columns: { deployment: 'Deployment' }
    },
    deployment_uuid: {
      columns: { deployment_uuid: 'Deployment UUID' }
    },
    namespace: {
      columns: { namespace: 'Namespace' }
    },
    namespace_uuid: {
      columns: { namespace_uuid: 'Namespace UUID' }
    },
    organization: {
      columns: { organization: 'Organization' }
    },
    organization_uuid: {
      columns: { organization_uuid: 'Organization UUID' }
    },
    market: {
      columns: { market: 'Market' }
    },
    market_uuid: {
      columns: { market_uuid: 'Market UUID' }
    },
    floor: {
      columns: { floor: 'Floor' }
    },
    floor_uuid: {
      columns: { floor_uuid: 'Floor UUID' }
    },
    zone: {
      columns: { zone: 'Zone' }
    },
    zone_uuid: {
      columns: { zone_uuid: 'Zone UUID' }
    },
    building: {
      columns: { building: 'Building' }
    },
    building_uuid: {
      columns: { building_uuid: 'Building UUID' }
    },
    campus: {
      columns: { campus: 'Campus' }
    },
    campus_uuid: {
      columns: { campus_uuid: 'Campus UUID' }
    },
    service_provider: {
      columns: { service_provider: 'Service Provider' }
    },
    service_provider_uuid: {
      columns: { service_provider_uuid: 'Service Provider UUID' }
    },
    status: {
      columns: { status: 'Status' }
    },
    sensor_ip: {
      columns: { sensor_ip: 'Sensor IP' }
    },
    wireless_station: {
      columns: { wireless_station: 'Wireless Station' }
    },
    wireless_station_ip: {
      columns: { wireless_station_ip: 'Wireless Station IP' }
    },
    wireless_station_name: {
      columns: { wireless_station_name: 'Wireless Station Name' }
    },
    wireless_channel: {
      columns: { wireless_channel: 'Wireless Channel' }
    },
    wireless_tx_power: {
      columns: { wireless_tx_power: 'Wireless TX Power' }
    },
    wireless_admin_state: {
      columns: { wireless_admin_state: 'Wireless Admin State' }
    },
    wireless_op_state: {
      columns: { wireless_op_state: 'Wireless OP State' }
    },
    wireless_mode: {
      columns: { wireless_mode: 'Wireless mode' }
    },
    wireless_slot: {
      columns: { wireless_slot: 'Wireless_slot' }
    },
    type: {
      columns: { type: 'Type' }
    },
    client_count: {
      columns: { client_count: 'Client Count' }
    }
  })

  aggregations(
    events: {
      name: 'Events', unit: 'events', value: :events, size: 1, round: true,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ]
    },
    load: {
      name: 'Load', unit: '', value: :load, size: 1, round: true,
      aggregations: [
        { type: 'doubleSum', name: 'sum_value', fieldName: 'sum_value' },
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      post_aggregations: [{
        type: 'arithmetic',
        name: 'load',
        fn: '/',
        fields: [
          { type: 'fieldAccess', name: 'sum_value', fieldName: 'sum_value' },
          { type: 'fieldAccess', name: 'events', fieldName: 'events' }
        ]
      }]
    },
    wireless_stations: {
      name: 'Wireless Stations', unit: 'stations', value: :wireless_stations,
      size: 1, round: true,
      aggregations: [
        { type: 'cardinality', name: 'wireless_stations', fieldNames: ['wireless_station'] }
      ]
    }
  )

  menu :worker_menu, :check_aps,
       { controller: :jobs, action: :start_check_aps },
       html: { method: :post },
       caption: "tr.ap_state.start",
       if: proc { !DelayedJob.check_aps? }

end

# extend DRUID_CONFIG
druid_ap_config = YAML.load(File.read("#{Rails.root}/lib/modules/ap_state/config/rbdruid_config.yml"))[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_ap_config

require_dependency 'check_aps_status_job'
require_dependency 'delayed_job_apstate'
require_dependency 'jobs_controller_apstate'
require_dependency 'alarm_ap_state'
require_dependency 'application_controller_ap_state'
require_dependency 'hooks/alarms_ap_state_hook'

to_prepare = Proc.new do
  unless JobsController.include?(RedBorder::JobsControllerApstate)
    JobsController.send(:include, RedBorder::JobsControllerApstate)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerApState)
    ApplicationController.send(:include, RedBorder::ApplicationControllerApState)
  end
end

Rails.configuration.to_prepare(&to_prepare)
