DittoCode::HoldFile.if 'ENTERPRISE'

#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class CorrelationEngineRulesController < ApplicationController

  prepend_view_path "#{Rails.root}/app/views"
  
  ## ======================================================================
  # => Before and after
  ## ======================================================================

  before_action :require_super_administrative_privileges
  before_action :index, only: [:create, :create_basic_rule2, :export_tree, :import_tree] # so we can access @cep_status from create method.
  before_action :load_form_settings, only: [:basic_rule_2]
  before_action :create_topic_filters_and_selects, only: [:create_basic_rule_2]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @rules = CorrelationEngineRule.all.order(name: :asc)
    @cep_status = ''
    @managers = Manager.all.sort_by(&:name) # we could have a cluster, with multiple managers

    # TODO: Verify service health check using Consul
    @managers.each do |manager|
      if manager.managers_per_services('redborder-cep').empty?
        @cep_status = false
      else
        @cep_status = true
        break
      end
    end
  end

  def new
    render layout: false
  end

  def edit
    @rule = CorrelationEngineRule.find(params[:id])
    render layout: false
  end

  def create
    @rule = CorrelationEngineRule.new(rule_params)
    @rule.output = output_params
    @rule.user_id = current_user.id

    respond_to do |format|
      if @rule.save
        format.html { redirect_to correlation_engine_rules_path(value: @cep_status), notice: "#{I18n.t('controllers.rule_created')}" }
        format.js { flash[:notice] = "#{I18n.t('controllers.rule_created')}" }
        format.json { head :no_content }
      else
        format.html do
          redirect_to correlation_engine_rules_path(@rule),
                      alert: @rule.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join
        end
        format.js
        format.json { render json: @rule.errors, status: :unprocessable_entity }
      end
    end
  end

  def update
    @rule = CorrelationEngineRule.find(params[:id])
    @rule.update(rule_params)
  end

  def destroy
    @rule = CorrelationEngineRule.find(params[:id])
    @rule.destroy
  end

  def load_form_settings
    @intrusion_values = []
    @vault_values = []
    @monitor_values = []
    @traffic_values = []
    @aux = []

    # Getting name of dimensions from Intrusion
    Redborder::Module.get(:ips, :columns).values.each { |value|
      if value.key?(:name)
        @intrusion_values << if value[:name].start_with? 'tr.' then "#{I18n.t value[:name].to_s[3..-1]}" else value[:name] end
      else
       @intrusion_values << if value[:columns].values[0].start_with? 'tr.' then "#{I18n.t value[:columns].values[0].to_s[3..-1]}" else value[:columns].values[0] end
      end
    }

    # Getting name of Monitor dimensions
    Redborder::Module.get(:monitor, :tabs).values.each { |value|
      @monitor_values << if value[:name].start_with? 'tr.' then "#{I18n.t value[:name].to_s[3..-1]}" else value[:name] end
    }

    # Getting name of dimensions from Vault
    Vault::COLUMNS.values.each{ |value|
     if value.key?(:name)
       @vault_values << if value[:name].start_with? 'tr.' then "#{I18n.t value[:name].to_s[3..-1]}" else value[:name] end
     else
       @vault_values << if value[:columns].values[0].start_with? 'tr.' then "#{I18n.t value[:columns].values[0].to_s[3..-1]}" else value[:columns].values[0] end
     end
    }

    # Getting name of Traffic dimensions
    Traffic::COLUMNS.values.each{ |value|
      if value.key?(:name)
        @traffic_values << if value[:name].start_with? 'tr.' then "#{I18n.t value[:name].to_s[3..-1]}" else value[:name] end
      else
       @traffic_values << if value[:columns].values[0].start_with? 'tr.' then "#{I18n.t value[:columns].values[0].to_s[3..-1]}" else value[:columns].values[0] end
      end
    }

    common_elements

    @traffic_aggregations = Redborder::Module.get(:flow, :aggregations).map {|k,v| k.to_s}.zip(Redborder::Module.get(:flow, :aggregations).map {|k,v| k.to_s})
    @vault_aggregations = Redborder::Module.get(:vault, :aggregations).map {|k,v| k.to_s}.zip(Redborder::Module.get(:vault, :aggregations).map {|k,v| k.to_s})
    @intrusion_aggregations = Redborder::Module.get(:ips, :aggregations).map {|k,v| k.to_s}.zip(Redborder::Module.get(:ips, :aggregations).map {|k,v| k.to_s})
    @monitor_aggregations =  Redborder::Module.get(:monitor, :aggregations).map {|k,v| k.to_s}.zip(Redborder::Module.get(:monitor, :aggregations).map {|k,v| k.to_s})

    # Building Arrays like [["Lan IP", "F.lan_ip"]]. Letter in second argument is to reference input inside the query,
    # T = Traffic.     V = Vault.    M = Monitor.    I = Intrusion.
    @vault_columns = (@vault_values.zip(Vault::COLUMNS.keys.map {|k| "V.#{k.to_s}"}) + [["Vault Timestamp", "V.timestamp"]]).sort
    @traffic_columns = (@traffic_values.zip(Traffic::COLUMNS.keys.map {|k| "T.#{k.to_s}"}) + [["#{I18n.t('cep_controller.traffic')} Timestamp", "T.timestamp"]]).sort
    @monitor_columns = @monitor_values.zip(Redborder::Module.get(:monitor, :tabs).keys.map {|k| "M.#{k.to_s}"}).sort
    @intrusion_columns = @intrusion_values.zip(Redborder::Module.get(:ips, :columns).keys.map {|k| "I.#{k.to_s}"}).sort

    @traffic_vault = @traffic_columns + @vault_columns
    @traffic_vault_aggregations = @traffic_aggregations + @vault_aggregations

    @traffic_monitor = @traffic_columns + @monitor_columns
    @traffic_monitor_aggregations = @traffic_aggregations + @monitor_aggregations

    @traffic_intrusion = @traffic_columns + @intrusion_columns
    @traffic_intrusion_aggregations = @traffic_aggregations + @intrusion_aggregations

    @monitor_intrusion = @monitor_columns + @intrusion_columns
    @monitor_intrusion_aggregations = @monitor_aggregations + @intrusion_aggregations

    @monitor_vault = @monitor_columns + @vault_columns
    @monitor_vault_aggregations = @monitor_aggregations + @vault_aggregations

    @intrusion_vault = @intrusion_columns + @vault_columns
    @intrusion_vault_aggregations = @intrusion_aggregations + @vault_aggregations

    @functions = [["#{I18n.t 'cep_controller.none'}", ""],["#{I18n.t 'cep_controller.average'}", "avg"],["#{I18n.t 'cep_controller.summation'}", "sum"],["#{I18n.t 'cep_controller.maximum'}", "max"],["#{I18n.t 'cep_controller.minimum'}", "min"],["#{I18n.t 'cep_controller.count'}", "count"],["#{I18n.t 'cep_controller.distinct'}", "distinctCount"],["#{I18n.t 'cep_controller.max_forever'}", "maxForever"],["#{I18n.t 'cep_controller.min_forever'}","minForever"],["#{I18n.t 'cep_controller.standard'}","stddev"]]
    @operator = [["#{I18n.t 'cep_controller.less_than'}", "<"], ["#{I18n.t 'cep_controller.less_equal'}", "<="], ["#{I18n.t 'cep_controller.greater_than'}", ">"], ["#{I18n.t 'cep_controller.greater_equal'}", "=>"], ["#{I18n.t 'cep_controller.equal'}", "=="], ["#{I18n.t 'cep_controller.not_equal'}", "!="], ["#{I18n.t 'cep_controller.contains'}", "contains"], ["#{I18n.t 'cep_controller.isNull'}", "isNull"]]
    @window = [["#{I18n.t('cep_controller.last_min')}", "window.time"], ["#{I18n.t 'cep_controller.last_min_batch'}", "window.timeBatch"], ["#{I18n.t 'cep_controller.last_messages'}", "window.length"], ["#{I18n.t 'cep_controller.last_messages_batch'}", "window.lengthBatch"]]
    @inputs = [["#{I18n.t 'cep_controller.traffic'}", "rb_flow_post"],["Intrusion", "rb_event_post"],["Monitor", "rb_monitor"], ["Vault", "rb_vault_post"]]
  end

  # With this method, we find the common elements between two arrays, and place the corresponding input at the beginning of the value
  # so we know if the dimension is from Traffic or from Vault, for example
  def common_elements
    @common_traffic_vault = @traffic_values & @vault_values
    @common_traffic_monitor = @traffic_values & @monitor_values
    @common_traffic_intrusion = @traffic_values & @intrusion_values
    @common_monitor_intrusion = @monitor_values & @intrusion_values
    @common_monitor_vault = @monitor_values & @vault_values
    @common_intrusion_vault = @intrusion_values & @vault_values

    @traffic_values.map.with_index do |n, i|
      if @common_traffic_vault.include?(n) || @common_traffic_monitor.include?(n) || @common_traffic_intrusion.include?("#{n}")
        @traffic_values[i] =  "#{I18n.t('cep_controller.traffic')} #{ n}"
      end
    end

    @vault_values.map.with_index do |n, i|
      if @common_traffic_vault.include?(n) || @common_monitor_vault.include?(n) || @common_intrusion_vault.include?("#{n}")
        @vault_values[i] =  "Vault #{ n}"
      end
    end

    @monitor_values.map.with_index do |n, i|
      if @common_traffic_monitor.include?(n) || @common_monitor_intrusion.include?(n) || @common_monitor_vault.include?("#{n}")
        @monitor_values[i] = "Monitor #{ n}"
      end
    end

    @intrusion_values.map.with_index do |n, i|
      if @common_traffic_intrusion.include?(n) || @common_monitor_intrusion.include?(n) || @common_intrusion_vault.include?("#{n}")
        @intrusion_values[i] = "Intrusion #{n}"
      end
    end


  end

  def basic_rule_2
    render layout: false
  end

  def create_topic_filters_and_selects
    @condition1 = ""
    @condition2 = ""
    @join = ""
    @select = ""

    #Letter used to reference each input inside the query.
    @first_ref = 'T' if params[:topic1] == 'rb_flow_post'
    @first_ref = 'V' if params[:topic1] == 'rb_vault_post'
    @first_ref = 'M' if params[:topic1] == 'rb_monitor'
    @first_ref = 'I' if params[:topic1] == 'rb_event_post'

    @second_ref = 'T' if params[:topic2] == 'rb_flow_post'
    @second_ref = 'V' if params[:topic2] == 'rb_vault_post'
    @second_ref = 'M' if params[:topic2] == 'rb_monitor'
    @second_ref = 'I' if params[:topic2] == 'rb_event_post'

    # Number of filters/selects that we added
    topic1_parameters = params[:correlation_engine_rule].select { |key, value| key.to_s.match(/^topic1_filter\d+/)}
    topic2_parameters = params[:correlation_engine_rule].select { |key, value| key.to_s.match(/^topic2_filter\d+/)}
    select_parameters = params[:correlation_engine_rule].select { |key, value| key.to_s.match(/^select\d+/)}

    #Build filters for the first topic
    i = 0
    while ((i < topic1_parameters.size)) do
      @condition1 << "[ " if (i == 0)
      var_not = "topic1_not_filter" + i.to_s
      var_filter = "topic1_filter" + i.to_s
      var_operator = "topic1_filter_operator" + i.to_s
      if params[:correlation_engine_rule][var_filter.to_sym]
        @condition1 << "not (" if params[:correlation_engine_rule][var_not.to_sym] == "1"
        if params[:correlation_engine_rule][var_operator.to_sym] == "isNull"
          @condition1 << "#{@join}#{params[:correlation_engine_rule][var_filter.to_sym][2..-1]} is null "
        else 
          @condition1 << "#{@join}#{params[:correlation_engine_rule][var_filter.to_sym][2..-1]} #{params[:correlation_engine_rule][var_operator.to_sym]} #{rule_params[:topic1_filter_value][i]} "
        end
        @condition1 << ")" if params[:correlation_engine_rule][var_not.to_sym] == "1"
        @join = "and "
        @condition1 << "]" if (i == topic1_parameters.size-1)
      end
      i += 1
    end

    #Build filters for the second topic
    i = 0
    @join = ""
    while ((i < topic2_parameters.size)) do
      @condition2 << "[" if (i == 0)
      var_not = "topic2_not_filter" + i.to_s
      var_filter = "topic2_filter" + i.to_s
      var_operator = "topic2_filter_operator" + i.to_s
      if params[:correlation_engine_rule][var_filter.to_sym]
        @condition2 << "not (" if params[:correlation_engine_rule][var_not.to_sym] == "1"
        if params[:correlation_engine_rule][var_operator.to_sym] == "isNull"
          @condition2 << "#{@join}#{params[:correlation_engine_rule][var_filter.to_sym][2..-1]} is null "
        else
          @condition2 << "#{@join}#{params[:correlation_engine_rule][var_filter.to_sym][2..-1]} #{params[:correlation_engine_rule][var_operator.to_sym]} #{rule_params[:topic2_filter_value][i]}"
        end
        @condition2 << ")" if params[:correlation_engine_rule][var_not.to_sym] == "1"
        @join = "and "
        @condition2 << "]" if (i == topic2_parameters.size-1)
      end
      i += 1
    end

    #Build the selections that we made (last step from the wizard)
    i = 1
    @join = ""
    while (i <= select_parameters.size) do
      var_function = "select_function" + i.to_s
      var_select = "select" + i.to_s
      var_name = "select_name" + i.to_s
      @select << @join
      @select << "#{params[:correlation_engine_rule][var_function.to_sym]}(" if params[:correlation_engine_rule][var_function.to_sym]
      @select << "#{params[:correlation_engine_rule][var_select.to_sym]}"
      @select << ")" if params[:correlation_engine_rule][var_function.to_sym]
      @select << " as #{params[:correlation_engine_rule][var_name.to_sym]}" if params[:correlation_engine_rule][var_name.to_sym]
      @join = ","
      i += 1
    end
  end

  def create_basic_rule_2
    @input = ["#{rule_params[:topic1]}", "#{rule_params[:topic2]}"]
    @output = "{\"outputStream\":\"#{rule_params[:output]}\"}"
    @execution_plan = "from #{rule_params[:topic1]}#{@condition1}##{rule_params[:window_type]}(#{rule_params[:window_value]}) as #{@first_ref} join #{rule_params[:topic2]}#{@condition2}##{rule_params[:window_type2]}(#{rule_params[:window_value2]}) as #{@second_ref} on #{rule_params[:join_attr]} #{rule_params[:join_type]} #{rule_params[:join_attr2]} select #{@select} insert into outputStream;".gsub('"', '')

    @rule = CorrelationEngineRule.new(name: rule_params[:name],
                                      execution_plan: @execution_plan,
                                      input: @input)
    @rule.output = @output
    @rule.user_id = current_user.id

    respond_to do |format|
      if @rule.save
        format.html { redirect_to correlation_engine_rules_path(value: @cep_status), notice: "#{I18n.t('controllers.rule_created')}" }
      else
        format.html do
          redirect_to correlation_engine_rules_path(@rule),
                      alert: @rule.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join
        end
      end
    end
  end

DittoCode::Exec.if 'ENTERPRISE' do

  def export_tree
    dir = "/tmp/#{Time.now.to_i}"
    Dir.mkdir dir
    content = []

    CorrelationEngineRule.where(enabled: true).each do |rule|
      output = "{\"output\": #{rule.output.to_json}, \"input\":#{rule.input}, \"executionPlan\": \"#{rule.execution_plan}\", \"name\":\"#{rule.name}\"}"
      content << JSON.parse(output)
    end

    File.open("#{dir}/state.json", 'w') do |f|
      f.write(content)
    end

    # Compress directory
    tar_file = "#{Rails.root}/tmp/Export_#{Time.now.to_i}_#{Random.rand(1000)}.tar.gz"
    system('tar', 'cvzf', tar_file, '-C', dir, 'state.json')

    send_file tar_file

    FileUtils.remove_dir(dir)
  end

  def import_tree
    if request.get?
      render layout: false
    elsif request.post?
      #datos de acceso para S3
      s3_config = YAML.load_file("#{Rails.root}/config/aws.yml")
      s3 =
        AWS::S3.new(
          access_key_id: s3_config[Rails.env]['access_key_id'],
          secret_access_key: s3_config[Rails.env]['secret_access_key'],
          s3_endpoint: s3_config[Rails.env]['s3_host_name']
        )

      #
      # Management of the case where despite having a correct extension when opening the file, 
      # the expected structure is the wrong one and the temporary folder where the information 
      # is unrolled is empty.
      #
      tempfile_path = params['correlation_engine_rule']['tgz_file'].tempfile.path
      fail "#{I18n.t('correlation_engine_rules.empty_error')}" if tempfile_path.nil? || tempfile_path.empty?

      #
      # Management of allowed extensions, where files with extension tgz, gz and tar are allowed.
      #
      filename = params['correlation_engine_rule']['tgz_file'].original_filename
      unless filename.match(/(tar|tgz|gz)$/)
        fail "#{I18n.t('correlation_engine_rules.ext_error')}"
      end


      obj =
        s3.buckets[s3_config[Rails.env]['bucket'] + "/"]
          .objects["imports/#{filename}"]

      obj.write(file: tempfile_path)

      download_s3(filename)
      dir = extract_file(filename)

      #
      #If the file is extracted successfully, check that the mandatory state.json file 
      #of the import structure is found.
      #
      unless File.exist?("/#{dir}/state.json")
        fail "#{I18n.t('correlation_engine_rules.format_error')}"
      end

      new_data = File.read("/#{dir}/state.json").gsub("=>",":").strip
      imported_content = JSON.parse(new_data)

      created_rule = false
      if params["override"] == 'true'
        imported_content.each do |rule|
          # If rule exists, delete it since we are going to override it.
          CorrelationEngineRule.where(name: rule["name"])[0].destroy if !CorrelationEngineRule.where(name: rule["name"]).empty?
          @rule = CorrelationEngineRule.new(name: rule["name"],
                                            execution_plan: rule["executionPlan"],
                                            input: rule["input"])
          @rule.output = rule["output"]
          @rule.user_id = current_user.id
          if @rule.save
            created_rule = true
          end
        end
      else
        imported_content.each do |rule|
          #If rule does not exist, create it. Since we not overriding, don't do anything if it existsi
          if CorrelationEngineRule.where(name: rule["name"]).empty?
            @rule = CorrelationEngineRule.new(name: rule["name"],
                                            execution_plan: rule["executionPlan"],
                                            input: rule["input"])
            @rule.output = rule["output"]
            @rule.user_id = current_user.id
            if @rule.save
              created_rule = true
            end
          end
        end
      end
      if created_rule
        message = "#{I18n.t('controllers.rule_created')}"
      else
        message = "#{I18n.t('controllers.rule_yet')}"
      end

      redirect_to correlation_engine_rules_path(value: @cep_status), notice: message
    end
    rescue => e
      rb_flash e.message, type: :error
      redirect_to action: :index
  end

  # Functions taken from jobs_helper, since we don't have access to those helpers
  def download_s3(file_name)
    s3_config = YAML.load_file("#{Rails.root}/config/aws.yml")
    s3 = AWS::S3.new(access_key_id: s3_config[Rails.env]['access_key_id'],
                     secret_access_key: s3_config[Rails.env]['secret_access_key'],
                     s3_endpoint: s3_config[Rails.env]['s3_host_name'])

    obj = s3.buckets[s3_config[Rails.env]['bucket'] + "/"]
          .objects["imports/#{file_name}"]

    # Download s3 file
    File.open("/tmp/#{file_name}", 'wb') do |file|
      file.write(obj.read)
    end
  end

  def extract_file(file_name)
    # Create unique dir
    dir = "#{Rails.root}/tmp/imported_files/#{file_name}"
    FileUtils.mkdir_p dir
    system('tar', 'xzf', "/tmp/#{file_name}", '-C', dir)

    # Delete tgz file
    FileUtils.rm_rf "/tmp/#{file_name}"

    dir
  end

end

  #
  # This method synchronizes the rules table with remote correlation engine,
  # using given API REST
  #

  def apply
    rules = CorrelationEngineRule.where(enabled: true)
    rules_ = []
    correlation_config = YAML.load_file("#{Rails.root}/config/correlation.yml")[Rails.env]

    rules.each do |rule|
      if rule.output.is_a?(String)
        begin
          output_json = rule.output.to_json.gsub(':', '=>')
          rule.output = JSON.parse(output_json) 
        rescue JSON::ParserError => e
          next  # Skip this rule if parsing fails
        end
      end

      rules_.push(
        id: rule.id.to_s,
        input: rule.input,
        output: rule.output,
        executionPlan: rule.execution_plan,
        version: rule.updated_at.to_i
      )
    end

    http = Net::HTTP.new(correlation_config['host'], correlation_config['port'])
    request = Net::HTTP::Post.new('/v1/synchronize')
    request.add_field('Content-Type', 'application/json')
    request.body = rules_.to_json
    begin
      @response = http.request(request)
    rescue Timeout::Error, Errno::EINVAL, Errno::ECONNRESET, EOFError,
           Net::HTTPBadResponse, Net::HTTPHeaderSyntaxError, Net::ProtocolError, SocketError,
           Errno::ECONNREFUSED => e
      Rails.logger.error e
      Rails.logger.error e.backtrace.join("\n")
    end
  end

  private

  #
  # This method builds a hash from the output stream-topic array
  #

  def output_params
    output = {}
    if params[:correlation_engine_rule][:stream].size == params[:correlation_engine_rule][:topic].size
      params[:correlation_engine_rule][:stream].each_with_index do |el, i|
        output[el] = params[:correlation_engine_rule][:topic][i]
      end
    end
    output
  end

  def rule_params
    params.require(:correlation_engine_rule)
      .permit(:name, :execution_plan, :enabled, :output, :topic1, :window_type, :window_value,
              :window_value2, :window_type2, :topic2, :join_attr, :join_attr2, :join_type,
              topic1_filter_value: [], topic2_filter_value: [], input: [])
  end
end
