#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Define a correlation eingine rule in redBorder app
#
class CorrelationEngineRule < ApplicationRecord
  # -------------------------------------------------------------
  # => Callbacks
  # -------------------------------------------------------------

  after_save :applied_false

  # -------------------------------------------------------------
  # => Validations
  # -------------------------------------------------------------

  validates :name, :input, :execution_plan, :output, presence: true
  validates :name, uniqueness: true
  validates :enabled, inclusion: { in: [true, false] }

  # -------------------------------------------------------------
  # => Relations
  # -------------------------------------------------------------

  belongs_to :user

  # -------------------------------------------------------------
  # => Functions
  # -------------------------------------------------------------

  def applied_false
    update_column(:applied, false)
  end

  #
  # Show input in an humanized way. Instead of show 'rb_flow', it must be shown 'Traffic'
  #
  def humanized_inputs
    inputs = Redborder::Module.registered_modules.inject({}) do |hash, (_k, v)|
      v[:cep_topic] ? hash.merge(v[:cep_topic] => v[:name]) : hash
    end

    # Translating module Traffic, since it has the tr. workaround that we used in the init.rb files for multilanguage
    inputs[:rb_flow_post] = I18n.t inputs[:rb_flow_post][3..-1] if inputs[:rb_flow_post].start_with? "tr."

    input.map { |i| inputs[i.to_sym] }
  end
end
