redBorder.aps = function (options) {
  var $container = $('[data-rb-aps="container"]'),
    dimensions = $container.data('rb-aps-dimensions'),
    image_width_feet = dimensions[0],
    image_height_feet = dimensions[1],
    parent_domains = [],
    timeoutVar = undefined,
    started = false,
    menu_position = {},
    elements = [],
    image_height_px,
    $holder, $image, $tooltip,
    image_width_px,
    domainBounds;

  //
  // Context Menu
  //

  function create_context_menu(x, y) {
    var context_menu = $(' \
      <ul id="context_menu" style="position: absolute; z-index: 1; top: ' + y + 'px; left: ' + x + 'px;"> \
        <li><a href="#" data-rb-aps-action="create_ap">' + I18n.t('javascript.create_ap') + '</a></li> \
        <li><a href="#" data-rb-aps-action="locate_ap">' + I18n.t('javascript.locate_ap') + '</a></li> \
        <li><a href="#" data-rb-aps-action="create_gps_marker">' + I18n.t('javascript.create_gps') + '</a></li> \
      </ul> \
    ');

    // Add the menu and start jQuery UI menu
    $container.append(context_menu);
    $('#context_menu', $container).menu();
  }

  function remove_context_menu() {
    $('#context_menu', $container).remove();
  }

  //
  // Tooltips
  //

  function fill_aps_tooltip(element) {
    var last_check_in, tooltip;

    tooltip = $('<div class="rb-aps-tooltip-field">' + I18n.t('javascript.name') + ' <span>' + element.name + '</span></div>' +
      '<div class="rb-aps-tooltip-field">IP: <span>' + (element.ip_address || 'N/A') + '</span></div>' +
      '<div class="rb-aps-tooltip-field">MAC: <span>' + element.mac_address + '</span></div>');

    $tooltip.html(tooltip);

    if (element.last_check_in != undefined && element.last_check_in != null) {
      last_check_in = new Date(element.last_check_in).toLocaleString();
      $tooltip.append('<div class="rb-aps-tooltip-field">' +
        I18n.t('javascript.last_check') + '<span>' + last_check_in + '</span>' +
        '</div>');
    }
  }

  function fill_gps_markers_tooltip(element) {
    var last_check_in, tooltip;

    tooltip = $('<div class="rb-aps-tooltip-field">' + I18n.t('javascript.latitude') + ' <span>' + element.latitude + '</span></div>' +
      '<div class="rb-aps-tooltip-field">' + I18n.t('javascript.longitude') + ' <span>' + element.longitude + '</span></div>');

    $tooltip.html(tooltip);
  }

  //
  // DOM Elements
  //

  function append_ap_html(ap, x, y) {
    var zones_id = [];

    if (ap.zones != undefined) {
      for(i = 0; i < ap.zones.length; i++){
        zones_id.push(ap.zones[i].id);
      }
    } else {
      zones_id = []
    }

    var marker = $(' \
      <svg height="40" width="40" class="rb-aps zone-ap" data-rb-aps-id="' + ap.id + '" \
        style="position: absolute; top: ' + (y - 20) + 'px; left: ' + (x - 20) + 'px" \
        data-rb-aps-zones="' + zones_id.join() + '" data-rb-aps-name="' + ap.name + '" > \
        <circle cx="20" cy="20" r="12" class="rb-aps-ap" data-rb-status="' + ap.status + '" /> \
      </svg> \
    ');

    $('circle', marker).data('data', ap);
    $holder.append(marker);
  }

  function append_gps_marker_html(gps_marker, x, y) {
    var marker = $(' \
      <svg height="40" width="40" class="rb-aps" data-rb-aps-id="' + gps_marker.id + '" \
        style="position: absolute; top: ' + (y - 20) + 'px; left: ' + (x - 20) + 'px" \
        " data-rb-aps-name="' + gps_marker.name + '"> \
        <circle cx="20" cy="20" r="15" class="rb-aps-gps" /> \
      </svg> \
    ');

    if (options.show_gps_markers === false) {
      marker.hide();
    }

    $('circle', marker).data('data', gps_marker);
    $holder.append(marker);
  }

  function add_parent_domains(new_parent_domains) {
    parent_domains = new_parent_domains;
  }

  //
  // Elements builders
  //

  function create_ap(e) {
    var x_in_px = menu_position.x,
      y_in_px = menu_position.y,
      x_relative = x_in_px / image_width_px,
      y_relative = y_in_px / image_height_px,
      x_in_feets = x_in_px * image_width_feet / image_width_px,
      y_in_feets = y_in_px * image_height_feet / image_height_px,
      form_url = $container.data('rb-aps-create-ap-url');
    remove_context_menu();

    $('#new_ap_modal').modal('show');

    $.get(form_url, { point: [x_relative, y_relative] }, function (data) {
      $('#new_ap_modal .modal-content').html(data);
      $('#access_point_x').val(x_relative);
      $('#access_point_y').val(y_relative);
    });
  }

  function locate_ap(e) {
    var x_in_px = menu_position.x,
      y_in_px = menu_position.y,
      x_relative = x_in_px / image_width_px,
      y_relative = y_in_px / image_height_px,
      form_url = $container.data('rb-aps-locate-ap-url');
    remove_context_menu();

    $('#new_locate_ap_modal').modal('show');

    $.get(form_url, { point: [x_relative, y_relative] }, function (data) {
        $('#new_locate_ap_modal .modal-content').html(data);
        $('#access_point_x').val(x_relative);
        $('#access_point_y').val(y_relative);
    });
  }

  function create_gps_marker(e) {
    var x_in_px = menu_position.x,
      y_in_px = menu_position.y,
      x_in_feets = x_in_px * image_width_feet / image_width_px,
      y_in_feets = y_in_px * image_height_feet / image_height_px,
      form_url = $container.data('rb-aps-create-gps-url'),
      gps_dom_elements;

    remove_context_menu();

    $('#new_gps_marker_modal').modal('show');

    $.get(form_url, function (data) {
      var gps_markers_map;

      $('#new_gps_marker_modal .modal-content').html(data);
      $('#gps_marker_x').val(x_in_feets);
      $('#gps_marker_y').val(y_in_feets);

      try {
        gps_markers_map = redBorder.map({
          container: 'gps_markers_map',
          type: 'satellite',
          draggableCursor: 'crosshair',
          scrollwheel: false,
          onClick: function (map, event) {
            var lat = Math.round(event.latLng.lat() * 1000000) / 1000000,
                lng = Math.round(event.latLng.lng() * 1000000) / 1000000;

            $('#gps_marker_latitude').val(lat);
            $('#gps_marker_longitude').val(lng);
          },
          onLoad: function (map, event) {
            gps_markers_map.reload();
            if (gps_markers_map.getMarkers().length > 0) {
              gps_markers_map.center();
            } else if (parent_domains !== undefined && parent_domains.length > 0) {
              // Center on parent
              gps_markers_map.getMap().fitBounds(getDomainBounds());
            }
          }
        });

        gps_dom_elements = $('svg circle.rb-aps-gps', $holder);
        gps_dom_elements.each(function (index) {
          var self = $(this),
              data = self.data('data');

          gps_markers_map.addMarker(data.latitude, data.longitude, {
            icon: {
              path: google.maps.SymbolPath.CIRCLE,
              scale: 12,
              strokeWeight: 2,
              fillColor: 'blue',
              fillOpacity: 1
            }
          });
        });

        redBorder.gps_markers_map = gps_markers_map;
      } catch (err) {
        console.error("APS GPS " + err);
      }
    });
  }

  /*
   * Calculate building bounds
   */
  function getDomainBounds() {
    if (domainBounds === undefined) {
      domainBounds = redBorder.utils.map.getPolygonsBounds(parent_domains);
    }

    return domainBounds;
  }

  function add_aps(aps) {
    for (var i = 0; i < aps.length; i++) {
      // Don't check types because lat and long can be null
      if(aps[i].longitude == undefined && aps[i].latitude == undefined &&
         aps[i].x == undefined && aps[i].y == undefined){
        continue;
      }
      aps[i].type = 'ap';
      elements.push(aps[i]);
    }

    if (started && elements.length != 0) create_elements();
  }

  function add_ap(ap) {
    ap.type = 'ap';
    elements.push(ap);

    if (started && elements.length != 0) create_elements();
  }

  function add_gps_markers(gps_markers) {
    for (var i = 0; i < gps_markers.length; i++) {
      gps_markers[i].type = 'gps_marker';
      elements.push(gps_markers[i]);
    }

    if (started && elements.length != 0) create_elements();
  }

  function add_gps_marker(gps_marker) {
    gps_marker.type = 'gps_marker';
    elements.push(gps_marker);
    if (started && elements.length != 0) create_elements();
  }

  //
  // Elements operations
  //

  function highlight_by_name(name) {
    var all_elements = $('svg circle', $holder);
    var length = all_elements.length;

    for (var i = 0; i < length; i++) {
      all_elements[i].classList.remove("rb-aps-highlighted");
      $(all_elements[i]).parent()[0].classList.remove("rb-aps-big");
    }

    var element = $("svg[data-rb-aps-name*='" + name + "'] circle", $holder);
    if (element.length === 1) {
      element[0].classList.add("rb-aps-highlighted");
    }
  }

  function highlight_by_names_and_types(type) {
    var all_elements = $('svg circle', $holder);
    var length = all_elements.length;

    for (var i = 0; i < length; i++) {
      all_elements[i].classList.remove("rb-aps-highlighted");
      $(all_elements[i]).parent()[0].classList.remove("rb-aps-big");
    }

    var selecteds = [];
    if ( type === 'ap') {
      selecteds = $('div#access_points tr.selected');
    } else {
      selecteds = $('div#gps_markers tr.selected');
    }

    for (var i=0; i<selecteds.length; i++) {
      var element = [];
      if ( type === 'ap') {
        var name = $(selecteds[i]).find('td[data-label="Name"]').html();
        element = $("svg[data-rb-aps-name*='" + name + "'] circle", $holder);
      } else {
        var id = $(selecteds[i]).data('gps-marker-id');
        element = $("svg[data-rb-aps-id*='" + id + "'] circle", $holder);
      }
      if (element.length === 1) {
        element[0].classList.add("rb-aps-highlighted");
      }
    }
  }

  function highlight(id) {
    var all_elements = $('svg circle', $holder);
    var length = all_elements.length;

    for (var i = 0; i < length; i++) {
      all_elements[i].classList.remove("rb-aps-highlighted");
      $(all_elements[i]).parent()[0].classList.remove("rb-aps-big");
    }

    var element = $("svg[data-rb-aps-id='" + id + "'] circle", $holder);
    if (element.length === 1) {
      element[0].classList.add("rb-aps-highlighted");
      element.parent()[0].classList.add("rb-aps-big");
    }
  }

  function highlight_by_zones(id) {
    var all_elements = $('svg circle', $holder);
    var length = all_elements.length;

    for (var i = 0; i < length; i++) {
      all_elements[i].classList.remove("rb-aps-highlighted");
      $(all_elements[i]).parent()[0].classList.remove("rb-aps-big");
    }

    $holder.find('.zone-ap').each(function( index ){
      var zone_ids = $(this).attr('data-rb-aps-zones')
      if(zone_ids != undefined && zone_ids.split(',').indexOf(id) > -1){
        $(this).find('circle')[0].classList.add("rb-aps-highlighted")
        $(this)[0].classList.add("rb-aps-big")
      }
    });
  }

  function hide_gps_markers() {
    $('.rb-aps-gps', $holder).parents('svg').hide();
  }

  function show_gps_markers() {
    $('.rb-aps-gps', $holder).parents('svg').show();
  }

  //
  // Helpers
  //

  function matrixToArray(str){
      return str.match(/(-?[0-9\.]+)/g);
  };

  //
  // Initialization code
  //

  function bind_events() {
    // Before bind, clear old events
    clear_events();

    // Adds the custom context menu when right clicking the image
    $image.bind("contextmenu", function (e) {
      var image_offset = $image.offset(),
        container_offset = $container.offset(),
        xpos = e.pageX - image_offset.left,
        ypos = e.pageY - image_offset.top,
        xmenu = e.pageX - container_offset.left,
        ymenu = e.pageY - container_offset.top;

      var scale;

      if($holder.css('transform') == undefined){
        scale = matrixToArray($holder.css('-webkit-transform'))[0];
      } else {
        scale = matrixToArray($holder.css('transform'))[0];
      }

      menu_position = {
        x: xpos * (1 / scale),
        y: ypos * (1 / scale)
      };

      remove_context_menu();
      if (window.redBorder.admin_user === true) {
        create_context_menu(xmenu, ymenu);
      }
      e.preventDefault();
      return false;
    });

    // Removes the custom context menu
    $image.bind("mousedown", function (e) {
      remove_context_menu();
      $('.zoomer-image, svg', $holder).css('cursor', 'move');
    });

    $image.bind("mouseup", function (e) {
      $('.zoomer-image, svg', $holder).css('cursor', 'crosshair');
    });

    // Calls the appropiate actions when clicking on context menu items
    $(document).on('click', '[data-rb-aps="container"] #context_menu a', function (e) {
      var action = $(this).data('rb-aps-action');

      if (action === 'create_ap') create_ap(e);
      else if (action === 'locate_ap') locate_ap(e);
      else if (action === 'create_gps_marker') create_gps_marker(e);

      e.preventDefault();
      return false;
    });

    $holder.on('mouseenter', 'svg circle', function (e) {
      var image_offset = $image.offset(),
          container_offset = $container.offset(),
          circle_position = $(this).offset(),
          xpos = circle_position.left - image_offset.left,
          ypos = circle_position.top - image_offset.top,
          xmenu = circle_position.left - container_offset.left,
          ymenu = circle_position.top - container_offset.top,
          element_data = $(this).data('data');

      $tooltip.css('top', ymenu - 25);
      $tooltip.css('left', xmenu + 40);

      if (element_data.type === 'ap') {
        fill_aps_tooltip(element_data);
      } else {
        fill_gps_markers_tooltip(element_data);
      }

      $tooltip.show();
    });

    $holder.on('mouseout', 'svg circle', function (e) {
      $tooltip.hide();
    })
  }

  // Clear listeners to prevent duplicity of events
  function clear_events() {
    $image.unbind("contextmenu");
    $image.unbind("mousedown");
    $image.unbind("mouseup");
    $(document).off('click', '[data-rb-aps="container"] #context_menu a');
    $holder.off('mouseenter');
    $holder.off('mouseout');
  }

  function create_elements() {
    for (var i = 0; i < elements.length; i++) {
      var x = elements[i].x * image_width_px;
      var y = elements[i].y * image_height_px;
      if (elements[i].type == 'ap') {
        append_ap_html(elements[i], x, y);
      } else {
        append_gps_marker_html(elements[i], x, y);
      }
    }

    elements = [];
  }

  (function initialize() {
    $container.css('position', 'relative');

    $container.zoomer({
      increment: 0.02,
      controls: {
        position: 'top-right'
      }
    });

    $holder = $('.zoomer-holder', $container);
    $image = $('.zoomer-image', $container);

    image_width_px = $image.width();
    image_height_px = $image.height();

    if (image_height_px > 0 && image_width_px > 0) {
      started = true;
      timeoutVar = undefined;
      bind_events();

      // Build the tooltip
      $tooltip = $('<div class="rb-aps-tooltip"></div>');
      $tooltip.hide();
      $container.append($tooltip);

      if (image_width_feet == 0 || image_height_feet == 0) {
        image_width_feet = image_width_px;
        image_height_feet = image_height_px;
      }

      if (elements.length != 0) {
        create_elements();
      }
    } else {
      timeoutVar = setTimeout(initialize, 500);
    }
  })();

  return {
    add_aps: add_aps,
    add_ap: add_ap,
    add_gps_markers: add_gps_markers,
    add_gps_marker: add_gps_marker,
    add_parent_domains: add_parent_domains,
    highlight: highlight,
    highlight_by_name: highlight_by_name,
    highlight_by_names_and_types: highlight_by_names_and_types,
    highlight_by_zones: highlight_by_zones,
    hide_gps_markers: hide_gps_markers,
    show_gps_markers: show_gps_markers
  }
};
