redBorder.widgets.graphs.bandwidth = function (container, widget) {
  var graph,
      graph_prior,
      graph_last_range,
      wtype,
      stepSecs = widget.graph.step / 1000,
      $container = $('#' + container),
      time_last_range = [],
      time_last_day = [];

  NAMES = {
    "last_5_minute": {
      current: I18n.t('javascript.last_5_minute_current'),
      last: I18n.t('javascript.last_5_minutes_last'),
      last_period: I18n.t('javascript.last_5_minutes_last_period')
    },
    "last_15_minute": {
      current: I18n.t('javascript.last_15_minutes_current'),
      last: I18n.t('javascript.last_15_minute_last'),
      last_period: I18n.t('javascript.last_15_minute_last_period')
    },
    "last_30_minute": {
      current: I18n.t('javascript.last_30_minutes_current'),
      last: I18n.t('javascript.last_30_minute_last'),
      last_period: I18n.t('javascript.last_30_minute_last_period')
    },
    "last_1_hour": {
      current: I18n.t('javascript.last_1_hour_current'),
      last: I18n.t('javascript.last_1_hour_last'),
      last_period: I18n.t('javascript.last_1_hour_last_period')
    },
    "last_2_hour": {
      current: I18n.t('javascript.last_2_hour_current'),
      last: I18n.t('javascript.last_2_hour_last'),
      last_period: I18n.t('javascript.last_2_hour_last_period')
    },
    "last_3_hour": {
      current: I18n.t('javascript.last_3_hour_current'),
      last: I18n.t('javascript.last_3_hour_last'),
      last_period: I18n.t('javascript.last_3_hour_last_period')
    },
    "last_12_hour": {
      current: I18n.t('javascript.last_12_hour_current'),
      last: I18n.t('javascript.last_12_hour_last'),
      last_period: I18n.t('javascript.last_12_hour_last_period')
    },
    "last_24": {
      current: I18n.t('javascript.last_24_hour_current'),
      last: I18n.t('javascript.last_24_hour_last'),
      last_period: I18n.t('javascript.last_24_hour_last_period')
    },
    "last_week": {
      current: I18n.t('javascript.last_week_current'),
      last: I18n.t('javascript.last_week_last'),
      last_period: I18n.t('javascript.last_week_last_period')
    },
    "custom": {
      current: I18n.t('javascript.custom'),
      last: '',
      last_period: ''
    }
  }

  // Get the total data based on the range
  time_last_range = redBorder.widgets.util.lastRange(widget.property.range);
  time_last_day   = redBorder.widgets.util.lastDay(widget.property.range);

  if (widget.property.range == 'custom') {
    var time_filters = {}
    $.each(widget.property.filters, function (index, value) {
      if (value['column'] == 'start_time' || value['column'] == 'end_time')
        time_filters[value['column']] = value['value']
    })
    time_last_day = [time_filters['start_time'], time_filters['end_time']]
    time_last_range = [time_filters['start_time'], time_filters['end_time']]
  }

  graph = new RB.Serie({
    container: container,
    autoscaling: true,
    step: widget.graph.step,
    domain: [widget.wtype.start_time_secs, widget.wtype.end_time_secs],
    timezone: widget.graph.tzoffset,
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: redBorder.widgets.unitsHandler(widget),
    hide_x_axis: true,
    margin: {
      top: 0,
      bottom: 0,
      left: 0,
      right: 0
    },
    predict_area: false
  });

  graph_prior = new RB.Serie({
    container: "w-stat-last_" + widget.id,
    autoscaling: true,
    step: widget.graph.step,
    domain: [time_last_range[0] + stepSecs, time_last_range[1] - stepSecs],
    timezone: widget.graph.tzoffset,
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: redBorder.widgets.unitsHandler(widget),
    hide_axises: true,
    tooltip: false,
    stroke: 2,
    margin: {
      top: 5,
      bottom: 5,
      left: 0,
      right: 0
    },
    predict_area: false
  });

  graph_last_range = new RB.Serie({
    container: "w-stat-last-period_" + widget.id,
    autoscaling: true,
    step: widget.graph.step,
    domain: [time_last_day[0] + stepSecs, time_last_day[1] - stepSecs],
    timezone: widget.graph.tzoffset,
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: redBorder.widgets.unitsHandler(widget),
    hide_axises: true,
    stroke: 2,
    tooltip: false,
    margin: {
      top: 5,
      bottom: 5,
      left: 0,
      right: 0
    },
    predict_area: false
  });

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    // Save wtype to use in reload
    wtype = _wtype;
  }

  function render(){

    // Draw the name
    drawNames();

    getData();
  }

  function getData(){
    var last = wtype.queryTotal({
      start_time: time_last_range[0],
      end_time: time_last_range[1]
    });

    var last_period = wtype.queryTotal({
      start_time: time_last_day[0],
      end_time: time_last_day[1]
    });

    var actual = wtype.queryTotal()

    $.when(last, last_period, actual).then(drawTotal);

    wtype.queryTimeserie().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });

    wtype.queryTimeserie({
      start_time: time_last_day[0],
      end_time: time_last_day[1],
      granularity: "hour",
    }).then(drawLastPeriod, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });

    wtype.queryTimeserie({
      start_time: time_last_range[0],
      end_time: time_last_range[1],
      granularity: "hour"
    }).then(drawLast, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    graph.add({
      data: data,
      name: 'Total',
      color: widget.property.color,
      type: widget.property.serie_type
    });

    graph.startRender();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function drawLast(data){
    graph_prior.add({
      data: data,
      name: 'Total',
      color: "#bbbbbb",
      type: widget.property.serie_type
    });

    graph_prior.startRender();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function drawLastPeriod(data){
    if (data.length > 0) {
      data = redBorder.utils.query.fillEmptyPoints(data, (1).hours(), new Date(time_last_day[0]*1000), new Date(time_last_day[1]*1000));
      graph_last_range.add({
        data: data,
        name: 'Total',
        color: "#bbbbbb",
        type: widget.property.serie_type
      });

      graph_last_range.startRender();
    }
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  // Draws the total in the boxes in secondary container
  function drawTotal(last, lastPeriod, actual) {
    var box = $container.parent().find('.widget-secondary');
    var multiplier = redBorder.widgets.util.getMultiplier(actual);

    // Display the elements
    drawValue(box.find('.client-last'), last, actual);
    drawValue(box.find('.client-last-period'), lastPeriod, actual);

    box.find('.w-stat-actual .title').html(NAMES[widget.property.range].current);
    box.find('.w-stat-actual .value').html((Math.round(actual * multiplier.num * 100) / 100) + " " + multiplier.prefix);
  }

  function drawNames(){
    var box = $container.parent().find('.widget-secondary');
    box.find('.client-last .w-stat-title').html(NAMES[widget.property.range].last);
    box.find('.client-last-period .w-stat-title').html(NAMES[widget.property.range].last_period);
  }

  function drawValue(selector, value, compared){
    var multiplier = redBorder.widgets.util.getMultiplier(compared);
    var multiplier_last = redBorder.widgets.util.getMultiplier(value);
    var diff, multiplierDiff;

    selector.find('.value').html((Math.round(value * multiplier_last.num * 100) / 100) + " " + multiplier_last.prefix);
    diff = compared - value;
    if (compared === 0) {
      selector.find('.w-percent').html('---');
    } else {
      if (diff > 0){
        selector.find('.w-percent').html(Number((diff/compared)*100).toFixed(1) + "%");
        selector.find('.w-percent').addClass('percent-up');
      } else {
        selector.find('.w-percent').html(Number((diff/compared)*100).toFixed(1) + "%");
        selector.find('.w-percent').addClass('percent-down');
      }
    }
  }

  function reload() {
    // Delete the serie
    graph.remove('Total');

    // Update the domain
    var diffSecs = widget.wtype.end_time_secs - widget.wtype.start_time_secs,
        coef = 1000 * 60,
        nEndTime, nStartTime;
    if (widget.property.range == 'custom') {
      nStartTime = new Date(widget.wtype.start_time_secs * 1000)
      nEndTime = new Date(widget.wtype.end_time_secs * 1000)
    } else {
      nEndTime = new Date(Math.floor(redBorder.timeNow.getTime()/coef)*coef);
      nStartTime = new Date(Math.ceil((((redBorder.timeNow.getTime()/1000) - diffSecs)*1000)/coef)*coef);
    }
    graph.forceDomain([nStartTime, nEndTime], true);

    getData();
  }

  function resize() {
    var height = 98;
    if(widget.name.length !== 0){
      height += 43;
    }
    var graphWidth = parseInt($('.widget[data-widget-id="'+ widget.id+'"]').width()*0.5*0.7);

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    graph.resize();
    graph_prior.resize(graphWidth);
    graph_last_range.resize(graphWidth);
  }

  // Hide the main element when resizing
  function hide() {
    var parent = $container.parent();
    parent.find('svg, .widget-secondary').fadeOut(100);
  }

  // Show the main element
  function show() {
    var parent = $container.parent();
    parent.find('svg, .widget-secondary').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
