redBorder.widgets.graphs.clients = function (container, widget) {
  var graph,
      $container = $('#' + container),
      wtype;

      NAMES = {
        "last_5_minute": {
          current: I18n.t('javascript.last_5_minute_current'),
          last: I18n.t('javascript.last_5_minutes_last'),
          last_period: I18n.t('javascript.last_5_minutes_last_period')
        },
        "last_15_minute": {
          current: I18n.t('javascript.last_15_minute_current'),
          last: I18n.t('javascript.last_15_minute_last'),
          last_period: I18n.t('javascript.last_15_minute_last_period')
        },
        "last_30_minute": {
          current: I18n.t('javascript.last_30_minute_current'),
          last: I18n.t('javascript.last_30_minute_last'),
          last_period: I18n.t('javascript.last_30_minute_last_period')
        },
        "last_1_hour": {
          current: I18n.t('javascript.last_1_hour_current'),
          last: I18n.t('javascript.last_1_hour_last'),
          last_period: I18n.t('javascript.last_1_hour_last_period')
        },
        "last_2_hour": {
          current: I18n.t('javascript.last_2_hour_current'),
          last: I18n.t('javascript.last_2_hour_last'),
          last_period: I18n.t('javascript.last_2_hour_last_period')
        },
        "last_3_hour": {
          current: I18n.t('javascript.last_3_hour_current'),
          last: I18n.t('javascript.last_3_hour_last'),
          last_period: I18n.t('javascript.last_3_hour_last_period')
        },
        "last_12_hour": {
          current: I18n.t('javascript.last_12_hour_current'),
          last: I18n.t('javascript.last_12_hour_last'),
          last_period: I18n.t('javascript.last_12_hour_last_period')
        },
        "last_24": {
          current: I18n.t('javascript.last_24_hour_current'),
          last: I18n.t('javascript.last_24_hour_last'),
          last_period: I18n.t('javascript.last_24_hour_last_period')
        },
        "last_week": {
          current: I18n.t('javascript.last_week_current'),
          last: I18n.t('javascript.last_week_last'),
          last_period: I18n.t('javascript.last_week_last_period')
        },
        "custom": {
          current: I18n.t('javascript.custom'),
          last: '',
          last_period: ''
        }
      }

  graph = new RB.Serie({
    container: container,
    autoscaling: true,
    step: widget.graph.step,
    domain: [widget.wtype.start_time_secs, widget.wtype.end_time_secs],
    timezone: widget.graph.tzoffset,
    margin: { top: 0, left: 0, right: 0, bottom: 10 },
    zoomable: false,
    autorender: false,
    shared_colors: true,
    units: redBorder.widgets.unitsHandler(widget),
    hide_x_axis: true,
    margin: {
      bottom: 0,
      left: 0,
      right: 0
    },
    predict_area: false,
    discrete_y_axis: true
  });

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    // Save wtype to use in reload
    wtype = _wtype;
  }

  function render(){
    // Draw the name
    drawNames();

    // Get and draw the data
    getData();
  }

  function getData(){
    var lastRange = redBorder.widgets.util.lastRange(widget.property.range),
      lastDay = redBorder.widgets.util.lastDay(widget.property.range),
      last,
      lastPeriod,
      actual;

    if (widget.property.range == 'custom') {
      var time_filters = {}
      $.each(widget.property.filters, function (index, value) {
        if (value['column'] == 'start_time' || value['column'] == 'end_time')
          time_filters[value['column']] = value['value']
      })
      lastDay = [time_filters['start_time'], time_filters['end_time']]
      lastRange = [time_filters['start_time'], time_filters['end_time']]
    }

    last = wtype.queryTotal({
      start_time: lastRange[0],
      end_time: lastRange[1]
    });

    lastPeriod = wtype.queryTotal({
      start_time: lastDay[0],
      end_time: lastDay[1]
    });

    actual = wtype.queryTotal();

    $.when(last, lastPeriod, actual).then(drawTotal);

    wtype.queryTimeserie().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    graph.add({
      data: data,
      name: 'Total',
      color: widget.property.color,
      type: widget.property.serie_type
    });

    graph.startRender();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  // Draws the total in the boxes in secondary container
  function drawTotal(last, lastPeriod, actual) {
    var box = $container.parent().find('.widget-secondary');
    var multiplier = redBorder.widgets.util.getMultiplier(actual);

    // Display the elements
    drawValue(box.find('.client-last'), last, actual);
    drawValue(box.find('.client-last-period'), lastPeriod, actual);

    box.find('.w-stat-actual .title').html(NAMES[widget.property.range].current);
    box.find('.w-stat-actual .value').html((Math.round(actual * multiplier.num * 100) / 100) + " " + multiplier.prefix);
  }

  function drawNames(){
    var box = $container.parent().find('.widget-secondary');
    box.find('.client-last .w-stat-title').html(NAMES[widget.property.range].last);
    box.find('.client-last-period .w-stat-title').html(NAMES[widget.property.range].last_period);
  }

  function drawValue(selector, value, compared){
    var multiplier = redBorder.widgets.util.getMultiplier(compared);
    var multiplier_last = redBorder.widgets.util.getMultiplier(value);
    var diff, multiplierDiff;

    selector.find('.value').html((Math.round(value * multiplier_last.num * 100) / 100) + " " + multiplier_last.prefix);
    diff = compared - value;
    if (diff == 0){
      text = "0.0%";
      css = 'percent-eq';
    } else if (diff > 0){
      text = "+" + Number((diff/compared)*100).toFixed(1) + "%";
      css = 'percent-up';
    } else {
      text = Number((diff/compared)*100).toFixed(1) + "%";
      css = 'percent-down';
    }

    selector.find('.w-percent').html(text).addClass(css);
  }

  function reload() {
    // Delete the serie
    graph.remove('Total');

    // Update the domain
    var diffSecs = widget.wtype.end_time_secs - widget.wtype.start_time_secs,
        coef = 1000 * 60,
        nEndTime, nStartTime;
    if (widget.property.range == 'custom') {
      nStartTime = new Date(widget.wtype.start_time_secs * 1000)
      nEndTime = new Date(widget.wtype.end_time_secs * 1000)
    } else {
      nEndTime = new Date(Math.floor(redBorder.timeNow.getTime()/coef)*coef);
      nStartTime = new Date(Math.ceil((((redBorder.timeNow.getTime()/1000) - diffSecs)*1000)/coef)*coef);
    }
    graph.forceDomain([nStartTime, nEndTime], true);

    getData();
  }

  function resize() {
    var height = 85;
    if(widget.name.length !== 0){
      height += 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    graph.resize();
  }

  // Hide the main element when resizing
  function hide() {
    var parent = $container.parent();
    parent.find('svg, .widget-secondary').fadeOut(100);
  }

  // Show the main element
  function show() {
    var parent = $container.parent();
    parent.find('svg, .widget-secondary').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    reload: reload,
    render: render
  }
}
