redBorder.widgets.graphs.clients_map = function (container, widget, aps) {
  var graph,
      wtype,
      $container = $('#' + container),
      $tooltip,
      force_max;

  force_max = parseInt(widget.property.force_max);

  if(isNaN(force_max)){
    force_max = undefined;
  }

  // Timeout to wait gridster ready
  setTimeout(load_map, 350);

  // The widget manager calls this method when the graph should start
  function init(_wtype) {
    // Save wtype to use in reload
    wtype = _wtype;
  }

  function render(){
    getData();
  }

  function getData(){
    wtype.queryMultidata().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  // Load the map.
  function load_map(){
    var heatmap = redBorder.heatMap({
      container: container,
      aggregation: widget.property.aggregation,
      clustered: (widget.property.cluster_aps == "true" ? true : false),
      searchBox: false,
      scrollwheel: false,
      disableDoubleClickZoom: true,
      mirrorControls: true,
      radius: 35,
      calculator: sum_clients,
      type: widget.property.map_type,
      force_max: force_max
    });

    graph = redBorder.infrastructureMap(
      $('#mapa .main_container'),
      $("#widget_placeholder_" + widget.id),
      undefined,
      heatmap,
      {
        heatmap: true,
        bindEvents: false
      }
    );
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(data) {
    graph.getRbHeatmap().addData(data.heatmap);
    graph.addAccessPoints(data.access_points);
    graph.addDomains(data.domains);
    graph.getRbMap().center();

    // Add the data
    var box = $container.parent().find('.widget-secondary');

    drawValueAndTotal(box.find('.client-associated'), data.clients.associated, data.total_clients, true);
    drawValueAndTotal(box.find('.client-probing'), data.total_clients, (data.clients.probing + data.clients.associated), true);
    drawValueAndTotal(box.find('.aps'), data.aps, data.total_access_points, true);
    drawBps(box.find('.bandwidth'), (data.bps*8)/(300));

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function drawValueAndTotal(selector, value, total, compared){
    var multiplier = redBorder.widgets.util.getMultiplier(value);
    var multiplierTotal = redBorder.widgets.util.getMultiplier(total);

    selector.find('.value').html((Math.round(value * multiplier.num * 100) / 100) + " " + multiplier.prefix + " / " + (Math.round(total * multiplierTotal.num * 100) / 100) + " " + multiplierTotal.prefix);

    if (compared == true){
      if (total == 0 && value == 0){
        text = "0%"
        css = 'percent-eq';
      } else if(total == 0){
        text = "100%"
        css = 'percent-up';
      } else if (value > total) {
        text = Number((value/total)*100).toFixed(1) + "%"
        css = 'percent-up';
      } else {
        text = Number((value/total)*100).toFixed(1) + "%"
        css = 'percent-down';
      }

      selector.find('.w-percent').html(text).addClass(css);
    }
  }

  function drawTotalAndPercent(selector, value, total){
    var multiplier = redBorder.widgets.util.getMultiplier(total);

    selector.find('.value').html((Math.round(total * multiplier.num * 100) / 100) + " " + multiplier.prefix);

    if (total == 0 && value == 0){
      selector.find('.w-percent').html("0%");
      selector.find('.w-percent').addClass('percent-eq');
    } else if(total == 0){
      selector.find('.w-percent').html("100%");
      selector.find('.w-percent').addClass('percent-up');
    } else if (value > total){
      selector.find('.w-percent').html(Number(((value)/total)*100).toFixed(1) + "%");
      selector.find('.w-percent').addClass('percent-up');
    } else {
      selector.find('.w-percent').html(Number(((value)/total)*100).toFixed(1) + "%");
      selector.find('.w-percent').addClass('percent-down');
    }
  }

  function drawBps(selector, value){
    var multiplier = redBorder.widgets.util.getMultiplier(value);

    if (value == 0){
      selector.find('.value').html('0 bps');
    } else {
      selector.find('.value').html((Math.round(value * multiplier.num * 100) / 100) + " " + multiplier.prefix + "bps");
    }
  }

  function cleanAps() {
    $container.find('svg').remove();
  }

  function reload(){
    wtype.queryMultidata().then(function(data){
      graph.removeAccessPoints();
      graph.addAccessPoints(data.access_points, true);
      graph.getRbHeatmap().reloadData(data.heatmap, true);

      // Add the data
      var box = $container.parent().find('.widget-secondary');

      drawValueAndTotal(box.find('.client-associated'), data.clients.associated, data.total_clients, true);
      drawValueAndTotal(box.find('.client-probing'), data.total_clients, (data.clients.probing + data.clients.associated), true);
      drawValueAndTotal(box.find('.aps'), data.aps, data.total_access_points, false);
      drawBps(box.find('.bandwidth'), data.bps/(60*5));

      redBorder.widgets.util.reloadDone($container.parents('.widget'));
    }, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    })
  }

  function resize() {
    height = 70;
    if(widget.name.length !== 0){
      height += 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    // We need to call show before resize to correct visualization of the map
    this.show();
    graph.resize();
  }

  // Hide the main element when resizing
  function hide() {
    var parent = $container.parent();
    parent.find('.widget-main, .widget-secondary').fadeOut(100);
  }

  // Show the main element
  function show() {
    var parent = $container.parent();
    parent.find('.widget-main, .widget-secondary').fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
