redBorder.widgets.graphs.floor_map = function (container, widget) {
  var graph,
      wtype,
      access_points = [],
      $container = $('#' + container),
      $image = $container.find('img'),
      $tooltip,
      dimensions = $container.data('rb-aps-dimensions'),
      image_width_feet = dimensions[0],
      image_height_feet = dimensions[1],
      keep_aspect = false,
      image_height_px,
      image_width_px,
      radius = 15,
      ref = -1,
      palette = ['#90A860', '#789937', '#486018', '#304800', '#183000'];

  if (widget.property.keep_ratio === "1"){
    keep_aspect = true;
  }

  function calculateReference(){
    access_points.forEach(function (d) {
      ref = (d.clients > ref) ? d.clients : ref;
    });

    if (ref > 0){
      ref = (ref + 1)/palette.length;
    }
  }

  // The widget manager calls this method when the graph should start
  function init(wtype) { }

  function render(){
    getAccessPoints().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  function getAccessPoints(){
    return $.ajax({
      url: '/sensors/' + widget.property.floor_id + '/access_points',
      type: 'GET',
      dataType: 'json'
    });
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw(aps) {

    access_points = aps.aps;
    calculateReference();

    if (keep_aspect){
      calculateAspectRatio();
    } else {
      image_height_px = $container.height();
      image_width_px = $container.width();
    }

    // Build the tooltip
    $tooltip = $('<div class="rb-aps-tooltip"></div>');
    $tooltip.hide();
    $container.append($tooltip);

    // Check the feet conversion
    if (image_width_feet == 0 || image_height_feet == 0) {
      image_width_feet = image_width_px;
      image_height_feet = image_height_px;
    }

    if (access_points.length != 0) {
      createElements();
      bindEvents();
    }

    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function createElements() {
    
    if (image_width_px > 1200) {
      radius = 8;
    } else if (image_width_px > 900) {
      radius = 6;
    } else if (image_width_px > 700) {
      radius = 4;
    } else {
      radius = 2;
    }

    for (var i = 0; i < access_points.length; i++) {
        if(access_points[i].latitude != undefined && access_points[i].longitude){
          var x = access_points[i].x * image_width_px;
          var y = access_points[i].y * image_height_px;
          appendApMarker(access_points[i], x, y);
        }
    }
}

  function appendApMarker(ap, x, y) {

    y = (y < 15) ? 15 : y;
    x = (x < 15) ? 15 : x;

    var size = (radius*2 + 4),
        pos  = (radius*2 + 4)/2;
        x_text = 4,
        font_size = 16;

    if (ap.clients == 0){
      var clients_text = '';
    } else {
      var clients_text = ap.clients;
    }

    if (ap.clients > 100) {
      font_size = 13;
      x_text = 11;
    } else if(ap.clients > 10){
      x_text = 8;
    }

    if(ap.clients > 0 || ap.status === 1){
      // Change the color based on the number of Clients
      var color = palette[parseInt(ap.clients/ref)];
    } else if(ap.status === 0) {
      var color = '#919191';
    } else {
      var color = '#d80808';
    }

    var marker = $(' \
      <svg height="'+size+'" width="'+size+'" class="rb-aps zone-ap" data-rb-aps-id="' + ap.id + '" \
        style="position: absolute; top: ' + (y - pos) + 'px; left: ' + (x - pos) + 'px" \
        data-rb-aps-name="' + ap.name + '" > \
        <circle cx="'+pos+'" cy="'+pos+'" r="'+ radius +'" class="rb-aps-ap" data-rb-status="' + ap.status + '" style="fill: '+ color +'" /> \
        <text fill="#fff" x="'+(pos - x_text)+'" y="'+(pos + 5)+'" font-size="' + font_size + '">'+ clients_text +'</text> \
      </svg> \
    ');

    $('circle', marker).data('data', ap);
    $container.append(marker);
  }

  function bindEvents(){
    $container.on('mouseenter', 'svg.rb-aps', function (e) {
      var image_offset = $image.offset(),
          container_offset = $container.offset(),
          circle_position = $(this).offset(),
          xpos = circle_position.left - image_offset.left,
          ypos = circle_position.top - image_offset.top,
          xmenu = circle_position.left - container_offset.left,
          ymenu = circle_position.top - container_offset.top,
          element_data = $(this).find('circle').data('data');

      $tooltip.css('top', ymenu - 25);
      $tooltip.css('left', xmenu + 40);

      fillApsTooltip(element_data);

      $tooltip.show();
    });

    $container.on('mouseleave', 'svg.rb-aps', function (e) {
      $tooltip.hide();
    })
  }

  function fillApsTooltip(element) {
    var last_check_in, tooltip;

    tooltip = $('<div class="rb-aps-tooltip-field">Name: <span>' + element.name + '</span></div>' +
      '<div class="rb-aps-tooltip-field">IP: <span>' + element.ip_address + '</span></div>' +
      '<div class="rb-aps-tooltip-field">MAC: <span>' + element.mac_address + '</span></div>');

    $tooltip.html(tooltip);

    if (element.last_check_in != undefined && element.last_check_in != null) {
      last_check_in = new Date(element.last_check_in).toLocaleString();
      $tooltip.append('<div class="rb-aps-tooltip-field">' +
        I18n.t('javascript.last_check') + ' <span>' + last_check_in + '</span>' +
        '</div>');
    }
  }

  function cleanAps() {
    $container.find('svg').remove();
  }

  function reload(){
    cleanAps();
    getAccessPoints().then(draw, function () {
      redBorder.widgets.util.reloadFail($container.parents('.widget'));
    });
  }

  function calculateAspectRatio(){
    var $img = $container.find('img');
    var height_c = $container.height();
    var width_c = $container.width();
    var height_r = $img[0].naturalHeight;
    var width_r = $img[0].naturalWidth;
    // Get the ratios
    var rc = height_c/width_c;
    var rr = height_r/width_r;

    // Clear classes of container
    $container.removeClass('keep_aspect_height');
    $container.removeClass('keep_aspect_width');
    $container.css('margin-top', '0');
    $container.css('width', 'inherit');

    if(rr > rc){
      // The limit is on height
      image_height_px = height_c;
      image_width_px = height_c/rr;
      $container.addClass('keep_aspect_height');
      $container.css('width', image_width_px);
    } else if(rr < rc){
      // The limit is on width
      image_width_px = width_c;
      image_height_px = parseInt(width_c*rr);
      $container.addClass('keep_aspect_width');
      // We must center the img
      $container.css('margin-top', ((height_c - image_height_px)/2) + 'px' );
    } else {
      // Same aspect ratio
      image_height_px = $container.height();
      image_width_px = $container.width();
    }

  }

  function resize() {
    height = 0;
    if(widget.name.length !== 0){
      height = 43;
    }

    $container.css('height', ($container.parents('.widget').height() - height) + 'px');
    $container.css('width', ($container.parents('.widget').width()) + 'px');

    if (keep_aspect){
      calculateAspectRatio();
    } else {
      image_height_px = $container.height();
      image_width_px = $container.width();
    }

    if (access_points.length != 0) {
      cleanAps();
      createElements();
      bindEvents();
    }
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
