redBorder.widgets.graphs.hardwarestatus = function (container, widget) {
  var $container = $('#' + container),
      ALIGNMENT_LEFT = "1",
      ALIGNMENT_CENTER = "2",
      ALIGNMENT_RIGHT = "3",
      ALIGNMENT_JUSTIFY = "4",
      $image = $container.find('img'),
      keep_aspect = false,
      wtype,
      graph;

   if (widget.property.keep_aspect_ratio === "1"){
    keep_aspect = true;
   }

  // Entry point for the graph
  function init(wtype) {
    if (keep_aspect){
      calculateAspectRatio();
    }
    draw();
  }

  function render() {
    if (keep_aspect){
      calculateAspectRatio();
    }
    $container.find('img').load(this.draw);
    getData();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function getData(){
    if (widget['property']['status'] == false) {
      $container.parents('.widget').addClass("alert-warning alert-white rounded");
      $container.find("i").removeClass("fa-check");
      $container.find("i").css("color","white");
      $container.find("i").addClass("fa-warning");
    }
    else if (widget['property']['status'] == true) {
      $container.parents('.widget').addClass("alert-success alert-white rounded");
      $container.find("i").removeClass("fa-times-circle");
      $container.find("i").addClass("fa-check");
    }
    else {
      $container.parents('.widget').addClass("alert-warning alert-white rounded");
      $container.find("i").removeClass("fa-check");
      $container.find("i").addClass("fa-warning");
      $container.find("h4").text("No sensor found").css( "font-size", "24px" );
    }
  }

  // Draws the widget. This data comes directly from the widget
  // type selected. You should prepare the data to be used if needed.
  function draw() {
    if (keep_aspect){
      calculateAspectRatio();
    }

    var font_size = widget.property.font_size,
        alignment = widget.property.text_alignment;
    var title = $container.find('h4');

    title.css('font-size', font_size);
    title.css('line-height', $container.height() - 1 + 'px');

    switch(alignment){
      case ALIGNMENT_LEFT:
        title.css('text-align', 'left');
        break;
      case ALIGNMENT_CENTER:
        title.css('text-align', 'center');
        break;
      case ALIGNMENT_RIGHT:
        title.css('text-align', 'right');
        break;
      case ALIGNMENT_JUSTIFY:
        title.css('text-align', 'justify');
        break;
    }
  }

  function reload() {
    getData();
    calculateAspectRatio();
    redBorder.widgets.util.reloadDone($container.parents('.widget'));
  }

  function calculateAspectRatio(){
    var height_c = $container.height();
    var width_c = $container.width();
    var height_r = $image[0].naturalHeight;
    var width_r = $image[0].naturalWidth;
    // Get the ratios
    var rc = height_c/width_c;
    var rr = height_r/width_r;

    // Clear classes of container
    $container.removeClass('keep_aspect_height');
    $container.removeClass('keep_aspect_width');
    $image.css('margin-left', '45px');

    if(rr > rc){
      // The limit is on height
      $container.addClass('keep_aspect_height');
    } else if(rr < rc){
      // The limit is on width
      $container.addClass('keep_aspect_width');
      // We must center the img
    }

    var height = ($container.height()/2) - ($container.find('img')[0].height/2);
    if (height < 40) { height = 40 }
    $image.css('margin-top', (height) + 'px' );

  }

  function resize() {
    $container.css('height', $container.parents('.widget').height() + 'px');
    calculateAspectRatio();
  }

  // Hide the main element when resizing
  function hide() {
    $container.fadeOut(100);
  }

  // Show the main element
  function show() {
    $container.fadeIn(100);
  }

  // Returns the graph object for debug purposes
  function getGraph() {
    return graph;
  }

  // Returns the widget
  function getWidget() {
    return widget;
  }

  return {
    init: init,
    resize: resize,
    getGraph: getGraph,
    getWidget: getWidget,
    hide: hide,
    show: show,
    draw: draw,
    reload: reload,
    render: render
  }
}
