/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

/*
 * Overview Channels widget
 */
redBorder.overview.widgets.channels_load = function(container, opts) {
  var $container = $(container),
      sensor_id = opts.sensor_id,
      range = opts.range,
      url,
      sensor_name = opts.sensor_name,
      sensor_uuid = opts.sensor_uuid,
      sensor_type = opts.sensor_type,
      predefined_filters = (opts.predefined_filters) || [],
      root = opts.root,
      start_time_secs,
      end_time_secs,
      step,
      reloaded = false,
      channelsName = { '2_4': '2.4GHz', '5': '5GHz' };

  initialize();

  /*
   * Initialize some data like URL
   */
  function initialize() {
    // Create the URL
    url = '/flow/channels_load';
    // Add containers
    appendContainers();
  }

  /*
   * Create containers needed to show the data
   */
  function appendContainers() {
    $container.addClass('owidget-channels');
    var html = channelBlock('2_4') +
               channelBlock('5');
    $container.append('<div class="clearfix">' + html + '</div>');
  }

  function channelBlock(channel) {
    return '<div class="text-center owidget-channels-block owidget-channels-' + channel + '">' +
             '<div class="owidget-channel-value" data-color="green"> - </div>' +
             '<p class="owidget-channel-name">' + channelsName[channel] + '</p>' +
           '</div>';
  }

  /*
   * Call getData and Draw methods
   */
  function render() {
    getData().then(draw, function(){
      console.log('Error retrieving data');
    });
  }

  /*
   * Fetch data from the Manager. This method return a promise object
   */
  function getData() {
    var filters = redBorder.overview.widgets.utils.getFilters(
      root, sensor_name, sensor_uuid, sensor_type, predefined_filters);
    end_time_secs = redBorder.overview.widgets.utils.getEndTimeSeconds(range, step);
    start_time_secs = redBorder.overview.widgets.utils.getStartTimeSeconds(range, step)
    return $.ajax({
      url: url,
      type: 'GET',
      dataType: 'json',
      data: {
        end_time: end_time_secs,
        start_time: start_time_secs,
        w: true,
        filters: JSON.stringify(filters)
      }
    });
  }

  /*
   * Draw the widget
   */
  function draw(data) {
    var channels = ['2_4', '5'];
    // Iterate over status to get the aps
    for(var i = 0; i < channels.length; i++) {
      var $block = $container.find('.owidget-channels-' + channels[i]);
      var value = data.channels_load[channels[i]];

      if(value != null){
        $block.find('.owidget-channel-value').html(value + '%');

        if(value > 70) {
          $block.find('.owidget-channel-value').attr('data-color', 'red');
        } else {
          $block.find('.owidget-channel-value').attr('data-color', 'green');
        }
      } else {
        $block.find('.owidget-channel-value').html('-');
        $block.find('.owidget-channel-value').attr('data-color', 'black');
      }
    }

    if (reloaded) { redBorder.overview.reloaded(); }
  }

  /*
   * Reload the widget
   */
  function reload() {
    reloaded = true;
    render();
  }

  /*
   * Update sensor to filters
   */
  function updateSensor(id, name, type_name, uuid) {
    sensor_name = name;
    sensor_type = type_name;
    sensor_uuid = uuid;
  }

  return {
    render: render,
    reload: reload,
    updateSensor: updateSensor
  }
}

/*
 * Overview AP widget
 */

redBorder.overview.widgets.aps = function(container, opts) {
  var $container = $(container),
      sensor_id = opts.sensor_id,
      url,
      reloaded = false,
      apStatusName = { 0: 'unknown', 1: 'up', 2: 'down' };

  initialize();

  /*
   * Initialize some data like URL
   */
  function initialize() {
    // Create the URL
    url = '/sensors/' + sensor_id + '/access_points_status';
    // Add containers
    appendContainers();
  }

  /*
   * Create containers needed to show the data
   */
  function appendContainers() {
    $container.addClass('owidget-aps');
    var html = apsBlock('up') +
               apsBlock('down');
    $container.append('<div class="clearfix">' + html + '</div>');
  }

  function apsBlock(status) {
    return '<div class="text-center owidget-aps-block owidget-aps-' + status + '">' +
             '<p class="owidget-aps-count"></p>' +
             '<div class="owidget-aps-status-color"></div>' +
           '</div>';
  }

  /*
   * Call getData and Draw methods
   */
  function render() {
    getData().then(draw, function(){
      console.log('Error retrieving data');
    });
  }

  /*
   * Fetch data from the Manager. This method return a promise object
   */
  function getData() {
    return $.ajax({
      url: url,
      type: 'GET',
      dataType: 'json',
      data: {
        w: true
      }
    });
  }

  /*
   * Draw the widget
   */
  function draw(data) {
    var result = data.status,
        status = ['up', 'down'];

    // Iterate over status to get the aps
    for(var i = 0; i < status.length; i++) {
      var $block = $container.find('.owidget-aps-' + status[i]);
      $block.find('.owidget-aps-count').html(result[status[i]]);
    }

    if (reloaded) { redBorder.overview.reloaded(); }
  }

  /*
   * Reload the widget
   */
  function reload() {
    reloaded = true;
    render();
  }

  /*
   * Update sensor to filter
   */
  function updateSensor(id, name, type_name, uuid) {
    url = '/sensors/' + id + '/access_points_status';
  }

  /*
   * Count the number of APs by Status
   */
  function apsByStatus(aps) {
    var apsCount = {
      up: 0,
      down: 0
    }

    for(var i = 0; i < aps.length; i++){
      var status = aps[i].status;

      if (status === null || status === undefined || status === 'unknown') {
        apsCount['down'] += 1;
      } else {
        apsCount[apStatusName[status]] += 1;
      }
    }
    // Return count
    return apsCount;
  }

  return {
    render: render,
    reload: reload,
    updateSensor: updateSensor
  }
}

/*
 * Overview Flow quality widget
 */

redBorder.overview.widgets.flowQuality = function(container, opts) {
  var $container = $(container),
      $moreinfo = $(container).find('.owidget-more'),
      range = opts.range,
      module_name = opts.module_name,
      sensor_name = opts.sensor_name,
      sensor_uuid = opts.sensor_uuid,
      sensor_type = opts.sensor_type,
      ignore_domain_types = opts.ignore_domain_types || [],
      more_info_url = opts.more_info_url,
      more_info_filters = (opts.more_info_filters === true || opts.more_info_filters === undefined),
      root = opts.root,
      groups = ["good", "average", "bad"],
      percent_groups = ["average", "bad"],
      reloaded = false,
      url,
      step;

  initialize();

  /*
   * Create the structure of the widget
   */
  function initialize() {
    // Initialize URL
    url = '/flow/tops/get_tops/client_rssi';

    if ($moreinfo.length > 0 && more_info_url !== undefined) {
      // Update more info URL
      $moreinfo.attr('href',
        redBorder.overview.widgets.utils.getMoreInfoURL(
          more_info_url, root, range, 'clients', more_info_filters,
          sensor_name, sensor_uuid, sensor_type));
    } else if ($moreinfo.length > 0) {
      $moreinfo.remove();
    }

    // Load step
    step = redBorder.overview.widgets.utils.getStep(range);

    // Create containers
    appendContainers();
  }

  /*
   * Create three containers
   */
  function appendContainers() {
    $container.addClass('owidget-fquality');
    var html = qualityBlock('good') +
               qualityBlock('average') +
               qualityBlock('bad');
    $container.append('<div class="clearfix">' + html + '</div>');
  }

  /*
   * Container for every type
   */
  function qualityBlock(type) {
    var title = type[0].toUpperCase() + type.slice(1);
    return '<div class="text-center owidget-fquality-block owidget-fquality-' + type + '">' +
             '<h2>' + title + '</h2>' +
             '<div class="owidget-fquality-value">-</div>' +
             '<div class="owidget-fquality-graph"><div class="owidget-fquality-fill"></div></div>' +
           '</div>';
  }

  /*
   * Call getData and Draw methods
   */
  function render() {
    if (redBorder.overview.widgets.utils.isIgnored(
        sensor_name, sensor_type, ignore_domain_types)) {
      draw(defaultData());
    } else {
      getData().then(draw, function(){
        if (reloaded) { redBorder.overview.reloaded(); }
        console.log('Error retrieving data');
      });
    }
  }

  /*
   * Fetch data from the Manager. This method return a promise object
   */
  function getData() {
    var filters = redBorder.overview.widgets.utils.getFilters(
      opts.root, sensor_name, sensor_uuid, sensor_type);
    // Set the data
    return $.ajax({
      url: url,
      type: 'GET',
      dataType: 'json',
      data: {
        granularity: 'all',
        aggregation: 'bytes',
        filters: JSON.stringify(filters),
        w: true,
        end_time: redBorder.overview.widgets.utils.getEndTimeSeconds(range, step),
        start_time: redBorder.overview.widgets.utils.getStartTimeSeconds(range, step)
      }
    }).then(function(data){
      return $.Deferred().resolve(groupData(data));
    });
  }

  /*
   * Group data based on client RSSI
   */
  function groupData(data) {
    // Data must be an array
    data = (data instanceof Array) ? data : [];

    var group = defaultData();

    for(var i = 0, l = data.length; i < l; i++) {
      var value = data[i];
      switch (value.client_rssi) {
        case "good":
          group.good += value.bytes
          break;
        case "excellent":
          group.good += value.bytes
          break;
        case "low":
          group.average += value.bytes
          break;
        case "medium":
          group.average += value.bytes
          break;
        case "bad":
          group.bad += value.bytes
          break;
      }
    }

    // Get percents
    var total = group.good + group.average + group.bad;

    for(var i = 0, l = percent_groups.length; i < l; i++) {
      var type = percent_groups[i];
      if (group[type] > 0) {
        group[type] = parseInt(Math.round((group[type] / total) * 100));
      }
    }

    if (group.average > 0 || group.bad > 0) {
      group.good = 100 - group.average - group.bad;
    } else if (group.good > 0) {
      group.good = 100;
    }

    return group;
  }

  function draw(data) {
    // Calc total
    var total = data.good + data.average + data.bad;

    for(var i = 0, l = groups.length; i < l; i++) {
      var group = groups[i];
      updateQualityBlock(group, data[group]);
    }

    // Update reload count
    if (reloaded) { redBorder.overview.reloaded(); }
  }

  function updateQualityBlock(group, percent) {
    var $block = $container.find('.owidget-fquality-' + group);
    $block.find('.owidget-fquality-value').html(percent + ' %');
    $block.find('.owidget-fquality-fill').attr('style', 'width: ' + percent + '%');
  }

  /*
   * Reload the widget
   */
  function reload() {
    reloaded = true;
    render();
  }

  /*
   * Default data
   */
  function defaultData() {
    return {
      good: 0,
      average: 0,
      bad: 0
    }
  }

  /*
   * Update sensor to filters
   */
  function updateSensor(id, name, type_name, uuid) {
    sensor_name = name;
    sensor_type = type_name;
    sensor_uuid = uuid;
  }

  // Return
  return {
    render: render,
    reload: reload,
    updateSensor: updateSensor
  }
}
