#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Type for the channel load widget
#
class WidgetType::ChannelLoad < WidgetType::Base
  graphs [:multiserie, :load]
  filters true
  widget_fields [:range]

  widget_after_initialize do
    self.product_type = 'flow'
    # This is not the real aggregation the real aggregation
    # is :load, which is hardcoded in access_point_druid.rb.
    # :quality is set here as the aggregation because we
    # want relative units (%, which :quality provides) and :load
    # is not declared on the flow module, so we can not access it.
    # I.e: We set the :quality aggregation to show % units on the graph
    self.aggregation = :quality
  end

  def load(user = @widget.parent.owner, options = {})
    source = Redborder::DruidSource.get(source: 'flow')
    filter = Redborder::Filter.new(source)
    options.merge!({
                     start_time: start_time,
                     end_time: end_time,
                     range: @widget.range,
                     aggregation: @widget.aggregation,
                     user: user
                  }) { |k, v1, v2| v1 }
    channels_load = ApState.channels_load(filter, options)
    {
      '2.4Ghz' => channels_load['2_4'] || {},
      '5.0Ghz' => channels_load['5'] || {}
    }
  end

  def list(user = @widget.parent.owner, options = {})
    titles = { '2_4' => '2.4Ghz', '5' => '5.0Ghz' }     # To prettify titles
    list = []
    ['2_4', '5'].each do |channel|
      list << {
        name: titles[channel],
        opts: channel
      }
    end
    { list: list}
  end

  def timeseries(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity
    ovrr_range = options[:range] || @widget.range
    ignore_user_namespaces = options[:include_all_namespaces] || false

    elements = list(user, options)
    # Init timeseries
    timeseries = {}
    filter_hash = @widget.fetch_filters
    search = Redborder::Filter.new(ApState, filter_hash)

    elements[:list].each do |e|
      opts = {
        include_all_namespaces: ignore_user_namespaces,
        granularity: ovrr_granularity,
        user: user,
        start_time: ovrr_start_time,
        end_time: ovrr_end_time,
        range: ovrr_range,
        channel: e[:opts]
      }
      timeseries[e[:opts]] = ApState.channel_load_serie(search, opts)
    end

    {
      timeserie: timeseries
    }
  end

  def total_timeserie(user = @widget.parent.owner, options = {})
    series = timeseries(user, options)

    total = []
    series[:timeserie].each do |_, serie|
      serie.each_with_index do |point, i|
        events_for_i = total[i][:event][:events] rescue 0
        total[i] = {
          event: { events: events_for_i + point[:event][:events] },
          timestamp: point[:timestamp]
        }
      end
    end
    {
      total: total
    }
  end
end