#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Define a zone. A zone is a space with multiple access points
# associated. It can has access points from differents domains.
#
# Every zone has a parent domain associated and it's showed
# into the children of parent domain.
#
# @module    Flow
# @version   1.0
#
class Zone < ApplicationRecord
  # -------------------------------------------------------------
  # RELATIONS
  # -------------------------------------------------------------

  # Parent domain
  belongs_to :sensor
  has_and_belongs_to_many :access_points

  # -------------------------------------------------------------
  # Validations
  # -------------------------------------------------------------

  validates :sensor_id, :name, presence: true

  # -------------------------------------------------------------
  # METHODS
  # -------------------------------------------------------------

  before_update :remove_access_points_relation, if: :valid? && :sensor_id_changed?

  #
  # Check if there are some access_points with
  # problems in this zone. If there are a problem,
  # this method return true.
  #
  def down_access_points?
    (access_points.where(status: 2).count > 0)
  end

  def uuid
    id
  end

  def as_json(options = {})
    result = super(options)
    result['mac_address'] = access_points.map{ |acc| acc.mac_address}
    result
  end

  private

  def remove_access_points_relation
    return if sensor.is_ancestor_of?(Sensor.find(sensor_id_was))
    # Update Zone APs that doesn't belong to new sensor
    access_points
      .delete(access_points.where.not(id: (access_points & sensor.access_points).map(&:id)))
  end
end
