#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

RedBorder::Application.routes.draw do
  resources :sensors, except: :show do
    [:flow, :mse, :meraki, :ale, :arubacentral, :device].each do |t|
      get t, on: :collection
      get t, on: :member
    end
    get :image, on: :member
    get :logo, on: :member
    patch :remove_image, on: :member
    patch :remove_logo, on: :member
    get :new_flow, on: :collection
    get :new_mse, on: :collection
    get :new_ale, on: :collection
    get :new_arubacentral, on: :collection
    get :new_device, on: :collection
    get :new_meraki, on: :collection
    post :create_flow, on: :collection
    post :create_mse, on: :collection
    post :create_ale, on: :collection
    post :create_arubacentral, on: :collection
    post :create_device, on: :collection
    post :create_meraki, on: :collection
    get 'flow/:id/search',
        action: :flow_letter_search, on: :collection, as: :flow_letter_search
    get 'flow/search', action: :flow_search, on: :collection
    get 'flow/load_more', action: :flow_load_more, on: :collection
    get 'mse/:id/search',
        action: :mse_letter_search, on: :collection, as: :mse_letter_search
    get 'mse/search', action: :mse_search, on: :collection
    get 'mse/load_more', action: :mse_load_more, on: :collection
    get 'ale/:id/search',
        action: :ale_letter_search, on: :collection, as: :ale_letter_search
    get 'ale/search', action: :ale_search, on: :collection
    get 'ale/load_more', action: :ale_load_more, on: :collection
    get 'arubacentral/:id/search',
        action: :arubacentral_letter_search, on: :collection, as: :arubacentral_letter_search
    get 'arubacentral/search', action: :arubacentral_search, on: :collection
    get 'arubacentral/load_more', action: :arubacentral_load_more, on: :collection
    get 'device/:id/search',
        action: :device_letter_search, on: :collection, as: :device_letter_search
    get 'device/search', action: :device_search, on: :collection
    get 'device/load_more', action: :device_load_more, on: :collection
    get 'meraki/:id/search',
        action: :meraki_letter_search, on: :collection, as: :meraki_letter_search
    get 'meraki/search', action: :meraki_search, on: :collection
    get 'meraki/load_more', action: :meraki_load_more, on: :collection
    get :new_home_network, on: :member
    post :home_network, on: :member, to: 'sensors#create_home_network'
    delete :home_network, on: :member, to: 'sensors#delete_home_network'
    get :new_router_mac, on: :member
    post :router_mac, on: :member, to: 'sensors#create_router_mac'
    delete :router_mac, on: :member, to: 'sensors#delete_router_mac'
    post :scan_aps, on: :member
    get :access_points, on: :member
    get :access_points_status, on: :member
    get :overview_aps, on: :member
    get :import_aps, on: :member
    post :import_aps, on: :member
    get :export_aps, on: :member
    get :load_more_aps, on: :member
    get :search_aps, on: :member
    get :search_gps_markers, on: :member
    get :geopositionate_aps, on: :member
    get '3d/:id',
        to: 'sensors#three_dimensions_map',
        as: :three_dimensions_map, on: :collection
    get 'access_point/:id',
        to: 'sensors#edit_access_point',
        as: :edit_access_point, on: :collection
    post 'access_point/:id',
         to: 'sensors#update_access_point',
         as: :update_access_point, on: :collection
    delete 'access_point/:id',
           to: 'sensors#delete_access_point',
           as: :delete_access_point, on: :collection
    delete :access_points,
           to: 'sensors#delete_all_access_points',
           as: :delete_all_access_points, on: :member
    get 'access_points/:status/status',
        to: 'sensors#status_access_points',
        as: :access_points_by_status, on: :member
    get :new_access_point, on: :member
    post :create_access_point, on: :member
    get :new_locate_access_point, on: :member
    post :locate_access_point, on: :member
    patch :assign_floor_to_access_point, on: :member
    get :view, on: :member
    get :view, on: :collection
    get :load_ap_data, on: :member
    get :load_more_children

    resources :zones, except: [:show, :index], shallow: true do
      post :add_access_point, on: :member
      post :add_access_points, on: :member
      post :remove_access_point, on: :member
      post :remove_access_points, on: :member
      get :search, on: :collection
    end
  end

  resources :traffics, path: '/flow' do
    member do
      get :load_tab
      get :load_tab_cardinality
      get :load_cardinality_table
    end
    collection do
      get :tops
      get :get_points
      get :comparative
      get :cardinality
      get :map
      patch :change_session_tab
      patch :change_session_total
      get :info
      get :help
      get :load_more
      get :load_more_tops
      get :load_more_comparative
      get :load_more_cardinality
      get :load_new_data
      get :search
      post :results
      get :element_childs
      get :lookup
      get :clients
      get :client_points
      get :search_clients
      get :export_tops_to_csv
      get :export_cardinality_to_csv
      get :export_raw_to_csv
      get :sort_columns
      get :sort_aggregations
      get 'aggregations/reorder',
          to: 'traffics#reorder_aggregations', as: 'reorder_flow_aggregations'
      get :aps_metrics
      get :channels_load
      get :channels_load_serie
      get :aps_serie
      get :aps_count
      patch 'change_chart_type/:id',
            action: :change_chart_type, as: :change_chart_type
      get 'tops/add_tab/:id',
          action: :add_tab, as: :add_traffics_top_tab
      get 'cardinality/add_tab/:id',
          action: :add_cardinality_tab, as: :add_traffics_cardinality_tab
      get 'cardinality/get_serie/:id',
          action: :get_cardinality_serie, as: :get_traffics_cardinality_serie
      patch 'tops/del_tab/:id',
            action: :del_tab, as: :del_traffics_top_tab
      patch 'cardinality/del_tab/:id',
            action: :del_cardinality_tab, as: :del_traffics_cardinality_tab
      get 'tops/get_serie/:id',
          action: :get_serie, as: :get_traffics_serie
      get 'tops/get_tops/:id',
          action: :get_tops, as: :get_traffics_tops
      get 'tops/get_table',
          action: :get_table, as: :get_traffics_table
      get 'tops/get_raw_table',
          action: :get_raw_table, as: :get_traffics_raw_table
      get 'get_maps_data/:id',
          action: :get_maps_data, as: :get_traffics_maps_data
      post 'export_tops_to_file',
           action: :export_tops_to_file, as: :export_tops_to_file
      post 'export_cardinality_to_file',
           action: :export_cardinality_to_file, as: :export_cardinality_to_file
      post 'export_raw_to_file',
           action: :export_raw_to_file, as: :export_raw_to_file
      get 'get_outliers/:id',
          action: :get_outliers, as: :get_traffics_outliers
    end
  end

  patch 'saved_flow_filter/:id/add',
        to: 'saved_traffic_filters#add_saved_filter',
        as: 'add_saved_flow_filter'
  post 'saved_flow_filters/create',
       to: 'saved_traffic_filters#create'
  get 'tabs/reorder_flow',
      to: 'traffics#reorder_tab', as: 'reorder_flow_tab'

  resources :saved_traffic_filters, path: '/saved/flow_filters' do

    collection do
      post :title
      post :update
    end

    member do
      get :view
      post :update
    end
  end

  resources :widgets, except: [:index, :create, :new, :edit, :show, :update, :destroy] do
    patch :remove_image, on: :member
    get :get_sensors_table_flow, action: :get_sensors_table_flow, as: :widget_get_sensors_table_flow, on: :member
  end

  resources :gps_markers

  resources :jobs do
    collection do
      post :start_mac_finder
      post :start_assets_inventory
      post :start_zabbix_inventory_job
    end
  end

  namespace :api do
    namespace :v1 do
      # Sensors flow
      resources :sensors_flow, only: [], param: :sensor_uuid, path: 'sensors' do
        # Sensors and APs
        # DEPRECATED
        get 'delete_aps', on: :member, action: :delete_aps
        # NEW
        delete 'access_points', on: :member, action: :delete_aps
        # END
        # DEPRECATED
        post 'override_aps', on: :member, action: :override_aps
        # NEW
        post 'access_points/override', on: :member, action: :override_aps
        post 'access_points/import',
             on: :member,
             controller: :access_points,
             action: :import
        # END
        # Get info and create
        %w(flow mse meraki ale arubacentral device).each do |s|
          get s, on: :member, action: s.to_sym
          get s, on: :collection, action: s.to_sym
          # DEPRECATED
          post "create/#{s}", on: :collection, action: "create_#{s}".to_sym
          # NEW
          post s, on: :collection, action: "create_#{s}".to_sym
          # END
          patch s, on: :member, action: "update_#{s}".to_sym
        end
      end

      resources :access_points, only: [:index, :create, :update] do
        delete '/', on: :collection, action: :destroy
      end
      # Cisco Meraki
      resources :meraki, only: [] do
        get '/:secret', to: 'meraki#index', on: :collection
      end
    end
  end

  get '/feed/v1/meraki/:secret', to: 'api/v1/meraki#index'
end
