#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module CheckCorruptDataJobFlow
    #
    # This method will be called before exec CheckCorruptDataJob perform
    # instance method. After finish theese tasks, it wil call super to exec
    # main perform method.
    #
    # This method check if there are an AccessPoint without any sensor
    # associated to destroy them.
    #
    def perform
      sensor_ids = Sensor.where(type: [Sensor::FLOW, Sensor::MERAKI]).map(&:id)
      # Destroy APs associated to Flow and Meraki sensors that are deleted
      access_points =
        AccessPoint
        .where('switch_id is not null and switch_id not in (?)', sensor_ids)
      # Destroy them
      aps_destroyed = access_points.destroy_all
      # Destroy APs associated to Domain sensors that are deleted
      domain_ids = Sensor.where(type: Sensor::DOMAIN).map(&:id)
      access_points_domains =
        AccessPoint
        .where('sensor_id is not null and sensor_id not in (?)', domain_ids)
      # Destroy them
      aps_destroyed += access_points_domains.destroy_all
      # Log the number of destroyed element
      if aps_destroyed.size > 0
        logit '--------------------------------------'
        logit "Destroyed #{aps_destroyed.size} AccessPoints."
        logit '--------------------------------------'
      end
    rescue StandardError => e
      logit "#{e}"
      logit e.backtrace.join("\n")
    ensure
      # Call basic perform method
      super
    end
  end
end
