#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class MacFinderJob < Struct.new(:verbose)
      include Redborder::Jobs::JobHelper
      require 'ipaddress'

      def perform

        User.current_user = User.find_by(admin: true)
        search = {}

        # Exclude client_mac that already has a related MacObject
        mac_values = MacObject.select('value').map(&:value)

        mac_values.each do |mac|
          search[:client_mac] ||= {}
          search[:client_mac][mac] = { operator: 'is_not' }
        end

        # Filter by client IPs
        search[:lan_ip] = { '10.4' => { operator: 'starts_with' } }

        result = Traffic.where(
          query_type: :group_by,
          range: :custom,
          start_time: Time.now - 5.minutes,
          end_time: Time.now,
          granularity: :all,
          sort: :bytes,
          limit: 1000,
          group_by: [:lan_ip, :client_mac],
          aggregation: :bytes,
          search: search
        )

        create_mac_objects(result)

      rescue

      ensure

        # Enqueue the job again
        Delayed::Job.enqueue(Redborder::Jobs::MacFinderJob
        .new(true), priority: 10, run_at: Time.now + 5.minutes)

      end

      def create_mac_objects(result)
        result.each do |row|
          next if row.lan_ip.nil? || row.client_mac.nil?
          # If is not a private IP or there is already a MacObject for that client
          # mac, iterate to the next value
          next unless IPAddress::IPv4.extract(row.lan_ip).private? && MacObject.find_by(value: row.client_mac).nil?

          begin
            name = timeout(0.5) { Socket.getnameinfo(Socket.sockaddr_in(80, row.lan_ip))[0] }

            # If the return value is the same as IP, the name has not been resolved
            next if name == row.lan_ip

            macobject = MacObject.new(
              name: name,
              value: row.client_mac,
              sensor_id: 1
            )

            macobject.save(validate: false)
          rescue Timeout::Error
          end
        end
      end

    end
  end
end
