#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module SensorsControllerFlow
    def self.included(base)

      base.class_eval do
        unloadable

        helper_method :sort_column, :sort_direction

        # -------------------------------------------------------------
        # Before and after
        # -------------------------------------------------------------

        before_action :privileges_on_top_domain_flow,
                      only: [:create_flow, :create_mse, :create_ale, :create_arubacentral, :create_device, :create_meraki,
                             :new_mse, :new_ale, :new_arubacentral, :new_device, :new_flow, :new_meraki]

        before_action :set_parent_id_flow,
                      only: [:new_mse, :new_ale, :new_arubacentral, :new_device, :new_flow, :new_meraki]
        before_action :set_flow_sensors_per_page,
                      only: [:flow, :flow_load_more, :mse,
                             :mse_load_more, :device, :device_load_more, :ale, :ale_load_more, :arubacentral, :arubacentral_load_more, :meraki, :meraki_load_more]
        before_action :set_default_sensor_view_flow,
                      only: [:flow, :mse, :ale, :arubacentral, :device, :meraki]

        before_action :set_sensor_flow,
                      only: [:new_home_network, :create_home_network,
                             :delete_home_network, :new_router_mac,
                             :create_router_mac, :delete_router_mac,
                             :access_points, :overview_aps, :access_points_status,
                             :image, :logo, :remove_image, :remove_logo]

        before_action :manage_privileges_flow, only: [
                        :new_home_network, :create_home_network,
                        :delete_home_network, :new_router_mac,
                        :create_router_mac, :delete_router_mac
                      ]

        before_action :redirect_to_view, only: :edit

        before_action :redirect_to_view, only: :edit

        before_action :initialize_sensor_if_permissions,
                      only: [:new_access_point, :create_access_point,
                             :new_locate_access_point, :locate_access_point,
                             :three_dimensions_map]

        before_action :initialize_domain_and_flow_sensors,
                      only: [:new_access_point]

        before_action :permissions_on_access_point,
                      only: [:locate_access_point]

        before_action :permissions_on_selected_switch_and_floor,
                      only: [:locate_access_point]

        before_action :check_permissions, only: [:view, :access_points]

        before_action :set_range_and_granularity, only: [:view]

        skip_before_action :authenticate_user!, only: [:logo, :image], raise: false

        # -------------------------------------------------------------
        # Methods
        # -------------------------------------------------------------

        def flow
          # The current page
          @page = 0

          # The current letter
          @current_letter = ''

          if params[:id].nil?
            sensor_ids = Sensor.where(type: Sensor::FLOW, is_deleted: false)
                         .select{|s| current_user.can?(:read, s)}.map(&:id)
          else
            @sensor = Sensor.find(params[:id])
            sensor_ids = @sensor.all_children
                         .where(type: Sensor::FLOW, is_deleted: false)
                         .select{|s| current_user.can?(:read, s)}.map(&:id)
          end

          # Use unscoped to remove the ordering setted in sensor.rb
          @sensors = Sensor.unscoped
                     .where(id: sensor_ids)
                     .order('LOWER(name) ASC')
                     .take(@sensors_page)
        end

        # -------------------------------------------------------------
        # Sensors methods
        # -------------------------------------------------------------
        def new_flow
          @sensor = Sensor
                    .new(type: Sensor::FLOW,
                         parent_id: @parent_id)
          render layout: false
        end

        #
        # Method for destroy Flow sensors
        #
        def destroy_5
          @sensor.update_column(:is_deleted, true)
          Delayed::Job.enqueue(
            Redborder::Jobs::DeleteSensorJob.new(
              @sensor.id, current_user.id, true, I18n.locale),
            priority: 10, run_at: Time.now)

          respond_to do |format|
            format.html { redirect_to sensors_url }
            format.json { render json: { query: true, id: @sensor.id } }
            format.js
          end
        end

        DittoCode::Exec.if 'ENTERPRISE' do
        %w(mse ale arubacentral device meraki).each do |type|
          # New methods
          # --------------------
          define_method("new_#{type}") do
            @sensor = Sensor
                      .new(type: Sensor.const_get(type.upcase),
                           parent_id: @parent_id)
            render layout: false
          end

          # Destroy methods
          # --------------------
          define_method("destroy_#{Sensor.const_get(type.upcase)}") do
            @sensor.update_column(:is_deleted, true)
            Delayed::Job.enqueue(
              Redborder::Jobs::DeleteSensorJob.new(
                @sensor.id, current_user.id, true, I18n.locale),
              priority: 10, run_at: Time.now)

            respond_to do |format|
              format.html { redirect_to sensors_url }
              format.json { render json: { query: true, id: @sensor.id } }
              format.js
            end
          end
        end
        end

        # -------------------------------------------------------------
        # End Sensors methods
        # -------------------------------------------------------------

        #
        # => Search a flow sensor based on the first letter of its name
        #
        def flow_letter_search
          # Get the letter
          @letter = params[:id]

          # Check the letter
          if [*"A".."Z"].include? @letter
            sensor_ids = if params[:sensor_id].nil?
              Sensor.where(type: Sensor::FLOW, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              @sensor.all_children.where(type: Sensor::FLOW, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            end
            @sensors = Sensor.where(id: sensor_ids).order('LOWER(name) ASC')
            render :flow_search
          else
            # Something went wrong
            redirect_to flow_sensors_path
          end

        end

        def flow_search
          @search = params[:search]
          sensor_ids = if params[:sensor_id].nil?
            Sensor.where(type: Sensor::FLOW, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
          else
            @sensor = Sensor.find(params[:sensor_id])
            @sensor.all_children.where(type: Sensor::FLOW, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
          end
          @sensors = Sensor.where(id: sensor_ids).order("LOWER(name) ASC")
        end

        def flow_load_more

          # Get the page
          @page = params[:page].to_i

          # Get the sensors
          if params[:sensor_id].nil?
            sensor_ids = Sensor.where(type: Sensor::FLOW, is_deleted: false)
                         .select{|s| current_user.can?(:read, s)}.map(&:id)
          else
            @sensor = Sensor.find(params[:sensor_id])
            sensor_ids = @sensor.all_children
                         .where(type: Sensor::FLOW, is_deleted: false)
                         .select{|s| current_user.can?(:read, s)}.map(&:id)
          end


          @sensors = Sensor.unscoped.where(id: sensor_ids).order('LOWER(name) ASC').offset(@page * @sensors_page).take(@sensors_page)

          # Set the current letter empty
          before_sensor = Sensor.unscoped.where(id: sensor_ids).order(:name).offset((@page * @sensors_page) - 1).first

          if before_sensor.nil?
            @current_letter = ''
          else
            @current_letter = before_sensor.secondary_name[0,1].upcase
          end

          respond_to do |format|
           format.js {}
           format.json { render json: @sensors }
          end

        end

        DittoCode::Exec.if 'ENTERPRISE' do
          # GET /sensors/mse
          def mse
            # The current page
            @page = 0

            # The current letter
            @current_letter = ''

            if params[:id].nil?
              sensor_ids = Sensor.where(type: Sensor::MSE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::MSE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end

            # Use unscoped to remove the ordering setted in sensor.rb
            @sensors = Sensor.unscoped
                       .where(id: sensor_ids)
                       .order('LOWER(name) ASC')
                       .take(@sensors_page)
          end

          # GET /sensors/ale
          def ale
            # The current page
            @page = 0

            # The current letter
            @current_letter = ''

            if params[:id].nil?
              sensor_ids = Sensor.where(type: Sensor::ALE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::ALE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end

            # Use unscoped to remove the ordering setted in sensor.rb
            @sensors = Sensor.unscoped
                       .where(id: sensor_ids)
                       .order('LOWER(name) ASC')
                       .take(@sensors_page)
          end

          # GET /sensors/arubacentral
          def arubacentral
            # The current page
            @page = 0

            # The current letter
            @current_letter = ''

            if params[:id].nil?
              sensor_ids = Sensor.where(type: Sensor::ARUBACENTRAL, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::ARUBACENTRAL, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end

            # Use unscoped to remove the ordering setted in sensor.rb
            @sensors = Sensor.unscoped
                       .where(id: sensor_ids)
                       .order('LOWER(name) ASC')
                       .take(@sensors_page)
          end

          # GET /sensors/device
          def device
            # The current page
            @page = 0

            # The current letter
            @current_letter = ''

            if params[:id].nil?
              sensor_ids = Sensor.where(type: Sensor::DEVICE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::DEVICE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end

            # Use unscoped to remove the ordering setted in sensor.rb
            @sensors = Sensor.unscoped
                       .where(id: sensor_ids)
                       .order('LOWER(name) ASC')
                       .take(@sensors_page)
          end

          # GET /sensors/meraki
          def meraki
            # The current page
            @page = 0

            # The current letter
            @current_letter = ''

            @sensors = if params[:id].nil?
              current_user.sensor.self_and_descendants
            else
              @sensor = Sensor.find(params[:id])
              if current_user.cannot?(:read, @sensor)
                Sensor.none
              else
                @sensor.self_and_descendants
              end
            end

            @sensors = @sensors.where(type: Sensor::MERAKI, is_deleted: false)
                               .reorder('LOWER(name) ASC')
                               .take(@sensors_page)
          end

          #
          # => Search a mse sensor based on the first letter of its name
          #
          def mse_letter_search

            # Get the letter
            @letter = params[:id]

            # Check the letter
            if [*"A".."Z"].include? @letter
              sensor_ids = if params[:sensor_id].nil?
                Sensor.where(type: Sensor::MSE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              else
                @sensor = Sensor.find(params[:sensor_id])
                @sensor.all_children.where(type: Sensor::MSE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              end
              @sensors = Sensor.where(id: sensor_ids).order('LOWER(name) ASC')
              render :mse_search
            else
              # Something went wrong
              redirect_to mse_sensors_path
            end

          end

          def mse_search
            @search = params[:search]
            sensor_ids = if params[:sensor_id].nil?
              Sensor.where(type: Sensor::MSE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              @sensor.all_children.where(type: Sensor::MSE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            end
            @sensors = Sensor.where(id: sensor_ids).order("LOWER(name) ASC")
          end

          def mse_load_more

            # Get the page
            @page = params[:page].to_i

            # Get the sensors
            if params[:sensor_id].nil?
              sensor_ids = Sensor.where(type: Sensor::MSE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::MSE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end


            @sensors = Sensor.unscoped.where(id: sensor_ids).order('LOWER(name) ASC').offset(@page * @sensors_page).take(@sensors_page)

            # Set the current letter empty
            before_sensor = Sensor.unscoped.where(id: sensor_ids).order(:name).offset((@page * @sensors_page) - 1).first

            if before_sensor.nil?
              @current_letter = ''
            else
              @current_letter = before_sensor.secondary_name[0,1].upcase
            end

            respond_to do |format|
             format.js {}
             format.json { render json: @sensors }
            end

          end

          #
          # => Search a ale sensor based on the first letter of its name
          #
          def ale_letter_search

            # Get the letter
            @letter = params[:id]

            # Check the letter
            if [*"A".."Z"].include? @letter
              sensor_ids = if params[:sensor_id].nil?
                Sensor.where(type: Sensor::ALE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              else
                @sensor = Sensor.find(params[:sensor_id])
                @sensor.all_children.where(type: Sensor::ALE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              end
              @sensors = Sensor.where(id: sensor_ids).order('LOWER(name) ASC')
              render :ale_search
            else
              # Something went wrong
              redirect_to ale_sensors_path
            end

          end

          def ale_search
            @search = params[:search]
            sensor_ids = if params[:sensor_id].nil?
              Sensor.where(type: Sensor::ALE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              @sensor.all_children.where(type: Sensor::ALE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            end
            @sensors = Sensor.where(id: sensor_ids).order("LOWER(name) ASC")
          end

          def ale_load_more

            # Get the page
            @page = params[:page].to_i

            # Get the sensors
            if params[:sensor_id].nil?
              sensor_ids = Sensor.where(type: Sensor::ALE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::ALE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end


            @sensors = Sensor.unscoped.where(id: sensor_ids).order('LOWER(name) ASC').offset(@page * @sensors_page).take(@sensors_page)

            # Set the current letter empty
            before_sensor = Sensor.unscoped.where(id: sensor_ids).order(:name).offset((@page * @sensors_page) - 1).first

            if before_sensor.nil?
              @current_letter = ''
            else
              @current_letter = before_sensor.secondary_name[0,1].upcase
            end

            respond_to do |format|
             format.js {}
             format.json { render json: @sensors }
            end

          end

          #
          # => Search a arubacentral sensor based on the first letter of its name
          #
          def arubacentral_letter_search

            # Get the letter
            @letter = params[:id]

            # Check the letter
            if [*"A".."Z"].include? @letter
              sensor_ids = if params[:sensor_id].nil?
                Sensor.where(type: Sensor::ARUBACENTRAL, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              else
                @sensor = Sensor.find(params[:sensor_id])
                @sensor.all_children.where(type: Sensor::ARUBACENTRAL, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              end
              @sensors = Sensor.where(id: sensor_ids).order('LOWER(name) ASC')
              render :arubacentral_search
            else
              # Something went wrong
              redirect_to arubacentral_sensors_path
            end

          end

          def arubacentral_search
            @search = params[:search]
            sensor_ids = if params[:sensor_id].nil?
              Sensor.where(type: Sensor::ARUBACENTRAL, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              @sensor.all_children.where(type: Sensor::ARUBACENTRAL, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            end
            @sensors = Sensor.where(id: sensor_ids).order("LOWER(name) ASC")
          end

          def arubacentral_load_more

            # Get the page
            @page = params[:page].to_i

            # Get the sensors
            if params[:sensor_id].nil?
              sensor_ids = Sensor.where(type: Sensor::ARUBACENTRAL, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::ARUBACENTRAL, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end


            @sensors = Sensor.unscoped.where(id: sensor_ids).order('LOWER(name) ASC').offset(@page * @sensors_page).take(@sensors_page)

            # Set the current letter empty
            before_sensor = Sensor.unscoped.where(id: sensor_ids).order(:name).offset((@page * @sensors_page) - 1).first

            if before_sensor.nil?
              @current_letter = ''
            else
              @current_letter = before_sensor.secondary_name[0,1].upcase
            end

            respond_to do |format|
             format.js {}
             format.json { render json: @sensors }
            end

          end

          #
          # => Search a device sensor based on the first letter of its name
          #
          def device_letter_search

            # Get the letter
            @letter = params[:id]

            # Check the letter
            if [*"A".."Z"].include? @letter
              sensor_ids = if params[:sensor_id].nil?
                Sensor.where(type: Sensor::DEVICE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              else
                @sensor = Sensor.find(params[:sensor_id])
                @sensor.all_children.where(type: Sensor::DEVICE, is_deleted: false).where("LOWER(name) LIKE ?", "#{@letter.downcase}%").select{|s| current_user.can?(:read, s)}.map(&:id)
              end
              @sensors = Sensor.where(id: sensor_ids).order('LOWER(name) ASC')
              render :device_search
            else
              # Something went wrong
              redirect_to device_sensors_path
            end

          end

          def device_search
            @search = params[:search]
            sensor_ids = if params[:sensor_id].nil?
              Sensor.where(type: Sensor::DEVICE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              @sensor.all_children.where(type: Sensor::DEVICE, is_deleted: false).where("LOWER(name) LIKE ?", "%#{@search.downcase.split.join('%')}%").select{|s| current_user.can?(:read, s)}.map(&:id)
            end
            @sensors = Sensor.where(id: sensor_ids).order("LOWER(name) ASC")
          end

          def device_load_more

            # Get the page
            @page = params[:page].to_i

            # Get the sensors
            if params[:sensor_id].nil?
              sensor_ids = Sensor.where(type: Sensor::DEVICE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            else
              @sensor = Sensor.find(params[:sensor_id])
              sensor_ids = @sensor.all_children
                           .where(type: Sensor::DEVICE, is_deleted: false)
                           .select{|s| current_user.can?(:read, s)}.map(&:id)
            end


            @sensors = Sensor.unscoped.where(id: sensor_ids).order('LOWER(name) ASC').offset(@page * @sensors_page).take(@sensors_page)

            # Set the current letter empty
            before_sensor = Sensor.unscoped.where(id: sensor_ids).order(:name).offset((@page * @sensors_page) - 1).first

            if before_sensor.nil?
              @current_letter = ''
            else
              @current_letter = before_sensor.secondary_name[0,1].upcase
            end

            respond_to do |format|
             format.js {}
             format.json { render json: @sensors }
            end

          end

          #
          # => Search a meraki sensor based on the first letter of its name
          #
          def meraki_letter_search
            # Get the letter
            @letter = params[:id]

            # Check the letter
            if [*"A".."Z"].include? @letter
              @sensors = if params[:sensor_id].nil?
                current_user.sensor.self_and_descendants
              else
                @sensor = Sensor.find(params[:sensor_id])

                if current_user.cannot?(:read, @sensor)
                  Sensor.none
                else
                  @sensor.self_and_descendants
                end
              end
              @sensors = @sensors.where(type: Sensor::MERAKI, is_deleted: false)
                         .where('LOWER(name) LIKE ?', "#{@letter.downcase}%")
                         .reorder('LOWER(name) ASC')
              render :meraki_search
            else
              # Something went wrong
              redirect_to meraki_sensors_path
            end
          end

          def meraki_search
            @search = params[:search]

            @sensors = if params[:sensor_id].nil?
              current_user.sensor.self_and_descendants
            else
              @sensor = Sensor.find(params[:sensor_id])

              if current_user.cannot?(:read, @sensor)
                Sensor.none
              else
                @sensor.self_and_descendants
              end
            end

            @sensors = @sensors.where(type: Sensor::MERAKI, is_deleted: false)
                       .where('LOWER(name) LIKE ?', "%#{@search.downcase.split.join('%')}%")
                       .reorder('LOWER(name) ASC')
          end

          def meraki_load_more
            # Get the page
            @page = params[:page].to_i

            # Get the sensors
            all_sensors = if params[:id].nil?
                            current_user.sensor.self_and_descendants
                          else
                            @sensor = Sensor.find(params[:id])
                            if current_user.cannot?(:read, @sensor)
                              Sensor.none
                            else
                              @sensor.self_and_descendants
                            end
                          end

            @sensors = all_sensors.where(type: Sensor::MERAKI, is_deleted: false)
                       .reorder('LOWER(name) ASC')
                       .offset(@page * @sensors_page)
                       .take(@sensors_page)

            # Set the current letter empty
            before_sensor = all_sensors.reorder('LOWER(name) ASC')
                            .offset((@page * @sensors_page) - 1).first

            if before_sensor.nil?
              @current_letter = ''
            else
              @current_letter = before_sensor.secondary_name[0, 1].upcase
            end

            respond_to do |format|
              format.js {}
              format.json { render json: @sensors }
            end
          end

          # POST /sensors/create_mse
          def create_mse
            # MSE has same params
            @sensor = Sensor.new(send("new_sensor_params_#{Sensor::MSE}"))

            if params[:sensor][:parent_id].blank? && current_user.super_admin?
              @parent_id = @sensor.parent_id = Sensor.root.id
            elsif params[:sensor][:parent_id].blank?
              @parent_id = @sensor.parent_id = current_user.sensor.id
            else
              @sensor.parent_id = params[:sensor][:parent_id]
              @parent_id = params[:sensor][:parent_id]
            end

            @sensor.real_parent_id = @parent_id if @sensor.parent.is_client_proxy?

            @permission = true

            respond_to do |format|
              if current_user.cannot?(:manage, @sensor.parent)
                @permission = false
                # No permissions on parent.
                format.html do
                  redirect_to sensors_path,
                              alert: "#{I18n.t 'lib_flow.privileges'}"
                end
                format.json do
                  render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                         status: :unauthorized
                end
                format.js do
                  flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
                end
              elsif @sensor.save
                @sensor.add_default_flow_monitors
                # You need to reload the user sensor rgt value before rendering menus
                # in order to set manage privileges for the current user
                current_user.reload
                Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id), priority: 5) unless DelayedJob.rb_wakeup_chef_cluster?
                format.html { redirect_to sensors_path, notice: "#{I18n.t 'lib_flow.created'}" }
                format.json { head :no_content }
                format.js
              else
                format.html { render action: 'new_mse' }
                format.json { render json: @sensor.errors, status: :unprocessable_entity }
                format.js
              end
            end
          end

          # POST /sensors/create_ale
          def create_ale
            # ALE has same params
            @sensor = Sensor.new(send("new_sensor_params_#{Sensor::ALE}"))

            if params[:sensor][:parent_id].blank? && current_user.super_admin?
              @parent_id = @sensor.parent_id = Sensor.root.id
            elsif params[:sensor][:parent_id].blank?
              @parent_id = @sensor.parent_id = current_user.sensor.id
            else
              @sensor.parent_id = params[:sensor][:parent_id]
              @parent_id = params[:sensor][:parent_id]
            end

            @sensor.real_parent_id = @parent_id if @sensor.parent.is_client_proxy?

            @permission = true

            respond_to do |format|
              if current_user.cannot?(:manage, @sensor.parent)
                @permission = false
                # No permissions on parent.
                format.html do
                  redirect_to sensors_path,
                              alert: "#{I18n.t 'lib_flow.privileges'}"
                end
                format.json do
                  render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                         status: :unauthorized
                end
                format.js do
                  flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
                end
              elsif @sensor.save
                @sensor.add_default_flow_monitors
                # You need to reload the user sensor rgt value before rendering menus
                # in order to set manage privileges for the current user
                current_user.reload
                Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id), priority: 5) unless DelayedJob.rb_wakeup_chef_cluster?
                format.html { redirect_to sensors_path, notice: "#{I18n.t 'lib_flow.created'}" }
                format.json { head :no_content }
                format.js
              else
                format.html { render action: 'new_ale' }
                format.json { render json: @sensor.errors, status: :unprocessable_entity }
                format.js
              end
            end
          end

          # POST /sensors/create_arubacentral
          def create_arubacentral
            # ARUBACENTRAL has same params
            @sensor = Sensor.new(send("new_sensor_params_#{Sensor::ARUBACENTRAL}"))

            if params[:sensor][:parent_id].blank? && current_user.super_admin?
              @parent_id = @sensor.parent_id = Sensor.root.id
            elsif params[:sensor][:parent_id].blank?
              @parent_id = @sensor.parent_id = current_user.sensor.id
            else
              @sensor.parent_id = params[:sensor][:parent_id]
              @parent_id = params[:sensor][:parent_id]
            end

            @sensor.real_parent_id = @parent_id if @sensor.parent.is_client_proxy?

            @permission = true

            respond_to do |format|
              if current_user.cannot?(:manage, @sensor.parent)
                @permission = false
                # No permissions on parent.
                format.html do
                  redirect_to sensors_path,
                              alert: "#{I18n.t 'lib_flow.privileges'}"
                end
                format.json do
                  render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                         status: :unauthorized
                end
                format.js do
                  flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
                end
              elsif @sensor.save
                @sensor.add_default_flow_monitors
                # You need to reload the user sensor rgt value before rendering menus
                # in order to set manage privileges for the current user
                current_user.reload
                Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id), priority: 5) unless DelayedJob.rb_wakeup_chef_cluster?
                format.html { redirect_to sensors_path, notice: "#{I18n.t 'lib_flow.created'}" }
                format.json { head :no_content }
                format.js
              else
                format.html { render action: 'new_arubacentral' }
                format.json { render json: @sensor.errors, status: :unprocessable_entity }
                format.js
              end
            end
          end

          # POST /sensors/create_device
          def create_device
            # ALE has same params
            @sensor = Sensor.new(send("new_sensor_params_#{Sensor::DEVICE}"))

            if params[:sensor][:parent_id].blank? && current_user.super_admin?
              @parent_id = @sensor.parent_id = Sensor.root.id
            elsif params[:sensor][:parent_id].blank?
              @parent_id = @sensor.parent_id = current_user.sensor.id
            else
              @sensor.parent_id = params[:sensor][:parent_id]
              @parent_id = params[:sensor][:parent_id]
            end

            @sensor.real_parent_id = @parent_id if @sensor.parent.is_client_proxy?

            @permission = true

            respond_to do |format|
              if current_user.cannot?(:manage, @sensor.parent)
                @permission = false
                # No permissions on parent.
                format.html do
                  redirect_to sensors_path,
                              alert: "#{I18n.t 'lib_flow.privileges'}"
                end
                format.json do
                  render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                         status: :unauthorized
                end
                format.js do
                  flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
                end
              elsif @sensor.save
                @sensor.add_default_flow_monitors
                # You need to reload the user sensor rgt value before rendering menus
                # in order to set manage privileges for the current user
                current_user.reload
                Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id), priority: 5) unless DelayedJob.rb_wakeup_chef_cluster?
                format.html { redirect_to sensors_path, notice: "#{I18n.t 'lib_flow.created'}" }
                format.json { head :no_content }
                format.js
              else
                format.html { render action: 'new_device' }
                format.json { render json: @sensor.errors, status: :unprocessable_entity }
                format.js
              end
            end
          end

          # POST /sensors/create_meraki
          def create_meraki
            # MSE has same params
            @sensor = Sensor.new(send("new_sensor_params_#{Sensor::MERAKI}"))
            @sensor.parent_id = (current_user.super_admin? ? Sensor.root.id : current_user.sensor.id) if params[:sensor][:parent_id].blank?
            @parent_id = params[:sensor][:parent_id] if params[:sensor][:parent_id].present?
            @permission = true

            respond_to do |format|
              if current_user.cannot?(:manage, @sensor.parent)
                @permission = false
                # No permissions on parent.
                format.html do
                  redirect_to sensors_path,
                              alert: "#{I18n.t 'lib_flow.privileges'}"
                end
                format.json do
                  render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                         status: :unauthorized
                end
                format.js do
                  flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
                end
              elsif @sensor.save
                @sensor.add_default_flow_monitors
                # You need to reload the user sensor rgt value before rendering
                # menus in order to set manage privileges for the current user
                current_user.reload
                Delayed::Job.enqueue(
                  Redborder::Jobs::RbWakeupChefClusterJob.new(true,
                                                              current_user.id),
                  priority: 5
                ) unless DelayedJob.rb_wakeup_chef_cluster?
                format.html do
                  redirect_to edit_sensor_path(@sensor),
                              notice: "#{I18n.t 'lib_flow.created2'}"
                end
                format.json { head :no_content }
                format.js
              else
                format.html { render action: 'new_meraki' }
                format.json do
                  render json: @sensor.errors, status: :unprocessable_entity
                end
                format.js
              end
            end
          end
        end

        # POST /sensors/create_fow
        def create_flow
          @sensor = Sensor.new(send("new_sensor_params_#{Sensor::FLOW}"))

          if params[:sensor][:parent_id].blank? && current_user.super_admin?
            @parent_id = @sensor.parent_id = Sensor.root.id
          elsif params[:sensor][:parent_id].blank?
            @parent_id = @sensor.parent_id = current_user.sensor.id
          else
            @sensor.parent_id = params[:sensor][:parent_id]
            @parent_id = params[:sensor][:parent_id]
          end

          @sensor.real_parent_id = @parent_id if @sensor.parent.is_client_proxy?

          @permission = true

          respond_to do |format|
            if current_user.cannot?(:manage, @sensor.parent)
              @permission = false
              # No permissions on parent.
              format.html do
                redirect_to sensors_path,
                            alert: "#{I18n.t 'lib_flow.privileges'}"
              end
              format.json do
                render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                       status: :unauthorized
              end
              format.js do
                flash[:error] = "#{I18n.t 'lib_flow.privileges2'}"
              end
            elsif @sensor.save
              @sensor.add_default_flow_monitors
              # You need to reload the user sensor rgt value before rendering menus
              # in order to set manage privileges for the current user
              current_user.reload
              Delayed::Job.enqueue(Redborder::Jobs::RbWakeupChefClusterJob.new(true, current_user.id), priority: 5) unless DelayedJob.rb_wakeup_chef_cluster?
              format.html { redirect_to sensors_path, notice: "#{I18n.t 'lib_flow.created'}" }
              format.json { head :no_content }
              format.js
            else
              format.html { render action: 'new_flow' }
              format.json { render json: @sensor.errors, status: :unprocessable_entity }
              format.js
            end
          end
        end

        [Sensor::FLOW, Sensor::CLIENT_PROXY, Sensor::DOMAIN].each do |section|
          define_method("view_#{section}") do
            @gps_markers = gps_markers params[:id]
            show_aps
            render "view_#{section}"
          end
        end

        def load_more_aps
          @search = params[:search]
          @page = params[:page]
          @status = params[:status]
          show_aps

          respond_to :js
        end

        # Return the access points that can view the sensor
        #
        def access_points
          # Render as JSON
          aps = if params[:located]
                  @sensor.located_access_points
                else
                  @sensor.access_points
                end
          # Render the JSON
          render json: { query: true, aps: aps || [] }
        end

        # Return the status of the access points that can view the sensor
        #
        def access_points_status
          status = @sensor.access_points.group(:status).count
          status_result = {}
          status_result[:up] = status[AccessPoint::STATUS[:UP]] || 0
          status_result[:down] = status[AccessPoint::STATUS[:DOWN]] || 0
          render json: { query: true, status: status_result }
        end

        # Return access points for overview
        #
        def overview_aps
          aps = if @sensor.building?
                  @sensor.children.map(&:to_floor_zone)
                else
                  @sensor.located_access_points
                end

          # Render the JSON
          render json: { query: true, aps: aps || [] }
        end

        def search_aps
          @search = params[:search]
          @status = params[:status]
          show_aps

          respond_to :js
        end

        def search_gps_markers
          @page = params[:page]
          @gps_markers = gps_markers params[:id]

          respond_to :js
        end

        # Generate form for creating new Access Point when
        # click on semsor map or floor image
        def new_access_point
          point = {}
          # point comes from a map
          if params[:latLngFormat]
            point[:latitude] = params[:point][1].to_f
            point[:longitude] = params[:point][0].to_f
          else
            point[:x] = params[:point] ? params[:point][0].to_f : nil
            point[:y] = params[:point] ? params[:point][1].to_f : nil
          end
          @domains = current_user.sensor.self_and_descendants.where(type: Sensor::DOMAIN)
          @access_point = AccessPoint.new(point)
          render layout: false
        end

        def create_access_point
          @access_point = AccessPoint.new(access_point_params)
          if @access_point.x && @access_point.y
            @access_point.recalculate_lnglat
          elsif @access_point.latitude && @access_point.longitude
            @access_point.recalculate_xy
          end

          respond_to do |format|
            if @access_point.save && @access_point.manage_permission?
              @zones = []
              if @sensor.is_flow? || @sensor.is_domain? || @sensor.is_meraki?
                sensors_ids = []
                @sensor.tree_domains(true).map do |s|
                  if current_user.can?(:read, s) && !s.zones.empty?
                    sensors_ids << s.id
                  end
                end
                @zones = Zone.where(sensor_id: sensors_ids).order('name ASC')
              end
              @access_points = @sensor.access_points
              format.html { redirect_to view_sensor_path(@sensor), notice: "#{I18n.t 'lib_flow.ap_created'}" }
              format.js { flash[:notice] = "#{I18n.t 'lib_flow.ap_created'}" }
              format.json { head :no_content }
            else
              format.html { redirect_to view_sensor_path(@sensor), alert: @access_point.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
              format.js
              format.json { render json: @access_point.errors, status: :unprocessable_entity }
            end
          end
        end

        def new_locate_access_point
          point = {}
          # point comes from a map
          if params[:latLngFormat]
            point[:latitude] = params[:point][1].to_f
            point[:longitude] = params[:point][0].to_f
          # point comes from a located floor image
          else
            point[:x] = params[:point][0].to_f
            point[:y] = params[:point][1].to_f
          end
          @access_point = AccessPoint.new(point)
          # unallocated access points
          @aps =  @sensor.access_points
                  .where('(latitude is NULL OR longitude is NULL) ' \
                    'AND (x is NULL OR y is NULL)')
                  .map { |ap| ["#{ap.name} | #{ap.mac_address}", ap.id] }
          render layout: false
        end

        def locate_access_point
          if @access_point.update(access_point_params)
            if @access_point.latitude && @access_point.longitude
              @access_point.recalculate_xy
            elsif @access_point.x && @access_point.y
              @access_point.recalculate_lnglat
            end
          end
        end

        def assign_floor_to_access_point
          floor = Sensor.find_by(id: params[:sensor_floor])
          return if floor.nil?
          @access_points = AccessPoint.where(id: params[:access_points])
          @sensor = Sensor.find(params[:id])
          @access_points.each do |ap|
            next unless ap.manage_permission?
            ap.update(sensor_id: floor.id)
          end
        end

        def edit_access_point
          @sensor = Sensor.find(params[:top_sensor_id])
          @access_point = AccessPoint.find(params[:id])
          initialize_domain_and_flow_sensors
          render layout: false
        end

        def update_access_point
          @access_point = AccessPoint.find(params[:id])
          current_location = {
            latitude: @access_point[:latitude],
            longitude: @access_point[:longitude],
            x: @access_point[:x],
            y: @access_point[:y]
          }
          # Recalculate location if it's changed
          if @access_point.manage_permission? &&
             @access_point.update(access_point_params)
            if @access_point[:x] != current_location[:x] &&
               @access_point[:y] != current_location[:y]
              @access_point.recalculate_lnglat
            elsif @access_point[:latitude] != current_location[:latitude] &&
                  @access_point[:longitude] != current_location[:longitude]
              @access_point.recalculate_xy
            end
          end
          # If we comes from relocate an access point, we must render 200
          if params[:relocate]
            render json: @access_point
          elsif @access_point.errors.empty?
            @sensor = Sensor.find(params[:access_point][:current_sensor])
          end
        end

        def delete_access_point
          @ap_id = params[:id]
          @access_point = AccessPoint.find(@ap_id)
          return unless @access_point
          @sensor = Sensor.find(params[:sensor_id])
          return unless current_user.can?(:manage, @sensor)
          @sensor.delete_selected_access_points(@ap_id)
        end

        def delete_all_access_points
          @sensor = Sensor.find(params[:id])
          if current_user.can?(:manage, @sensor)
            if params[:selected_aps]
              @aps_ids = params[:selected_aps]
              @sensor.delete_selected_access_points(@aps_ids)
            else
              @sensor.delete_all_access_points
              @msg = "#{I18n.t 'lib_flow.ap_deleted'}"
            end
          else
            @msg = "#{I18n.t 'lib_flow.privileges3'}"
          end
          respond_to do |format|
            format.html { redirect_to :back, flash: { notice: @msg } }
            format.js
          end
        end

        def three_dimensions_map
          lim = {}
          floors = []
          if @sensor.floor?
            located_floors = @sensor.gps_markers.count > 2 ? [@sensor] : []
          else
            located_floors =
              @sensor.self_and_descendants.joins(:gps_markers)
              .where(is_deleted: false, domain_type: Sensor::FLOOR)
              .group('sensors.id').having('COUNT(gps_markers.id) > 2')
          end

          return render json: {} if located_floors.empty?

          located_floors.each do |f|
            l = lnglat_corners(f)
            lim[:min_lng] = l[:min_lng] if lim[:min_lng].nil? ||
                                           l[:min_lng] < lim[:min_lng]
            lim[:min_lat] = l[:min_lat] if lim[:min_lat].nil? ||
                                           l[:min_lat] < lim[:min_lat]
            lim[:max_lng] = l[:max_lng] if lim[:max_lng].nil? ||
                                           l[:max_lng] > lim[:max_lng]
            lim[:max_lat] = l[:max_lat] if lim[:max_lat].nil? ||
                                           l[:max_lat] > lim[:max_lat]

            floors.push({
              id: f.id,
              name: f.name,
              parent: f.parent.id,
              sw: l[:sw],
              se: l[:se],
              ne: l[:ne],
              nw: l[:nw],
              level: f.floor_level,
              image_url: original_image_overlay_map_path(f.overlay_map),
              rotation: f.property[:rotation],
              access_points: {
                aps: f.access_points.where.not('latitude is NULL OR longitude is NULL'),
                up: f.access_points.where(status: 1).count,
                down: f.access_points.where(status: 2).count,
                unknown: f.access_points.where(status: 0).count,
                unallocated: f.access_points
                             .where('latitude is NULL OR longitude is NULL').count
              }
            })
          end
          lim = buildFloorsContainer(lim)
          matrix = getTransMatrix(lim)
          floors.each do |f|
            f[:sw] = PrimeInfrastructure::Functions
                     .lnglat_to_point(matrix[:matrix_trans],
                                      matrix[:matrix_offset],
                                      [f[:sw][0, 0], f[:sw][1, 0]])
            f[:se] = PrimeInfrastructure::Functions
                     .lnglat_to_point(matrix[:matrix_trans],
                                      matrix[:matrix_offset],
                                      [f[:se][0, 0], f[:se][1, 0]])
            f[:nw] = PrimeInfrastructure::Functions
                     .lnglat_to_point(matrix[:matrix_trans],
                                      matrix[:matrix_offset],
                                      [f[:nw][0, 0], f[:nw][1, 0]])
            f[:ne] = PrimeInfrastructure::Functions
                     .lnglat_to_point(matrix[:matrix_trans],
                                      matrix[:matrix_offset],
                                      [f[:ne][0, 0], f[:ne][1, 0]])

            f[:width] = Math.hypot(f[:se][0, 0] - f[:sw][0, 0],
                                   f[:se][1, 0] - f[:sw][1, 0])
            f[:height] = Math.hypot(f[:nw][0, 0] - f[:sw][0, 0],
                                    f[:nw][1, 0] - f[:sw][1, 0])
            f[:access_points][:aps].each do |ap|
              ap[:x] = f[:width] * ap.x / ap.sensor.property[:dimensions][:width]
              ap[:y] = f[:height] * ap.y / ap.sensor.property[:dimensions][:height]
            end
          end
          render json: floors
        end

        #
        # => Return the access points of a sensor
        #    with the selected status
        #
        def status_access_points
          @status = params[:status]
          @search = params[:search]
          @sensor = Sensor.find(params[:id])

          # Check if the user can read the data
          if !current_user.can?(:read, @sensor)
            respond_to do |format|
              format.html { redirect_to sensors_url, alert: "#{I18n.t 'lib_flow.privileges3'}" }
              format.json { head :no_content, status: 403 }
              format.js {render "", status: 403}
            end
          else

            # Get the APs by status
            show_aps

          end
        end

        def new_home_network
          render :layout => false
        end

        def create_home_network
          node = @sensor.chef_node
          home_network = {"name" => params[:home_network][:name], "value" => NetAddr::CIDR.create(params[:home_network][:value]).to_s}

          @found = false

          if !node["redborder"].nil?
            node["redborder"]["homenets"].to_a.each do |server|
              @found = (server["name"] == params[:home_network][:name])
              break if @found
            end
          end

          if !@found
            node.normal["redborder"] = {} if node["redborder"].nil?
            node.normal["redborder"]["homenets"] = [] if ( node["redborder"]["homenets"].nil? or node["redborder"]["homenets"].class != Chef::Node::ImmutableArray)
            node.normal["redborder"]["homenets"] << home_network
            node.save
          end

          respond_to :js
        end

        def delete_home_network
          node = @sensor.chef_node
          homenets = node["redborder"]["homenets"].dup
          homenets.delete_at(params[:home_network_id].to_i)
          node.normal["redborder"]["homenets"] = homenets
          node.save
          respond_to :js
        end

        def new_router_mac
          render :layout => false
        end

        def create_router_mac
          node = @sensor.chef_node
          router_mac = {"name" => params[:router_mac][:name], "value" => params[:router_mac][:value]}

          @found = false

          if !node["redborder"].nil?
            node["redborder"]["routers_macs"].to_a.each do |server|
              @found = (server["name"] == params[:router_mac][:name])
              break if @found
            end
          end

          if !@found
            node.normal["redborder"] = {} if node["redborder"].nil?
            node.normal["redborder"]["routers_macs"] = [] if ( node["redborder"]["routers_macs"].nil? or node["redborder"]["routers_macs"].class != Chef::Node::ImmutableArray)
            node.normal["redborder"]["routers_macs"] << router_mac
            node.save
          end

          respond_to :js
        end

        def delete_router_mac
          node = @sensor.chef_node
          routers_macs = node["redborder"]["routers_macs"].dup
          routers_macs.delete_at(params[:router_mac_id].to_i)
          node.normal["redborder"]["routers_macs"] = routers_macs
          node.save
          respond_to :js
        end

        def import_aps
          @sensor = Sensor.find(params[:id])

          if request.get?
            render layout: false
          elsif request.post?
            content_file = params[:aps_file][:csv_file]
            ap_count, not_saved, wrong_header = Sensor.add_aps_from_file(content_file, @sensor.id, params[:add_new_aps])
            # Geopositionate all descendant APs
            if ap_count > 0
              @sensor.self_and_descendants.each do |sensor|
                sensor.geopositionate_aps
              end
            end
            @sensor.update_radius_access_points
            msg = "#{ap_count} #{I18n.t 'lib_flow.ap_added'}. #{not_saved} #{I18n.t 'lib_flow.ap_not_added'}."
            msg += "#{I18n.t 'lib_flow.correct_header'}." if wrong_header
            redirect_to :back, flash: { notice: msg }
          end
        end

        def export_aps
          @sensor = Sensor.find(params[:id])
          filename = in_time_machine { Time.now.strftime("%Y%m%d%H%M%S") + "-#{@sensor.name.capitalize}-Access-Points.csv" }
          content = aps_to_csv_format
          send_data content, filename: filename
        end

        #
        # GET /sensors/:id/view
        #
        def view
          @children = @sensor.children.limit(Sensor::CHILDREN_PAGE)
          @search = params[:search]

          type = case @sensor.type
            when Sensor::MERAKI
              Sensor::FLOW

            else
              @sensor.type
            end
            send("view_#{type}")
        rescue ActionView::MissingTemplate, NoMethodError
          render :view
        end

        # Gives another 3 children sensors with the requested offset
        def load_more_children
          @offset = params[:offset].to_i
          @sensor = Sensor.find params[:sensor_id]
          @more_children = @sensor.children.offset(@offset).limit(3)
        end

        def sensor_if_permissions
          if params[:id].nil?
            respond_to do |format|
              format.html do
                render alert: "#{I18n.t 'lib_flow.select_sensor'}"
              end
              format.json do
                render json: { error: "#{I18n.t 'lib_flow.select_sensor'}" },
                       status: :unauthorized
              end
              format.js { head 403 }
            end
          else
            @sensor = Sensor.find(params[:id])
            manage_privileges
          end
        end

        #
        # Scan Access Points via SNMP
        #
        def scan_aps(user = current_user)
          @sensor = Sensor.find(params[:id])
          aps = @sensor.access_points.count
          @sensor.scan_aps if user.can?(:manage, @sensor)
          redirect_to view_sensor_path(@sensor.id),
            notice: "#{@sensor.access_points.count - aps} #{I18n.t 'lib_flow.aps_added'} #{aps} "\
            "#{I18n.t 'lib_flow.aps_updated'} #{@sensor.name}."
        end

        def geopositionate_aps
          sensor = Sensor.find params[:id]

          if sensor.geopositionate_aps
            flash[:notice] = 'Access points were successfully updated.'
          else
            flash[:error] = "#{I18n.t 'lib_flow.some_error'}" \
                            " #{I18n.t 'lib_flow.points'}"
          end

          redirect_to view_sensor_path(sensor)
        end

        def image
          img = open(
              @sensor.captive_portal_background_image_uri,
              ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
            ).read
          render text: img, content_type: @sensor.captive_portal_background_image.content_type
        end

        def remove_image
          @sensor.update(captive_portal_background_image: nil)
        end

        def logo
          img = open(
              @sensor.captive_portal_logo_image_uri,
              ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
            ).read
          render text: img, content_type: @sensor.captive_portal_logo_image.content_type
        end
        
        def remove_logo
          @sensor.update(captive_portal_logo_image: nil)
        end


        private

        def set_range_and_granularity
          @filter = Redborder::Filter.new(ApState)
          @range = params[:range] || 'last_3_hour'
          @end_time = in_time_machine { Time.now.change(sec: 0) }
          @start_time = set_start_time(@range, @end_time)
          @start_time_for_slider = @start_time
          @end_time_for_slider = @end_time
          @aggregation = :wireless_stations
          @granularity = params[:granularity] = (Redborder::Rbdruid
                                                .granularity_list(start_time: @start_time,
                                                                  end_time: @end_time) - [:minute, :pt2m, :pt5m]).first
        end

        def set_start_time(range, end_time)
          case range.to_sym
          when :live
            end_time - 3.minutes
          when :last_5_minute
            end_time - 5.minutes
          when :last_15_minute
            end_time - 15.minutes
          when :last_30_minute
            end_time - 30.minutes
          when :last_1_hour
            end_time - 1.hour
          when :last_2_hour
            end_time - 2.hour
          when :last_3_hour
            end_time - 3.hour
          when :last_12_hour
            (end_time - 12.hour).beginning_of_hour
          when :last_24
            (end_time - 24.hour).beginning_of_hour
          when :last_week
            (end_time - 1.week).beginning_of_hour
          else
            end_time - 1.hour
          end
        end

        def set_default_sensor_view_flow
          session[:sensor_view] = params[:action].to_sym
        end

        def set_sensor_flow
          set_sensor
        end

        def manage_privileges_flow
          manage_privileges
        end

        def gps_markers(sensor_id)
          sensor = Sensor.find sensor_id

          sensor.gps_markers
            .order(sort_column + ' ' + sort_direction)
            .paginate(page: params[:page], per_page: 10)
        end

        def update_aps_status_by_druid(aps)
          aps_status =
            ApState.get_access_points_status(aps.map(&:mac_address))
          aps.each do |ap|
            unless (ap.status == AccessPoint::STATUS[:UNKNOWN]) &&
                   (ap.created_at > (Time.now - 5.minutes))
              ap.update(status: aps_status[ap.mac_address])
            end
          end
        end

        def show_aps
          @sensor = Sensor.find(params[:id])
          sensor_ids = @sensor.all_children_ids(true)
          @access_points = AccessPoint
              .order(sort_column + ' ' + sort_direction)
              .where("sensor_id IN (?) OR switch_id IN (?)", sensor_ids, sensor_ids)
              .where('LOWER(name) LIKE :search OR LOWER(ip_address)
                     LIKE :search OR LOWER(mac_address) LIKE :search',
                     search: "%#{params[:search].try(:downcase)}%")

          @access_points = load_aps_by_status(@status, @access_points) unless
                             @status.nil? || @status.blank?

          # Floors must be calculated before paginate access points
          if @sensor.is_flow? || @sensor.is_meraki?
            @floors =
              @access_points.map do |ap|
                if ap.sensor && ap.sensor.domain_type == Sensor::FLOOR &&
                   current_user.can?(:read, ap.sensor)
                  ap.sensor
                end
              end

            @floors = @floors.uniq
                      .reject(&:nil?)
                      .sort { |a,b| a.name.downcase <=> b.name.downcase } || []
          end
          @assignable_domains = current_user.sensor.self_and_descendants.where(
              type: Sensor::DOMAIN, is_deleted: false)
          @access_points_ids = @access_points.map(&:id)
          @access_points     = @access_points.paginate(page: params[:page], per_page: 10) || []
          # Show zones on floor sensors
          @zones = []

          if @sensor.is_flow? || @sensor.is_domain? || @sensor.is_meraki?
            sensors_ids = []
            @sensor.tree_domains(true).map do |s|
              sensors_ids << s.id if current_user.can?(:read, s)
            end
            @zones = Zone.where(sensor_id: sensors_ids).order('name ASC')
          end
        end

        #
        # Returns the aps filtered by status.
        #
        # == Parameters:
        # status::
        #   String with status
        # access_points::
        #   ActiveRecord relation
        #
        # == Returns:
        # New ActiveRecord relation
        #
        def load_aps_by_status(status, access_points)
          if status.split(',').include? '3'
            condition = 'latitude is NULL OR longitude is NULL OR status in (?)'
            condition += ' OR sensor_id is NULL' if @sensor.is_flow?
            access_points.where(condition, status.split(','))
          else
            access_points.where(status: status.split(','))
          end
        end

        def aps_to_csv_format
          sensor_ids = @sensor.all_children_ids(true)
          access_points = AccessPoint
              .order(sort_column + ' ' + sort_direction)
              .where("sensor_id IN (?) OR switch_id IN (?)", sensor_ids, sensor_ids)
              .where('LOWER(name) LIKE :search OR LOWER(ip_address)
                     LIKE :search OR LOWER(mac_address) LIKE :search',
                     search: "%#{params[:search].try(:downcase)}%")

          access_points = load_aps_by_status(params[:status], access_points) unless
                             params[:status].nil? || params[:status].blank?
          csv = 'Name;IP Address;Mac Address;Sensor IP;' \
                "Latitude;Longitude;X;Y\n"
          csv += access_points
                 .map { |ap| "#{ap.name};#{ap.ip_address};" \
                             "#{ap.mac_address};#{ap.switch_name};" \
                             "#{ap.latitude};#{ap.longitude};#{ap.x};#{ap.y}"
                      }.join("\n")
        end

        def require_super_administrative_privileges_flow
          require_super_administrative_privileges
        end

        define_method("sensor_params_#{Sensor::FLOW}") do
          params
            .require(:sensor)
            .permit(:parent_id, :ip, :version_flow, :general_description,
                    :nmsp, :spanport, 
                    :captive_portal_background_image, :captive_portal_background_image_file_name, :captive_portal_background_image_content_type, :captive_portal_background_image_file_size, :captive_portal_background_image_updated_at, 
                    :captive_portal_logo_image, :captive_portal_logo_image_file_name, :captive_portal_logo_image_content_type, :captive_portal_logo_image_file_size, :captive_portal_logo_image_updated_at, 
                    :captive_portal_name, :captive_portal_name_color, :captive_portal_background_color,
                    :nmsp_client_stats, :nmsp_wireless_health,
                    :dns_ptr_client, :dns_ptr_target, :radius_secret,
                    :snmp_community, :snmp_version, :observation_id, :license_id, :type)
        end

        define_method("new_sensor_params_#{Sensor::FLOW}") do
          params
            .require(:sensor)
            .permit(:type, :name, :ip, :parent_id, :snmp_community, :observation_id, :license_id)
        end

        DittoCode::Exec.if 'ENTERPRISE' do
        define_method("sensor_params_#{Sensor::MSE}") do
          params.require(:sensor)
                .permit(:parent_id, :ip, :version_mse, :general_description,
                        :send_http)
          # :snmp_community, :snmp_version
        end

        define_method("sensor_params_#{Sensor::ALE}") do
          params.require(:sensor)
                .permit(:parent_id, :ip, :version_ale, :name, :controller_ip, :rest_api_user, :rest_api_key, :general_description,
                        :send_http)
          # :snmp_community, :snmp_version
        end

        define_method("sensor_params_#{Sensor::ARUBACENTRAL}") do
          params.require(:sensor)
                .permit(:parent_id, :arubacentral_url, :arubacentral_username, :arubacentral_password, :arubacentral_client_id, :arubacentral_client_secret, :arubacentral_customer_id, :name, :general_description, :send_http)
          # :snmp_community, :snmp_version
        end

        define_method("sensor_params_#{Sensor::DEVICE}") do
          params.require(:sensor)
                .permit(:parent_id, :ip, :protocol, :name, :device_ip, :rest_api_user, :rest_api_password, :general_description,
                        :send_http,:snmp_community, :snmp_version, :license_id)
        end

        define_method("sensor_params_#{Sensor::MERAKI}") do
          params.require(:sensor)
                .permit(:parent_id, :ip, :version_meraki, :general_description,
                        :meraki_validator, :snmp_community, :snmp_version,
                        :captive_portal_background_image, :captive_portal_background_image_file_name, :captive_portal_background_image_content_type, :captive_portal_background_image_file_size, :captive_portal_background_image_updated_at, 
                        :captive_portal_logo_image, :captive_portal_logo_image_file_name, :captive_portal_logo_image_content_type, :captive_portal_logo_image_file_size, :captive_portal_logo_image_updated_at, 
                        :captive_portal_name, :captive_portal_name_color, :captive_portal_background_color, :radius_secret, :type)
        end

        define_method("new_sensor_params_#{Sensor::MSE}") do
          params.require(:sensor).permit(:type, :name, :ip, :parent_id,
                                         :nmsp, :version_mse, :license_id)
        end

        define_method("new_sensor_params_#{Sensor::ALE}") do
          params.require(:sensor).permit(:type, :name, :ip, :parent_id,
                                         :nmsp, :version_ale, :name, :controller_ip, :rest_api_user, :rest_api_key, :license_id)
        end

        define_method("new_sensor_params_#{Sensor::ARUBACENTRAL}") do
          params.require(:sensor).permit(:type, :name, :parent_id, :name, :nmsp, :license_id,
                                         :arubacentral_url, :arubacentral_username, 
                                         :arubacentral_password, :arubacentral_client_id,
                                         :arubacentral_client_secret, :arubacentral_customer_id)
        end

        define_method("new_sensor_params_#{Sensor::DEVICE}") do
          params.require(:sensor).permit(:type, :name, :ip, :parent_id, :snmp_community,
                                         :nmsp, :protocol, :rest_api_user, :rest_api_password, :license_id)
        end

        define_method("new_sensor_params_#{Sensor::MERAKI}") do
          params.require(:sensor).permit(:type, :name, :ip, :parent_id,
                                         :meraki_validator, :snmp_community, :license_id)
        end
        end

        def set_parent_id_flow
          set_parent_id
        end

        def privileges_on_top_domain_flow
          privileges_on_top_domain
        end

        #
        # Initialize Sensor from params checking if user has permissions on it
        #
        def initialize_sensor_if_permissions
          @sensor = Sensor.find(params[:id])
          return if current_user.can?(:manage, @sensor)
          respond_to do |format|
            format.html do
              redirect_to sensor_path(@sensor),
                alert: "#{I18n.t 'lib_flow.privileges'}"
            end
            format.json do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
            format.js do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
          end
        end

        #
        # Return min and max lnglat point for given floor
        #
        def lnglat_corners(f)
          m_trans = f[:property][:matrix_trans]
          m_offset = f[:property][:matrix_offset]
          w = f[:property][:dimensions][:width]
          h = f[:property][:dimensions][:height]
          nw = PrimeInfrastructure::Functions
               .point_to_lnglat(m_trans, m_offset, [0, 0])
          ne = PrimeInfrastructure::Functions
               .point_to_lnglat(m_trans, m_offset, [w, 0])
          sw = PrimeInfrastructure::Functions
               .point_to_lnglat(m_trans, m_offset, [0, h])
          se = PrimeInfrastructure::Functions
               .point_to_lnglat(m_trans, m_offset, [w, h])
          {
            min_lng: [nw[0,0], ne[0,0], sw[0,0], se[0,0]].min,
            min_lat: [nw[1,0], ne[1,0], sw[1,0], se[1,0]].min,
            max_lng: [nw[0,0], ne[0,0], sw[0,0], se[0,0]].max,
            max_lat: [nw[1,0], ne[1,0], sw[1,0], se[1,0]].max,
            se: se,
            sw: sw,
            ne: ne,
            nw: nw
          }
        end

        #
        # Build square container for floors
        #
        def buildFloorsContainer(lim)
          lat = (lim[:max_lat].abs - lim[:min_lat].abs).abs
          lng = (lim[:max_lng].abs - lim[:min_lng].abs).abs

          if lat > lng
            dif = lat - lng;
            lim[:max_lng] = lim[:max_lng] + dif/2
            lim[:min_lng] = lim[:min_lng] - dif/2
          elsif lng > lat
            dif = lng - lat;
            lim[:max_lat] = lim[:max_lat] + dif/2
            lim[:min_lat] = lim[:min_lat] - dif/2
          end
          lim
        end

        #
        # Get Trans Matrix for 3d floor map container
        #
        def getTransMatrix(limits)
          # define default size for access points 3d floors container
          s = 550

          matrix = Matrix[
            [0, 0, 0, 0, 1, 0],
            [0, 0, 0, 0, 0, 1],
            [s, 0, 0, 0, 1, 0],
            [0, 0, s, 0, 0, 1],
            [s, s, 0, 0, 1, 0],
            [0, 0, s, s, 0, 1]]

          lat_lng_column = Matrix.column_vector(
            [limits[:min_lng], limits[:min_lat], limits[:max_lng],
             limits[:min_lat], limits[:max_lng], limits[:max_lat]])

          # The solution of the system above
          sol = matrix.inverse * lat_lng_column

          # This matrix lets us transform every feet point in lat-lng points
          base_transf = Matrix[
            [sol[0, 0], sol[1, 0]],
            [sol[2, 0], sol[3, 0]]
          ]

          # The difference between the two reference systems
          diff = Matrix.column_vector([sol[4, 0], sol[5, 0]])

          {
            matrix_trans: base_transf.to_a,
            matrix_offset: diff.to_a
          }
        end

        #
        # Initialize domains and switches depending on where user is
        # managing access points
        #
        def initialize_domain_and_flow_sensors
          if @sensor.is_domain? || @sensor.is_client_proxy?
            @switches = current_user.sensor.self_and_descendants.where(
              is_deleted: false,
              type: [Sensor::FLOW, Sensor::MERAKI]
            )
          elsif @sensor.is_flow? || @sensor.is_meraki?
            @domains = current_user.sensor.self_and_descendants.where(
              is_deleted: false,
              type: Sensor::DOMAIN
            )
          end
        end

        #
        # Check if user can manage selected floor and switch
        #
        def permissions_on_selected_switch_and_floor
          ap_params = params.fetch(:access_point, {})
          floor = Sensor.find_by(id: ap_params[:sensor_id])
          switch = Sensor.find_by(id: ap_params[:switch_id])
          return if (floor.nil? || current_user.can?(:manage, floor)) &&
                    (switch.nil? || current_user.can?(:manage, switch))
          respond_to do |format|
            format.html do
              redirect_to sensor_path(@sensor),
                alert: "#{I18n.t 'lib_flow.privileges'}"
            end
            format.json do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
            format.js do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
          end
        end

        def permissions_on_access_point
          ap_params = params.fetch(:access_point, {})
          unless ap_params[:id].nil?
            @access_point = AccessPoint.find(ap_params[:id])
          end
          return if @access_point && @access_point.manage_permission?
          respond_to do |format|
            format.html do
              redirect_to sensor_path(@sensor),
                alert: "#{I18n.t 'lib_flow.privileges'}"
            end
            format.json do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
            format.js do
              render json: { error: "#{I18n.t 'lib_flow.privileges'}" },
                     status: :unauthorized
            end
          end

        end

        def set_flow_sensors_per_page
          @sensors_page = Sensor::SENSORS_PAGE
        end

        def redirect_to_view
          redirect_to view_sensor_path(@sensor) if @sensor.floor?
        end

        def sort_column
          # Allow just a one-word lenght paramater, to avoid SQLi
          if params[:sort] && params[:sort].match(/^\w+$/)
            params[:sort]
          else
            "created_at"
          end
        end

        def sort_direction
          %w[asc desc].include?(params[:direction]) ? params[:direction] : "desc"
        end

        def access_point_params
          params.require(:access_point)
            .permit(:name, :ip_address, :mac_address, :latitude, :longitude,
                    :switch_name, :switch_id, :sensor_id, :x, :y, :description)
        end

        def check_permissions
          if params[:id].nil?
            redirect_to view_sensor_path(current_user.sensor_id)
          else
            @sensor = Sensor.find(params[:id])
            return if current_user.can?(:read, @sensor)
            error_404('')
          end
        end
      end
    end
  end
end
