require 'test_helper'

class SensorsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      RedBorder.send(:remove_const, 'LICENSING_MODE')
      RedBorder::LICENSING_MODE = 'global'

      @admin = users(:admin)
      @root = sensors(:root)
      sign_in @admin
    end

    should 'delete all APs from root' do
      request.env["HTTP_REFERER"] = '/sensors/tree'
      delete :delete_all_access_points, id: @root.id
      assert_response :found
    end

    should 'create a flow sensor in a proxy' do
      @proxy = sensors(:client_proxy)
      Sensor.any_instance.expects(:data_to_client_proxy_flow).at_least_once
      assert_difference 'Sensor.count', 1 do
        post 'create_flow',
             sensor: { type: Sensor::FLOW, name: 'flow_proxy',
                       ip: '1.2.3.4', parent_id: @proxy.id,
                       license_id: licenses(:two).id }
      end
    end

    should 'create a flow sensor outside a proxy' do
      @namespace = sensors(:namespace)
      Sensor.any_instance.expects(:data_to_client_proxy_flow).never
      assert_difference 'Sensor.count', 1 do
        post 'create_flow',
             sensor: { type: Sensor::FLOW, name: 'flow_not_proxy',
                       ip: '1.2.3.5', parent_id: @namespace.id,
                       license_id: licenses(:two).id }
      end
    end
  end
end
