require 'test_helper'

#
# Test Access Point View
#
class AccessPointsTest < ActionDispatch::IntegrationTest
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'Navigation on Access Points View' do
    setup do
      @admin = users(:admin_domain)
      @aps = @admin.sensor.access_points
      # Login as admin
      login_as(@admin, scope: :user)
    end

    # Test if every AP is handled in AP filters
    should 'handle every AP' do
      visit "/sensors/#{@admin.sensor.id}/view"
      wait_for_ajax(page)
      assert(page.has_css?('.access_points_status', count: 1),
             'You must be access points status buttons')
      up =
        page.find('.access_points_status a[data-status="1"]')['data-count'].to_i
      down =
        page.find('.access_points_status a[data-status="2"]')['data-count'].to_i
      unknown =
        page.find('.access_points_status a[data-status="0"]')['data-count'].to_i
      assert_equal(@aps.count, up + down + unknown,
                   'Every AP must be handled')
    end

    # Test search box
    should 'Search for an AP' do
      visit "/sensors/#{@admin.sensor.id}/view"
      fill_in('search', with: @aps.first.mac_address)
      page.execute_script("apply_ap_filter($('#search'))")
      wait_for_ajax(page)
      assert(page.has_css?('#access_points_from_sensor table tbody tr',
                           count: 1))
    end
  end
end
