require 'test_helper'

class SensorsTest < ActionDispatch::IntegrationTest
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'admin user' do
    # Get first user
    setup do
      @user = users(:admin)
      @sensor = sensors(:flow)
      # Login him
      login_as(@user, scope: :user)

      chef = ChefStub.new
      Chef::Node.stubs(:load).returns(chef)
      Chef::Role.stubs(:load).returns(chef)
      Chef::Node.stubs(:from_hash).returns(chef)
      Chef::Role.stubs(:from_hash).returns(chef)

      active_aps = ['aa:aa:aa:aa:aa:aa', 'aa:aa:aa:aa:aa:ab']
      @aps = []

      @aps << ApState.new(
        nil,
        {
          'bytes' => 0,
          'wireless_station' => active_aps[0],
          'wireless_station_name' => 'Unnamed',
          'status' => 'on'
        }
      )

      @aps << ApState.new(
        nil,
        {
          'bytes' => 0,
          'wireless_station' => active_aps[1],
          'wireless_station_name' => 'Unnamed',
          'status' => 'on'
        }
      )

      @aps << ApState.new(
        nil,
         {
          'bytes' => 0,
          'wireless_station' => 'aa:aa:aa:aa:aa:ac',
          'wireless_station_name' => 'Unnamed',
          'status' => 'off'
        }
      )

      ApState.stubs(:get_active_access_points).returns(active_aps)
      ApState.stubs(:get_access_points).returns(@aps)
    end

    should 'scan APs' do
      visit "/sensors/#{@sensor.id}/view"
      find_link('Options').click
      find_link('Scan Access Points').click
      aps_count = AccessPoint.where(switch_id: @sensor.id).count
      assert page.has_css?('table tbody tr', count: aps_count)
    end
  end
end
