require 'test_helper'

#
# Testing for Access Point model. It should test validations and CRUD
# operations
#
class AccessPointTest < ActiveSupport::TestCase
  # Test relations
  should belong_to :sensor

  # Presence
  should validate_presence_of :name
  should validate_presence_of :mac_address

  # Uniqueness
  should validate_uniqueness_of(:mac_address).case_insensitive
  

  context 'An access point' do
    setup do
      @ap = access_points(:ap6)
      @admin = users(:admin)
      @non_admin = users(:user1)
    end

    should 'be managed by user with permissions' do
      assert @ap.manage_permission?(@admin), 'Admin must manage an AP'
      assert_not @ap.manage_permission?(@non_admin),
                 'Non Admin user must not manage an AP'
    end

    should 'return status string' do
      assert @ap.status_string == 'Unknown'
      @ap.status = 1
      assert @ap.status_string == 'Up'
      @ap.status = 2
      assert @ap.status_string == 'Down'
    end

    should 'not save enrichment nil values' do
      @ap.enrichment = { 'market_uuid' => 'MR-1990842277207996',
                         'building_uuid' => nil,
                         'floor_uuid' => nil }
      @ap.save
      assert @ap.enrichment == { 'market_uuid' => 'MR-1990842277207996' }
    end
  end
end
