require 'test_helper'

#
# Test methods added for Traffic class
#
class TrafficTest < ActiveSupport::TestCase
  context 'An organization user' do
    setup do
      @user = users(:user2)
      User.current_user = @user

      # It's neccessary to include the time ranges this way because
      # it fails if we put @start_time in the should section in function
      # of @end_time
      @end_time = Time.at(Time.now.to_i)
      @start_time_1 = @end_time - sensors(:organization).visualization_range.to_i.days
      @start_time_2 = @end_time - 1.week
      @start_time_3 = @end_time - 1.hour

      @query_params = {
        query_type: :group_by,
        start_time: @start_time_1,
        end_time: @end_time,
        group_by: [:wan_ip],
        limit: 20,
        granularity: :minute,
        sort: :timestamp,
        direction: :DESCENDING,
        range: :last_1_hour,
        aggregation: :bytes,
        remove_zeros: false,
        skip_empty: true,
        controller: :traffics,
        timezone: 'UTC',
        search: {},
        sensors: { sensor_uuid: [] },
        data_sources: 'rb_flow'
      }

      @namespaces = @user.data_sources_ids
      data_sources = @namespaces.map { |x| "rb_flow_#{x}"}
      Redborder::DataSource.stubs(:data_sources).returns(data_sources)
    end

    should 'view limited data' do
      Redborder::Rbdruid.expects(:query).with{ |*args|
        args[0][:start_time] == @start_time_1
      }.returns([]).at_least_once

      Traffic.where(
        query_type: @query_params[:query_type],
        start_time: @start_time_1,
        end_time: @end_time,
        group_by: @query_params[:group_by],
        limit: @query_params[:limit]
      )
    end

    should 'use all namespaces if include_all_namespaces is true' do
      data_sources = Redborder::DataSource.data_sources(@namespaces, :traffics, @user)
      @user.active_data_sources = [@user.data_sources_ids.first]

      Redborder::Rbdruid.expects(:query).with{ |*args|
        args[0][:data_sources] == data_sources
      }.returns([]).at_least_once

      Traffic.where(
        query_type: @query_params[:query_type],
        start_time: @start_time_2,
        end_time: @end_time,
        group_by: @query_params[:group_by],
        include_all_namespaces: true,
        limit: @query_params[:limit]
      )
    end

    should 'view requested data' do
      Redborder::Rbdruid.expects(:query).with{ |*args|
        args[0][:start_time] == @start_time_3
      }.returns([]).at_least_once

      Traffic.where(
        query_type: @query_params[:query_type],
        start_time: @start_time_3,
        end_time: @end_time,
        group_by: @query_params[:group_by],
        limit: @query_params[:limit]
      )
    end
  end
end
