/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

$(document).on('click', 'li.rule > .row > .action > .dropdown > .dropdown-toggle.disabled', function(e){
  e.preventDefault();
  return false;
});

var csrf = $('meta[name="csrf-token"]').attr('content');

var SearchRule;
SearchRule = function() {
  function SearchRule(selectData, submit_url, callback) {
    var self = this;
    self.html = Handlebars.templates['search-rule']({});
    self.columns = selectData.columns;

    //Translating from init.rb
    for(var i = 0; i < self.columns.length; i++) {
      if (self.columns[i].value.startsWith("tr")) {
        self.columns[i].value = I18n.t(self.columns[i].value.slice(3))
      }
    }

    self.operators = selectData.operators;
    self.selectData = selectData;
    self.factory_policy_id = selectData.factory_policy_id;
    self.category_id = selectData.category_id;
    self.group_id = selectData.group_id;
    self.family_id = selectData.family_id;
    self.flowbit_id = selectData.flowbit_id;
    self.flowbit_state_id = selectData.flowbit_state_id;
    self.protocol = selectData.protocol;
    self.severity = selectData.severity;
    self.rule_source_id = selectData.rule_source_id;
    self.action_id = selectData.action_id;
    self.class_type_name = selectData.class_type_name;
    self.is_modified = selectData.is_modified;
    self.default_enabled = selectData.default_enabled;
    self.is_new = selectData.is_new;
    self.is_deprecated = selectData.is_deprecated;

    self.submit_url = submit_url;


    self.init();
    return self;
  }

  SearchRule.prototype = {

    init: function(callback) {
      var self = this;
      var width = "100%";

      self.factory_policy_id_html = Handlebars.templates['select']({
        name: "factory_policy_id-select",
        width: width,
        multiple: false,
        data: self.factory_policy_id
      });

      self.category_id_html = Handlebars.templates['select']({
        name: "category_id-select",
        width: width,
        multiple: false,
        data: self.category_id
      });

      self.group_id_html = Handlebars.templates['select']({
        name: "group_id-select",
        width: width,
        multiple: false,
        data: self.group_id
      });

      self.family_id_html = Handlebars.templates['select']({
        name: "family_id-select",
        width: width,
        multiple: false,
        data: self.family_id
      });

      self.flowbit_id_html = Handlebars.templates['select']({
        name: "flowbit_id-select",
        width: width,
        multiple: false,
        data: self.flowbit_id
      });

      self.flowbit_state_id_html = Handlebars.templates['select']({
        name: "flowbit_state_id-select",
        width: width,
        multiple: false,
        data: self.flowbit_state_id
      });

      self.protocol_html = Handlebars.templates['select']({
        name: "protocol-select",
        width: width,
        multiple: false,
        data: self.protocol
      });

      self.severity_html = Handlebars.templates['select']({
        name: "severity-select",
        width: width,
        multiple: false,
        data: self.severity
      });

      self.rule_source_id_html = Handlebars.templates['select']({
        name: "rule_source_id-select",
        width: width,
        multiple: false,
        data: self.rule_source_id
      });

      self.action_id_html = Handlebars.templates['select']({
        name: "action_id-select",
        width: width,
        multiple: false,
        data: self.action_id
      });

      self.class_type_name_html = Handlebars.templates['select']({
        name: "class_type_name-select",
        width: width,
        multiple: false,
        data: self.class_type_name
      });

      self.is_modified_html = Handlebars.templates['select']({
        name: "is_modified-select",
        width: width,
        multiple: false,
        data: self.is_modified
      });

      self.default_enabled_html = Handlebars.templates['select']({
        name: "default_enabled-select",
        width: width,
        multiple: false,
        data: self.default_enabled
      });

      self.is_new_html = Handlebars.templates['select']({
        name: "is_new-select",
        width: width,
        multiple: false,
        data: self.is_new
      });

      self.is_deprecated_html = Handlebars.templates['select']({
        name: "is_deprecated-select",
        width: width,
        multiple: false,
        data: self.is_deprecated
      });

      self.columns_html = Handlebars.templates['select']({
        name: "column-select",
        placeholder: "Choose a query term...",
        data: {
          value: self.columns
        }
      });

      self.operators_html = function($html, data) {
        var select = Handlebars.templates['select']({
          name: "operators-select",
          data: {
            value: data
          }
        });

        $html.find('div.operator-select').html(select);
      };

    },

    searchUI: function(data, callback) {
      var self = this;
      $('#content #title').after(Handlebars.templates['search'](data));

      $('.search-content-add').on('click', function(e) {
        e.preventDefault();
        self.add(this);
      });

      $('.search-content-remove').on('click', function(e) {
        e.preventDefault();
        self.remove(this);
      });

      $('div.search-content-enable input').on('click', function() {
        if ($(this).is(':checked')) {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', false).css('opacity', 1);
        } else {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', true).css('opacity', 0.8);
        };

        $('.add_chosen').trigger("chosen:updated");
      });

      $('button.submit-search').on('click', function(e) {
        e.preventDefault();
        self.submit();
      });

      $('#content #title').on('click', 'a.reset-search-form', function(e) {
        e.preventDefault();

        $('#search .rules').empty();

        self.add();
        self.add();
        self.add();
      });

      if (callback && (typeof callback === "function")) {
        callback();
      };
    },

    add: function(that, newOptions, callback) {
      var self = this;
      var options = {};

      if (options && (typeof options === "object")) {
        options = newOptions;
      };

      var $html = $(self.html);
      $html.find('div.column-select').html(self.columns_html);

      if (that && (typeof that === "object")) {
        $(that).parents('.search-content-box').after($html);
      } else {
        $('#search .rules').append($html);
      };

      $html.find('.add_chosen')
      .chosen({allow_single_deselect: false, width: '100%'})
      .change(function(event, data) {
        var value = $(this).val();
        var that = $(this).parents('.search-content-box');

        if (value === "factory_policy_id") {
          that.find('.value').html(self.factory_policy_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "category_id") {
          that.find('.value').html(self.category_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "group_id") {
          that.find('.value').html(self.group_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "family_id") {
          that.find('.value').html(self.family_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "flowbit_id") {
          that.find('.value').html(self.flowbit_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "flowbit_state_id") {
          that.find('.value').html(self.flowbit_state_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "protocol") {
          that.find('.value').html(self.protocol_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "severity") {
          that.find('.value').html(self.severity_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "rule_source_id") {
          that.find('.value').html(self.rule_source_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "action_id") {
          that.find('.value').html(self.action_id_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "class_type_name") {
           that.find('.value').html(self.class_type_name_html);
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "is_modified") {
           that.find('.value').html(self.is_modified_html);
          self.operators_html($html, self.operators.boolean_input);
        } else if (value === "default_enabled") {
           that.find('.value').html(self.default_enabled_html);
          self.operators_html($html, self.operators.boolean_input);
        } else if (value === "is_new") {
           that.find('.value').html(self.is_new_html);
          self.operators_html($html, self.operators.boolean_input);
        } else if (value === "is_deprecated") {
           that.find('.value').html(self.is_deprecated_html);
          self.operators_html($html, self.operators.boolean_input);
        } else if (value === "rule_sid") {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "rule_gid") {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.selector_input);
        } else if (value === "") {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.text_input);
        } else {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.text_input);
        };

        that.find('.add_chosen').chosen({
          allow_single_deselect: false,
          width: '100%'
        });

        if (callback && (typeof callback === "function")) {
          callback(that);
        };

        that.trigger('rbsearch:search:change');
      });

      $('.search-content-box:first')
      .find('.search-content-remove')
      .css('opacity', 1)
      .unbind('hover')
      .unbind('mouseover');

      return $html;
    },

    remove: function(that) {
      var self = this;
      if ($('.search-content-box').length > 2) {
        $(that).parents('.search-content-box').remove();
      } else if ($('.search-content-box').length == 2) {
        $(that).parents('.search-content-box').remove();

        $('.search-content-box:first')
        .find('.search-content-remove')
        .css('opacity', 0.4)
        .attr('title', I18n.t('javascript.you_must'));
      };
    },

    pack: function() {
      var self = this;
      var matchAll = false;

      if ($('#search select.global-match-setting').val() === "all") {
        matchAll = true;
      };

      var json = {
        match_all: matchAll,
        items: {}
      };

      $('.search-content-box').each(function(index, item) {
        var enabled = $(item).find('.search-content-enable input').is(':checked');
        var column = $(item).find('.column-select select').val();
        var operator = $(item).find('.operator-select select').val();
        var value = $(item).find('.value input, .value select').val();

        if ((column !== "") || (value !== "")) {
          json.items[index] = {
            column: column,
            operator: operator,
            value: value,
            enabled: enabled
          };
        };
      });

      return json;
    },

    submit: function(callback, otherOptions) {
      var self = this;
      var search = self.pack();
      var csrf = $('meta[name="csrf-token"]').attr('content');

      if (typeof otherOptions !== "object") {
        otherOptions = {};
      };

      url = self.submit_url;





      var update_url = function() {
        if (history && history.pushState) {
          history.pushState(null, document.title, url);
        };
      };

      if (callback && (typeof callback === "function")) {
        callback(search, self);
      };

      if (search && !$.isEmptyObject(search.items)) {
        if (otherOptions.search_id) {
          post_to_url(url, {
            match_all: search.match_all,
            search: search.items,
            title: otherOptions.title,
            search_id: ""+otherOptions.search_id+"",
            authenticity_token: csrf
          });
        } else {
          post_to_url(url, {
            match_all: search.match_all,
            search: search.items,
            authenticity_token: csrf
          });
        };
      } else {
        flash_message.push({type: 'error', message: I18n.t('javascript.you_cannot')});
        flash();
      };
    },

    build: function(search) {
      var self = this;

      if (typeof search === "string") {
        var data = JSON.parse(search);
      } else if (typeof search === "object") {
        var data = search;
      } else {
        var data = {};
      };

      if (data.items) {
       var rules = data.items;
      } else if (data.search) {
        var rules = data.search;
      } else {
        var rules = {};
      };

      if (data.match_all && data.match_all === "false") {
       $('select.global-match-setting option[value="any"]').prop('selected', 'selected');
      } else {
        $('select.global-match-setting option[value="all"]').prop('selected', 'selected');
      };

      for (id in rules) {
        var item = rules[id];

        var rule = self.add(false, {});

        var column = rule.find('div.column-select select');

        rule.bind('rbsearch:search:change', function() {
          var operator = $(this).find('div.operator-select select');
          var value = $(this).find('div.value select, div.value input');

          operator.find("option[value='"+item.operator+"']").prop('selected', 'selected');
          operator.trigger("chosen:updated");

          value.prop('value', item.value);

          rule.unbind('rbsearch:search:change');
        });

        rule.attr('data-rule-id', id);

        column.find("option[value='"+item.column+"']").prop('selected','selected');
        column.trigger("chosen:updated").trigger('change');

        if (item.enabled === "false") {
          rule.find('div.search-content-enable input').prop('checked', false);
          rule.find('.value *, .operator-select *, .column-select *')
          .attr('disabled', true).css('opacity', 0.8);
        };
      };

      $('#search .rules .add_chosen').trigger("chosen:updated");
    },

  };

  return SearchRule;

}();

$(document).on('click', 'article#rules input[type=checkbox].rule-select-all', function(e){

  if ($(this).prop("checked")) {
    $('article#rules input[type=checkbox]').prop('checked', true);
  } else {
    $('article#rules input[type=checkbox]').prop('checked', false);
  }

});

$(document).on('click', 'article#rules li.category input[type=checkbox]', function(e){

  var category_id = $(this).parents("li.category").attr("data-category");

  if ($(this).prop("checked")) {
    $('article#rules li[data-category='+ category_id +']:not(.category) input[type=checkbox]').prop('checked', true);

  } else {
    $('article#rules li[data-category='+ category_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules input[type=checkbox].rule-select-all').prop('checked', false);
  }

});

$(document).on('click', 'article#rules li.group input[type=checkbox]', function(e){

  var category_id = $(this).parents("li.group").attr("data-category");
  var group_id    = $(this).parents("li.group").attr("data-group");

  if ($(this).prop("checked")) {
    $('article#rules li[data-category='+ category_id +'][data-group='+ group_id +']:not(.group) input[type=checkbox]').prop('checked', true);
  } else {
    $('article#rules li[data-category='+ category_id +'][data-group='+ group_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules input[type=checkbox].rule-select-all').prop('checked', false);
    $('article#rules li.category[data-category='+ category_id +'] input[type=checkbox]').prop('checked', false);
  }

});

$(document).on('click', 'article#rules li.family input[type=checkbox]', function(e){

  var category_id = $(this).parents("li.family").attr("data-category");
  var group_id    = $(this).parents("li.family").attr("data-group");
  var family_id   = $(this).parents("li.family").attr("data-family");

  if ($(this).prop("checked")) {
    $('article#rules li[data-category='+ category_id +'][data-group='+ group_id +'][data-family='+ family_id +']:not(.family) input[type=checkbox]').prop('checked', true);
  } else {
    $('article#rules li[data-category='+ category_id +'][data-group='+ group_id +'][data-family='+ family_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules input[type=checkbox].rule-select-all').prop('checked', false);
    $('article#rules li.category[data-category='+ category_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules li.group[data-category='+ category_id +'][data-group='+ group_id +'] input[type=checkbox]').prop('checked', false);
  }

});

$(document).on('click', 'article#rules li.rule input[type=checkbox]', function(e){

  var category_id = $(this).parents("li.rule").attr("data-category");
  var group_id    = $(this).parents("li.rule").attr("data-group");
  var family_id   = $(this).parents("li.rule").attr("data-family");

  if (!$(this).prop("checked")) {
    $('article#rules input[type=checkbox].rule-select-all').prop('checked', false);
    $('article#rules li.category[data-category='+ category_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules li.group[data-category='+ category_id +'][data-group='+ group_id +'] input[type=checkbox]').prop('checked', false);
    $('article#rules li.family[data-category='+ category_id +'][data-group='+ group_id +'][data-family='+ family_id +'] input[type=checkbox]').prop('checked', false);
  }

});

$(document).on('click', 'article#rules .action ul.dropdown-menu a', function(e){

  var self = $(this);
  var parent = $(self).parents('li.rule, li.family, li.group, li.category');

  var category_id = $(parent).attr("data-category");
  var group_id    = $(parent).attr("data-group");
  var family_id   = $(parent).attr("data-family");

  if(!parent.hasClass('rule')){
    var text = 'article#rules li';
    if(category_id){text += '[data-category='+ category_id +']'}
    if(group_id){text += '[data-group='+ group_id +']'}
    if(family_id){text += '[data-family='+ family_id +']'}

    parent = $(text);
  }

  parent.find('div.action .dropdown').hide();
  parent.find('div.action').append("<i class='fa fa-spinner fa-spin'></i>");

});

$(document).on('click', 'body.rules.index ul.dropdown-menu.actions a', function(e){
  var action_id = $(this).attr('data-action-id');
  var msg = $(".navbar-buttons input#msg").val();
  $("form#massive_update input#action_id").val(action_id);
  $("form#massive_update input#msg").val(msg);

  $("form#massive_update").submit();

  return false;
});

function trigger_rule_check(){
  $.ajax({
    method: 'GET',
    url: '/rule_versions/check_update',
    dataType: 'script'
  });
}
