#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class ClassificationsController < ApplicationController
  self.view_paths = 'lib/modules/ips/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_filter :require_super_administrative_privileges

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    in_time_machine do
      @classifications = Event.where(
        query_type: :group_by,
        range: :raw_column,
        start_time: Time.now - 1.year,
        end_time: Time.now,
        granularity: :all,
        group_by: 'classification'
      )

      @total = Event.where(
        query_type: :timeseries,
        range: :raw_column,
        start_time: Time.now - 1.year,
        end_time: Time.now,
        granularity: :all
      ).first.events
    end
  end

  def new
    @classification = Classification.new
    render layout: false
  end

  def create
    @classification = Classification.new(classification_params)

    if @classification.save
      @classifications = Classification.paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
      flash[:notice] = "#{I18n.t('controllers.classification_created')}"
    else
      flash[:error] = @classification.errors.full_messages
        .map { |msg| "<li>#{msg}</li>" }.join
    end

    respond_to :js
  end

  def edit
    @classification = Classification.find(params[:id])
    render layout: false
  end

  def update
    @classification = Classification.find(params[:id])

    if @classification.update_attributes(classification_params)
      @classifications = Classification.paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
      flash[:notice] = "#{I18n.t('controllers.classification_updated')}"
    else
      flash[:error] = @classification.errors.full_messages
        .map { |msg| "<li>#{msg}</li>" }.join
    end

    respond_to :js
  end

  def destroy
    @classification = Classification.find(params[:id])
    @classification.destroy

    redirect_to classifications_path,
                notice: "#{I18n.t('controllers.classification_deleted')}"
  end

  private

  def classification_params
    params.require(:classification).permit(:name, :description)
  end
end
