#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class EventsController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  # Includes
  include Redborder::Rbdruid

  self.view_paths = 'lib/modules/ips/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  helper_method :sort_column, :sort_direction

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_saved_ips_filters, only: [
                  :index, :tops, :comparative, :cardinality, :export_raw_to_file
                ]

  before_action :redirect_if_empty, only: [:index, :tops, :map, :comparative,
                                           :cardinality, :heatmap]

  skip_before_action :filter_params

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def lookup
    @lookup = Redborder::Rblookup.new(params[:address])
    render layout: false
  end

  def get_maps_data
    tab = params[:id]
    timezone = params[:timezone]
    limit = params.fetch(:limit, current_user.per_page_count)
    @start_time = params[:start_time]
    @end_time = params[:end_time]

    content = @source.where(
      query_type: :group_by,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: limit,
      sort: @aggregation,
      search: search.dup,
      group_by: @source::COLUMNS[tab.to_sym][:columns].keys,
      search: @filter.get,
      aggregation: @aggregation
    )

    ips = content.collect(&@source::COLUMNS[tab.to_sym][:ips])

    respond_to do |format|
      format.json { render json: Redborder::Geoip.latlong(ips) }
    end
  end

  private

  def redirect_if_empty
    @sensors = Sensor.where type: [Sensor::IPS, Sensor::IPS_PROXY, Sensor::IPS_GENERIC]
    redirect_to help_events_path if @sensors.empty?
  end

  def set_saved_monitor_filters
    @filters = SavedMonitorFilter.where(user_id: current_user.id)
    @filters_public = SavedMonitorFilter.where.not(user_id: current_user.id).select(&:public)
  end

  def get_scatterplot_tab
    columns = @source::COLUMNS[:scatterplot][:columns].keys

    content = @source.where({
      query_type: :group_by,
      range: @range,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: current_user.per_page_count,
      sort: @aggregation,
      group_by: columns,
      aggregation: @aggregation,
      search: @filter.get
    })

    {content: content}
  end

  #
  # Set the filters. Change current filters if params[:search] is defined.
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'ips')
    @filter = Redborder::Filter.new(@source, load_filters)
  end

  def get_gmap_tabs(tab)
    content = @source.where({
      query_type: :group_by,
      range: @range,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: current_user.per_page_count,
      sort: @aggregation,
      aggregation: @aggregation,
      group_by: @source::COLUMNS[tab.to_sym][:columns].keys,
      search: @filter.get
    })

    ips = content.collect(&@source::COLUMNS[tab.to_sym][:ips])
    {content: content, graph: Redborder::Geoip.latlong(ips)}
  end

  def set_saved_ips_filters
    @filters = SavedEventFilter.where(user_id: current_user.id)
    @filters_public = SavedEventFilter.where.not(user_id: current_user.id).select(&:public)
  end
end
