#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ReputationsController < ApplicationController
  self.view_paths = 'lib/modules/ips/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  skip_before_action :authenticate_user!, only: [:iplist, :geoip], raise: false
  before_action :validate_cert,
                :set_reputation_policy_by_sensor, only: [:iplist, :geoip]
  before_action :set_reputation_and_check_privileges,
                only: [:edit, :update, :destroy]
  before_action :set_rep_policy_and_check_privileges,
                only: [:new, :create, :update, :reorder]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def new
    @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]
    @reputation = @reputation_policy.reputations
                  .build(reputation_type: params[:reputation_type])
    render layout: false
  end

  def create
    @reputation = Reputation.create(reputation_params)
  end

  def edit
    render layout: false
  end

  def update
    @reputation.update(reputation_params)
  end

  def destroy
    @reputation.destroy

    if params[:sensor_id]
      sensor = Sensor.find(params[:sensor_id])
      redirect_to sensor_reputation_policy_path(sensor,
                                                @reputation.reputation_policy)
    else
      redirect_to reputation_policy_path(@reputation.reputation_policy)
    end
  end

  def iplist
    if @reputation_policy.nil?
      @iplist = []
    else
      @iplist = @reputation_policy.reputations
                .where(reputation_type: Reputation::IP_NET_TYPE)
    end

    render layout: false
  end

  def geoip
    if @reputation_policy.nil?
      @countries, @continents = [[], []]
    else
      @countries 	= @reputation_policy.reputations
                    .where(reputation_type: Reputation::COUNTRY_TYPE)
      @continents = @reputation_policy.reputations
                    .where(reputation_type: Reputation::CONTINENT_TYPE)
    end

    render layout: false
  end

  def reorder
    reputation_type = params[:reputation_type]
    order = params[:positions].split(';')
    order.each_with_index do |auto_id, position|
      Reputation.find_by(id: auto_id,
                         reputation_type: reputation_type,
                         reputation_policy_id: @reputation_policy.id)
        .update(position: (position + 1))
    end

    render js: ''
  end

  private

  def set_reputation_policy_by_sensor
    real_sensor = Sensor.find(params[:sensor_id])

    @sensor = if real_sensor.is_sensor?
                group_sensor = Sensor.find_by(real_parent: real_sensor,
                                              group_id: params[:group_id].to_i,
                                              type: Sensor::IPS_GROUP)
                Sensor.find_by(real_parent: group_sensor,
                               group_id: params[:group_id].to_i,
                               binding_id: params[:binding_id].to_i,
                               type: Sensor::IPS_BINDING)
              else
                real_sensor
              end
    @reputation_policy = @sensor.reputation_policy
  end

  def set_rep_policy_and_check_privileges
    reputation_policy_id = if params[:reputation]
                             params[:reputation][:reputation_policy_id]
                           else
                             params[:reputation_policy_id]
                           end

    @reputation_policy = ReputationPolicy.find(reputation_policy_id)
    return unless current_user.cannot? :manage, @reputation_policy.owner
    respond_to do |format|
      format.html do
        redirect_to sensors_path,
                    alert: "#{I18n.t('controllers.no_privileges')}"
      end
      format.js do
        render json: { error: "#{I18n.t('controllers.no_privileges')}" },
               status: :unauthorized
      end
    end
  end

  def set_reputation_and_check_privileges
    @reputation = Reputation.find(params[:id])
    return unless current_user.cannot? :manage, @reputation.reputation_policy.owner
    respond_to do |format|
      format.html do
        redirect_to sensors_path,
                    alert: "#{I18n.t('controllers.no_privileges')}"
      end
      format.js do
        render json: { error: "#{I18n.t('controllers.no_privileges')}" },
               status: :unauthorized
      end
    end
  end

  def reputation_params
    params.require(:reputation)
      .permit(:reputation_policy_id, :action, :value, :reputation_type)
  end
end
