#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class RulesController < ApplicationController
  include Redborder::Rule

  self.view_paths = "lib/modules/ips/app/views"
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  skip_before_filter :authenticate_user!, only: :index, raise: false

  before_filter only: :index do |controller|
    if controller.request.format.text?
      validate_cert
    else
      authenticate_user!
    end
  end

  before_action :set_policy, :set_rule_db_version
  before_action :set_query, only: [
    :index, :groups, :families, :rules, :massive_update, :update_action]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  # GET /rules
  # GET /rules.json
  def index
    respond_to do |format|
      format.html do
        @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]

        query = Category.joins(@query_tables).where(@is_query)

        @not_query.each do |item|
          query = query.where.not(item)
        end

        query = query.where(
          'LOWER(rules.text) like ?',
          "%#{@msg.downcase.split.join('%')}%") if @msg.present?

        @categories = query.order(:name).group('categories.id')

        render 'rule_db_version_index' if @rule_db_version
      end

      format.text do
        real_sensor = Sensor.find(params[:sensor_id])

        @sensor =
          # Sensor IPS v3
          if real_sensor.is_sensor?
            group_sensor = Sensor.where(real_parent: real_sensor,
                                        group_id: params[:group_id].to_i).first

            Sensor.where(real_parent: group_sensor,
                         group_id: params[:group_id].to_i,
                         binding_id: params[:binding_id].to_i).first
          # Sensor IPS v2
          else
            real_sensor
          end

        case params[:type]
        when 'active_rules'
          DittoCode::Exec.if 'ENTERPRISE' do
          @policy_rules = @sensor.policy_rules.includes(:rule, :action)
                          .where.not(
                            rules: { category_id: [Category.preprocessors.id,
                                                   Category.file_capture.id] })
                          .references(:rules)
          end
          DittoCode::Exec.if 'COMMUNITY' do
          @policy_rules = @sensor.policy_rules.includes(:rule, :action)
                          .where.not(
                            rules: { category_id: [Category.preprocessors.id,
                                                   Category.file_capture.id] })
                          .references(:rules).take(3)
          end
          render layout: false
        when 'preprocessor_rules'
          @policy_rules = PolicyRule.includes(:rule).where(
            policy: @sensor.policy,
            rules: { category_id: Category.preprocessors.id }
          )
          render layout: false
        when 'file_capture_rules'
          @policy_rules = PolicyRule.includes(:rule).where(
            policy: @sensor.policy,
            rules: { category_id: Category.file_capture.id }
          )
          render 'file_capture', layout: false
        end
      end

      format.csv { export_csv }
    end
  end

  def groups
    @category = Category.find(params[:category_id])

    (@is_query[:rules] ||= {})[:category_id] = @category.id

    query = Group.joins(@query_tables).where(@is_query)

    @not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?',
                        "%#{@msg.downcase.split.join('%')}%") if @msg.present?

    @groups = query.order(:name).group('groups.id')

    render 'rule_db_version_groups' if @rule_db_version
  end

  def families
    @category = Category.find(params[:category_id])
    @group = Group.find(params[:group_id])

    (@is_query[:rules] ||= {})[:category_id] = @category.id
    @is_query[:rules][:group_id] = @group.id

    query = Family.joins(@query_tables).where(@is_query)

    @not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?',
                        "%#{@msg.downcase.split.join('%')}%") if @msg.present?

    @families = query.order(:name).group('families.id')

    render 'rule_db_version_families' if @rule_db_version
  end

  def rules
    @category = Category.find(params[:category_id])
    @group    = Group.find(params[:group_id])
    @family   = Family.find(params[:family_id])

    @actions = Action.all

    (@is_query[:rules] ||= {})[:category_id] = @category.id
    @is_query[:rules][:group_id] = @group.id
    @is_query[:rules][:family_id] = @family.id

    query =
    if @policy && @query_tables.is_a?(Symbol)
      Rule.includes(:factory_policies, :flowbits, policy_rules: [:action])
      .where(@is_query[:rules])
    elsif @policy
      Rule.includes(:factory_policies, :flowbits, policy_rules: [:action])
      .joins(@query_tables[:rules]).where(@is_query)
    elsif @query_tables.is_a?(Symbol)
      Rule.includes(:factory_policies, :flowbits, policy_rules: [:action])
      .where(@is_query[:rules])
    else
      Rule.includes(:factory_policies, :flowbits, policy_rules: [:action])
      .joins(@query_tables[:rules]).where(@is_query)
    end

    @not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?',
                        "%#{@msg.downcase.split.join('%')}%") if @msg.present?

    if @policy
      @rules = (query.where(policy_rules: { policy: @policy }) + query)
               .uniq.sort { |r1, r2| r1.short_msg <=> r2.short_msg }
    else
      @rules = query
      render 'rule_db_version_rules'
    end
  end

  def update_action
    @action = Action.find(params[:action_id])
    @params_rule = params.select do |k, _v|
      %w(category_id group_id family_id id).include? k
    end

    (@is_query[:rules] ||= {})[:category_id] = params[:category_id] if params[:category_id]
    @is_query[:rules][:group_id] = params[:group_id] if params[:group_id]
    @is_query[:rules][:family_id] = params[:family_id] if params[:family_id]
    @is_query[:rules][:id] = params[:id] if params[:id]

    query =
    if @query_tables.is_a?(Symbol)
      Rule.where(@is_query[:rules])
    else
      Rule.joins(@query_tables[:rules]).where(@is_query)
    end

    @not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?',
                        "%#{@msg.downcase.split.join('%')}%") if @msg.present?

    rules = query
    rule_ids = rules.map(&:id)

    PolicyRule.transaction do
      if @action.name == 'clear'
        PolicyRule.where(rule_id: rule_ids, policy: @policy).delete_all
      else
        edit_list = PolicyRule.where(rule_id: rule_ids, policy: @policy)
        new_list = rule_ids - edit_list.map(&:rule_id)

        if new_list.present?
          values = new_list.map! { |r| "(#{r},#{@policy.id},#{@action.id},#{current_user.id})" }.join(',')
          sql = "INSERT INTO policy_rules (rule_id,policy_id,action_id,user_id) VALUES #{values}"
          ActiveRecord::Base.connection.execute(sql)
        end

        edit_list.update_all(
          action_id: @action.id, user_id: current_user.id
        )
      end
    end

    Sensor.where(policy: @policy).update_all(need_apply: true)
  end

  def show
    @rule = Rule.find(params[:id])
    @action = params[:event_action]

    respond_to do |format|
      format.html { render layout: false }
      format.js
    end
  end

  def search
    @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]
    @filter = {}
    items = {}
    index = 0

    unless params[:new_filter].present?
      session[:rule][:filter].each do |k, v|
        v.each do |key, value|
          items[index] = { column: k.to_s, operator: value[:operator],
                           value: key.to_s, enabled: true }
          index += 1
        end
      end
    end

    @filter = { match_all: false, items: items } unless items.blank?
  end

  def results
    @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]

    if params.key?(:search) && !params[:search].blank?
      if params[:search].is_a?(String)
        value ||= JSON.parse(params[:search])
        params[:search] = value
      end

      session[:rule][:filter] = {}

      enabled_count = 0
      for item in params[:search] do
        x = item.last
        enabled = (x['enabled'] || x[:enabled]).to_s

        if !enabled.blank?
          enabled_count += 1 if enabled.to_s == 'true'
        else
          enabled_count += 1
        end
      end

      if enabled_count == 0
        redirect_to :back,
                    alert: 'There was a problem parsing the search rules.'
      end

      params[:search].each do |key, value|

        next if !value[:enabled]

        value[:value] = nil if value[:value].blank?

        if session[:rule][:filter][value[:column].to_sym] and session[:rule][:filter][value[:column].to_sym][value[:value]]
          session[:rule][:filter][value[:column].to_sym][value[:value]] = {operator: value[:operator]}
        elsif session[:rule][:filter][value[:column].to_sym]
          session[:rule][:filter][value[:column].to_sym].merge! ({value[:value] => {operator: value[:operator]}})
        elsif session[:rule][:filter]
          session[:rule][:filter].merge! ({value[:column].to_sym => {value[:value] => {operator: value[:operator]}}})
        end

      end

    end

    if @sensor && @policy
      redirect_to sensor_policy_rules_path(@sensor, @policy)
    elsif @policy
      redirect_to policy_rules_path(@policy)
    else
      redirect_to rule_db_version_rules_path(@rule_db_version)
    end
  end

  def delete_filter
    vals    = []
    values  = params[:values]

    if params[:id].to_s == 'all'
      session[:rule][:filter] = {}
    else
      values = [] if (values.nil? || !values.respond_to?('each'))

      values << params[:value]

      values.each do |v|
        vals << v
      end

      vals.each do |val|
        if session[:rule][:filter][params[:id].to_sym] && session[:rule][:filter][params[:id].to_sym][val]
          if session[:rule][:filter][params[:id].to_sym].count > 1
            session[:rule][:filter][params[:id].to_sym].delete(val)
          else
            session[:rule][:filter].delete(params[:id].to_sym)
          end
        elsif session[:rule][:filter][params[:id].to_sym]
          session[:rule][:filter][params[:id].to_sym].merge! ({ val => { operator: 'not' } })
        else
          session[:rule][:filter].merge! ({ params[:id].to_sym => { val => { operator: 'not' } } })
        end
      end

    end

    redirect_to :back
  end

  def massive_update
    if /\Adefault_/.match params[:action_id]
      massive_update_default_action
    else
      massive_update_one_action
    end
  end

  def import
    render layout: false
  end

  def import_file
    s3_credentials = YAML.load(File.read(AWS_FILE_PATH))[Rails.env]
    s3_credentials["bucket"].chomp!('/')
    s3_credentials.symbolize_keys!

    s3_client = Aws::S3::Client.new(
      access_key_id: s3_credentials[:access_key_id],
      secret_access_key: s3_credentials[:secret_access_key],
      region: 'us-east-1',
      endpoint: "https://" + s3_credentials[:s3_host_name],
      force_path_style: true
    )

    filename = params[:policy][:csv_file].original_filename.tr(' ', '_')
    s3_resource = Aws::S3::Resource.new(client: s3_client)
    bucket = s3_resource.bucket(s3_credentials[:bucket])
    object = bucket.object("imports/#{filename}")

    object.write(file: params[:policy][:csv_file].tempfile.path)

    @policy.update(updating: true)

    Delayed::Job.enqueue(
      Redborder::Jobs::ImportRulesToPolicyJob.new(
        @policy.id, params[:add_rules],
        filename,
        current_user.id
      ),
      priority: 10, run_at: Time.now
    )

    @sensor = @policy.owner

    redirect_to sensor_policies_path(@sensor), notice: "#{I18n.t('controllers.process_enqueue')}"
  end

  def new_note
    @rule_note = RuleNote.new(rule_id: params[:rule_id])
    render layout: false
  end

  def create_note
    rule_note = RuleNote.new(note_params)
    rule_note.user = current_user
    @rule = rule_note.rule
    rule_note.save

    render 'rule_notes'
  end

  def destroy_note
    rule_note = RuleNote.find(params[:rule_note_id])

    if !current_user.admin && current_user.id != rule_note.user_id
      render js: '', status: :forbidden && return
    end

    @rule = rule_note.rule
    rule_note.destroy

    render 'rule_notes'
  end

  protected

    def massive_update_one_action
      @action = Action.find(params[:action_id])
      rule_ids = []

      query =
      if @query_tables.is_a?(Symbol)
        Rule.where(@is_query[:rules])
      else
        Rule.joins(@query_tables[:rules]).where(@is_query)
      end

      @not_query.each do |item|
        query = query.where.not(item)
      end

      query = query.where('LOWER(rules.text) like ?',
                          "%#{@msg.downcase.split.join('%')}%") if @msg.present?

      rule_ids += query.where(category_id: params['category-selector'])
                  .map(&:id) if params['category-selector']

      params['group-selector'].try('each') do |group|
        category_id, group_id = group.split('-')
        rule_ids += query.where(category_id: category_id,
                                group_id: group_id).map(&:id)
      end

      params['family-selector'].try('each') do |family|
        category_id, group_id, family_id = family.split('-')
        rule_ids += query.where(category_id: category_id, group_id: group_id,
                                family_id: family_id).map(&:id)
      end

      rule_ids += query.where(id: params['rule-selector'])
                  .map(&:id) if params['rule-selector']

      # Avoid working on the same tasks multiple times if you activate some
      # groups/families inside a category(also activated) before massive_update.
      rule_ids.uniq!

      PolicyRule.transaction do
        if @action.name == 'clear'
          PolicyRule.where(rule_id: rule_ids, policy: @policy).delete_all
        else
          edit_list = PolicyRule.where(rule_id: rule_ids, policy: @policy)
          new_list = rule_ids - edit_list.map(&:rule_id)

          if new_list.present?
            values = new_list.map! { |r| "(#{r},#{@policy.id},#{@action.id},#{current_user.id})" }.join(',')
            sql = "INSERT INTO policy_rules (rule_id,policy_id,action_id,user_id) VALUES #{values}"
            ActiveRecord::Base.connection.execute(sql)
          end

          edit_list.update_all(
            action_id: @action.id, user_id: current_user.id
          )
        end
      end

      Sensor.where(policy: @policy).update_all(need_apply: true)

      if params[:sensor_id]
        @sensor = Sensor.find(params[:sensor_id])
        redirect_to sensor_policy_rules_path(@sensor, @policy),
                    notice: "#{rule_ids.count} #{I18n.t('controllers.rules_been_updated')}"
      else
        redirect_to policy_rules_path(@policy),
                    notice: "#{rule_ids.count} #{I18n.t('controllers.rules_been_updated')}"
      end
    end

    def massive_update_default_action
      factory_policy_name = /\Adefault_(.+)\z/.match(params[:action_id])[1]
      factory_policy = FactoryPolicy.find_by name: factory_policy_name

      rule_ids = []

      query =
      if @query_tables.is_a?(Symbol)
        Rule.where(@is_query[:rules])
      else
        Rule.joins(@query_tables[:rules]).where(@is_query)
      end

      @not_query.each do |item|
        query = query.where.not(item)
      end

      query = query.where('LOWER(rules.text) like ?',
                          "%#{@msg.downcase.split.join('%')}%") if @msg.present?

      query = query.includes(:factory_policy_rules).where(
        factory_policy_rules: { factory_policy_id: factory_policy.id }
      )

      rule_ids += query.where(category_id: params['category-selector'])
                  .map(&:id) if params['category-selector']

      params['group-selector'].try('each') do |group|
        category_id, group_id = group.split('-')
        rule_ids += query.where(category_id: category_id,
                                group_id: group_id).map(&:id)
      end

      params['family-selector'].try('each') do |family|
        category_id, group_id, family_id = family.split('-')
        rule_ids += query.where(category_id: category_id, group_id: group_id,
                                family_id: family_id).map(&:id)
      end

      rule_ids += query.where(id: params['rule-selector'])
                  .map(&:id) if params['rule-selector']

      PolicyRule.transaction do

        rule_ids.each do |rule_id|
          action_id = FactoryPolicyRule.find_by(
            rule_id: rule_id, factory_policy: factory_policy
          ).action_id
          policy_rule = PolicyRule.where(
            rule_id: rule_id, policy: @policy
          ).first_or_create(action_id: action_id, user: current_user)
          policy_rule.update(action_id: action_id, user_id: current_user.id)
        end

      end

      Sensor.where(policy: @policy).update_all(need_apply: true)

      if params[:sensor_id]
        @sensor = Sensor.find(params[:sensor_id])
        redirect_to sensor_policy_rules_path(@sensor, @policy),
                    notice: "#{rule_ids.count} #{I18n.t('controllers.rules_been_updated')}"
      else
        redirect_to policy_rules_path(@policy),
                    notice: "#{rule_ids.count} #{I18n.t('controllers.rules_been_updated')}"
      end
    end

  private
    # Use callbacks to share common setup or constraints between actions.
    def set_sensor
      if request.format.text?
        real_sensor = Sensor.find(params[:sensor_id])
        group_sensor = Sensor.where(real_parent: real_sensor,
                                    group_id: params[:group_id].to_i).first
        @sensor = Sensor.where(real_parent: group_sensor,
                               group_id: params[:group_id].to_i,
                               binding_id: params[:binding_id].to_i).first
      else
        @sensor = Sensor.find(params[:sensor_id]) unless params[:sensor_id].nil?
      end
    end

    def set_policy
      if params[:policy_id]
        @policy = Policy.find(params[:policy_id])
        return error_403("#{I18n.t('controllers.policy_updating')}") if @policy.updating
      end
    end

    def set_rule_db_version
      @rule_db_version = RuleDbVersion.find(
        params[:rule_db_version_id]
      ) if params[:rule_db_version_id]
    end

    def event_filter_params
      params.require(:event_filter).permit(
        :filter_type, :source_ip, :destination_ip, :count,
        :seconds, :rule_sid, :rule_gid)
    end

    def note_params
      params.require(:rule_note).permit(:body, :rule_id)
    end

    def export_csv
      filename = in_time_machine { Time.now.strftime('%Y%m%d%H%M%S') + '-SignaturePolicy-' + @policy.clean_name + '.csv' }
      content = @policy.filtered_rules_to_csv_format(@is_query, @not_query, @msg, @query_tables)
      send_data content, filename: filename
    end
end
