#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class StoredPoliciesController < ApplicationController

  self.view_paths = "lib/modules/ips/app/views"
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  before_action :set_sensor, :set_policy

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @stored_policies = @policy.stored_policies.order('created_at desc')
                       .paginate(page: params[:page],
                                 per_page: current_user.per_page_count)
  end

  def new
    @stored_policy = @policy.stored_policies.build
    render layout: false
  end

  def create
    @stored_policy = @policy.stored_policies.build(stored_policy_params)
    @stored_policy.user = current_user
    @stored_policy.rules = @policy.rules.count

    path = "#{Rails.root}/tmp/policy_#{@policy.id}.csv"

    File.open(path, 'w+') do |f|
      f.write(@policy.rules_to_csv_format)
    end

    @stored_policy.stored_rules = File.open(path)
    @stored_policy.save
  end

  def edit
    @stored_policy = StoredPolicy.find(params[:id])
    render layout: false
  end

  def update
    @stored_policy = StoredPolicy.find(params[:id])
    @stored_policy.update(stored_policy_params)
  end

  def destroy
    stored_policy = StoredPolicy.find(params[:id])
    stored_policy.destroy
    redirect_to sensor_policy_snapshots_path(@sensor, @policy)
  end

  def load
    stored_policy = StoredPolicy.find(params[:snapshot_id])
    @policy.update(updating: true)
    Delayed::Job.enqueue(Redborder::Jobs::LoadStoredPolicyJob.new(
      true, stored_policy.id, current_user.id, I18n.locale), priority: 10, run_at: Time.now)

    redirect_to sensor_policies_path(@sensor)
  end

  def download
    @stored_policy = StoredPolicy.find(params[:snapshot_id])
    data = open(@stored_policy.stored_rules.expiring_url,
                ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE)
    send_data data.read, filename: @stored_policy.filename
  end

  private

  def set_sensor
    @sensor = Sensor.find(params[:sensor_id])
    return unless current_user.cannot?(:manage, @sensor)
    redirect_to sensors_path,
                alert: "#{I18n.t('controllers.no_privileges')}"
  end

  def set_policy
    @policy = Policy.find(params[:policy_id])
    return unless current_user.cannot?(:manage, @policy.owner)
    redirect_to sensor_policies_path(@sensor),
                alert: "#{I18n.t('controllers.no_privileges')}"
  end

  def stored_policy_params
    params.require(:stored_policy).permit(:tag)
  end
end
