#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module EventsHelper
  def event_rule_options(event)
    sensor = Sensor.where(name: event.send(:sensor_name)).first
    rule_db_version_ids = RuleDbVersion.where(checked: true).map(&:id)
    rule = Rule.where(
      rule_sid: event.send(:sig_id),
      rule_gid: event.send(:sig_generator),
      rule_db_version_id: rule_db_version_ids
    ).order('rule_rev DESC').first

    user = User.current_user

    return unless rule.present?

    list = content_tag(:li, class: 'dropdown') do
      list_2 = link_to '#', class: 'dropdown-toggle', 'data-toggle' => 'dropdown' do
        ("#{I18n.t('ips_helper.references')}" + content_tag(:b, nil, class: 'caret')).html_safe
      end
      list_2 += content_tag :ul, class: 'dropdown-menu' do
        references = ''

        rule.rb_reference_rules.each do |reference|
          references += content_tag(:li, link_to(reference.rb_reference.humanize, reference.rb_reference.prefix.to_s + reference.url.to_s, target: '_blank'))
        end

        references += content_tag(:li, link_to("#{I18n.t('ips_helper.signature')}", rule.signature_url, target: '_blank')) if rule.signature_url

        references.html_safe
      end

      list_2.html_safe
    end
    list += content_tag(:li) do
      (link_to "#{I18n.t('ips_helper.view_rule')}", rule_path(id: rule, event_action: event.send(:action)), 'data-toggle' => 'modal', 'data-target' => '#rule_info_modal', 'data-backdrop' => 'static') + ' '
    end
    if sensor.present? && sensor.policy.present? && user.can?(:manage, sensor.policy.owner)
      list += content_tag(:li, link_to("#{I18n.t('ips_helper.policy_rule')}", "/sensors/#{sensor.id}/policies/#{sensor.policy.id}/rules?msg=#{url_encode(rule.msg)}"))
    end
    if sensor.present? && user.can?(:read, sensor) && ![Sensor::IPS_V2, Sensor::IPS_GENERIC].include?(sensor.type)
      list += content_tag(:li) do
        (link_to "#{I18n.t('ips_helper.view_raw')}", raw_sensors_path(name: sensor.name, group_name: event.send(:group_name), sig_generator: event.send(:sig_generator), sig_id: event.send(:sig_id), src: event.send(:src), dst: event.send(:dst), timestamp: event.send(:timestamp)), 'data-toggle' => 'modal', 'data-target' => '#raw_info_modal', 'data-backdrop' => 'static') + ' '
      end
    end
    if sensor.present? && sensor.policy.present? && user.can?(:manage, sensor.policy.owner)
      list += content_tag(:li) do
        (link_to content_tag(:i, nil, class: 'fa fa-plus') + "#{I18n.t('ips_helper.event_suppression')}",
        new_policy_rule_event_filter_path(policy_id: sensor.policy.id, rule_id: rule.id, type: 'suppression', source_ip: event.src, destination_ip: event.dst),
         'data-toggle' => "modal", 'data-target' => '#event_suppression_modal', 'data-backdrop' => 'static') + ' '
      end

      limit = sensor.policy.event_filters.where(policy: sensor.policy, rule_sid: rule.rule_sid, rule_gid: rule.rule_gid).where.not(filter_type: 'suppress')
      if limit.blank?
        list += content_tag(:li) do
          link_to content_tag(:i, nil, class: 'fa fa-plus') + "#{I18n.t('ips_helper.event_filtering')}", new_policy_rule_event_filter_path(policy_id: sensor.policy.id, rule_id: rule.id, type: 'filtering'), 'data-toggle' => "modal", 'data-target' => '#event_filtering_modal', 'data-backdrop' => 'static'
        end
      else
        list += content_tag(:li) do
          link_to content_tag(:i, nil, class: 'fa fa-plus') + "#{I18n.t('ips_helper.event_filtering')}", edit_policy_rule_event_filter_path(limit.first, policy_id: sensor.policy.id, rule_id: rule.id), 'data-toggle' => 'modal', 'data-target' => '#event_filtering_modal', 'data-backdrop' => 'static'
        end
      end
    end

    list
  end
end
