#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class Rule < ApplicationRecord
  has_many :factory_policy_rules, dependent: :delete_all
  has_many :factory_policies, through: :factory_policy_rules
  has_many :flowbit_rules, dependent: :delete_all
  has_many :flowbits, through: :flowbit_rules
  has_many :rb_reference_rules, dependent: :delete_all
  has_many :rb_references, through: :rb_reference_rules
  has_many :policy_rules
  has_many :rule_notes, -> { order(created_at: :desc) }

  belongs_to :category
  belongs_to :group
  belongs_to :family
  belongs_to :class_type
  belongs_to :rule_db_version
  delegate :rule_source, to: :rule_db_version

  has_and_belongs_to_many :impacts
  has_and_belongs_to_many :services

  MODIFIER = {
    nocase: "#{I18n.t('models.nocase')}",
    rawbytes: "#{I18n.t('models.rawbytes')}",
    depth: "#{I18n.t('models.depth')}",
    offset: "#{I18n.t('models.offset')}",
    distance: "#{I18n.t('models.distance')}",
    within: "#{I18n.t('models.within')}",
    http_client_body: "#{I18n.t('models.http_client_body')}",
    http_cookie: "#{I18n.t('models.http_cookie')}",
    http_raw_cookie: "#{I18n.t('models.http_raw_cookie')}",
    http_header: "#{I18n.t('models.http_header')}",
    http_raw_header: "#{I18n.t('models.http_raw_header')}",
    http_method: "#{I18n.t('models.http_method')}",
    http_uri: "#{I18n.t('models.http_uri')}",
    http_raw_uri: "#{I18n.t('models.http_raw_uri')}",
    http_stat_code: "#{I18n.t('models.http_stat_mode')}",
    http_stat_msg: "#{I18n.t('models.http_stat_msg')}",
    http_encode: "#{I18n.t('models.http_encode')}",
    fast_pattern: "#{I18n.t('models.fast_pattern')}"
  }.freeze

  HELP = {
    msg: "#{I18n.t('models.msg')}",
    reference: "#{I18n.t('models.reference')}",
    gid: "#{I18n.t('models.gid')}",
    sid: "#{I18n.t('models.sid')}",
    rev: "#{I18n.t('models.rev')}",
    classtype: "#{I18n.t('models.classtype')}",
    priority: "#{I18n.t('models.priority')}",
    metadata: "#{I18n.t('models.metadata')}",
    content: "#{I18n.t('models.content')}",
    uricontent: "#{I18n.t('models.uricontent')}",
    urilen: "#{I18n.t('models.urilen')}",
    isdataat: "#{I18n.t('models.isdataat')}",
    pcre: "#{I18n.t('models.pcre')}",
    pkt_data: "#{I18n.t('models.pkt_data')}",
    file_data: "#{I18n.t('models.file_data')}",
    base64_decode: "#{I18n.t('models.base64_decode')}",
    base64_data: "#{I18n.t('models.base64_data')}",
    byte_test: "#{I18n.t('models.byte_test')}",
    byte_jump: "#{I18n.t('models.byte_jump')}",
    byte_extract: "#{I18n.t('models.byte_extract')}",
    ftpbounce: "#{I18n.t('models.ftpbounce')}",
    asn1: "#{I18n.t('models.asn1')}",
    cvs: "#{I18n.t('models.csv')}",
    dce_iface: nil,
    dce_opnum: nil,
    dce_stub_data: nil,
    sip_method: nil,
    sip_stat_code: nil,
    sip_header: nil,
    sip_body: nil,
    gtp_type: nil,
    gtp_info: nil,
    gtp_version: nil,
    ssl_version: nil,
    ssl_state: nil,
    fragoffset: "#{I18n.t('models.fragoffset')}",
    ttl: "#{I18n.t('models.ttl')}",
    tos: "#{I18n.t('models.tos')}",
    id: "#{I18n.t('models.id')}",
    ipopts: "#{I18n.t('models.ipopts')}",
    fragbits: "#{I18n.t('models.fragbits')}",
    dsize: "#{I18n.t('models.dsize')}",
    flags: "#{I18n.t('models.flags')}",
    flow: "#{I18n.t('models.flow')}",
    flowbits: "#{I18n.t('models.flowbits')}",
    seq: "#{I18n.t('models.seq')}",
    ack: "#{I18n.t('models.ack')}",
    window: "#{I18n.t('models.window')}",
    itype: "#{I18n.t('models.itype')}",
    icode: "#{I18n.t('models.icode')}",
    icmp_id: "#{I18n.t('models.icmp_id')}",
    icmp_seq: "#{I18n.t('models.icmp_seq')}",
    rpc: "#{I18n.t('models.rpc')}",
    ip_proto: "#{I18n.t('models.ip_proto')}",
    sameip: "#{I18n.t('models.sameip')}",
    stream_reassemble: "#{I18n.t('models.stream_reassemble')}",
    stream_size: "#{I18n.t('models.stream_size')}",
    logto: "#{I18n.t('models.logto')}",
    session: "#{I18n.t('models.session')}",
    resp: "#{I18n.t('models.resp')}",
    react: "#{I18n.t('models.react')}",
    tag: "#{I18n.t('models.tag')}",
    activates: "#{I18n.t('models.activates')}",
    activated_by: "#{I18n.t('models.activated_by')}",
    count: "#{I18n.t('models.count')}",
    replace: "#{I18n.t('models.replace')}",
    detection_filter: "#{I18n.t('models.detection_filter')}"
  }.freeze

  SIGNATURE_VRT_URL = 'http://www.snort.org/rule_docs/$$gid$$-$$sid$$'
  SIGNATURE_ET_URL  = 'http://doc.emergingthreats.net/$$sid$$'

  # Don't show whitelist and blacklist reputation rules.
  default_scope { where.not('rule_gid = ? and rule_sid in (?)', 136, [1, 2]) }

  def signature_url
    signature_url = nil

    if (100..999_999).include?(rule_sid)
      signature_url = SIGNATURE_VRT_URL
    elsif (2_000_000..2_099_999).include?(rule_sid)
      signature_url = SIGNATURE_ET_URL
    else
      return
    end

    signature_url.sub(/\$\$sid\$\$/, rule_sid.to_s).sub(/\$\$gid\$\$/, rule_gid.to_s)
  end

  def self.protocols
    select(:protocol).group(:protocol).map(&:protocol)
  end

  def self.custom_query(query_tables, is_query, not_query, msg)
    query = if query_tables.is_a?(Symbol)
              where(is_query[:rules])
            else
              joins(query_tables[:rules]).where(is_query)
            end

    not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?', "%#{msg.downcase.split.join('%')}%") if msg.present?

    query
  end

  def self.active_custom_query(policy, query_tables, is_query, not_query, msg)
    if query_tables.is_a?(Symbol)
      query_tables_tmp = {rules: [:policy_rules]}
    else
      query_tables_tmp = query_tables.dup
      query_tables_tmp[:rules] += [:policy_rules]
    end

    is_query_tmp = is_query.dup

    if is_query_tmp[:policy_rules]
      is_query_tmp[:policy_rules].merge!(policy_id: policy.id)
    else
      is_query_tmp[:policy_rules] = { policy_id: policy.id }
    end

    query = joins(query_tables_tmp[:rules]).where(is_query_tmp)

    not_query.each do |item|
      query = query.where.not(item)
    end

    query = query.where('LOWER(rules.text) like ?', "%#{msg.downcase.split.join('%')}%") if msg.present?

    query
  end

  def threshold_type
    return nil if !has_threshold
    typeMatch = /type\s+(?<type>\w+)/.match(threshold_options)
    typeMatch.nil? ? nil : typeMatch[:type]
  end

  def threshold_track
    return nil if !has_threshold
    trackMatch = /track\s+(?<track>\w+)/.match(threshold_options)
    trackMatch.nil? ? nil : trackMatch[:track]
  end

  def threshold_count
    return nil if !has_threshold
    countMatch = /count\s+(?<count>\d+)/.match(threshold_options)
    countMatch.nil? ? nil : countMatch[:count]
  end

  def threshold_seconds
    return nil if !has_threshold
    secondsMatch = /seconds\s+(?<seconds>\d+)/.match(threshold_options)
    secondsMatch.nil? ? nil : secondsMatch[:seconds]
  end

end
