#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class RuleDbVersion < ApplicationRecord
  include Redborder::Audit
  include StorageFile

  validates :tag, length: { maximum: 16 }

  has_many :rules, dependent: :destroy
  has_many :class_types, dependent: :destroy
  has_many :policy_rule_db_versions, dependent: :destroy
  has_many :policies, through: :policy_rule_db_versions

  belongs_to :rule_source

  storage_file :so_rules,
               path: 'rb-webui/:class/:id/so_rules/:filename',
               styles: {}

  before_destroy :modify_md5_source

  audited only: [:checked],
          on: [:update, :destroy]

  def update_available?
    RuleDbVersion.where(rule_source_id: rule_source_id, completed: true, checked: true).where("created_at > ?", created_at).count > 0
  end

  def is_last?
    RuleDbVersion.where(rule_source_id: rule_source_id, completed: true).where("created_at < ?", created_at).blank?
  end

  def can_be_deleted?
    policies_from_olders = Policy.includes(:rule_db_versions)
                           .where(rule_db_versions: { rule_source_id: rule_source_id })
                           .where('rule_db_versions.created_at < ?', created_at)

    policies_from_olders.count.zero? && policies.count.zero? && !is_deleted
  end

  def resume_tag
    tag or created_at.in_time_zone.strftime("%Y-%m-%d")
  end

  def full_tag
    if tag.blank?
      created_at.in_time_zone.strftime("%Y-%m-%d %H:%M:%S")
    else
      tag + " (" + created_at.in_time_zone.strftime("%Y-%m-%d %H:%M:%S") + ")"
    end
  end

  def so_rules_uri
    return nil if so_rules.path.nil?
    StorageFile.get_uri so_rules.path
  end

  def self_and_olders
    RuleDbVersion.where("id = ? OR (rule_source_id = ? AND created_at < ?)", id, rule_source_id, created_at)
  end

  def readable_policies(user = User.current_user)
    self.policies.where(sensor_id: user.sensor.self_and_descendants.pluck(:id))
  end

  private

  def modify_md5_source
    if completed and RuleDbVersion.where(rule_source_id: rule_source_id, completed: true).one?
      rule_source.update(md5: nil)
    end
  end

end
