#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class StoredPolicy < ApplicationRecord

  include StorageFile

  belongs_to :policy
  belongs_to :user

  storage_file :stored_rules, path: 'rb-webui/:class/:id'

  validates :tag, length: { maximum: 16 }
  validates_uniqueness_of :tag, scope: :policy_id

  def filename
    if tag.nil?
      "#{created_at.strftime("%Y%m%d%H%M%S")}_#{policy.name}.csv"
    else
      "#{tag.gsub(/\s+/, "_")}_#{policy.name}.csv"
    end
  end

  def stored_rules_path
    DittoCode::Exec.if 'ENTERPRISE' do
      return stored_rules.expiring_url
    end

    DittoCode::Exec.if 'COMMUNITY' do
      return stored_rules.path
    end
  end

end
