#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading IPS module'

Redborder::Module.register :ips do

  name 'tr.ips.intrusion'
  path 'ips'
  session_id 'event'
  data_source 'Event'
  customizable_tabs true
  selectable_widget true
  cep_topic :rb_event_post
  incident_source_name 'redBorder Intrusion'

  menu :product_menu, :ips,
       { controller: :events, action: :index },
       caption: 'tr.ips.intrusion', module: 'ips',
       html: { class: 'block-ui-but' }

  overview_widgets [
    {
      name: "tr.ips.events",
      partial: 'events',
      position: 1,
      info: "tr.ips.info"
    }
  ]
  widgets
  columns ({
    action:
      { enabled: false,
        size: 1, columns: { action: 'tr.ips.action' }, category: "IPS/IDS" },
    incident_uuid:
        { enabled: false,
          size: 1, columns: { incident_uuid: 'Incident UUID' }, category: "UUIDs" },
    classification:
      { enabled: false,
        size: 3, columns: { classification: 'tr.ips.classification' }, category: "IPS/IDS" },
    conversation:
      { enabled: false,
        size: 1, columns: { src: 'tr.ips.source', dst: 'tr.ips.destination' }, name: "Conversation", hide_raw: true, category: "Network" },
    domain_name:
      { enabled: false,
        size: 2, columns: { domain_name: 'tr.ips.domain_name' }, category: "IPS/IDS", observable_type: "domain" },
    ethlength_range:
      { enabled: false,
        size: 1, columns: { ethlength_range: "Ethlength" }, category: "Datalink" },
    group_name:
      { enabled: false,
        size: 1, columns: { group_name: 'tr.ips.ips_group' }, category: "IPS/IDS" },
    sig_generator:
      { enabled: false,
        size: 1, columns: { sig_generator: "GID" }, category: "IPS/IDS" },
    icmptype:
      { enabled: false,
        size: 1, columns: { icmptype: 'tr.ips.icmp.'}, category: "Datalink" },
    iplen_range:
      { enabled: false,
        size: 1, columns: { iplen_range: 'tr.ips.ip_length' }, category: "Network" },
    l4_proto:
      { enabled: false,
        size: 1, columns: { l4_proto: 'tr.ips.protocol' }, complementary_columns: { l4_proto_name: "Protocol Name" }, category: "Network", object_type: "ProtocolObject" },
    rev:
      { enabled: false,
        size: 1, columns: { rev: "Revision" }, category: "IPS/IDS" },
    sensor_name:
      { enabled: true,
        size: 2, columns: { sensor_name: "Sensor" }, complementary_columns: { sensor_ip: "Sensor IP"}, category: "Network", shared: false },
    priority:
      { enabled: false,
        size: 1, columns: { priority: 'tr.ips.severity' }, category: "IPS/IDS" },
    msg:
      { enabled: true,
        size: 6, columns: { msg: 'tr.ips.signature' }, category: "IPS/IDS", shared: false },
    sig_id:
      { enabled: false,
        size: 1, columns: { sig_id: "SID" }, category: "IPS/IDS" },
    scatterplot:
      { enabled: false,
        size: 1, columns: { src: 'tr.ips.source', dst: "Destination" }, name: "Scatterplot", hide_raw: true, category: "Network" },
    ethsrc:
      { enabled: false,
        size: 4, columns: { ethsrc: "SRC MAC" }, complementary_columns: { ethsrc_name: "Name -- Vendor" }, category: "Datalink", object_type: "MacObject", observable_type: "mac_address" },
    ethsrc_vendor:
      { enabled: false,
        size: 2, columns: { ethsrc_vendor: "SRC Vendor" }, category: "Datalink" },
    src:
      { enabled: true,
        size: 2, columns: { src: 'tr.ips.src_address' }, complementary_columns: { src_name: "SRC Name" }, category: "Network", object_type: "NetObject", observable_type: "ip" },
    src_is_malicious:
     { enabled: false, size: 1, columns: { src_is_malicious: 'tr.ips.src_malicious' }, category: 'Malicious' },
    dst_is_malicious:
    { enabled: false, size: 1, columns: { dst_is_malicious: 'tr.ips.dst_malicious' }, category: 'Malicious' },
    sha256_is_malicious:
    { enabled: false, size: 1, columns: { sha256_is_malicious: 'tr.ips.sha256_malicious' }, category: 'Malicious' },
    file_uri_is_malicious:
    { enabled: false, size: 1, columns: { file_uri_is_malicious: 'tr.ips.file_uri_malicious' }, category: 'Malicious' },
    file_hostname_is_malicious:
    { enabled: false, size: 1, columns: { file_hostname_is_malicious: 'tr.ips.file_hostname_malicious' }, category: 'Malicious' },
    src_country_code:
    { enabled: false,
      size: 1, columns: { src_country_code: 'tr.ips.src_country' }, complementary_columns: { src_country: "SRC Country Name" }, category: "Location" },
  src_net_name:
    { enabled: false,
      size: 2, columns: { src_net_name: 'tr.ips.src_net_address' }, category: "Network" },
  src_port:
    { enabled: false,
      size: 1, columns: { src_port: 'tr.ips.src_port' }, complementary_columns: { src_port_name: "SRC Port Name" }, category: "Transport", object_type: "ServiceObject", observable_type: "port" },
  src_as_name:
    { enabled: false,
      size: 1, columns: { src_as_name: 'tr.ips.src_as' }, category: "Location" },
  src_map:
    { enabled: false,
      size: 1, columns: { src: 'tr.ips.src_address' }, complementary_columns: { src_name: "SRC Name" }, name: "SRC MAP", ips: :src, hide_raw: true, category: "Location" },
  ethdst:
    { enabled: false,
      size: 4, columns: { ethdst: "DST MAC" }, complementary_columns: { ethdst_name: "Name -- Vendor" }, category: "Datalink", object_type: "MacObject", observable_type: "mac_address" },
  ethdst_vendor:
    { enabled: false,
      size: 2, columns: { ethdst_vendor: "DST Vendor" }, category: "Datalink" },
  dst:
    { enabled: true,
      size: 2, columns: { dst: 'tr.ips.dst_address' }, complementary_columns: { dst_name: "DST Name" }, category: "Network", object_type: "NetObject", observable_type: "ip" },
  dst_country_code:
    { enabled: false,
      size: 1, columns: { dst_country_code: 'tr.ips.dst_country' }, complementary_columns: { dst_country: "DST Country Name" }, category: "Location" },
  dst_net_name:
    { enabled: false,
      size: 2, columns: { dst_net_name: 'tr.ips.dst_net_address' }, category: "Network" },
  dst_port:
    { enabled: false,
      size: 1, columns: { dst_port: 'tr.ips.dst_port' }, complementary_columns: { dst_port_name: "DST Port Name" }, category: "Transport", object_type: "ServiceObject", observable_type: "port" },
  dst_as_name:
    { enabled: false,
      size: 1, columns: { dst_as_name: "DST AS" }, category: "Location" },
  dst_map:
    { enabled: false,
      size: 1, columns: { dst: 'tr.ips.dst_address' }, complementary_columns: { dst_name: "DST Name" }, name: "DST MAP", ips: :dst, hide_raw: true, category: "Location" },
  tos:
    { enabled: false,
      size: 1, columns: { tos: 'TOS' }, category: 'Network' },
  ttl:
    { enabled: false,
      size: 1, columns: { ttl: "TTL" }, category: "Datalink" },
  vlan:
    { enabled: false,
      size: 1, columns: { vlan: "VLAN ID" }, complementary_columns: { vlan_name: "VLAN Name" }, category: "Datalink", object_type: "VlanObject" },
  sha256:
    {
      enabled: false,
      size: 1, columns: { sha256: 'SHA256' }, category: 'File', observable_type: "sha256" },
  file_size:
    {
      enabled: false,
      size: 1, columns: { file_size: 'tr.ips.file_size' }, category: 'File' },
  file_uri:
    {
      enabled: false,
      size: 1, columns: { file_uri: 'tr.ips.file_uri' }, category: 'File', observable_type: "file_path" },
  file_hostname:
    {
      enabled: false,
      size: 1, columns: { file_hostname: 'tr.ips.file_hostname' }, category: 'File' },
    sensor_uuid:
        { enabled: false, size: 1,  hide_global: true,
          columns: { sensor_uuid: 'Sensor UUID' }, category: 'UUIDs' },
  service_provider:
    { enabled: false, size: 1, hide_global: true,
      columns: { service_provider: 'tr.ips.service_provider' },
      category: 'Location' },
  service_provider_uuid:
    { enabled: false, size: 1, hide_global: true,
      columns: { service_provider_uuid: 'tr.ips.service_uuid' },
      category: 'UUIDs' },
  namespace:
    { enabled: false, hide_global: true,
      size: 1, columns: { namespace: 'tr.ips.namespace' }, category: 'Location' },
  namespace_uuid:
    { enabled: false, size: 1, hide_global: true,
      columns: { namespace_uuid: 'tr.ips.namespace_uuid' },
      category: 'UUIDs' },
  deployment:
    { enabled: false, hide_global: true,
      size: 1, columns: { deployment: 'tr.ips.deployment' }, category: 'Location' },
  deployment_uuid:
    { enabled: false, size: 1, hide_global: true,
      columns: { deployment_uuid: 'tr.ips.deployment_uuid' },
      category: 'UUIDs' },
  market:
    { enabled: false, hide_global: true,
      size: 1, columns: { market: 'tr.ips.market' }, category: 'Location' },
  market_uuid:
    { enabled: false, size: 1, hide_global: true,
      columns: { market_uuid: 'tr.ips.market_uuid' },
      category: 'UUIDs' },
  organization:
    { enabled: false, hide_global: true,
      size: 1, columns: { organization: 'tr.ips.org' }, category: 'Location' },
  organization_uuid:
    { enabled: false, size: 1, hide_global: true,
      columns: { organization_uuid: 'tr.ips.org_uuid' },
      category: 'UUIDs' },
  campus:
    { enabled: false,  size: 1, hide_global: true,
      columns: { campus: 'Campus' },
      category: 'Location' },
  campus_uuid:
    { enabled: false,  size: 1, hide_global: true,
      columns: { campus_uuid: 'Campus UUID' },
      category: 'UUIDs' },
  building:
    { enabled: false,  size: 1, hide_global: true,
      columns: { building: 'tr.ips.building' },
      category: 'Location' },
  building_uuid:
    { enabled: false,  size: 1, hide_global: true,
      columns: { building_uuid: 'tr.ips.building_uuid' },
      category: 'UUIDs' },
  floor:
    { enabled: false,  size: 1, hide_global: true,
      columns: { floor: 'tr.ips.floor' },
      category: 'Location' },
  floor_uuid:
    { enabled: false,  size: 1, hide_global: true,
      columns: { floor_uuid: 'tr.ips.floor_uuid' },
      category: 'Location' }
})

DittoCode::Exec.if 'ENTERPRISE' do
  columns ({
    darklist_score_name:      {enabled: false,  size: 1, columns: { darklist_score_name: 'tr.ips.darklist_score' }, category: "Darklist" },
    darklist_category:        {enabled: false,  size: 1, columns: { darklist_category: 'tr.ips.darklist_category' }, category: "Darklist" },
#      darklist_protocol:        {enabled: false,  size: 1, columns: { darklist_protocol: "Darklist Protocol" }, category: "Darklist" },
    darklist_direction:       {enabled: false,  size: 1, columns: { darklist_direction: 'tr.ips.darklist_direction' }, category: "Darklist" }
  })
end

aggregations ({
  events: {
    name: "tr.ips.events2", unit: 'events', value: :events, size: 1,
    aggregations: [
      { type: 'longSum', name: 'events', fieldName: 'events' }
    ],
    description: "tr.ips.number"
  },
  events_per_sec: {
    name: "tr.ips.events3", unit: 'events/s', value: :events_per_sec, size: 1,
    aggregations: [
      { type: 'longSum', name: 'events', fieldName: 'events' }
    ],
    post_aggregations: [
      {
        type: 'arithmetic',
        name: 'events_per_sec',
        fn: '/',
        fields: [
          {
            type: 'fieldAccess',
            fieldName: 'events'
          },
          {
            type: 'constant',
            value: proc { |t| t }
          }
        ]
      }
    ],
    description: "tr.ips.per_second"
  },
  signatures: {
    name: "tr.ips.signature3", unit: 'signatures',
    value: :signatures, size: 1, round: true,
    aggregations: [
      { type: 'hyperUnique', name: 'signatures', fieldName: 'signatures' }
    ],
    description: "tr.ips.unique"
  },
  risk: {
    name: "tr.ips.risk", value: :risk,
    size: 1, round: false, unit: '',
    aggregations: [
      { type: 'longSum', name: 'events',    fieldName: 'events' },
      { type: 'longSum', name: 'sum_dl_score', fieldName: 'sum_dl_score' }
    ],
    post_aggregations: [
      {
        type: 'arithmetic',
        name: 'risk',
        fn: '/',
        fields: [
          { type: 'fieldAccess', fieldName: 'sum_dl_score' },
          { type: 'fieldAccess', fieldName: 'events' }
        ]
      }
    ],
    description: "tr.ips.percentage"
  },
})

# menu :sensor_tree_menu, :new_ips_sensor,
#      { controller: :sensors, action: :new_ips },
#      caption: "IPS",
#      html: { 'data-backdrop' => 'static',
#              'data-target' => '#new_sensor_modal',
#              'data-toggle' => 'modal' }

Redborder::MenuManager.map :domain_menu do |menu|
  menu.push :signature_policies, { controller: :policies, action: :index },
            param: :sensor_id, caption: "tr.ips.signature",
            after: :export_tree,
            html: { class: 'block-ui-but menu-divider' },
            admin: true,
            module: 'ips'
  menu.push :reputation_policies,
            { controller: :reputation_policies, action: :index },
            param: :sensor_id, caption: "tr.ips.reputation",
            html: { class: 'block-ui-but' },
            admin: true,
            module: 'ips'
  menu.push :update_policies,
            { controller: :policies, action: :global_update },
            param: :sensor_id,
            caption: "tr.ips.update",
            html: { class: 'block-ui-but' },
            admin: true,
            module: 'ips'
  menu.push :apply_configuration,
            { controller: :sensors, action: :configuration_update },
            caption: "tr.ips.apply",
            html: { class: 'block-ui-but menu-divider' },
            last: true,
            admin: true,
            module: 'ips'

  menu.push :view, '#', caption: "tr.ips.view", after: :edit
  menu.push :ips_view, { controller: 'sensors', action: 'ips' },
            caption: 'tr.ips.ips_sensors', parent: :view, last: true,
            html: { class: 'block-ui-but' },
            module: 'ips'
end

menu :sensors_view_menu, :ips_view,
     { controller: 'sensors', action: 'ips' },
     caption: 'tr.ips.ips_sensors',
     html: { class: 'block-ui-but' }

menu :tools_menu, :rule_version,
     { controller: :rule_db_versions, action: :index },
     caption: "tr.ips.rule",
     after: :alerts,
     html: { class: 'block-ui-but' }

menu :user_home_page_collection_menu, :Intrusion, {}, caption: '/ips'
menu :widget_sensor_table_type_menu, :table_ips, {}, caption: 'Table IPS'
menu :segments_menu,
     :ips,
     { controller: :monitor,
       action: :more_segments,
       datasource: 'rb_event' },
     caption: "tr.ips.event",
     html: { 'data-remote' => true,
             'data-source' => 'rb_event' }

# Options menu for Sensors view
unless ENV['RAILS_ENV'] && ENV['RAILS_ENV'] == 'test'
  menu :options_sensors_view_menu, :globa_sig_policies,
       { controller: :policies, action: :index},
       param: :sensor_id,
       caption: "tr.ips.globals",
       html: { class: 'menu-divider' },
       admin: true
  menu :options_sensors_view_menu, :globa_rep_policies,
       { controller: :reputation_policies, action: :index},
       param: :sensor_id,
       caption: "tr.ips.globalr",
       admin: true
  menu :options_sensors_view_menu, :globa_policy_update,
       { controller: :policies, action: :global_update},
       param: :sensor_id,
       caption: "tr.ips.globalp",
       admin: true,
       html: { class: 'block-ui-but' }
  menu :options_sensors_view_menu, :globa_apply_conf,
       { controller: :sensors, action: :configuration_update},
       param: :id,
       caption: "tr.ips.globala",
       last: true,
       admin: true,
       html: { class: 'block-ui-but' }
end

menu :worker_menu, :start_rules_update,
     { controller: :jobs, action: :start_rule_update },
     html: { method: :post },
     caption: "tr.ips.start",
     first: true,
     if: proc { !DelayedJob.rule_update? }
end

[:ips_menu, :ips_group_menu, :ips_binding_menu].each do |product_menu|
Redborder::MenuManager.map product_menu do |menu|
  menu.push :edit,
            { controller: 'sensors', action: 'edit' },
            caption: "tr.ips.edit",
            first: true,
            html: { class: 'block-ui-but' },
            admin: true,
            module: 'ips'
  menu.push :signature_policies,
            { controller: :policies, action: :index },
            param: :sensor_id,
            caption: "tr.ips.signature",
            html: { class: 'block-ui-but menu-divider' },
            admin: true,
            module: 'ips'
  menu.push :reputation_policies,
            { controller: :reputation_policies, action: :index },
            param: :sensor_id,
            caption: "tr.ips.reputation",
            html: { class: 'block-ui-but menu-divider-bottom' },
            admin: true,
            module: 'ips'
  menu.push :apply_configuration,
            { controller: :sensors, action: :configuration_update },
            caption: "tr.ips.apply",
            html: { class: 'block-ui-but menu-divider' },
            last: true,
            admin: true,
            module: 'ips'
end
end

# extend DRUID_CONFIG
druid_ips_config =
YAML.load(
  File.read("#{Rails.root}/lib/modules/ips/config/rbdruid_config.yml")
)[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_ips_config

require_dependency 'sensor_ips'
require_dependency 'sensor_ips_generic'
require_dependency 'delayed_job_ips'
require_dependency 'setting_ips'
require_dependency 'widget_ips'
require_dependency 'report_block_ips'
require_dependency 'alarm_ips'
require_dependency 'user_ips'
require_dependency 'net_object_ips'
require_dependency 'vlan_object_ips'

DittoCode::Exec.if 'ENTERPRISE' do
require_dependency 'license_ips'
end

# Require services
require_dependency 'add_global_filters_ips'

# Require controllers
require_dependency 'jobs_controller_ips'
require_dependency 'sensors_controller_ips'
require_dependency 'api/v1/sensors_controller_ips'
require_dependency 'settings_controller_ips'
require_dependency 'widgets_controller_ips'
require_dependency 'application_controller_ips'

# Require helpers
require_dependency 'widgets_helper_ips'
require_dependency 'sensors_helper_ips'

# Require jobs
require_dependency 'apply_update_job'
require_dependency 'binding_vars_job'
require_dependency 'clone_policy_job'
require_dependency 'custom_rule_updatedb_job'
require_dependency 'delete_policy_job'
require_dependency 'delete_rule_db_version_job'
require_dependency 'delete_rule_source_job'
require_dependency 'load_stored_policy_job'
require_dependency 'policy_update_rule_db_versions_job'
require_dependency 'rule_updatedb_job'
require_dependency 'sensor_update_dbversion_job'
require_dependency 'stored_policies_job'

# Require hooks
require_dependency 'hooks/alarms_ips_hook'
require_dependency 'hooks/ipvars_portvars_hook'
require_dependency 'hooks/domain_edit_hook'
require_dependency 'hooks/widget_data_ips_hook'
require_dependency 'hooks/settings_general_parameters_hook'
require_dependency 'hooks/settings_form_ids_ips_hook'

to_prepare = proc do
unless JobsController.include?(RedBorder::JobsControllerIPS)
  JobsController.send(:include, RedBorder::JobsControllerIPS)
end
unless SensorsController.include?(RedBorder::SensorsControllerIPS)
  SensorsController.send(:include, RedBorder::SensorsControllerIPS)
end
unless Api::V1::SensorsController.include?(RedBorder::Api::V1::SensorsControllerIPS)
  Api::V1::SensorsController.send(:include, RedBorder::Api::V1::SensorsControllerIPS)
end
unless SettingsController.include?(RedBorder::SettingsControllerIPS)
  SettingsController.send(:include, RedBorder::SettingsControllerIPS)
end
unless WidgetsController.include?(RedBorder::WidgetsControllerIPS)
  WidgetsController.send(:include, RedBorder::WidgetsControllerIPS)
end
unless ApplicationController.include?(RedBorder::ApplicationControllerIPS)
  ApplicationController.send(:include, RedBorder::ApplicationControllerIPS)
end
unless SensorsHelper.include?(RedBorder::SensorsHelperIPS)
  SensorsHelper.send(:include, RedBorder::SensorsHelperIPS)
end
unless WidgetsHelper.include?(RedBorder::WidgetsHelperIPS)
  WidgetsHelper.send(:include, RedBorder::WidgetsHelperIPS)
end
end

Rails.configuration.to_prepare(&to_prepare)
