#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

RedBorder::Application.routes.draw do

  # resources :classifications

  resources :sensors, except: :show do
    get :ips, on: :collection
    get :ips, on: :member
    get :raw, on: :collection
    get :new_ips, on: :collection
    post :create_ips, on: :collection
    get 'ips/:id/search', action: :ips_letter_search, on: :collection, as: :ips_letter_search
    get 'ips/search', action: :ips_search, on: :collection
    get 'ips/load_more', action: :ips_load_more, on: :collection
    post :associate, on: :collection
    post :associate_v2, on: :collection
    get :edit_variable, on: :member
    patch :variable, on: :member, action: :update_variable
    delete :variable, on: :member, action: :delete_variable
    get :import_ipvars, on: :member, action: :import_vars, type: "ipvars"
    get :import_portvars, on: :member, action: :import_vars, type: "portvars"
    post :import_vars_file, on: :member
    get :export_ipvars, on: :member, action: :export_vars, type: "ipvars"
    get :export_portvars, on: :member, action: :export_vars, type: "portvars"
    post :update_rpms, action: :update_rpms, on: :member, as: :update_rpms
    get :restore_commands, on: :collection

    get :new_vlan, on: :member
    post :create_vlan, on: :member
    delete :vlan, on: :member, action: :destroy_object, type: "vlan"
    get :new_network, on: :member
    post :create_network, on: :member
    delete :network, on: :member, action: :destroy_object, type: "network"
    post "/bypass/:segment/:mode", on: :member, action: :bypass
    post "/segments/:segment/:mode", on: :member, action: :segment_enabled

    get :classifications, on: :member, constraints: {format: /txt/}
    get :unicode_map, on: :member, constraints: {format: /txt/}
    get :new_group, on: :member
    post :create_group, on: :member
    post :change_group_name, on: :member
    post :change_snort_mode, on: :member
    get :new_binding, on: :member
    post :create_binding, on: :member
    post :apply_update, on: :collection
    get :get_rule_db_version_ids, on: :member, constraints: {format: /json/}

    resources :active_rules,
              controller: 'rules',
              only: [:index],
              type: 'active_rules',
              defaults: { format: 'txt' }

    resources :preprocessor_rules,
              controller: 'rules',
              only: [:index],
              type: 'preprocessor_rules',
              defaults: { format: 'txt' }

    resources :file_capture_rules,
              controller: 'rules',
              only: [:index],
              type: 'file_capture_rules',
              defaults: { format: 'txt' }

    get "iplist" => "reputations#iplist", defaults: {format: 'txt'}
    get "geoip" => "reputations#geoip", defaults: {format: 'txt'}

    resources :policies do
      get :global_update, on: :collection
      get :search, action: :search, on: :collection
      patch :global_update, on: :collection
      patch :assign_policy, on: :member, controller: "sensors"
      post :clone, on: :member
      resources :rules, only: :index do
        get :search, on: :collection
        post :results, on: :collection
      end
      resources :event_filters, only: :index do
        get "new", to: "event_filters#new_event_filter", on: :collection, as: "new"
      end
      resources :snapshots, controller: "stored_policies" do
        post :load
        get :download
      end
    end

    resources :reputation_policies do
      patch :assign, on: :member
      post :clone, on: :member
      get :import, on: :member
      post :import_file, on: :member
      delete :all, on: :member, action: :destroy_all
    end
  end

  resources :policies do

    patch :update_rule_db_versions, on: :member

    resources :rules do
      get :import, on: :collection
      post :import_file, on: :collection
      get :search, on: :collection
      post :results, on: :collection
      post :massive_update, on: :collection
      get ":category_id/groups", action: :groups, on: :collection
      get ":category_id/:group_id/families", action: :families, on: :collection
      get ":category_id/:group_id/:family_id/rules", action: :rules, on: :collection
      resources :event_filters
    end

    resources :event_filters, only: :index do
      get "edit", to: "event_filters#edit_details", as: "edit_details"
      patch :update, to: "event_filters#update_details", as: "update_details"
      get "new", to: "event_filters#new_event_filter", on: :collection, as: "new"
      post :create, to: "event_filters#create_event_filter", on: :collection
      get :import, on: :collection
      post :import_file, on: :collection
    end

  end

  resources :reputation_policies do
    resources :reputations do
      post :reorder, on: :collection
    end
  end

  resources :reputations

  patch 'rules_filter/:id/del' => 'rules#delete_filter', as: "delete_rules_filter"
  patch 'policies/:policy_id/rules' => 'rules#update_action', as: 'rule_update_action'

  resources :events, path: '/ips' do
    get :load_tab, on: :member
    get :load_tab_cardinality, on: :member
    get :load_cardinality_table, on: :member
    collection do
      patch :change_session_tab
      patch :change_session_total
      get :info
      get :help
      get :load_more
      get :load_more_tops
      get :load_more_comparative
      get :load_more_cardinality
      get :load_new_data
      get :tops
      get :comparative
      get :cardinality
      get :search
      post :results
      get :element_childs
      get :lookup
      get :export_tops_to_csv
      get :export_cardinality_to_csv
      get :export_raw_to_csv
      get :sort_columns
      get :sort_aggregations
      get 'aggregations/reorder',
          to: 'events#reorder_aggregations', as: 'reorder_ips_aggregations'
      patch "change_chart_type/:id", action: :change_chart_type, as: :change_chart_type
      get "tops/add_tab/:id", action: :add_tab, as: :add_events_top_tab
      get "cardinality/add_tab/:id", action: :add_cardinality_tab, as: :add_events_cardinality_tab
      get "cardinality/get_serie/:id", action: :get_cardinality_serie, as: :get_events_cardinality_serie
      patch "tops/del_tab/:id", action: :del_tab, as: :del_events_top_tab
      patch "cardinality/del_tab/:id", action: :del_cardinality_tab, as: :del_events_cardinality_tab
      get "tops/get_serie/:id", action: :get_serie, as: :get_events_serie
      get "tops/get_tops/:id", action: :get_tops, as: :get_events_tops
      get "tops/get_raw_table", action: :get_raw_table, as: :get_events_raw_table
      get "get_maps_data/:id", action: :get_maps_data, as: :get_events_maps_data
      post "export_tops_to_file", action: :export_tops_to_file, as: :export_tops_to_file
      post 'export_cardinality_to_file', action: :export_cardinality_to_file, as: :export_cardinality_to_file
      post 'export_raw_to_file', action: :export_raw_to_file, as: :export_raw_to_file
      get 'get_outliers/:id', action: :get_outliers, as: :get_events_outliers
    end
  end

  patch 'saved_ips_filter/:id/add' => 'saved_event_filters#add_saved_filter', as: "add_saved_ips_filter"
  post 'saved_ips_filters/create' => 'saved_event_filters#create'
  get 'tabs/reorder_ips' => 'events#reorder_tab', :as => 'reorder_ips_tab'

  resources :saved_event_filters, :path => "/saved/ips_filters" do

    collection do
      post :title
      post :update
    end

    member do
      get :view
      post :update
    end
  end

  resources :rules do
    get "notes/new", to: "rules#new_note", as: "new_note"
    post :notes, to: "rules#create_note"
    delete "notes/:rule_note_id", to: "rules#destroy_note", as: "delete_note"
  end

  resources :rule_db_versions, :path => "/rule_versions" do
    post :force_update, on: :collection
    get :check_update, on: :collection
    post :create_source, on: :collection
    post :create_custom_source, on: :collection
    get :so_rules_file, on: :member
    get 'source/new', action: :new_source, as: :new_source, on: :collection
    get 'source/new_custom', action: :new_custom_source, as: :new_custom_source, on: :collection
    post :source, action: :create_source, as: :source, on: :collection
    get 'source/:rule_source_id/edit', action: :edit_source, as: :edit_source, on: :collection
    patch 'source/:rule_source_id', action: :update_source, as: :update_source, on: :collection
    delete 'source/:rule_source_id', action: :destroy_source, as: :destroy_source, on: :collection
    get :help, on: :collection

    resources :rules, only: [:index, :show] do
      get ':category_id/groups', to: 'rules#groups', on: :collection
      get ':category_id/:group_id/families', to: 'rules#families', on: :collection
      get ':category_id/:group_id/:family_id/rules', to: 'rules#rules', on: :collection
      get :search, on: :collection
      post :results, on: :collection
    end
  end

  resources :event_filters, only: [:create, :update]
  delete 'event_filters/:id/delete' => 'event_filters#delete', as: 'delete_event_event_filter'

  resource :settings do
    get :new_ip_var, action: :new_variable, type: 'ipvars'
    get :new_port_var, action: :new_variable, type: 'portvars'
    post :variable, action: :create_variable
    delete :variable, action: :delete_variable
    get :edit_variable
    patch :variable, on: :member, action: :update_variable
    get :import_ipvars, action: :import_vars, type: 'ipvars'
    get :import_portvars, action: :import_vars, type: 'portvars'
    post :import_vars_file
    get :export_ipvars, action: :export_vars, type: 'ipvars'
    get :export_portvars, action: :export_vars, type: 'portvars'
  end

  resources :jobs do
    collection do
      post :start_rule_update
    end
  end

  resources :widgets, except: [:index, :create, :new, :edit, :show, :update, :destroy] do
    get :get_sensors_table_ips, action: :get_sensors_table_ips, as: :widget_get_sensors_table_ips, on: :member
  end

  namespace :api do
    namespace :v1 do
      post 'ips/apply_job_info', to: 'sensors#apply_job_info'
      get 'ips/has_new_config', to: 'sensors#has_new_config'
      get 'ips/config', to: 'sensors#config_file'
    end
  end
end
