#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module Api
    module V1
      module SensorsControllerIPS
        def self.included(base)
          base.class_eval do
            unloadable

            @skip_methods += [:apply_job_info, :has_new_config, :config_file]
            skip_before_action :authenticate_user!, raise: false, only: @skip_methods
            skip_before_action :authenticate_user_from_token!, only: @skip_methods
            before_action :validate_cert_api_ips, only: [:apply_job_info, :has_new_config, :config_file]
            before_action :set_sensor_api_ips, only: [:has_new_config, :config_file]

            define_method("verify_sensor_#{Sensor::IPS_PROXY}") do
              node_json = {
                'name' => "rbipscp-#{@sensor.id}",
                'run_list' => [@sensor.send("run_list_text_#{@sensor.type}"), "role[#{@sensor.chef_name}]"],
                'attributes' => {
                  'rbname' => @sensor.name,
                  'ipaddress' => @sensor.ip,
                  'redborder' => {
                    'monitors' => [],
                    'segments' => {},
                    'sensor_id' => @sensor.id,
                    'snort' => {},
                    'sensor_uuid' => @sensor.uuid
                  }
                }
              }

              service_provider = @sensor.ancestors.find_by(domain_type: Sensor::SERVICE_PROVIDER)
              deployment       = @sensor.ancestors.find_by(domain_type: Sensor::DEPLOYMENT)
              namespace        = @sensor.ancestors.find_by(domain_type: Sensor::NAMESPACE)
              organization     = @sensor.ancestors.find_by(domain_type: Sensor::ORGANIZATION)
              market           = @sensor.ancestors.find_by(domain_type: Sensor::MARKET)

              %w(service_provider deployment namespace organization market).each do |v|
                node_json['attributes']['redborder']["#{v}"] = eval(v).try('name')
                node_json['attributes']['redborder']["#{v}_uuid"] = eval(v).try('uuid')
              end

              node = Chef::Node.from_hash(node_json)
              node.create

              # create client
              node_name = "rbipscp-#{@sensor.id}"
              client_json = { 'name' => node_name, 'validator' => true }
              @client = Chef::ApiClient.from_hash(client_json)
              @client = @client.save

              # Give to client permissions over the node
              rest = Chef::ServerAPI.new(Chef::Config[:chef_server_url])
              # possible options: %w(read update delete grant)
              %w(update).each do |perm|
                ace = rest.get_rest("nodes/#{node_name}/_acl")[perm]
                ace['actors'] << node_name unless ace['actors'].include?(node_name)
                rest.put_rest("nodes/#{node_name}/_acl/update", perm => ace)
              end

              render json: { status: 'claimed', cert: @client['private_key'].to_json, nodename: node_name }
            end

            define_method("verify_sensor_#{Sensor::IPS_GENERIC}") do
              namespace = @sensor.ancestors.find_by(domain_type: Sensor::NAMESPACE)
              index_partitions = namespace.chef_role.override_attributes['redborder']['index_partitions'] if namespace

              node_json = {
                'name' => "rbipsg-#{@sensor.id}",
                'run_list' => [@sensor.send("run_list_text_#{@sensor.type}"), "role[#{@sensor.chef_name}]"],
                'attributes' => {
                  'rbname' => @sensor.name,
                  'ipaddress' => @sensor.ip,
                  'redborder' => {
                    'monitors' => [],
                    'segments' => {},
                    'sensor_id' => @sensor.id,
                    'snort' => {},
                    'sensor_uuid' => @sensor.uuid,
                    'ipsgeneric' => {
                      'config' => {
                        'rules_path' => '/etc/snort/rules/local.rules',
                        'filter_rules_path' => '/etc/snort/threshold.conf',
                        'unified2_files' => '/var/log/snort',
                        'unified2_name' => 'snort.log',
                        'snort_stats_file' => '/var/log/snort/snort.stats',
                        'commands' => "service snortd restart\nservice barnyard2 restart\ninitctl stop rb-monitor\ninitctl start rb-monitor",
                        'gen_file' => '/etc/snort/gen-msg.map',
                        'reference_file' => '/etc/snort/reference.config',
                        'classification_file' => '/etc/snort/classification.config'
                      },
                      'rules' => {
                        'need_apply' => true
                      },
                      'index_partitions' => index_partitions || 5
                    }
                  }
                }
              }

              service_provider = @sensor.ancestors.find_by(domain_type: Sensor::SERVICE_PROVIDER)
              deployment       = @sensor.ancestors.find_by(domain_type: Sensor::DEPLOYMENT)
              namespace        = @sensor.ancestors.find_by(domain_type: Sensor::NAMESPACE)
              organization     = @sensor.ancestors.find_by(domain_type: Sensor::ORGANIZATION)
              market           = @sensor.ancestors.find_by(domain_type: Sensor::MARKET)

              %w(service_provider deployment namespace organization market).each do |v|
                node_json['attributes']['redborder']["#{v}"] = eval(v).try('name')
                node_json['attributes']['redborder']["#{v}_uuid"] = eval(v).try('uuid')
              end

              node = Chef::Node.from_hash(node_json)
              node.create

              # create client
              node_name = "rbipsg-#{@sensor.id}"
              client_json = { 'name' => node_name, 'validator' => true }
              @client = Chef::ApiClient.from_hash(client_json)
              @client = @client.save
             
              # Give to client permissions over the node
              rest = Chef::ApiClient.new(Chef::Config[:chef_server_url])
              # possible options: %w(read update delete grant)
              %w(update).each do |perm|
                ace = rest.get_rest("nodes/#{node_name}/_acl")[perm]
                ace['actors'] << node_name unless ace['actors'].include?(node_name)
                rest.put_rest("nodes/#{node_name}/_acl/update", perm => ace)
              end

              # Give to client permissions over the node
              rest = Chef::ServerAPI.new(Chef::Config[:chef_server_url])
              # possible options: %w(read update delete grant)
              %w(update).each do |perm|
                ace = rest.get_rest("nodes/#{node_name}/_acl")[perm]
                ace['actors'] << node_name unless ace['actors'].include?(node_name)
                rest.put_rest("nodes/#{node_name}/_acl/update", perm => ace)
              end

              render json: { status: 'claimed', cert: @client['private_key'].to_json, nodename: node_name }
            end

            def apply_job_info
              if params[:uuid]
                # search for the stored job
                stored_job = StoredDelayedJob.where('options like ?', "%#{params[:uuid]}%").first
                return error_404("#{I18n.t 'sensors_controller_ips.no_job'}") unless stored_job.present?

                sensor_ids = stored_job.options[:ids]

                # update the sensors associated to the job
                sensors = Sensor.where(id: sensor_ids)
                sensors.update_all(need_apply: false, need_reputation_apply: false, applied_at: Time.now)
                stored_job_humanize = sensors.map(&:full_name).join(', ')

                stored_job.content = params[:content]
                stored_job.options = { ids: sensors.map(&:id), humanize: stored_job_humanize }
                stored_job.save

                sensors_generic = Sensor.unscoped
                                  .includes(:policy)
                                  .where(type: Sensor::IPS_GENERIC,
                                         policies: { updating: false })

                sensors_generic.each do |sensor|
                  role = sensor.chef_role
                  role.override_attributes['redborder']['ipsgeneric'] ||= {}
                  role.override_attributes['redborder']['ipsgeneric']['rules'] ||= {}
                  role.override_attributes['redborder']['ipsgeneric']['rules']['need_apply'] = false
                  role.override_attributes['redborder']['ipsgeneric']['rules']['uuid'] = nil
                  role.save
                end

                render json: { query: true }
              else
                error_500("#{I18n.t 'sensors_controller_ips.bad_request'}")
              end
            end

            # Will return code 200 if the sensor has new rules to be downloaded.
            # In other case, it will return 304.
            def has_new_config
              return error_404("#{I18n.t 'sensors_controller_ips.no_sensor'}") unless @sensor.present?
              role = @sensor.chef_role
              node = @sensor.chef_node
              node.normal['ohai_time'] = Time.now.to_i
              node.save
              new_rules = role.override_attributes['redborder']['ipsgeneric']['rules']['need_apply'] rescue false
              if new_rules
                uuid = role.override_attributes['redborder']['ipsgeneric']['rules']['uuid']
                render text: uuid, content_type: 'text/plain'
              else
                render text: "#{I18n.t 'sensors_controller_ips.rules'}", content_type: 'text/plain', status: 304
              end
            rescue StandardError => e
              error_500("#{I18n.t 'sensors_controller_ips.error'}")
            end

            def config_file
              if @sensor
                @policy_rules = @sensor.policy_rules.includes(:action, :rule)
                if @sensor.policy.nil?
                  @suppress_events_rules, @filter_events_rules = [[], []]
                else
                  @filter_events_rules = @sensor.policy.filter_events_rules
                  # We don't want to show suppress events that track by source and
                  # destination, so we use the ips_v2 method although it is a generic ips.
                  @suppress_events_rules = @sensor.policy.suppress_events_rules_for_ips_v2
                end
              end
              @proxy_verify_cert = (RedBorder::CONFIG[:proxy_verify_cert].present? && RedBorder::CONFIG[:proxy_verify_cert].eql?('true')) ? 0 : 1
              render 'sensors/config.text.erb'
            rescue StandardError => e
              error_500("#{I18n.t 'sensors_controller_ips.error'}")
            end

            private

            def validate_cert_api_ips
              validate_cert
            end

            def set_sensor_api_ips
              sensor_id = @username.split('-').last
              @sensor = Sensor.find(sensor_id)
            end
          end
        end
      end
    end
  end
end
