#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class BindingVarsJob < Struct.new(:verbose, :sensor_id, :user_id)

      include Redborder::Jobs::JobHelper

      def perform

        content = ""

        begin

          sensor = Sensor.find(sensor_id)

          content += "#{I18n.t 'jobs.recalculating'} #{sensor.full_name} #{I18n.t 'jobs.childs'}.\n"

          sensor.all_children(true).select{|s| s.is_sensor_binding?}.each do |sensor|
            content += "  - #{I18n.t 'jobs.binding_sensor'}: #{sensor.full_name} "
            role_real_sensor_parent = sensor.real_sensor_parent.chef_role
            ["ipvars", "portvars"].each{|type| role_real_sensor_parent.override_attributes["redborder"]["snort"]["groups"]["#{sensor.group_id}"]["bindings"]["#{sensor.binding_id}"][type] = sensor.role_value_for_vars(type)}
            role_real_sensor_parent.save
            content += "[OK]\n"
          end

        rescue Exception => e
          content += "\n\n"
          content += "#{e.class}: #{e.message}\n"
          e.backtrace.each do |line|
            content += "#{line}\n"
          end
          logit "#{e.class}: #{e.message}" if verbose
        ensure

          stored_job = StoredDelayedJob.create(job: "BindingVarsJob", user_id: user_id, options: { id: sensor_id, humanize: sensor.full_name }, content: content)

        end

      end

    end

  end

end
