#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class ClonePolicyJob < Struct.new(:verbose, :policy_src_id, :policy_dest_id, :user_id, :locale)
      include Redborder::Jobs::JobHelper

      def perform
        I18n.locale = locale
        content = ''

        begin
          policy = Policy.find(policy_src_id)
          new_policy = Policy.find(policy_dest_id)

          policy.policy_rules.each do |policy_rule|
            new_policy_rule = policy_rule.dup
            new_policy_rule.policy_id = new_policy.id
            new_policy_rule.save
          end

          policy.event_filters.each do |event_filter|
            new_event_filter = event_filter.dup
            new_event_filter.policy_id = new_policy.id
            new_event_filter.save
          end

          new_policy.update(updating: false)

          content += "#{I18n.t 'jobs.the_policy'} #{new_policy.name} #{I18n.t 'jobs.created'}.\n"

          Notification.create(
            user_id: user_id,
            comment: "#{I18n.t 'jobs.the_policy'} #{new_policy.name} #{I18n.t 'jobs.created'}.\n"
          )

        rescue Exception => e
          content += "#{e.class}: #{e.message}\n"
          e.backtrace.each do |line|
            content += "#{line}\n"
          end
          logit "#{e.class}: #{e.message}" if verbose
        ensure
          stored_job = StoredDelayedJob.create(job: 'ClonePolicyJob', user_id: user_id, options: { humanize: new_policy.name }, content: content)
        end
      end
    end
  end
end
