#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

  	class DeletePolicyJob < Struct.new(:policy_id, :user_id, :verbose, :locale)

  		include Redborder::Jobs::JobHelper

  		def perform
        I18n.locale = locale

  			content = ""

  			begin

  				@policy = Policy.find(policy_id)
  				policy_name = @policy.name

          Sensor.where(policy_id: @policy.id).update_all(policy_id: nil)

          @policy.destroy

  				content += "#{I18n.t 'jobs.the_policy'} #{policy_name} #{I18n.t 'jobs.deleted'}.\n"
  				Notification.create(user_id: user_id, comment: "#{I18n.t 'jobs.the_policy'} #{policy_name} #{I18n.t 'jobs.deleted'}.")

  			rescue Exception => e

  				content += "#{e.class}: #{e.message}\n"

          e.backtrace.each do |line|
            content += "#{line}\n"
          end

          logit "#{e.class}: #{e.message}" if verbose

  			ensure

  				stored_job = StoredDelayedJob.create(job: "DeletePolicyJob", user_id: user_id, options: { humanize: policy_name }, content: content)

  			end

  		end

  	end

  end

end
