#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class DeleteRuleDbVersionJob < Struct.new(:rule_db_version_id, :user_id, :verbose, :locale)

      include Redborder::Jobs::JobHelper

      def perform
        I18n.locale = locale

        content = ("#{I18n.t 'jobs.node'}" + `hostname`.chomp! + "\n\n")

        begin

          rule_db_version = RuleDbVersion.find(rule_db_version_id)
          rule_source = rule_db_version.rule_source

          rule_db_versions = rule_db_version.self_and_olders.order('created_at ASC')

          selected_rule_db_version = rule_db_versions.first
          rule_db_date = selected_rule_db_version.created_at
          selected_rule_db_version.destroy

          content += "#{I18n.t 'jobs.rule_version'} #{rule_source.name} (#{rule_db_date.strftime('%Y-%m-%d %H:%M')}) #{I18n.t 'jobs.deleted'}.\n"

          if rule_db_versions.count > 1
            Delayed::Job.enqueue(
              Redborder::Jobs::DeleteRuleDbVersionJob.new(
                rule_db_version_id, user_id, true, I18n.locale
              ), priority: 10, run_at: Time.now
            )
          else
            Notification.create(user_id: user_id, comment: "#{I18n.t 'jobs.rule_version'} #{rule_source.name} (#{rule_db_date.strftime('%Y-%m-%d %H:%M')}) #{I18n.t 'jobs.olders'}.")
          end

        rescue Exception => e

          content += "#{e.class}: #{e.message}\n"

          e.backtrace.each do |line|
            content += "#{line}\n"
          end

          logit "#{e.class}: #{e.message}" if verbose

        ensure

          stored_job = StoredDelayedJob.create(job: "DeleteRuleDbVersionJob", user_id: user_id, options: { humanize: rule_source.name }, content: content)

        end

      end

    end

  end

end
