#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class DeleteRuleSourceJob < Struct.new(:rule_source_id, :user_id, :verbose, :locale)

      include Redborder::Jobs::JobHelper

      def perform

        I18n.locale = locale

        content = ("#{I18n.t 'jobs.node'}" + `hostname`.chomp! + "\n\n")

        begin

          rule_source = RuleSource.find(rule_source_id)

          rule_source.destroy

          content += "#{I18n.t 'jobs.rule_source'} #{rule_source.name} #{I18n.t 'jobs.deleted'}.\n"
          Notification.create(user_id: user_id, comment: "#{I18n.t 'jobs.rule_source'} #{rule_source.name} #{I18n.t 'jobs.deleted'}.")

        rescue Exception => e

          content += "#{e.class}: #{e.message}\n"

          e.backtrace.each do |line|
            content += "#{line}\n"
          end

          logit "#{e.class}: #{e.message}" if verbose

        ensure

          stored_job = StoredDelayedJob.create(job: "DeleteRuleSourceJob", user_id: user_id, options: { humanize: rule_source.name }, content: content)

        end

      end

    end

  end

end
