#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class ImportRulesToPolicyJob < Struct.new(:policy_id, :add_rules, :file_name, :user_id)
      def perform
        content = ("#{I18n.t 'jobs.node'}" + `hostname`.chomp! + "\n\n")

        policy = Policy.find(policy_id)

        content += "#{I18n.t 'jobs.importing'} #{policy.name}."

        s3_config = YAML.load_file("#{Rails.root}/config/aws.yml")
        s3 = AWS::S3.new(access_key_id: s3_config[Rails.env]['access_key_id'],
                         secret_access_key: s3_config[Rails.env]['secret_access_key'],
                         s3_endpoint: s3_config[Rails.env]['s3_host_name'])

        obj = s3.buckets[s3_config[Rails.env]['bucket']]
              .objects["imports/#{file_name}"]

        file_path = "/tmp/#{file_name}"

        # Download s3 file
        File.open(file_path, 'wb') do |file|
          file.write(obj.read)
        end

        if add_rules
          created_count, updated_count = policy.add_rules_from_file(file_path)
          content += "#{created_count} #{I18n.t 'jobs.rules_added'}. #{updated_count} #{I18n.t 'jobs.rules_updated'}.\n"
        else
          rules_count = policy.replace_rules_from_file(file_path)
          content += "#{rules_count} #{I18n.t 'jobs.rules_added'}.\n"
        end

        content += "#{I18n.t 'jobs.import_finished'}.\n"

      rescue StandardError => se

        content += "#{se.class}: #{se.message}\n"
        se.backtrace.each do |line|
          content += "#{line}\n"
        end

      ensure
        policy.update(updating: false)
        obj.delete if obj

        StoredDelayedJob.create(
          job: 'ImportRulesToPolicyJob',
          user_id: user_id,
          content: content
        )
      end
    end
  end
end
