#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class LoadStoredPolicyJob < Struct.new(:verbose, :stored_policy_id, :user_id, :locale)
      include Redborder::Jobs::JobHelper

      def perform
        I18n.locale = locale
        content = ""
        stored_policy = StoredPolicy.find(stored_policy_id)
        content += "#{I18n.t 'jobs.restoring'} #{stored_policy.policy.name} #{I18n.t 'jobs.signature'}.\n"
        data = open(stored_policy.stored_rules.expiring_url, {ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE})
        rules_count = stored_policy.policy.replace_rules_from_file(data)
        content += "#{rules_count} #{I18n.t 'jobs.rules_added'}."
        Notification.create(user_id: user_id,
                              comment: "#{I18n.t 'jobs.the_backup'} #{stored_policy.created_at.strftime('%Y-%m-%d %H:%M:%S')} #{I18n.t 'jobs.restored'} #{stored_policy.policy.name}.")
      rescue Exception => e
        content += "#{e.class}: #{e.message}\n"
        e.backtrace.each do |line|
          content += "#{line}\n"
        end
        logit "#{e.class}: #{e.message}" if verbose
      ensure
        stored_policy.policy.update(updating: false)
        StoredDelayedJob.create(job: "LoadStoredPolicyJob", user_id: user_id, content: content, options: {policy_id: stored_policy.policy_id, humanize: "#{stored_policy.policy.name} #{stored_policy.created_at.strftime('%Y-%m-%d %H:%M:%S')}"})
      end

    end
  end
end
