#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class PolicyUpdateRuleDbVersionsJob < Struct.new(:policy_id, :user_id, :verbose, :locale)

    	include Redborder::Jobs::JobHelper

    	def perform
        I18n.locale = locale

    		content = ""

        begin

          policy = Policy.find(policy_id)

          if policy.block_updates

            content += "#{I18n.t 'jobs.the_signature'} #{policy.name} #{I18n.t 'jobs.blocked'}.\n"

          else

            content += "#{I18n.t 'jobs.updating_policy'} #{policy.name}...\n"

            policy.update_rule_db_versions

            content += "#{I18n.t 'jobs.cleaning'}\n"

            PolicyRule.includes(:rule).where("policy_id = ? AND rules.rule_db_version_id NOT IN (?)", policy.id, policy.rule_db_versions.map(&:id)).references(:rules).destroy_all

            content += "#{I18n.t 'jobs.checking'}\n"

            Sensor.where(policy_id: policy_id).update_all(need_apply: true)

            content += "#{I18n.t 'jobs.the_policy'} #{policy.name} #{I18n.t 'jobs.updated'}.\n"

            Notification.create(user_id: user_id, comment: "#{I18n.t 'jobs.the_policy'} #{policy.name} #{I18n.t 'jobs.updated'}.")

          end

        rescue Exception => e

          content += "#{e.class}: #{e.message}\n"

          e.backtrace.each do |line|
            content += "#{line}\n"
          end

          logit "#{e.class}: #{e.message}" if verbose

        ensure

          stored_job = StoredDelayedJob.create(job: "PolicyUpdateDbVersionsJob", user_id: user_id, options: { humanize: policy.name }, content: content)

          policy.update(updating: false)

        end

      end

    end

  end

end
