#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class StoredPoliciesJob < Struct.new(:verbose, :sensor_ids, :user_id)
      include Redborder::Jobs::JobHelper

      def perform
        content = ""
        policies = Policy.includes(:sensors).where(sensors: {id: sensor_ids, type: Sensor::IPS_BINDING})

        policies.each do |policy|

          content += "- #{I18n.t 'jobs.snapshot'} #{policy.name} (#{policy.rules.count} #{I18n.t 'jobs.rules2'})\n"

          path = "#{Rails.root}/tmp/policy_#{policy.id}.csv"

          stored_policy = policy.stored_policies.build(policy: policy, rules: policy.rules.count, user_id: user_id)
          File.open(path, "w+") do |f|
            f.write(policy.rules_to_csv_format)
          end
          stored_policy.stored_rules = File.open(path)
          stored_policy.save
        end

      rescue Exception => e
        content += "#{e.class}: #{e.message}\n"
        e.backtrace.each do |line|
          content += "#{line}\n"
        end

        logit "#{e.class}: #{e.message}" if verbose
      ensure
        StoredDelayedJob.create(job: "StoredPoliciesJob", user_id: user_id, content: content, options: {ids: policies.map(&:id), humanize: policies.map(&:name).join(", ")})
      end

    end
  end
end
