#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module ReportBlockIPS

    def self.included(base)
      base.extend(ClassMethods)
      base.send(:include, InstanceMethods)

      base.class_eval do
        validates :aggregation, inclusion: { in: Event::AGGREGATIONS.keys.map(&:to_s) }, allow_blank: true, if: Proc.new { |w| w.product_type == "ips" }
        validate :required_params_ips, if: Proc.new { |w| w.product_type == "ips" }
        validate :cardinality_group_by_inclusion_ips, if: Proc.new { |w| w.product_type == "ips" }
      end
    end

    module ClassMethods
    end

    module InstanceMethods

      private

        def required_params_ips
          makeValidation = {
            report_block_type: report_block_type
          }

          if report_block_type == "Raw"
            if graph_type == "ball"
              makeValidation[:ball_max] = ball_max
            elsif graph_type != "value" and graph_type != "streams"
              makeValidation[:color] = color
            end

            makeValidation[:range] = range
          elsif report_block_type == "Top"
            makeValidation[:range] = range

            unless graph_type == "map"
              makeValidation[:column] = column
              makeValidation[:top_num] = top_num
            end

            if ["stacked", "area", "line", "sbars", "bars"].include? graph_type
              makeValidation[:show_total] = show_total
              makeValidation[:aggregation] = aggregation
            end
          elsif report_block_type == "Cardinality"
            if graph_type == "ball"
              makeValidation[:ball_max] = ball_max
            elsif graph_type == "value"
              makeValidation[:reference] = reference
            elsif ["stacked", "area", "line", "sbars", "bars"].include? graph_type
              makeValidation[:show_total] = show_total
            else
              makeValidation[:cardinality_group_by] = cardinality_group_by
            end
            makeValidation[:column] = column
            makeValidation[:range] = range
          else
            makeValidation[:compare_range] = compare_range
          end

          makeValidation.each do |key, value|
            errors.add(key, "#{I18n.t('models.present2')}") if value.nil?
          end
        end

        def cardinality_group_by_inclusion_ips
          allow_columns = ["none"] + Event::COLUMNS.except(:conversation, :scatterplot, :src_map, :dst_map).to_a.map{|c| c[0].to_s unless c[1][:hide_raw]}
          unless allow_columns.include?(cardinality_group_by) || cardinality_group_by.nil?
            errors.add(:cardinality_group_by, "Invalid value")
          end
        end

        def print_filters_ips
          to_return = ""

          source = Redborder::DruidSource.get(source: 'ips')

          JSON.parse(filters).each do |k, v|
            to_return << "#{Event::COLUMN_NAMES[v['column'].to_sym]} #{v['operator']} #{source.data_column(v['column'], v['value'])}<br />"
          end

          to_return
        end

        def step_ips
          if granularity and Redborder::Rbdruid.granularity_list({start_time: get_regular_start_time, end_time: end_time}).include?(granularity.to_sym)
            Redborder::Rbdruid::GRANULARITY_TYPE[granularity.to_sym][:step]
          else
            real_granularity = Redborder::Rbdruid.granularity_list({start_time: get_regular_start_time, end_time: end_time}).first
            Redborder::Rbdruid::GRANULARITY_TYPE[real_granularity.to_sym][:step]
          end
        end

        def search_ips
          parse_filter(filters, Event::COLUMN_NAMES.keys, :events)
        end
    end
  end
end

ReportBlock.send(:include, RedBorder::ReportBlockIPS)
