#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# => Module helper added to sensor helper. This module contains methods
#    to help the render of sensor's UI.
#
module RedBorder
  module SensorsHelperIPS
    def self.included(base)
      base.send(:include, InstanceMethods)

      base.class_eval do
        unloadable
        alias_method_chain :available_licenses, :ips_sensors

        #
        # => Return version depending on sensor's type
        #
        def ips_version(sensor_type)
          case sensor_type
          when Sensor::IPS, Sensor::IPS_PROXY, Sensor::IPS_GENERIC
            3
          when Sensor::IPS_V2
            2
          end
        end
      end
    end

    module InstanceMethods
      #
      # Return a collection to be used in a select input that represent the available licenses
      # for a certain sensor.
      #
      def available_licenses_with_ips_sensors(sensor)
        type = sensor.type_to_s.eql?('sensor_proxy') ? 'sensor' : sensor.type_to_s
        licenses = License.available_by_type(type, sensor.ancestor_organization)
        licenses << sensor.license if sensor.license && !sensor.license.expired?

        licenses.uniq.map { |l| ["#{l.uuid} - #{l.expire_at.strftime('%Y-%m-%d')} (#{l.available_slots(type)})", l.id] }
      end
    end
  end
end
