require 'test_helper'

class EventsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      @admin = users(:admin)
      @root = sensors(:root)
      sign_in @admin
    end

    should 'not modify aggregation in session when getting a serie' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :get_serie, id: 'raw', granularity: 'hour', aggregation: 'signatures', w: true,
                      end_time: '1475668984', start_time: '1475625815', format: :json
      assert_not (session[:event][:aggregations] || []).include?(:signatures)
    end

    should 'modify aggregations when adding one' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :tops, aggregation: 'events', aggregation_action: 'add',
                 granularity: 'minute', range: 'last_1_hour'
      assert session[:event][:aggregations].include?(:events)
    end

    should 'modify aggregations when using replace parameter' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :index, aggregation: 'signatures', aggregation_action: 'replace', range: 'last_24',
                  stream_columns: %w(sensor_name msg)
      assert_equal session[:event][:aggregations], [:signatures]
    end

    should 'modify dimensions when going from raw table widget' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :index, aggregation: 'signatures', aggregation_action: 'replace', range: 'last_24',
                  stream_columns: %w(sensor_name msg)
      assert_equal session[:event][:active_columns], [:sensor_name, :msg]
    end
  end
end
