require 'test_helper'

class ReputationPoliciesControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      License.any_instance.stubs(:check_signature).returns(true)
      License.any_instance.stubs(:cluster_uuid).returns(Setting.cluster_uuid)

      @admin       = users(:admin)
      @root        = sensors(:root)
      @ips_group  = sensors(:ips_group)
      @rep_policy = reputation_policies(:one)

      sign_in @admin
    end

    should 'assign a reputation policy to an IPS Group with valid license' do
      patch :assign, id: @rep_policy.id, sensor_id: @ips_group.id
      @ips_group.reload

      assert_equal @ips_group.reputation_policy_id, @rep_policy.id
      assert @ips_group.need_apply
    end

    should 'not assign a reputation policy to an IPS Group with invalid license' do
      Sensor.any_instance.stubs(:license).returns(nil)

      patch :assign, id: @rep_policy.id, sensor_id: @ips_group.id
      @ips_group.reload

      assert_nil @ips_group.reputation_policy_id
      assert_not @ips_group.need_apply
    end
  end
end
