require 'test_helper'

class SensorsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      License.any_instance.stubs(:check_signature).returns(true)
      License.any_instance.stubs(:cluster_uuid).returns(Setting.cluster_uuid)

      @admin       = users(:admin)
      @root        = sensors(:root)
      @ips_binding = sensors(:ips_binding)
      @policy      = policies(:one)

      sign_in @admin
    end

    should 'assign a rule policy to an IPS Binding with valid license' do
      patch :assign_policy, id: @policy.id, sensor_id: @ips_binding.id
      @ips_binding.reload

      assert_equal @ips_binding.policy_id, @policy.id
      assert @ips_binding.need_apply
    end

    should 'not assign a rule policy to an IPS Binding with invalid license' do
      Sensor.any_instance.stubs(:license).returns(nil)

      patch :assign_policy, id: @policy.id, sensor_id: @ips_binding.id
      @ips_binding.reload

      assert_nil @ips_binding.policy_id
      assert_not @ips_binding.need_apply
    end

    should 'apply with valid license' do
      assert_difference "Delayed::Job.where('handler like ?', '%ApplyUpdateJob%').count", 1 do
        post :apply_update, sensor_ids: {standard: [@ips_binding.id] }
      end
    end

    should 'not apply with invalid license' do
      Sensor.any_instance.stubs(:license).returns(nil)

      assert_no_difference "Delayed::Job.where('handler like ?', '%ApplyUpdateJob%').count" do
        post :apply_update, sensor_ids: {standard: [@ips_binding.id] }
      end
    end
  end
end
