/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

// Tab selector
$(document).on('click', 'li.tab-selector', function() {
  if ($(this).attr("load") === "false") return;
  var id = $(this).attr('id');
  $('div.tab-pane').removeClass('in active');
  $('div.tab-pane#' + id).addClass('in active');
  // post controller to update tab
  $.post("/location/store_tab", { tab: id } );
});

$(document).on('click', 'li.tab-selector a', function() {
  if (event !== undefined)
    event.preventDefault();
});

// Load Tabs

function loadTab(params) {
  var url = "/location/load_tab";

  $.get(url, params, function() {
    var $tab = $("#location." + params.view + " ul.nav-tabs li#" + params.tab);
    $tab.attr("load", "true");
    $tab.find("a").attr("data-toggle", "tab");
  }, "script");
}

function drawChord(opts) {
  new RB.Chords({ container: opts.container, matrix: opts.matrix, neighs: opts.neighs });
}

// DatePicker Custom
$(document).on("click", ".hasDatePicker", function(e) {
  $( "#datepicker" ).datepicker("show")
  e.preventDefault();
});

function setDatePicker(url) {
  $("#datepicker").datepicker({
    onSelect: function(date) {
      var date = new Date(date).getTime() / 1000;
      window.location.href = url + "&custom=" + date;
    },
    maxDate: '+0D +0W +0M +0Y'
  });
}

// Load More

function loadMore(url, data, $that) {
  // Replace element Text.
  $that.text("...");
  $that.addClass("disabled");

  // Perform Get Request
  $.get(url, data, function() {
    $that.text(I18n.t('javascript.load_more'));
    $that.removeClass("disabled");
    // Increment Offset
    data.load_more_offset += 1;
    $that.data("offset", data.load_more_offset);
  }, "script");
}

// Set Listener for loading more tops element
$(document).on("inview", "#location .tops-offset #offset-btn", function(e) {
  var $this = $(this),
    data = $this.data();
  loadMore("/location/tops", {
    start_time: data.startTime,
    end_time: data.endTime,
    tab: data.tab,
    load_more_offset: data.offset,
    granularity: data.granularity
  }, $this);
});

// Set Listener for loading more movements map element
$(document).on("inview", "#location .movements-map-offset #offset-btn", function(e) {
  var $this = $(this),
    data = $this.data();
  loadMore("/location/movements_map", {
    start_time: data.startTime,
    end_time: data.endTime,
    tab: data.tab,
    load_more_offset: data.offset,
    granularity: data.granularity
  }, $this);
});

// Update General Info Table

function updateGeneralInfoData(tab, addedValue) {
  var $tableInfo = $(".tab-pane#" + tab + " .table-info"),
    $total = $tableInfo.find(".total"),
    $visibleRows = $tableInfo.find(".visible-rows"),
    visibleRows = $(".tab-pane#" + tab + " .tops-table-container tbody tr").length,
    visible = $total.data("visible") + addedValue,
    total = $total.data("total");

  var saturate = function(value, limit) {
    return value > limit ? limit : value;
  };

  // Override visible value
  $total.data("visible", visible);

  // Override text
  $total.text(saturate(visible * 100 / total, 100).toFixed(2) + "%");

  // Update visible rows
  $visibleRows.text(visibleRows);
}

// Update General Info Table for Movements map view

function updateGeneralInfoDataMovementsMap(tab, addedValue) {
  var $tableInfo = $(".table-info"),
    $total = $tableInfo.find(".total"),
    $visibleRows = $tableInfo.find(".visible-rows"),
    visibleRows = $(".tops-table-container tbody tr").length,
    visible = $total.data("visible") + addedValue,
    total = $total.data("total");

  var saturate = function(value, limit) {
    return value > limit ? limit : value;
  };

  // Override visible value
  $total.data("visible", visible);

  // Override text
  $total.text(saturate(visibleRows * 100 / total, 100).toFixed(2) + "%");

  // Update visible rows
  $visibleRows.text(visibleRows);
}

function fixChordTop(windowpos) {
  var $currentChord = $(".movements .chord-container:visible"),
    $currentPane = $(".movements .tab-pane:visible");

  if ($currentPane.offset().top <= (windowpos - 10)) {
    $currentChord.addClass("fixed-chord");
    // Fix Chord
  } else {
    $currentChord.removeClass("fixed-chord");
  }
}

/* ========================================================================
 *  SEARCH LOCATION
 * ======================================================================== */

var SearchLocation;
var campus_location_html;
var building_location_html;
var floor_location_html;
var zone_location_html;

SearchLocation = function() {

  function SearchLocation(selectData, callback) {
    var self = this;
    self.html = Handlebars.templates['search-rule']({});
    self.columns = selectData.location_columns;

    //Translating from init.rb
    for(var i = 0; i < self.columns.length; i++) {
      if (self.columns[i].value.startsWith("tr")) {
        self.columns[i].value = I18n.t(self.columns[i].value.slice(3))
      }
    }

    self.operators = selectData.operators;

    self.campuses = selectData.campuses;
    self.buildings = selectData.buildings;
    self.floors = selectData.floors;
    self.zones = selectData.zones;

    self.init();
    return self;
  };

  SearchLocation.prototype = {

    init: function(callback) {
      var self = this;
      var width = "100%";

      self.columns_html = Handlebars.templates['select']({
        name: "column-select",
        placeholder: I18n.t('javascript.choose_query'),
        width: width,
        data: {
          value: self.columns
        }
      });

      self.campuses_html = Handlebars.templates['select']({
        name: "domains-select",
        width: width,
        multiple: false,
        data: self.campuses
      });
      campus_location_html = self.campuses_html;

      self.buildings_html = Handlebars.templates['select']({
        name: "domains-select",
        width: width,
        multiple: false,
        data: self.buildings
      });
      building_location_html = self.buildings_html;

      self.floors_html = Handlebars.templates['select']({
        name: "domains-select",
        width: width,
        multiple: false,
        data: self.floors
      });
      floor_location_html = self.floors_html;

      self.zones_html = Handlebars.templates['select']({
        name: "domains-select",
        width: width,
        multiple: false,
        data: self.zones
      });
      zone_location_html = self.zones_html;

      self.operators_html = function($html, data) {
        var select = Handlebars.templates['select']({
          name: "operators-select",
          width: width,
          data: {
            value: data
          }
        });

        $html.find('div.operator-select').html(select);
      };

      self.datetime_picker = function(that) {
        that.datetimepicker({
          timeFormat: 'HH:mm',
          dateFormat: 'yy-mm-dd',
          numberOfMonths: 1,
          separator: ' '
        });
      };

    },

    searchUI: function(data, callback) {
      var self = this;
      $('#content #title').after(Handlebars.templates['search'](data));

      $('.search-content-add').on('click', function(e) {
        e.preventDefault();
        self.add(this);
      });

      $('.search-content-remove').on('click', function(e) {
        e.preventDefault();
        self.remove(this);
      });

      $('div.search-content-enable input').on('click', function() {
        if ($(this).is(':checked')) {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', false).css('opacity', 1);
        } else {
          $(this)
          .parents('.search-content-box')
          .find('.value *, .operator-select *, .column-select *')
          .attr('disabled', true).css('opacity', 0.8);
        };

        $('.add_chosen').trigger("chosen:updated");
      });

      $('button.submit-search').on('click', function(e) {
        e.preventDefault();
        self.submit();
      });

      $('#content #title').on('click', 'a.reset-search-form', function(e) {
        e.preventDefault();

        $('.rules').empty();

        self.add();
        self.add();
        self.add();
      });

      if (callback && (typeof callback === "function")) {
        callback();
      };
    },

    add: function(that, newOptions, callback) {
      var self = this;
      var options = {};

      if (options && (typeof options === "object")) {
        options = newOptions;
      };

      var $html = $(self.html);
      $html.find('div.column-select').html(self.columns_html);
      self.operators_html($html, self.operators.text_input);

      if (that && (typeof that === "object")) {
        $(that).parents('.search-content-box').after($html);
      } else {
        $('.rules').append($html);
      };

      $html.find('.add_chosen')
      .chosen({allow_single_deselect: false, width: '100%'})
      .change(function(event, data) {
        var value = $(this).val();
        var that = $(this).parents('.search-content-box');

        if (value === "start_time") {
          that.find('.value').html('<input id="time-'+ new Date().getTime() +'" class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text">');
          self.datetime_picker(that.find('.value input:first'))
          self.operators_html($html, self.operators.datetime);
        } else if (value === "end_time") {
          that.find('.value').html('<input id="time-'+ new Date().getTime() +'" class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text">');
          self.datetime_picker(that.find('.value input:first'))
          self.operators_html($html, self.operators.datetime);
        } else if (value === "domain") {
          that.find('.value').html(self.domains_html);
          self.operators_html($html, self.operators.select);
        } else if (value === "sensor_name") {
          that.find('.value').html(self.sensors_html);
          self.operators_html($html, self.operators.select);
        } else if (value === "zone") {
          that.find('.value').html(self.zones_html);
          self.operators_html($html, self.operators.select);
        } else if (value == 'window') {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="number" min="0" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.integer_input);
        } else if (value === "campus_uuid") {
          that.find('.value').html(self.campuses_html);
          self.operators_html($html, self.operators.select);
        } else if (value === "building_uuid") {
          that.find('.value').html(self.buildings_html);
          self.operators_html($html, self.operators.select);
        } else if (value === "floor_uuid") {
          that.find('.value').html(self.floors_html);
          self.operators_html($html, self.operators.select);
        } else if (value === "zone_uuid") {
          that.find('.value').html(self.zones_html);
          self.operators_html($html, self.operators.select);
        } else {
          that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
          self.operators_html($html, self.operators.text_input);
        }

        that.find('.add_chosen').chosen({
          allow_single_deselect: false,
          width: "100%"
        });

        if (callback && (typeof callback === "function")) {
          callback(that);
        };

        that.trigger('rbsearch:search:change');
      });

      $('.search-content-box:first')
      .find('.search-content-remove')
      .css('opacity', 1)
      .unbind('hover')
      .unbind('mouseover');

      return $html;
    },

    remove: function(that) {
      var self = this;
      if ($('.search-content-box').length > 2) {
        $(that).parents('.search-content-box').remove();
      } else if ($('.search-content-box').length == 2) {
        $(that).parents('.search-content-box').remove();

        $('.search-content-box:first')
        .find('.search-content-remove')
        .css('opacity', 0.4)
        .attr('title', I18n.t('javascript.you_must'));
      };
    },

    pack: function() {
      var self = this;
      var matchAll = false;

      if ($('#search select.global-match-setting').val() === "all") {
        matchAll = true;
      };

      var json = {
        match_all: matchAll,
        items: {}
      };

      $('.search-content-box').each(function(index, item) {
        var enabled = $(item).find('.search-content-enable input').is(':checked');
        var column = $(item).find('.column-select select').val();
        var operator = $(item).find('.operator-select select').val();
        var value = $(item).find('.value input, .value select').val();

        if (enabled){
          if (column == "start_time" || column == "end_time"){
            if ($(item).find('.value input').datetimepicker("getDate") == null) {
              alert(I18n.t('javascript.valid_date'));
              throw I18n.t('javascript.valid_date')
            }
            else{
              value = $(item).find('.value input').datetimepicker("getDate").getTime() / 1000;
              //value = new Date(value).getTime() / 1000; it doesn't work in Safari
            }
          }

          if ((column !== "") || (value !== "")) {
            json.items[index] = {
              column: column,
              operator: operator,
              value: value,
              enabled: enabled
            };
          };
        }
      });

      return json;
    },

    submit: function(callback, otherOptions) {
      var self = this;
      var search = self.pack();
      var csrf = $('meta[name="csrf-token"]').attr('content');

      if (typeof otherOptions !== "object") {
        otherOptions = {};
      };

      var update_url = function() {
        if (history && history.pushState) {
          history.pushState(null, document.title, '/filters/location/advanced');
        };
      };

      if (callback && (typeof callback === "function")) {
        callback(search, self);
      };

      if (search && !$.isEmptyObject(search.items)) {
        if (otherOptions.search_id) {
          post_to_url('/filters/location/advanced', {
            match_all: search.match_all,
            search: search.items,
            title: otherOptions.title,
            search_id: ""+otherOptions.search_id+"",
            authenticity_token: csrf
          });
        } else {
          post_to_url('/filters/location/advanced', {
            match_all: search.match_all,
            search: search.items,
            authenticity_token: csrf
          });
        };
      } else {
        flash_message.push({type: 'error', message: I18n.t('javascript.you_cannot')});flash();
      };
    },

    build: function(search) {
      var self = this;

      if (typeof search === "string") {
        var data = JSON.parse(search);
      } else if (typeof search === "object") {
        var data = search;
      } else {
        var data = {};
      };
      if (data.items) {
       var rules = data.items;
      } else if (data.search) {
        var rules = data.search;
      } else {
        var rules = {};
      };

      if (data.match_all && data.match_all === "false") {
       $('select.global-match-setting option[value="any"]').prop('selected', 'selected');
      } else {
        $('select.global-match-setting option[value="all"]').prop('selected', 'selected');
      };

      for (id in rules) {
        var item = rules[id];
        if (item.column != "client_profile"){
          var rule = self.add(false, {});

          var column = rule.find('div.column-select select');

          rule.bind('rbsearch:search:change', function() {
            var operator = $(this).find('div.operator-select select');
            var value = $(this).find('div.value select, div.value input');

            operator.find("option[value='"+item.operator+"']").prop('selected', 'selected');
            operator.trigger("chosen:updated");

            if (item.column === "start_time" || item.column === "end_time") {
              if ($.isNumeric(item.value)) {
                date = new Date(item.value*1000);
              } else {
                date = new Date(Date.parse(item.value, "yyyy-MM-dd HH:mm"))
              }
              date_string = $.datepicker.formatDate( "yy-mm-dd", date);
              date_hours = date.getHours();
              date_minutes = date.getMinutes();

              if (date_hours < 10) { date_hours = "0" + date_hours; }
              if (date_minutes < 10) {date_minutes = "0" + date_minutes; }

              value.prop('value', date_string + " " + date_hours + ":" + date_minutes);
            } else {
              if (operator.val() == "empty" || operator.val() == "not_empty") {
                $(this).find('.value').html('<input class="search-content-value" placeholder="" name="" type="text" disabled x-webkit-speech lang="en-US">');
              }else {
                if (operator.val() !== 'contains') {
                  var that = $(this).parents('.search-content-box');
                }
                if ((item.column === "sensor_name" || item.column === "domain") && operator.val() == "contains") {
                  $(this).find('.value').html('<input class="search-content-value" placeholder="" name="" type="text" disable x-webkit-speech lang="en-US">');
                  var value = $(this).find('div.value select, div.value input');
                } else if (item.column === "sensor_name" || item.column === "domain") {
                  that.find('.value').html(self.sensors_html);
                } else if (item.column === 'zone') {
                  that.find('.value').html(self.zones_html);
                } else if (item.column === 'campus_uuid') {
                  that.find('.value').html(self.campuses_html);
                } else if (item.column === 'building_uuid') {
                  that.find('.value').html(self.buildings_html);
                } else if (item.column === 'floor_uuid') {
                  that.find('.value').html(self.floors_html);
                } else if (item.column === 'zone_uuid') {
                  that.find('.value').html(self.zones_html);
                }
                value.prop('value', item.value);
              }

            }

            rule.unbind('rbsearch:search:change');
          });

          rule.attr('data-rule-id', id);

          column.find("option[value='"+item.column+"']").prop('selected','selected');
          column.trigger("chosen:updated").trigger('change');

          if (item.enabled === "false") {
            rule.find('div.search-content-enable input').prop('checked', false);
            rule.find('.value *, .operator-select *, .column-select *')
            .attr('disabled', true).css('opacity', 0.8);
          };
        }
      };
      $('.rules .add_chosen').trigger("chosen:updated");
    },

  };

  return SearchLocation;
}();

var flash_message   = [];
var csrf = $('meta[name="csrf-token"]').attr('content');

$(document).on('click', 'div#new_saved_location_filter_modal button.new-saved-search-record', function(e) {
  e.preventDefault();

  var url = '/saved_location_filters/create';
  var title = $('input#saved_location_filter_title').val();
  var search_public = $('input#saved_location_filter_public').is(":checked");

  if ($('#saved-search-form #saved_location_filter_title').val()=="") {
    $('#saved-search-form #saved_location_filter_title').css('border' , "2px solid #FD2E3F");
  } else {
    var dd = false;

    if (typeof rule !== "undefined") {
      dd = rule.pack();
    } else {
      var json = $('div#search-params').text();

      if (json) {
        try {
          var tmp = JSON.parse(json)
          dd = {
            filter: tmp.search
          };
        } catch(e) {
          dd = false;
        }
      };
    };

    if (dd) {
      $('#saved-search-form .form-actions button.new-saved-search-record').attr('disabled', true);
      $('#saved-search-form .form-actions button.new-saved-search-record').text(I18n.t('javascript.please'));

      $.ajax({
        url: url,
        global: false,
        dataType: 'json',
        cache: false,
        type: 'POST',
        data: {
          "authenticity_token": csrf,
          "filter": {
            "title": title,
            "public": search_public,
            "filter": dd
          }
        },
        success: function(data) {
          flash_message.push({type: 'success', message: I18n.t('javascript.your_search')});
          flash();
          $('.modal').modal('hide');
        },
        error: function(xhr) {
          var error = $.parseJSON(xhr.responseText).error;
          flash_message.push({type: 'error', message: error});
          flash();
          $(document).trigger('limp.close');
          $('.modal').modal('hide');
        }
     });

    } else {
      flash_message.push({type: 'error', message: I18n.t('javascript.unknown_error')});
      flash();
      $(document).trigger('limp.close');
      $('.modal').modal('hide');
    };
  }
  return false;
});

$(document).on('change', 'article#location .operator-select .add_chosen', function(event, data) {
  var value = $(this).val();
  var that = $(this).parents('.search-content-box');
  var column = that.find('div.column-select select').val();

  if (value === "empty" || value === "not_empty") {
    that.find('.value').html('<input class="search-content-value" placeholder="" name="" type="text" disabled>');
  } else if (column == "sensor_name" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "sensor_name" && value != "contains") {
    that.find('.value').html(sensors_location_html);
  } else if (column == "zone" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "zone" && value != "contains") {
    that.find('.value').html(zones_location_html);
  } else if (column == "domain" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "domain" && value != "contains") {
    that.find('.value').html(domain_location_html);
  } else if (column == "campus_uuid" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "campus_uuid" && value != "contains") {
    that.find('.value').html(campus_location_html);
  } else if (column == "building_uuid" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "building_uuid" && value != "contains") {
    that.find('.value').html(building_location_html);
  } else if (column == "floor_uuid" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "floor_uuid" && value != "contains") {
    that.find('.value').html(floor_location_html);
  } else if (column == "zone_uuid" && value == "contains") {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  } else if (column == "zone_uuid" && value != "contains") {
    that.find('.value').html(zone_location_html);
  } else if (value == 'window') {
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="number" min="0" x-webkit-speech lang="en-US">');
  } else if (column != "start_time" && column != "end_time" && column != "domain" && column != "group_id" && column != "sensor_name"){
    that.find('.value').html('<input class="search-content-value" placeholder="' + I18n.t('javascript.enter_search') + '" name="" type="text" x-webkit-speech lang="en-US">');
  };

  that.find('.add_chosen').chosen({
    allow_single_deselect: false,
    width: "100%"
  });
});


function reloadRawIntervalFLOW (time) {
  var limit = time,
      n = limit;

  if (rbRawInterval !== undefined) {
    clearInterval(rbRawInterval);
    rbRawInterval = undefined;
  }

  rbRawInterval = setInterval(function () {
    if ($("body").hasClass("locations") && $("body").hasClass("index")) {
      $('li.reload a').html('<i class="fa fa-refresh"></i>' + (--n));

      if (n === 0) {
        n = limit;
        reloadRaw('location');
      }
    } else {
      clearInterval(rbRawInterval);
      rbRawInterval = undefined;
    }
  }, 1000);

  $('li.reload-option a').text(I18n.t('javascript.stop_reloading'));
};

$(document).on('submit', 'article#tops_locations form.navbar-search.form-horizontal', function(e){
  var input_val = $(this).find('input.search-query').val();

  if(input_val.length < 1){
    e.preventDefault();
  }
});


var timerIds = [];
var isLoading = false; 

function toggleLoad() {
  if (isLoading) {
    stopLoading();
  } else {
    startLoading();
  }
}

function startLoading() {
  var elements = $("a.top-element:not(.active)");
  elements.each(function(index, value) {
    var timerId = setTimeout(function() {
      $(value).click();
      // Verificar si este es el último elemento y, de ser así, detener la carga
      if (index === elements.length - 1) {
        stopLoading(); // Llama a stopLoading cuando se haya cargado el último elemento
      }
    }, index * 500);
    timerIds.push(timerId);
  });
  isLoading = true;
  if(elements.length === 0) { // Si no hay elementos para cargar, detener inmediatamente
      stopLoading();
  }
  updateButton();
}

function stopLoading() {
  timerIds.forEach(function(timerId) {
    clearTimeout(timerId);
  });
  timerIds = [];
  isLoading = false;
  updateButton();
}

function updateButton() {
  var $button = $("#toggle-autoload-link");
  if (isLoading) {
    $button.html('<i class="fa fa-stop"></i> Stop load visibles');
  } else {
    $button.html('<i class="fa fa-play"></i> Auto load visibles');
  }
}
$(document).on('click', '#toggle-autoload-button a', function(event) {
  event.preventDefault();
  toggleLoad();
});
