/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

 // Functions for Location Session View

function setHistogramHeight(id) {
  var $histogram = $("#histogram-" + id),
    width = $histogram.width();
  $histogram.height(width / 5);
}

function sessionsTimeSerie(opts) {
  var container = "timeserie-" + opts.id,
    step = 3600000,
    dataMax = Math.max.apply(null, opts.data.map(function(d){ return d.event.events; })),
    timeSerie;

  if (opts.mustParse) {
    opts.data = parseSerieData(opts.data);
  }

  timeSerie = new RB.Serie({
    container: container,
    width: $("#" + container).width(),
    autoscaling: true,
    step: step,
    predict_area: false,
    tooltip: false,
    units: opts.unit,
    domain: [opts.startTime + step / 1000, opts.startTime - step / 1000 ]
  });

  timeSerie.add({
    data: opts.data,
    name: opts.id,
    type: "bars"
  });

  timeSerie.add({
    data: parseVisitorsData(opts.visitors, dataMax),
    type: "line",
    dashed: true,
    stroke: 5
  });
  
  window.addEventListener('resize', function() {
    timeSerie.resize();
  });
}

function parseVisitorsData(visitorsData, max) {
  var visitorsDataMax = Math.max.apply(null, visitorsData.map(function(d){ return d.event.events; })),
    coef = max / visitorsDataMax;
  for (var i = 0, x = visitorsData.length; i < x; i++) {
    visitorsData[i].event.events *= coef;
  }
  return visitorsData;
}

function parseSerieData(serieData) {
  var serieData_ = {},
    timestamp,
    element;

  for (var i = 0, x = serieData.length; i < x; i++) {
    timestamp = serieData[i].timestamp.split(" ");
    timestamp[1] = timestamp[1].split(":")[0] + ":00:00";
    timestamp = timestamp.join(" ");

    serieData_[timestamp] = serieData_[timestamp] || { sum: 0, elements: 0 };
    serieData_[timestamp].sum += serieData[i].event.events;
    serieData_[timestamp].elements += 1;
  }

  serieData = [];

  for (var i in serieData_) {
    element = serieData_[i];
    serieData.push({ event: { events: element.sum / element.elements }, timestamp: i });
  }
  return serieData;
}