#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class LocationController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  # Includes
  include Redborder::Rbdruid
  include Redborder::Location::Setter
  include Redborder::Location::SessionStorageManager
  include Redborder::Location::QueryManager
  include Redborder::Location::SessionsQueryManager
  include Redborder::Location::WeatherQueryManager
  include LocationHelper

  self.view_paths = 'lib/modules/location/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"
  append_view_path 'lib/modules/location/app/views/location'

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  # Common setters
  # Defined at Location::Setter
  before_action :set_source, :set_session

  before_action :set_tab, only: %i[get_client_latlong movements_map movements load_tab load_serie get_serie get_tops tops comparative store_tab export_tops_to_file]

  before_action :store_locations,
                only: %i[get_tops tops map movements_map movements comparative sessions weather]

  before_action :set_default_action,
                only: %i[index tops map movements_map movements comparative sessions weather]

  before_action :set_filter,
                only: %i[get_tops tops map movements_map movements comparative add_tab load_more_comparative add_tab load_tab load_serie sensor_children
                         get_serie get_client_latlong sessions weather search export_tops_to_file]

  before_action :int_times, :set_times,
                only: %i[add_tab get_serie get_client_latlong load_serie export_tops_to_file get_tops movements_map]

  before_action :set_range,
                only: %i[get_tops tops map movements_map movements comparative load_more_comparative add_tab
                        load_tab sessions weather]

  before_action :set_granularity,
                only: %i[tops map movements_map movements comparative add_tab load_tab load_serie sessions weather get_tops get_serie get_client_latlong movements_map]

  before_action :restore_locations,
                only: %i[tops map movements_map movements comparative add_tab load_tab load_serie sessions weather get_tops movements_map]

  # Store view in session
  before_action :store_view, only: %i[tops map movements_map movements comparative sessions weather]

  # Restore variables from session
  before_action :restore_view, :restore_chart_type,
                only: %i[tops map movements_map movements comparative add_tab load_tab load_serie sessions weather get_tops movementes_map]

  # We dont need to call restore_aggregation because we use set_aggregation what does the same and more
  # before_action :restore_aggregation,
  #               only: %i[tops map movements comparative load_more_comparative add_tab
  #                        load_tab load_serie sessions weather get_tops]

  #  NOTE: Before calling 'store_aggregation', session variables (@session) and aggregations (@aggregations) must exist.
  before_action :set_aggregation, only: %i[get_serie export_tops_to_file tops map movements comparative 
                                           load_more_comparative add_tab load_tab load_serie sessions weather get_tops movements_map]
  
  # Store aggregation in session
  before_action :store_aggregation, only: [:tops, :map, :movements, :sessions]

  # Actions For Load More
  before_action :set_load_more_options, only: [:tops, :movements_map], if: -> { params[:load_more_offset] }

  before_action :filters_from_url, only: %i[get_tops get_serie tops get_client_latlong movements movements_map]


  before_action :should_skip_empty, only: [:get_serie, :get_client_latlong]

  before_action :set_module_settings, only: :search
  before_action :set_saved_location_filters,
                only: %i[tops map movements_map movements comparative sessions weather]

  # TODO: Check why add_tab is here?
  before_action :set_filter_for_csv,
                only: %i[add_tab export_tops_to_file]

  skip_before_action :filter_params, :save_aggregation_to_session

  ## ======================================================================
  # => Methods
  ## ======================================================================

  #
  # Main Views Actions
  #

  def index
    action = { action: @session[:view] || @source.default_view }
    redirect_to action.merge(params.except(:action))
  end

  def tops
    @data = tops_data

    # Remove first N elements of data when request comes from load_more
    if @load_more_offset
      @data.shift((@load_more_offset - 1) * current_user.per_page_count)
    else
      @total = total_data
    end
  end

  def map
    @data = map_data
    @remote = true
  end

  def get_client_latlong
    @value = params[:value]
    @client_profile = params[:client_profile]

    # filter processing
    if params[:filters].present? && !JSON.parse(params[:filters]).empty?
      parsed_filters = JSON.parse(params[:filters])
      @uuid = parsed_filters["0"]["value"] if parsed_filters["0"] && parsed_filters["0"]["value"]
    end

    @data = client_latlong_data

    respond_to do |format|
      format.js
      format.json { render json: @data }
    end
  end

  def movements_map
    @data = movements_map_data

    if @load_more_offset
      @data.shift((@load_more_offset - 1) * current_user.per_page_count)
    else
      @total = total_data
    end
  end

  def movements
    @data = movements_data

    respond_to do |format|
      format.json { render json: @data[:chord] }
      format.html
      format.js
    end
  end

  #
  # Action for Sessions View
  #
  def sessions
    # Apply Default Filters
    @filter.add(:transition, 0)
    @filter.add(:type, 'floor')
    # Set @histogram, @percent_dwell_time, @cumulative_dwell_time, @visitors
    %w[histogram percent_dwell_time cumulative_dwell_time visitors].each do |data|
      instance_variable_set(:"@#{data}_data", send("sessions_#{data}_data"))
    end
  end

  def weather
    @filter.add(:transition, 0)
    @filter.add(:type, 'floor')
    weather_aggregation = params[:data_type] || session[:data_type] || 'temperature'

    session[:data_type] = weather_aggregation

    %w[visitors temperature wind_speed rain_1h snow_1h].each do |data|
      instance_variable_set(:"@#{data}_data", send("weather_#{data}_data"))
    end

    @data_for_chart = instance_variable_get("@#{weather_aggregation}_data")
    @weather_aggregation = weather_aggregation 

    @data_for_summary = process_and_condense_weather_data(weather_summary_data)
  end

  def save_tab_in_session(sess, tab)
    
  end

  def set_tab
    if action_name == 'movements_map'
      @tab = :client_mac
    elsif params[:id]
      @tab = params[:id].to_sym
    elsif params[:tab]
      @tab = params[:tab].to_sym
    else
      @tab = (@session[:tab] || @source.default_tab).to_sym
    end

    # TODO: Check wtf is this
    if action_name == 'movements' && @tab == :status
      @tab = @source::TABS.except(:status).keys.first
    end

    # TODO: Check if this is needed for all actions that calls this function
    if @tab =~ /_uuid$/
      @tab = @tab.to_s.gsub(/_uuid$/, "").to_sym
    end

    if @tab == :dot11_status
      @tab = :status
    end

  end

  def set_aggregation
    session_data = session[@source.session]
    @aggregations = session_data && session_data[:aggregations] ? session_data[:aggregations] : [@source.default_aggregation]

    @aggregations ||= [@source.default_aggregations]

    if params[:aggregation_action] == 'replace'
      @aggregations = [*params[:aggregation]].map(&:to_sym)
    elsif params[:aggregation_action] == 'add'
      @aggregations.unshift(params[:aggregation].to_sym)
    elsif params[:aggregation_action] == 'delete' && @aggregations.size > 1
      @aggregations.delete(params[:aggregation].to_sym)
    end

    @aggregation = if params[:aggregation] && params[:aggregation_action].nil?
                      [*params[:aggregation]].first.to_sym
                    else
                      @aggregations.first.to_sym
                    end
    if @aggregations.nil? || !@aggregations.include?(@aggregation)
      @aggregations = [*params[:aggregation]].map(&:to_sym)
    end
  end

  ##
  ## Interface
  ##

  def store_tab
    unless @tab.to_s.include?('_uuid')
      @session[:tab] = @tab 
    end
    render nothing: true
  end

  # TODO: Check what is this function doing here, 
  # is the goal to override the filters_controller one?
  def save_tab
    render nothing: true
  end

  def load_tab
    @view = %w[tops movements total serie weather map].include?(params[:view]) ? params[:view] : 'tops'
    @data = send("#{@view}_data")
    @total = total_data if @view == 'tops'
  end

  def load_serie
    @uuid = params[:uuid]
    @serie = serie_data
  end

  # This method returns raw data for Overview widgets
  #
  def get_serie
    @comparative = params[:comparative] == 'true'
    @client_profile = params[:client_profile]

    # Note: It looks in tops view the value in serie is pass as :uuid
    # but in widgets like timeseries is pass with :value
    @uuid = params[:uuid]
    @value = params[:value]
   
    if @comparative
      # idea, refactor this
      @granularity = case params[:id].to_sym
               when :pt24h then :pt1m
               when :pt168h then :pt5m
               when :pt720h then :hour
               when :pt4320h then :pt8h
               end

      # @value = Time.parse(params[:value]).to_i rescue nil
      # @serie = serie_data(comparative)
      @serie = get_comparative_serie
    else
      @serie = (params[:w] && params[:id] != 'tops') ? serie_data : serie_query
    end

    respond_to do |format|
      format.js
      format.json { render json: @serie }
    end
  end

  # this is and adaptation used by the Heat Map widget
  def get_tops_for_client_latlong
    # params correction when the creation of the widget
    # TODO: modified the defaults  params of the widgets instead of this workaround
    if params[:aggregation] == 'bps'
      params[:aggregation] = 'clients'
      params[:start_time] = (Time.at(params[:end_time].to_i) - (60 * 5)).to_i.to_s
    end

    tab = assign_param(@source::COLUMNS.keys, params[:id].to_sym)
    @serie = @source.where(
      query_type: :group_by,
      range: @range,
      start_time: Time.at(params[:start_time].to_i),
      end_time: Time.at(params[:end_time].to_i),
      granularity: :all,
      group_by: [:client_latlong],
      search: @filter.get,
      limit: params[:limit],
      aggregation: params[:aggregation]
    )

    respond_to do |format|
      format.json do
        render json: @serie.map do |s|
          { tab.to_sym => s.read_attribute(tab),
            @aggregation.to_sym => s.read_attribute(@aggregation),
            secondary_name:
              if @source::COLUMNS[tab.to_sym][:complementary_columns] && tab.to_sym != :sensor_name
                s.send(@source::COLUMNS[tab.to_sym][:complementary_columns].keys.first.to_s)
              else
                s.read_attribute(tab)
              end }
        end
      end
    end
  end

  def get_tops_for_others_columns
    tab = assign_param(@source::COLUMNS.keys, params[:id].to_sym)
    @serie = @source.topN(
      query_type: :group_by,
      start_time: @start_time,
      end_time: @end_time,
      range: @range || :custom,
      granularity: :all,
      limit: params[:limit],
      group_by: [@source::COLUMNS[tab.to_sym][:columns].keys[0]],
      search: @filter.get,
      aggregation: @aggregation
    )

    respond_to do |format|
      format.json do
        render json: {
          data: @serie,
        }
      end
    end
  end

  def export_tops_to_file
    tab = params[:tab]

    start_time = params[:start_time].to_time
    end_time   = params[:end_time].to_time
    range      = :custom
    lookup = [:client_mac, :client_mac_vendor, :calendar_item_name].include?(tab.to_sym)

    csv_info = @source.where(
      query_type: :group_by,
      range: range,
      start_time: start_time,
      end_time: end_time,
      sort: [@aggregation,
              @source::COLUMNS[tab.to_sym][:columns].keys.first],
      limit: params[:limit],
      granularity: :all,
      group_by: @source::COLUMNS[tab.to_sym][:columns].keys + 
                (@source::COLUMNS[tab.to_sym][:complementary_columns] ? @source::COLUMNS[tab.to_sym][:complementary_columns].keys : []),
      aggregation: @aggregations,
      search: @filter.get
    )
    locations = {
      locations: session[:location][:locations],
      lookup: lookup,
      aggregations: @source::AGGREGATIONS
    }
    filename = in_time_machine do
      Time.now.strftime('%Y%m%d%H%M%S') + "-#{@source}Tops_" + @source::COLUMNS[tab.to_sym][:columns].values.first.gsub(' ', '_') + '.csv'
    end
    content = @source.to_csv(csv_info, tab.to_s, @aggregations, locations)
    send_data content, filename: filename
  end

  def get_tops
    @start_time = params[:start_time]
    @end_time = params[:end_time]
    @client_profile = params[:client_profile]
    @aggregation = params[:aggregation]
    case params[:id].to_sym
    when :client_latlong
      get_tops_for_client_latlong
    when :client_mac, :client_mac_vendor, :calendar_item_name
      get_tops_for_others_columns
    else
      @serie = tops_data
      respond_to do |format|
        format.json do
          render json: {
            data: @serie,
            locations: @locations
          }
        end
      end
    end
  end

  private

  def set_saved_location_filters
    @saved_filters = SavedLocationFilter.where(user_id: current_user.id)
    @saved_filters_public = SavedLocationFilter.where
                                               .not(user_id: current_user.id)
                                               .select(&:public)
  end
end
