#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class SavedLocationFiltersController < ApplicationController
  include Redborder::Rbdruid
  # Storage of the filters
  include Redborder::Filters::Storage::Session

  self.view_paths = 'lib/modules/location/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  # Set source and options
  before_action :set_source, :set_module_settings, only: [:view]

  before_action :set_saved_location_filter, only: [
    :show, :view, :edit, :update, :title, :destroy, :add_saved_filter
  ]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @filters = (SavedLocationFilter.where(user_id: current_user.id) |
                SavedLocationFilter.where.not(user_id: current_user.id).select(&:public))
  end

  def new
    @filter = SavedLocationFilter.new
    render layout: false
  end

  def create
    return unless params[:filter]

    if params[:filter].is_a?(String)
      params[:filter] = JSON.parse(params[:filter])
    end

    # Chech has any param
    has_something = false
    unless params[:filter][:filter][:items].nil?
      params[:filter][:filter][:items].each do |_k, filter_item|
        if filter_item[:enabled] && !filter_item[:enabled].nil?
          has_something = true
        end
      end
    end

    if has_something
      params[:filter][:user_id] = current_user.id
      params[:filter][:filter] = params[:filter][:filter].to_json
      @filter = SavedLocationFilter.new(saved_location_filter_params)

      if @filter.save
        render json: @filter
      else
        render json: { error: @filter.errors.full_messages.map {|s| s.gsub("#{I18n.t('controllers.checksum_taken')}","#{I18n.t('controllers.filter_already')}")} }, status: 422
      end
    else
      render json: { error: "#{I18n.t('controllers.empty_filters')}" }, status: 422
    end
  end

  def add_saved_filter
    filters = ActiveSupport::JSON.decode(@filter.filter)
    content = (filters['search'] || filters['items']).to_json

    if content
      source = Redborder::DruidSource.get(source: 'location')
      f = Redborder::Filter.new(source)
      f.parse(content, true)
      save_filters(f.filters)

      redirect_to :back
    else
      redirect_to :back, flash: {
        error: "#{I18n.t('controllers.parsing_search')}"
      }
    end
  rescue ActionController::RedirectBackError
    redirect_to session[:location][:default_action], flash: {
      error: "#{I18n.t('controllers.parsing_search')}"
    }
  end

  def show
    if @filter
      if current_user.id == @filter.user.id || @filter.public
        render json: @filter
      else
        render json: {}
      end
    else
      render json: {}
    end
  end

  def view
    if @filter
      redirect_to saved_location_filters_path unless
        current_user.id == @filter.user.id
    else
      redirect_to saved_location_filters_path
    end
  end

  def edit
  end

  def update
    if @filter && current_user.id == @filter.user.id
      if params.key?(:filter)

        if params[:filter].is_a?(String)
          params[:filter] = JSON.parse(params[:filter])
        end

        @filter.filter = params[:filter].to_json
      end

      @filter.public = params[:public] if params.key?(:public)

      if @filter.save
        render json: @filter
      else
        render json: { error: @filter.errors }
      end
    else
      render json: {}
    end
  end

  def title
    return unless @filter && current_user.id == @filter.user.id
    @filter.title = params[:title] if params.key?(:title)

    if params.key?(:filter)

      if params[:filter].is_a?(String)
        params[:filter] = JSON.parse(params[:filter])
      end

      @filter.filter = params[:filter]
    end

    if @filter.save
      render text: @filter.title
    else
      render json: @filter.errors
    end
  end

  def destroy
    if @filter && current_user.id == @filter.user.id
      if @filter.destroy
        rb_flash "#{I18n.t('controllers.filter')} '#{@filter.title}' #{I18n.t('controllers.remove_success')}"
      else
        rb_flash "#{I18n.t('controllers.failed_filter')} '#{@filter.title}' #{I18n.t('controllers.successfully')}",
                 type: :error
      end
    end

    redirect_to saved_location_filters_path
  end

  private

  #
  # Set the source for these filters
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'location')
  end

  #
  # => Use callbacks to share common setup or constraints between actions.
  #
  def set_saved_location_filter
    @filter = SavedLocationFilter.find(params[:id])
  end

  #
  # => Never trust parameters from the scary internet,
  #    only allow the white list through.
  #
  def saved_location_filter_params
    params.require(:filter).permit(:user_id, :filter, :checksum,
                                   :title, :public)
  end
end
