DittoCode::HoldFile.if 'ENTERPRISE'

#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info "[RB] Loading Location module"

Redborder::Module.register :location do

  name 'tr.location2.mobility'
  path 'location'
  session_id 'location'
  data_source 'Location'

  menu :product_menu, :location,
       { controller: :location, action: :index },
       caption: "tr.location2.mobility", module: 'location',
       last: true,
       html: { class: 'block-ui-but' }

  overview_widgets [
    {
      name: 'tr.location2.visitors',
      partial: 'visitors',
      position: 1,
      info: "tr.location2.info"
    } #,
    #{
    #  name: 'Quality',
    #  partial: 'quality',
    #  position: 2,
    #  info: 'The average wireless network signal quality reported by the ' +
    #        'wireless controller based on RSSI calculations in the selected ' +
    #        'time period and it\'s tendency over the previous period.<br/><br/>' +
    #        '<b>0%</b>: RSSI is under -100dBm for every client<br/>' +
    #        '<b>100%</b>: RSSI is over -50dBm for every client'
    #},
    # {
    #   name: 'Average visitors',
    #   partial: 'average_visitors',
    #   position: 2
    # }
  ]

  widgets([
    {
      name: 'tr.location2.widgets.movements',
      id: 'movements',
      icon: 'fa-adjust',
      report: false,
      auto: false
    },
    {
      id: 'heatmap',
      name: 'tr.location2.widgets.heatmap',
      icon: 'icons/heatmap.svg'
    }],
    custom: false
  )

  columns(
    old: {
      columns: {
        old: 'Old'
      },
      hide_raw: true
    },
    new: {
      columns: {
        new: 'New'
      },
      hide_raw: true
    },
    client_mac: {
      columns: {
        client_mac: 'Client MAC'
      },
      complementary_columns: { client_mac_vendor: 'Client Mac Vendor' },
      shared: false,
      hide_global: true
    },
    client_mac_vendor: {
      enabled: false, size: 1, 
      columns: {
        client_mac_vendor: 'Client MAC Vendor',
      },
      shared: false,
      hide_global: true
    },
    calendar_item_name: {
      columns: {
        calendar_item_name: 'Calendar Item Name'
      },
      shared: false,
      hide_global: true
    },
    type: {
      columns: {
        type: 'Type'
      },
      hide_raw: true
    },
    transition: {
      columns: {
        transition: 'Transition'
      },
      hide_raw: true
    },
    client_latlong: {
      columns: {
        client_latlong: 'LatLng'
      },
      hide_raw: true
    },
    organization_uuid: {
      columns: {
        organization_uuid: 'Organization'
      },
      shared: false,
      hide_global: true,
      hide_raw: true
    },
    deployment_uuid: {
      columns: {
        deployment_uuid: 'Deployment'
      },
      shared: false,
      hide_global: true,
      hide_raw: true
    },
    market_uuid: {
      columns: {
        market_uuid: 'Market'
      },
      shared: false,
      hide_global: true,
      hide_raw: true
    },
    namespace_uuid: {
      columns: {
        namespace_uuid: 'Namespace'
      },
      shared: false,
      hide_global: true,
      hide_raw: true
    },
    campus_uuid: {
      columns: {
        campus_uuid: 'Campus'
      },
      shared: false,
      hide_global: true
    },
    building_uuid: {
      columns: {
        building_uuid: 'Building'
      },
      shared: false,
      hide_global: true
    },
    floor_uuid: {
      columns: {
        floor_uuid: 'Floor'
      },
      shared: false,
      hide_global: true
    },
    zone_uuid: {
      columns: {
        zone_uuid: 'Zone'
      },
      shared: false
    },
    dot11_status: {
      columns: {
        dot11_status: 'Status'
      },
      shared: false
    },
    client_profile: {
      columns: {
        client_profile: 'Client Profile'
      },
      shared: false,
      hide_raw: true
    },
    histogram_data: {
      columns: { histogram_data: 'Histogram Data' },
      shared: false,
      hide_raw: true
    },
    loyalty: {
      columns: {
        loyalty: 'Loyalty'
      },
      shared: false,
      hide_raw: true
    },
    weather_description: {
      columns: { weather_description: 'Weather Description' },
      shared: false,
      hide_raw: true
    },
    weather_icon: {
      columns: { weather_icon: 'Weather Icon' },
      shared: false,
      hide_raw: true
    }
  )

  tabs(
    campus: {
      name: 'Campus',
      parse: true,
      columns: [:campus_uuid],
      filter: { id: :campus_uuid, name: 'Campus' },
      icon: { type: :img, path: 'icons/campus.svg' }
    },
    building: {
      name: 'tr.location2.building',
      columns: [:building_uuid],
      filter: { id: :building_uuid, name: 'Building' },
      parse: true,
      icon: { type: :font, path: 'building' }
    },
    floor: {
      name: 'tr.location2.floor',
      columns: [:floor_uuid],
      parse: true,
      filter: { id: :floor_uuid, name: 'Floor' },
      icon: { type: :img, path: 'icons/floor.svg' }
    },
    zone: {
      name: 'tr.location2.zone',
      columns: [:zone_uuid],
      parse: true,
      filter: { id: :zone_uuid, name: 'Zone' },
      icon: { type: :font, path: 'map-marker' }
    },
    client_mac: {
      name: 'tr.location2.client_mac',
      columns: [:client_mac],
      parse: true,
      filter: { id: :client_mac, name: 'Client MAC' },
      icon: { type: :font, path: 'laptop' }
    },
    calendar_item_name: {
      name: 'tr.location2.calendar_item_name',
      columns: [:calendar_item_name],
      parse: true,
      filter: { id: :calendar_item_name, name: 'Calendar item name' },
      icon: { type: :font, path: 'calendar-o' }
    },
    status: {
      name: 'tr.location2.status',
      columns: [:dot11_status],
      filter: { id: :dot11_status, name: 'Status' },
      icon: { type: :font, path: 'dot-circle-o' }
    },
    client_mac_vendor: {
      name: 'tr.location2.client_mac_vendor',
      columns: [:client_mac_vendor],
      filter: { id: :client_mac_vendor, name: 'Client Mac Vendor' },
      icon: { type: :font, path: 'dot-circle-o' }
    }
  )

  aggregations(
    clients: {
      name: 'tr.location2.visitors', unit: 'visitors', value: :clients, size: 1, round: true,
      aggregations: [{ type: 'hyperUnique', name: 'clients', fieldName: 'clients' }],
      description: "tr.location2.number"
    },
    average_time: {
      name: 'tr.location2.average4', unit: 'minutes', value: :average_time, size: 1, round: true,
      aggregations: [
        { type: 'longSum', name: 'events',  fieldName: 'events' },
        { type: 'hyperUnique', name: 'clients',  fieldName: 'clients' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'average_time',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'events' },
            { type: 'hyperUniqueCardinality', fieldName: 'clients' }
          ]
        }
      ],
      description: "tr.location2.average"
    },
    dwell_time: {
      name: 'tr.location2.dwell_time', unit: 'minutes', value: :dwell_time, size: 1, round: true,
      aggregations: [
        { type: 'longSum', name: 'events',  fieldName: 'events' },
        { type: 'hyperUnique', name: 'sessions',  fieldName: 'sessions' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'dwell_time',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'events' },
            { type: 'hyperUniqueCardinality', fieldName: 'sessions' }
          ]
        }
      ],
      description: "tr.location2.time"
    },
    temperature: {
      name: 'temperature',
      unit: 'degrees',
      value: :temperature,
      size: 1,
      ignore: true,
      aggregations: [
        { type: 'doubleSum', name: 'raw_temperature', fieldName: 'sum_temperature' },
        { type: 'count', name: 'events',  fieldName: 'events'}
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'temperature',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'raw_temperature' },
            { type: 'fieldAccess', fieldName: 'events' }
          ]
        }
      ],
      description: "Show temperature divided by unique client count for different locations"
    },
    wind_speed: {
      name: 'Wind Speed',
      unit: 'degrees',
      value: :wind_speed,
      size: 1,
      ignore: true,
      aggregations: [
        { type: 'doubleSum', name: 'raw_wind_speed', fieldName: 'sum_wind_speed' },
        { type: 'count', name: 'events',  fieldName: 'events'}
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'wind_speed',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'raw_wind_speed' },
            { type: 'fieldAccess', fieldName: 'events' }
          ]
        }
      ],
      description: "Show wind divided by unique client count for different locations"
    },
    rain_1h: {
      name: 'rain_1h',
      unit: 'degrees',
      value: :rain_1h,
      size: 1,
      ignore: true,
      aggregations: [
        { type: 'doubleSum', name: 'raw_rain_1h', fieldName: 'sum_rain_1h' },
        { type: 'count', name: 'events',  fieldName: 'events'}
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'rain_1h',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'raw_rain_1h' },
            { type: 'fieldAccess', fieldName: 'events' }
          ]
        }
      ],
      description: "Show rain divided by unique client count for different locations"
    },
    snow_1h: {
      name: 'snow_1h',
      unit: 'degrees',
      value: :snow_1h,
      size: 1,
      ignore: true,
      aggregations: [
        { type: 'doubleSum', name: 'raw_snow_1h', fieldName: 'sum_snow_1h' },
        { type: 'count', name: 'events',  fieldName: 'events'} # Métrica hyperUnique para clientes
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'snow_1h',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'raw_snow_1h' },
            { type: 'fieldAccess', fieldName: 'events' }
          ]
        }
      ],
      description: "Show snow divided by unique client count for different locations"
    },
    ratio: {
      name: 'tr.location2.asociated', unit: '%', value: :ratio, size: 1, round: false,
      aggregations: [
        {
          type: 'filtered',
          filter: {
            type: 'selector',
            dimension: 'dot11_status',
            value: 'ASSOCIATED'
          },
          aggregator: {
            type: 'hyperUnique',
            name: 'associated',
            fieldName: 'clients'
          }
        },
        {
          type: 'hyperUnique',
          name: 'visitors',
          fieldName: 'clients'
        }
      ],
      post_aggregations: [{
        type: 'arithmetic',
        name: 'ratio',
        fn: '*',
        fields: [{
          type: 'arithmetic',
          name: 'ratio',
          fn: '/',
          fields: [
            { type: 'hyperUniqueCardinality', name: 'tot', fieldName: 'associated' },
            { type: 'hyperUniqueCardinality', name: 'rows', fieldName: 'visitors' }
          ]
        },
        {
          type: 'constant',
          name: 'const',
          value: 100
        }]
      }],
      description: "tr.location2.percentage"
    },
    cumulative_dwell_time: {
      name: 'tr.location2.cumulative',
      unit: 'minutes',
      value: :cumulative_dwell_time,
      size: 1,
      round: true,
      aggregations: [
        { type: 'longSum', name: 'sum_dwell_time', fieldName: 'sum_dwell_time' },
        { type: 'hyperUnique', name: 'sessions', fieldName: 'sessions' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'cumulative_dwell_time',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'sum_dwell_time' },
            { type: 'hyperUniqueCardinality', fieldName: 'sessions' }
          ]
        }
      ],
      description: "tr.location2.time"
    },
    quality: {
      name: 'tr.location2.quality', value: :quality,
      size: 1, round: true, unit: '%',
      aggregations: [
        { type: 'longSum', name: 'flows',    fieldName: 'events' },
        { type: 'longSum', name: 'sum_rssi', fieldName: 'sum_rssi' }
      ],
      post_aggregations: [{
        name: 'quality',
        type: 'javascript',
        fieldNames: [:sum_rssi, :flows],
        function: "function (sum_rssi, flows) { var quality = sum_rssi / flows; if (quality == 0 || quality < -100) { return 0; } else if (quality > -50) { return 100; } else { return (2 * (quality + 100)); } }"
      }],
      description: "tr.location2.average2"
    },
    hist_dwell: {
      name: 'hist_dwell',
      value: :hist_dwell,
      round: true,
      ignore: true,
      aggregations: [
        { type: 'approxHistogramFold', name: 'hist_dwell', fieldName: 'hist_dwell' }
      ],
      post_aggregations: [
        {
          type: 'buckets',
          name: 'histogram_data',
          fieldName: 'hist_dwell',
          bucketSize: 60,
          offset: 60
        }
      ]
    },
    fps: {
      name: 'tr.location2.flows', unit: 'fps', value: :fps, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'fps',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              fieldName: 'events'
            },
            {
              type: 'constant',
              value: proc { |t| t }
            }
          ]
        }
      ],
      description: "tr.location2.number2"
    },
    loyalty: {
      name: 'tr.location2.loyalty', unit: 'repetitions', value: :loyalty, size: 1, round: true,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' },
        { type: 'longSum', name: 'sum_repetitions', fieldName: 'sum_repetitions' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'loyalty',
          fn: '/',
          fields: [
            { type: 'fieldAccess', fieldName: 'sum_repetitions' },
            { type: 'fieldAccess', fieldName: 'events' }
          ]
        }
      ],
      description: "tr.location2.average3"
    }
  )
end

# Require controllers
require_dependency 'application_controller_location'

# Require modules
require_dependency 'movements_parser'
require_dependency 'session_storage_manager'
require_dependency 'query_manager'
require_dependency 'sessions_query_manager'
require_dependency 'weather_query_manager'
require_dependency 'setter'
require_dependency 'widgets_helper_location'

# extend DRUID_CONFIG
druid_location_config =
  YAML.load(File.read("#{Rails.root}/lib/modules/location/config/rbdruid_config.yml"))[Rails.env]
  .symbolize_keys

RedBorder::DRUID_CONFIG.merge! druid_location_config

to_prepare = proc do
  unless ApplicationController.include?(RedBorder::ApplicationControllerLocation)
    ApplicationController.send(:include, RedBorder::ApplicationControllerLocation)
  end
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperLocation)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperLocation)
  end
end

Rails.configuration.to_prepare(&to_prepare)
