module Redborder
  module Location
    #
    # Parse Location Data (from Druid) preparing it for views
    #
    class MovementsParser
      def initialize(data, location_names)
        @data = data
        @locations = @data.map { |move| [move.old, move.new] }.flatten.uniq
        @location_names = location_names
      end

      #
      # Returns as json
      #
      def to_json
        data_json = initialized_json_data
        @data.each do |movement|
          data_json[movement.old][movement.new] = movement.clients
        end
        humanized_json_data(data_json)
      end

      #
      # Returns a Hash with { locations: Array of locations, matrix: data matrix }
      #
      def to_chord
        json_data = to_json
        { locations: json_data.keys, matrix: json_data.map { |_, movements| movements.values } }
      end

      def to_movements_table
        data = @locations.map { |location| [location, { in: 0, out: 0 }] }.to_h
        @data.each do |movement|
          if movement.old != movement.new
            clients = movement.clients.round
            data[movement.old][:out] += clients
            data[movement.new][:in] += clients
          end
          data[movement.old][:uuid] ||= movement.old
          data[movement.new][:uuid] ||= movement.new
        end
        data.map { |uuid, value| [(@location_names[uuid] || uuid), value] }.to_h
      end

      private

      #
      # Replace every uuid by location name
      #
      def humanized_json_data(data)
        data.map do |uuid, movements|
          [@location_names[uuid] || uuid,
           movements.map { |uuid_, clients| [@location_names[uuid_] || uuid_, clients] }.to_h]
        end.to_h
      end

      #
      # Initialize json data with 0 values in order to fill it
      #
      def initialized_json_data
        initialized_json_data = {}
        @locations.each do |location|
          initialized_json_data[location] = {}
          @locations.each do |location_|
            initialized_json_data[location][location_] = 0
          end
        end
        initialized_json_data
      end
    end
  end
end
