#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Location
    #
    # Session Management for Location Module
    #
    # This module implements methods to store/restore
    # variables from session
    #
    module SessionStorageManager
      #
      # Store Chart Type via JS request
      # JS Response will upload chart type
      #
      def store_chart_type
        @session[:chart_type] = params[:chart_type]
        # Force set @chart_type
        restore_chart_type
      end

      #
      # Store Selected Tab in session
      #
      def store_tab
        @session[:tab] = params[:tab]
        render nothing: true
      end

      #
      # Store Activation of Status Tab
      #
      def store_status_tab
        @session[:status_tab] = params[:status_tab] == 'true'
        restore_view
      end

      #
      # Store Client Profile Filter Level at Session
      #
      def store_client_profile
        # If no CP parameter, SOFT level will be stored.
        @session[:client_profile] = params[:client_profile] ||
                                    ::Location::CLIENT_PROFILE[:soft][:filters]
        redirect_to(action: @session[:view])
      end

      private

      # Store Methods

      def store_locations
        @session[:locations] =
          (current_user.domains_with_permission +
          current_user.zones_with_permission).map do |location|
            [location.uuid.to_s, location.name]
          end.to_h.merge!('outside' => 'Outside')
          .merge!(::Location::STATUS)
      end

      def store_view
        # Actions could be Location::VIEWS
        @session[:view] = action_name.to_sym
      end

      def store_aggregation
        return unless params['w'].nil?
        @session[:aggregations] = @aggregations
        @session[:aggregation] = @aggregation
      end

      # Restore Methods

      def restore_locations
        @locations = @session[:locations]
      end

      def restore_view
        @view = @session[:view] || @source.default_view
      end

      def restore_aggregation
        @aggregations = @session.fetch(:aggregations, [@source.default_aggregation])
        @aggregations ||= [@source.default_aggregations]

        if params[:aggregation_action] == 'add'
          @aggregations |= [params[:aggregation].to_sym]
        elsif params[:aggregation_action] == 'delete' && @aggregations.size > 1
          @aggregations.delete(params[:aggregation].to_sym)
        end

        @aggregation = @aggregations.first
      end

      def restore_chart_type
        @chart_type = @session[:chart_type].try(&:to_sym) || @source.default_chart_type
      end
    end
  end
end
