#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base Redborder Module
#
module Redborder
  #
  # Base Location Module
  #
  module Location
    #
    # Location SessionsQueryManager implement methods to perform session queries
    #
    module SessionsQueryManager
      
      PERCENT_FACTOR = 100.to_f / 60
      FACTOR = 60

      private

      #
      # Override set_range method for sessions
      #
      def set_range
        return super unless action_name == 'sessions'
        @range = params[:range].try(:to_sym) || :today
        now = Time.now
        beginning_of_today = now.beginning_of_day
        @start_time =
          case @range
          when :today
            beginning_of_today
          when :yesterday
            beginning_of_today - 1.day
          when :one_week_ago
            beginning_of_today - 1.week
          when :last_12
            now - 12.hours
          when :last_24
            now - 24.hours
          when :custom
            Time.at(params[:custom].to_i)
          end
        @end_time = %i(today last_12 last_24).include?(@range) ? now : @start_time.end_of_day
      end

      # Data Methods

      #
      # Get Data for Histogram
      #
      def sessions_histogram_data
        result = histogram_query
        return [] if result.empty?
        # Parse Histogram Result to build the Graph,
        # Move this code to Client Side in the future
        result = result.first.histogram_data
        # Ignore 1st value
        result['breaks'].shift
        result['breaks'].each_with_index.map do |name, i|
          events = ((result['counts'][i] - (result['counts'][i + 1] || 0)) / FACTOR).to_i
          events = 0 if events < 0
          { name: "#{name.to_i / 60} h", events: events }
        end
      end

      #
      # Get Data for Perent DT Serie
      #
      def sessions_percent_dwell_time_data
        sessions_serie(aggregation: :dwell_time, granularity: :hour)
          .each { |el| el[:event][:events] *= PERCENT_FACTOR }
      end

      #
      # Get Data for Cumulative DT Serie
      #
      def sessions_cumulative_dwell_time_data
        sessions_serie(aggregation: :cumulative_dwell_time, granularity: :minute)
      end

      #
      # Get Data for Visitors Serie
      # Must be parsed at client side
      #
      def sessions_visitors_data
        sessions_serie(aggregation: :clients, granularity: :hour)
      end

      # Query Methods

      def histogram_query
        @source.where(
          query_type: :group_by,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          aggregation: :hist_dwell,
          group_by: [:transition],
          search: @filter.get
        )
      end

      def sessions_serie(opts)
        @source.timeserie(
          start_time: @start_time,
          end_time: @end_time,
          range: :custom,
          granularity: opts[:granularity],
          search: @filter.get,
          aggregation: opts[:aggregation]
        )
      end
    end
  end
end
