module Redborder
  module Location
    #
    # Module for setting common global variables for Location Module
    #
    module Setter
      private

      def set_source
        @source ||= ::Location
      end

      def set_filter
        @filter ||= Redborder::Filter.new(@source, load_filters)
        apply_client_profile_filters
        remove_wrong_shared_filters
      end

      def set_filter_for_csv
        @filter.add(:transition, @source::TRANSITION[:STATIC])
        
        set_type_filter
      end

      #
      # Apply Client Profile Configuration from session
      #
      def apply_client_profile_filters
        return if (@filter.get(:client_profile) || []).count == @session[:client_profile].count
        @filter.remove(:client_profile)
        @session[:client_profile].each do |client_profile, _|
          @filter.add(:client_profile, client_profile)
        end
        save_filters(@filter.filters)
      end

      #
      # As on the location module there are some filter renamed differently than the rest
      # For example floor_uuid column is show as Flow, we cannot share those columns
      # If you see the init.rb of the location module you will notice that for example
      # floor_uuid is shared = false, this method force that if Traffic has a column name
      # floor_uuid and shared is true (shared filter) then force the remove because we have
      # as shared false in the location module.
      # TODO: Think in a different way to fix all this (for example dont rename floor_uuid to floor)
      # in the columns of the location
      #
      def remove_wrong_shared_filters
        return if (@filter.filters[:shared] || []).count == 0
        @filter.filters[:shared].each do |column, value|
          @filter.remove(column) if @source.private_filter?(column)
        end
        save_filters(@filter.filters)
      end

      #
      # Domains for Sensor Tree
      #
      def set_domains
        @domains ||= Sensor.root_children.where(type: Sensor::DOMAIN)
      end

      #
      # Initialize start time and end time based on params
      #
      def set_times
        start_time = parse_time_or_default(params[:start_time], Time.now - 1.hour)
        end_time = parse_time_or_default(params[:end_time], Time.now)

        @start_time = in_time_machine { ensure_time(start_time) || parse_time(params[:start_time_serie]) || Time.now - 1.hour }
        @end_time = in_time_machine { ensure_time(end_time) || parse_time(params[:end_time_serie]) || Time.now }
      end

      def parse_time_or_default(time_value, default_time)
        if time_value.is_a?(Time)
          time_value
        else
          parse_time(time_value) rescue default_time
        end
      end

      def ensure_time(time_value)
        time_value if time_value.is_a?(Time)
      end

      def parse_time(time_string)
        Time.parse(time_string) rescue nil
      end

      def set_granularity
        list =
          Redborder::Rbdruid.granularity_list(start_time: @start_time, end_time: @end_time) +
          Redborder::Rbdruid.slow_granularity_list(start_time: @start_time, end_time: @end_time)
        @granularity = params[:granularity].try(&:to_sym) || list.first
        # Protect from unpermitted granularity
        @granularity = list.first unless list.include?(@granularity)
      end

      def set_session
        session[:location] ||= {}
        @session = session[:location]

        # Default Client Profile Configuration
        @session[:client_profile] ||= ::Location::CLIENT_PROFILE[:medium][:filters]
      end

      def set_load_more_options
        @load_more_offset = params[:load_more_offset].to_i
        @start_time = Time.at(params[:start_time].to_i)
        @end_time = Time.at(params[:end_time].to_i)
      end

    end
  end
end
