#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base Redborder Module
#
module Redborder
  #
  # Base Location Module
  #
  module Location
    #
    # Location SessionsQueryManager implement methods to perform session queries
    #
    module WeatherQueryManager
      
      PERCENT_FACTOR = 100.to_f / 60
      FACTOR = 60

      private

      # Data Methods


      #
      # Get Data for Visitors Serie
      # Must be parsed at client side
      #
      def weather_visitors_data
        weather_serie(aggregation: :clients, granularity: @granularity)
      end

      def weather_temperature_data
        weather_serie(aggregation: :temperature, granularity: @granularity)
      end

      def weather_wind_speed_data
        weather_serie(aggregation: :wind_speed, granularity: @granularity)
      end

      def weather_rain_1h_data
        weather_serie(aggregation: :rain_1h, granularity: @granularity)
      end

      def weather_snow_1h_data
        weather_serie(aggregation: :snow_1h, granularity: @granularity)
      end

      def weather_weather_description_data
        weather_serie(column: :weather_description, granularity: @granularity)
      end

      def weather_summary_data
        summary_query(aggregations: [:temperature, :rain_1h])
      end

      # Query Methods

      def histogram_query
        @source.where(
          query_type: :group_by,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          aggregation: :hist_dwell,
          group_by: [:transition],
          search: @filter.get
        )
      end

      def weather_serie(opts)
        @source.timeserie(
          start_time: @start_time,
          end_time: @end_time,
          range: :custom,
          granularity: @granularity,
          search: @filter.get,
          aggregation: opts[:aggregation]
        )
      end

      def summary_query(opts)
        @source.where(
          query_type: :group_by,
          start_time: @start_time,
          end_time: @end_time,
          granularity: @granularity,
          aggregation: opts[:aggregations],
          group_by: [:weather_description, :weather_icon],
          search: @filter.get
        )
      end
    end
  end
end
