#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module WidgetsHelperLocation
    def self.included(base)

      base.send(:include, InstanceMethods)

      base.class_eval do
        unloadable
        def more_info_url_location(widget)
          product = '/' + widget.product_type + '/'

          section = if widget.widget_type
            case widget.widget_type.to_sym
            when :top then "tops/"
            when :unique, :grouped_unique, :single_unique then "cardinality/"
            when :compare then "comparative/"
            when :floor_map then return widget.floor_id ? view_sensor_path(widget.floor_id) : nil
            else ""
            end
          else
            ""
          end

          params = {}
          params[:range] = widget.range if widget.widget_type.to_sym != :compare
          params[:compare_range] = widget.range if widget.widget_type.to_sym == :compare
          params[:search] = widget.filters if widget.filters
          params[:granularity] = widget.granularity if widget.granularity && widget.widget_type.to_sym != :compare
          if widget.serie_type && [:stacked, :area, :line, :sbars, :bars].include?(widget.serie_type.to_sym)
            params[:chart_type] = widget.serie_type
          elsif widget.graph_type.to_sym == :stream
            params[:chart_type] = :streams
          end
          params[:group_by] = widget.group_by or 'none' if (widget.widget_type.to_sym == :grouped_unique || widget.widget_type.to_sym == :single_unique)
          params[:aggregation] = widget.aggregation if (widget.widget_type.to_sym != :grouped_unique && widget.widget_type.to_sym != :single_unique)
          params[:client_profile] = widget.client_profile if widget.respond_to?(:client_profile)
          params[:stream_columns] = widget.stream_columns.delete_if(&:blank?).map(&:to_sym) if [:stream, :table_raw].include?(widget.graph_type.to_sym)
          params[:tab] = if widget.graph_type.eql?('map') && widget.map_column
            widget.map_column
          elsif widget.column
            widget.column
          end

          product + section + '?' + params.to_query
        end

        #
        # Return the floor with images current user can read
        #
        def floor_with_image_collection
          current_user
            .floor_with_permission
            .includes(:overlay_map)
            .select { |s| s.overlay_map.present? }
        end
      end
    end

    module InstanceMethods

    end

  end
end
