/*!
 * Copyright (c) 2014 ENEO Tecnología S.L.
 * This file is part of redBorder.
 * redBorder is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * redBorder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 * You should have received a copy of the GNU Affero General Public License
 * along with redBorder. If not, see <http://www.gnu.org/licenses/>.
 */

/* Detect click on a cluster node to filter the view */
$(document).on('click', '.cluster-filter', function(event){
  event.preventDefault();

  var el = $(this),
      name;

  if (el.data('name') !== undefined){
    name = el.data('name');
  } else {
    name = el.parents('.cluster-node').data('name');
  }

  if (el.hasClass('filtered')){
    clusterRemoveFilters(el);
  } else if (name == 'error' || name == 'warning') {
    // Type of the global filter
    var type = el.data('type')
    // Remove another filters...
    clusterRemoveFilters($('.cluster-filter.filtered'));
    // Filter now
    el.addClass('filtered');
    // Check by type
    if (type == 'all'){
      elements = $('.cluster-card');
    } else {
      elements = $(type + ' .cluster-card');
      $('.cluster-cards:not('+ type +') .cluster-card').addClass('not-reference');
    }
    // Iterate
    elements.each(function(i){
      var card = $(this);
      if(card.hasClass(name)){
        card.addClass('reference');
      } else {
        card.addClass('not-reference');
      }
    });
  } else {
    // Remove another filters...
    clusterRemoveFilters($('.cluster-filter.filtered'));
    // Filter now
    el.addClass('filtered');
    $('.cluster-card').each(function(i){
      var card = $(this);
      if(card.data('reference').indexOf(name) > -1){
        card.addClass('reference');
      } else if (card.data('name') !== undefined && card.data('name').indexOf(name) > -1) {
        card.addClass('reference');
      } else {
        card.addClass('not-reference');
      }
    });
  }
});

function clusterRemoveFilters(element){
  element.removeClass('filtered');
  $('.cluster-card.not-reference').removeClass('not-reference');
  $('.cluster-card.reference').removeClass('reference');
}

/* --------------------------------
 *  Show the payload
 * -------------------------------- */

$(document).on('click', '.task-payload', function(event){
  event.preventDefault();
  var json_payload = JSON.stringify($(this).data('payload'), undefined, 2);

  redBorder.modals.showModalWith(
    I18n.t('javascript.payload'),
    '<pre class="pretty-json">' + JSONSyntaxHighlight(json_payload) + '</pre>',
    redBorder.modals.dismissButton('Back'),
    '#default_large'
  )
});

/* --------------------------------
 *  Copy the id
 * -------------------------------- */

function prepare_copy_id_tasks(){
  if(detectFlashSupport()){
    var client = new ZeroClipboard($(".cluster-task-copy-id"));

    client.on('ready', function(event) {
      client.on('copy', function(event) {
        var clipboard = event.clipboardData;
        var id = $(event.target).data('id');
        clipboard.setData("text/plain", id);
      });

      // callback triggered on successful copying
      client.on('aftercopy', function(event) {
        var el = $(event.target);
        $(event.target).parent().find('.tooltip-inner').text('Task ID is copied!')
      });
    });

    // In case of error - such as Flash not being available
    client.on('error', function() {
      ZeroClipboard.destroy();
      $(".cluster-task-copy-id").addClass('.cluster-task-copy-id-alt');
    });

  } else {
    $(".cluster-task-copy-id").addClass('.cluster-task-copy-id-alt');
  }
}

/* --------------------------------
 *  Show the id
 * -------------------------------- */

$(document).on('click', '.cluster-task-copy-id-alt', function(event){
  event.preventDefault();
  redBorder.modals.showModalWith(
    I18n.t('javascript.identifier'),
    '<pre>' + $(this).data('id') + '</pre>',
    redBorder.modals.dismissButton('Back'),
    '#default_large'
  )
});

/* --------------------------------
 *  Filter by task status
 * -------------------------------- */

$(document).on('click', '.task-status-ratio .percent-item', function(event){
  var item = $(this);

  if(item.hasClass('filtered')){
    item.removeClass('filtered');
    item.siblings('.percent-item').removeClass('not-filtered');
    // Restore
    $.ajax({
      url: '/monitor/load_more_tasks?page=1&type=complete',
      type: 'GET',
      dataType: 'script',
      cache: false
    });
  } else {
    item.removeClass('not-filtered').addClass('filtered');
    item.siblings('.percent-item').removeClass('filtered').addClass('not-filtered');

    var status;
    if(item.hasClass('green')){
      status = 'success';
    } else {
      status = 'failed';
    }

    // Restore
    $.ajax({
      url: '/monitor/complete_tasks?status=' + status,
      type: 'GET',
      dataType: 'script',
      cache: false
    });
  }
});

$(document).on('click', '.task-namespace-filter', function(event){
  event.preventDefault();
  var item = $(this),
      tab = item.parent().data('tab');

  if(item.hasClass('btn-success')){
    item.removeClass('btn-success');
    // Restore
    $.ajax({
      url: '/monitor/load_more_tasks?page=1&type=' + tab,
      type: 'GET',
      dataType: 'script',
      cache: false
    });
  } else {
    // Filter
    item.addClass('btn-success');
    item.siblings('.task-namespace-filter').removeClass('btn-success');
    // Get data
    $.ajax({
      url: '/monitor/namespace_tasks?namespace=' + item.data('namespace') + '&tab=' + tab,
      type: 'GET',
      dataType: 'script',
      cache: false
    });
  }
});
