redBorder.widgets.types.monitor = function (widget, data) {
  var category = widget.property.category,
      elements = widget.property.elements;

  // We dont want the show total option activated, lets disable it
  widget.property.show_total = false;

  // Lets force the product type to monitor, just in case the
  // model itself didnt do it before
  widget.property.product_type = 'monitor';

  // Do the query to get the tops data
  function queryList(opts) {
    var container = "widget_placeholder_" + widget.id
    var $container = $('#' + container)
    if (widget['property']['status'] == false) {
      $container.parents('.widget').find('.widget-header').find('h4').find('i').remove();
      $container.parents('.widget').find('.widget-header').find('h4').append('<i class="fa fa-exclamation-triangle" style="color:#E90C0C; margin-left: 10px"></i>');
    }

    if (data === undefined || data.list === undefined) {
      return redBorder.widgets.util.query({
        widget: widget,
        url: '/monitor/get_tops/' + category,
        typeOpts: {
          aggregation: widget.property.aggregation,
          monitor_tab: widget.property.category,
          timezone: widget.property.timezone,
          limit: elements
        },
        graphOpts: opts
      }).then(function (data) {
        var datos = redBorder.widgets.util.parseTopData(data, 'average', 'sensor_name', elements);
        if (widget.property.aggregation == "events") {
          datos = redBorder.widgets.util.parseTopData(data, 'events', 'bulkstats', elements);
        }
        return datos.sort(function (a, b) {
          return b.events - a.events;
        });
      });
    } else {
      var datos = redBorder.widgets.util.parseTopData(data.list, 'average', 'sensor_name', elements);
      if (widget.property.aggregation == "events") {
        datos = redBorder.widgets.util.parseTopData(data, 'events', 'bulkstats', elements);
      }
      var ordered = datos.sort(function (a, b) {
                      return b.events - a.events;
                    });
      return $.Deferred().resolve(ordered);
    }
  }

  function queryTimeserie(value) {
    if (data === undefined) {
      var opts = {}
      if (value !== undefined) {
        opts = { value: value }
      }
      if (widget.property.aggregation == "events") {
        return redBorder.widgets.util.query({
          widget: widget,
          url: '/monitor/get_serie/' + category,
          typeOpts: {
            aggregation: widget.property.aggregation,
            timezone: widget.property.timezone,
            monitor_tab: widget.property.category,
            granularity: widget.property.granularity,
            bulkstats: value
          },
          graphOpts: opts
        });
      } else {
        return redBorder.widgets.util.query({
          widget: widget,
          url: '/monitor/get_serie/' + category,
          typeOpts: {
            aggregation: widget.property.aggregation,
            timezone: widget.property.timezone,
            monitor_tab: widget.property.category,
            monitor_sensor: value,
            granularity: widget.property.granularity
          },
          graphOpts: opts
        });
      }
    } else if (data.timeserie !== undefined) {
      return $.Deferred().resolve(data.timeserie);
    } else {
      return $.Deferred().resolve(data.timeseries[value]);
    }
  }

  // Gets the total data from that interval
  function queryTotal(value) {
    if (data === undefined || data.total === undefined) {
      var opts = {}

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/monitor/get_serie/' + category,
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          monitor_tab: widget.property.category,
          monitor_sensor: value,
          granularity: 'all'
        },
        graphOpts: opts
      }).then(function (data) {
        if(data.length === 0){
          return 0;
        } else {
          return data[0].event.events;
        }
      });
    } else {
      if (data.total.length === 0){
        return $.Deferred().resolve(0);
      }
      return $.Deferred().resolve(data.total[0].event.events);
    }
  }

  function queryPerformance(value) {
    if (data === undefined) {
      var opts = {}
      var intervals = {
        low: 0,
        medium: 0,
        high: 0,
        total: 0
      };

      return redBorder.widgets.util.query({
        widget: widget,
        url: '/monitor/get_serie/' + category,
        typeOpts: {
          aggregation: widget.property.aggregation,
          timezone: widget.property.timezone,
          monitor_tab: widget.property.category,
          monitor_sensor: value,
          granularity: widget.property.granularity
        },
        graphOpts: opts
      }).then(function (data) {
        intervals.total = data.length;
        data.forEach(function (e) {
          if (e.event.events < parseInt(widget.property.low_limit)) {
            intervals.low++;
          } else if (e.event.events < parseInt(widget.property.medium_limit)) {
            intervals.medium++;
          } else {
            intervals.high++;
          }
        });
        return intervals;
      });
    } else if (data.timeserie !== undefined) {
      return $.Deferred().resolve(data);
    } else {
      return $.Deferred().resolve(data);
    }
  }

  return {
    queryList: queryList,
    queryTimeserie: queryTimeserie,
    queryTotal: queryTotal,
    queryPerformance: queryPerformance
  }
}
