#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class MonitorCategoriesController < ApplicationController

  self.view_paths = 'lib/modules/monitor/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Controller properties
  ## ======================================================================

  layout false, only: [:new]

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  # Preload settings
  before_action :set_source, :set_module_settings, :set_tftp_ip, only: [:index, :add]
  before_action :set_sensor
  before_action :check_tabs, only: [:index, :new, :add]
  after_action :monitors_to_chef, only: [:add, :destroy]

  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index
    @monitor_categories =
      @sensor.monitor_categories
      .select { |mc| (Setting.send("monitor_#{mc.category}") == '1' ||
                     (Setting.send("monitor_#{mc.category}") == '0' &&
                      current_user.super_user?)) &&
                      SensorMonitor::TABS[mc.category.to_sym]
                      }
      .sort{ |mc1, mc2|  SensorMonitor::TABS[mc1.category.to_sym][:name] <=> SensorMonitor::TABS[mc2.category.to_sym][:name] } rescue []
  end

  def check_tabs
    CustomTab.all.each do |custom_tab|
      if !Redborder::Module.registered_modules[:monitor][:tabs].key? custom_tab.tab_name.to_sym
          Redborder::Module.registered_modules[:monitor][:tabs][custom_tab.tab_name.to_sym] = {  :name=>custom_tab.name,
                                                                                      :enabled=>custom_tab.enabled,
                                                                                      :percentage=>custom_tab.percentage,
                                                                                      :category=>custom_tab.category,
                                                                                      :units=>custom_tab.units,
                                                                                      :instance => custom_tab.tab_name.to_s + "_per_instance"
                                                                                   }

      end
    end

  end

  def new
    @monitor_categories =
      (MonitorCategory.select { |m| m.sensor_types.include? @sensor.type } - @sensor.monitor_categories)
      .select { |mc| (Setting.send("monitor_#{mc.category}") == '1' ||
                     (Setting.send("monitor_#{mc.category}") == '0' &&
                      current_user.super_user?)) &&
                      SensorMonitor::TABS[mc.category.to_sym] 
                       }
      .sort { |mc1, mc2| 
      SensorMonitor::TABS[mc1.category.to_sym][:name] <=> SensorMonitor::TABS[mc2.category.to_sym][:name] 
      } rescue []
  end

  def add
    @sensor.monitor_categories << MonitorCategory.find(params[:id])
    @monitor_categories = @sensor.monitor_categories
                          .select { |mc| (Setting.send("monitor_#{mc.category}") == '1' ||
                                         (Setting.send("monitor_#{mc.category}") == '0' &&
                                          current_user.super_user?)) }
  end

  def destroy
    @sensor.monitor_categories.delete(MonitorCategory.find(params[:id]))
    render js: "$('tr[data-monitor_category_id=#{params[:id]}]').remove();"
  end

  def restore
    DittoCode::Exec.if 'ENTERPRISE' do
      @sensor.monitor_categories.delete_all
      @sensor.monitor_categories << MonitorCategory.where(default: true)
        .select { |mc| mc.sensor_types.include? @sensor.type }
      @sensor.monitors_to_chef
    end
    redirect_to sensor_monitor_categories_path(@sensor)
  end

  private

  def set_sensor
    @sensor = Sensor.find(params[:sensor_id])
    return error_404('') unless current_user.can?(:read, @sensor)
  end

  def monitors_to_chef
    @sensor.monitors_to_chef
    @sensor.try(:send, "data_to_client_proxy_#{@sensor.type_to_s}") if @sensor.real_parent
  end

  #
  # Set the source, used to load inactive settings.
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'monitor')
  end

  #
  # Set the source, used to load inactive settings.
  #
  def set_tftp_ip
    @sensor = Sensor.find(params[:sensor_id])
    @vips = Setting.virtual_ips
    if @sensor.parent and @sensor.parent.is_client_proxy?
      @tftp_ip = @sensor.parent.ip
    else
      @tftp_ip = (@vips[:external_webui].nil? ? `cat /etc/hosts | grep rb-webui | awk '{print $1 }'`.strip : @vips[:external_webui])
    end
  end
end
