###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

require 'zk'
require 'getopt/std'

class MonitorController < Redborder::Commons::CommonController
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  include Redborder::Rbdruid

  # Tasks to show
  TASKS_PER_PAGE = 10

  self.view_paths = 'lib/modules/monitor/app/views'
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  ## ======================================================================
  # => Controller properties
  ## ======================================================================

  respond_to :html
  respond_to :js, :json, only: [:get_serie, :get_tops]
  helper :all
  layout false, only: [:change_chart_type]

  ## ======================================================================
  # => Before actions and filters
  ## ======================================================================

  # Init categories and settings to load dropdown menu on section navigation
  skip_before_action :verify_authenticity_token, raise: false # Turn off CSRF token
  before_action :set_module_settings, only: [:index, :add_tab, :del_tab,
                                             :search]
  before_action :set_range, :set_granularity, :set_chart_type, :set_saved_monitor_filters,
                :set_reports, only: [:index]
  before_action :int_times, only: [:get_serie, :get_tops]
  before_action :require_super_administrative_privileges,
                only: [:load_more_tasks, :indexed_data, :list, :create_monitor_category,
                       :update_monitor_category, :destroy_monitor_category]
  before_action :require_super_reader_privileges_cluster,
                only: [:zookeeper_nodes, :segments, :more_segments,
                       :load_segments_in_range, :delete_segment,
                       :delete_segments, :diagram, :cluster_diagram,
                       :hardware, :historicals, :indexing, :psql_nodes]
  before_action :require_super_administrative_privileges_cluster,
                only: [:delete_segment, :delete_segments]
  before_action :set_monitor_categories_modal_variables, only: [:new_monitor_category, :edit_monitor_category,
                                                                :update_monitor_category, :list,
                                                                :create_monitor_category, :get_categories]
  before_action :check_tabs, only: [:index, :add_tab, :list]
  before_action :set_monitor_category, only: [:delete_monitor_category, :edit_monitor_category, :remove_bulkstats_schema ]
  after_action :refresh_monitor_categories, only: [:update_monitor_category, :delete_monitor_category]
  after_action :clean_custom_tabs, only: [:delete_monitor_category, :update_monitor_category]
  after_action :force_bulkstats_update, only: [:create_monitor_category, :update_monitor_category]
  skip_before_action :set_default_action, :filter_params,
                     :save_aggregation_to_session
  ## ======================================================================
  # => Actions
  ## ======================================================================

  def index

    @tabs = @source::TABS.keys
    get_active_tabs(:active_tabs)
    unless session[:monitor][:active_tabs].include?(session[:monitor][:tab])
      session[:monitor][:tab] = session[:monitor][:active_tabs].first
    end

    if params[:category]
      if @tabs.map(&:to_s).include? params[:category]
        unless session[:monitor][:active_tabs].include?(params[:category].to_sym)
          session[:monitor][:active_tabs] << params[:category].to_sym
        end
        session[:monitor][:tab] = params[:category].to_sym
      else
        session[:monitor][:tab] = @tabs.first
      end
    end

    @granularity = params[:granularity]
    @rangemilis = ((@end_time - @start_time) * 1000).to_i
    @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
    @aggregation = :monitors
    @aggregations ||= [@aggregation, :events]

    @columns = @source::COLUMNS.keys
    @columns.delete(:bulkstats)

    @monitor = {}
    @is_tops_query = false
    @offset = params.fetch(:offset, 0).to_i
    session[:monitor][:active_tabs].each do |tab|
      if tab.to_sym == session[:monitor][:tab].to_sym
        content = @source.where(
          query_type: :group_by,
          monitor_tab: tab,
          range: @range,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          sort: [:max, :sensor_name],
          limit: current_user.per_page_count,
          offset: @offset,
          group_by: @columns,
          aggregation: :monitors,
          search: @filter.get
        )
        if !content.select{|element| element.min.is_a?(String) == true}.empty?
          @columns = @source::COLUMNS.keys
          @columns = [:monitor,:bulkstats]

          @is_tops_query = true
          content = @source.topN(
            query_type: :group_by,
            monitor_tab: tab,
            start_time: @start_time,
            end_time: @end_time,
            range: @range || :custom,
            granularity: :all,
            limit: (@load_more_offset || 1) * current_user.per_page_count,
            offset: @offset,
            group_by: @columns,
            search: @filter.get,
            aggregation: :events
          )
        end
        @monitor[tab.to_sym] = { content: content }
      else
        @monitor[tab.to_sym] ||= {}
        @monitor[tab.to_sym][:content] ||= @source.where(
          query_type: :group_by,
          monitor_tab: tab,
          range: @range,
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all,
          sort: [:max, :sensor_name],
          limit: current_user.per_page_count,
          offset: @offset,
          group_by: @columns,
          aggregation: :monitors,
          search: @filter.get
        )
      end
    end
  end

  def force_bulkstats_update
    DelayedJob.force_bulkstats_update if monitor_category_params[:technology] == "bulkstats"
  end

  def check_tabs
    # For every tab in session[:monitor] and in session[:active_tabs] check
    # if it is not registered in the Module delete it from session
    get_active_tabs(:active_tabs)
    session[:monitor][:active_tabs].each do |active_tab|
       if !Redborder::Module.registered_modules[:monitor][:tabs].key? active_tab.to_sym
        delete_tab_from_session(active_tab)
       end
    end
    if !Redborder::Module.registered_modules[:monitor][:tabs].key? session[:monitor][:tab].to_s.to_sym
        delete_tab_from_session(session[:monitor][:tab].to_s)
        session[:monitor][:tab] = session[:monitor][:active_tabs].first
    end

    CustomTab.all.each do |custom_tab|
      if !Redborder::Module.registered_modules[:monitor][:tabs].key? custom_tab.tab_name.to_sym
          Redborder::Module.registered_modules[:monitor][:tabs][custom_tab.tab_name.to_sym] = {  :name=>custom_tab.name,
                                                                                      :enabled=>custom_tab.enabled,
                                                                                      :percentage=>custom_tab.percentage,
                                                                                      :category=>custom_tab.category,
                                                                                      :units=>custom_tab.units,
                                                                                      :instance => custom_tab.tab_name.to_s + "_per_instance"
                                                                                   }

      end
    end

  end

  #
  # Retrieve data about Historical Nodes
  #
  def historicals
    cluster = Redborder::Cluster.instance
    @historicals = cluster.historicals.sort_by(&:tier)
    @colors = {}
    @historicals.each_with_index do |node, i|
      @colors[node.host] = color_palette(i)
    end
    @nodes_per_table = [(@historicals.size.to_f / 3).ceil, 1].max
    @tiers = cluster.tiers.sort_by(&:name)

    # Load errors
    @errors = check_size_error(@historicals, @errors, :host,
                               threshold: 80, type: :warning)
    @errors =
      check_size_error(@tiers, @errors, :name, threshold: 80, type: :error)
    @errors =
      check_empty_error(@historicals, @errors, :host, :warning)
    @errors =
      check_empty_error(@tiers, @errors, :name, :warning)
    @historical_errors =
      @errors.select { |k, _| @historicals.map(&:host).include?(k) }
    @tier_errors =
      @errors.select { |k, _| @tiers.map(&:name).include?(k) }
  rescue DruidConfig::Exceptions::NotAvailableNodes
    @not_available_nodes = true
  end

  #
  # Status of Druid Indexing service
  #
  def indexing
    cluster = Redborder::Cluster.instance
    @per_page = TASKS_PER_PAGE
    @running_tasks = cluster.running_tasks
    @waiting_tasks = cluster.waiting_tasks
    @pending_tasks = cluster.pending_tasks
    @complete_tasks = cluster.complete_tasks
    @namespace_tasks = { complete: Hash.new(0), running: Hash.new(0) }
    @complete_tasks.each do |task|
      @namespace_tasks[:complete][namespace_task(task)] += 1
    end
    @running_tasks.each do |task|
      @namespace_tasks[:running][namespace_task(task)] += 1
    end
    @success_ratio =
      (@complete_tasks.select(&:success?).size.to_f / @complete_tasks.size) * 100
    @colors = {}
    @workers = cluster.workers
    @total =  @workers.map(&:capacity).sum
    @workers.each_with_index do |node, i|
      @colors[node.host] = color_palette(i)
    end
    @worker_capacity = (@workers.map(&:used).sum.to_f / @total) * 100
    @errors = {}
    @errors = check_size_error(@workers, @errors, :host,
                               threshold: 90, type: :warning)
  rescue DruidConfig::Exceptions::NotAvailableNodes
    @not_available_nodes = true
  end

  #
  # Show statistics about indexed data from Namespaces and data sources
  #
  def indexed_data
    @sensor = Sensor.root
    @month = params['month'] ? params['month'].to_time : Time.now
    @now = Time.now
    @segments = DruidSegments.new(@month, @sensor)
    # Resume table
    @total = @segments.total
    max_namespace = @segments.by_namespaces.values.max
    max_topic = @segments.by_topics.values.max
    @biggest = {
      namespace:
        @segments.by_namespaces.select { |_, v| v == max_namespace }.keys.first,
      topic: @segments.by_topics.select { |_, v| v == max_topic }.keys.first
    }
    # Graphs
    @start_time = @month.beginning_of_month
    @end_time = @start_time + 1.month
    @timeserie_accumulated = @segments.timeserie_accumulated
    @timeserie_per_day = @segments.timeserie_per_day
  end

DittoCode::Exec.if 'ENTERPRISE' do
  def zookeeper_nodes
    @status = Redborder::Node.get_zookeeper_status
  end

  #
  # => Show segments of storage.
  #    It's showed inside a tab
  #
  def segments
    @segments = Segment
    @namespaces = current_user.data_sources
    if params[:datasource]
      @segments = @segments.where('datasource like ?', "#{params[:datasource]}%")
    end
    @segments =
      @segments.paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
  end

  #
  # => Get more segments of storage. It's rendered on js.
  #    It's showed inside a tab
  #
  def more_segments
    @filter = params.fetch(:datasource, :all)
    @namespace = params.fetch(:namespace, :all)
    # Check if the namespace is root
    root = Sensor.root.uuid.eql?(params[:namespace])
    params[:namespace] = @filter if root

    @segments = Segment
    if params[:datasource] && params[:datasource] != 'all'
      # Find segments with datasouce names started by params[:datasource]
      @segments = @segments.where('datasource like ?', "#{params[:datasource]}%")
    end

    if params[:namespace] && params[:namespace].to_sym != :all
      # Find segments with datasouce names ended by params[:namespace]
      @segments = @segments.where('datasource like ?', "%#{params[:namespace]}")
    elsif root
      # Find segments with datasouce names in all datasouces and default(root) namespace
      datasources = RedBorder::DRUID_CONFIG.map {|k, v| v["datasource"] rescue nil }.compact
      @segments = @segments.where('datasource IN (:datasources)', datasources: datasources)
    end

    if params[:date]
      @segments =
        @segments.where(
          'start like ?',
          "#{params[:date].split.join('%')}%"
        )
    end

    @segments =
      @segments.paginate(
        page: params[:page],
        per_page: current_user.per_page_count
      )
  end

  def load_segments_in_range
    datasource = params.fetch(:datasource, :all)
    namespace = params.fetch(:namespace, :all)

    root = Sensor.root.uuid.eql?(params[:namespace])
    namespace = datasource if root

    if datasource.to_sym == :all
      results =
        Segment.where(
          "to_timestamp(start, 'YYYY-MM-DD\"T\"HH24:MI:SS\".000Z\"') >= ? \
          AND to_timestamp(start, 'YYYY-MM-DD\"T\"HH24:MI:SS\".000Z\"') <= ?",
          params[:start], params[:end]
        )

      if namespace.to_sym == :all
        datasources = RedBorder::DRUID_CONFIG.map {|k, v| v["datasource"] rescue nil }.compact
        results = results.where('datasource IN (:datasources)', datasources: datasources)
      end
    else
      results =
        Segment.where('datasource like ?', "#{params[:datasource]}%")
        .where(
          "to_timestamp(start, 'YYYY-MM-DD\"T\"HH24:MI:SS\".000Z\"') >= ? \
          AND to_timestamp(start, 'YYYY-MM-DD\"T\"HH24:MI:SS\".000Z\"') <= ?",
          params[:start], params[:end]
        )
    end

    if namespace.to_sym != :all
      results = results.where('datasource like ?', "%#{params[:namespace]}")
    end

    array = []
    results.each do |elem|
      array << [DateTime.parse(elem.start).to_i, elem.payload_hash['size']]
    end

    render json: array
  end

  def delete_segment
    segment = Segment.find_by(id: params[:id])
    segment.destroy if segment
  end

  def delete_segments
    segments = params[:ids]
    segments.each do |segment_id|
      segment = Segment.find_by(id: segment_id)
      segment.destroy if segment
    end
    render :delete_segment
  end

  def psql_nodes
    @status = Redborder::Node.get_postgresql_status
  end
end

  #
  # => Show the diagram of the topology.
  #    It's showed inside a tab
  #
  def diagram
    # Cluster mode set in config file, it can be, 'rb-enrich' or 'realtime'
    @cluster_mode = YAML.load_file(Rails.root
      .join('config/redborder_config.yml'))[Rails.env]['cluster_mode']
    @nodes = []
    @nprobes = []
    @s3_nodes = []
    @web_nodes = []
    @druid_historicals = []
    @druid_brokers = []
    @kafka_nodes = []
    @middle_managers = []
    @rb_enrich = []
    @realtimes = []
    @http2k = []
    @postgresql = []
    @memcached = []

    begin
      @nodes = Redborder::Node.get_kafka_info.first
      @kafka_nodes = @nodes.select { |k, _| [:rb_event, :rb_flow].include? k }
                     .values.flat_map(&:values)

      managers = Manager.all
      managers.each do |manager|
        @s3_nodes << manager.name if manager.services['minio'][:status]
        @web_nodes << manager.name if manager.services['webui'][:status]
        @druid_historicals << manager.name if manager.services['druid-historical'][:status]
        @druid_brokers << manager.name if manager.services['druid-broker'][:status]
        #@nprobes << manager.name if manager.services['nprobe'][:status]
        @middle_managers << manager.name if manager.services['druid-middlemanager'][:status]
        #@rb_enrich << manager.name if manager.services['rb-enrich'][:status]
        @realtimes << manager.name if manager.services['druid-realtime'][:status]
        @http2k << manager.name if manager.services['http2k'][:status]
        @postgresql << manager.name if manager.services['postgresql'][:status]
        @memcached << manager.name if manager.services['memcached'][:status]
      end
    rescue StandardError => e
      @realtimes = []
      @nodes = []
      @rb_enrich = []
      @middle_managers = []
      @s3_nodes = []
      @web_nodes = []
      @druid_historicals = []
      @druid_brokers = []
      @nprobes = []
      @kafka_nodes = []
      @http2k = []
      @postgresql = []
      @memcached = []
      Rails.logger.error e
    end
  end

  def cluster_diagram
    diagram
    @container_id = params[:container_id]
    render layout: false, partial: 'cluster_diagram'
  end

  #
  # => Provides information from device sensors to the 'device' view.
  #    
  def device
    begin
      @sensorid = params[:id] || nil
      @chassis_info = {}
      @processor_info = {}
      @pcie_info = {}

      @sensors = if @sensorid 
                    Sensor.where(:id => @sensorid)
                 else 
                    Sensor.where(:type => Sensor::DEVICE)
                 end   

      @sensors.each do |sensor|
           
        @start_time = Time.now - 120
        @end_time = Time.now

        # retrieve the chassis information
        @chassis_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_chassisinfo' }, 
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )
          
        # retrieve the processor information
        @processor_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_processorinfo' }, 
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )

        # retrieve the pcie information
        @pcie_info[sensor.name.to_sym] = @source.selectmonitor(
          custom_filter: { "sensor_name" => sensor.name.to_s, "monitor" => 'redfish_system_pcieinfo' }, 
          start_time: @start_time,
          end_time: @end_time,
          granularity: :all
        )

      end
      render layout: false if request.xhr?
      
    rescue StandardError => e
      @chassis_info = nil
      @processor_info = nil
      @pcie_info = nil
      Rails.logger.error e
    end
  end


  # This method executes when you click on the Test button when creating a Monitor Category.
  # Runs snmpbulkwalk with the oid and returns the values.
  def test_oid
    community = Sensor.find(params[:sensor_id]).snmp_community
    sensor_ip = Sensor.find(params[:sensor_id]).ip
    output = ''
    commands = "nice -n 19 snmpbulkwalk -v 2c -c #{community} #{sensor_ip} #{params[:oid]} | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}'  | sed 's/ [A-Za-z]*$//' | awk '{print $NF}' | grep -o '[[:digit:]]*' | paste -s -d';' -"
    Net::SSH.start(
      'localhost', 'redBorder',
      auth_methods: ['publickey'], paranoid: false, timeout: 5,
      keys: ["#{Rails.root}/config/#{RedBorder::CONFIG[:rsa_filename]}"],
      config: false) do |ssh|
      # Execute the command
      if commands.respond_to?('each')
        commands.each { |cmd| output += ssh.exec!(cmd) }
      else
        output += ssh.exec!(commands)
      end
    end
    if request.xhr?
      render json: {result: output}, :layout => false
    end
  end

  def hardware
    begin
      @managers = Manager.all.sort_by(&:name)
      hardware_info
    rescue StandardError => e
      @info_disk = nil
      @info_memory = nil
      Rails.logger.error e
    end
  end

  #
  # Load more task for indexing
  #
  def load_more_tasks
    cluster = Redborder::Cluster.instance
    @type = params[:type]
    @page = params[:page]
    @per_page = TASKS_PER_PAGE
    @colors = {}
    if @type != 'complete'
      @workers = cluster.workers
      @workers.each_with_index do |node, i|
        @colors[node.host] = color_palette(i)
      end
    end
    @tasks =
      case @type
      when 'running'
        cluster.running_tasks
      when 'waiting'
        cluster.waiting_tasks
      when 'pending'
        cluster.pending_tasks
      else
        cluster.complete_tasks
      end
    @tasks = @tasks.paginate(page: @page, per_page: @per_page)
  end

  #
  # Return completed tasks based on the status
  #
  def complete_tasks
    @status = params[:status]
    @page = params[:page] || 1
    @per_page = TASKS_PER_PAGE
    cluster = Redborder::Cluster.instance
    @tasks = @status == 'failed' ? cluster.failed_tasks : cluster.success_tasks
    @tasks = @tasks.paginate(page: @page, per_page: @per_page)
    @tab = 'complete'
  end

  #
  # Return completed tasks based on the status
  #
  def namespace_tasks
    @namespace = params[:namespace]
    @tab = params[:tab]
    @page = params[:page] || 1
    @per_page = TASKS_PER_PAGE
    cluster = Redborder::Cluster.instance
    @tasks = case @tab
             when 'running'
               cluster.running_tasks
             when 'complete'
               cluster.complete_tasks
             end
    @colors = {}
    @workers = cluster.workers
    @workers.each_with_index do |node, i|
      @colors[node.host] = color_palette(i)
    end
    @tasks = @tasks
             .select { |t| namespace_task(t) == @namespace }
             .paginate(page: @page, per_page: @per_page)
  end

  #
  # Load the task status and return it to the user
  #
  def task_log
    s3 = Redborder::S3.new
    @task_id = params[:id]
    @download = params[:download] || false
    cluster = Redborder::Cluster.instance
    @task = cluster.task(@task_id)

    if @task.running?
      @log = @task.log
      render text: @log, content_type: 'text/plain' if @download
    else
      if s3.public? && @download
        redirect_to s3.object_url(:get, "rblog/#{@task_id}/log")
      else
        begin
          @log = open(
            s3.object_url(:read, "rblog/#{@task_id}/log"),
            ssl_verify_mode: OpenSSL::SSL::VERIFY_NONE
          ).read
        rescue OpenURI::HTTPError => e
          # Only protect against 404 errors
          if e.message.include?('404')
            @log = "#{I18n.t('controllers.log_not_found')}"
          else
            @log = "#{I18n.t('controllers.error_getting_log')}"
            Rails.logger.error(e)
            Rails.logger.error e.backtrace.join("\n")
          end
        end
        # Render if we wan't to download the log
        render text: @log, content_type: 'text/plain' if @download
      end
    end
  end

  def load_more
    @offset = params.fetch(:offset, 0).to_i
    @tab = params[:tab]
    @start_time = params[:start_time].to_time
    @end_time = params[:end_time].to_time
    @granularity = params[:granularity]
    @aggregation = :monitors
    @is_tops_query = false
    @columns = @source::COLUMNS.keys
    @columns.delete(:bulkstats)
    @series = @source.where(
      query_type: :group_by,
      sort: [:max, :sensor_name],
      monitor_tab: @tab,
      limit: current_user.per_page_count,
      start_time: @start_time,
      end_time: @end_time,
      offset: @offset,
      granularity: :all,
      group_by: @columns,
      aggregation: @aggregation,
      search: @filter.get
    )

    if @series.count > 0 && !@series.select{|element| element.min.is_a?(String) == true}.empty?
      @is_tops_query = true
      @columns = @source::COLUMNS.keys
      @columns = [:monitor,:bulkstats]
      @series = @source.topN(
        query_type: :group_by,
        monitor_tab: @tab,
        start_time: @start_time,
        end_time: @end_time,
        range: @range || :custom,
        granularity: :all,
        limit: (@load_more_offset || 1) * current_user.per_page_count,
        group_by: @columns,
        search: @filter.get,
        aggregation: :events
      )
    end

  end

  def load_tab
    @tab = params[:id]
    @tabs = @source::TABS.keys

    @start_time = Time.at(params[:start_time].to_i)
    @end_time = Time.at(params[:end_time].to_i)
    @granularity = params[:granularity]
    @range = params[:range]
    @aggregation = :monitors
    @is_tops_query = false
    @columns = @source::COLUMNS.keys
    @columns.delete(:bulkstats)
    @monitor = @source.where(
      query_type: :group_by,
      range: @range,
      sort: [:max, :sensor_name],
      monitor_tab: @tab,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: current_user.per_page_count,
      group_by: @columns,
      aggregation: @aggregation,
      search: @filter.get
    )
    if @monitor.count > 0 && !@monitor.select{|element| element.min.is_a?(String) == true}.empty?
      @is_tops_query = true
      @columns = @source::COLUMNS.keys
      @columns = [:monitor,:bulkstats]
      @monitor = @source.topN(
        query_type: :group_by,
        monitor_tab: @tab,
        start_time: @start_time,
        end_time: @end_time,
        range: @range || :custom,
        granularity: :all,
        limit: (@load_more_offset || 1) * current_user.per_page_count,
        group_by: @columns,
        search: @filter.get,
        aggregation: :events
      )
    end

  end

  def add_tab
    @tab = params[:id]
    @tabs = @source::TABS.keys
    session[:monitor][:active_tabs] << @tab.to_sym
    session[:monitor][:tab] = @tab.to_sym

    @start_time = params[:start_time].to_time
    @end_time = params[:end_time].to_time
    @granularity = params[:granularity]
    @rangemilis = ((@end_time - @start_time) * 1000).to_i
    @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
    @aggregation = :monitors
    @columns = @source::COLUMNS.keys
    @columns.delete(:bulkstats)
    @is_tops_query = false
    @monitor = @source.where(
      query_type: :group_by,
      range: @range,
      sort: [:max, :sensor_name],
      monitor_tab: @tab,
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      limit: current_user.per_page_count,
      group_by: @columns,
      aggregation: @aggregation,
      search: @filter.get
    )

    if @monitor.count > 0 && !@monitor.select{|element| element.min.is_a?(String) == true}.empty?
      @is_tops_query = true
      @columns = @source::COLUMNS.keys
      @columns = [:monitor,:bulkstats]
      @monitor = @source.topN(
        query_type: :group_by,
        monitor_tab: @tab,
        start_time: @start_time,
        end_time: @end_time,
        range: @range || :custom,
        granularity: :all,
        limit: (@load_more_offset || 1) * current_user.per_page_count,
        group_by: @columns,
        search: @filter.get,
        aggregation: :events
      )
    end

  end

  def del_tab
    @tab = params[:id]
    @tabs = @source::TABS.keys
    session[:monitor][:active_tabs].delete(@tab.to_sym)

    @start_time = params[:start_time].to_time
    @end_time = params[:end_time].to_time
    @granularity = params[:granularity]

    @new_tab = session[:monitor][:active_tabs].first.to_s
  end

  # In theory this methis is only gonna be used when the aggregation is "events"
  # At this moment this only is needed on bulkstats string values..
  def get_tops_serie
    #Save the old tab to rewrite by the bulkstats one..
    @oldtab = @tab
    @tab = "bulkstats"

    @action  = 'tops'
    timezone = params[:timezone]

    granularities = Redborder::Rbdruid.granularity_list(
      start_time: params[:start_time].to_time,
      end_time: params[:end_time].to_time)
    @granularity  = if params[:granularity].present?
                      params[:granularity]
                    else
                      granularities.first
                    end

    if @value.nil?
      search = @filter.get if params[:total].nil?
      @serie = @source.timeserie(
        range: @range,
        start_time: params[:start_time],
        end_time: params[:end_time],
        granularity: @granularity,
        aggregation: @aggregation,
        timezone: timezone,
        skip_empty: @should_skip_empty, # ??
        search: search
      )
    else
      @value = nil if params[:value].eql?('N/A')

      case @tab.to_sym
      when :wireless_station
        @filter.remove :zone
      when :sensor_name
        @filter.remove :domain
      end

      # Remove the column before add the value. If we hold all filters,
      # all graphs will be the same
      @filter.remove(@tab)

      # Add the filters needed to make the query..
      @filter.add(@tab.to_sym, [@value])
      @filter.add(:monitor, @oldtab)

      @serie = @source.timeserie(
        monitor_tab: params[:monitor_tab],
        monitor_sensor: params[:monitor_sensor],
        group_name: params[:group_name],
        instance: params[:instance].present? ? params[:instance] : nil,
        range: @range,
        start_time: params[:start_time],
        end_time: params[:end_time],
        granularity: @granularity,
        aggregation: @aggregation,
        group_by: @tab, # remove?
        timezone: timezone,
        skip_empty: @should_skip_empty, # remove?
        search: @filter.get,
        bulkstats: @value
      )
      @tab = @oldtab
    end
  end

  def get_serie
    @tab   = params[:id]
    @value = params[:value]
    @aggregation = params[:aggregation] if params[:aggregation]
    if @aggregation == "events"
      get_tops_serie
    else
      @value = params[:monitor_sensor]

      if params[:instance].present?
        if params[:group_name].present?
          @value = (params[:instance] + ' (' + @value + ' - ' +
            params[:group_name] + ')')
        else
          @value = (params[:instance] + ' (' + @value + ')')
        end
      else
        @value = (@value + ' - ' +
          params[:group_name]) if params[:group_name].present?
      end

      timezone = params[:timezone]
      granularities = Redborder::Rbdruid.granularity_list(
        start_time: params[:start_time].to_time,
        end_time: params[:end_time].to_time
      )

      @granularity = if params[:granularity].present?
                      params[:granularity]
                    else
                      granularities.first
                    end

      result = @source.where(
        query_type: :timeseries,
        monitor_tab: params[:monitor_tab],
        monitor_sensor: params[:monitor_sensor],
        group_name: params[:group_name],
        instance: params[:instance].present? ? params[:instance] : nil,
        range: @range,
        granularity: @granularity,
        start_time: params[:start_time],
        end_time: params[:end_time],
        aggregation: @aggregation,
        timezone: timezone,
        search: @filter.get
      )
      @serie = result.map { |e| { event: { events: e.send(@aggregation) }, timestamp: e.timestamp.utc.to_s } }

      if result.count > 0 && !result.select{|element| element.min.is_a?(String) == true}.empty?
        @value = params[:value]
        @aggregation = "events"
        @tab   = params[:id]
        get_tops_serie
      end
  end

  respond_with @serie
  end

  def get_tops
    timezone = params[:timezone]
    @aggregation = :monitors

    @serie = @source.where(
      query_type: :group_by,
      monitor_tab: params[:monitor_tab],
      granularity: :all,
      start_time: params[:start_time],
      end_time: params[:end_time],
      limit: params.fetch(:limit, current_user.per_page_count),
      search: @filter.get,
      group_by: @source::COLUMNS.keys,
      timezone: timezone,
      aggregation: @aggregation
    )

    respond_with @serie
  end

  def get_instances
    @tab      = params[:id]
    @value    = if params[:group_name].present?
                  (params[:monitor_sensor] + params[:group_name])
                else
                  params[:monitor_sensor]
                end

    @group    = params[:group_name] if params[:group_name].present?
    instances = @source::TABS[@tab.to_sym][:instance]
    timezone  = params[:timezone]

    @start_time  = params[:start_time].to_time
    @end_time    = params[:end_time].to_time
    @aggregation = :monitors

    @monitor = @source.where(
      query_type: :group_by,
      range: @range,
      sort: :instance,
      monitor_tab: instances,
      monitor_sensor: params[:monitor_sensor],
      group_name: params[:group_name],
      start_time: @start_time,
      end_time: @end_time,
      granularity: :all,
      group_by: (@source::COLUMNS.keys + [:instance]),
      aggregation: @aggregation,
      timezone: timezone,
      search: @filter.get
    )
  end

  def export_tops_to_csv
    @tab = session[:monitor][:tab]
    @start_time = params[:start_time].to_time
    @end_time = params[:end_time].to_time

    @available_tabs = session[:monitor][:active_tabs].collect{|k| [SensorMonitor::TABS[k.to_sym][:name].present? ? SensorMonitor::TABS[k.to_sym][:name] : SensorMonitor::TABS[k.to_sym][:columns].values.first, k] }
    @available_tabs = @available_tabs.collect{|k| if k[0].start_with? "tr." then ["#{I18n.t k[0][3..-1]}", k[1]] else [k[0], k[1]] end}

    render layout: false
  end

  def export_tops_to_file
    tab = params[:tab]

    start_time = params[:start_time].to_time
    end_time   = params[:end_time].to_time
    percentage = @source::TABS[tab.to_sym][:percentage]
    range      = :custom

    @monitor_events = @source.where(
      query_type: :group_by,
      monitor_tab: tab,
      range: range,
      start_time: start_time,
      end_time: end_time,
      granularity: :all,
      sort: [:max, :sensor_name],
      limit: current_user.per_page_count,
      group_by: @source::COLUMNS.keys,
      aggregation: :monitors,
      search: @filter.get
    )

    filename = in_time_machine { Time.now.strftime("%Y%m%d%H%M%S") + "-Monitor_" + tab.to_s + ".csv" }
    content = @source.to_csv(@monitor_events, percentage)
    send_data content, filename: filename
  end

  # Tools -> Monitor Categories
  def list
    sensor_type = "[ #{Sensor::DEVICE} ]"
    @categories = get_monitor_tabs_from_columns
    @monitor_categories = MonitorCategory.all.order('LOWER(name)').select{|m| m.sensor_types.include? Sensor::DEVICE }
  end

  def get_names_from_monitor_category
    names = []
    MonitorCategory.all.each do |monitor|
      operations = monitor.operation.is_a?(Hash) ? [monitor.operation] :
                                                    monitor.operation
      names += operations.map { |o| o['name'] }
    end
    names.flatten.uniq
  end

  def initialize_monitors_data_bag
    monitor_tabs = Chef::DataBagItem.new
    monitor_tabs.data_bag 'rBglobal'
    monitor_tabs['id'] = 'monitors'
    # Save Monitor Default Tabs to DataBag
    monitor_tabs['monitors'] = get_names_from_monitor_category
    monitor_tabs.save
    monitor_tabs
  end

  def add_item_to_monitors_data_bag(tab_name)
    monitor_tabs = nil
    begin
      monitor_tabs = Chef::DataBagItem.load('rBglobal', 'monitors')
    rescue Net::HTTPServerException
      monitor_tabs = initialize_monitors_data_bag
    end
    monitor_tabs['monitors'] = get_names_from_monitor_category if !monitor_tabs.key? 'monitors'
    monitor_tabs['monitors'].push(tab_name) if !monitor_tabs['monitors'].include? tab_name
    monitor_tabs.save
  end

  def delete_item_from_monitors_data_bag(tab_name)
    monitor_tabs = nil
    begin
      monitor_tabs = Chef::DataBagItem.load('rBglobal', 'monitors')
    rescue Net::HTTPServerException
      monitor_tabs = initialize_monitors_data_bag
    end
    monitor_tabs['monitors'].delete(tab_name) if monitor_tabs.key? 'monitors'
    monitor_tabs.save
  end

  def delete_items_from_monitors_data_bag(tabs)
    monitor_tabs = nil
    begin
      monitor_tabs = Chef::DataBagItem.load('rBglobal', 'monitors')
    rescue Net::HTTPServerException
      monitor_tabs = initialize_monitors_data_bag
    end
    tabs.each do |tab_name|
       monitor_tabs['monitors'].delete(tab_name) if monitor_tabs.key? 'monitors'
    end
    monitor_tabs.save
  end

  def get_monitor_tabs_from_columns
    tabs = {}
    Redborder::Module.registered_modules[:monitor][:tabs].each do |k,v|
      tabs[k] = v[:name]
    end
    tabs
  end

  def set_monitor_categories_modal_variables
    @sensor_types = {
                      "DEVICE" => Sensor::DEVICE
    }

    @technologies = {
      "SNMP" => "snmp",
      "Redfish" => "redfish",
      "IPMI" => "ipmi",
      "SNMP Operation" => "snmp_op",
      "Bulkstats" => "bulkstats"
    }

    # Build categories (tabs) hash
    @categories = get_monitor_tabs_from_columns

    @categories[:new_custom_tab] = "#{I18n.t('controllers.add_new')}"

    @main_categories = {
      "Hardware" => "Hardware",
      "IDS/IPS" => "IDS/IPS",
      "Network" => "Network",
      "System" => "System",
      "Custom" => "Custom"
    }

    @impacts = {
      "Very Low" => 1,
      "Low" => 2,
      "Medium" => 3,
      "Very High" => 4
    }

    # List of actual redfish commands
    @redfish = {}
    PredefinedOperation.where("technology = 'redfish'").each do |p_operation|
      @redfish[p_operation.name] = p_operation.operation
    end

    @ipmi = {}
    PredefinedOperation.where("technology = 'ipmi'").each do |p_operation|
      @ipmi[p_operation.name] = p_operation.operation
    end

  end

  def get_categories
    respond_to do |format|
      format.html  # index.html.erb
      format.json  { render :json => @categories.select {|k, v| v.include? params[:term] }.values }
    end
  end

  def new_monitor_category
     if params[:sensor]
       @sensor = JSON.parse(params[:sensor])
     end
     render layout: false
  end

  def operation_sanitation (operation)
    begin
      JSON.load(operation)
    rescue JSON::ParserError => e
      nil
    end
  end

  # Get a name and a unit to create a new tab for monitor
  # returns the tab_name  or
  # nil if we could not create it
  def create_new_custom_tab(name,unit='')
      # Normalize name and unit
      tab_unit = unit.to_s.strip.gsub(" ","")
      tab_name = 'custom' + "_" + name.downcase.strip.gsub(' ','_').downcase
      if !Redborder::Module.registered_modules[:monitor][:tabs].key? tab_name.to_sym
        # Save the new custom tab in DB so we can add every time rb-webui restarts
        @custom_tab = CustomTab.new(
                              name: name,
                              category: "Custom",
                              percentage: (tab_unit == '%'),
                              units: tab_unit
                            )
        if @custom_tab.save
          Redborder::Module.registered_modules[:monitor][:tabs][@custom_tab.tab_name.to_sym] = {  :name=>@custom_tab.name,
                                                                                      :enabled=>@custom_tab.enabled,
                                                                                      :percentage=>@custom_tab.percentage,
                                                                                      :category=>@custom_tab.category,
                                                                                      :units=>@custom_tab.units,
                                                                                      :instance => @custom_tab.tab_name.to_s + "_per_instance"
                                                                                    }
          add_item_to_monitors_data_bag(@custom_tab.tab_name)
          @categories = get_monitor_tabs_from_columns
          return @custom_tab.tab_name
        else
          return nil
        end
      else
        return nil
      end
  end

  # In case that we delete a tab that is set as default or current active_tabs
  # we delete it from the session
  def delete_tab_from_session(dimension)
    # Clean the tab from the session
    @tab = dimension
    @tabs = @source::TABS.keys
    session[:monitor][:active_tabs].delete(@tab.to_sym)
    @new_tab = session[:monitor][:active_tabs].first.to_s
    controller = "monitor".to_sym
    action  = if session[controller][:default_action].present?
                session[controller][:default_action][:action].to_sym
              else
                :tops
              end

    columns = case action
              when :index
                :active_columns
              when :tops
                :active_tabs
              when :cardinality
                :active_cardinality_tabs
              end

    current_user.properties[controller][action] = session[controller][columns]
    current_user.save!
  end

  def delete_tabs_from_session(tabs)
    tabs.each do |tab_name|
      session[:monitor][:active_tabs].delete(tab_name.to_sym)
    end
    @new_tab = session[:monitor][:active_tabs].first.to_s

    action  = session[:monitor][:default_action].present? ? session[:monitor][:default_action][:action].to_sym : :tops
    columns = case action
              when :index
                :active_columns
              when :tops
                :active_tabs
              when :cardinality
                :active_cardinality_tabs
              end
    current_user.properties[:monitor][action] = session[:monitor][columns]
    current_user.save!
  end

  # Counts the number of Monitor Categories that has a bulkstats file
  # Checking the file name
  def is_there_any_bulkstats_monitor_categories?
    (MonitorCategory.where.not(bulkstats_schema_file_name: nil).count > 0) ? true : false
  end

  def clean_bulkstats_custom_tabs(tab_names_in_use)

    #First retrieve the custom_tabs name from the database
    bulkstats_custom_tabs = CustomTab.where(category: "Bulkstats").map{|ct| ct.tab_name}
    # Destroy all the Bulkstats Custom Tab
    CustomTab.delete_all("category = 'Bulkstats'")
    delete_tabs_from_session(bulkstats_custom_tabs)
    delete_items_from_monitors_data_bag(bulkstats_custom_tabs)
    bulkstats_custom_tabs.each do |tab_name|
        Redborder::Module.registered_modules[:monitor][:tabs].tap { |hs| hs.delete(tab_name.to_sym) }
    end
  end

  # Delete custom tabs if there is no monitors left that are using it
  def clean_custom_tabs
    @categories = get_monitor_tabs_from_columns

    # Construct actual names in use
    names = get_names_from_monitor_category

    # Clean the custom tabs that are not in use
    CustomTab.where.not(category: "Bulkstats").each do |custom_tab|
      if !names.include? custom_tab.tab_name
        if custom_tab.destroy
          Redborder::Module.registered_modules[:monitor][:tabs].tap { |hs| hs.delete(custom_tab.tab_name.to_sym) }
          delete_tab_from_session(custom_tab.tab_name)
          delete_item_from_monitors_data_bag(custom_tab.tab_name)
        end
      end
    end
    clean_bulkstats_custom_tabs(names) if !is_there_any_bulkstats_monitor_categories?
  end

  def remove_bulkstats_schema
    @monitor_controller.update(bulkstats_schema: nil)
  end


  def build_operation_redfish(category)
    #monitor_category_params[:redfish].gsub("%tab_name",category)
    return @redfish[monitor_category_params[:redfish]] rescue nil
  end

  def build_operation_ipmi(category)
    #monitor_category_params[:ipmi].gsub("%tab_name",category)
    return @ipmi[monitor_category_params[:ipmi]].gsub("%tab_name",category) rescue nil
  end

  def build_operation_bulkstats(category)
    "{\"name\":\"bulkstats_schema\",\"system\":\"bulkstats_schema\",\"unit\":\"\",\"name_split_suffix\":\"_per_instance\",\"split\":\";\",\"split_op\":\"mean\",\"instance_prefix\":\"bulkstats_schema-\"}"
  end

  def build_operation_snmp_op(category)
    params_oids = monitor_category_params[:snmp_vars].split(",").map(&:strip)
    oids = params_oids.select{ |oid| /^(?=.{0,64}$).*^([\.]*[0-9][\d]*([\.][0-9][\d]*)*)$|^(([a-z])|([A-Z]))((\:)|(\-)|(\.)|([a-z])|([A-Z])|(\d))*$/.match(oid) }
    return nil if (params_oids.count != oids.count) or params_oids.empty? # This means there was at least one incorrect OID so we shoudl continue

    operation = "["
    oids.each_with_index{ |oid, index|
        name = "#{category}_var#{index+1}"
        system = 'nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip '+ "#{oid}"+" | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}'  | sed 's/ [A-Za-z]*$//' | awk '{print $NF}' | grep -o '[[:digit:]]*' | paste -s -d';' -"
        operation = operation + "{\"name\": \"#{name}\",\"system\":\"#{system}\",\"unit\": \"\",\"name_split_suffix\":\"_per_instance\",\"split\":\";\",\"split_op\":\"mean\",\"instance_prefix\":\"#{name}-\",\"send\": 0},"
        add_item_to_monitors_data_bag("#{name}")
    }

    operation = operation + " {\"name\": \"#{@category}\",\"op\": \"#{monitor_category_params[:snmp_oper].strip.gsub("var","#{@category}_var")}\",\"send\": 1}]"
    return operation
  end

  def build_operation_snmp(category)
    # Clean OIDs first
    oids = monitor_category_params[:oid].split(",").map(&:strip).select{ |oid| /^(?=.{0,64}$).*^([\.]*[0-9][\d]*([\.][0-9][\d]*)*)$|^(([a-z])|([A-Z]))((\:)|(\-)|(\.)|([a-z])|([A-Z])|(\d))*$/.match(oid) }
    if oids.size > 1 # If the user introduced more than 1 oid in the same field separated by commas, we have to visualize them as instances
      system = create_system_with_instances(oids)
      operation = "{\"name\": \"#{category}\",\"system\":\"#{system}\",\"unit\": \"\",\"name_split_suffix\":\"_per_instance\",\"split\":\";\",\"split_op\":\"mean\",\"instance_prefix\":\"#{category}-\",\"send\": 1}"
    elsif oids.size > 0
      system = 'nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip '+ "#{oids.first}"+" | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}'  | sed 's/ [A-Za-z]*$//' | awk '{print $NF}' | grep -o '[[:digit:]]*' | paste -s -d';' -"
      operation = "{\"name\": \"#{category}\",\"system\":\"#{system}\",\"unit\": \"\",\"name_split_suffix\":\"_per_instance\",\"split\":\";\",\"split_op\":\"mean\",\"instance_prefix\":\"#{category}-\"}"
    end

    return operation
  end

  def build_operation(category, technology)
    return send("build_operation_#{technology}",category)
  end
  
  def build_category_redfish   
    JSON.parse(@redfish[monitor_category_params[:redfish]])["name"] rescue "cpu"
  end

  def build_category_bulkstats
    "bulkstats_schema"
  end

  def build_category(technology)
    category = monitor_category_params[:category]

    # Try to create a new custom tab name
    if monitor_category_params[:custom_tab_name].present?
      category = create_new_custom_tab(monitor_category_params[:custom_tab_name], monitor_category_params[:custom_tab_unit]);
    end

    category = send("build_category_#{technology}") if self.respond_to?("build_category_#{technology}".to_sym)
    return category
  end

  
  # Creates the system command like this ==>  system = { command1: command2; command3 } | paste -d';' -s' so this way we dont have tu use double quotes, which causes problems.
  # This function concatenates the outputs of those commands so the final output looks like "6;6;6" (in case of introducing 3 OIDs) so then "split: ';'" from @operation splits those Values
  # and creates them as instances of the monitor
  def create_system_with_instances (oids)
    system = '{ '
    oids.each { |oid|
      if !oid.empty?
        system += ' nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip ' + "#{oid}" + ' | awk \'/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}\'  | sed \'s/ [A-Za-z]*$//\' | awk \'{print $NF}\';'
      end
    }
    system += " } | paste -d';' -s"
    return system
  end


  def create_monitor_category
    @technology = monitor_category_params[:technology]
    @category = build_category(@technology)
    @operation = build_operation(@category, @technology)

    @monitor_category = MonitorCategory.new(
                          name: monitor_category_params[:name],
                          description: monitor_category_params[:description],
                          technology: @technology,
                          category: @category,
                          main_category: 'Custom',
                          impact: 3,
                          operation: operation_sanitation(@operation),
                          sensor_types: [ Sensor::DEVICE ],
                          default: monitor_category_params[:default],
                          bulkstats_schema: monitor_category_params[:bulkstats_schema].present? ? monitor_category_params[:bulkstats_schema] : nil
                        )
    respond_to do |format|
      if  @monitor_category.save
          format.html { redirect_to monitor_categories_list_path, notice: "#{I18n.t('controllers.monitor_created')}" }
          format.js { flash[:notice] = "#{I18n.t('controllers.monitor_created')}" }
          format.json { head :no_content }
      else
          format.html { redirect_to monitor_categories_list_path(@monitor_category), alert: @monitor_category.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
          format.js
          format.json { render json: @monitor_category.errors, status: :unprocessable_entity }
      end
    end
  end

  # Creating a monitor category and adding it to the parent sensor at the same time.
  def create_and_add_monitor_category
    @technology = monitor_category_params[:technology]
    @category = build_category(@technology)
    @operation = build_operation(@category, @technology)

    @sensor = Sensor.find(params[:sensor_id])

    @monitor_category = MonitorCategory.new(
                          name: monitor_category_params[:name],
                          description: monitor_category_params[:description],
                          technology: @technology,
                          category: @category,
                          main_category: 'Custom',
                          impact: 3,
                          operation: operation_sanitation(@operation),
                          sensor_types: [ Sensor::DEVICE ],
                          default: monitor_category_params[:default],
                          bulkstats_schema: monitor_category_params[:bulkstats_schema].present? ? monitor_category_params[:bulkstats_schema] : nil
                        )

     respond_to do |format|
        if  @monitor_category.save
            @sensor.monitor_categories << @monitor_category
            format.html { redirect_to "/sensors/#{@sensor.id}/monitor_categories", notice: "#{t '.created_added'}" }
            format.js { flash[:notice] = "#{t '.created_added'}" }
            format.json { head :no_content }
        else
           format.html { redirect_to "/sensors/#{@sensor.id}/monitor_categories", alert: @monitor_category.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
           format.js
           format.json { render json: @monitor_category.errors, status: :unprocessable_entity }
        end
     end
  end

  def update_monitor_category
    @technology = monitor_category_params[:technology]
    @category = build_category(@technolgoy)
    @operation = build_operation(@category, @technology)

    delete_relation_sensor_monitor_category = false

    @sensors = Sensor.where(:type => Sensor::DEVICE)
    @sensors.each do |sensor|
      sensors_with_same_monitor_category = sensor.monitor_categories.where(:category => @category).select(:id)
      delete_relation_sensor_monitor_category = true if sensors_with_same_monitor_category.count > 0 and
                                                        !sensors_with_same_monitor_category.first.nil? and
                                                          sensors_with_same_monitor_category.first.id.to_s != params[:id].to_s
    end

    @monitor_category = MonitorCategory.update(params[:id],
                        name: monitor_category_params[:name],
                        description: monitor_category_params[:description],
                        technology: @technology,
                        category: @category,
                        main_category: 'Custom',
                        impact: 3,
                        operation: operation_sanitation(@operation),
                        sensor_types: [ Sensor::DEVICE ],
                        default: monitor_category_params[:default],
                        bulkstats_schema: monitor_category_params[:bulkstats_schema].present? ? monitor_category_params[:bulkstats_schema] : nil
                      )

    if delete_relation_sensor_monitor_category
      @sensors.each do |sensor|
        sensor.monitor_categories.delete(MonitorCategory.find(@monitor_category.id))
      end
    end

    respond_to do |format|
    if  @monitor_category.save
        format.html { redirect_to monitor_categories_list_path, notice: "#{I18n.t('controllers.monitor_updated')}" }
        format.js { flash[:notice] = "#{I18n.t('controllers.monitor_updated')}" }
        format.json { head :no_content }
    else
        format.html { redirect_to monitor_categories_list_path(@monitor_category), alert: @monitor_category.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
        format.js
        format.json { render json: @monitor_category.errors, status: :unprocessable_entity }
      end
    end
  end

  def set_monitor_category
    @monitor_category = MonitorCategory.find(params[:id])
  end

  def delete_monitor_category
    DelayedJob.force_bulkstats_update if @monitor_category.technology == "bulkstats"
    @monitor_category.destroy
  end

  # Return a json with all OIDs that match the param :term
  def get_oids
    term = params[:term] || ''
    oids = []
    MonitorCategory.all.each do |monitor|
      operations = monitor.operation.is_a?(Hash) ? [monitor.operation] :
                                                    monitor.operation
      oids = oids + operations.map { |o| o['oid'] }
    end

    respond_to do |format|
      format.html
      format.json  { render :json => oids.uniq.select{|o| o =~ /term.to_s/i } }
    end
  end

  def get_first_oid_from_operation(operation)
      operations = operation.is_a?(Hash) ? [operation] : operation
      oids = operations.map { |o| o['oid'] }
      oids.first
  end

  # Get all the Oids but not the last one (last one is operation)
  def get_oids_from_operation(operation)
    operations = operation.is_a?(Hash) ? [operation] : operation
    systems = operations.map { |o| o['system'] }
    oids = []
    systems.each do |s|
      if !s.nil?
        system_map = s.split(" ")
        oid_position = 9
        oid = (system_map.count >= oid_position) ? system_map[oid_position] : ""
        oids.push( oid )
      end
    end
    #oids.pop
    return oids
  end

  def get_oid_from_bulkwalk_operation(operation, index)
    operations = operation.is_a?(Hash) ? [operation] : operation
    if operation.is_a?(Hash)
      if operation["system"][0] == "{" #This is the case of multiple oids in the same field.
        oids = operation["system"].to_s.scan(/\s(?:\.\d+)+ | (?:(?:\w+\-)+\w+::\w+\W\d) /) #get all oids that appears in the command. First pattern is for OIDs, second for MIBs                                                                          # like .1.3.6.4.6.2012.4.5.7. Second is for MIBS: UCD-SNMP-MIB::laLoad.1
        return oids.map(&:strip).join(",")
      end
    end
    if index.to_i == 0 # get oid from system command. Normal snmp.
      system = operations.map { |o| o['system']} # TODO: check if empty
      system_map = system[0].split(" ")
      oid_position = 9
      return (system_map.count >= oid_position) ? system_map[oid_position] : ""
    else # index.to_i == 1  get snmp operation. ie: var1*var2
      oper = operations.map { |o| o['op']}
      return oper.last
    end
  end

  def get_redfish(monitor_category)
      operations = monitor_category.operation.is_a?(Hash) ? [operation] : operation
  end

  def get_ipmi(monitor_category)
      operations = operation.is_a?(Hash) ? [operation] : operation
      oids = operations.map { |o| o['oid'] }
      oids.first
  end

  def edit_monitor_category
    @monitor_category.snmp_vars = []
    if @monitor_category.technology == "snmp"
      @monitor_category.oid = get_oid_from_bulkwalk_operation(@monitor_category.operation, 0)
    end
    if @monitor_category.technology == "snmp_op"
      @monitor_category.snmp_vars = get_oids_from_operation(@monitor_category.operation)
      @monitor_category.snmp_oper = get_oid_from_bulkwalk_operation(@monitor_category.operation,1).gsub(/\w+_/, "")
    end
    @monitor_category.redfish = (@monitor_category.technology == "redfish") ? @redfish.key(@monitor_category.operation.to_json) : @redfish.keys.first
    @monitor_category.ipmi = (@monitor_category.technology == "ipmi") ? @ipmi.key(@monitor_category.operation.to_json.gsub(@monitor_category.category.to_s, "%tab_name")) : @ipmi.keys.first
    @monitor_category.operation = @monitor_category.operation.to_json
    render layout: false
  end

  def refresh_monitor_categories
    Sensor.all.each do |sensor|
      sensor.monitors_to_chef
    end
  end

  # For updating a monitor that already exists in our database
  def update_import_monitor_category (monitor, operation)
      @monitor_category = MonitorCategory.update(monitor['id'],
                          name: monitor['name'],
                          description: monitor['description'],
                          technology: monitor['technology'],
                          category: monitor['category'],
                          main_category: monitor['main_category'],
                          impact: 3,
                          operation: operation,
                          sensor_types: [ Sensor::DEVICE ],
                          default: monitor['default']
                        )

      respond_to do |format|
      if  !@error and @monitor_category.save
          format.html { redirect_to monitor_categories_list_path, notice: "#{I18n.t('controllers.monitor_updated2')}" }
          format.js { flash[:notice] = "#{I18n.t('controllers.monitor_updated2')}" }
          format.json { head :no_content }
      else
          format.html { redirect_to monitor_categories_list_path(@monitor_category), alert: @monitor_category.errors.full_messages.map { |msg| "<li>#{msg}</li>" }.join }
          format.js
          format.json { render json: @monitor_category.errors, status: :unprocessable_entity }
        end
      end
  end

  # Create the monitors
  def create_import_monitor_category (monitor)
      @monitor_category = MonitorCategory.new(
                          name: monitor['name'],
                          description: monitor['description'],
                          technology: monitor['technology'],
                          category: monitor['category'],
                          main_category: monitor['main_category'],
                          impact: 3,
                          operation: monitor['operation'],
                          sensor_types: [ Sensor::DEVICE ],
                          default: monitor['default']
                        )
      @monitor_category.save
  end


  # Returns true if a monitor we want to import is already created
  def search_if_monitor_repeated (monitor, all_monitors)
    all_monitors.each do |all|
      if (all['name'] == monitor['name'])
        return true
      else
        return false
      end
    end
  end

  def import_monitor
    if request.get?
      render layout: false
    elsif request.post?
      #create where the item is temporarly generated
      import_filename = params['imported_file']['tgz_file'].tempfile.path
      new_filename = params['imported_file']['tgz_file'].original_filename.gsub('Export_', '').gsub(' ', '_').gsub('.tar.gz', '').gsub(/_\d+/, '')
      new_path_dir = "/tmp/" + new_filename
      #checking if the Directory is created before.
      if Dir.exist?(new_path_dir)
        FileUtils.rm_rf(new_path_dir)
      end
      FileUtils.mkdir_p(new_path_dir)
      #unpack the tar file using a primitive system
      `tar xvzf #{import_filename} -C #{new_path_dir}`

      file = File.read(new_path_dir + '/jsonFiles/monitors.json')
      data_hash = JSON.parse(file)
      data_hash.each do |monitor|
        create_import_monitor_category(monitor)
      end
      redirect_to monitor_categories_list_path, notice: "#{I18n.t('controllers.monitors_imported')}"
    end
  end

  def export_monitor
    # Create structure
    dir = "/tmp/#{Time.now.to_i}"
    Dir.mkdir dir unless File.exists?(dir) #if the directory already exists
    FileUtils.mkdir_p "#{dir}/jsonFiles"

    File.open("#{dir}/jsonFiles/monitors.json", 'w') do |f|
      f.write(MonitorCategory.where("main_category = 'Custom'").to_json) #save monitors created by the user.
    end

    # Compress directory
    tar_file = "#{Rails.root}/tmp/Export_#{Time.now.to_i}_#{Random.rand(1000)}.tar.gz"
    system('tar', 'cvzf', tar_file, '-C', dir, 'jsonFiles')

    send_file tar_file

  end

  private

  def monitor_category_params
    params.require(:monitor_category)
      .permit(:name, :description, :technology, :category, :main_category,
              :impact, :operation, :sensor_types, :default,:oid, :snmp_vars, :snmp_oper,
              :custom_tab_name, :custom_tab_unit, :redfish, :ipmi, :bulkstats_schema, :device_sensor)
  end

  def params_aggregation
    aggregations_allowed = SensorMonitor::AGGREGATIONS.values.map { |x| x[:value].to_sym }
    aggregations_allowed.include?(params[:aggregation].to_sym) ? params[:aggregation] : :monitor
  end

  #
  # Get the namespace based on a task ID
  #
  def namespace_task(task)
    datasource = task.payload['dataSource']
    m = /\Arb_(?<topic>([a-z])+(_[a-z]+){0,1})(?<namespace>_[a-zA-Z0-9-]*)*\z/
        .match(datasource)
    return 'default' if m.nil?
    m[:namespace] ? m[:namespace][1..m[:namespace].size] : 'default'
  end

  def check_size_error(elements, current, name, options = {})
    current ||= {}
    threshold = options[:threshold] ? options[:threshold] : 75
    type = options[:type] ? options[:type] : :warning
    elements.each do |el|
      next if el.used_percent < threshold
      current[el.send(name)] = type
    end
    current
  end

  def check_empty_error(elements, current, name, type)
    current ||= {}
    elements.each do |el|
      next if el.max_size != 0
      current[el.send(name)] = type
    end
    current
  end

  def check_capacity_error(size, current)
    current ||= {}
    current[:capacity] = :warning if size > 80
    current
  end

  def get_active_tabs(type)
    @source::TABS.each{ |k,v| (session[:monitor][type] ||= []) << k if v[:enabled] } if session[:monitor][type].blank?
  end

  def hardware_info
    @info_disk = {}
    @info_memory = {}
    @info_cpu = {}

    @managers.each do |manager|
      chef_node = Chef::Node.load(manager.name)

      @info_disk[manager.name.to_sym] = chef_node['filesystem']['by_mountpoint']['/']
      @info_memory[manager.name.to_sym] = chef_node['memory']
      @info_cpu[manager.name.to_sym] = chef_node['cpu']
    end
  end

  #
  # Set the filters. Change current filters if params[:search] is defined.
  #
  def set_source
    @source = Redborder::DruidSource.get(source: 'monitor')
    @filter = Redborder::Filter.new(@source, load_filters)
  end

  def set_saved_monitor_filters
    @filters = SavedMonitorFilter.where(user_id: current_user.id)
    @filters_public = SavedMonitorFilter.where.not(user_id: current_user.id)
                      .select(&:public)
  end

  #
  # Check administrative privileges. If the request comes from a widget,
  # render the template of no privileges
  #
  def require_super_administrative_privileges_cluster
    return true if user_signed_in? && current_user.super_admin?
    # Check if request comes from a widget
    if params['w'] == 'true'
      render template: 'managers/no_privileges', layout: false, status: 403
    else
      no_enough_privileges
    end
  end

  #
  # Check root reader privileges. If the request comes from a widget,
  # render the template of no privileges
  #
  def require_super_reader_privileges_cluster
    return true if user_signed_in? && current_user.super_reader?
    # Check if request comes from a widget
    if params['w'] == 'true'
      render template: 'managers/no_privileges', layout: false, status: 403
    else
      no_enough_privileges
    end
  end
end
