###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

require 'pathname'

class MonitorCategory < ApplicationRecord
  include StorageFile

  storage_file :bulkstats_schema, path: '/rb-webui/:class/:id/:filename'

  attr_accessor :oid, :snmp_vars, :snmp_oper, :custom_tab_name, :custom_tab_unit, :redfish, :ipmi, :device_sensor

  serialize :operation, JSON
  serialize :sensor_types, Array

  has_and_belongs_to_many :sensors

  # ================================================================
  # Validations
  # ================================================================
  validates :operation, presence: true
  validates_attachment :bulkstats_schema, content_type: {
    content_type: %w(text/plain)
  }, size: { in: 0..10.megabytes }


  def impact_percent
    impact * 100 / 5
  end

  def bulkstats_schema_uri
    return nil if bulkstats_schema.path.nil?
    StorageFile.get_uri bulkstats_schema.path
  end

  def bulkstats_schema_path
    if StorageFile.s3_enabled?
      "s3://redborder" + bulkstats_schema.path
    else
      bulkstats_schema.path
    end
  end

end
