#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class WidgetType::Monitor < WidgetType::Base
  graphs [:multiserie, :serie, :value, :performance_index, :ball, :box]

  widget_fields [:range, :elements, :category, :aggregation, :skip_empty]

  widget_after_initialize do
    self.elements ||= 5
    # Walkaround to set the product type as monitor
    self.product_type = 'monitor'
    #self.aggregation = :monitors # Commented when we added the "events" aggregation..
    self.skip_empty ||= false
  end

  # Validate the number of elements
  widget_validate :elements_num do
    self.elements = 2 if elements.to_i < 2
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def list(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    serie = source.where(
      user: user,
      query_type: :group_by,
      monitor_tab: @widget.category,
      granularity: :all,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      group_by: source::COLUMNS.keys,
      timezone: @widget.timezone,
      aggregation: @widget.aggregation,
      search: @widget.fetch_filters
    )

    { list: serie }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeseries(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)
    elements = list(user, options)
    # Init timeseries
    timeseries = {}

    elements[:list].each do |e1|
      serie = source.where(
        user: user,
        query_type: :timeseries,
        range: @widget.range,
        monitor_tab: @widget.category,
        monitor_sensor: e1.sensor_name,
        start_time: ovrr_start_time,
        end_time: ovrr_end_time,
        granularity: ovrr_granularity,
        aggregation: @widget.aggregation,
        timezone: @widget.timezone,
        search: @widget.fetch_filters
      ).map do |e2|
        { event: { events: (e2.monitors) }, timestamp: e2.timestamp.utc.to_s }
      end

      timeseries.merge!(e1.sensor_name => serie)
    end

    # [ TODO ] Angel. Add filters
    {
      timeseries: timeseries
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # An array or value with the data.
  #
  def timeserie(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    ovrr_granularity = options[:granularity] || @widget.granularity

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)
    elements = list(user, options)
    # Init timeseries
    timeseries = {}

    total = source.timeserie(
        user: user,
        range: @widget.range,
        monitor_tab: @widget.category,
        start_time: ovrr_start_time,
        end_time: ovrr_end_time,
        granularity: :all,
        aggregation: @widget.aggregation,
        timezone: @widget.timezone,
        search: @widget.fetch_filters
      )

    # [ TODO ] Add filters
    {
      timeserie: timeseries
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def total(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    total = source.timeserie(
        user: user,
        range: @widget.range,
        monitor_tab: @widget.category,
        start_time: ovrr_start_time,
        end_time: ovrr_end_time,
        granularity: :all,
        aggregation: @widget.aggregation,
        timezone: @widget.timezone,
        search: @widget.fetch_filters
      )

    {
      total: total
    }
  end

  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #
  # == Returns:
  # A blank value because monitor hasn't got this property.
  # This methos is defined in order to prevent multiseries errors.
  #
  def total_timeserie(user = @widget.parent.owner, options = {})
    {
      total_timeserie: []
    }
  end
  
  #
  # Preload the data to show the widget. This method is used to generate
  # reports because WickedPDF can't call to the server when render.
  #
  # == Parameters:
  # user::
  #   The user who call to render the report. This is used to set permissions
  # options::
  #   A hash with multiple options to force on DruidQueries.
  #     - start_time
  #     - end_time
  #     - granularity
  #
  # == Returns:
  # A value with the total data of a topic in a range
  #
  def performance(user = @widget.parent.owner, options = {})
    # Override the report block time if it's defined in the option
    ovrr_start_time = options[:start_time] || start_time
    ovrr_end_time = options[:end_time] || end_time
    # There is no option to change the granularity in the widget. 
    # However, it's set like this in case the option is added in the future.
    ovrr_granularity = options[:granularity] || @widget.granularity || "minute"

    intervals = {
      low: 0,
      medium: 0,
      high: 0,
      total: 0
    }

    # Use DruidSource to set the dataSource based on the product_type
    source = Redborder::DruidSource.get(source: @widget.product_type)

    data = source.timeserie(
      user: user,
      monitor_tab: @widget.category,
      start_time: ovrr_start_time,
      end_time: ovrr_end_time,
      granularity: ovrr_granularity,
      aggregation: @widget.aggregation,
      timezone: @widget.timezone,
      search: @widget.fetch_filters
    )

    intervals[:total] = data.length;
    data.each do |d|
      if d[:event][:events] < @widget.property['low_limit'].to_f
        intervals[:low] += 1
      elsif d[:event][:events] < @widget.property['medium_limit'].to_f
        intervals[:medium] += 1
      else
        intervals[:high] += 1
      end
    end
    intervals
  end
end