###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

Rails.logger.info '[RB] Loading Monitor module'

Redborder::Module.register :monitor do

  name 'Monitor'
  path 'monitor'
  session_id 'monitor'
  data_source 'SensorMonitor'
  selectable_widget true
  cep_topic :rb_monitor

  widgets_section :infrastructure, [{
    id: :monitor,
    name: 'Monitor',
    icon: 'fa-bar-chart'
  }]
  columns(
    monitor: {
      columns: {
        monitor: 'Monitor'
      },
      shared: false,
      hide_global: true
    },
    bulkstats: {
      columns: {
        bulkstats: 'Bulkstats'
      },
      shared: false,
      hide_global: true
    },
    sensor_name: {
      columns: {
        sensor_name: 'tr.monitor2.sensor_name'
      },
      shared: false
    },
    service_provider: {
      columns: { service_provider: 'tr.monitor2.sp' }, hide_global: true
    },
    service_provider_uuid: {
      columns: { service_provider_uuid: 'tr.monitor2.sp_uuid' }, hide_global: true
    },
    namespace: {
      columns: { namespace: 'tr.monitor2.namespace' }, hide_global: true
    },
    namespace_uuid: {
      columns: { namespace_uuid: 'tr.monitor2.namespace_uuid' }, hide_global: true
    },
    deployment:  {
      columns: { deployment: 'tr.monitor2.deployment' }, hide_global: true
    },
    deployment_uuid:  {
      columns: { deployment_uuid: 'tr.monitor2.deployment_uuid' }, hide_global: true
    },
    market:  {
      columns: { market: 'tr.monitor2.market' }, hide_global: true
    },
    market_uuid:  {
      columns: { market_uuid: 'tr.monitor2.market_uuid' }, hide_global: true
    },
    organization:  {
      columns: { organization: 'tr.monitor2.org' }, hide_global: true
    },
    organization_uuid:  {
      columns: { organization_uuid: 'tr.monitor2.org_uuid' }, hide_global: true
    },
    campus:  {
      columns: { campus: 'Campus' }, hide_global: true
    },
    campus_uuid:  {
      columns: { campus_uuid: 'tr.monitor2.campus_uuid' }, hide_global: true
    },
    building:  {
      columns: { building: 'tr.monitor2.building' }, hide_global: true
    },
    building_uuid:  {
      columns: { building_uuid: 'tr.monitor2.building_uuid' }, hide_global: true
    },
    floor:  {
      columns: { floor: 'tr.monitor2.floor' }, hide_global: true
    },
    floor_uuid:  {
      columns: { floor_uuid: 'tr.monitor2.floor_uuid' }, hide_global: true
    },
    group_name: {
      columns: {
        group_name: 'tr.monitor2.group'
      },
      shared: false,
      hide_global: true
    }
  )
  customizable_tabs true

  tabs(
    cpu: {
      name: 'CPU',
      enabled: true,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :cpu_per_instance
    },
    avio: {
      name: 'Avio',
      enabled: false,
      percentage: false,
      category: 'System',
      units: 'ms',
      instance: :avio_per_instance
    },
    disk: {
      name: 'tr.monitor2.disk',
      enabled: false,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :disk_per_instance
    },
    disk_load: {
      name: 'tr.monitor2.disk_load',
      enabled: false,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :disk_load_per_instance
    },
    load_1: {
      name: 'tr.monitor2.load',
      enabled: true,
      percentage: false,
      category: 'System',
      units: '',
      instance: :load_1_per_instance
    },
    memory: {
      name: 'tr.monitor2.memory_used',
      enabled: true,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :memory_per_instance
    },
    memory_buffer: {
      name: 'tr.monitor2.memory_buffered',
      enabled: false,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :memory_buffer_per_instance
    },
    memory_cache: {
      name: 'tr.monitor2.memory_cached',
      enabled: false,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :memory_cache_instance
    },
    swap: {
      name: 'tr.monitor2.swap',
      enabled: false,
      percentage: true,
      category: 'System',
      units: '%',
      instance: :swap_per_instance
    },
    avg_bytes_per_wire_packet: {
      name: 'AvgBytes/Pkt',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'bytes',
      instance: :avg_bytes_per_wire_packet_per_instance
    },
    wire_bits_per_sec_realtime: {
      name: 'Bits/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'bit/s',
      instance: :wire_bits_per_sec_realtime_per_instance
    },
    pkt_drop_percent: {
      name: 'Drops',
      enabled: false,
      percentage: true,
      category: 'IDS/IPS',
      units: '%',
      instance: :pkt_drop_percent_per_instance
    },
    alerts_per_second: {
      name: 'tr.monitor2.events',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: '',
      instance: :alerts_per_second_per_instance
    },
    new_sessions_per_second: {
      name: 'tr.monitor2.new_sessions',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: '',
      instance: :new_sessions_per_second_per_instance
    },
    patmatch_percent: {
      name: 'tr.monitor2.pattern',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: '%',
      instance: :patmatch_percent_per_instance
    },
    packets_wire_per_sec_realtime: {
      name: 'Pkts/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'pkts/s',
      instance: :packets_wire_per_sec_realtime_per_instance
    },
    total_sessions: {
      name: 'tr.monitor2.sessions',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions',
      instance: :total_sessions_per_instance
    },
    snort_cpu: {
      name: 'Snort CPU',
      enabled: false,
      percentage: true,
      category: 'IDS/IPS',
      units: '%',
      instance: :snort_cpu_per_instance
    },
    stream_flushes_per_second: {
      name: 'StreamFlushes/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: '',
      instance: :stream_flushes_per_second_per_instance
    },
    stream5_mem_in_use: {
      name: 'Stream5 Memory',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'bytes',
      instance: :stream5_mem_in_use_per_instance
    },
    ratio_syn_synacks: {
      name: 'Syn/Synacks',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: '',
      instance: :ratio_syn_synacks_per_instance
    },
    syns_per_second: {
      name: 'Syns/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sync/s',
      instance: :syns_per_second_per_instance
    },
    synacks_per_second: {
      name: 'Syn-Acks/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'syn-acks/s',
      instance: :synacks_per_second_per_instance
    },
    deleted_sessions_per_second: {
      name: 'tr.monitor2.cached_sessions',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions/s',
      instance: :deleted_sessions_per_second_per_instance
    },
    frag_creates_per_second: {
      name: 'Frag Creates()s/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'frag/s',
      instance: :frag_creates_per_second_per_instance
    },
    frag_inserts_per_second: {
      name: 'Frag Inserts()s/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'frag/s',
      instance: :frag_inserts_per_second_per_instance
    },
    frag_flushes_per_second: {
      name: 'Frag Flushes/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'frag/s',
      instance: :frag_flushes_per_second_per_instance
    },
    current_frags: {
      name: 'tr.monitor2.cached_frags',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'frags',
      instance: :current_frags_per_instance
    },
    max_frags: {
      name: 'tr.monitor2.max_cached',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'frags',
      instance: :max_frags_per_instance
    },
    new_udp_sessions_per_second: {
      name: 'New Cached UDP Ssns/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions/s',
      instance: :new_udp_sessions_per_second_per_instance
    },
    deleted_udp_sessions_per_second: {
      name: 'Cached UDP Ssns Del/s',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions/s',
      instance: :deleted_udp_sessions_per_second_per_instance
    },
    total_udp_sessions: {
      name: 'Cached UDP Ssns',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions',
      instance: :total_udp_sessions_per_instance
    },
    max_udp_sessions: {
      name: 'Max Cached UDP Ssns',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions',
      instance: :max_udp_sessions_per_instance
    },
    tcp_sessions_closed_per_second: {
      name: 'tr.monitor2.closed',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions/s',
      instance: :tcp_sessions_closed_per_second_per_instance
    },
    tcp_sessions_timedout_per_second: {
      name: 'tr.monitor2.timedout_sessions',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'sessions/s',
      instance: :tcp_sessions_timedout_per_second_per_instance
    },
    reflect_fwd_error: {
      name: 'Reflect FWD error',
      enabled: false,
      percentage: false,
      category: 'IDS/IPS',
      units: 'pkts',
      instance: :reflect_fwd_error_per_interface
    }
  )

  DittoCode::Exec.if 'ENTERPRISE' do
    tabs(
      channel_load_2_4: {
        name: 'tr.monitor2.channel_load1',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :channel_load_2_4_per_instance
      },
      channel_load_5: {
        name: 'tr.monitor2.channel_load2',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :channel_load_5_per_instance
      },
      dhcp_latency: {
        name: 'tr.monitor2.dhcp',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :dhcp_latency_per_instance
      },
      http_latency: {
        name: 'tr.monitor2.http',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :http_latency_per_instance
      },
      latency: {
        name: 'tr.monitor2.ping',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :latency_per_instance
      },
      pkts_lost: {
        name: 'tr.monitor2.lost',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :pkts_lost_per_instance
      },
      pkts_percent_rcv: {
        name: 'tr.monitor2.received',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :pkts_percent_rcv_per_instance
      },
      router_latency: {
        name: 'tr.monitor2.router',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :router_latency_per_instance
      },
      router_pkts_lost: {
        name: 'tr.monitor2.router_lost',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :router_pkts_lost_per_instance
      },
      router_pkts_percent_rcv: {
        name: 'tr.monitor2.router_received',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :router_pkts_percent_rcv_per_instance
      },
      dns_latency: {
        name: 'tr.monitor2.dns_latency',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :dns_latency_per_instance
      },
      dns_pkts_lost: {
        name: 'tr.monitor2.dns_lost',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :dns_pkts_lost_per_instance
      },
      dns_pkts_percent_rcv: {
        name: 'tr.monitor2.dns_received',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :dns_pkts_percent_rcv_per_instance
      },
      dns_query_latency: {
        name: 'tr.monitor2.dns_query_latency',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'ms',
        instance: :dns_query_latency_per_instance
      },
      dns_query_lost: {
        name: 'tr.monitor2.dns_query_lost',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '%',
        instance: :dns_query_lost_per_instance
      },
      nat: {
        name: 'NAT',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '',
        instance: :nat_per_instance
      },
      session: {
        name: 'tr.monitor2.sessions',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: '',
        instance: :session_per_instance
      },
      wireless_stations: {
        name: 'tr.monitor2.wireless',
        enabled: false,
        percentage: false,
        category: 'Network',
        units: 'aps',
        instance: :wireless_stations_per_instance
      },
      memory_bytes: {
        name: 'Memory Bytes',
        enabled: false,
        percentage: false,
        category: 'Custom',
        admin: true,
        units: 'bytes',
        instance: :memory_bytes_per_instance
      },
      memory_available: {
        name: 'Memory Available',
        enabled: false,
        percentage: false,
        category: 'Custom',
        admin: true,
        units: '%',
        instance: :memory_available_per_instance
      },
      disk_bytes: {
        name: 'Disk Bytes',
        enabled: false,
        percentage: false,
        category: 'Custom',
        admin: true,
        units: 'bytes',
        instance: :disk_bytes_per_instance
      },
      hot_tier_capacity: {
        name: 'Hot_tier',
        enabled: false,
        percentage: true,
        category: 'Custom',
        admin: true,
        units: '%',
        instance: :hot_tier_capacity
      },
      default_tier_capacity: {
        name: 'Default_tier',
        enabled: false,
        percentage: true,
        category: 'Custom',
        admin: true,
        units: '%',
        instance: :default_tier_capacity
      },
      kafka_disk_cached_pages: {
        name: 'Kafka cache pages',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        admin: true,
        units: '%',
        instance: :kafka_disk_cached_pages_per_instance
      },
      cache_hits: {
        name: 'Cache Hits',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        admin: true,
        units: '%',
        instance: :cache_hits_per_instance
      },
      enrichmentstreamtask_messages: {
        name: 'tr.monitor2.messages_all',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        units: 'msgs',
        admin: true,
        instance: :enrichmentstreamtask_messages_per_instance
      },
      flowprocessor_messages: {
        name: 'tr.monitor2.messages_flow',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        units: 'msgs',
        admin: true,
        instance: :flowprocessor_messages_per_instance
      },
      locationprocessor_messages: {
        name: 'tr.monitor2.messages_loc',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        units: 'msgs',
        admin: true,
        instance: :locationprocessor_messages_per_instance
      },
      nmspprocessor_messages: {
        name: 'tr.monitor2.messages_nmsp',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        units: 'msgs',
        admin: true,
        instance: :nmspprocessor_messages_per_instance
      },
      monitorprocessor_messages: {
        name: 'tr.monitor2.messages_monitor',
        enabled: false,
        percentage: false,
        category: 'Cluster',
        units: 'msgs',
        admin: true,
        instance: :monitorprocessor_messages_per_instance
      },
      disk_aggregated: {
        name: 'tr.monitor2.disk_aggregated',
        enabled: false,
        percentage: true,
        category: 'System',
        units: '%',
        instance: :disk_aggregated_per_instance
      },
      disk_aggregated_load: {
        name: 'tr.monitor2.disk_aggregated_load',
        enabled: false,
        percentage: true,
        category: 'System',
        units: '%',
        instance: :disk_aggregated_load_per_instance
      },
      disk_raw: {
        name: 'tr.monitor2.disk_raw',
        enabled: false,
        percentage: true,
        category: 'System',
        units: '%',
        instance: :disk_raw_per_instance
      },
      disk_raw_load: {
        name: 'tr.monitor2.disk_raw_load',
        enabled: false,
        percentage: true,
        category: 'System',
        units: '%',
        instance: :disk_raw_load_per_instance
      },
      waiting_tasks: {
          name: 'tr.monitor2.waiting',
          enabled: false,
          percentage: false,
          category: 'System',
          units: 'tasks',
          instance: :waiting_tasks_per_instance
      },
      pending_tasks: {
          name: 'tr.monitor2.pending',
          enabled: false,
          percentage: false,
          category: 'System',
          units: 'tasks',
          instance: :pending_tasks_per_instance
      },
      running_tasks: {
          name: 'tr.monitor2.running',
          enabled: false,
          percentage: false,
          category: 'System',
          units: 'tasks',
          instance: :running_tasks_per_instance
      },
      system_temp: {
        name: 'tr.monitor2.system',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: 'degrees',
        instance: :system_temp_per_instance
      },
      peripheral_temp: {
        name: 'tr.monitor2.peripheral',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: 'degrees',
        instance: :peripheral_temp_per_instance
      },
      pch_temp: {
        name: 'PCH Temp',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: 'degrees',
        instance: :pch_temp_per_instance
      },
      fan: {
        name: 'tr.monitor2.fan',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: 'rpm',
        instance: :fan_per_instance
      },
      memory_zookeeper: {
        name: 'tr.monitor2.zookeeper',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_zookeeper_per_instance
      },
      memory_webui: {
        name: 'tr.monitor2.webui',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_webui_per_instance
      },
      # memory_rb_enrich: {
      #  name: 'tr.monitor2.rb_enrich',
      #  enabled: false,
      #  percentage: true,
      #  category: 'Service - Memory',
      #  units: '%',
      #  admin: true,
      #  instance: :memory_rb_enrich_per_instance
      #},
      memory_redborder_postgresql: {
        name: 'tr.monitor2.redborder_postgresql',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_redborder_postgresql_per_instance
      },
      memory_f2k: {
        name: 'tr.monitor2.f2k',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_f2k_per_instance
      },
      memory_redborder_nmsp: {
        name: 'tr.monitor2.redborder_nmsp',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_redborder_nmsp_per_instance
      },
      memory_n2klocd: {
        name: 'tr.monitor2.n2klocd',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_n2klocd_per_instance
      },
      memory_http2k: {
        name: 'tr.monitor2.http2k',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_http2k_per_instance
      },
      memory_kafka: {
        name: 'tr.monitor2.kafka',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_kafka_per_instance
      },
      memory_druid_coordinator: {
        name: 'tr.monitor2.druid_coordinator',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_coordinator_per_instance
      },
      memory_druid_middleManager: {
        name: 'tr.monitor2.druid_middleManager',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_middleManager_per_instance
      },
      memory_druid_overlord: {
        name: 'tr.monitor2.druid_overlord',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_overlord_per_instance
      },
      memory_druid_broker: {
        name: 'tr.monitor2.druid_broker',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_broker_per_instance
      },
      memory_druid_realtime: {
        name: 'tr.monitor2.druid_realtime',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_realtime_per_instance
      },
      memory_druid_historical: {
        name: 'tr.monitor2.druid_historical',
        enabled: false,
        percentage: true,
        category: 'Service - Memory',
        units: '%',
        admin: true,
        instance: :memory_druid_historical_per_instance
      },
      redfish_thermal_temperature_cpu_average: {
        name: 'tr.monitor2.cpu_temp',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_cpu_average_per_instance
      },
      redfish_thermal_temperature_cpu_health: {
        name: 'tr.monitor2.cpu_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_cpu_health_per_instance
      },
      redfish_thermal_fan_health: {
        name: 'tr.monitor2.fan_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_fan_health_per_instance
      },
      redfish_thermal_temperature_intake_average: {
        name: 'tr.monitor2.intake_temperature',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_intake_average_per_instance
      },
      redfish_thermal_temperature_intake_health: {
        name: 'tr.monitor2.intake_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_intake_health_per_instance
      },
      redfish_power_powercontrol_averageconsumedwatts: {
        name: 'tr.monitor2.powercontrol_watts',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_power_powercontrol_averageconsumedwatts_per_instance
      },
      redfish_power_powerredundancy_health: {
        name: 'tr.monitor2.power_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_power_powerredundancy_health_per_instance
      },
      redfish_power_powersupply_health: {
        name: 'tr.monitor2.powersupply_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_power_powersupply_health_per_instance
      },
      redfish_thermal_temperature_powersupply_average: {
        name: 'tr.monitor2.powersupply_temperature',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_powersupply_average_per_instance
      },
      redfish_thermal_temperature_powersupply_health: {
        name: 'tr.monitor2.powersupply_temp_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_powersupply_health_per_instance
      },
      redfish_thermal_temperature_systemboard_average: {
        name: 'tr.monitor2.systemboard_temperature',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_systemboard_average_per_instance
      },
      redfish_thermal_temperature_systemboard_health: {
        name: 'tr.monitor2.systemboard_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_systemboard_health_per_instance
      },
      redfish_thermal_temperature_front_health: {
        name: 'tr.monitor2.front_temp_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_front_health_per_instance
      },
      redfish_thermal_temperature_networkingdevices_health: {
        name: 'tr.monitor2.networking_devices_temp_health',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_temperature_networkingdevices_per_instance
      },
      redfish_thermal_fan_speed: {
        name: 'tr.monitor2.fan_speed',
        enabled: false,
        percentage: false,
        category: 'Redfish',
        units: '',
        instance: :redfish_thermal_fan_speed_per_instance
      },
      thermal: {
        name: 'Thermal',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: '',
        instance: :thermal_per_instance
      },
      power: {
        name: 'Power',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: '',
        instance: :power_per_instance
      },
      bulkstats_schema: {
        name: 'Bulkstats Schema',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: '',
        instance: :bulkstats_schema_per_instance
      },
      systems: {
        name: 'tr.monitor2.systems',
        enabled: false,
        percentage: false,
        category: 'Hardware',
        units: '',
        instance: :systems_per_instance
      },
      logstash_load_1: {
        name: 'Logstash Load 1m',
        enabled: false,
        percentage: true,
        category: 'Logstash',
        units: '',
        instance: :logstash_load_1
      },
      logstash_load_5: {
        name: 'Logstash Load 5m',
        enabled: false,
        percentage: true,
        category: 'Logstash',
        units: '',
        instance: :logstash_load_5
      },
      logstash_load_15: {
        name: 'Logstash Load 15m',
        enabled: false,
        percentage: true,
        category: 'Logstash',
        units: '',
        instance: :logstash_load_15
      },
      logstash_cpu: {
        name: 'Logstash CPU',
        enabled: false,
        percentage: true,
        category: 'Logstash',
        units: '%',
        instance: :logstash_cpu
      },
      logstash_heap: {
        name: 'Logstash Heap Used',
        enabled: false,
        percentage: true,
        category: 'Logstash',
        units: '%',
        instance: :logstash_heap
      },
      logstash_events: {
        name: 'Logstash Events',
        enabled: false,
        percentage: false,
        category: 'Logstash',
        units: 'events',
        instance: :logstash_events
      },
      logstash_memory: {
        name: 'Logstash Memory',
        enabled: false,
        percentage: false,
        category: 'Logstash',
        units: 'bytes',
        instance: :logstash_memory
      },
      logstash_events_per_pipeline: {
        name: 'Logstash Events Per Pipeline',
        enabled: false,
        percentage: false,
        category: 'Logstash',
        units: 'events',
        instance: :logstash_events_per_pipeline
      },
      logstash_events_count_queue: {
        name: 'Logstash Events Queue',
        enabled: false,
        percentage: false,
        category: 'Logstash',
        units: 'events',
        instance: :logstash_count_events_queue
      },
      logstash_events_count_queue_bytes: {
        name: 'Logstash Events Queue Bytes',
        enabled: false,
        percentage: false,
        category: 'Logstash',
        units: 'bytes',
        instance: :logstash_count_events_queue_bytes
      }
    )
  end

  aggregations(
    monitors: {
      name: 'Monitor', value: :monitors, size: 1, unit: '',
      aggregations: [
        { type: 'longSum',     name: 'monitor_sum', fieldName: 'events'    },
        { type: 'doubleSum',   name: 'sum_value',   fieldName: 'sum_value' },
        { type: 'doubleMin',   name: 'min',         fieldName: 'min_value' },
        { type: 'doubleMax',   name: 'max',         fieldName: 'max_value' }
      ],
      timeserie_aggregations: [
        { type: 'longSum',     name: 'monitor_sum', fieldName: 'events'    },
        { type: 'doubleSum',   name: 'sum_value',   fieldName: 'sum_value' }
      ],
      post_aggregations: [
        {
          type: 'arithmetic',
          name: 'monitors',
          fn: '/',
          fields: [
            {
              type: 'fieldAccess',
              fieldName: 'sum_value'
            },
            {
              type: 'fieldAccess',
              fieldName: 'monitor_sum'
            }
          ]
        }
      ]
    },
    sum: {
      name: 'Sum', value: :sum, size: 1, unit: '',
      aggregations: [
        { type: 'doubleSum', name: 'sum', fieldName: 'sum_value' }
      ]
    },
    events: {
      name: 'tr.monitor2.events2', unit: 'events', value: :events, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      description: 'tr.monitor2.number_of_events'
    }
  )

  # Include links in tools
  #menu :tools_menu,
  menu :product_menu,
      :monitor,
      { controller: :monitor, action: :index },
      caption: 'Monitor', html: { class: 'block-ui-but' }
  #    caption: 'Infrastructure monitor', html: { class: 'block-ui-but' },

  #    after: :setting

  menu :tools_menu,
       :monitor_categories,
       {
         controller: :monitor,
         action: :list
       },
       caption: "tr.monitor2.monitor",
       html: { class: 'block-ui-but' },
       after: :correlation_engine_rules,
       super_admin: true

  menu :user_home_page_collection_menu, :Monitor, {}, caption: '/monitor'
  Redborder::MenuManager.map :segments_menu do |menu|
    menu.push :all,
              { controller: :monitor, action: :more_segments },
              caption: "tr.monitor2.all",
              html: { 'data-remote' => true, 'data-source' => 'all' },
              first: true
    menu.push :monitor,
              { controller: :monitor,
                action: :more_segments,
                datasource: 'rb_monitor' },
              caption: 'Monitor',
              html: { 'data-remote' => true, 'data-source' => 'rb_monitor' }
  end

  Redborder::MenuManager.map :domain_menu do |menu|
    menu.push :historicals,
              {
                controller: 'monitor',
                action: 'historicals'
              },
              parent: :cluster_data,
              caption: "tr.monitor2.historicals",
              html: { class: 'block-ui-but' },
              super_admin: true,
              if: proc { |s| s.is_domain? }
    menu.push :realtime,
              {
                controller: 'monitor',
                action: 'indexing'
              },
              parent: :cluster_data,
              caption: "tr.monitor2.realtime",
              html: { class: 'block-ui-but' },
              super_admin: true,
              if: proc { |s| s.is_domain? }
  end

  Redborder::MenuManager.map :options_sensors_view_menu do |menu|
    menu.push :historicals,
              {
                controller: 'monitor',
                action: 'historicals'
              },
              parent: :cluster_data,
              caption: "tr.monitor2.historicals",
              html: { class: 'block-ui-but' },
              super_admin: true
    menu.push :realtime,
              {
                controller: 'monitor',
                action: 'indexing'
              },
              parent: :cluster_data,
              caption: "tr.monitor2.realtime",
              html: { class: 'block-ui-but' },
              super_admin: true
  end
end

if Redborder::Module.registered_modules.keys.include? :ips
  [:ips_menu, :ips_group_menu].each do |menu_type|
    Redborder::MenuManager.map menu_type do |menu|
      menu.push :monitors,
                { controller: 'monitor_categories', action: 'index' },
                param: :sensor_id,
                caption: "tr.monitor2.monitors",
                admin: true,
                module: 'monitor'
    end
  end
else
  Redborder::Module.registered_modules[:monitor][:tabs]
    .delete_if { |_k, v| v[:category] == 'IDS/IPS' }
end


# Add custom tabs from MonitorCategory

# MonitorCategory.where("technology = 'snmp' and main_category = 'Custom'").select{|m| m.sensor_types.include? Sensor::DEVICE }.each do |monitor|
#   operations = monitor.operation.is_a?(Hash) ? [monitor.operation] :
#                                                 monitor.operation
#   names = operations.map { |o| o['name'] }
#   names.each do |tab_name|
#     if !Redborder::Module.registered_modules[:monitor][:tabs].key? tab_name
#       Redborder::Module.registered_modules[:monitor][:tabs][tab_name.to_sym]  =
#       {:name=>monitor.name, :enabled=>true, :percentage=>true, :category=>"Custom", :units=>""}
#     end
#   end
# end

CustomTab.all.each do |custom_tab|
  tab_name = 'custom' + "_" + custom_tab.name.downcase.strip.gsub(' ','_').downcase
  if !Redborder::Module.registered_modules[:monitor][:tabs].key? tab_name.to_sym
      Redborder::Module.registered_modules[:monitor][:tabs][tab_name.to_sym] = {  :name=>custom_tab.name,
                                                                                  :enabled=>custom_tab.enabled,
                                                                                  :percentage=>custom_tab.percentage,
                                                                                  :category=>custom_tab.category,
                                                                                  :units=>custom_tab.units,
                                                                                  :instance => tab_name.to_s + "_per_instance"
                                                                                }

  end
end

# extend DRUID_CONFIG
druid_monitor_config = YAML.load(
  File.read("#{Rails.root}/lib/modules/monitor/config/rbdruid_config.yml")
)[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_monitor_config

require_dependency 'widget_monitor'
require_dependency 'widget_type_monitor'
require_dependency 'report_block_monitor'
require_dependency 'alarm_monitor'
require_dependency 'sensor_category_monitor'

# Require jobs
require_dependency 'check_monitor_alarms_job'
require_dependency 'bulkstats_update_job'
require_dependency 'delayed_job_monitor'

# Require controllers
require_dependency 'application_controller_monitor'
require_dependency 'settings_controller_monitor'

# Require helpers
require_dependency 'widgets_helper_monitor'

# Require hooks
require_dependency 'hooks/alarms_monitor_hook'
require_dependency 'hooks/widget_data_monitor_hook'

to_prepare = proc do
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperMonitor)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperMonitor)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerMonitor)
    ApplicationController.send(:include,
                               RedBorder::ApplicationControllerMonitor)
  end
  unless SettingsController.include?(RedBorder::SettingsControllerMonitor)
    SettingsController.send(:prepend, RedBorder::SettingsControllerMonitor)
  end
  unless Redborder::Jobs::CheckAlarmsJob.include?(Redborder::Jobs::CheckMonitorAlarmsJob)
   Redborder::Jobs::CheckAlarmsJob.send(:include, Redborder::Jobs::CheckMonitorAlarmsJob)
  end
end

Rails.configuration.to_prepare(&to_prepare)
