#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

RedBorder::Application.routes.draw do

  resources :sensors, except: :show do
    resources :monitor_categories do
      post :add, on: :member
      post :restore, on: :collection
    end
  end

  resources :monitor, only: [:index] do
    get :load_tab, on: :member
    collection do
      get :historicals
      get :indexing
      get :load_more_tasks
      get :complete_tasks
      get :namespace_tasks
      get :load_more
      get :task_log
      patch :change_session_tab
      get :search
      post :results
      get :export_tops_to_csv
      post "export_tops_to_file", action: :export_tops_to_file, as: :export_tops_to_file
DittoCode::Exec.if 'ENTERPRISE' do
      get :zookeeper_nodes
      get :psql_nodes
end
      get :segments
      post :delete_segments
      get :more_segments
      get :load_segments_in_range
      get :hardware
      get :device
      get :diagram
      get :cluster_diagram
      get :indexed_data
      get :help
    end

    member do
      delete :delete_segment, :constraints => { :id => /[a-zA-Z0-9_\-:\.]+/ }
    end
  end

  patch 'monitor/change_chart_type/:id' => 'monitor#change_chart_type', as: 'change_chart_type_monitor'
  patch 'saved_monitor_filter/:id/add' => 'saved_monitor_filters#add_saved_filter', as: "add_saved_monitor_filter"
  get 'monitor/search' => 'monitor#search', as: 'search_monitor'

  get 'monitor/add_tab/:id' => 'monitor#add_tab', as: 'add_monitor_tab'
  patch 'monitor/del_tab/:id' => 'monitor#del_tab', as: 'del_monitor_tab'
  get 'monitor/get_serie/:id' => 'monitor#get_serie', as: 'get_monitor_serie'
  get 'monitor/get_tops/:id' => 'monitor#get_tops', as: 'get_monitor_tops'
  get 'monitor/get_instances/:id' => 'monitor#get_instances', as: 'get_monitor_instances'

  get 'monitor/test_oid' => 'monitor#test_oid', as: 'get_monitor_test_oid'

  post 'saved_monitor_filters/create' => 'saved_monitor_filters#create'
  get 'tabs/reorder_monitor' => 'monitor#reorder_tab', :as => 'reorder_monitor_tab'

  # Monitor Categories
  get 'monitor/list' => 'monitor#list', as: 'monitor_categories_list'
  get 'monitor/edit_monitor_category/:id' => 'monitor#edit_monitor_category', as: 'edit_monitor_category'
  get 'monitor/create_add_monitor_category/:id' => 'monitor#create_add_monitor_category', as: 'create_add_monitor_category'
  patch 'monitor/update_monitor_category/:id' => 'monitor#update_monitor_category', as: 'update_monitor_category'
  get 'monitor/new_monitor_category' => 'monitor#new_monitor_category', as: 'new_monitor_category'
  post 'monitor/create_monitor_category' => 'monitor#create_monitor_category', as: 'create_monitor_category'
  post 'monitor/create_and_add_monitor_category' => 'monitor#create_and_add_monitor_category', as: 'create_and_add_monitor_category'
  delete 'monitor/delete_monitor_category/:id' => 'monitor#delete_monitor_category', as: 'delete_monitor_category'
  get 'monitor/get_categories' => 'monitor#get_categories', as: 'get_categories'
  get 'monitor/get_oids' => 'monitor#get_oids', as: 'get_oids'

  #Monitor categories
  get 'monitor/import_monitor_categories', to: 'monitor#import_monitor', as: 'import_monitor'
  get 'monitor/export_monitor_categories', to: 'monitor#export_monitor'
  post 'monitor/import_monitor_categories', to: 'monitor#import_monitor'
  patch 'monitor/remove_bulkstats_schema', to:'monitor#remove_bulkstats_schema'

  resources :saved_monitor_filters, :path => "/saved/monitor_filters" do

    collection do
      post :title
      post :update
    end

    member do
      get :view
      post :update
    end
  end

  namespace :api do
    resources :monitor, path: '/monitor', only: [:index] do
      get :info, on: :member
    end
    namespace :v1 do
      resources :custom_tabs, only: [:index, :create, :update] do
        delete '/', on: :collection, action: :destroy
        post '/import', on: :collection, action: :import
      end
      resources :monitor, only: [:index] do
        get '/get_bulkstats_columns', on: :collection, action: :get_bulkstats_columns
      end
    end
  end

end
