###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

not_activated = %w(
  monitor_nat monitor_session monitor_swap monitor_throughput_rb_flow
  monitor_throughput_rb_flow_post monitor_throughput_rb_loc
  monitor_throughput_rb_radius monitor_throughput_rb_event
  monitor_throughput_rb_monitor monitor_throughput_rb_trap
  monitor_syns_per_second monitor_synacks_per_second
  monitor_frag_creates_per_second monitor_frag_inserts_per_second
  monitor_frag_flushes_per_second monitor_current_frags
  monitor_max_frags monitor_total_udp_sessions
  monitor_max_udp_sessions monitor_deleted_sessions_per_second
  monitor_new_udp_sessions_per_second monitor_deleted_udp_sessions_per_second
  monitor_tcp_sessions_closed_per_second
  monitor_tcp_sessions_timedout_per_second
  monitor_organization_received_bytes
  monitor_avio
)

Redborder::Module.get(:monitor, :tabs).keys.each do |col|
  col = "monitor_#{col}"
  value = not_activated.include?(col) ? '0' : '1'
  Setting.where(name: col).first_or_create(name: col, value: value)
end

# Load Default Monitor Categories
if Redborder::Module.registered_modules.include?(:ips)
  # System Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'cpu',
    description: 'Monitors the load of CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'cpu_idle', oid: 'UCD-SNMP-MIB::ssCpuIdle.0',
        unit: '%', send: 0 },
      { name: 'cpu', op: '100-cpu_idle', unit: '%' }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  # System Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'cpu',
    description: 'Monitors the load of CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'cpu_idle', oid: 'UCD-SNMP-MIB::ssCpuIdle.0',
        unit: '%', send: 0 },
      { name: 'cpu', op: '100-cpu_idle', unit: '%' }
    ],
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  # System Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'cpu',
    description: 'Monitors the load of CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'cpu_idle', oid: 'UCD-SNMP-MIB::ssCpuIdle.0',
        unit: '%', send: 0 },
      { name: 'cpu', op: '100-cpu_idle', unit: '%' }
    ],
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'cpu',
    description: 'Monitors the load of CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'cpu_idle', oid: 'UCD-SNMP-MIB::ssCpuIdle.0',
        unit: '%', send: 0 },
      { name: 'cpu', op: '100-cpu_idle', unit: '%' }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'disk',
    description: 'Monitors usage space on disks using SNMP.',
    technology: 'snmp',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk',
      oid: 'UCD-SNMP-MIB::dskPercent.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'disk',
    description: 'Monitors usage space on disks using SNMP.',
    technology: 'snmp',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk',
      oid: 'UCD-SNMP-MIB::dskPercent.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'disk',
    description: 'Monitors usage space on disks using SNMP.',
    technology: 'snmp',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk',
      oid: 'UCD-SNMP-MIB::dskPercent.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'disk',
    description: 'Monitors usage space on disks using SNMP.',
    technology: 'snmp',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk',
      oid: 'UCD-SNMP-MIB::dskPercent.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk_load',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'disk_load',
    description: 'Monitors disk load.',
    technology: 'system',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk_load',
      system: "snmptable -v 2c -c redborder 127.0.0.1 diskIOTable|grep ' dm-0 ' | awk '{print $7}'",
      unit: '%'
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk_load',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'disk_load',
    description: 'Monitors disk load.',
    technology: 'system',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk_load',
      system: "snmptable -v 2c -c redborder 127.0.0.1 diskIOTable|grep ' dm-0 ' | awk '{print $7}'",
      unit: '%'
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'disk_load',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'disk_load',
    description: 'Monitors disk load.',
    technology: 'system',
    category: 'disk',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'disk_load',
      system: "snmptable -v 2c -c redborder 127.0.0.1 diskIOTable|grep ' dm-0 ' | awk '{print $7}'",
      unit: '%'
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'load_1',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'load_1',
    description: 'Monitors System Load average using SNMP.',
    technology: 'snmp',
    category: 'load_1',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'load_1',
      oid: 'UCD-SNMP-MIB::laLoad.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'load_1',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'load_1',
    description: 'Monitors System Load average using SNMP.',
    technology: 'snmp',
    category: 'load_1',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'load_1',
      oid: 'UCD-SNMP-MIB::laLoad.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'load_1',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'load_1',
    description: 'Monitors System Load average using SNMP.',
    technology: 'snmp',
    category: 'load_1',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'load_1',
      oid: 'UCD-SNMP-MIB::laLoad.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'load_1',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'load_1',
    description: 'Monitors System Load average using SNMP.',
    technology: 'snmp',
    category: 'load_1',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'load_1',
      oid: 'UCD-SNMP-MIB::laLoad.1',
      unit: '%'
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'memory',
    description: 'Monitors the memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total', oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_free',  oid: 'UCD-SNMP-MIB::memAvailReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_buffer', oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_cache', oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory',
        op: '100*(memory_total-memory_free-memory_total_buffer-memory_total_cache)/memory_total',
        unit: '%' }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'memory',
    description: 'Monitors the memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total', oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_free',  oid: 'UCD-SNMP-MIB::memAvailReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_buffer', oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_cache', oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory',
        op: '100*(memory_total-memory_free-memory_total_buffer-memory_total_cache)/memory_total',
        unit: '%' }
    ],
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'memory',
    description: 'Monitors the memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total', oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_free',  oid: 'UCD-SNMP-MIB::memAvailReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_buffer', oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_cache', oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory',
        op: '100*(memory_total-memory_free-memory_total_buffer-memory_total_cache)/memory_total',
        unit: '%' }
    ],
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'memory',
    description: 'Monitors the memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total', oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_free',  oid: 'UCD-SNMP-MIB::memAvailReal.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_buffer', oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory_total_cache', oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0, unit: 'kB', integer: 1 },
      { name: 'memory',
        op: '100*(memory_total-memory_free-memory_total_buffer-memory_total_cache)/memory_total',
        unit: '%' }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_buffer',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'memory_buffer',
    description: 'Memory buffered in %',
    technology: 'snmp',
    category: 'memory_buffer',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_buffer',
        oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_buffer',
        op: '100*memory_total_buffer/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_buffer',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'memory_buffer',
    description: 'Memory buffered in %',
    technology: 'snmp',
    category: 'memory_buffer',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_buffer',
        oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_buffer',
        op: '100*memory_total_buffer/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_buffer',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'memory_buffer',
    description: 'Memory buffered in %',
    technology: 'snmp',
    category: 'memory_buffer',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_buffer',
        oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_buffer',
        op: '100*memory_total_buffer/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_buffer',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'memory_buffer',
    description: 'Memory buffered in %',
    technology: 'snmp',
    category: 'memory_buffer',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_buffer',
        oid: 'UCD-SNMP-MIB::memBuffer.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_buffer',
        op: '100*memory_total_buffer/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_cache',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'memory_cache',
    description: 'Memory cached in %',
    technology: 'snmp',
    category: 'memory_cache',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_cache',
        oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_cache',
        op: '100*memory_total_cache/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_cache',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'memory_cache',
    description: 'Memory cached in %',
    technology: 'snmp',
    category: 'memory_cache',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_cache',
        oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_cache',
        op: '100*memory_total_cache/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_cache',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'memory_cache',
    description: 'Memory cached in %',
    technology: 'snmp',
    category: 'memory_cache',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_cache',
        oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_cache',
        op: '100*memory_total_cache/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_cache',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'memory_cache',
    description: 'Memory cached in %',
    technology: 'snmp',
    category: 'memory_cache',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_total',
        oid: 'UCD-SNMP-MIB::memTotalReal.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total_cache',
        oid: 'UCD-SNMP-MIB::memCached.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_cache',
        op: '100*memory_total_cache/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'swap',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'swap',
    description: 'Swap usage in %',
    technology: 'snmp',
    category: 'swap',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'swap_total',
        oid: 'UCD-SNMP-MIB::memTotalSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap_free',
        oid: 'UCD-SNMP-MIB::memAvailSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap',
        op: '100*(swap_total-swap_free)/swap_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'swap',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'swap',
    description: 'Swap usage in %',
    technology: 'snmp',
    category: 'swap',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'swap_total',
        oid: 'UCD-SNMP-MIB::memTotalSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap_free',
        oid: 'UCD-SNMP-MIB::memAvailSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap',
        op: '100*(swap_total-swap_free)/swap_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'swap',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'swap',
    description: 'Swap usage in %',
    technology: 'snmp',
    category: 'swap',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'swap_total',
        oid: 'UCD-SNMP-MIB::memTotalSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap_free',
        oid: 'UCD-SNMP-MIB::memAvailSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap',
        op: '100*(swap_total-swap_free)/swap_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'swap',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'swap',
    description: 'Swap usage in %',
    technology: 'snmp',
    category: 'swap',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'swap_total',
        oid: 'UCD-SNMP-MIB::memTotalSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap_free',
        oid: 'UCD-SNMP-MIB::memAvailSwap.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'swap',
        op: '100*(swap_total-swap_free)/swap_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'reflect_fwd_error',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'reflect_fwd_error',
    description: 'Total packets forwarded with errors (accumulative)',
    technology: 'system',
    category: 'reflect_fwd_error',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'reflect_fwd_error',
      system: "sudo /usr/lib/redborder/bin/rb_get_pfring_stats.sh -p 'Reflect FWD Error'",
      name_split_suffix: '_per_eth',
      unit: 'pkts',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'eth',
      timestamp_given: 0
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'reflect_fwd_error',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'reflect_fwd_error',
    description: 'Total packets forwarded with errors (accumulative)',
    technology: 'system',
    category: 'reflect_fwd_error',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'reflect_fwd_error',
      system: "sudo /usr/lib/redborder/bin/rb_get_pfring_stats.sh -p 'Reflect FWD Error'",
      name_split_suffix: '_per_eth',
      unit: 'pkts',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'eth',
      timestamp_given: 0
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'reflect_fwd_error',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'reflect_fwd_error',
    description: 'Total packets forwarded with errors (accumulative)',
    technology: 'system',
    category: 'reflect_fwd_error',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'reflect_fwd_error',
      system: "sudo /usr/lib/redborder/bin/rb_get_pfring_stats.sh -p 'Reflect FWD Error'",
      name_split_suffix: '_per_eth',
      unit: 'pkts',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'eth',
      timestamp_given: 0
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  # Hardware Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'system_temp',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'system_temp',
    description: 'System temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'system_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'system_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'System Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'system_temp',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'system_temp',
    description: 'System temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'system_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'system_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'System Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  # Hardware Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'system_temp',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'system_temp',
    description: 'System temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'system_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'system_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'System Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'peripheral_temp',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'peripheral_temp',
    description: 'Additional system temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'peripheral_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'peripheral_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'Peripheral[ Temp]*'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'peripheral_temp',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'peripheral_temp',
    description: 'Additional system temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'peripheral_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'peripheral_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'Peripheral[ Temp]*'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'peripheral_temp',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'peripheral_temp',
    description: 'Additional system temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'peripheral_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'peripheral_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'Peripheral[ Temp]*'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pch_temp',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'pch_temp',
    description: '',
    technology: 'ipmi',
    category: 'pch_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'pch_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'PCH Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pch_temp',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'pch_temp',
    description: '',
    technology: 'ipmi',
    category: 'pch_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'pch_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'PCH Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pch_temp',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'pch_temp',
    description: '',
    technology: 'ipmi',
    category: 'pch_temp',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'pch_temp',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s 'PCH Temp'",
      unit: 'celsius',
      integer: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'fan',
    "%#{Sensor::IPS}%"
  ).first_or_create(
    name: 'fan',
    description: 'CPU temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'fan',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'fan',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Fan -a -s 'FAN[ ]*'",
      unit: 'rpm',
      name_split_suffix: '_per_instance',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'fan-',
      integer: 1
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'fan',
    "%#{Sensor::IPS_PROXY}%"
  ).first_or_create(
    name: 'fan',
    description: 'CPU temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'fan',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'fan',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Fan -a -s 'FAN[ ]*'",
      unit: 'rpm',
      name_split_suffix: '_per_instance',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'fan-',
      integer: 1
    },
    sensor_types: [Sensor::IPS_PROXY],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'fan',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'fan',
    description: 'CPU temperature in Celsius degrees',
    technology: 'ipmi',
    category: 'fan',
    main_category: 'Hardware',
    impact: 2,
    operation: {
      name: 'fan',
      system: "sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Fan -a -s 'FAN[ ]*'",
      unit: 'rpm',
      name_split_suffix: '_per_instance',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'fan-',
      integer: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  # IPS/IDS Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'avg_bytes_per_wire_packet',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'avg_bytes_per_wire_packet',
    description: 'Average packet size in Bytes',
    technology: 'system',
    category: 'avg_bytes_per_wire_packet',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'avg_bytes_per_wire_packet',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'avg_bytes_per_wire_packet'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'avg_bytes_per_wire_packet',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'avg_bytes_per_wire_packet',
    description: 'Average packet size in Bytes',
    technology: 'system',
    category: 'avg_bytes_per_wire_packet',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'avg_bytes_per_wire_packet',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'avg_bytes_per_wire_packet'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'avg_bytes_per_wire_packet',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'avg_bytes_per_wire_packet',
    description: 'Average packet size in Bytes',
    technology: 'system',
    category: 'avg_bytes_per_wire_packet',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'avg_bytes_per_wire_packet',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'avg_bytes_per_wire_packet'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'wire_bits_per_sec_realtime',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'wire_bits_per_sec_realtime',
    description: 'Traffic per second in Bits/s',
    technology: 'system',
    category: 'wire_bits_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'wire_bits_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'wire_bits_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'bits/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'wire_bits_per_sec_realtime',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'wire_bits_per_sec_realtime',
    description: 'Traffic per second in Bits/s',
    technology: 'system',
    category: 'wire_bits_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'wire_bits_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'wire_bits_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'bits/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'wire_bits_per_sec_realtime',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'wire_bits_per_sec_realtime',
    description: 'Traffic per second in Bits/s',
    technology: 'system',
    category: 'wire_bits_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'wire_bits_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'wire_bits_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'bits/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkt_drop_percent',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'pkt_drop_percent',
    description: 'Packets dropped in %',
    technology: 'system',
    category: 'pkt_drop_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'pkt_drop_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'pkt_drop_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkt_drop_percent',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'pkt_drop_percent',
    description: 'Packets dropped in %',
    technology: 'system',
    category: 'pkt_drop_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'pkt_drop_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'pkt_drop_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkt_drop_percent',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'pkt_drop_percent',
    description: 'Packets dropped in %',
    technology: 'system',
    category: 'pkt_drop_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'pkt_drop_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'pkt_drop_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'alerts_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'alerts_per_second',
    description: 'Events per second',
    technology: 'system',
    category: 'alerts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'alerts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'alerts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'events/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'alerts_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'alerts_per_second',
    description: 'Events per second',
    technology: 'system',
    category: 'alerts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'alerts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'alerts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'events/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'alerts_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'alerts_per_second',
    description: 'Events per second',
    technology: 'system',
    category: 'alerts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'alerts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'alerts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'events/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_sessions_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'new_sessions_per_second',
    description: 'New sessions per second as reported by Stream5',
    technology: 'system',
    category: 'new_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'new_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_sessions_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'new_sessions_per_second',
    description: 'New sessions per second as reported by Stream5',
    technology: 'system',
    category: 'new_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'new_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_sessions_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'new_sessions_per_second',
    description: 'New sessions per second as reported by Stream5',
    technology: 'system',
    category: 'new_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'new_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'patmatch_percent',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'patmatch_percent',
    description: '% of bytes that go through pattern matching. Can be higher than 100%',
    technology: 'system',
    category: 'patmatch_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'patmatch_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'patmatch_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'patmatch_percent',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'patmatch_percent',
    description: '% of bytes that go through pattern matching. Can be higher than 100%',
    technology: 'system',
    category: 'patmatch_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'patmatch_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'patmatch_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'patmatch_percent',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'patmatch_percent',
    description: '% of bytes that go through pattern matching. Can be higher than 100%',
    technology: 'system',
    category: 'patmatch_percent',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'patmatch_percent',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'patmatch_percent'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'packets_wire_per_sec_realtime',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'packets_wire_per_sec_realtime',
    description: 'Packets per second',
    technology: 'system',
    category: 'packets_wire_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'packets_wire_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'packets_wire_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'pkts/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'packets_wire_per_sec_realtime',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'packets_wire_per_sec_realtime',
    description: 'Packets per second',
    technology: 'system',
    category: 'packets_wire_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'packets_wire_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'packets_wire_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'pkts/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'packets_wire_per_sec_realtime',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'packets_wire_per_sec_realtime',
    description: 'Packets per second',
    technology: 'system',
    category: 'packets_wire_per_sec_realtime',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'packets_wire_per_sec_realtime',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'packets_wire_per_sec_realtime'",
      name_split_suffix: '_per_instance',
      unit: 'pkts/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_sessions',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'total_sessions',
    description: 'Active sessions as reported by Stream5',
    technology: 'system',
    category: 'total_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'total_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_sessions',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'total_sessions',
    description: 'Active sessions as reported by Stream5',
    technology: 'system',
    category: 'total_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'total_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_sessions',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'total_sessions',
    description: 'Active sessions as reported by Stream5',
    technology: 'system',
    category: 'total_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'total_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'snort_cpu',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'snort_cpu',
    description: 'CPU load in % related to Snort',
    technology: 'system',
    category: 'snort_cpu',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'snort_cpu',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'cpu[0]'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'snort_cpu',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'snort_cpu',
    description: 'CPU load in % related to Snort',
    technology: 'system',
    category: 'snort_cpu',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'snort_cpu',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'cpu[0]'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'snort_cpu',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'snort_cpu',
    description: 'CPU load in % related to Snort',
    technology: 'system',
    category: 'snort_cpu',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'snort_cpu',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'cpu[0]'",
      name_split_suffix: '_per_instance',
      unit: '%',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream_flushes_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'stream_flushes_per_second',
    description: 'Sessions flushed from memory per second as reported by Stream5',
    technology: 'system',
    category: 'stream_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'stream_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'flushes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream_flushes_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'stream_flushes_per_second',
    description: 'Sessions flushed from memory per second as reported by Stream5',
    technology: 'system',
    category: 'stream_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'stream_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'flushes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream_flushes_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'stream_flushes_per_second',
    description: 'Sessions flushed from memory per second as reported by Stream5',
    technology: 'system',
    category: 'stream_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'stream_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'flushes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream5_mem_in_use',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'stream5_mem_in_use',
    description: 'Memory used by Stream5 preprocessor in Mbytes',
    technology: 'system',
    category: 'stream5_mem_in_use',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream5_mem_in_use',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'stream5_mem_in_use'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream5_mem_in_use',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'stream5_mem_in_use',
    description: 'Memory used by Stream5 preprocessor in Mbytes',
    technology: 'system',
    category: 'stream5_mem_in_use',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream5_mem_in_use',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'stream5_mem_in_use'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'stream5_mem_in_use',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'stream5_mem_in_use',
    description: 'Memory used by Stream5 preprocessor in Mbytes',
    technology: 'system',
    category: 'stream5_mem_in_use',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'stream5_mem_in_use',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'stream5_mem_in_use'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )


  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'ratio_syn_synacks',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'ratio_syn_synacks',
    description: 'Ratio of SYN to SYNACK packets',
    technology: 'system',
    category: 'ratio_syn_synacks',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'ratio_syn_synacks',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'ratio_syn_synacks'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'ratio_syn_synacks',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'ratio_syn_synacks',
    description: 'Ratio of SYN to SYNACK packets',
    technology: 'system',
    category: 'ratio_syn_synacks',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'ratio_syn_synacks',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'ratio_syn_synacks'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'ratio_syn_synacks',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'ratio_syn_synacks',
    description: 'Ratio of SYN to SYNACK packets',
    technology: 'system',
    category: 'ratio_syn_synacks',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'ratio_syn_synacks',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'ratio_syn_synacks'",
      name_split_suffix: '_per_instance',
      unit: 'bytes',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: true
  )

  #Non default values:
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'syns_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'syns_per_second',
    description: '',
    technology: 'system',
    category: 'syns_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'syns_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'syns_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  #Non default values:
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'syns_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'syns_per_second',
    description: '',
    technology: 'system',
    category: 'syns_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'syns_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'syns_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  #Non default values:
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'syns_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'syns_per_second',
    description: '',
    technology: 'system',
    category: 'syns_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'syns_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'syns_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'synacks_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'synacks_per_second',
    description: '',
    technology: 'system',
    category: 'synacks_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'synacks_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'synacks_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn-acks/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'synacks_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'synacks_per_second',
    description: '',
    technology: 'system',
    category: 'synacks_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'synacks_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'synacks_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn-acks/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'synacks_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'synacks_per_second',
    description: '',
    technology: 'system',
    category: 'synacks_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'synacks_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'synacks_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'syn-acks/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_sessions_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'deleted_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'deleted_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_sessions_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'deleted_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'deleted_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_sessions_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'deleted_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'deleted_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_creates_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'frag_creates_per_second',
    description: '',
    technology: 'system',
    category: 'frag_creates_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_creates_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'frag_creates_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_creates_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'frag_creates_per_second',
    description: '',
    technology: 'system',
    category: 'frag_creates_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_creates_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_creates_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_creates_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'frag_creates_per_second',
    description: '',
    technology: 'system',
    category: 'frag_creates_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_creates_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_creates_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_inserts_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'frag_inserts_per_second',
    description: '',
    technology: 'system',
    category: 'frag_inserts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_inserts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'frag_inserts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_inserts_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'frag_inserts_per_second',
    description: '',
    technology: 'system',
    category: 'frag_inserts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_inserts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_inserts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_inserts_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'frag_inserts_per_second',
    description: '',
    technology: 'system',
    category: 'frag_inserts_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_inserts_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_inserts_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_flushes_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'frag_flushes_per_second',
    description: '',
    technology: 'system',
    category: 'frag_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'frag_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_flushes_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'frag_flushes_per_second',
    description: '',
    technology: 'system',
    category: 'frag_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'frag_flushes_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'frag_flushes_per_second',
    description: '',
    technology: 'system',
    category: 'frag_flushes_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'frag_flushes_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'frag_flushes_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'frag/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'current_frags',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'current_frags',
    description: '',
    technology: 'system',
    category: 'current_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'current_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'current_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frag',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'current_frags',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'current_frags',
    description: '',
    technology: 'system',
    category: 'current_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'current_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'current_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frag',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'current_frags',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'current_frags',
    description: '',
    technology: 'system',
    category: 'current_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'current_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'current_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frag',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_frags',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'max_frags',
    description: '',
    technology: 'system',
    category: 'max_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'max_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frags',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_frags',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'max_frags',
    description: '',
    technology: 'system',
    category: 'max_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'max_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frags',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_frags',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'max_frags',
    description: '',
    technology: 'system',
    category: 'max_frags',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_frags',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'max_frags'",
      name_split_suffix: '_per_instance',
      unit: 'frags',
      split: ';',
      split_op: 'mean',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_udp_sessions_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'new_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'new_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'new_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_udp_sessions_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'new_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'new_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'new_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'new_udp_sessions_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'new_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'new_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'new_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'new_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_udp_sessions_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'deleted_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'deleted_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_udp_sessions_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'deleted_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'deleted_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'deleted_udp_sessions_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'deleted_udp_sessions_per_second',
    description: '',
    technology: 'system',
    category: 'deleted_udp_sessions_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'deleted_udp_sessions_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'deleted_udp_sessions_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_udp_sessions',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'total_udp_sessions',
    description: '',
    technology: 'system',
    category: 'total_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'total_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_udp_sessions',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'total_udp_sessions',
    description: '',
    technology: 'system',
    category: 'total_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'total_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'total_udp_sessions',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'total_udp_sessions',
    description: '',
    technology: 'system',
    category: 'total_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'total_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'total_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_udp_sessions',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'max_udp_sessions',
    description: '',
    technology: 'system',
    category: 'max_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'max_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_udp_sessions',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'max_udp_sessions',
    description: '',
    technology: 'system',
    category: 'max_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'max_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'max_udp_sessions',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'max_udp_sessions',
    description: '',
    technology: 'system',
    category: 'max_udp_sessions',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'max_udp_sessions',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'max_udp_sessions'",
      name_split_suffix: '_per_instance',
      unit: 'sessions',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_closed_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'tcp_sessions_closed_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_closed_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_closed_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'tcp_sessions_closed_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_closed_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'tcp_sessions_closed_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_closed_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_closed_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'tcp_sessions_closed_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_closed_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'tcp_sessions_closed_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_closed_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_closed_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'tcp_sessions_closed_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_timedout_per_second',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'tcp_sessions_timedout_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_timedout_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_timedout_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -g %group_id -d 120 -p 'tcp_sessions_timedout_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GROUP],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_timedout_per_second',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'tcp_sessions_timedout_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_timedout_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_timedout_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'tcp_sessions_timedout_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_V2],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'tcp_sessions_timedout_per_second',
    "%#{Sensor::IPS_GENERIC}%"
  ).first_or_create(
    name: 'tcp_sessions_timedout_per_second',
    description: '',
    technology: 'system',
    category: 'tcp_sessions_timedout_per_second',
    main_category: 'IDS/IPS',
    impact: 2,
    operation: {
      name: 'tcp_sessions_timedout_per_second',
      system: "sudo /usr/lib/redborder/bin/rb_get_perfmonitor_stats.sh -t -d 120 -p 'tcp_sessions_timedout_per_second'",
      name_split_suffix: '_per_instance',
      unit: 'sessions/s',
      split: ';',
      split_op: 'sum',
      instance_prefix: 'instance-',
      timestamp_given: 1
    },
    sensor_types: [Sensor::IPS_GENERIC],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'latency',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'latency',
    description: '',
    technology: 'system',
    category: 'latency',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'latency',
      system: "nice -n 19 fping -q -s kafka.service 2>&1| grep 'avg round trip time'|awk '{print $1}'",
      unit: 'ms'
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'latency',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'latency',
    description: '',
    technology: 'system',
    category: 'latency',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'latency',
      system: "nice -n 19 fping -q -s kafka.service 2>&1| grep 'avg round trip time'|awk '{print $1}'",
      unit: 'ms'
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkts_lost',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'pkts_lost',
    description: '',
    technology: 'system',
    category: 'pkts_lost',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'pkts_lost',
      system: "nice -n 19 fping -p 20 -c 10 kafka.service 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
      unit: '%'
    },
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkts_lost',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'pkts_lost',
    description: '',
    technology: 'system',
    category: 'pkts_lost',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'pkts_lost',
      system: "nice -n 19 fping -p 20 -c 10 kafka.service 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
      unit: '%'
    },
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkts_percent_rcv_ips',
    "%#{Sensor::IPS_GROUP}%"
  ).first_or_create(
    name: 'pkts_percent_rcv_ips',
    description: '',
    technology: 'system',
    category: 'pkts_percent_rcv',
    main_category: 'Network',
    impact: 4,
    operation: [
      { name: 'pkts_lost',
        system: "nice -n 19 fping -p 20 -c 10 kafka.service 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
        unit: '%'
      },
      { name: 'pkts_percent_rcv',
        op: "100 - pkts_lost",
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'pkts_percent_rcv_ips',
    "%#{Sensor::IPS_V2}%"
  ).first_or_create(
    name: 'pkts_percent_rcv_ips',
    description: '',
    technology: 'system',
    category: 'pkts_percent_rcv',
    main_category: 'Network',
    impact: 4,
    operation: [
      { name: 'pkts_lost',
        system: "nice -n 19 fping -p 20 -c 10 kafka.service 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
        unit: '%'
      },
      { name: 'pkts_percent_rcv',
        op: "100 - pkts_lost",
        unit: '%'
      }
    ],
    sensor_types: [Sensor::IPS_V2],
    default: true
  )

end

if Redborder::Module.registered_modules.include?(:flow)
  # System Category
  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'cpu',
    description: 'Monitors the load of ASR CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'cpu',
      oid: 'SNMPv2-SMI::enterprises.9.9.109.1.1.1.1.4.2',
      unit: '%'
    },
    sensor_types: [Sensor::FLOW],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu_2',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'cpu_2',
    description: 'Monitors the load of WLC CPU via SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'cpu',
      oid: '1.3.6.1.4.1.14179.1.1.5.1.0',
      unit: '%'
    },
    sensor_types: [Sensor::FLOW],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'cpu_3',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'cpu_3',
    description: 'Monitors the load of CPU via Host Resource SNMP.',
    technology: 'snmp',
    category: 'cpu',
    main_category: 'System',
    impact: 1,
    operation: {
      name: 'cpu',
      oid: 'HOST-RESOURCES-MIB::hrProcessorLoad.8',
      unit: '%'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'memory',
    description: 'Monitors the ASR memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_free',
        oid: '1.3.6.1.4.1.9.9.48.1.1.1.6.1',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_used',
        oid: '1.3.6.1.4.1.9.9.48.1.1.1.5.1',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory',
        op: '100*(memory_used)/(memory_used + memory_free)',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::FLOW],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_2',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'memory_2',
    description: 'Monitors the WLC memory usage via SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_free',
        oid: '1.3.6.1.4.1.14179.1.1.5.3.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory_total',
        oid: '1.3.6.1.4.1.14179.1.1.5.2.0',
        send: 0,
        unit: 'kB',
        integer: 1
      },
      { name: 'memory',
        op: '100*(memory_total-memory_free)/memory_total',
        unit: '%'
      }
    ],
    sensor_types: [Sensor::FLOW],
    default: true
  )

  MonitorCategory.where(
    'name = ? AND sensor_types like ?',
    'memory_3',
    "%#{Sensor::FLOW}%"
  ).first_or_create(
    name: 'memory_3',
    description: 'Monitors the memory usage via Host Resource SNMP.',
    technology: 'snmp',
    category: 'memory',
    main_category: 'System',
    impact: 1,
    operation: [
      { name: 'memory_used',
        oid: 'HOST-RESOURCES-MIB::hrStorageUsed.1',
        send: 0,
        unit: '4kB',
        integer: 1
      },
      { name: 'memory_total',
        oid: 'HOST-RESOURCES-MIB::hrStorageSize.1',
        send: 0,
        unit: '4kB',
        integer: 1
      },
      { name: 'memory',
        op: '100*memory_used/memory_total',
        unit: '%',
        send: 1
      }
    ],
    sensor_types: [Sensor::FLOW],
    default: false
  )

  # Hardware Category
  MonitorCategory.where(
    name: 'wlc_temperature'
  ).first_or_create(
    name: 'wlc_temperature',
    description: 'WLC temperature in Celsius degrees.',
    technology: 'snmp',
    category: 'system_temp',
    main_category: 'Hardware',
    impact: 1,
    operation: {
      name: 'system_temp',
      oid: '1.3.6.1.4.1.14179.2.3.1.13.0',
      unit: 'celsius'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  # MonitorCategory.where(name: 'asr_temperature').first_or_create(name:
  # 'asr_temperature', description: 'ASR temperature in Celsius degrees',
  # technology: 'snmp', category: 'system_temp', main_category: 'Hardware',
  # impact: 1, operation: {name: 'system_temp', oid:
  # 'SNMPv2-SMI::mib-2.99.1.1.1.4.9019 ', unit: 'celsius'},
  # sensor_types: [Sensor::FLOW], default: false)
  # Network Category
  MonitorCategory.where(
    name: 'nat'
  ).first_or_create(
    name: 'nat',
    description: 'Network Address Translation counts.',
    technology: 'system',
    category: 'nat',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'nat',
      system: "usr/lib/redborder/bin/rb_get_cisco_nat.sh -u '%telnet_user' -p '%telnet_password' -i '%sensor_ip'",
      unit: 'nat'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'session'
  ).first_or_create(
    name: 'session',
    description: 'Cisco Sessions',
    technology: 'system',
    category: 'session',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'session',
      system: "usr/lib/redborder/bin/rb_get_cisco_sessions.sh -u '%telnet_user' -p '%telnet_password' -i '%sensor_ip'",
      unit: 'nat'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'latency_flow'
  ).first_or_create(
    name: 'latency_flow',
    description: '',
    technology: 'system',
    category: 'latency',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'latency',
      system: "nice -n 19 fping -q -s %sensor_ip 2>&1| grep 'avg round trip time'|awk '{print $1}'",
      unit: 'ms'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'pkts_lost_flow'
  ).first_or_create(
    name: 'pkts_lost_flow',
    description: '',
    technology: 'system',
    category: 'pkts_lost',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'pkts_lost',
      system: "nice -n 19 fping -p 20 -c 10 %sensor_ip 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
      unit: '%'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'pkts_percent_rcv_flow'
  ).first_or_create(
    name: 'pkts_percent_rcv_flow',
    description: '',
    technology: 'system',
    category: 'pkts_percent_rcv',
    main_category: 'Network',
    impact: 4,
    operation: [
      { name: 'pkts_lost',
        system: "nice -n 19 fping -p 20 -c 10 %sensor_ip 2>&1 | tail -n 1 | awk '{print $5}' | sed 's/%.*$//' | tr '/' ' ' | awk '{print $3}'",
        unit: '%'
      },
      { name: 'pkts_percent_rcv',
        op: "100 - pkts_lost",
        unit: '%'
      }
    ],
    sensor_types: [Sensor::FLOW],
    default: true
  )

  MonitorCategory.where(
    name: 'http_latency_flow'
  ).first_or_create(
    name: 'http_latency_flow',
    description: '',
    technology: 'system',
    category: 'http_latency',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'http_latency',
      system: "nice -n 19 httping -G -g 'https://%sensor_ip' -c 1 2>/dev/null|tail |grep round-trip|tr '/' ' '|awk '{print $7}'",
      unit: 'ms'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'channel_load_2_4'
  ).first_or_create(
    name: 'channel_load_2_4',
    description: '',
    technology: 'system',
    category: 'channel_load_2_4',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'channel_load_2_4',
      system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip 1.3.6.1.4.1.14179.2.2.13.1.3 | grep '\.0 = '|awk '{print $4}' | /usr/bin/jq -s 'add/length'",
      unit: 'ms'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'channel_load_5'
  ).first_or_create(
    name: 'channel_load_5',
    description: '',
    technology: 'system',
    category: 'channel_load_5',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'channel_load_5',
      system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip 1.3.6.1.4.1.14179.2.2.13.1.3 | grep '\.1 = '|awk '{print $4}' | /usr/bin/jq -s 'add/length'",
      unit: 'ms'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

  MonitorCategory.where(
    name: 'wireless_stations'
  ).first_or_create(
    name: 'wireless_stations',
    description: '',
    technology: 'system',
    category: 'wireless_stations',
    main_category: 'Network',
    impact: 4,
    operation: {
      name: 'wireless_stations',
      system: "echo $(( $(nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip 1.3.6.1.4.1.9.9.513.1.2.3.1.1|wc -l ) / 2 ))",
      unit: 'aps'
    },
    sensor_types: [Sensor::FLOW],
    default: false
  )

end

#redfish
po = PredefinedOperation.where(
  'name = ?',
  'Redfish Thermal'
).first_or_create(name: "Redfish Thermal", technology: 'redfish', operation: '{"name":"thermal","system":"/usr/lib/redborder/bin/rb_get_redfish.sh -t temperature_health","unit":"","name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"redfish_thermal-","integer":1}')

po = PredefinedOperation.where(
  'name = ?',
  'Redfish Power'
).first_or_create(name: "Redfish Power", technology: 'redfish', operation: '{"name":"power","system":"/usr/lib/redborder/bin/rb_get_redfish.sh -t power_health","unit":"","name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"redfish_power-","integer":1}')

po = PredefinedOperation.where(
  'name = ?',
  'Redfish Systems'
).first_or_create(name: "Redfish Systems", technology: 'redfish', operation: '{"name":"systems","system":"/usr/lib/redborder/bin/rb_get_redfish.sh -t system_health","unit":"","name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"redfish_systems-","integer":1}')

#ipmi
po = PredefinedOperation.where(
  'name = ?',
  'Fan'
).first_or_create(name: "Fan", technology: 'ipmi', operation: '{"name":"%tab_name","system":"sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Fan -a -s \'FAN[ ]*\'","unit":"rpm","name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"%tab_name-","integer":1}')

po = PredefinedOperation.where(
  'name = ?',
  'PCH Temperature'
).first_or_create(name: "PCH Temperature", technology: 'ipmi', operation: '{"name":"%tab_name","system":"sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s \'PCH Temp\'","unit":"celsius","integer":1,"name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"%tab_name-","integer":1}')

po = PredefinedOperation.where(
  'name = ?',
  'System Temperature'
).first_or_create(name: "System Temperature", technology: 'ipmi', operation: '{"name":"%tab_name","system":"sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s \'System Temp\'","unit":"celsius","integer":1,"name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"%tab_name-","integer":1}')

po = PredefinedOperation.where(
  'name = ?',
  'Peripheral Temperature'
).first_or_create(name: "Peripheral Temperature", technology: 'ipmi', operation: '{"name":"%tab_name","system":"sudo /usr/lib/redborder/bin/rb_get_sensor.sh -t Temperature -s \'Peripheral[ Temp]*\'","unit":"celsius","integer":1,"name_split_suffix":"_per_instance","split":";","split_op":"mean","instance_prefix":"%tab_name-","integer":1}')

# end of predefined operations

#Custom tabs

# CustomTab.where(
# 'name = ?',
# 'Memory Bytes'
# ).first_or_create(
# name: "Memory Bytes",
# category: "Custom",
# percentage: false,
# units: "Bytes"
# )

# CustomTab.where(
# 'name = ?',
# 'Memory Available'
# ).first_or_create(
# name: "Memory Available",
# category: "Custom",
# percentage: false,
# units: "Bytes"
# )

# CustomTab.where(
# 'name = ?',
# 'Disk Bytes'
# ).first_or_create(
# name: "Disk Bytes",
# category: "Custom",
# percentage: false,
# units: "Bytes"
# )

#Default Monitors for devices

MonitorCategory.where(
        name: 'CPU Load Centos 7/Ubuntu'
).first_or_create(
        name: 'CPU Load Centos 7/Ubuntu',
        description: 'Monitor the last minute load of a Centos 7/Ubuntu machine',
        technology: 'snmp',
        category: 'load_1',
        main_category: 'System',
        impact: 1,
        operation: {
                name: 'load_1',
                system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip .1.3.6.1.4.1.2021.10.1.3 | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}' | sed 's/ [A-Za-z]*$//' | awk '{print $NF}'",
                name_split_suffix: '_per_instance',
                unit: '%',
                split: ';',
                split_op: 'mean',
                instance_prefix: 'load_1-',
        },
        sensor_types: [Sensor::DEVICE],
        default: false
)


MonitorCategory.where(
        name: 'CPU Usage Centos 7/Ubuntu'
).first_or_create(
        name: 'CPU Usage Centos 7/Ubuntu',
        description: 'Monitor the CPU usage of a Centos 7/Ubuntu machine',
        technology: 'snmp',
        category: 'cpu',
        main_category: 'System',
        impact: 1,
        operation: {
                name: 'cpu',
                system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip .1.3.6.1.4.1.2021.11.9.0 | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}' | sed 's/ [A-Za-z]*$//' | awk '{print $NF}'",
                name_split_suffix: '_per_instance',
                unit: '%',
                split: ';',
                split_op: 'mean',
                instance_prefix: 'cpu-',
        },
        sensor_types: [Sensor::DEVICE],
        default: false
)

# MonitorCategory.where(
#         name: 'Available RAM Centos 7/Ubuntu'
# ).first_or_create(
#         name: 'Available RAM Centos 7/Ubuntu',
#         description: 'Monitor the available RAM in Centos 7/Ubuntu machine',
#         technology: 'snmp',
#         category: 'custom_memory_available',
#         main_category: 'System',
#         impact: 1,
#         operation: {
#                 name: 'custom_memory_available',
#                 system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip .1.3.6.1.4.1.2021.4.5.0 | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}' | sed 's/ [A-Za-z]*$//' | awk '{print $NF}'",
#                 name_split_suffix: '_per_instance',
#                 unit: '',
#                 split: ';',
#                 split_op: 'mean',
#                 instance_prefix: 'custom_memory_available-',
#         },
#         sensor_types: [Sensor::DEVICE],
#         default: false
# )

# MonitorCategory.where(
#         name: 'Free RAM Centos 7/Ubuntu'
# ).first_or_create(
#         name: 'Free RAM Centos 7/Ubuntu',
#         description: 'Monitor the free RAM in Centos 7/Ubuntu machine',
#         technology: 'snmp',
#         category: 'custom_memory_bytes',
#         main_category: 'System',
#         impact: 1,
#         operation: {
#                 name: 'custom_memory_bytes',
#                 system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip .1.3.6.1.4.1.2021.4.11 | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}' | sed 's/ [A-Za-z]*$//' | awk '{print $NF}'",
#                 name_split_suffix: '_per_instance',
#                 unit: '',
#                 split: ';',
#                 split_op: 'mean',
#                 instance_prefix: 'custom_memory_bytes-',
#         },
#         sensor_types: [Sensor::DEVICE],
#         default: false
# )
MonitorCategory.where(
        name: 'Monitor usage space on disks Centos 7/Ubuntu'
).first_or_create(
        name: 'Monitor usage space on disks Centos 7/Ubuntu',
        description: 'Monitor usage space on disks Centos 7/Ubuntu in Centos 7/Ubuntu machine',
        technology: 'snmp',
        category: 'disk',
        main_category: 'System',
        impact: 1,
        operation: {
                name: 'disk',
                system: "nice -n 19 snmpbulkwalk -v 2c -c %snmp_community %sensor_ip UCD-SNMP-MIB::dskPercent.1 | awk '/kB$/{ $(NF-1)= $(NF-1)*1024;} {print $0}' | sed 's/ [A-Za-z]*$//' | awk '{print $NF}'",
                name_split_suffix: '_per_instance',
                unit: '',
                split: ';',
                split_op: 'mean',
                instance_prefix: 'disk-',
        },
        sensor_types: [Sensor::DEVICE],
        default: false
)

manager_name = `hostname`.strip
manager_name = manager_name.split('.').first if manager_name.include?('.')

user = User.find_by(login: 'admin', admin: true) or User.first
Alarm.find_or_create_by(name: "Infrastructure: Manager #{manager_name} Disk usage is High 80 percent", condition: "{\"0\":{\"column\":\"sensor_name\",\"operator\":\"is\",\"value\":\"#{manager_name}\",\"enabled\":true}}", enabled: true, interval: 3600, product_type: "monitor", upper_limit: 80, time_window: 600, upper_limit_enabled: true, user_id: user.id, aggregation: "Average", monitor_tab: "disk", timezone: "UTC", multiplier_upper: 0, severity_id: 5, notify_to_syslog: false, notify_to_email: true,notify_to_slack: false, email_list: user.email)

Alarm.find_or_create_by(name: "Infrastructure: Manager #{manager_name} System Temperature is High (+50)", condition: "{\"0\":{\"column\":\"sensor_name\",\"operator\":\"is\",\"value\":\"#{manager_name}\",\"enabled\":true}}", enabled: true, interval: 3600, product_type: "monitor", upper_limit: 50, time_window: 600, upper_limit_enabled: true, user_id: user.id, timezone: "UTC", aggregation: "Average", monitor_tab: "system_temp", multiplier_upper: 0, severity_id: 5, notify_to_syslog: false, notify_to_email: true,notify_to_slack: false, email_list: user.email)
