###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

module RedBorder
  module AlarmMonitor

    def self.included(base)
      base.extend(ClassMethods)
      base.send(:include, InstanceMethods)

      base.class_eval do
        validates :monitor_tab,
                  inclusion: {
                    in: proc { SensorMonitor::TABS.to_a.map { |a| a[0].to_s } }
                  },
                  allow_blank: true
      end
    end

    module ClassMethods
    end

    module InstanceMethods
      def aggregation
        if product_type == 'monitor'
          'monitors'
        else
          super
        end
      end

      def units_monitor(_limit, _aggregation)
        case monitor_tab
        when 'cpu', 'disk', 'memory',
          'swap', 'patmatch_percent', 'snort_cpu_usr',
          'pkt_drop_percent' then "\%"
        when 'alerts_per_second' then 'a/s'
        when 'wire_mbits_per_sec_realtime' then 'Mbps'
        when 'pkt_stats_pkts_recv' then 'Pps'
        when 'pkt_stats_pkts_drop' then 'Dps'
        else ''
        end
      end
    end
  end
end

Alarm.send(:include, RedBorder::AlarmMonitor)
