#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    # Add querying methods for checking the monitor alarms
    #
    module CheckMonitorAlarmsJob

      def self.included(base)
        base.extend(ClassMethods)
        base.send(:include, InstanceMethods)

        base.class_eval do
          unloadable
          alias_method_chain :query, :monitor
        end
      end

      module ClassMethods
      end

      module InstanceMethods
        def query_with_monitor(data_source, start_time, end_time, filter, alarm)
          if alarm.product_type == 'monitor'
            data_source.where(
              query_type: :timeseries,
              range: :custom,
              start_time: start_time,
              end_time: end_time,
              granularity: :all,
              monitor_tab: alarm.monitor_tab,
              search: filter.get,
              aggregation: :monitors,
              user: alarm.user
            )
          else
            query_without_monitor(data_source, start_time, end_time, filter, alarm)
          end
        end
      end   # end module InstanceMethods
    end   # end class
  end   # end module Jobs
end   # end module Redborder
