###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

module RedBorder
  module SettingsControllerMonitor
    #
    # This function update the MonitorCategories. It's executed before
    # SettingsController#update.
    #
    def update
      settings = (params[:settings] || {}).dup.symbolize_keys
      # Save Monitor Default Tabs to DataBag
      databag = Chef::DataBagItem.new
      databag.data_bag Setting::DBNAME
      databag['id'] = 'monitors'
      monitor_tabs = settings.select { |k, v| k.match(/^monitor/) && v == '1' }
                             .keys.map { |k| k.to_s.sub('monitor_', '') }
      databag['monitors'] = monitor_tabs.flatten.uniq
      databag.save
      # Call SettingsController#update
      super
    end

    private

    #
    # Return monitor settings formatted to use in databag initialization
    #
    # == Returns::
    # Hash with monitor's names as keys and names of operations as values
    #
    def load_monitor_settings
      settings = {}
      MonitorCategory.all.each do |monitor|
        operations = monitor.operation.is_a?(Hash) ? [monitor.operation] :
                                                      monitor.operation
        names = operations.map { |o| o['name'] }
        settings[monitor.name] ||= []
        settings[monitor.name] += names
      end
      # Return the settings
      settings
    end
  end
end
