#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module WidgetMonitor

    def self.included(base)
      base.extend(ClassMethods)
      base.send(:include, InstanceMethods)

      base.class_eval do
        validate :monitor_value_has_sensor_name,
                 if: Proc.new { |w| w.product_type == 'monitor' && w.graph_type == 'value' }
      end
    end

    module ClassMethods
    end

    module InstanceMethods
      private
        def monitor_value_has_sensor_name
          current_filters = JSON.parse(filters || '{}')
          has_sensor_name = false

          current_filters.each do |_, filter|
            has_sensor_name = true if filter['column'] == 'sensor_name'
          end

          return if has_sensor_name
          errors.add(:graph_type,
                     "#{I18n.t 'lib_monitor.specify'}")
        end

        def print_filters_monitor
          to_return = ''

          source = Redborder::DruidSource.get(source: 'monitor')

          JSON.parse(filters).values.each do |v|
            to_return << "#{SensorMonitor::COLUMN_NAMES[v['column'].to_sym]} \
                          #{v['operator']} \
                          #{source.data_column(v['column'], v['value'])}<br />"
          end

          to_return
        end
    end
  end
end
