#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Provide an api to work with Scanner sensors
#
class Api::V1::SensorsScannerController < Api::V1::BaseSensorsController
  # ----------------------------------------------
  # Before and after actions
  # ----------------------------------------------
  skip_before_action :authenticate_user_from_token!, only: [:scans, :finish, :scan_status], raise: false
  skip_before_action :authenticate_user!, only: [:scans, :finish, :scan_status], raise: false
  skip_before_filter :verify_authenticity_token, only: [:scans, :finish, :scan_status], raise: false
  
  before_action :set_parent, only: :create
  before_action :manage_parent, only: :create
  before_action :set_sensor, only: [:scans, :scan_status, :finish]
  before_action :set_scan, only: [:scan_status, :finish]
  # ----------------------------------------------
  # Actions
  # ----------------------------------------------

  #
  # Create a new sensor scanner
  #
  def create
    @sensor = Sensor.new(params_scanner)
    @sensor.parent = @parent
    @sensor.type = Sensor::SCANNER

    # Check error when save
    return error_on_sensor(@sensor) unless @sensor.save
    # Wakeup chef and return
    wakeup_chef
    render json: {
      query: true,
      sensor: scanner_json
    }
  end

  #
  # Update the scan status to "finished"
  # 
  def finish
    @scan.finish! unless @scan.nil?
    render json: { query: true }
  end

  #
  # Update the scan status to "running"
  #
  def running
    @scan.running!
    render json: { query: true }
  end

  #
  # Update the scan status (Not in use for now)
  #
  # def scan_status
  #   return render json: { query: false, error: "Unknown status"}  unless @scan.valid_status? params[:status]
  #   return render json: { query: true }                               if @scan.update(:status => params[:status])
  #   return render json: { query: false, error: "Cannot update the status" }
  # end

  #
  # Return a JSON of all the scans with running status
  #
  def scans
    scans = []
    @sensor.running_and_cancelling_scans.each { |scan|
      scans.push( { 
                      scan_id: scan.id, 
                      target_addr: scan.profile.target_addr, 
                      target_port: scan.profile.target_port,
                      status: scan.status,
                      profile_type: scan.profile.profile_type
                })
    }

    render json: { query: true, scans: scans }
  end

  private

  # Params and renderization
  # ----------------------------------------------
  
  def params_scanner
    params.permit(*Sensor.import_columns(Sensor::SCANNER))
  end

  def scanner_json
    { id: @sensor.id, uuid: @sensor.uuid, name: @sensor.name, ip: @sensor.ip }
  end

  def set_sensor
    @sensor = Sensor.find_by(type: Sensor::SCANNER, :uuid => params[:sensor_uuid])
    return render json: { query: false, error: "Unknow sensor uuid"} if @sensor.nil?
  end
  
  def set_scan
    @scan = Scan.find_by(:id => params[:id])
    return render json: { query: false, error: "Unknown scan id"} if @scan.nil?
  end

end
