#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class NetworkMapController < ApplicationController
  include Redborder::Rule
  # Storage of the filters
  include Redborder::Filters::Storage::Session
  # Includes
  include Redborder::Rbdruid

  self.view_paths = "lib/modules/scanner/app/views"
  layout "#{Rails.root}/app/views/layouts/application"
  prepend_view_path "#{Rails.root}/app/views"

  before_action :set_source, only: [:index]
  before_action :set_scan, only: [:index]

  # Set network map data and links from the first finished scan
  def index
    @network_data, @network_links = @network_data, @network_links = [], []

    # Get user hostdiscovery profiles
    profiles = []
    sensors = current_user.sensor.children.where(type: Sensor::SCANNER)
    sensors.each { |s| profiles += s.profiles.select { |p| p.hostdiscovery_type? } }

    # Get finished scans from those profiles
    @scans = Scan.where(profile_id: profiles.map(&:id), status: :finished).order(updated_at: :desc)
    Rails.logger.info "[NetworkMapController] @scans is #{@scans}"
    # Scan of the network map, requested by params or the last updated_at one
    Rails.logger.info "[NetworkMapController] at this poing @scan is #{@scan}"

    @scan ||= @scans.first

    return unless @scan

    Rails.logger.info "[NetworkMapController] loading scan #{@scan.id}"
    Rails.logger.info "[NetworkMapController] @scan is #{@scan}"

    data_and_links = NetworkMap.load_data_and_links(@scan, @source)
    @network_data = data_and_links[:data] || []
    @network_links = data_and_links[:links] || []
  end

  private

  def set_scan
    @scan = Scan.find(params[:scan][:id]) if params[:scan] and params[:scan][:id]
  end

  def set_source
    @source = HostDiscovery
    @filter = Redborder::Filter.new(@source, load_filters)
  end
end #Class NetworkMapController
