#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

class ScansController < ApplicationController
    include Redborder::Rule
    # Storage of the filters
    include Redborder::Filters::Storage::Session
    # Includes
    include Redborder::Rbdruid

    self.view_paths = "lib/modules/scanner/app/views"
    layout "#{Rails.root}/app/views/layouts/application"
    prepend_view_path "#{Rails.root}/app/views"
  
    ## ======================================================================
    # => Before actions and filters
    ## ======================================================================
  
    skip_before_filter :authenticate_user!, only: :index, raise: false
  
    before_filter only: :index do |controller|
      if controller.request.format.text?
        validate_cert
      else
        authenticate_user!
      end
    end
  
    before_action :set_profile, :set_source, :set_sensor

  
    ## ======================================================================
    # => Actions
    ## ======================================================================
  
    # GET /rules
    # GET /rules.json
    def index
      @scans = Scan.where(profile_id: @profile.id)
      # @sensor = Sensor.where(id: @profile.sensor_id)
    end
  
    def run_scan
      @profile = Profile.find(params[:id])
      return 403 if @profile.nil?
      unless @profile.running?  #Because the button pressed can be a little bit late or double clicked
        @scan = Scan.new
        Rails.logger.debug("Scan is created with id: " + @scan.id.to_s)
        @scan.profile = @profile
        @scan.running!
      end
      redirect_to sensor_profiles_path
    end

    def cancel_scan
      running_scans = Scan.where(profile_id: params[:id], status: "running")
      for scan in running_scans do
        scan.cancelling!
        Rails.logger.debug("Cancelling scan with id: " + scan.id.to_s)
      end
      redirect_to sensor_profiles_path
    end

    def show
      @scan = Scan.find(params[:id])
      
      respond_to do |format|
        format.html { render layout: false }
        format.js
      end
    end

    def destroy
      @scan = Scan.find(params[:scan_id])
      @scan.destroy

      # Create schedule scan if user deletes its scan
      @profile.create_scan unless @profile.on_queue? or @profile.on_demand? or @profile.once?
      redirect_to sensor_profile_scans_path(@profile.owner, @profile.id),
                  notice: "Scan successfully deleted"
    end

    def filters

    end

    def import
      render layout: false
    end

    protected
  
    private

    def set_sensor
      @sensor = Sensor.find(params[:sensor_id]) if params[:sensor_id]
    end

    def set_profile
        if params[:profile_id]
            @profile = Profile.find(params[:profile_id])
        end
    end

    def set_source
      @source = Redborder::DruidSource.get(source: 'scanner')
      @filter = Redborder::Filter.new(@source, load_filters)
    end
  
  end
  