#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# HostDiscovery DataSource. It takes the data from nmap, traceroute..
# The initialization only create the read access to the vars.
#
class HostDiscovery < Redborder::DataSource
    COLUMNS = Redborder::Module.get :scanner, :columns_hostdiscovery
    SESSION_ID = Redborder::Module.get(:scanner, :session_id).to_sym
    AGGREGATIONS = Redborder::Module.get :scanner, :aggregations
  
    #
    # Create an attr_Reader for every columns.
    #
    COLUMNS.map{|k, v| v[:columns].keys + (v[:complementary_columns].try('keys') or [])}.flatten.each do |column|
      attr_reader column
    end
  
    #
    # Create the columns names
    #
    COLUMN_NAMES = { start_time: 'Start Time', 
                     end_time: 'End Time',
                     domain: 'Domain' }
    COLUMNS.each { |k, v| COLUMN_NAMES[k] = v[:columns][k] }
  
    AGGREGATIONS.each do |aggregation, _aggregation_data|
      attr_reader aggregation
    end
  
    attr_reader :timestamp, :result
  
    #
    # Create a HostDiscovery object with the data returned from Druid.
    #
    # == Parameters:
    # args::
    #   Hash with the data to build a Hostdiscovery model.
    #
    # == Returns:
    # Hashtag object
    #
    def initialize(_, args)
      if args.is_a?(Hash)
        args.each do |key, value|
          self.instance_variable_set(("@" + key).to_sym, value)
        end
  
        @timestamp = args["timestamp"].to_time.in_time_zone if args["timestamp"].present?
      end
  
      @src_country = get_country_name(@src_country_code)
  
      # Round the aggregations that must be rounded
      AGGREGATIONS.each do |aggregation, aggregation_data|
        current_value = instance_variable_get("@#{aggregation}")
        if aggregation_data[:round] && !current_value.nil?
          instance_variable_set("@#{aggregation}", current_value.round)
        end
      end
    end
  
    #
    # Return the Symbol identifies the module
    #
    def self.id
      :scanner
    end
  
    #
    # Return the Symbol of the session of module
    #
    def self.session
      SESSION_ID
    end
  
    #
    # Return the Symbol identifies the controller
    #
    def self.controller
      :hostdiscovery
    end
  
    #
    # Return the a hash with default action to redirect
    #
    def self.redirect
      { controller: 'scanner', action: 'tops' }
    end
  
    #
    # Return the Symbol identifies the controller
    #
    def self.default_aggregation
      :events
    end
  
    #
    # Return all scanner sensors
    #
    # == Parameters:
    # name::
    #   Array or String of names. If its nil, return all flow sensors.
    #
    def self.sensors(name = nil)
      return Sensor.where(type: RedBorder::SensorScanner::SCANNER,
                          is_deleted: false) if name.nil?
      Sensor.where(type: RedBorder::SensorScanner::SCANNER, name: name,
                   is_deleted: false)
    end
  
    # =================================================
    # FILTERS
    # =================================================
     
    #
    # Add the columns, zones and domains to build filter form
    # It's overrides the filter_to_json from filtrable
    #
    # == Parameters:
    # user::
    #   User who get filters
    # json::
    #   If true, return filters as JSON. True by default.
    #
    # == Returns:
    # A Hash with the basic operators and the domains, columns, sensors and zones
    # to build the filter form
    #
    def self.filter_to_json(user = User.current_user, json = true)
      domains = user
                .sensor
                .self_and_descendants
                .where.not(id: 1)
                .where(type: Sensor::DOMAIN)
  
      super.merge!(
        scanner_columns: COLUMN_NAMES.to_a.map do |c|
          { value: c[1],
            id: c[0],
            type: ([:start_time, :end_time].include?(c[0].to_sym) ? :datetime : :text_input) }
        end.compact.sort_by do |h|
          h[:value].to_s
        end,
        scanner_sensors: {
          type: :dropdown,
          value: user
            .sensor
            .self_and_descendants
            .where.not(id: 1)
            .where(type: Sensor::SCANNER)
            .map { |c| { value: c.name, id: c.name } }
        },
        domains: {
          type: :dropdown,
          value: domains.map { |c| { value: c.path_name, id: c.id } }
        }
      )
      # Check format
      json ? f.to_json : f
    end
  
    def self.set_user_privileges_filter(params)
    end
  end
