#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
class Scan < ApplicationRecord
  
  belongs_to :profile

  ##########################
  #       CONSTANTS
  ##########################
  FINISH_STATUS  = "finished"
  RUNNING_STATUS = "running"
  ON_QUEUE_STATUS = "on queue"
  CANCELLING_STATUS = "cancelling"
  #
  # Set status to finished and save it
  #
  def finish!
    self.status = FINISH_STATUS 
    self.save
    self.next_scan_iteration
  end

  def finished?
    status == FINISH_STATUS
  end

  def running! 
    self.status = RUNNING_STATUS 
    self.save
  end

  def running?
    status == RUNNING_STATUS
  end

  def on_queue!
    self.status = ON_QUEUE_STATUS
    self.save
  end

  def on_queue?
    status == ON_QUEUE_STATUS
  end

  def cancelling!
    return unless running?
    self.status = CANCELLING_STATUS
    self.save
  end

  def cancelling?
    status == CANCELLING_STATUS
  end

  #
  # If the profile has schedule, it creates the next iteration scan
  # When Profile[run_at] is equal to scan[run_at] is not a on demand scan and could be iterate
  #
  def next_scan_iteration
    profile = self.profile
    if profile.run_at == self.run_at
      unless profile.on_demand? or profile.once?
        profile.create_scan
        profile.save
      end
    end
  end

  #
  # Given a status return true if its valid or false if not
  #
  def valid_status?(status_param)
    [FINISH_STATUS, RUNNING_STATUS, ON_QUEUE_STATUS, CANCELLING_STATUS].include? status_param
  end

end
  