#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading Scanner module'

Redborder::Module.register :scanner do

  name 'Scanner'
  path 'scanner'
  session_id 'scanner'
  data_source 'Scanner'
  customizable_tabs true
  selectable_widget true
  cep_topic :rb_scanner_post

  menu :product_menu, :scanner, { controller: :scanner, action: :tops },
        caption: 'Scanner',
        html: { class: 'block-ui-but' }

  widgets [
  ]

  var_col = columns ({
    source:       { enabled: false, size: 1, columns: { source: "Source" }, category: 'General' },
    target:       { enabled: false, size: 1, columns: { target: "Target" }, category: 'General' },
    status:       { enabled: false, size: 1, columns: { status: "Status" }, category: 'General' },
    action:       { enabled: false, size: 1, columns: { action: "Action" }, category: 'General' },
    category:     { enabled: false, size: 1, columns: { category: "Category" }, category: 'General' },
    scan_id:      { enabled: false, size: 1, columns: { scan_id: "Scan ID" }, category: 'Scanner' },
    ipv4:         { enabled: true,  size: 1, columns: { ipv4: "IP Address", hostname: "Hostname", mac: "MAC", vendor: "Vendor" }, category: 'Scanner' },
    vendor:       { enabled: false, size: 1, columns: { vendor: "Vendor" }, category: 'Scanner' },
    scan_type:    { enabled: false, size: 1, columns: { scan_type: 'Scan Type'  }, category: 'Scanner', hide_global: true },
    servicename:  { enabled: false, size: 1, columns: { servicename: 'Service name' }, category: 'Service' },
    mac:          { enabled: false, size: 1, columns: { mac: 'Mac' }, category: 'Scanner' },
    subnet:       { enabled: false, size: 1, columns: { subnet: 'Subnet' }, category: 'Scanner' },
    cpe:          { enabled: false, size: 1, columns: { cpe: 'CPE' }, category: 'Scanner' },
    protocol:     { enabled: false, size: 1, columns: { protocol: 'Protocol' }, category: 'CVE' },
    port:         { enabled: true,  size: 1, columns: { port: 'Port', protocol: 'Protocol', port_state: 'Port State'  }, category: 'Service' },
    product:      { enabled: true,  size: 1, columns: { product: 'Service',  version: "Version", servicename: 'Service name', cpe: "CPE" }, category: 'Service' },
    cve:          { enabled: true,  size: 1, columns: { cve: 'Vulnerabilities', severity: 'Severity', metric: "metric", score: 'score'  }, category: 'CVE' },
    port_state:   { enabled: false, size: 1, columns: { port_state: 'Port State' }, category: 'Service' },
    severity:     { enabled: true,  size: 1, columns: { severity: 'Severity' }, category: 'CVE' },
    score:        { enabled: false, size: 1, columns: { score: 'Score' }, category: 'CVE' },
    metric:       { enabled: false, size: 1, columns: { metric: 'Metric' }, category: 'CVE' },
    version:      { enabled: false, size: 1, columns: { version: 'Version' }, category: 'Service' },
    hostname:     { enabled: false, size: 1, columns: { hostname: 'Hostname' }, category: 'RFC-5424' },
    sensor_name:  { enabled: false, size: 1, columns: { sensor_name: 'Sensor' }, category: 'Network' },
    fromhost_ip:  { enabled: false, size: 1, columns: { fromhost_ip: "SRC" }, category: 'Network' },
    os:           { enabled: false, size: 1, columns: { os: 'Operating System'}, category: "Network"},
    sensor_uuid:  { enabled: false, size: 1, columns: { sensor_uuid: 'Sensor UUID' }, category: 'UUIDs' },
    proxy_uuid:   { enabled: false, size: 1, columns: { proxy_uuid: 'Client Proxy' }, category: 'UUIDs' },

    # Location and UUIDs categories
    service_provider:      { enabled: false, size: 1, columns: { service_provider: "tr.scanner2.service" }, category: 'Location', hide_global: true },
    service_provider_uuid: { enabled: false, size: 1, columns: { service_provider_uuid: "tr.scanner2.service_u" }, category: 'UUIDs', hide_global: true },
    namespace:             { enabled: false, size: 1, columns: { namespace: "tr.scanner2.namespace" }, category: 'Location', hide_global: true },
    namespace_uuid:        { enabled: false, size: 1, columns: { namespace_uuid: "tr.scanner2.namespace_u" }, category: 'UUIDs', hide_global: true },
    deployment:            { enabled: false, size: 1, columns: { deployment: "tr.scanner2.deployment" }, category: 'Location', hide_global: true },
    deployment_uuid:       { enabled: false, size: 1, columns: { deployment_uuid: "tr.scanner2.deployment_u" }, category: 'UUIDs', hide_global: true },
    market:                { enabled: false, size: 1, columns: { market: "tr.scanner2.market" }, category: 'Location', hide_global: true },
    market_uuid:           { enabled: false, size: 1, columns: { market_uuid: "tr.scanner2.market_u" }, category: 'UUIDs', hide_global: true },
    organization:          { enabled: false, size: 1, columns: { organization: "tr.scanner2.org" }, category: 'Location', hide_global: true },
    organization_uuid:     { enabled: false, size: 1, columns: { organization_uuid: "tr.scanner2.org_u" }, category: 'UUIDs', hide_global: true },
    campus:                { enabled: false, size: 1, columns: { campus: 'Campus' }, category: 'Location', hide_global: true },
    campus_uuid:           { enabled: false, size: 1, columns: { campus_uuid: 'Campus UUID' }, category: 'UUIDs', hide_global: true },
    building:              { enabled: false, size: 1, columns: { building: "tr.scanner2.building" }, category: 'Location', hide_global: true },
    building_uuid:         { enabled: false, size: 1, columns: { building_uuid: "tr.scanner2.building_u" }, category: 'UUIDs', hide_global: true },
    floor:                 { enabled: false, size: 1, columns: { floor: "tr.scanner2.floor" }, category: 'Location', hide_global: true },
    floor_uuid:            { enabled: false, size: 1, columns: { floor_uuid: "tr.scanner2.floor_u" }, category: 'UUIDs', hide_global: true }
  })

  columns_hostdiscovery ({
    adjacency_matrix: {enabled: false, size: 1, columns: {adjacency_matrix: 'Adjacency matrix'}, category: 'General'},
    scan: {enabled: false, size: 1, columns: {scan: 'scan'}, category: 'General'},
    scan_id: {enabled: false, size: 1, columns: {scan_id: 'Scan id'}, category: 'General'},
    scan_type: {enabled: false, size: 1, columns: {scan_type: 'Scan type'}, category: 'General'},
    name: {enabled: false, size: 1, columns: {name: 'Name'}, category: 'General'},
    uuid: {enabled: false, size: 1, columns: {uuid: 'Uuid'}, category: 'General'},
    hosts: {enabled: false, size: 1, columns: {hosts: 'Hosts'}, category: 'General'},
    indexes: {enabled: false, size: 1, columns: {indexes: 'Indexes'}, category: 'General'},
    subnets: {enabled: false, size: 1, columns: {subnets: 'Subnets'}, category: 'General'},
    hostnames: {enabled: false, size: 1, columns: {hostnames: 'Hostnames'}, category: 'General'},
    macs: {enabled: false, size: 1, columns: {macs: 'Macs'}, category: 'General'},
    vendors: {enabled: false, size: 1, columns: {vendors: 'Vendors'}, category: 'General'},
    oss: {enabled: false, size: 1, columns: {oss: 'Oss'}, category: 'General'},
    sensor_name: {enabled: false, size: 1, columns: {sensor_name: 'Sensor name'}, category: 'General'},
    proxy_uuid: {enabled: false, size: 1, columns: {proxy_uuid: 'Proxy uuid'}, category: 'General'},
    sensor_uuid: {enabled: false, size: 1, columns: {sensor_uuid: 'Sensor uuid'}, category: 'General'},
    service_provider: {enabled: false, size: 1, columns: {service_provider: 'Service provider'}, category: 'General'},
    service_provider_uuid: {enabled: false, size: 1, columns: {service_provider_uuid: 'Service Provider uuid'}, category: 'General'}
  })


  aggregations ({
    events: {
      name: 'tr.scanner2.events', unit: 'events', value: :events, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      description: "tr.scanner2.number"
    }
  })

  menu :sensor_tree_menu, :new_scanner_sensor, { controller: :sensors, action: :new_scanner },
        caption: "<i class='sensor-menu-icon scanner'></i> Scanner".html_safe,
        html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal', 'data-toggle' => 'modal'}

  # Menu for Scanner sensors
  menu :scanner_menu, :edit, { controller: :sensors, action: :edit }, caption: 'Edit', first: true, html: {class: 'block-ui-but'}, admin: true
  menu :scanner_menu, :profiles,
              { controller: :profiles, action: :index },
                param: :sensor_id,
                caption: "Scanner Profiles",  #TODO: translate?
                html: { class: 'block-ui-but menu-divider' },
                admin: true,
                module: 'scanner'

  Redborder::MenuManager.map :domain_menu do |menu|
    menu.push :add_scanner_sensor, { controller: :sensors, action: :new_scanner },
              caption: 'Scanner',
              icon: 'sensor-menu-icon scanner',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'scanner'
  end

  Redborder::MenuManager.map :client_proxy_menu do |menu|
  menu.push :add_scanner_sensor, { controller: :sensors, action: :new_scanner },
              caption: 'Scanner',
              icon: 'sensor-menu-icon scanner',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              module: 'scanner'
  end

  #/scanner/network_map
  menu :module_views_menu, :network_map, { action: :load_network_map },
       caption: 'Network Map', html: { class: 'block-ui-but' },
       common: true, module: :scanner
  
  menu :sensors_view_menu, :scanner_view,
       { controller: :sensors, action: :scanner },
       caption: 'tr.scanner2.scanner_sensors',
       html: {class: 'block-ui-but'}


  menu :user_home_page_collection_menu, :Scanner, {}, caption: '/scanner'
end

# extend DRUID_CONFIG
druid_scanner_config = YAML.load(File.read("#{Rails.root}/lib/modules/scanner/config/rbdruid_config.yml"))[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_scanner_config

require_dependency 'widget_scanner'
require_dependency 'report_block_scanner'
require_dependency 'alarm_scanner'
require_dependency 'sensor_scanner'

DittoCode::Exec.if 'ENTERPRISE' do
require_dependency 'license_scanner'
end

# Require controllers
require_dependency 'widgets_controller_scanner'
require_dependency 'application_controller_scanner'
require_dependency 'sensors_controller_scanner'

# Require hooks
require_dependency 'hooks/alarms_scanner_hook'
require_dependency 'hooks/widget_data_scanner_hook'

# Require helpers
require_dependency 'widgets_helper_scanner'

to_prepare = proc do
  unless WidgetsController.include?(RedBorder::WidgetsControllerScanner)
    WidgetsController.send(:include, RedBorder::WidgetsControllerScanner)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerScanner)
    ApplicationController.send(:include, RedBorder::ApplicationControllerScanner)
  end
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperScanner)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperScanner)
  end
  unless SensorsController.include?(RedBorder::SensorsControllerScanner)
    SensorsController.send(:include, RedBorder::SensorsControllerScanner)
  end
end

Rails.configuration.to_prepare(&to_prepare)
