module NetworkMap
  include Redborder::Filters::Storage::Session
  attr_accessor :adjacency_matrix, :hosts, :path_list, :hosts_not_recorded

  def initialize
  end

  # Get the data from druid of a scan
  def self.load_data_and_links(scan, source)
    @source = source || HostDiscovery
    @start_time = Time.now - 2.month
    @end_time = Time.now

    options = {
      query_type: :select,
      dataSource: @source,
      start_time: @start_time,
      end_time: @end_time,
      dimensions: [
        "adjacency_matrix", "scan_id", "scan_type", "host", "index", "hostname",
        "subnet", "macs", "vendors", "oss", "sensor_name", "sensor_uuid",
      ],
      granularity: "all",
      filter: {
        type: "selector",
        dimension: "scan_id",
        value: scan.id.to_s,
      },
      pagingSpec: {
        pagingIdentifiers: {},
        threshold: 1000,
      },
    }

    filter = Redborder::Filter.new(@source, {})
    filter.add(:scan_id, scan.id)

    conditions = options.deep_dup

    user = options.fetch(:user, User.current_user)

    conditions = @source.defaults(conditions, user)
    conditions[:search] = filter.get
    result = @source.perform_query(conditions)

    path = set_path(result)
    html_map = mapping_results(result)

    { :data => html_map, :links => self.translate_path(path) }
  end

  def self.set_path(data)
    network_links = []
    @path_list ||= []  # inicializamos @path_list si no está definido previamente

    return [] unless data.is_a?(Array) && data.first.is_a?(Hash) && data.first.values.size > 1

    event = data.first["events"].first

    return [] if event.blank? || event.empty?

    # Continuación del procesamiento...
    hosts = event["event"]["host"]
    if event["event"]["adjacency_matrix"].is_a?(Array)
      adjacency_matrix = event["event"]["adjacency_matrix"].map { |row| row[1..-2].split(", ").map(&:to_i) }
    else
      adjacency_matrix = []
    end

    @path_list.push({
      timestamp: event["timestamp"],
      scan_id: event["scan_id"],
      scan_type: event["scan_type"],
      host: hosts,
      subnet: event["subnet"],
      macs: event["macs"],
      hostname: event["hostname"],
      vendors: event["vendors"],
      oss: event["oss"],
      sensor_name: event["sensor_name"],
      sensor_uuid: event["sensor_uuid"],
      index: event["index"],
      adjacency_matrix: adjacency_matrix,
    })

    if hosts && hosts.is_a?(Array)
      hosts.each_with_index do |source_host, row|
        chars = adjacency_matrix[row] || []  # Asegura que chars sea un Array
        chars.each_with_index do |char, col|
          if char == 1 # Comparamos con el entero 1
            target_host = hosts[col]
            network_links.push({
              id: "#{source_host}-#{target_host}",
              source: source_host,
              target: target_host,
            })
          end
        end
      end
    end

    network_links.uniq
  end

  def self.mapping_results(data)
    host_nodes = []
    identifier = 0

    return host_nodes unless data.is_a?(Array)

    event_data = data.find { |item| item["events"] && item["events"][0] && item["events"][0]["event"].present? }
    return host_nodes unless event_data

    event = event_data["events"][0]["event"]

    hosts = event["host"]
    return host_nodes unless hosts.is_a?(Array)

    macs = event["macs"] || []
    vendors = event["vendors"] || []
    oss = event["oss"] || []
    subnet = event["subnet"] || []
    hostname = event["hostname"] || []

    hosts.each_with_index do |host, index|
      host_index = {}
      host_index[:id] = identifier
      host_index[:name] = host
      host_index[:hostname] = hostname[index]
      host_index[:subnet] = subnet[index] || ""
      host_index[:mac] = macs[index] || ""
      host_index[:vendor] = vendors[index] || ""
      host_index[:oss] = oss[index] || ""

      host_nodes.push(host_index)
      identifier += 1
    end

    return host_nodes
  end

  def self.translate_path(path_list)
    translate_path = []
    return path_list unless path_list && @hosts

    path_list.each do |path|
      target = @hosts.index(path[:target])
      source = @hosts.index(path[:source])
      next unless target && source  # Esto omitirá el ciclo si target o source es nil

      index = target.to_s + "-" + source.to_s
      translate_path.push({ :id => index, :source => source, :target => target })
    end
    translate_path
  end

  def self.set_adjacency_matrix()
    matrix_index = fill_index()
    @adjacency_matrix = Array.new(@hosts.size) { Array.new(@hosts.size) }
    # Rails.logger.info "\n\nMatrix Index:"
    # Rails.logger.info matrix_index

    for i in 0...@hosts.size
      for j in 0...@hosts.size
        @adjacency_matrix[i][j] = 0
        if i == j
          @adjacency_matrix[i][j] = 1
        end
      end
    end

    @path_list.each do |path|
      path.each do |element|
        #Just ignore hosts that arent in the data retrieved list (for not deep scan option)
        if (matrix_index[element[0]] != nil and matrix_index[element[1]] != nil)
          index_source = matrix_index[element[0]]
          index_target = matrix_index[element[1]]
          @adjacency_matrix[index_source][index_target] = 1
          @adjacency_matrix[index_target][index_source] = 1
        end
      end
    end
  end

  # fill the adjacency matrix with 0
  def self.fill_index()
    matrix_index = {}
    position = 0
    # Rails.logger.info "\n\nHosts List:"
    # Rails.logger.info @hosts
    @hosts.each do |host|
      matrix_index[host] = position
      position += 1
    end
    matrix_index
  end

  def get_node_value(ip)
  end

  def self.set_hosts_list(data)
    @hosts = []
    data.each do |row|
      @hosts.push(row["ipv4"])
    end

    #needs to refactorize and put the path host
    @hosts_not_recorded = []
    @path_list.each do |path|
      path.each do |element|
        unless @hosts.include? element[0]
          @hosts.push(element[0])
          @hosts_not_recorded.push(element[0])
        end
        unless @hosts.include? element[1]
          @hosts.push(element[1])
          @hosts_not_recorded.push(element[1])
        end
      end
    end
  end

  def self.calculate_node_value(data)
    value = []
    for i in 0...@hosts.size
      path_found = 0
      for j in 0...@hosts.size
        if @adjacency_matrix[i][j].to_s == "1"
          path_found += 1
        end
      end
      value.push(path_found)
    end
    return value
  end
end # Module
