#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module ReportBlockScanner

    def self.included(base)
      base.extend(ClassMethods)
      base.send(:include, InstanceMethods)

      base.class_eval do
        validates :aggregation, inclusion: { in: Scanner::AGGREGATIONS.keys.map(&:to_s) }, allow_blank: true, if: proc { |w| w.product_type == 'scanner' }
        validate :required_params_scanner, if: proc { |w| w.product_type == 'scanner' }
        validate :cardinality_group_by_inclusion_scanner, if: proc { |w| w.product_type == 'scanner' }
      end
    end

    module ClassMethods
    end

    module InstanceMethods

      private

        def default_aggregation_scanner
          :bytes
        end

        def required_params_scanner
          make_validation = {
            report_block_type: report_block_type
          }

          if report_block_type == 'Raw'
            if graph_type == 'ball'
              make_validation[:ball_max] = ball_max
            elsif graph_type != 'value' && graph_type != 'streams'
              make_validation[:color] = color
            end

            make_validation[:range] = range
          elsif report_block_type == 'Top'
            make_validation[:range] = range

            unless graph_type == 'map'
              make_validation[:column] = column
              make_validation[:top_num] = top_num
            end

            if %w(stacked area line sbars bars).include? graph_type
              make_validation[:show_total] = show_total
              make_validation[:aggregation] = aggregation
            end
          elsif report_block_type == 'Cardinality'
            if graph_type == 'ball'
              make_validation[:ball_max] = ball_max
            elsif graph_type == 'value'
              make_validation[:reference] = reference
            elsif %w(stacked area line sbars bars).include? graph_type
              make_validation[:show_total] = show_total
            else
              make_validation[:cardinality_group_by] = cardinality_group_by
            end
            make_validation[:column] = column
            make_validation[:range] = range
          elsif report_block_type == 'Compare'
            make_validation[:compare_range] = compare_range
          end

          make_validation.each do |key, value|
            errors.add(key, "#{I18n.t('models.present2')}") if value.nil?
          end
        end

        def cardinality_group_by_inclusion_scanner
          allow_columns = ['none'] + Scanner::COLUMNS.to_a.map { |c| c[0].to_s unless c[1][:hide_raw] }
          unless allow_columns.include?(cardinality_group_by) || cardinality_group_by.nil?
            errors.add(:cardinality_group_by, 'Invalid value')
          end
        end

        def print_filters_scanner
          to_return = ''

          source = Redborder::DruidSource.get(source: 'scanner')

          JSON.parse(filters).each do |k, v|
            to_return << "#{Scanner::COLUMN_NAMES[v['column'].to_sym]} #{v['operator']} #{source.data_column(v['column'], v['value'])}<br />"
          end

          to_return
        end

        def step_scanner
          if granularity && Redborder::Rbdruid.granularity_list(start_time: get_regular_start_time, end_time: end_time).include?(granularity.to_sym)
            Redborder::Rbdruid::GRANULARITY_TYPE[granularity.to_sym][:step]
          else
            real_granularity = Redborder::Rbdruid.granularity_list(start_time: get_regular_start_time, end_time: end_time).first
            Redborder::Rbdruid::GRANULARITY_TYPE[real_granularity.to_sym][:step]
          end
        end

        def search_scanner
          parse_filter(filters, Scanner::COLUMN_NAMES.keys, :scanner)
        end

    end

  end
end

ReportBlock.send(:include, RedBorder::ReportBlockScanner)
