#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module RedBorder
  module WidgetsHelperScanner
    def self.included(base)
      base.send(:include, InstanceMethods)

      base.class_eval do
        unloadable

        def more_info_url_scanner(widget)
          product = "/" + widget.product_type + "/"

          section = if widget.widget_type
              case widget.widget_type.to_sym
              when :top then "tops/"
              when :unique, :grouped_unique, :single_unique then "cardinality/"
              when :compare then "comparative/"
              when :floor_map then return widget.floor_id ? view_sensor_path(widget.floor_id) : nil
              when :channel_load, :user_experience then "tops/"
              else ""
              end
            else
              ""
            end

          params = {}
          params[:range] = widget.range if widget.widget_type.to_sym != :compare
          params[:compare_range] = widget.range if widget.widget_type.to_sym == :compare
          params[:search] = widget.filters if widget.filters
          params[:granularity] = widget.granularity if widget.granularity and widget.widget_type.to_sym != :compare
          if widget.serie_type && [:stacked, :area, :line, :sbars, :bars].include?(widget.serie_type.to_sym)
            params[:chart_type] = widget.serie_type
          elsif widget.graph_type.to_sym == :stream
            params[:chart_type] = :streams
          end
          params[:group_by] = widget.group_by or "none" if (widget.widget_type.to_sym == :grouped_unique || widget.widget_type.to_sym == :single_unique)
          params[:aggregation] = widget.aggregation if (widget.widget_type.to_sym != :grouped_unique &&
                                                        widget.widget_type.to_sym != :single_unique &&
                                                        widget.widget_type.to_sym != :channel_load &&
                                                        widget.widget_type.to_sym != :user_experience)
          params[:stream_columns] = widget.stream_columns.delete_if(&:blank?).map(&:to_sym) if widget.graph_type.to_sym == :stream
          params[:tab] = if widget.graph_type.eql?("map") and widget.map_column
              widget.map_column
            elsif widget.column
              widget.column
            end
          params[:tab] = :client_rssi if widget.widget_type.to_sym == :channel_load ||
                                         widget.widget_type.to_sym == :user_experience

          return product + section + "?" + params.to_query
        end

        def event_rule_options(event)
          ac = ActionView::Base.new
          references_list = ["https://cve.mitre.org/cgi-bin/cvename.cgi?name=", "https://nvd.nist.gov/vuln/detail/"]
          references_name = ["MITRE", "NIST"]

          list = ac.content_tag(:li, class: "dropdown") do
            list_2 = ac.link_to "#", class: "dropdown-toggle", "data-toggle" => "dropdown" do
              ("#{I18n.t("ips_helper.references")}" + ac.content_tag(:b, nil, class: "caret")).html_safe
            end
            list_2 += ac.content_tag :ul, class: "dropdown-menu" do
              references = ""

              references_list.each_with_index do |reference, i|
                references += ac.content_tag(:li, ac.link_to(references_name[i], reference + event.cve, target: "_blank"))
              end

              references.html_safe
            end
            list_2.html_safe
          end
        end

        def get_redborder_manager_ip
          output = `ip -4 addr show dev bond0 | grep inet`
          output.scan(/\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3}/)
        end
      end
    end

    module InstanceMethods
    end
  end
end
