require 'test_helper'

class ScannerControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!
  include Devise::TestHelpers

  context 'super admin user' do
    setup do
      @admin = users(:admin)
      @root = sensors(:root)
      sign_in @admin
    end

    should 'not modify aggregation in session when getting a serie' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :get_serie, id: 'raw', granularity: 'hour', aggregation: 'events', w: true,
                      end_time: '1475668984', start_time: '1475625815', format: :json
      assert_not (session[:scanner][:aggregations] || []).include?(:events)
    end

    should 'modify aggregations when adding one' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :tops, aggregation: 'events', aggregation_action: 'add',
                 granularity: 'minute', range: 'last_1_hour'
      assert session[:scanner][:aggregations].include?(:events)
    end

    should 'modify aggregations when using replace parameter' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :index, aggregation: 'events', aggregation_action: 'replace', range: 'last_24',
                  stream_columns: %w(hostname message)
      assert_equal session[:scanner][:aggregations], [:events]
    end

    should 'modify dimensions when going from raw table widget' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :index, aggregation: 'events', aggregation_action: 'replace', range: 'last_24',
                  stream_columns: %w(hostname message)
      assert_equal session[:scanner][:active_columns], [:hostname, :message]
    end

    should 'get a comparative serie with a single aggregation' do
      start_time = Time.parse('2016-10-26 15:00:00 +0200')
      end_time = Time.parse('2016-10-26 16:00:00 +0200')

      Redborder::Rbdruid.expects(:query).with do |*args|
        args[0][:query_type] == :timeseries &&
          args[0][:start_time] == start_time &&
          args[0][:end_time] == end_time
      end.returns([]).at_least_once

      get :get_serie, comparative: 'true', diff: '158400.0',
                      end_time_serie: end_time.to_s,
                      start_time_serie: start_time.to_s,
                      value: '2016-10-26 15:00:00 +0200', id: 'pt24h',
                      granularity: 'all', format: :json
      assert_not ActiveSupport::JSON.decode(response.body).eql?(nil)
      assert_response :success
    end

    should 'get the comparative table with the aggregations selected' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      get :comparative
      assert_response :success
    end

    should 'select a slow query granularity' do
      Redborder::Rbdruid.expects(:query).returns([]).at_least_once

      Redborder::Rbdruid.expects(:query).with do |*args|
        args[0][:granularity] == 'pt2m'
      end.returns([]).at_least_once

      get :tops, granularity: 'pt2m', range: 'last_24'
    end

    should 'not select an unpermitted granularity' do
      default_granularity =
        Redborder::Rbdruid.granularity_list(start_time: 1.week.ago, end_time: Time.now).first

      Redborder::Rbdruid.expects(:query).returns([]).at_least_once
      Redborder::Rbdruid.expects(:query).with do |*args|
        args[0][:granularity] == default_granularity
      end.returns([]).at_least_once

      get :tops, granularity: 'pt2m', range: 'last_week'
    end
  end
end
