require 'test_helper'

#
# Test methods added from SensorScanner to Sensor
#
class SensorScannerTest < ActiveSupport::TestCase
  context 'A Scanner sensor' do
    setup do
      RedBorder.send(:remove_const, 'LICENSING_MODE')
      RedBorder::LICENSING_MODE = 'global'

      @scanner = sensors(:scanner)
      @deployment = sensors(:deployment)
    end

    should 'be created' do
      assert @scanner.valid?, @scanner.errors.full_messages
    end

    should 'not be created with invalid params' do
      name_not_valid = @scanner.name
      s_not_valid_name = Sensor.new(name: name_not_valid, type: @scanner.type, parent: @scanner.root)
      assert_not s_not_valid_name.valid?, 'Scanner sensor is not valid: duplicated name.'
    end

    should 'be created with the same name but different type or parent' do
      name_valid = @scanner.name
      s_valid = Sensor.new(name: name_valid,
                           type: @scanner.type,
                           parent: @deployment,
                           license: licenses(:two))
      assert s_valid.valid?, s_valid.errors.full_messages
    end

    should 'be moved into a market' do
      market = sensors(:market)

      @scanner.parent_id = market.id
      assert @scanner.valid?, @scanner.errors.full_messages
    end

    should 'not contain children' do
      market = sensors(:market)

      market.parent_id = @scanner.id
      assert_not market.valid?, 'Scanner sensor can\'t contain children'
    end
  end
end
