#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading Vault module'

Redborder::Module.register :vault do

  name 'Vault'
  path 'vault'
  session_id 'vault'
  data_source 'Vault'
  customizable_tabs true
  selectable_widget true
  cep_topic :rb_vault_post
  incident_source_name 'redBorder Vault'

  menu :product_menu, :vault, { controller: :vault, action: :tops },
        caption: 'Vault',
        html: { class: 'block-ui-but' }

  widgets [
  ]

  var_col = columns ({
    sensor_name:
    { enabled: true, size: 1,
      columns: { sensor_name: 'Sensor' }, category: 'Network' },
    message:
      { enabled: true, size: 1,
        columns: { message: "tr.vault2.message" }, category: 'RFC-5424' },
    source:
      { enabled: false, size: 1,
        columns: { source: "tr.vault2.source" }, category: 'General', observable_type: "ip" },
    target:
      { enabled: false, size: 1,
        columns: { target: "tr.vault2.target" }, category: 'General', observable_type: "ip" },
    status:
      { enabled: false, size: 1,
        columns: { status: "tr.vault2.status" }, category: 'General' },
    action:
      { enabled: false, size: 1,
        columns: { action: "tr.vault2.action" }, category: 'General' },
    event_info:
      { enabled: true, size: 1,
        columns: { event_info: "Event Info" }, category: 'General' },
    client_id:              
      { enabled: false, size: 1, 
        columns: { client_id: "tr.vault2.client_id" }, category: 'General' },
    category:
      { enabled: false, size: 1,
        columns: { category: "tr.vault2.category" }, category: 'General' },
    pri:
      { enabled: false, size: 1,
        columns: { pri: "tr.vault2.priority" }, category: 'RFC-5424' },
    pri_text:
      { enabled: true, size: 1,
        columns: { pri_text: "tr.vault2.priority_text" }, category: 'RFC-5424' },
    syslogfacility:
      { enabled: false, size: 1,
        columns: { syslogfacility: 'Facility' }, category: 'RFC-5424' },
    syslogfacility_text:
      { enabled: false, size: 1,
        columns: { syslogfacility_text: 'tr.vault2.facility_text' }, category: 'RFC-5424' },
    syslogseverity:
      { enabled: false, size: 1,
        columns: { syslogseverity: 'tr.vault2.severity' }, category: 'RFC-5424' },
    syslogseverity_text:
      { enabled: false, size: 1,
        columns: { syslogseverity_text: 'tr.vault2.severity_text' }, category: 'RFC-5424' },
    hostname:
      { enabled: true, size: 1,
        columns: { hostname: 'tr.vault2.hostname' }, category: 'RFC-5424', observable_type: "hostname" },
    app_name:
      { enabled: true, size: 1,
        columns: { app_name: "tr.vault2.app" }, category: 'RFC-5424' },
    fromhost_ip:
      { enabled: false, size: 1,
        columns: { fromhost_ip: "tr.vault2.src" }, category: 'Network', object_type: "NetObject", observable_type: "ip" },
    sensor_uuid:
      { enabled: false, size: 1,
        columns: { sensor_uuid: 'Sensor UUID' }, category: 'UUIDs' },
    proxy_uuid:
      { enabled: false, size: 1,
        columns: { proxy_uuid: 'tr.vault2.client_proxy' }, category: 'UUIDs' },
    incident_uuid:
      { enabled: false,
        size: 1, columns: { incident_uuid: 'Incident UUID' }, category: "UUIDs" },

    # Alarms
    alarm_id:              { enabled: false, size: 1, columns: { alarm_id: 'Alarm ID' }, category: "Alarms" },
    alarm_name:            { enabled: false, size: 1, columns: { alarm_name: 'Alarm Name' }, category: "Alarms" },
    alarm_condition:       { enabled: false, size: 1, columns: { alarm_condition: 'Alarm Condition' }, category: "Alarms" },
    alarm_product_type:    { enabled: false, size: 1, columns: { alarm_product_type: 'Alarm Product Type' }, category: "Alarms" },
    alarm_user:            { enabled: false, size: 1, columns: { alarm_user: 'Alarm User' }, category: "Alarms" },
    alarm_severity:        { enabled: false, size: 1, columns: { alarm_user: 'Alarm Severity' }, category: "Alarms" },

    # Location and UUIDs categories
    service_provider:      { enabled: false, size: 1, columns: { service_provider: "tr.vault2.service" }, category: 'Location', hide_global: true },
    service_provider_uuid: { enabled: false, size: 1, columns: { service_provider_uuid: "tr.vault2.service_u" }, category: 'UUIDs', hide_global: true },
    namespace:             { enabled: false, size: 1, columns: { namespace: "tr.vault2.namespace" }, category: 'Location', hide_global: true },
    namespace_uuid:        { enabled: false, size: 1, columns: { namespace_uuid: "tr.vault2.namespace_u" }, category: 'UUIDs', hide_global: true },
    deployment:            { enabled: false, size: 1, columns: { deployment: "tr.vault2.deployment" }, category: 'Location', hide_global: true },
    deployment_uuid:       { enabled: false, size: 1, columns: { deployment_uuid: "tr.vault2.deployment_u" }, category: 'UUIDs', hide_global: true },
    market:                { enabled: false, size: 1, columns: { market: "tr.vault2.market" }, category: 'Location', hide_global: true },
    market_uuid:           { enabled: false, size: 1, columns: { market_uuid: "tr.vault2.market_u" }, category: 'UUIDs', hide_global: true },
    organization:          { enabled: false, size: 1, columns: { organization: "tr.vault2.org" }, category: 'Location', hide_global: true },
    organization_uuid:     { enabled: false, size: 1, columns: { organization_uuid: "tr.vault2.org_u" }, category: 'UUIDs', hide_global: true },
    campus:                { enabled: false, size: 1, columns: { campus: 'Campus' }, category: 'Location', hide_global: true },
    campus_uuid:           { enabled: false, size: 1, columns: { campus_uuid: 'Campus UUID' }, category: 'UUIDs', hide_global: true },
    building:              { enabled: false, size: 1, columns: { building: "tr.vault2.building" }, category: 'Location', hide_global: true },
    building_uuid:         { enabled: false, size: 1, columns: { building_uuid: "tr.vault2.building_u" }, category: 'UUIDs', hide_global: true },
    floor:                 { enabled: false, size: 1, columns: { floor: "tr.vault2.floor" }, category: 'Location', hide_global: true },
    floor_uuid:            { enabled: false, size: 1, columns: { floor_uuid: "tr.vault2.floor_u" }, category: 'UUIDs', hide_global: true }
  })

  dimensions = {}
  Dir.glob('/opt/rb/var/rb-extensions/*/dimensions.yml') do |item|
     dimensions = dimensions.merge(YAML.load_file(item)) rescue dimensions
  end

  dimensions.each do | key, dimension |
   var_col.merge!(key.to_sym=>{:enabled=>false, :size=>1, :columns=>{key.to_sym=>"#{dimension['name'].capitalize}"}, :category=>"#{dimension['category'].capitalize}"}) if !var_col.include? key
  end

  aggregations ({
    events: {
      name: 'tr.vault2.events', unit: 'events', value: :events, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      description: "tr.vault2.number"
    }
  })

  menu :sensor_tree_menu, :new_vault_sensor, { controller: :sensors, action: :new_vault },
        caption: "<i class='sensor-menu-icon vault'></i> Vault".html_safe,
        html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal', 'data-toggle' => 'modal'}

  # Menu for Vault sensors
  menu :vault_menu, :edit, { controller: :sensors, action: :edit }, caption: 'Edit', first: true, html: {class: 'block-ui-but'}, admin: true

  Redborder::MenuManager.map :domain_menu do |menu|
    menu.push :add_vault_sensor, { controller: :sensors, action: :new_vault },
              caption: 'Vault',
              icon: 'sensor-menu-icon vault',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING,
                                   Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                                 (RedBorder::LICENSING_MODE == 'global' &&
                                  [Sensor::ORGANIZATION, Sensor::MARKET,
                                   Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                   Sensor::NAMESPACE].include?(s.domain_type))
                           },
              module: 'vault'
  end

  Redborder::MenuManager.map :client_proxy_menu do |menu|
  menu.push :add_vault_sensor, { controller: :sensors, action: :new_vault },
              caption: 'Vault',
              icon: 'sensor-menu-icon vault',
              parent: :add_sensor,
              html: {'data-backdrop' => 'static', 'data-target' => '#new_sensor_modal',
                     'data-toggle' => 'modal'},
              admin: true,
              module: 'vault'
  end

  menu :sensors_view_menu, :vault_view,
       { controller: :sensors, action: :vault },
       caption: 'tr.vault2.vault_sensors',
       html: {class: 'block-ui-but'}


  menu :user_home_page_collection_menu, :Vault, {}, caption: '/vault'
end

# extend DRUID_CONFIG
druid_vault_config = YAML.load(File.read("#{Rails.root}/lib/modules/vault/config/rbdruid_config.yml"))[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_vault_config

require_dependency 'widget_vault'
require_dependency 'report_block_vault'
require_dependency 'alarm_vault'
require_dependency 'sensor_vault'

DittoCode::Exec.if 'ENTERPRISE' do
require_dependency 'license_vault'
end

# Require controllers
require_dependency 'widgets_controller_vault'
require_dependency 'application_controller_vault'
require_dependency 'sensors_controller_vault'

# Require hooks
require_dependency 'hooks/alarms_vault_hook'
require_dependency 'hooks/widget_data_vault_hook'

# Require helpers
require_dependency 'widgets_helper_vault'

to_prepare = proc do
  unless WidgetsController.include?(RedBorder::WidgetsControllerVault)
    WidgetsController.send(:include, RedBorder::WidgetsControllerVault)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerVault)
    ApplicationController.send(:include, RedBorder::ApplicationControllerVault)
  end
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperVault)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperVault)
  end
  unless SensorsController.include?(RedBorder::SensorsControllerVault)
    SensorsController.send(:include, RedBorder::SensorsControllerVault)
  end
end

Rails.configuration.to_prepare(&to_prepare)
