require 'test_helper'

#
# Test methods added from SensorVault to Sensor
#
class SensorVaultTest < ActiveSupport::TestCase
  context 'A Vault sensor' do
    setup do
      RedBorder.send(:remove_const, 'LICENSING_MODE')
      RedBorder::LICENSING_MODE = 'global'

      @vault = sensors(:vault)
      @deployment = sensors(:deployment)
    end

    should 'be created' do
      assert @vault.valid?, @vault.errors.full_messages
    end

    should 'not be created with invalid params' do
      name_not_valid = @vault.name
      s_not_valid_name = Sensor.new(name: name_not_valid, type: @vault.type, parent: @vault.root)
      assert_not s_not_valid_name.valid?, 'Vault sensor is not valid: duplicated name.'

      ip_not_valid = @vault.ip
      s_not_valid_ip = Sensor.new(name: 'Vault not valid', type: @vault.type, parent: @vault.root, ip: ip_not_valid)
      assert_not s_not_valid_ip.valid?, 'Vault sensor is not valid: duplicated ip.'
    end

    should 'be created with the same name but different type or parent' do
      name_valid = @vault.name
      s_valid = Sensor.new(name: name_valid,
                           type: @vault.type,
                           parent: @deployment,
                           ip: '192.168.104.18',
                           license: licenses(:two))
      assert s_valid.valid?, s_valid.errors.full_messages
    end

    should 'be moved into a market' do
      market = sensors(:market)

      @vault.parent_id = market.id
      assert @vault.valid?, @vault.errors.full_messages
    end

    should 'not contain children' do
      market = sensors(:market)

      market.parent_id = @vault.id
      assert_not market.valid?, 'Vault sensor can\'t contain children'
    end

    should 'not be created with the same IP and parent' do
      sensor = Sensor.new(name: 'sensor_vault',
                          type: @vault.type,
                          parent: @vault.parent,
                          ip: @vault.ip)

      assert_not sensor.valid?
    end
  end
end
