#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

Rails.logger.info '[RB] Loading Wireless module'

Redborder::Module.register :wireless do

  name 'Wireless'
  path 'wireless'
  session_id 'wireless'
  data_source 'Wireless'
  customizable_tabs true
  selectable_widget true
  cep_topic :rb_wireless_post

  menu :product_menu, :wireless, { controller: :wireless, action: :tops },
        caption: 'Wireless',
        html: { class: 'block-ui-but' }

  widgets [
  ]

  var_col = columns ({
    sensor_name:
      { enabled: true, size: 1,
        columns: { sensor_name: 'tr.wireless2.sensor' }, category: 'General' },
    sensor_uuid:
      { enabled: false, size: 1,
        columns: { sensor_uuid: 'tr.wireless2.sensor_uuid' }, category: 'UUIDs' },
    client_profile:
      { enabled: true, size: 1,
        columns: { client_profile: 'tr.wireless2.client_profile' }, category: 'General' },
    client_mac:
       { enabled: true, size: 1, 
         columns: { client_mac: 'tr.wireless2.client_mac' }, complementary_columns: { client_mac_name: 'tr.wireless2.client_name', client_mac_name_two: 'tr.wireless2.vendor' }, name: 'Client MAC', object_type: 'MacObject', category: 'General' },
    client_mac_vendor:
       { enabled: false, size: 1, 
         columns: { client_mac_vendor: "tr.wireless2.client_mac_vendor" }, category: 'General' },
    client_latlong:
      { enabled: false, size: 1, 
        columns: { client_latlong: 'tr.wireless2.coordinates' }, category: 'General' },
    coordinates_map:
      { enabled: false, size: 1, 
      columns: { client_latlong: 'tr.wireless2.coordinates_map' }, hide_raw: true, category: 'Location' },
    wireless_station:
      { enabled: false, size: 1, 
        columns: { wireless_station: "tr.wireless2.wireless_station" }, complementary_columns: { wireless_station_name: 'tr.wireless2.wireless_station_name' }, category: 'General' },
    client_rssi:
      { enabled: false, size: 1, 
        columns: { client_rssi: "tr.wireless2.client_rssi" }, complementary_columns: { client_rssi_num: 'tr.wireless2.value' }, category: 'General' },
    dot11_status:
      { enabled: false, size: 1,
        columns: { dot11_status: "tr.wireless2.dot11_status" }, category: 'General' },

    # Location and UUIDs categories
    service_provider:      { enabled: false, size: 1, columns: { service_provider: "tr.wireless2.service" }, category: 'Location', hide_global: true },
    service_provider_uuid: { enabled: false, size: 1, columns: { service_provider_uuid: "tr.wireless2.service_u" }, category: 'UUIDs', hide_global: true },
    namespace:             { enabled: false, size: 1, columns: { namespace: "tr.wireless2.namespace" }, category: 'Location', hide_global: true },
    namespace_uuid:        { enabled: false, size: 1, columns: { namespace_uuid: "tr.wireless2.namespace_u" }, category: 'UUIDs', hide_global: true },
    deployment:            { enabled: false, size: 1, columns: { deployment: "tr.wireless2.deployment" }, category: 'Location', hide_global: true },
    deployment_uuid:       { enabled: false, size: 1, columns: { deployment_uuid: "tr.wireless2.deployment_u" }, category: 'UUIDs', hide_global: true },
    market:                { enabled: false, size: 1, columns: { market: "tr.wireless2.market" }, category: 'Location', hide_global: true },
    market_uuid:           { enabled: false, size: 1, columns: { market_uuid: "tr.wireless2.market_u" }, category: 'UUIDs', hide_global: true },
    organization:          { enabled: false, size: 1, columns: { organization: "tr.wireless2.org" }, category: 'Location', hide_global: true },
    organization_uuid:     { enabled: false, size: 1, columns: { organization_uuid: "tr.wireless2.org_u" }, category: 'UUIDs', hide_global: true },
    campus:                { enabled: false, size: 1, columns: { campus: 'Campus' }, category: 'Location', hide_global: true },
    campus_uuid:           { enabled: false, size: 1, columns: { campus_uuid: 'Campus UUID' }, category: 'UUIDs', hide_global: true },
    building:              { enabled: false, size: 1, columns: { building: "tr.wireless2.building" }, category: 'Location', hide_global: true },
    building_uuid:         { enabled: false, size: 1, columns: { building_uuid: "tr.wireless2.building_u" }, category: 'UUIDs', hide_global: true },
    floor:                 { enabled: false, size: 1, columns: { floor: "tr.wireless2.floor" }, category: 'Location', hide_global: true },
    floor_uuid:            { enabled: false, size: 1, columns: { floor_uuid: "tr.wireless2.floor_u" }, category: 'UUIDs', hide_global: true },
    zone:                  { enabled: false, size: 1, columns: { zone: 'Zone' }, category: 'Location' }
  })

  aggregations ({
    clients: {
      name: 'tr.wireless2.visitors', unit: 'visitors', value: :clients, size: 1, round: true,
      aggregations: [{ type: 'hyperUnique', name: 'clients', fieldName: 'clients' }],
      description: "tr.wireless2.visitors_description"
    },
    events: {
      name: 'tr.wireless2.events', unit: 'events', value: :events, size: 1,
      aggregations: [
        { type: 'longSum', name: 'events', fieldName: 'events' }
      ],
      description: "tr.wireless2.events_description"
    }
  })


  menu :user_home_page_collection_menu, :Wireless, {}, caption: '/wireless'
end

# extend DRUID_CONFIG
druid_wireless_config = YAML.load(File.read("#{Rails.root}/lib/modules/wireless/config/rbdruid_config.yml"))[Rails.env].symbolize_keys
RedBorder::DRUID_CONFIG.merge! druid_wireless_config

require_dependency 'widget_wireless'
require_dependency 'report_block_wireless'
require_dependency 'alarm_wireless'

# Require controllers
require_dependency 'widgets_controller_wireless'
require_dependency 'application_controller_wireless'

# Require hooks
require_dependency 'hooks/alarms_wireless_hook'
require_dependency 'hooks/widget_data_wireless_hook'

# Require helpers
require_dependency 'widgets_helper_wireless'

to_prepare = proc do
  unless WidgetsController.include?(RedBorder::WidgetsControllerWireless)
    WidgetsController.send(:include, RedBorder::WidgetsControllerWireless)
  end
  unless ApplicationController.include?(RedBorder::ApplicationControllerWireless)
    ApplicationController.send(:include, RedBorder::ApplicationControllerWireless)
  end
  unless WidgetsHelper.include?(RedBorder::WidgetsHelperWireless)
    WidgetsHelper.send(:include, RedBorder::WidgetsHelperWireless)
  end
end

Rails.configuration.to_prepare(&to_prepare)
