#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

RedBorder::Application.routes.draw do
  resources :sensors, except: :show do
    get :wireless, on: :collection
    get :wireless, on: :member
    get :new_wireless, on: :collection
    post :create_wireless, on: :collection
    get 'wireless/:id/search',
        action: :wireless_letter_search, on: :collection, as: :wireless_letter_search
    get 'wireless/find',
        action: :wireless_search, on: :collection, as: :wireless_search
    get 'wireless/load_more', action: :wireless_load_more, on: :collection
  end
  resources :wireless, path: '/wireless' do
    member do
      get :load_tab
      get :load_tab_cardinality
      get :load_cardinality_table
    end
    collection do
      get :tops
      get :comparative
      # get :cardinality
      patch :change_session_tab
      patch :change_session_total
      get :info
      get :help
      get :load_more
      get :load_more_tops
      get :load_more_comparative
      get :load_more_cardinality
      get :load_new_data
      get :search
      post :results
      get :element_childs
      get :export_tops_to_csv
      get :export_cardinality_to_csv
      get :export_raw_to_csv
      get :sort_columns
      get :sort_aggregations
      get 'aggregations/reorder',
          to: 'wireless#reorder_aggregations', as: 'reorder_wireless_aggregations'
      patch 'change_chart_type/:id',
            action: :change_chart_type, as: :change_chart_type
      get 'tops/add_tab/:id',
          action: :add_tab, as: :add_wireless_top_tab
      get 'cardinality/add_tab/:id',
          action: :add_cardinality_tab, as: :add_wireless_cardinality_tab
      get 'cardinality/get_serie/:id',
          action: :get_cardinality_serie, as: :get_wireless_cardinality_serie
      patch 'tops/del_tab/:id',
            action: :del_tab, as: :del_wireless_top_tab
      patch 'cardinality/del_tab/:id',
            action: :del_cardinality_tab, as: :del_wireless_cardinality_tab
      get 'tops/get_serie/:id',
          action: :get_serie, as: :get_wireless_serie
      get 'tops/get_tops/:id',
          action: :get_tops, as: :get_wireless_tops
      get 'tops/get_raw_table',
          action: :get_raw_table, as: :get_wireless_raw_table
      post 'export_tops_to_file',
           action: :export_tops_to_file, as: :export_tops_to_file
      post 'export_cardinality_to_file',
           action: :export_cardinality_to_file, as: :export_cardinality_to_file
      post 'export_raw_to_file',
           action: :export_raw_to_file, as: :export_raw_to_file
      get 'get_outliers/:id',
          action: :get_outliers, as: :get_wireless_outliers
    end
  end

  patch 'saved_wireless_filter/:id/add',
        to: 'saved_wireless_filters#add_saved_filter',
        as: 'add_saved_wireless_filter'
  post 'saved_wireless_filters/create',
       to: 'saved_wireless_filters#create'
  get 'tabs/reorder_wireless',
      to: 'wireless#reorder_tab', as: 'reorder_wireless_tab'

  resources :saved_wireless_filters, path: '/saved/wireless_filters' do
    collection do
      post :title
      post :update
    end

    member do
      get :view
      post :update
    end
  end

  namespace :api do
    namespace :v1 do
      resources :sensors_wireless, only: [], param: :sensor_uuid, path: 'sensors' do
        # Create
        post :wireless, on: :collection, action: :create
      end
    end
  end
end
