#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Rbsearch

  #
  # Search Rule
  #
	module SearchRule

		COLUMN = {
			factory_policy_id: 'factory_policy_id',
			category_id: 'category_id',
			group_id: 'group_id',
			family_id: 'family_id',
			flowbit_id: 'flowbit_id',
			flowbit_state_id: 'flowbit_state_id',
			protocol: 'protocol',
			severity: 'severity',
			rule_source_id: 'rule_source_id',
			action_id: 'action_id',
			class_type_name: 'class_type_name',
			reference: 'reference',
			is_modified: 'is_modified',
      default_enabled: 'default_enabled',
			is_new: 'is_new',
			is_deprecated: 'is_deprecated',
			rule_sid: 'rule_sid',
			rule_gid: 'rule_gid',
      msg: 'msg'

		}

		OPERATOR = {
			is: "= ?",
			not: "!= ?",
			contains: "LIKE ?",
			contains_not: 'NOT LIKE ?',
      starts_with: "LIKE ?",
      ends_with: "LIKE ?",
			gte: ">= ?",
			lte: "<= ?",
			lt: "< ?",
			gt: "> ?",
			in: "IN",
			not_in: "NOT IN (?)",
			notnull: "NOT NULL ?",
			isnull: "IS NULL",
			between: "BETWEEN ? AND ?"
		}

		COLUMN_NAME = {
			factory_policy_id: "Factory Policy",
			category_id: "Category",
			group_id: "Group",
			family_id: "Family",
			flowbit_id: "Flowbit",
			flowbit_state_id: "Flowbit State",
			protocol: "Protocol",
			severity: "Severity",
			rule_source_id: "Source",
			action_id: "Action State",
			class_type_name: "ClassType",
			reference: "Reference",
			is_modified: "Modified Rule",
			is_new: "New Rule",
			is_deprecated: "Deprecated Rule",
      default_enabled: 'Default Enabled',
			rule_sid: "Signature ID",
			rule_gid: "Generator ID",
      msg: "Signature"
		}

		def self.json
			@factory_policies = FactoryPolicy.select(:name, :id).order(:name)
			@categories 			= Category.select(:name, :id).order(:name)
			@groups 					= Group.select(:name, :id).order(:name)
			@families 				= Family.select(:name, :id).order(:name)
			@flowbits 				= Flowbit.select(:name, :id).order(:name)
			@flowbit_states 	= FlowbitState.select(:name, :id).order(:name)
			@protocols 				= Rule.protocols
			@severities 			= Severity.select(:name, :id).order(:name)
			@sources 					= RuleSource.select(:name, :id).order(:name)
			@actions 					= Action.select(:name, :id).where(selectable: true).where.not(name: 'clear').order(:name)
			@classtypes 			= ClassType.select(:name).group(:name).order(:name)

			{
				operators: {
          selector_input: [
            {id: :is, value: "is"},
            {id: :not, value: "is not"}
          ],
					text_input: [
						{id: :contains, value: "contains"},
						{id: :contains_not, value: "not contains"},
            {id: :starts_with, value: "starts with"},
            {id: :ends_with, value: "ends with"}
					],
					boolean_input: [
            {id: :is, value: "is"}
          ]
				},
				columns: [
					{value: "Factory Policy", id: :factory_policy_id, type: :select},
					{value: "Category", id: :category_id, type: :select},
					{value: "Group", id: :group_id, type: :select},
					{value: "Family", id: :family_id, type: :select},
					{value: 'Flowbit', id: :flowbit_id, type: :select},
					{value: 'Flowbit State', id: :flowbit_state_id, type: :select},
					{value: 'Protocol', id: :protocol, type: :select},
					{value: 'Severity', id: :severity, type: :select},
					{value: 'Source', id: :rule_source_id, type: :select},
					{value: 'Action State', id: :action_id, type: :select},
					{value: 'ClassType', id: :class_type_name, type: :select},
					{value: 'Reference', id: :reference, type: :text_input},
					{value: 'Modified Rule', id: :is_modified, type: :select},
					{value: 'New Rule', id: :is_new, type: :select},
					{value: 'Deprecated Rule', id: :is_deprecated, type: :select},
          {value: 'Default Enabled', id: :default_enabled, type: :select},
					{value: "Signature ID", id: :rule_sid, type: :text_input},
					{value: "Generator ID", id: :rule_gid, type: :text_input},
          {value: "Signature", id: :msg, type: :text_input}
				],
				factory_policy_id: {
					type: :dropdown,
					value: @factory_policies.collect do |x|
						{id: x.id, value: x.name}
					end
        },
        category_id: {
					type: :dropdown,
					value: @categories.collect do |x|
						{id: x.id, value: x.name}
					end
        },
        group_id: {
					type: :dropdown,
					value: @groups.collect do |x|
						{id: x.id, value: x.name}
					end
        },
        family_id: {
					type: :dropdown,
					value: @families.collect do |x|
						{id: x.id, value: x.name}
					end
        },
        flowbit_id: {
          type: :dropdown,
          value: @flowbits.collect do |x|
            {id: x.id, value: x.name}
          end
        },
        flowbit_state_id: {
          type: :dropdown,
          value: @flowbit_states.collect do |x|
            {id: x.id, value: x.name}
          end
        },
        protocol: {
        	type: :dropdown,
        	value: @protocols.collect do |x|
        		{id: x, value: x.nil? ? "None" : x.upcase}
        	end
        },
        severity: {
          type: :dropdown,
          value: @severities.collect do |x|
            {id: x.id, value: x.name}
          end
        },
        rule_source_id: {
          type: :dropdown,
          value: @sources.collect do |x|
            {id: x.id, value: x.name}
          end
        },
        action_id: {
          type: :dropdown,
          value: @actions.collect do |x|
            {id: x.id, value: x.name}
          end
        },
        class_type_name: {
          type: :dropdown,
          value: @classtypes.collect do |x|
            {id: x.name, value: x.name}
          end
        },
        is_modified: {
        	type: :dropdown,
        	value: [{id: 1, value: "Yes"}, {id: 0, value: "No"}]
        },
        is_new: {
        	type: :dropdown,
        	value: [{id: 1, value: "Yes"}, {id: 0, value: "No"}]
        },
        is_deprecated: {
        	type: :dropdown,
        	value: [{id: 1, value: "Yes"}, {id: 0, value: "No"}]
        },
        default_enabled: {
          type: :dropdown,
          value: [{id: 1, value: "Yes"}, {id: 0, value: "No"}]
        }
			}.to_json.html_safe

		end

		def self.data_column(column, value)

			case column.to_sym
			when :factory_policy_id
				FactoryPolicy.find(value.to_i).name
			when :category_id
				Category.find(value.to_i).name
			when :group_id
				Group.find(value.to_i).name
			when :family_id
				Family.find(value.to_i).name
			when :flowbit_id
				Flowbit.find(value.to_i).name
			when :flowbit_state_id
				FlowbitState.find(value.to_i).name
			when :protocol
				value.present? ? value : "None"
			when :severity
				Severity.find(value.to_i).name
			when :rule_source_id
				RuleSource.find(value.to_i).name
			when :action_id
				Action.find(value.to_i).name
			when :is_deprecated, :is_new, :is_modified, :default_enabled
				value.to_i.zero? ? "No" : "Yes"
			else
				value
			end

		end

	end

end