###############################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
###############################################################################

require 'redborder/menu_manager'

Redborder::MenuManager.map :domain_menu do |menu|
  menu.push :edit, { controller: 'sensors', action: 'edit' },
            caption: 'tr.redborder2.edit',
            first: true,
            html: { class: 'block-ui-but' },
            admin: true

  menu.push :overview, { controller: 'dashboards', action: 'overview' },
            caption: 'tr.redborder2.overview', after: :edit, admin: true,
            if: proc { |s| s.is_domain? }

  menu.push :view, '#', caption: 'tr.redborder2.view', after: :overview

  menu.push :add_domain, '#',
            caption: 'tr.redborder2.add', after: :view, admin: true,
            if: proc { |s| s.is_domain? && s.can_have_children? }

  menu.push :add_namespace, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::NAMESPACE },
            caption: 'tr.redborder2.namespace', parent: :add_domain, super_admin: true,
            icon: 'fa fa-database fixed-icon',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.service_provider? }

  menu.push :add_organization, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::ORGANIZATION },
            caption: 'tr.redborder2.organization', parent: :add_domain, super_admin: true,
            icon: 'fa fa-group fixed-icon',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.namespace? }

  menu.push :add_market, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::MARKET },
            caption: 'tr.redborder2.market', parent: :add_domain, admin: true,
            icon: 'fa fa-globe fixed-icon',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.is_domain? && [Sensor::ORGANIZATION, Sensor::DEPLOYMENT, Sensor::CAMPUS, Sensor::BUILDING].include?(s.domain_type) }

  menu.push :add_deployment, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::DEPLOYMENT },
            caption: 'tr.redborder2.deployment', parent: :add_domain, admin: true,
            icon: 'fa fa-cube fixed-icon',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.is_domain? && [Sensor::ORGANIZATION, Sensor::MARKET, Sensor::CAMPUS, Sensor::BUILDING].include?(s.domain_type) }

  menu.push :add_campus, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::CAMPUS },
            caption: 'tr.redborder2.campus', parent: :add_domain, admin: true,
            icon: 'icons/campus.svg',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.is_domain? && [Sensor::ORGANIZATION, Sensor::MARKET, Sensor::DEPLOYMENT, Sensor::BUILDING].include?(s.domain_type) }

  menu.push :add_building, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::BUILDING },
            caption: 'tr.redborder2.building', parent: :add_domain, admin: true,
            icon: 'fa fa-building-o fixed-icon',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.is_domain? && [Sensor::ORGANIZATION, Sensor::MARKET, Sensor::DEPLOYMENT, Sensor::CAMPUS].include?(s.domain_type) }

  menu.push :add_floor, { controller: 'sensors', action: 'new_domain', domain_type: Sensor::FLOOR },
            caption: 'tr.redborder2.floor', parent: :add_domain, admin: true,
            icon: 'icons/floor.svg',
            html: { 'data-backdrop' => 'static', 'data-target' => '#default_large', 'data-toggle' => 'modal' },
            if: proc { |s| s.is_domain? && [Sensor::ORGANIZATION, Sensor::MARKET, Sensor::DEPLOYMENT, Sensor::CAMPUS, Sensor::BUILDING].include?(s.domain_type) }

  menu.push :add_sensor, '#',
            caption: 'tr.redborder2.add_s', after: :add_domain, admin: true,
            html: { class: 'menu-divider-bottom' },
            if: proc { |s| (RedBorder::LICENSING_MODE == 'organization' &&
                                [Sensor::ORGANIZATION, Sensor::MARKET,
                                 Sensor::CAMPUS, Sensor::BUILDING,
                                 Sensor::DEPLOYMENT].include?(s.domain_type)) ||
                               (RedBorder::LICENSING_MODE == 'global' &&
                                [Sensor::ORGANIZATION, Sensor::MARKET,
                                 Sensor::CAMPUS, Sensor::BUILDING, Sensor::DEPLOYMENT,
                                 Sensor::NAMESPACE].include?(s.domain_type)) }

  menu.push :map, { controller: 'sensors', action: 'map' },
            caption: 'tr.redborder2.map', parent: :view, first: true

  menu.push :mse_view, { controller: 'sensors', action: 'client_proxy' },
            caption: 'tr.redborder2.proxies', parent: :view, after: :tiles_view,
            html: { class: 'block-ui-but menu-divider' }

  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :import_tree,
              {
                controller: 'sensors',
                action: 'import_tree'
              },
              caption: "tr.import_tree",
              after: :add_sensor,
              html: { 'data-backdrop' => 'static',
                      'data-toggle' => 'modal',
                      'data-target' => '#import_tree_modal' },
              admin: true,
              if: proc { |s| s.is_domain? && s.domain_type != Sensor::FLOOR }
    menu.push :export_tree,
              {
                controller: 'sensors',
                action: 'export_tree'
              },
              caption: 'tr.export_tree',
              html: { data: { 'no-turbolink' => 'true' } },
              after: :import_tree
    menu.push :expand_tree,
              '#',
              caption: 'tr.redborder2.expand',
              after: :export_tree
end

  DittoCode::Exec.if 'ENTERPRISE' do
    if RedBorder::LICENSING_MODE == 'organization'
    menu.push :organization_licenses,
              { controller: 'licenses', action: 'index' }, param: :sensor_id,
              caption: 'tr.redborder2.licenses',
              after: :add_sensor,
              admin: true,
              html: { class: 'menu-divider-bottom' },
              if: proc { |s| s.organization? }
    end


    menu.push :cluster_data,
              '#',
              caption: 'tr.redborder2.data',
              after: :organization_licenses,
              super_admin: true,
              if: proc { |s| s.is_domain? }
    menu.push :indexed_data,
              {
                controller: 'sensors',
                action: 'indexed_data'
              },
              parent: :cluster_data,
              caption: 'tr.redborder2.indexed',
              html: { class: 'block-ui-but' },
              super_admin: true,
              if: proc { |s| s.is_domain? }
  end
end

Redborder::MenuManager.map :sensors_view_menu do |menu|
  menu.push :tree_view, { controller: 'sensors', action: 'tree' },
            caption: 'tr.redborder2.tree', first: true, html: { class: 'block-ui-but' }
  menu.push :client_proxy_view, { controller: 'sensors', action: 'client_proxy' },
            caption: 'tr.redborder2.proxies', after: :tiles_view,
            html: { class: 'block-ui-but menu-divider' }
end

Redborder::MenuManager.map :module_views_menu do |menu|
  menu.push :raw, { action: :index },
            caption: 'Raw', html: { class: 'block-ui-but' },
            common: true
  menu.push :tops, { action: :tops },
            caption: 'Tops', html: { class: 'block-ui-but' },
            common: true
  menu.push :compare, { action: :comparative },
            caption: 'tr.redborder2.compare', html: { class: 'block-ui-but' },
            common: true
  menu.push :unique, { action: :cardinality },
            caption: 'tr.redborder2.unique', html: { class: 'block-ui-but' },
            common: true
end

Redborder::MenuManager.map :options_sensors_view_menu do |menu|
  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :import_tree,
              { controller: :sensors, action: :import_tree },
              params: :id,
              caption: 'tr.import_tree',
              first: true,
              admin: true,
              html: { 'data-backdrop' => 'static',
                      'data-toggle' => 'modal',
                      'data-target' => '#import_tree_modal' }
    menu.push :export_tree,
              '/sensors/export_tree',
              caption: 'tr.export_tree',
              html: { data: { 'no-turbolink' => 'true' } },
              after: :import_tree
  end

  menu.push :wakeup_chef_cluster,
            { controller: 'jobs', action: 'start_rb_wakeup_chef_cluster' },
            caption: 'tr.redborder2.wakeup',
            html: { class: 'menu-divider',
                    data: { 'toggle' => 'modal', 'target' => '#default_modal' }
                  },
            after: :export_tree,
            super_admin: true
  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :cluster_data,
              '#',
              html: { class: 'menu-divider' },
              caption: 'tr.redborder2.data',
              after: :wakeup_chef_cluster,
              super_admin: true
    menu.push :indexed_data,
              {
                controller: 'sensors',
                action: 'indexed_data'
              },
              parent: :cluster_data,
              caption: 'tr.redborder2.indexed',
              html: { class: 'block-ui-but' },
              super_admin: true
  end
end

Redborder::MenuManager.map :tools_menu do |menu|
  menu.push :alarm,
            { controller: 'alarms', action: 'index' },
            caption: 'tr.redborder2.alarms',
            first: true,
            html: { class: 'block-ui-but' }
  menu.push :audit,
            { controller: 'audits', action: 'index' },
            caption: 'tr.redborder2.audits',
            html: { class: 'block-ui-but' }
  
  menu.push :model_source,
      { controller: :model_sources, action: :index },
      caption: 'tr.redborder2.models',
      html: { class: 'block-ui-but' }

  menu.push :threat_intel_source,
    { controller: :threat_intel_sources, action: :index },
    caption: 'tr.redborder2.threat_intel_source',
    html: { class: 'block-ui-but' }

  DittoCode::Exec.if 'ENTERPRISE' do
  menu.push :manager,
            { controller: 'managers', action: 'index' },
            caption: 'tr.redborder2.cluster_settings',
            html: { class: 'block-ui-but' },
            super_reader: true
  end

# DittoCode::Exec.if 'ENTERPRISE' do
#   menu.push :correlation_engine_rules,
#             { controller: 'correlation_engine_rules', action: 'index' },
#             caption: 'Correlation Engine Rules',
#             html: { class: 'block-ui-but' },
#             super_admin: true
# end

  menu.push :setting,
            { controller: 'settings', action: 'show' },
            caption: 'tr.redborder2.general',
            html: { class: 'block-ui-but' },
            super_reader: true
            
  menu.push :integration,
            { controller: 'integrations', action: 'index' },
            caption: 'tr.redborder2.integrations',
            html: { class: 'block-ui-but' },
            super_reader: true

  menu.push :mitre,
            { controller: 'mitre', action: 'index' },
            caption: 'tr.redborder2.mitre',
            html: { class: 'block-ui-but' },
            super_reader: true

  DittoCode::Exec.if 'ENTERPRISE' do
  menu.push :licenses,
            { controller: 'licenses', action: 'index' },
            caption: 'tr.redborder2.licenses',
            html: { class: 'block-ui-but' },
            admin: true,
            if: proc { (RedBorder::LICENSING_MODE == 'global' && User.current_user.sensor.is_root?) ||
                       (RedBorder::LICENSING_MODE == 'organization' && User.current_user.sensor.organization?) }
  end

  menu.push :lookup,
            { controller: 'lookups', action: 'index' },
            caption: 'tr.redborder2.lookup',
            html: { class: 'block-ui-but' },
            super_reader: true
  menu.push :object,
            { controller: 'redborder_objects', action: 'index' },
            caption: 'tr.redborder2.objects',
            html: { class: 'block-ui-but' }
  menu.push :object_type,
            { controller: 'object_types', action: 'index' },
            caption: 'tr.redborder2.objects_type',
            html: { class: 'block-ui-but' }
  menu.push :overlay_map,
            { controller: 'overlay_maps', action: 'index' },
            caption: 'Overlay Maps',
            html: { class: 'block-ui-but' },
            super_reader: true
  menu.push :playbook_templates,
            { controller: 'playbook_templates', action: 'index' },
            caption: 'Playbooks',
            html: { class: 'block-ui-but' }
  menu.push :plugins,
            { controller: 'plugins', action: 'index' },
            caption: 'Plugins',
            html: { class: 'block-ui-but' },
            super_admin: true
  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :alerts,
              { controller: 'alerts', action: 'index' },
              caption: 'tr.redborder2.platform',
              html: { class: 'block-ui-but' },
              super_admin: true
    #menu.push :support,
    #          { controller: 'support', action: 'index' },
    #          caption: 'tr.redborder2.support',
    #          html: { class: 'block-ui-but' }
  end
  menu.push :user,
            { controller: 'users', action: 'index' },
            caption: 'tr.redborder2.users',
            html: { class: 'block-ui-but' },
            admin: true
  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :job,
              { controller: 'jobs', action: 'index' },
              caption: 'tr.redborder2.worker',
              last: true,
              html: { class: 'block-ui-but' }
  end
  DittoCode::Exec.if 'COMMUNITY' do
    menu.push :job,
              { controller: 'jobs', action: 'index' },
              caption: 'tr.redborder2.worker',
              last: true,
              html: { class: 'block-ui-but' }
  end
end

Redborder::MenuManager.map :user_home_page_collection_menu do |menu|
  DittoCode::Exec.if 'ENTERPRISE' do
    menu.push :Overview, {}, caption: '/overview', first: true
  end
  menu.push :Dashboard, {}, caption: '/dashboard'
  menu.push :Sensors, {}, caption: '/sensors', last: true
end

Redborder::MenuManager.map :widget_sensor_table_type_menu do |menu|
  menu.push :Tree, {}, caption: 'Tree', first: true
  menu.push :Map, {}, caption: 'Map', last: true
end

Redborder::MenuManager.map :worker_menu do |menu|
  # menu.push :force_geoip,
  #           { controller: :jobs, action: :start_geoip_update },
  #           html: { method: :post },
  #           caption: 'tr.redborder2.update',
  #           if: proc { DelayedJob.geoip_update? }
  # menu.push :start_geoip,
  #           { controller: :jobs, action: :start_geoip_update },
  #           html: { method: :post },
  #           caption: 'tr.redborder2.start',
  #           if: proc { !DelayedJob.geoip_update? }
  menu.push :start_alarm,
            { controller: :jobs, action: :start_alarm },
            html: { method: :post },
            caption: 'tr.redborder2.start_alarm',
            if: proc { !DelayedJob.alarm? }
  menu.push :start_report,
            { controller: :jobs, action: :start_report },
            html: { method: :post },
            caption: 'tr.redborder2.start_report',
            if: proc { !DelayedJob.report? }
  menu.push :glpi_inventory_job,
            { controller: :jobs, action: :start_glpi_inventory_job},
            caption: 'tr.redborder2.glpi',
            if: proc { !DelayedJob.glpi_inventory_job? && Integration.glpi_params_present?  }
  menu.push :threat_intel_source_job,
            { controller: :jobs, action: :start_threat_intel_source_download},
            caption: 'tr.redborder2.threat_intel_source_update',
            if: proc { !DelayedJob.threat_intel_source_job? }
  menu.push :incident_to_syslog,
            { controller: :jobs, action: :start_incident_to_syslog_job},
            caption: 'tr.redborder2.incidents_to_syslog',
            if: proc { !DelayedJob.incident_to_syslog? }   
  menu.push :wakeup_chef,
            { controller: :jobs, action: :start_rb_wakeup_chef_cluster },
            html: { data: { 'toggle' => 'modal', 'target' => '#default_modal' } },
            caption: 'tr.redborder2.wakeup',
            if: proc { !DelayedJob.rb_wakeup_chef_cluster? }
  menu.push :update_vulnerabilities_db,
            { controller: :jobs, action: :start_rb_vulnerabilities_update_db },
            html: { method: :post },
            caption: 'tr.redborder2.vulnerabilities_db',
            if: proc { !DelayedJob.rb_vulnerabilities_update_db? }
  menu.push :check_corrupt_data,
            { controller: :jobs, action: :start_checks },
            html: { method: :post },
            caption: 'tr.redborder2.start_check',
            super_admin: true,
            if: proc { !DelayedJob.check_corrupt_data? }
  menu.push :update_content_incidents,
            { controller: :jobs, action: :start_update_content_incidents },
            html: { method: :post },
            caption: 'tr.redborder2.update_content_incidents',
            if: proc { !DelayedJob.update_content_incidents? }
  menu.push :incidents_outliers,
            { controller: :jobs, action: :start_incidents_outliers },
            html: { method: :post },
            caption: 'tr.redborder2.incidents_outliers',
            if: proc { !DelayedJob.incidents_outliers? }
  menu.push :delete_stored_jobs,
            { controller: :jobs, action: :start_delete_stored_jobs },
            html: { method: :post },
            caption: 'tr.redborder2.delete_stored_jobs',
            if: proc { !DelayedJob.delete_stored_jobs? }
end

Redborder::MenuManager.map :client_proxy_menu do |menu|
  menu.push :edit,
            { controller: 'sensors', action: 'edit' },
            caption: 'tr.redborder2.edit',
            first: true,
            html: { class: 'block-ui-but' },
            admin: true
  menu.push :add_sensor, '#',
            caption: 'tr.redborder2.add_s',
            after: :edit,
            html: { class: 'block-ui-but' },
            admin: true
end
