# #######################################################################
# # Copyright (c) 2014 ENEO Tecnologia S.L.
# # This file is part of redBorder.
# # redBorder is free software: you can redistribute it and/or modify
# # it under the terms of the GNU Affero General Public License as published by
# # the Free Software Foundation, either version 3 of the License, or
# # (at your option) any later version.
# # redBorder is distributed in the hope that it will be useful,
# # but WITHOUT ANY WARRANTY; without even the implied warranty of
# # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# # GNU Affero General Public License for more details.
# # You should have received a copy of the GNU Affero General Public License
# # along with redBorder. If not, see <http://www.gnu.org/licenses/>.
# #######################################################################

# module Redborder #:nodoc:
#   module As #:nodoc:
#     #
#     # Module to cache querries on Rails. Long queries will be cached N seconds
#     #
#     module Cache
#       #
#       # Check if exist a cache of a value based on the statement.
#       # If there aren't any cache, it creates a new cache for 1 minutes
#       #
#       # == Paramenters:
#       # stmt::
#       #   Statement for the query
#       # cache::
#       #   (Optional) Boolean indicates if we must cache the query
#       #
#       # == Returns:
#       # (Array) Result of the query to Aerospike
#       #
#       def cache_or_query(stmt, cache = true)
#         if cache
#           # Write or get from cache
#           res = Rails.cache.fetch "rb-webui:aerospike:#{statement_id(stmt)}",
#                             expires_in: 2.minutes do
#             data = []
#             begin
#               data = aero_query(stmt)
#             rescue StandardError => e
#               # Don't cache if query fails
#               # Return error and log
#               Rails.logger.error e
#               Rails.logger.error e.backtrace.join("\n")
#               break
#             end
#             # Cache data
#             data
#           end
#         else
#           res = []
#           begin
#             res = aero_query(stmt)
#           rescue StandardError => e
#             # Don't cache if query fails
#             # Return error and log
#             Rails.logger.error e
#             Rails.logger.error e.backtrace.join("\n")
#           end
#         end
#         # Return an empty array if query fails
#         res || []
#       end

#       #
#       # Perform the Query. This may cause an exception, please use a rescue
#       # block.
#       #
#       # == Paramenters:
#       # stmt::
#       #   Statement for the query
#       #
#       # == Returns:
#       # (Array) Result of the query to Aerospike
#       #
#       def aero_query(stmt)
#         data = []
#         connect.query(stmt).each do |row|
#           data << row.bins
#         end
#         # Return data
#         data
#       end

#       #
#       # Get an unique id for a Statement. This is used to create a cache for the
#       # query.
#       #
#       # == Paramenters:
#       # stmt::
#       #   Statement for the query
#       # md5::
#       #   Get the MD5 of the ID
#       #
#       # == Returns:
#       # String with an unique ID
#       #
#       def statement_id(stmt, md5 = true)
#         str = "{ namespace: #{stmt.namespace.downcase}," \
#               " collection: #{stmt.set_name.downcase}, " \
#               " columns: #{stmt.bin_names.sort.join.downcase}, " \
#               " filters: [ #{join_filters(stmt)} ] }"
#         return str unless md5
#         # Get MD5
#         id = Digest::MD5.new
#         id << str
#         id.hexdigest
#       end

#       #
#       # Get the filters as string of the current statement.
#       #
#       # == Paramenters:
#       # stmt::
#       #   Statement for the query
#       #
#       # == Returns:
#       # String with an unique ID
#       #
#       def join_filters(stmt)
#         # TODO: Get a better id for filters
#         stmt.filters.map { |f| "#{f}" }.join(', ')
#       end
#     end
#   end
# end
