#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  class Aws
    #
    # Return a collection of AWS regions for an input
    #
    # == Returns:
    # An Array of Arrays
    #
    def  self.regions
      [
        ['Asia Pacific, Tokyo (ap-northeast-1)', 'ap-northeast-1'],
        ['Asia Pacific, Singapore (ap-southeast-1)', 'ap-southeast-1'],
        ['Asia Pacific, Sydney (ap-southeast-2)', 'ap-southeast-2'],
        ['EU, Frankfurt (eu-central-1)', 'eu-central-1'],
        ['EU, Ireland (eu-west-1)', 'eu-west-1'],
        ['South America, Sao Paulo (sa-east-1)', 'sa-east-1'],
        ['US East, N. Virginia (us-east-1)', 'us-east-1'],
        ['US West, N. California (us-west-1)', 'us-west-1'],
        ['US West, Oregon (us-west-2)', 'us-west-2']
      ]
    end

    #
    # Check if the current region is a valid AWS region
    #
    # == Parameters:
    # region::
    #   String with region to check or empty string
    #
    # == Returns:
    # True if its a valid region or the element it's empty
    def self.region?(region)
      return true if region.present? && regions.map(&:last).include?(region)
      false
    end
  end
end
