#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Chef
    module Role
      # Include default values for Chef role
      module OverrideAttributes
        private

        # Build override attributes based on override_attributes given and
        # default options
        #
        # @param override_attributes [Hash] Override attributes available.
        #   It can be an empty Hash.
        # @return [Hash] Override attributes
        #
        def fetch_override_attributes(override_attributes)
          override_attributes = if override_attributes.present?
                                  override_attributes
                                else
                                  default_override_attributes
                                end
          # Check and return
          check_override_attributes(override_attributes)
        end

        # Default override_attributes for Chef sensor nodes
        #
        # @param physical [Boolean] The sensor that wants to create role is physical
        # @return [Hash] Override attributes
        #
        def default_override_attributes
          {
            'redborder' => {
              'snort' => {
                'default' => {
                  'ipvars' => {},
                  'portvars' => {},
                  'save_pcap' => nil,
                  'mode' => nil,
                  'autobypass' => nil
                }
              },
              'rsyslog' => {},
              'proxy' => {},
              'snmp' => { 'trap_server' => [] },
              'monitors' => []
            }
          }
        end

        # Chefk if override Attributes contains 'redBorder' key
        #
        def check_override_attributes(override_attributes)
          return override_attributes if override_attributes['redborder'].is_a?(Hash)
          Rails.logger.warn("override_attributes['redborder'] is not initializated")
          override_attributes['redborder'] = {}
          # Return new override_attributes
          override_attributes
        end
      end
    end
  end
end
