#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  #
  # Singleton class to return always same instance of DruidConfig
  #
  class Cluster
    # Global Config
    @druid_config = nil

    #
    # Fetch an instance of DruidConfig
    #
    def self.instance
      return @druid_config if @druid_config
      @druid_config = DruidConfig::Cluster.new(
        RedBorder::DRUID_CONFIG[:zk_connect],
        discovery_path: '/druid/discoveryPath',
        zk_keepalive: true,
        http_timeout: RedBorder::DRUID_CONFIG[:timeout] || 20
      )
    end

    #
    # Reset client
    #
    def self.reset!
      @druid_config = nil
      instance
    end

    #
    # Close Zookeeper watcher
    #
    def self.close!
      return unless @druid_config
      @druid_config.close!
      @druid_config = nil
    end

    #
    # Disable initializer
    #
    def initialize
      fail "#{I18n.t 'lib_redborder.instance'}"
    end

    # Return CDOMAIN of the cluster
    #
    # @return [String] CDOMAIN
    #
    def self.cdomain
      return @cdomain if @cdomain
      @cdomain = Manager.all.sample.node['redborder']['cdomain']
    rescue StandardError
      @cdomain = 'redborder.cluster'
    end

    # Return UUID of the cluster
    #
    # @return [String, nil] Cluster UUID
    #
    def self.uuid
      return @uuid if @uuid
      @uuid =
        Redborder::Databag.load('global', 'cluster', false, false)
                                   .try(:[], 'uuid')
    end
  end
end
