#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Module to group queries and add to the controller
    #
    class CommonController < ApplicationController
      # -------------------------------------------------------------
      # Include common modules
      # -------------------------------------------------------------
      include Redborder::Commons::Query::Format
      include Redborder::Commons::Query::General
      include Redborder::Commons::Query::Raw
      include Redborder::Commons::Query::Tops
      include Redborder::Commons::Query::Cardinality
      include Redborder::Commons::Query::Comparative

      include Redborder::Commons::View::Raw
      include Redborder::Commons::View::Tops
      include Redborder::Commons::View::Cardinality
      include Redborder::Commons::View::Comparative

      # -------------------------------------------------------------
      # Constants
      # -------------------------------------------------------------
      SORT_DIRECTION = %w(ascending descending)

      # -------------------------------------------------------------
      # Before actions and filters
      # -------------------------------------------------------------

      # Init source. Load filters and apply them if params[:search] is defined.
      # This before action must be ALWAYS at top.
      before_action :set_source, :filters_from_url

      # Init categories and settings to load dropdown menu on section navigation
      before_action :set_module_settings, only: [
        :index, :tops, :heatmap, :comparative, :cardinality, :add_tab,
        :del_tab, :add_cardinality_tab, :del_cardinality_tab,
        :load_tab_cardinality, :info, :sort_columns, :load_more, :load_new_data,
        :element_childs, :search, :export_raw_to_file, :get_raw_table
      ]

      before_action :set_default_action, only: [
        :index, :tops, :map, :comparative, :cardinality, :heatmap
      ]

      before_action :set_reports, only: [
        :index, :tops, :map, :floor_map, :cardinality, :comparative, :heatmap
      ]

      before_action :set_chart_type, only: [
        :index, :tops, :map, :floor_map, :comparative, :cardinality, :heatmap,
        :export_raw_to_file, :clients, :client_points, :search_clients,
        :load_more, :get_raw_table
      ]

      before_action :set_range, only: [
        :index, :tops, :map, :floor_map, :comparative, :load_more,
        :cardinality, :heatmap, :export_raw_to_file, :clients,
        :client_points, :search_clients, :get_raw_table, :load_new_data
      ]

      before_action :set_granularity, only: [
        :index, :tops, :map, :floor_map, :comparative, :load_more,
        :cardinality, :heatmap, :export_raw_to_file, :clients,
        :client_points, :search_clients, :get_raw_table, :load_new_data
      ]

      before_action :set_aggregation, only: [
        :index, :tops, :map, :floor_map, :comparative,
        :element_childs, :add_tab, :load_tab, :load_more, :load_more_tops,
        :load_more_comparative, :load_new_data, :get_serie,
        :get_cardinality_serie, :get_tops, :get_raw_table,
        :load_cardinality_table, :load_more_cardinality,
        :export_tops_to_file, :export_cardinality_to_file,
        :export_raw_to_file, :get_maps_data, :info, :get_outliers,
        :add_cardinality_tab, :cardinality, :heatmap
      ]

      before_action :save_aggregation_to_session, only: [
        :index, :tops, :comparative, :cardinality, :heatmap,
        :load_more_tops, :add_tab, :element_childs, :add_cardinality_tab,
        :comparative, :load_more_comparative, :load_new_data,
        :map, :load_more, :floor_map
      ]

      before_action :filter_params, only: [
        :index, :tops, :comparative, :element_childs, :add_tab,
        :load_more, :load_more_tops, :load_more_comparative,
        :load_new_data, :get_serie, :get_cardinality_serie, :get_tops,
        :get_raw_table, :load_cardinality_table, :load_more_cardinality,
        :export_tops_to_file, :export_cardinality_to_file,
        :export_raw_to_file, :get_maps_data, :info, :get_outliers,
        :heatmap, :add_cardinality_tab
      ]

      before_action :int_times, only: [
        :get_serie, :get_cardinality_serie, :get_tops, :get_raw_table,
        :load_new_data, :load_cardinality_table, :load_more_cardinality,
        :get_maps_data, :load_more_tops, :load_more_comparative, :get_outliers,
        :channels_load, :channels_load_serie, :aps_serie, :aps_count,
        :aps_metrics, :heatmap
      ]

      before_action :should_skip_empty, only: :get_serie

      # -------------------------------------------------------------
      # Action methods
      # -------------------------------------------------------------

      #
      # [GET] Load the advanced search form.
      #
      def search
        items = {}
        items = @filter.serialize unless params[:new_filter].present?
        @filters = { items: items } unless items.blank?
      end

      #
      # [GET] Show a modal to reorder the aggregations
      #
      def sort_aggregations
        @aggregations = session[@source.session][:aggregations]
        render layout: false
      end

      #
      # [GET] Reorder the current tabs in the app (for example, on tops)
      #
      def reorder_tab
        # params[:group] for compare or tops
        session[@source.session][params[:group].to_sym] =
          params[:tabs].map(&:to_sym)
        render js: ''
      end

      #
      # [GET] Reorder the current aggregations in the app
      #
      def reorder_aggregations
        # params[:group] for compare or tops
        session[@source.session][:aggregations] =
          params[:aggregations].map(&:to_sym)

        # set default aggregation
        session[@source.session][:aggregation] =
          session[@source.session][:aggregations].first
        render js: ''
      end

      #
      # [PATCH] Change the value to hide or show total in graphs
      #
      def change_session_total
        session[@source.session][:hide_total] = params[:value]
        render js: ''
      end

      #
      # [PATCH] Change the current tab of the user
      #
      def change_session_tab
        # params[:group] for compare or tops.
        # Monitor hasn't got group
        group = (params[:group] || :tab).to_sym
        session[@source.session][group] = params[:tab].to_sym
        # Render an empty JS, the client must reload web
        render js: ''
      end

      #
      # [PATCH] Change the graph type of the user
      #
      def change_chart_type
        @chart_type = params[:chart_type]
        @action = params[:id]
        session[@source.session][:chart_type][@action] = @chart_type
        render layout: false
      end

      #
      # Assign a received param value to a specific variable
      #
      def assign_param(columns, name)
        return name if columns.include?(name)
        columns.first
      end

      #
      # [ TODO ] Create independent methods for each view.
      #          Place each one in the propper file.
      #
      def get_serie
        @tab         = params[:id]
        @value       = params[:value]
        @comparative = params[:comparative] == 'true'

        @comparative ? get_comparative_serie : get_tops_serie

        respond_to do |format|
          format.js
          format.json { render json: @serie }
        end
      end

      #
      # [ TODO ] Create independent methods for each view.
      #          Place each one in the propper file.
      #
      def add_tab
        @tab = params[:id]
        @range = params[:range]
        @comparative = params[:comparative] == 'true'

        @comparative ? add_comparative_tab : add_tops_tab
      end

      #
      # [ TODO ] Create independent methods for each view.
      #          Place each one in the propper file.
      #
      def del_tab
        @tab = params[:id]
        @range = params[:range]
        @comparative = params[:comparative] == 'true'

        @comparative ? del_comparative_tab : del_tops_tab
      end

      def heatmap
        @data = @source.topN(
          query_type: :group_by,
          granularity: :all,
          start_time: @start_time,
          end_time: @end_time,
          group_by: [:coordinates],
          limit: 10000,
          search: @filter.get,
          aggregation: @aggregation
        )
        @data.each { |t| t.client_latlong = t.coordinates }
      end

      def get_outliers
        tab = params[:id]
        value = params[:value]
        timezone = params[:timezone]

        granularities = Redborder::Rbdruid.granularity_list(
          start_time: params[:start_time].to_time,
          end_time: params[:end_time].to_time)
        granularity  = if params[:granularity].present?
                          params[:granularity]
                        else
                          granularities.first
                        end

        if tab.to_sym.eql?(:conversation) && @source.can_filter?(:src) && @source.can_filter?(:dst)
          @filter.add('src', [value.first])
          @filter.add('dst', [value.last])
        elsif tab.to_sym.eql?(:conversation) && @source.can_filter?(:lan_ip) && @source.can_filter?(:wan_ip)
          @filter.add('lan_ip', [value.first])
          @filter.add('wan_ip', [value.last])
        elsif value
          value = nil if params[:value].eql?('N/A')

          case tab.to_sym
          when :wireless_station
            @filter.remove :zone
          when :sensor_name
            @filter.remove :domain
          end

          # Remove the column before add the value. If we hold all filters,
          # all graphs will be the same
          @filter.remove(tab)
          @filter.add(tab.to_sym, [value])
        end

        params[:end_time] = Time.now if params[:end_time] > Time.now
        serie = if tab.to_sym.eql?(:conversation)
                  @source.where(
                    query_type: :group_by,
                    range: @range,
                    start_time: params[:start_time],
                    end_time: params[:end_time],
                    granularity: granularity,
                    group_by: @source::COLUMNS[tab.to_sym][:columns].keys,
                    aggregation: @aggregation,
                    timezone: timezone,
                    search: @filter.get,
                    outliers: true
                  )
                else
                  @source.timeserie(
                    range: @range,
                    start_time: params[:start_time],
                    end_time: params[:end_time],
                    granularity: granularity,
                    aggregation: @aggregation,
                    timezone: timezone,
                    search: @filter.get,
                    outliers: true
                  )
                end

        respond_to do |format|
          format.js
          format.json {
            if serie.is_a? Array or serie.is_a? Hash
              render json: serie
            else
              render json: { error: serie }, status: 500
            end
          }
        end
      end

      def get_predicted_serie
        start_time = Time.at(params[:start_time].to_i)
        end_time = Time.at(params[:end_time].to_i)

        prediction = @source.predict(
          query_type: :timeseries,
          granularity: :minute,
          start_time: start_time,
          end_time: end_time
        )

        render json: prediction
      end

      # -------------------------------------------------------------
      # Private methods
      # -------------------------------------------------------------

      private

      #
      # Set the column to search on a druid query.
      #
      # == Returns:
      # Symbol of the column
      #
      def sort_column
        return @aggregation if @granularity && @granularity.to_sym == :all
        return :timestamp unless params.key?(:sort)
        params[:sort].to_sym
      end

      #
      # Set the direction to sort druid responses.
      #
      # == Returns:
      # Symbol of direction to sort
      #
      def sort_direction
        return :DESCENDING unless SORT_DIRECTION.include? params[:direction]
        # Send the current direction
        params[:direction].to_sym
      end

      #
      # Set a new timestamp for each tab depending on the time range.
      #
      # == Returns:
      # Timestamp for each table row.
      #
      def get_timestamp_from_tab
        if @tab.to_sym.eql?(:pt24h)
          @serie_comparatives.first.timestamp.beginning_of_hour
        elsif @tab.to_sym.eql?(:pt168h)
          @serie_comparatives.first.timestamp.beginning_of_day
        elsif @tab.to_sym.eql?(:pt720h)
          @serie_comparatives.first.timestamp.beginning_of_week
        elsif @tab.to_sym.eql?(:pt4320h)
          @serie_comparatives.first.timestamp.beginning_of_month
        end
      end

      #
      # Set a new domain for each tab depending on the time range.
      #
      # == Returns:
      # Domain graph for each tab.
      #
      def get_domain_graph_from(content)
        step = GRANULARITY_TYPE[COMPARATIVE_COLUMNS[@tab.to_sym][:granularity][:graph]][:step]

        timestamp =
          if content.nil? || content.first.nil?
            if @tab.eql?(:pt24h)
              in_time_machine { Time.zone.now.beginning_of_day }
            else
              in_time_machine { Time.zone.now.beginning_of_hour }
            end
          else
            if @tab.to_sym.eql?(:pt24h)
              content.first.timestamp.beginning_of_day.change({ day: in_time_machine { Time.now.day }})
            elsif @tab.to_sym.eql?(:pt168h)
              content.first.timestamp.beginning_of_day
            elsif @tab.to_sym.eql?(:pt720h)
              content.first.timestamp.beginning_of_week
            elsif @tab.to_sym.eql?(:pt4320h)
              content.first.timestamp.beginning_of_month
            end
          end

        [
          timestamp.to_i,
          timestamp.to_i + COMPARATIVE_COLUMNS[@tab.to_sym][:step].to_i - step / 1000
        ]
      end
      # -------------------------------------------------------------
      # Initializers
      # -------------------------------------------------------------

      #
      # Initiate the reports to build the Add to report menu
      #
      def set_reports
        @reports = current_user.writable_reports
      end

      #
      # Use callbacks to share common setup or constraints between actions.
      #
      def set_default_action
        # Don't save when comes from widgets
        return if params['w'].present?

        session[@source.session][:default_action] = {
          controller: params[:controller],
          action: params[:action]
        } if request.get?
        Redborder::MenuManager.map(:product_menu)
          .find(@source.id).url = session[@source.session][:default_action]
      end

      #
      # Initialize the aggregation. Check to default aggregation on source.
      #
      def set_aggregation
        @aggregations = session[@source.session]
                        .fetch(:aggregations, [@source.default_aggregation])

        # See Redmine#4174
        @aggregations ||= [@source.default_aggregations]

        if params[:aggregation_action] == 'replace'
          @aggregations = [*params[:aggregation]].map(&:to_sym)
        elsif params[:aggregation_action] == 'add'
          @aggregations.unshift(params[:aggregation].to_sym)
        elsif params[:aggregation_action] == 'delete' && @aggregations.size > 1
          @aggregations.delete(params[:aggregation].to_sym)
        end

        @aggregation = if params[:aggregation] && params[:aggregation_action].nil?
                         [*params[:aggregation]].first.to_sym
                       else
                         @aggregations.first.to_sym
                       end
      end

      #
      # Load enabled columns. @Settings must be initialized
      #
      # == Returns:
      # An array of symbols with the enabled columns
      #
      def load_columns
        return @source::COLUMNS if @inactive_settings.nil?
        @source::COLUMNS.select do |k, v|
          !v[:hide_raw] && !@inactive_settings.has_key?("#{@source.id}_#{k}")
        end
      end

      #
      # Load current active columns. @Settings must be initialized
      #
      # == Returns:
      # An array of symbols with the activle columns
      #
      def load_active_columns
        session[@source.session][:active_columns].select do |k|
          !@inactive_settings.key?("#{@source.id}_#{k}")
        end
      end

      # -------------------------------------------------------------
      # Store
      # -------------------------------------------------------------

      #
      # Save current aggregation to session.
      #
      def save_aggregation_to_session
        #
        # [ TODO ] Angel. This param is added to check if the query comes
        #                 from a widget. If it's true, this request is from a
        #                 widget. We musn't save the aggregation from a widget,
        #                 but is this the best way?
        #                 The problem was with a client map who calls /flow/map
        #                 to render the JSON with info. One solution is to
        #                 create another method for the widget. The problem of
        #                 these is we must duplicate the logic.
        #
        return unless params['w'].nil?
        session[@source.session][:aggregations] = @aggregations
        session[@source.session][:aggregation] = @aggregation
      end

      def filter_params
        return if params[:aggregation].present?
        session[@source.session][:aggregation] ||= @source::AGGREGATIONS.keys.first
        @aggregation = session[@source.session][:aggregation].to_sym
        @aggregations = session[@source.session][:aggregations] ||
                        [@aggregation, (@source::AGGREGATIONS.keys - [@aggregation]).first].compact
      end

      def get_active_tabs(type)
        if session[@source.session][type].blank?
          if session[@source.session][:default_action] == { controller: @source.controller.to_s, action: 'index' } || session[@source.session][:default_action] == { controller: @source.controller.to_s, action: 'cardinality' }
            @source::COLUMNS.each{|k,v| (session[@source.session][type] ||= []) << k if (v[:enabled] and !v[:hide_raw] and Setting.send("#{@source.id.to_s}_#{k}".to_sym) == '1')}
          elsif session[@source.session][:default_action] == { controller: @source.controller.to_s, action: 'tops' }
            @source::COLUMNS.each{|k,v| (session[@source.session][type] ||= []) << k if (v[:enabled] and Setting.send("#{@source.id.to_s}_#{k}".to_sym) == '1')}
          elsif session[@source.session][:default_action] == { controller: @source.controller.to_s, action: 'comparative' }
            COMPARATIVE_COLUMNS.each{|k,v| (session[@source.session][type] ||= []) << k if v[:enabled]}
          end
        else
          if params[:deltab].present?
            if session[@source.session][type].include?(params[:deltab].to_sym)
              session[@source.session][type].delete(params[:deltab].to_sym)
              return
            end
          end
          return unless params[:newtab].present?
          if session[@source.session][type].include?(params[:newtab].to_sym)
            return
          else
            session[@source.session][type] = @source::COLUMNS.keys.select{ |key| (session[@source.session][type] + [params[:newtab].to_sym]).include?(key) }
          end
        end
      end
    end
  end
end