#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common queries
    #
    module Query
      #
      # Define the comon queries for Comparative actions
      #
      module Cardinality
        #
        # Return the data for tab on comaprative view.
        #
        # == Parameters:
        # tab::
        #   Symbol with the current tab
        #
        # == Returns:
        # Array of hashes with data.
        #
        def cardinality_tab_data(tab)
          post_cardinality(
            @source.where(
              query_type: :cardinality,
              cardinality_column: tab,
              range: @range || params[:range],
              start_time: @start_time || params[:start_time],
              end_time: @end_time || params[:end_time],
              granularity: @granularity || params[:granularity],
              search: @filter.get,
              cardinality_forecast: params[:cardinality_forecast]
            )
          )
        end

        #
        # Return cardinality total.
        #
        # == Parameters:
        # tab::
        #   Symbol with the current tab
        #
        # == Returns:
        # Hash with the total
        #
        def cardinality_total(tab)
          @source.where(
            query_type: :cardinality,
            cardinality_column: tab,
            range: @range || params[:range],
            start_time: @start_time || params[:start_time],
            end_time: @end_time || params[:end_time],
            granularity: :all,
            search: @filter.get
          ).first.try(:result) || 0
        end

        #
        # Return cardinality events for table.
        #
        # == Parameters:
        # tab::
        #   Symbol with the current tab
        #
        # == Returns:
        # Hash with the total
        #
        def cardinality_table(tab)
          @source.where(
            query_type: :group_by_cardinality,
            cardinality_column: tab,
            group_by: @source::COLUMNS[@group.to_sym][:columns].keys,
            range: :custom,
            start_time: @start_time,
            end_time: @end_time,
            sort: [:result,
                   @source::COLUMNS[@group.to_sym][:columns].keys.first],
            granularity: :all,
            limit: params.fetch(:limit, current_user.per_page_count),
            offset: @offset || 0,
            search: @filter.get
          )
        end
      end
    end
  end
end
