#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common queries
    #
    module Query
      #
      # Define the comon queries for Comparative actions
      #
      module Comparative
        #
        # Return the data for tab on comaprative view.
        #
        # == Returns:
        # Array of hashes with data.
        #
        def comparative_tab_data(aggregations = @aggregations)
          end_time = if @end_time > in_time_machine { Time.zone.now }
                       in_time_machine { Time.zone.now }
                     else
                       @end_time
                     end

          @source.where(
            query_type: :timeseries,
            range: :custom,
            start_time: @start_time,
            end_time: end_time,
            granularity: @granularity.to_sym,
            aggregation: aggregations,
            search: @filter.get
          )
        end

        #
        # Return the total.
        #
        # == Returns:
        # Hash with the total
        #
        def comparative_total
          @source.total(
            range: :custom,
            start_time: @start_time,
            end_time: in_time_machine { Time.now },
            aggregation: @aggregation,
            search: @filter.get
          )
        end

        #
        # Return the past series.
        #
        # == Returns:
        # Comparative series and its totals.
        #
        def past_comparative_series(total=false)
          columns = Redborder::Rbdruid::COMPARATIVE_COLUMNS[@tab.to_sym]
          @end_time = @start_time
          @start_time = @start_time - columns[:look_back]
          series = comparative_tab_data

          if series.present?
            series =  @source.where(
                          query_type: :timeseries,
                          range: :custom,
                          start_time: @start_time,
                          end_time: @end_time,
                          granularity: columns[:granularity][:comparative],
                          aggregation: @aggregations,
                          search: @filter.get
                        )

            if @tab.to_sym.eql?(:pt24h)
              @end_time = series.first.timestamp.end_of_hour
              @start_time = @end_time.beginning_of_hour
            elsif @tab.to_sym.eql?(:pt168h)
              @end_time = series.first.timestamp.end_of_day
              @start_time = @end_time.beginning_of_day
            elsif @tab.to_sym.eql?(:pt720h)
              @end_time = series.first.timestamp.end_of_week
              @start_time = @end_time.beginning_of_week
            elsif @tab.to_sym.eql?(:pt4320h)
              @end_time = series.first.timestamp.end_of_month
              @start_time = @end_time.beginning_of_month
            end

            return if @filter.exist_any?(:start_time) &&
              columns[:start].call(Time.at(@filter.get('start_time').keys.first)) > @start_time

            @serie_comparatives = comparative_tab_data
            return @serie_comparatives unless total
            @total = @serie_comparatives.present? ? comparative_total : 0
          end
        end
      end
    end
  end
end
