#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common queries
    #
    module Query
      #
      # Helper functions to format the response
      #
      module Format
        #
        # Map events to adapt every DataSource event to timeserie format:
        # {
        #   event: { events: '..' },
        #   timestamp: '...'
        # }
        #
        # == Parameters:
        # data::
        #   Array of data returned by Druid
        #
        # == Returs:
        # Array of formatted elements
        #
        def post_timeseries(data)
          data.map do |e|
            {
              event:
              { events: e.send(@aggregation) },
              timestamp: e.timestamp.to_s
            }
          end
        end

        #
        # Map events to adapt every DataSource event to cardinality format:
        # {
        #   event: { events: '..' },
        #   timestamp: '...'
        # }
        # == Parameters:
        # data::
        #   Array of data returned by Druid
        #
        # == Returs:
        # Array of formatted elements
        #
        def post_cardinality(data)
          data.map do |e|
            {
              event:
              { events: e.result },
              timestamp: e.timestamp.to_s
            }
          end
        end
      end
    end
  end
end
