#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common queries
    #
    module Query
      #
      # Define the comon queries for RAW (index) actions
      #
      module Raw
        #
        # Return the data for the main graph on raw. All variables and methods
        # must be defined.
        #
        # == Returns:
        # Array of formatted data
        #
        def raw_graph
          post_timeseries(
            @source.where(
              query_type: :timeseries,
              range: @range,
              start_time: @start_time,
              end_time: @end_time,
              granularity: @granularity,
              search: @filter.get,
              aggregation: @aggregation,
              sort: sort_column,
              direction: sort_direction
            )
          )
        end

        #
        # Return data to set the stream graph in RAW view.
        #
        # == Returns:
        # Array of data
        #
        def raw_graph_events(raise_error = false)
          @source.where(
            query_type: :group_by,
            range: :custom,
            start_time: @start_time,
            end_time: @end_time,
            granularity: :all,
            sort: @aggregation,
            direction: sort_direction,
            #limit: current_user.per_page_count + @offset,
            group_by: session[@source.session][:active_columns],
            aggregation: @aggregations,
            search: @filter.get,
            raise_error: raise_error
          )
        end

        #
        # Return data to set the events in RAW view.
        #
        # == Returns:
        # Array of data
        #
        def raw_events(raise_error=false)
          @source.where(
            query_type: :group_by,
            range: :custom,
            start_time: @start_time,
            end_time: @end_time,
            granularity: @granularity,
            sort: sort_column,
            direction: sort_direction,
            limit: current_user.per_page_count,
            offset: @offset,
            group_by: session[@source.session][:active_columns],
            aggregation: @aggregations,
            search: @filter.get,
            raise_error: raise_error
          )
        end

        #
        # Return the total of RAW events.
        #
        # == Returns:
        # Hash with the total
        #
        def raw_total
          @source.total(
            range: @range,
            start_time: @start_time,
            end_time: @end_time,
            aggregation: @aggregation,
            search: @filter.get
          )
        end
      end
    end
  end
end
