#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common queries
    #
    module Query
      #
      # Define the comon queries for Tops actions
      #
      module Tops
        #
        # Return the data for the main graph on Tops. All variables and methods
        # must be defined.
        #
        # == Parameters:
        # filters::
        #   A hash with the filters to apply
        #
        # == Returns:
        # Array of formatted data
        #
        def tops_graph(filters = {})
          post_timeseries(
            @source.where(
              query_type: :timeseries,
              range: @range,
              start_time: @start_time,
              end_time: @end_time,
              granularity: @granularity,
              aggregation: @aggregation,
              search: filters
            )
          )
        end

        #
        # Return the data for tab on tops view.
        #
        # == Parameters:
        # tab::
        #   Symbol with the current tab
        #
        # == Returns:
        # Array of hashes with data.
        #
        def tops_tab_data(tab)
          @source.where(
            query_type: :group_by,
            range: @range,
            start_time: @start_time,
            end_time: @end_time,
            sort: [@aggregation,
                   @source::COLUMNS[tab][:columns].keys.first],
            granularity: :all,
            limit: current_user.per_page_count,
            group_by: @source::COLUMNS[tab][:columns].keys,
            aggregation: @aggregations,
            search: @filter.get
          )
        end
      end
    end
  end
end
