#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common views
    #
    module View
      #
      # Define the comon methods for Cardinality views
      #
      module Cardinality
        # Includes
        include Redborder::Rbdruid

        def cardinality
          @tabs = load_columns.keys
          get_active_tabs(:active_cardinality_tabs)

          if params[:tab]
            unless session[@source.session][:active_cardinality_tabs]
                   .include?(params[:tab].to_sym)
              session[@source.session][:active_cardinality_tabs] << params[:tab].to_sym
            end
            session[@source.session][:cardinality_tab] = params[:tab].to_sym
          end

          unless params[:group_by].nil?
            session[@source.session][:cardinality_group_by] = params[:group_by]
          end

          @granularity = params[:granularity]
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]

          unless session[@source.session][:active_cardinality_tabs]
                 .include?(session[@source.session][:cardinality_tab])
            session[@source.session][:cardinality_tab] = session[@source.session][:active_cardinality_tabs].first
          end

          @cardinalities = {}

          session[@source.session][:active_cardinality_tabs].each do |tab|
            tab = tab.to_sym
            if tab == session[@source.session][:cardinality_tab].to_sym

              if @filter.exist_any?(tab) && ![:sensor_name, :sensor_uuid].include?(tab)
                @filter.exclude_empty(tab)
                # @filter.add(tab, [''], 'is_not')
              end

              # Fetch data from common queries
              content = cardinality_tab_data(tab)
              total = content.empty? ? 0 : cardinality_total(tab)

              @cardinalities[tab.to_sym] = { content: content, total: total }
            else
              @cardinalities[tab.to_sym] = { content: [], total: 0 }
            end
          end

          set_total_options
        end

        def load_tab_cardinality
          @tab = params[:id].to_sym
          @tabs = load_columns.keys
          @start_time = Time.at(params[:start_time].to_i)
          @end_time = Time.at(params[:end_time].to_i)
          @granularity = params[:granularity]
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
          @range = params[:range]

          if @filter.exist_any?(@tab) && !@tab.eql?(:sensor_name)
            @filter.exclude_empty(@tab)
          end

          @serie = cardinality_tab_data(@tab)
          @total = @serie.empty? ? 0 : cardinality_total(@tab)
        end

        def load_cardinality_table
          @tab = params[:id].to_sym
          @group = assign_param(@source::COLUMNS.keys, params[:group_by].to_sym)
          @start_time = Time.at(params[:start_time].to_i)
          @end_time = Time.at(params[:end_time].to_i)
          @granularity = params[:granularity]
          @range = params[:range]

          # Delete cardinality table
          session[@source.session].delete(:cardinality_group_by)
          @content = cardinality_table(@tab)

          set_total_options

          respond_to do |format|
            format.js
            format.json {
              render json: @content.map do |s|
                { result: s.result,
                  @group.to_sym => s.read_attribute(@group),
                  secondary_name:
                    if (@source::COLUMNS[@group.to_sym][:complementary_columns] && @group.to_sym != :sensor_name)
                      s.send(@source::COLUMNS[@group.to_sym][:complementary_columns].keys.first.to_s)
                    else
                      s.read_attribute(@group)
                    end
                }
              end
            }
          end
        end

        def get_cardinality_serie
          @tab = params[:id].to_sym
          @value = params[:value]
          @group = params[:group_by]
          @granularity = params[:granularity]
          if @group
            @filter.remove(@group.to_sym) if @filter.get.keys.include?(@group.to_sym)
            @filter.add(@group, [@value]) unless @value.nil?
          end
          @serie = cardinality_tab_data(@tab)

          respond_to do |format|
            format.js
            format.json { render json: @serie }
          end
        end

        def load_more_cardinality
          @tab = params[:tab]
          @group = params[:group]
          @start_time = Time.at(params[:start_time])
          @end_time = Time.at(params[:end_time])
          @offset = params[:offset].to_i
          @granularity = params[:granularity]
          @range = params[:range]

          @tops_cardinality = cardinality_table(@tab)
        end

        def add_cardinality_tab
          @tab = params[:id]
          @tabs = load_columns.keys
          session[@source.session][:active_cardinality_tabs] << @tab.to_sym
          session[@source.session][:cardinality_tab] = @tab.to_sym

          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time
          @granularity = params[:granularity]
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
          @range = params[:range]

          @filter.exclude_empty(@tab)

          # Fetch data from common queries
          @serie = cardinality_tab_data(@tab)
          @total = @serie.empty? ? 0 : cardinality_total(@tab)
        end

        def del_cardinality_tab
          @tab = params[:id]
          @tabs = load_columns.keys
          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time
          @range = params[:range]
          @granularity = params[:granularity]

          return @tab = nil unless session[@source.session][:active_cardinality_tabs].count > 1
          session[@source.session][:active_cardinality_tabs].delete(@tab.to_sym)
          @new_tab = session[@source.session][:active_cardinality_tabs].first.to_s
        end

        def export_cardinality_to_csv
          @tab = session[@source.session][:cardinality_tab]
          @group = params[:group]
          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time

          @available_tabs = session[@source.session][:active_cardinality_tabs].collect{|k| [@source::COLUMNS[k.to_sym][:name].present? ? @source::COLUMNS[k.to_sym][:name] : @source::COLUMNS[k.to_sym][:columns].values.first, k]}
          @available_tabs = @available_tabs = @available_tabs.collect{|k| if k[0].start_with? "tr." then ["#{I18n.t k[0][3..-1]}", k[1]] else [k[0], k[1]] end}

          @group_by_tabs = @source::COLUMNS.collect{|k, v| [v[:columns][k], k] if v[:columns][k]}.compact
          @group_by_tabs = @group_by_tabs = @group_by_tabs.collect{|k| if k[0].start_with? "tr." then ["#{I18n.t k[0][3..-1]}", k[1]] else [k[0], k[1]] end}

          render layout: false
        end

        def export_cardinality_to_file
          tab = params[:tab]

          tab = case tab.to_sym
                when :src_map
                  :src
                when :dst_map
                  :dst
                else
                  tab
                end

          start_time = params[:start_time].to_time
          end_time   = params[:end_time].to_time
          group = params[:group]

          csv_info = @source.where(
            query_type: :group_by_cardinality,
            cardinality_column: tab,
            group_by: @source::COLUMNS[group.to_sym][:columns].keys,
            range: :custom,
            start_time: start_time,
            end_time: end_time,
            sort: [:result, @source::COLUMNS[group.to_sym][:columns].keys.first],
            granularity: :all,
            limit: params[:limit],
            aggregation: @aggregation,
            search: @filter.get
          )

          filename = in_time_machine { Time.now.strftime("%Y%m%d%H%M%S") + "-#{@source}Unique_" + "#{tab.to_s}_#{group.to_s}" + ".csv" }
          content = @source.to_csv(csv_info, group.to_s, ['result'])
          send_data content, filename: filename
        end
      end
    end
  end
end
