#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common views
    #
    module View
      #
      # Define the comon methods for Comparative views
      #
      module Comparative
        # Includes
        include Redborder::Rbdruid

        def comparative
          @tabs = COMPARATIVE_COLUMNS.keys
          get_active_tabs(:active_comparative_tabs)
          session[@source.session][:comparative_tab] = session[@source.session][:active_comparative_tabs].first unless session[@source.session][:active_comparative_tabs].include?(session[@source.session][:comparative_tab])

          if params[:compare_range]
            session[@source.session][:active_comparative_tabs] << params[:compare_range].to_sym unless session[@source.session][:active_comparative_tabs].include?(params[:compare_range].to_sym)
            session[@source.session][:comparative_tab] = params[:compare_range].to_sym
          end

          @granularity = :all
          @elements = {}
          session[@source.session][:active_comparative_tabs].each do |tab|
            @tab = tab.to_sym

            if @filter.exist_any?(:end_time)
              @end_time = Time.at(@filter.get('end_time').keys.first)
              @start_time = COMPARATIVE_COLUMNS[@tab][:start].call(Time.at(@filter.get('end_time').keys.first))
            else
              @end_time = in_time_machine { Time.zone.now }
              @start_time = in_time_machine { COMPARATIVE_COLUMNS[@tab][:start].call(Time.zone.now) }
            end

            step = GRANULARITY_TYPE[COMPARATIVE_COLUMNS[@tab][:granularity][:graph]][:step]

            # Get the data from common methods
            content = comparative_tab_data
            total = comparative_total

            # Find past events if there are no events right now
            content = past_comparative_series if content.blank?

            if content.present?
              new_event = { 'timestamp' => @start_time }
              @aggregations.each do |aggregation|
                new_event[aggregation.to_s] = content.inject(0) do |res, elem|
                  res += elem.send(aggregation)
                end
              end
              content = [@source.new(current_user, new_event)]
            end

            # Set the graph domain from the first event
            domain = get_domain_graph_from(content)

            @elements[@tab] = {
              content: content.nil? ? [] : content,
              total: total,
              step: step,
              domain: domain
            }
          end
        end

        def get_comparative_serie
          @action = 'comparative'
          # Value is unix timestamp and is used to active serie
          @value = @value.to_time.to_i
          @granularity = COMPARATIVE_COLUMNS[@tab.to_sym][:granularity][:graph]
          @start_time = in_time_machine { params[:start_time_serie].to_time }
          @end_time = in_time_machine { params[:end_time_serie].to_time }
          if @tab.to_sym.eql?(:pt24h)
            @serie = comparative_tab_data(@aggregation)
                     .map do |t|
                       {
                         event: { events: t.send(@aggregation) },
                         timestamp: (t.timestamp - t.timestamp.hour.hours)
                                    .change(
                                      day: in_time_machine { Time.zone.now.day },
                                      month: in_time_machine { Time.zone.now.month },
                                      year: in_time_machine { Time.zone.now.year }
                                    ).to_s
                       }
                     end
          else
            @serie = comparative_tab_data(@aggregation)
                     .map do |t|
                       {
                         event: { events: t.send(@aggregation) },
                         timestamp: (t.timestamp + params[:diff].to_i).to_s
                       }
                     end
          end
        end

        def add_comparative_tab
          @tabs = COMPARATIVE_COLUMNS.keys
          session[@source.session][:active_comparative_tabs] << @tab.to_sym
          session[@source.session][:comparative_tab] = @tab.to_sym

          @granularity = :all
          @start_time = in_time_machine { COMPARATIVE_COLUMNS[@tab.to_sym][:start].call(Time.zone.now) }
          step = GRANULARITY_TYPE[COMPARATIVE_COLUMNS[@tab.to_sym][:granularity][:graph]][:step]
          @end_time = in_time_machine { Time.zone.now }

          # Get the data from common methods
          content = comparative_tab_data
          total = comparative_total

          # Find past events if there are no events right now
          content = past_comparative_series if content.blank?

          if content.present?
            new_event = { 'timestamp' => @start_time }
            @aggregations.each do |aggregation|
              new_event[aggregation.to_s] = content.inject(0) do |res, elem|
                res += elem.send(aggregation)
              end
            end
            content = [@source.new(current_user, new_event)]
          end

          # Set the graph domain from the first event
          domain = get_domain_graph_from(content)

          @elements = {
            content: content.nil? ? [] : content,
            total: total,
            step: step,
            domain: domain
          }
        end

        def del_comparative_tab
          @tabs = COMPARATIVE_COLUMNS.keys

          return @tab = nil unless session[@source.session][:active_comparative_tabs].count > 1
          session[@source.session][:active_comparative_tabs].delete(@tab.to_sym)
          @new_tab = session[@source.session][:active_comparative_tabs].first.to_s
        end

        def load_more_comparative
          @tab = params[:tab]
          @first_event = params[:first_event]
          @granularity = :all
          step = GRANULARITY_TYPE[COMPARATIVE_COLUMNS[@tab.to_sym][:granularity][:table]][:step]

          @start_time = Time.at(
            (params[:start_time_serie].to_time.to_i * 1000 - step) / 1000
          )
          @end_time = Time.at(
            (params[:end_time_serie].to_time.to_i * 1000 - step) / 1000
          )

          return if @filter.exist_any?(:start_time) &&
            COMPARATIVE_COLUMNS[@tab.to_sym][:start].call(Time.at(@filter.get('start_time').keys.first)) > @start_time

          @serie_comparatives = comparative_tab_data

          @total = @serie_comparatives.present? ? comparative_total : 0

          past_comparative_series(true) if @serie_comparatives.size.zero?

          return if @serie_comparatives.empty?
          new_event = { 'timestamp' => @start_time }
          @aggregations.each do |aggregation|
            new_event[aggregation.to_s] = @serie_comparatives.inject(0) do |res, elem|
              res += elem.send(aggregation)
            end
          end
          @serie_comparatives = [@source.new(current_user, new_event)]
        end
      end
    end
  end
end
