#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common views
    #
    module View
      #
      # Define the comon methods for RAW views
      #
      module Raw
        # Includes
        include Redborder::Rbdruid
        #
        # => Show the events in RAW view
        #
        def index
          get_active_tabs(:active_columns)

          ranges = ["live","last_5_minute", "last_15_minute", "last_30_minute", "last_1_hour", "custom"]
          if (!ranges.include?@range or !params[:range])
            @range = 'live'
            params[:range] = 'live'
            @end_time = Time.now
            @start_time = @end_time- 3.minutes
            params[:end_time] = @end_time
            params[:start_time] = @start_time
          end

          @dimensions = load_columns
          @granularity = params[:granularity] || :all
          @offset = params[:offset].present? ? (params[:offset].to_i) : 0
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
          @end_time_graph = @end_time
          @element_graph = raw_graph if @granularity.to_sym != :all
          if params[:stream_columns].present?
            session[@source.session][:active_columns] = params[:stream_columns].map(&:to_sym)
          end

          @columns = (load_active_columns + @aggregations) << :timestamp
          # Get events and total
          #@element = raw_events
          @element = Array.new
          if session[@source.session][:chart_type][params[:action]].to_sym.eql?(:streams)
            begin
              @element_streams = raw_graph_events(raise_error = true)
            rescue Exception => e
              flash[:warning] = e.to_s
            end
          end
          @total = raw_total
        end

        def get_raw_table
          limit = params.fetch(:limit, current_user.per_page_count)
          limit = nil if limit == 'false'
          columns = params.fetch(:columns, [])

          @elements = @source.where(
            query_type: :group_by,
            range: :custom,
            start_time: params[:start_time].to_time,
            end_time: params[:end_time].to_time,
            granularity: params[:granularity],
            limit: limit,
            sort: @aggregation,
            direction: sort_direction,
            group_by: columns,
            aggregation: @aggregation,
            search: @filter.get
          )

          respond_to do |format|
            format.json { render json: @elements }
          end
        end

        def load_more
          @start_time = params[:start_time].to_time
          @end_time   = params[:end_time].to_time
          @granularity = params[:granularity]

          if @granularity.to_sym != :all
            @finished = (@end_time - GRANULARITY_TYPE[@granularity.to_sym][:step] / 1000) <= @start_time
            return if @finished
          end

          @dimensions = @source::COLUMNS

          @offset = (params[:offset].to_i || 0)
          @columns = (load_active_columns + @aggregations) << :timestamp
          begin
            @element = raw_events(raise_error = true)
          rescue Exception => e
            @element = nil
          end

          if session[@source.session][:chart_type]['index'].to_sym.eql?(:streams)
            #@element_streams = raw_graph_events
            @element_streams
          end
        end

        def load_new_data
          @start_time = Time.at(params[:start_time])
          @end_time   = Time.at(params[:end_time])
          @reload_index = params[:reload_index]

          get_active_tabs(:active_columns)
          @dimensions = @source::COLUMNS

          @granularity = params[:granularity]
          @offset = params[:offset].present? ? (params[:offset].to_i) : 0

          @columns = (load_active_columns + @aggregations) << :timestamp

          @element = raw_events
          @element_graph = raw_graph
        end

        def info
          @source.set_custom_values(params)
          # @aggregation = params[:aggregation]
          @source::COLUMNS.each{|k,v| (@keys ||= []) << k unless v[:hide_raw]}

          @elements = @source.where(
            query_type: :group_by,
            range: params[:range], # remove? set only for ips
            custom_filter: params[:custom_filter].except('timestamp'),
            granularity: params[:granularity],
            start_time: params[:start_time],
            end_time: params[:end_time],
            group_by: @keys,
            direction: params[:direction], # remove? set only for ips
            aggregation: @aggregations,
            search: @filter.get
          ).paginate(page: (params[:page] or 1), per_page: 5)
        end

        def element_childs
          parent_granularity = params[:granularity]

          if params[:granularity].to_sym == :minute
            redirect_to(action: :info, params: params.merge(aggregation: @aggregation)) and return
          elsif params[:granularity].to_sym == :all
            params[:start_time] = @start_time = params[:custom_filter][:timestamp].to_time
            params[:end_time] = @end_time = params[:custom_filter][:timestamp].to_time + 30.day
          end

          get_active_tabs(:active_columns)
          # @dimensions = @source::COLUMNS

          @source.set_custom_values(params)

          @granularity = (params[:granularity] || :minute)
          @parent_counter = "#{@source.session}_#{params[:custom_filter][:timestamp].to_time.to_i}_#{parent_granularity}_#{params[:element_counter]}"
          @offset = params[:offset].to_i
          @reload_index = params[:reload_index]

          @columns = (load_active_columns + @aggregations) << :timestamp

          @elements = @source.where(
            query_type: :group_by,
            range: params[:range], # remove? set only for ips
            start_time: params[:start_time],
            end_time: params[:end_time],
            granularity: params[:granularity],
            sort: sort_column, # remove? set only for ips
            direction: sort_direction, # remove? set only for ips
            custom_filter: params[:custom_filter].except('timestamp'),
            group_by: session[@source.session][:active_columns],
            aggregation: @aggregations,
            search: @filter.get
          )

          while @elements.size == 1 && params[:granularity] != :minute
            params[:custom_filter][:timestamp] = @elements.first.send('timestamp')
            @source.set_custom_values(params)
            @granularity = (params[:granularity] || :minute)

            @elements = @source.where(
              query_type: :group_by,
              range: params[:range], # remove? set only for ips
              start_time: params[:start_time],
              end_time: params[:end_time],
              granularity: params[:granularity],
              sort: sort_column, # remove? set only for ips
              direction: sort_direction, # remove? set only for ips
              custom_filter: params[:custom_filter].except('timestamp'),
              group_by: session[@source.session][:active_columns],
              aggregation: @aggregations,
              search: @filter.get
            )
          end
        end

        def export_raw_to_csv
          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time
          @granularity = params[:granularity]

          render layout: false
        end

        def export_raw_to_file
          start_time = params[:start_time].to_time
          end_time   = params[:end_time].to_time
          granularity = params[:granularity]
          sort = sort_column
          direction = sort_direction

          csv_info = @source.where(
            query_type: :group_by,
            start_time: start_time,
            end_time: end_time,
            granularity: granularity,
            sort: sort,
            direction: direction,
            limit: params[:limit],
            group_by: session[@source.session][:active_columns],
            aggregation: @aggregations,
            search: @filter.get
          )

          filename = in_time_machine { Time.now.strftime("%Y%m%d%H%M%S") + "-#{@source}RAW.csv" }
          content = @source.raw_to_csv(csv_info, session[@source.session][:active_columns], @aggregations)
          send_data content, filename: filename
        end

        #
        # [GET] Show a modal to reorder the tabs
        #
        # @todo : Is this method necesary?
        #
        def sort_columns
          @columns = session[@source.session][:active_columns]
                     .select { |k| !@inactive_settings.has_key?("#{@source.id}_#{k}") }
          render layout: false
        end

      end
    end
  end
end
