#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  #
  # Common functionality over modules
  #
  module Commons
    #
    # Common views
    #
    module View
      #
      # Define the comon methods for Tops views
      #
      module Tops
        # Includes
        include Redborder::Rbdruid

        def tops
          @tabs = @source::COLUMNS.keys
          @granularity = params[:granularity]
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]
          @filters_items = @filter.serialize(true)

          get_active_tabs(:active_tabs)
          unless session[@source.session][:active_tabs].include?(session[@source.session][:tab])
            session[@source.session][:tab] = session[@source.session][:active_tabs].first
          end

          if params[:tab]
            unless session[@source.session][:active_tabs].include?(params[:tab].to_sym)
              session[@source.session][:active_tabs] << params[:tab].to_sym
            end
            session[@source.session][:tab] = params[:tab].to_sym
          end

          respond_to do |format|
            format.any(:html, :js) {
              @element_graph = tops_graph
              @element_graph_filtered =
                tops_graph(@filter.get) unless @filter.get.empty?
              @total = total

              @element = {}

              session[@source.session][:active_tabs].each do |tab|
                tab = tab.to_sym
                if tab == session[@source.session][:tab].to_sym
                  case tab
                  when :src_map, :dst_map, :public_ip_map
                    @element[tab] = get_gmap_tabs(tab)
                  when :scatterplot
                    @element[:scatterplot] = get_scatterplot_tab
                  when :trafficmap
                    @element[:trafficmap] = get_trafficmap_tab
                  else
                    content = tops_tab_data(tab)

                    if tab.to_sym == :coordinates_map
                      @element[tab] = { content: content, graph: content }
                    else
                      @element[tab] = { content: (content || []) }
                    end
                  end
                else
                  @element[tab] = { content: [], graph: [] }
                end
              end
              set_total_options
            }
          end
        end

        def load_tab
          @tab = params[:id].to_sym
          @elements = {}
          @start_time = Time.at(params[:start_time].to_i)
          @end_time = Time.at(params[:end_time].to_i)
          # @aggregation = params[:aggregation]
          @total = params[:total].to_i
          @granularity = params[:granularity]
          @range = params[:range]

          case @tab
          when :src_map, :dst_map, :public_ip_map
            @elements[@tab.to_sym] = get_gmap_tabs(@tab)
          when :scatterplot
            @elements[:scatterplot] = get_scatterplot_tab
          when :trafficmap
            @elements[:trafficmap] = get_trafficmap_tab
          else
            content = tops_tab_data(@tab)
            if @tab == :coordinates_map
              @elements[@tab.to_sym] = { content: content, graph: content }
            else
              @elements[@tab.to_sym] = { content: (content || []) }
            end
          end
        end

        def load_more_tops
          @offset = params[:offset].to_i
          @tab = params[:tab]
          @start_time = params[:start_time]
          @end_time = params[:end_time]
          @total = params[:total]
          @granularity = params[:granularity]
          @range = params[:range]

          @series = @source.where(
            query_type: :group_by,
            sort: [@aggregation,
                   @source::COLUMNS[@tab.to_sym][:columns].keys.first],
            limit: current_user.per_page_count,
            offset: @offset,
            start_time: @start_time,
            end_time: @end_time,
            granularity: :all,
            group_by: @source::COLUMNS[@tab.to_sym][:columns].keys,
            aggregation: @aggregations,
            search: @filter.get
          )

          return unless [:src_map, :dst_map, :public_ip_map].include?(@tab.to_sym)
          ips = @series.collect(&@source::COLUMNS[@tab.to_sym][:ips])
          @graph = Redborder::Geoip.latlong(ips)
        end

        def get_tops
          tab = assign_param(@source::COLUMNS.keys, params[:id].to_sym)
          timezone = params[:timezone] # remove? removed in flow, setted in ips && social
          @serie = @source.where(
            query_type: :group_by,
            range: @range, # remove? setted in flow && social, removed in ips
            start_time: params[:start_time],
            end_time: params[:end_time],
            granularity: :all,
            sort: @aggregation,
            timezone: timezone, # remove? removed in flow, setted in ips && social
            limit: params.fetch(:limit, current_user.per_page_count),
            group_by: [@source::COLUMNS[tab.to_sym][:columns].keys[0]],
            aggregation: @aggregation,
            search: @filter.get
          )

          respond_to do |format|
            format.json {
              render json: @serie.map do|s|
                { tab.to_sym => s.read_attribute(tab),
                  @aggregation.to_sym => s.read_attribute(@aggregation),
                  secondary_name:
                    if @source::COLUMNS[tab.to_sym][:complementary_columns] && tab.to_sym != :sensor_name
                      s.send(@source::COLUMNS[tab.to_sym][:complementary_columns].keys.first.to_s)
                    else
                      s.read_attribute(tab)
                    end
                }
              end
            }
          end
        end

        def get_tops_serie
          @action  = 'tops'
          timezone = params[:timezone]

          granularities = Redborder::Rbdruid.granularity_list(
            start_time: params[:start_time].to_time,
            end_time: params[:end_time].to_time)
          @granularity  = if params[:granularity].present?
                            params[:granularity]
                          else
                            granularities.first
                          end

          if @value.nil?
            search = @filter.get if params[:total].nil?
            @serie = @source.timeserie(
              range: @range,
              start_time: params[:start_time],
              end_time: params[:end_time],
              granularity: @granularity,
              aggregation: @aggregation,
              timezone: timezone,
              skip_empty: @should_skip_empty, # ??
              search: search
            )
          elsif @tab.to_sym.eql?(:conversation)
            if @source.can_filter?(:src) && @source.can_filter?(:dst)
              @filter.remove('src') and @filter.add('src', [@value.first])
              @filter.remove('dst') and @filter.add('dst', [@value.last])
            elsif @source.can_filter?(:lan_ip) && @source.can_filter?(:wan_ip)
              @filter.remove('lan_ip') and @filter.add('lan_ip', [@value.first])
              @filter.remove('wan_ip') and @filter.add('wan_ip', [@value.last])
            end
            @serie = @source.timeserie(
              range: @range,
              start_time: params[:start_time],
              end_time: params[:end_time],
              granularity: @granularity,
              aggregation: @aggregation,
              timezone: timezone,
              skip_empty: @should_skip_empty,
              search: @filter.get
            )
            @value = params[:index]
          else
            @value = nil if params[:value].eql?('N/A')

            case @tab.to_sym
            when :wireless_station
              @filter.remove :zone
            when :sensor_name
              @filter.remove :domain
            end

            # Remove the column before add the value. If we hold all filters,
            # all graphs will be the same
            @filter.remove(@tab)
            @filter.add(@tab.to_sym, [@value])
            @serie = @source.timeserie(
              range: @range,
              start_time: params[:start_time],
              end_time: params[:end_time],
              granularity: @granularity,
              aggregation: @aggregation,
              group_by: @tab, # remove?
              timezone: timezone,
              skip_empty: @should_skip_empty, # remove?
              search: @filter.get
            )
          end
        end

        def add_tops_tab
          @tabs = @source::COLUMNS.keys
          session[@source.session][:active_tabs] << @tab.to_sym
          session[@source.session][:tab] = @tab.to_sym

          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time
          @granularity = params[:granularity]
          @rangemilis = ((@end_time - @start_time) * 1000).to_i
          @granularitymilis = GRANULARITY_TYPE[@granularity.to_sym][:step]

          if [:src_map, :dst_map, :public_ip_map].include?(@tab.to_sym)
            info = get_gmap_tabs(@tab.to_sym)
            @elements, @graph = info[:content], info[:graph]
          elsif @tab.to_sym == :scatterplot
            @elements = get_scatterplot_tab[:content]
            @total = 0
          elsif @tab.to_sym == :trafficmap
            @elements = get_trafficmap_tab[:content]
            @total = 0
          else
            @elements = tops_tab_data(@tab.to_sym)
            @graph = @elements if @tab.to_sym == :coordinates_map
          end

          @total =  if @elements.present?
                      @source.total(
                        rage: @range,
                        start_time: @start_time,
                        end_time: @end_time,
                        granularity: :all, # remove?
                        sort: @aggregation, # remove?
                        aggregation: @aggregation,
                        search: @filter.get
                      )
                    else
                      0
                    end

          set_total_options
        end

        def del_tops_tab
          @tabs = @source::COLUMNS.keys
          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time
          @granularity = params[:granularity]

          return @tab = nil unless session[@source.session][:active_tabs].count > 1
          session[@source.session][:active_tabs].delete(@tab.to_sym)
          @new_tab = session[@source.session][:active_tabs].first.to_s
          set_total_options
        end

        def export_tops_to_csv
          @tab = session[@source.session][:tab]
          @start_time = params[:start_time].to_time
          @end_time = params[:end_time].to_time

          @available_tabs = session[@source.session][:active_tabs].collect{|k| [@source::COLUMNS[k.to_sym][:name].present? ? @source::COLUMNS[k.to_sym][:name] : @source::COLUMNS[k.to_sym][:columns].values.first, k]}
          @available_tabs = @available_tabs.collect{|k| if k[0].start_with? "tr." then ["#{I18n.t k[0][3..-1]}", k[1]] else [k[0], k[1]] end}

          render layout: false
        end

        def export_tops_to_file
          tab = params[:tab]

          tab = case tab.to_sym
                when :src_map
                  :src
                when :dst_map
                  :dst
                else
                  tab
                end

          start_time = params[:start_time].to_time
          end_time   = params[:end_time].to_time
          range      = :custom

          csv_info = @source.where(
            query_type: :group_by,
            range: range,
            start_time: start_time,
            end_time: end_time,
            sort: [@aggregation,
                   @source::COLUMNS[tab.to_sym][:columns].keys.first],
            limit: params[:limit],
            granularity: :all,
            group_by: @source::COLUMNS[tab.to_sym][:columns].keys,
            aggregation: @aggregations,
            search: @filter.get
          )

          filename = in_time_machine { Time.now.strftime("%Y%m%d%H%M%S") + "-#{@source}Tops_" + tab.to_s + ".csv" }
          content = @source.to_csv(csv_info, tab.to_s, @aggregations)
          send_data content, filename: filename
        end

        def get_table
          limit = params.fetch(:limit, current_user.per_page_count)
          limit = nil if limit == 'false'
          columns = params.fetch(:columns, [])

          @elements = @source.where(
            query_type: :group_by,
            range: :custom,
            start_time: params[:start_time].to_time,
            end_time: params[:end_time].to_time,
            granularity: params[:granularity],
            limit: limit,
            sort: @aggregation,
            direction: sort_direction,
            group_by: columns,
            aggregation: @aggregation,
            search: @filter.get
          )

          respond_to do |format|
            format.json { render json: @elements }
          end
        end

      end
    end
  end
end
