#
# Base redBorder module
#
module Redborder
  #
  # Define the basic methods to cache some variables for Druid Queries
  #
  module DataSourceCache
    #
    # Cache the IP of a sensor.
    #
    # == Parameters:
    # name::
    #   String with the name of the sensor.
    # type::
    #   Integer or Array of integer with the types of sensors to search
    #
    # == Returns:
    # String with the IP of the sensor.
    #
    def get_sensor_ip(name, type)
      return nil if name.nil? || type.nil?
      Rails.cache.fetch "rb-webui:sensor:#{type}:#{name}", expires_in: 1.day do
        Sensor.find_by(name: name, type: type).try('ip') || 'N/A'
      end
    end
    #
    # Cache the vendor of a given MAC
    #
    # == Parameters:
    # mac::
    #   String with the MAC
    #
    # == Returns:
    # An string with the MAC vendor
    #
    def get_mac_vendor_full(mac)
      return nil unless mac
      Rails.cache.fetch "rb-webui:macvendor:#{mac}", expires_in: 1.day do
        Redborder::RbMacVendor.lookup(mac)
      end
    end

    #
    # Get the country name based on country code. This method read codes from
    # redBorder::COUNTRIES
    #
    # == Parameters:
    # cc::
    #   String with the country code
    #
    # == Returns::
    # String with the country name
    #
    def get_country_name(cc = nil)
      return nil if cc.nil?
      # Store the value on cache
      Rails.cache.fetch "rb-webui:country:#{cc}", expires_in: 1.day do
        RedBorder::COUNTRIES[cc] || 'N/A'
      end
    end
  end
end
