#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  #
  # Static factory to abstract the process to get data from a Druid source.
  # Every module define his druid model if it's exists, and based on the type of
  # the query, call to the real druid model of the module.
  #
  module DruidSource
    #
    # Initialize the DruidSource based on the product and the registered modules
    # in the platform.
    #
    # == Parameters:
    # args::
    #   Hash with the initialize arguments. The source argument define the
    #   Data Source to get data.
    #
    def self.get(args = {})
      source_name = args.fetch(:source, nil)
      unless source_name
        fail NotImplementedError, "#{I18n.t 'lib_redborder.specify'}" \
                                  "#{I18n.t 'lib_redborder.get'}"
      end

      # Try to initiate the module
      begin
        Redborder::Module
          .get(source_name.to_sym, :data_source)
          .constantize
      rescue NoMethodError
        # Raise an exception because the DataSource isn't implemented!
        raise NotImplementedError, "#{I18n.t 'lib_redborder.with_name'}" \
                                   "#{source_name} #{I18n.t 'lib_redborder.doesnt_exist'}"
      end
    end
  end
end
