#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module ExportableUtils
    EXTRACT_PATH = "#{Rails.root}/tmp/"

    #
    # => Extract the file in the EXTRACT_PATH and returns
    #    the folder path with the decompressed files
    #
    def self.extract_files(file)
      # Path of the tempfile and its name
      path = file.tempfile.path
      filename = file.original_filename
      fail "#{I18n.t 'lib_redborder.not_empty'}" if path.nil? || path.empty?

      # Folder where the file will be extracted
      folder = EXTRACT_PATH + "imported-#{SecureRandom.uuid}"

      # Remove tmp dir if it exists
      FileUtils.rm_rf folder

      # Create needed directory
      Dir.mkdir folder

      # Decompress the file
      untar filename, path, folder

      # Returns the folder
      folder
    end

    #
    # => Creates a folder to export an element
    #    Returns the folder where the element can be placed
    #
    def self.generate_export_folder
      # The folder where all will be compressed
      folder = EXTRACT_PATH + "exported-#{SecureRandom.uuid}"

      # Remove tmp dir if it exists
      FileUtils.rm_rf folder

      # Create needed directories
      Dir.mkdir folder
      Dir.mkdir "#{folder}/media"

      folder
    end

    #
    # => Compress a folder using the tar command
    #
    def self.compress_folder(folder, name)
      # Create the tar file
      system "cd #{folder}; tar czf exportable.tar.gz *"

      # Return the tar file path
      "#{folder}/exportable.tar.gz"
    end

    private

    #
    #  => Uses the tar command to decompress the file
    #
    def self.untar(filename, path, folder)
      # Check the extension of the file
      if filename.match(/(tgz|gz)$/)
        # .tar.gz file
        system('tar', 'xzf', path, '-C', folder)
      elsif filename.match(/(tar)$/)
        # .tar file, the need to delete the -z option
        system('tar', 'xf', path, '-C', folder)
      else
        # Extension not allowed
        fail "#{I18n.t 'lib_redborder.extension'}"
      end
    end
  end
end
