#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

#
# Base redBorder module
#
module Redborder
  module Filters
    #
    # Module to works with storage of filters.
    #
    module Storage
      #
      # Session storage for the filters
      #
      module Session
        # Id to store filters
        ID = :filters

        #
        # Load filters from the storage
        #
        # == Returns:
        # Hash with the current filters
        #
        def load_filters
          # Use deep_dup to prevent autosave when change the @filters value
          # on filter class.
          session[ID].deep_dup || {}
        end

        #
        # Save the filters into the storage.
        #
        # == Parameters:
        # filters::
        #   A Hash with the filters to store
        #
        def save_filters(filters = {})
          # We must perform a deep_dup to prevent session[ID] link on filters.
          session[ID] = filters.deep_dup
        end
      end
    end
  end
end
