#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
    module Jobs
      class AssignMitreJob < Struct.new(:verbose, :user_id)

        include Redborder::Jobs::JobHelper
  
        def perform
          content = "Running AssignMitreJob...\n"
          begin
            rule_assigner = RedBorder::IntrusionRulesToMitre.new
            rule_assigner.start
            content += "Number of affected rules: #{rule_assigner.get_affected_rules}\n"
          rescue Exception => e
            content += log_exception(e)
            logit("#{e.class}: #{e.message}") if @verbose
          ensure
            StoredDelayedJob.create(job: "AssignMitreJob", content: content, user_id: user_id)
          end
        end

        def log_exception(exception)
          logit(exception)
          logit(exception.backtrace.join("\n"))
          exception.inspect
        end
      end
    end
  end