#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    #
    # Delete sensors that are mark as deleted if no DeleteSensorJob is enqueue
    # As well, update IPS sensors as not applying when no apply job is enqueue
    #
    class CheckCorruptDataJob < Struct.new(:verbose)
      include Redborder::Jobs::JobHelper

      def perform
        # Check deleted sensors
        unless DelayedJob.deleting_sensors?
          Sensor.unscoped.where(is_deleted: true).destroy_all
        end

        # Check orphan sensors
        Sensor.where.not(id: Sensor.root.id).find_each do |sensor|
          sensor.destroy unless Sensor.exists?(sensor.parent_id)
        end

        # Check IPS sensors that are applying changes
        if Redborder::Module.registered_modules.include?(:ips) &&
           !DelayedJob.apply_update?
          Sensor.unscoped.where(applying: true).update_all(applying: false)
        end

       # Check Corrupted Update Rules Job
       # If its running on a non existing worker, destroy it and creates a new one under the same user
       rule_update_job = DelayedJob.rule_update
       if rule_update_job && rule_update_job.running? && (!DelayedJob.process.map(&:pid).include?rule_update_job.pid)
             user_id = rule_update_job.user_id
             rule_update_job.destroy!
             RuleSource.where(is_custom: false, enable: true).update_all(updating: true)
             Delayed::Job.enqueue(Redborder::Jobs::RuleUpdatedbJob.new(user_id, true), priority: 10, run_at: Time.now + 5.seconds)
       end

       # Check Corrupted Update Vulnerabilities Job
       # If its running on a non existing worker, destroy it and creates a new one under the same user
       rb_vulnerabilities_update_db_job = DelayedJob.rb_vulnerabilities_update_db
       if rb_vulnerabilities_update_db_job && rb_vulnerabilities_update_db_job.running? && (!DelayedJob.process.map(&:pid).include?rb_vulnerabilities_update_db_job.pid)
             user_id = rb_vulnerabilities_update_db_job.user_id
             rb_vulnerabilities_update_db_job.destroy!
             Delayed::Job.enqueue(Redborder::Jobs::RbVulnerabilitiesUpdateDBJob.new(user_id, true),
                          priority: 5, run_at: Time.now + 2.seconds)
       end

       plugin_update_job = DelayedJob.plugin_update
       if plugin_update_job && plugin_update_job.running? && (!DelayedJob.process.map(&:pid).include?plugin_update_job.pid)
             user_id = plugin_update_job.user_id
             plugin_id = plugin_update_job.plugin_id
             version = plugin_update_job.version
             enabled = plugin_update_job.enabled
             plugin_update_job.destroy!
             
             Delayed::Job.enqueue(Redborder::Jobs::UpdatePluginJob.new(plugin_id, user_id, version, enabled), priority: 10, run_at: Time.now + 5.seconds)
       end

      rescue StandardError => e
        logit "#{e}"
        logit e.backtrace.join("\n")
      ensure
        Delayed::Job.enqueue(
          Redborder::Jobs::CheckCorruptDataJob.new(true),
          priority: 5,
          run_at: 5.minutes.from_now
        )
      end
    end
  end
end
