#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class CheckReportsJob < Struct.new(:verbose)

      def perform
        Report.all.each do |report|
          report_days = report.schedule_days.try("split",",") || []
          hour_time_now = Time.now.beginning_of_hour.strftime("%H:%M")

          day = if report.schedule_type == "weekly"
            Time.now.wday.zero? ? "7" : Time.now.wday.to_s
          else
            Time.now.day == Time.now.end_of_month.day ? "28" : Time.now.day.to_s
          end

          Redborder::Jobs::SendReportJob.new(report.id, true).perform if report.schedule_hour == hour_time_now &&
                                                                         report_days.include?(day)
        end

        # Enqueue the job again
        Delayed::Job.enqueue(Redborder::Jobs::CheckReportsJob.new(true), priority: 10, run_at: (Time.now + 1.hour).beginning_of_hour)
      end
    end
  end
end
