#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class DefaultDashboardsJob < Struct.new(:verbose, :owner)

      include Redborder::Jobs::JobHelper

      def perform
        content = "Creating default dashboards for user #{owner.name}:\n"
        begin
          dashboards = Dir.glob("#{Rails.root}/files/dashboards/*.tar.gz")
          dashboards.each do |dashboard|
            filename = File.basename(dashboard)
            content += "* Creating #{filename} dashboard.\n"
            file_params = ActionDispatch::Http::UploadedFile.new(
              tempfile: File.new(dashboard),
              filename: filename
            )
            d = Dashboard.import(file_params)
            d.owner = owner
            d.save
          end
        rescue StandardError => e
          content += "#{e.class}: #{e.message}\n"
          e.backtrace.each do |line|
            content += "#{line}\n"
          end
          logit "#{e.class}: #{e.message}" if verbose
        ensure
          StoredDelayedJob.create(job: 'DefaultDashboardsJob', user_id: owner.id, content: content)
        end
      end
    end
  end
end