#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class GeoipUpdateJob < Struct.new(:user_id, :verbose)
      
      include Redborder::Jobs::JobHelper

      def perform

        @content = ("Node executing this job: " + `hostname`.chomp! + "\n\n")

        begin

          if Setting.geoip_country_custom == "0"
            @content += "Updating GeoIP database..."
            update("country", Setting.geoip_country, "#{Rails.root}/tmp/GeoIP-tmp.dat")
          end

          if Setting.geoip_countryv6_custom == "0"
            @content += "Updating GeoIP IPv6 database..."
            update("countryv6", Setting.geoip_countryv6, "#{Rails.root}/tmp/GeoIPv6-tmp.dat")
          end

          if Setting.geoip_city_custom == "0"
            @content += "Updating GeoLiteCity database..."
            update("city", Setting.geoip_city, "#{Rails.root}/tmp/GeoLiteCity-tmp.dat")
          end

          if Setting.geoip_cityv6_custom == "0"
            @content += "Updating GeoLiteCity IPv6 database..."
            update("cityv6", Setting.geoip_cityv6, "#{Rails.root}/tmp/GeoLiteCityv6-tmp.dat")
          end

          if Setting.geoip_asn_custom == "0"
            @content += "Updating GeoIPASNum database..."
            update("asn", Setting.geoip_asn, "#{Rails.root}/tmp/GeoIPASNum-tmp.dat")
          end

          if Setting.geoip_asnv6_custom == "0"
            @content += "Updating GeoIPASNum IPv6 database..."
            update("asnv6", Setting.geoip_asnv6, "#{Rails.root}/tmp/GeoIPASNumv6-tmp.dat")
          end

          @content += `sudo /opt/rb/bin/rb_update_geodb_cookbook.sh`

          if @content.blank?
            @content += 'Nothing to do.'
          end

          hour, minute = Setting.rule_update_time.split(':').map(&:to_i)

          if Setting.rule_update_periodicity == 'weekly'
            Delayed::Job.enqueue(Redborder::Jobs::GeoipUpdateJob.new(user_id, verbose),
                                 priority: 10,
                                 run_at: 1.week.from_now.beginning_of_day + hour.hour + minute.minute)
          else
            Delayed::Job.enqueue(Redborder::Jobs::GeoipUpdateJob.new(user_id, verbose),
                                 priority: 10,
                                 run_at: 1.day.from_now.beginning_of_day + hour.hour + minute.minute)
          end

        rescue Exception => e
          logit e
          logit e.backtrace.join("\n")
          @content += "#{e.inspect}\n"
        ensure
          stored_job = StoredDelayedJob.create(job: 'GeoipUpdateJob',
                                               content: @content,
                                               user_id: user_id)
        end
      end

      def update(type, address, tmp_file)
        uri = URI address

        if Setting.proxy?
          proxy_class = Net::HTTP::Proxy(Setting.proxy_address, Setting.proxy_port, Setting.proxy_user, Setting.proxy_password)
          http = proxy_class.new(uri.host, uri.port)
          http.use_ssl = true if uri.scheme == "https"
          resp = http.get(uri.request_uri)
        else
          http = Net::HTTP.new(uri.host, uri.port)
          http.use_ssl = true if uri.scheme == "https"
          resp = http.get(uri.request_uri)
        end

        if resp.is_a?(Net::HTTPOK)
          open(tmp_file, "wb") do |file|
            gz = Zlib::GzipReader.new(StringIO.new(resp.body.to_s))
            file.write(gz.read)
          end
        end

        md5sum_old = Setting.send("geoip_#{type}_md5")
        md5sum_new = Digest::MD5.hexdigest(File.read(tmp_file))

        if md5sum_old == md5sum_new
          @content += " (not modified)\n"
          return false
        elsif Redborder::Geoip.check_database(tmp_file, type)
          @content += " (updated)\n"
          FileUtils.cp(tmp_file, "#{Rails.root}/tmp/geodb/#{type}.dat")
          Setting.send("geoip_#{type}_date=", Time.now.to_i)
          Setting.send("geoip_#{type}_md5=", Digest::MD5.hexdigest(File.read(tmp_file)))
          Setting.send("geoip_#{type}_date=", Time.now.to_i)
          return true
        else
          @content += " (not a valid file)\n"
          return false
        end

      end

    end

  end

end