#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
require 'syslog'

module Redborder
  module Jobs

    class IncidentToSyslogJob < Struct.new(:user_id, :verbose)
      attr_accessor :content
      include Redborder::Jobs::JobHelper

      def perform
        content = "Node executing this job: #{`hostname`.chomp!}\n\n"
        syslog_configured = false
        begin
          # Fetching the user by user_id, return if no user found
          user = User.find_by(id: user_id)
          return unless user

          syslog_configured = SyslogService.syslog_integration_configured?

          # Get the params to avoid to continue
          unless syslog_configured
            content += 'Syslog is not configured yet\n'
          else          
            incidents = get_alive_incidents
            content += "Incidents to check: #{incidents.count}\n"
            search_params = {}

            # Processing each incident
            incidents.each do |incident|
              pending = incident.send_pending_syslog_messages
              content += "\nIncident #{incident.id} checked. Pending messages sent #{pending}.\n"
             end
          end
        rescue => e
          # Handle any errors during execution, log backtrace and raise the exception
          e.backtrace.each do |line|
            content += "#{line}\n"
          end
          e.set_backtrace([])
          logit "#{e.class}: #{e.message}" if verbose
          raise e
        ensure
          StoredDelayedJob.create(job: "IncidentToSyslogJob", user_id: user_id, content: content)

          if syslog_configured && DelayedJob.where('handler like ?',
            '%!ruby/struct:Redborder::Jobs::IncidentToSyslogJob%').count <= 1
            # Re-enqueue the job to run again after 10 minutes
            Delayed::Job.enqueue(self.class.new(user_id, verbose), run_at: Time.now + 10.minutes)
          end
        end
      end
      
      private

      def get_alive_incidents
        start_time = 24.hours.ago
        end_time = Time.now
        Incident.where(created_at: start_time..end_time, visible: true)
      end
    end
  end
end