#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class ModelUpdatedbJob < Struct.new(:user_id, :verbose)
      
      include Redborder::Jobs::JobHelper

      def perform
        ModelSource.where(enable: true).update_all(updating: true)
        content = Redborder::ModelsUpdate.start(true)
        is_job_failed = Redborder::ModelsUpdate.get_failed
      ensure
        ModelSource.all.update_all(updating: false)
        sources_names = ModelSource.where(enable: true).map(&:clean_name).join(', ')
        StoredDelayedJob.create(user_id: user_id, job: "ModelUpdatedbJob", options: { humanize: sources_names }, content: content)
        if is_job_failed
          attempts = Delayed::Job.where(handler: self.to_yaml).pluck(:attempts).first
          next_run_time = reschedule_at(Time.now, attempts)
          # The job is queued manually, so we need to update and check the number of attempts manually
          if attempts + 1 < 3
            Delayed::Job.enqueue(self.class.new(user_id, verbose), run_at: next_run_time, attempts: attempts + 1, priority: 10, last_error: nil)
          end
        end
      end

      def reschedule_at(current_time, attempts)
        current_time.advance(seconds: 5)
      end

    end

  end

end