#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class OutlierIncidentJob < Struct.new(:verbose)
      include Redborder::Jobs::JobHelper

      def perform
        content = "Executing OutlierIncidentJob.. \n"

        unless outliers_service_enabled?
          content += "\nrb-aioutliers service is disabled. \nEnable the service and anomaly detection functionality in General Settings to execute OutlierIncidentJob job successfully."
          user_id = User.first.id
          StoredDelayedJob.create(user_id: user_id, job: "OutlierIncidentJob", content: content)
          return
        end

        begin
          log_function_call = ""

          incidents, log_function_call = Redborder::IncidentsOutliers.create_incidents_for_all_sensors

          content += log_function_call

          content += "Number of new outlier incidents: #{incidents}"

          next_run_time = reschedule_at(Time.now)
          if DelayedJob.where('handler like ?',
            '%!ruby/struct:Redborder::Jobs::OutlierIncidentJob%').count <= 1
            Delayed::Job.enqueue(Redborder::Jobs::OutlierIncidentJob.new(true), priority: 10, run_at: next_run_time)
          end
        rescue Exception => e
          logit "#{e}"
          content += "#{e}\n"
        ensure
          user_id = User.first.id
          stored_job = StoredDelayedJob.create(user_id: user_id, job: "OutlierIncidentJob", content: content)
        end
      end

      private

      def reschedule_at(current_time)
        if Setting.incident_update_periodicity == "weekly"
          hour, minute = Setting.incident_update_time.split(":").map(&:to_i)
          current_time.beginning_of_week.advance(weeks: 1, hours: hour, minutes: minute)
        elsif Setting.incident_update_periodicity == "daily"
          hour, minute = Setting.incident_update_time.split(":").map(&:to_i)
          current_time.beginning_of_day.advance(days: 1, hours: hour, minutes: minute)
        elsif Setting.incident_update_periodicity == "hourly"
          current_time.advance(hours: 1)
        else
          current_time.advance(hours: 12)
        end
      end

      def outliers_service_enabled?
        Manager.all.any? do |manager|
          manager.node['redborder']['services']['rb-aioutliers'] &&
          manager.node['redborder']['manager']['services']['current']['rb-aioutliers'] &&
          Setting.forecast?
        end
      end
    end
  end
end
