#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
module Redborder
  module Jobs
    class ResponseActionJob < Struct.new(:verbose, :user_id, :response_action_id, :integration_service, :action, :action_params)
      include Redborder::Jobs::JobHelper

      def perform
        content = ''
        begin
          content += "Excuting action #{action} of service #{integration_service}..\n"
          content += "Action Input Variables: #{action_params}\n"
          service = IntegrationService.new(integration_service)
          content += service.execute(action, action_params).to_s
          content += "\n"
        rescue StandardError => e
          content += "#{e.class}: #{e.message}\n"
          e.backtrace.each do |line|
            content += "#{line}\n"
          end
          logit "#{e.class}: #{e.message}" if verbose
        ensure
          StoredDelayedJob.create(
            job: 'ResponseActionJob',
            user_id: user_id,
            content: content,
            options: { response_action_id: response_action_id }
          )
        end
      end
    end
  end
end
