#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class SendReportJob < Struct.new(:report_id, :verbose)
      def perform
        report = Report.find(report_id)
        send_report(report)
      end

      def send_report(report)
        # Check if we have a list of emails
        return if report.email_list.nil? || report.email_list.empty?

        # create an instance of ActionView, so we can use the render
        # method outside of a controller
        av = ActionView::Base.new
        av.view_paths = ActionController::Base.view_paths

        # need these in case your view constructs any links or
        # references any helper methods.
        av.class_eval do
          include Rails.application.routes.url_helpers
          include ApplicationHelper
          include DashboardHelper
          include WidgetsHelper
          include ReportBlocksHelper
          include ManagersHelper
        end

        pdf_html = av.render template: 'reports/email.pdf.erb',
                             layout: nil,
                             locals: { report: report,
                                       blocks: report.blocks_data,
                                       user: report.user
                                     }

        # use wicked_pdf gem to create PDF from the doc HTML
        doc_pdf =
          WickedPdf.new.pdf_from_string(
            pdf_html, page_size: 'A4',
                      javascript_delay: 3000,
                      orientation: report.orientation_name,
                      margin: { top: 0, bottom: 0, left: 0, right: 0 } # default 10 (mm) if not specified
          )

        # save PDF to disk
        pdf_path = Rails.root.join('tmp', "#{report.name}.pdf")
        File.open(pdf_path, 'wb') do |file|
          file << doc_pdf
        end

        ReportMailer.report(report, pdf_path).deliver_now
      end
    end
  end
end
