#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
    module Jobs
      # This Jobs check alarms every 5 minutes and notify users when some of them
      # must be fired.
      #
      class UpdatePluginJob < Struct.new(:plugin_id,:user_id,:version,:enabled)
        include Redborder::Jobs::JobHelper

        def perform

          @content = ("#{I18n.t 'lib_redborder.node'} " + `hostname`.chomp! + "\n\n")
          begin
            @plugin = Plugin.where(:id => plugin_id).first
            @plugin_version = @plugin.plugin_versions.select{|p| p.version == version}.first
            if @plugin and @plugin_version
                @content += ("#{I18n.t 'lib_redborder.working_with'} #{@plugin.name}-#{version} \n\n")
                @config = YAML.load_file("#{Rails.root}/config/plugins_config.yml")
                plugins_dir = @config["plugins_dir"] || nil
                if plugins_dir
                    # Comprobar que los ficherso existan..
                    plugin_script = "#{plugins_dir}/extensions/configure-#{@plugin.name}-#{version}.sh"
                    if File.exists?(plugin_script)
                        if @plugin.installed?
                            @content += ("#{I18n.t 'lib_redborder.uninstalling'} \n\n")
                            @content += `/usr/lib/redborder/bin/rb_manager_utils.sh -e -n all -s "#{plugin_script} -r"`
                            @plugin.plugin_versions.each{ |p| p.update(installed: false) }
                        end
                        if enabled != "false"
                            @content += ("#{I18n.t 'lib_redborder.installing_plugin'} \n\n")
                            @content += `/usr/lib/redborder/bin/rb_manager_utils.sh -e -n all -s "#{plugin_script} -i"`
                            @plugin_version.update(installed: true)
                        end
                    end #File.exists..
                end #plugins_dir

            end #if @plugin
          rescue Exception => e
            logit e
            logit e.backtrace.join("\n")
            @content += "#{e.inspect}\n"
          ensure
            # Poner is_aplying a false
            @plugin.update(is_applying: false)
            @content += ("\n\n#{I18n.t 'lib_redborder.process_finish'} \n\n")
            stored_job = StoredDelayedJob.create(job: "UpdatePluginJob", user_id: user_id, content: @content)
          end
        
        end # perform

      end # class
    end #module Jobs
  end # module Redborder
