#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Jobs
    class UploadVmJob < Struct.new(:verbose, :file_name, :tempfile, :vmachine, :user_id)

      include Redborder::Jobs::JobHelper

      def perform
        content = ("Node executing this job: " + `hostname`.chomp! + "\n\n")

        s3_config = YAML.load_file("#{Rails.root}/config/aws.yml")
        s3 =
          AWS::S3.new(
            access_key_id: s3_config[Rails.env]['access_key_id'],
            secret_access_key: s3_config[Rails.env]['secret_access_key'],
            s3_endpoint: s3_config[Rails.env]['s3_host_name']
          )

        obj =
          s3.buckets[s3_config[Rails.env]['bucket']]
            .objects["isos/#{file_name}"]

        content += "Uploading file to isos/#{file_name}\n"

        obj.write(file: tempfile)

        content += "Upload finished successfully\n"

        manager_role = Chef::Role.manager

        content += "Updating chef role\n"

        unless manager_role.override_attributes['redborder']['cuckoo']
          manager_role.override_attributes['redborder']['cuckoo'] ||= {}
        end

        manager_role.override_attributes['redborder']['cuckoo']['machines'] ||= []
        vm = JSON.parse(vmachine)
        vm['s3_path'] = "isos/#{file_name}"

        replicas = vm.delete("replicas").to_i

        content += "Creating #{replicas} replica#{replicas > 1 ? "s" : ""}\n"

        if manager_role.override_attributes['redborder']['cuckoo']['machines'].present?
          previous_vm = manager_role.override_attributes['redborder']['cuckoo']['machines'].last
        else
          previous_vm = {'name' => 'cuckoo0', 'ip' => '192.168.122.100'}
        end

        ip_num = IPAddr.new(previous_vm['ip']).to_i + 1
        cuckoo_index = previous_vm['name'].scan( /\d+$/ ).first.to_i + 1

        (1..replicas).each do |i|
          machine = vm.clone
          machine['name'] = 'cuckoo' + cuckoo_index.to_s
          machine['ip'] = IPAddr.new(ip_num, Socket::AF_INET).to_s
          machine['vm_name'] = machine['name'] + ".qcow2"
          content += "Replica #{i}: #{machine}\n"

          manager_role.override_attributes['redborder']['cuckoo']['machines'] << machine

          ip_num += 1
          cuckoo_index += 1
        end

        manager_role.save

        content += "Update finished successfully\n"

      rescue StandardError => se

        content += "#{se.class}: #{se.message}\n"
        se.backtrace.each do |line|
          content += "#{line}\n"
        end
        logit "#{se.class}: #{se.message}" if verbose

      ensure
        # Delete tmp file
        FileUtils.rm tempfile

        logit 'Creating StoredDelayedJob' if verbose
        StoredDelayedJob.create(
          job: 'UploadVm',
          user_id: user_id,
          content: content
        )
      end
    end
  end
end
