#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  #
  # Class to interact with Kafka topics
  #
  class Kafka
    # Constants
    KAFKA_BROKER = ['kafka.service:9092'.freeze].freeze

    # Readers
    attr_reader :topic

    # Initialize the topic
    #
    # @param topic [String] Kafka topic to read/write
    #
    def initialize(topic)
      @topic = topic
    end

    # Send a message to the topic.
    #
    # @param messages [Array, Hash] Data to send to Kafka topic
    #
    def send(messages)
      messages = [messages] if messages.is_a?(Hash)
      messages = Array(messages).map do |el|
        ::Poseidon::MessageToSend.new(@topic, el.to_json)
      end
      # Send the data
      producer.send_messages(messages)
    end

    private

    # Initialize the producer if it isn't initialized. We use Poseidon gem
    # and the Kafka url based on hosts.
    #
    # @return [Poseidon::Producer] Poseidon Producer to send data
    #
    def producer
      return @producer if @producer
      # Create a partitioner
      partitioner = proc do |key, partition_count|
        Zlib.crc32(key) % partition_count
      end
      # Initialize producer
      @producer =
        Poseidon::Producer.new(KAFKA_BROKER, @topic, partitioner: partitioner)
    end
  end
end
