#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  class LinkRenderer < WillPaginate::ActionView::LinkRenderer

    #
    # => Change the to_html method of will_paginate to render a list
    #    to adapt the code to bootstrap
    #
    def to_html
      html = pagination.map do |item|
        item.is_a?(Fixnum) ?
          page_number(item) :
          send(item)
      end.join(@options[:link_separator])

      @options[:container] ? html_container(html) : html
    end

    def link(text, target, attributes = {})
      attributes["data-remote"] = true if @options[:remote]
      super
    end

    protected

    def html_container(html)
      tag(:nav, tag(:ul, html, container_attributes))
    end

    def page_number(page)
      unless page == current_page
        html = link(page, page, :rel => rel_value(page))
        tag(:li, html)
      else
        html = link(page, page)
        tag(:li, html, class: "active")
      end
    end

    def previous_page
      num = @collection.current_page > 1 && @collection.current_page - 1
      previous_or_next_page(num, @options[:previous_label], "previous_page")
    end

    def next_page
      num = @collection.current_page < total_pages && @collection.current_page + 1
      previous_or_next_page(num, @options[:next_label], "next_page")
    end

    def gap
      text = @template.will_paginate_translate(:page_gap) { "&hellip;" }
      html = link(text, nil, class: "page_gap")
      tag(:li, html, class: "disabled")
    end

    def previous_or_next_page(page, text, classname)
      if page
        html = link(text, page, :class => classname)
        tag(:li, html)
      else
        tag(:li, tag(:span, text), class: "#{classname} disabled")
      end
    end
  end
end
