#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  class Node

    CONTROLLERPATH = "/controller"
    ALLOWED_TOPICS = ["rb_monitor"]

    if Redborder::Module.registered_modules.keys.map(&:to_s).include? "flow"
      ALLOWED_TOPICS.push "rb_flow", "rb_flow_post"
    end

    if Redborder::Module.registered_modules.keys.map(&:to_s).include? "ips"
      ALLOWED_TOPICS.push "rb_event", "rb_event_post"
    end

    def self.get_occupation
      zk_data = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
      conn = nil

      begin
        conn = ZK.new zk_data[:zk_connect]
        masters = conn.children '/druid/discoveryPath/coordinator'
        master_data = conn.get "/druid/discoveryPath/coordinator/#{masters.delete(masters.sample)}"
      rescue Exception => e
        raise "#{Time.now} [ERROR] GetOccupation: #{I18n.t 'lib_redborder.cannot_zk'} #{e}"
      end

      parsed_data = JSON.parse master_data[0]
      server = parsed_data['address'] + ':' + parsed_data['port'].to_s

      begin
        url = URI('http://' + server + '/info/cluster')
        found = false
        until found
          host, port = url.host, url.port if url.host && url.port
          req = Net::HTTP::Get.new(url.path)
          res = Net::HTTP.start(url.host, url.port) {|http|  http.request(req) }
          res.header['location'] ? url = URI.parse(res.header['location']) : found = true
        end
      rescue Exception => e
        raise "#{Time.now} [ERROR] GetOccupation: #{I18n.t 'lib_redborder.cannot_druid'} #{e}"
      end

      ret = []
      managers = Manager.all.sort_by(&:name)
      managers.each do |host|
        begin
          begin
            if conn.exists?"/druid/segments/#{host.name}:8083"
              segments = (conn.children "/druid/segments/#{host.name}:8083").count if !conn.nil?
            else
              segments = "0"
            end
          rescue Exception => e
            raise "#{Time.now} [ERROR] GetOccupation: #{I18n.t 'lib_redborder.error_segments'} #{host.name}: #{e}"
            segments = "unknown"
          end
          begin
            node = ::Chef::Node.load host.name
          rescue Exception => e
            raise "#{Time.now} [ERROR] GetOccupation: #{I18n.t 'lib_redborder.error_chef'} #{host.name}: #{e}"
          end

          if node['redborder']['druid'].nil?
            tier = 'default'
          else
            tier = node['redborder']['druid']['historical']['tier']
          end
          ret << {
            name: host.name,
            segments: segments,
            memory: node['memory']['total'],
            tier: tier,
            pending: node['redborder']['pending_changes'],
            last_seen: node['ohai_time']
          }
        rescue Exception => e
          raise "#{Time.now} [ERROR] GetOccupation: #{I18n.t 'lib_redborder.the'} #{host.name} #{I18n.t 'lib_redborder.chef_node'}. (#{e})"
        end
      end

      total_segments = 0
      ret.each{|i| total_segments += i[:segments].to_i }
      ret.each{|i| i[:segments_percentage] = (i[:segments].to_f * 100 / total_segments).round(2)}
      ret.sort_by! { |x| x[:name] }
    ensure
      conn.close unless conn.nil?
    end

    def self.get_brokers_nodes
      zk_data = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
      brokers_names = []
      begin
        zk = ZK.new(zk_data[:zk_connect])
        brokers = zk.children("/druid/discoveryPath/broker").map{|k| k.to_s}.sort.uniq.shuffle

        brokers.each do |b|
          zktdata = zk.get("/druid/discoveryPath/broker/#{b}")[0]
          zktdata = YAML.load(zktdata)
          if zktdata["address"] and zktdata["port"]
            brokers_names << zktdata["address"]
          end
        end
      rescue Exception => e
        Rails.logger.error "ERROR: #{I18n.t 'lib_redborder.exception_on'} #{zk_host}"
        Rails.logger.error e.backtrace.join("\n")
      ensure
        if !zk.nil? and !zk.closed?
          zk.close
        end
      end
      return brokers_names
    end

    def self.get_historical_nodes
      zk_data = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
      begin
        zk = ZK.new(zk_data[:zk_connect])
        historical_nodes = []
        if zk.exists? "/druid/announcements"
          zk.children("/druid/announcements").each do |children|
            historical_nodes << children.match(/(.*):8083/)[1] if children.match(/(.*):8083/)
          end
        end
      rescue Exception => e
        Rails.logger.error "ERROR: #{I18n.t 'lib_redborder.exception_on'} #{zk_host}"
        Rails.logger.error e.backtrace.join("\n")
      ensure
        if !zk.nil? and !zk.closed?
          zk.close
        end
      end
      return historical_nodes
    end

    def self.get_kafka_info
      opt = Getopt::Std.getopts("hn")
      zk_data = YAML.load(File.read('config/rbdruid_config.yml'))[Rails.env].symbolize_keys
      zk_host = zk_data[:zk_connect]

      if zk_host.nil?
        Rails.logger.error "#{I18n.t 'lib_redborder.no_zk'}"
        return {}
      end

      begin
        zk = ZK.new(zk_host)
        broker_names = {}
        kafka_information = {}

        if zk.nil?
          Rails.logger.error "#{I18n.t 'lib_redborder.cannot_connect'} #{zk_host}"
          return {}
        else
          reasigning_partitions = zk.exists?("/admin/reassign_partitions") and zk.exists?("/admin/preferred_replica_election")
        end

        brokerids = zk.children("/brokers/ids").map(&:to_i).uniq.sort
        topics = zk.children("/brokers/topics").sort.uniq

        if brokerids.present?
          brokerids.each do |brid|
            zkdata, stat = zk.get("/brokers/ids/#{brid}")
            zkdata = YAML.load(zkdata)
            broker_names[brid] = zkdata["host"]
          end
        else
          Rails.logger.error "#{I18n.t 'lib_redborder.no_brokers'} #{zk_host}"
        end

        topics.each do |topic|
          next unless ALLOWED_TOPICS.include? topic
          kafka_information[topic.to_sym] = {}
          index = 0
          partitions = zk.children(partitions_path(topic)).map(&:to_i).sort.uniq
          zkdata, stat = zk.get(topic_path(topic))
          zkdata = YAML.load(zkdata)

          zkdata["partitions"].each do |p, replicas|
            kafka_information[topic.to_sym][p] = {}
            if zk.exists?("#{topic_path(topic)}/partitions/#{p}/state")
              # Druid realtime
              if zk.exists? "/consumers/rb-group/owners/#{topic}/#{p}"
                zkdata2,stat = zk.get("/consumers/rb-group/owners/#{topic}/#{p}")
                match_re = zkdata2.match /rb-group_([^-]*)-.*/
                if match_re.nil? or match_re.size < 2
                    druid_rt = "?"
                else
                    druid_rt = match_re[1]
                end
              else
                druid_rt = "?"
              end

              # rt = []
              # if zk.exists? "/druid/announcements"
              #   zk.children("/druid/announcements").each do |children|
              #     rt << children.match(/(.*):8082/)[1] if children.match(/(.*):8082/)
              #   end
              # end
              # druid_rt = "?" unless rt.include?druid_rt

              # Spout
              if zk.exists? "/consumers/rb-storm/owners/#{topic}/#{p}"
                zkdata2, stat = zk.get("/consumers/rb-storm/owners/#{topic}/#{p}")
                match_re = zkdata2.match /^rb-storm_([^:]*):/
                if match_re.nil? or match_re.size < 2
                    storm = "?"
                else
                    storm = match_re[1]
                end
              else
                storm = "?"
              end

              pdata, stat = zk.get("#{topic_path(topic)}/partitions/#{p}/state")
              pdata = YAML.load(pdata)
              info = if opt["n"]
                {topic: topic,
                 partition: p,
                 leader: pdata["leader"],
                 replicas: replicas.join(","),
                 isr: pdata["isr"].join(","),
                 storm: storm,
                 druid_rt: (druid_rt.nil? ? "" : druid_rt)}
              else
                {topic: topic,
                 partition: p,
                 leader: (broker_names[pdata["leader"]].nil? ? "unknown_id_#{pdata["leader"]}" : broker_names[pdata["leader"]]),
                 replicas: replicas.map{|x| (broker_names[x].nil? ? "unknown_id_#{x}" : broker_names[x])}.join(","),
                 isr: pdata["isr"].map{|x| (broker_names[x].nil? ? "unknown_id_#{x}" : broker_names[x])}.join(","),
                 storm: storm,
                 druid_rt: (druid_rt.nil? ? "" : druid_rt)}
              end
            else
              info = {topic: topic, partition: p}
            end
            kafka_information[topic.to_sym][p] = info
            index = index + 1
          end
        end
      rescue Exception => e
        Rails.logger.error "ERROR: Exception on #{zk_host}"
        Rails.logger.error e.backtrace.join("\n")
      ensure
        if !zk.nil? and !zk.closed?
          zk.close
        end
      end

      return kafka_information, reasigning_partitions
    end

    def self.get_postgresql_status
      status = `sudo /bin/env BOOTUP=none /opt/rb/bin/rb_pcp.sh status`
      if status.nil? or status.empty?
        Rails.logger.error "ERROR: #{I18n.t 'lib_redborder.error_executing'}"
        return
      end

      nodes = []
      status.lines.each_with_index do |line, index|
        nodes << line.split(/\s+/) if index > 2
      end

      return nodes
    end

    def self.get_zookeeper_status
      status = `sudo /bin/env BOOTUP=none /usr/lib/redborder/bin/rb_get_zkinfo.sh`
      if status.nil? or status.empty?
        Rails.logger.error "ERROR: Error executing rb_get_zkinfo.sh file"
        return
      end

      if status.lines.count < 18
        Rails.logger.error "ERROR: #{I18n.t 'lib_redborder.the_file'}"
        return
      end

      nodes = []
      data = []
      status.lines.each_with_index do |line, index|
        nodes = line.split(/\s+/) if index == 1
        data << line.split(/\s+/) if index > 2
      end

      return {
        nodes: nodes,
        data: data
      }
    end

    def self.topic_path(topic)
      return "/brokers/topics/#{topic}"
    end

    def self.partitions_path(topic)
      return "#{topic_path(topic)}/partitions"
    end

  end
end
