#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################
module Redborder
  class MacVendor
    attr_accessor :prefix_cache

    def initialize(opts={})
      @prefix_cache = {}
      @preloaded = false
      preload_cache_via_local_data
    end

    def lookup(mac)
      prefix = self.normalize_prefix mac
      @prefix_cache[prefix]
    end

    # Attempts to turn anything MAC-like into the first six digits, e.g.: AABBCC
    def normalize_prefix(mac)
      mac.strip.upcase.gsub(/^0[xX]/, '').gsub(/[^0-9A-F]/, '')[0..5]
    end

    def preload_cache_via_string(oui_txt)
      # First entry is a header
      entries = oui_txt.split("\n")

      entries.each do |entry|
        begin
          entry_match = /^(?<prefix>\w{6})\|(?<company_name>.+)$/.match(entry)
          mac_prefix = entry_match[:prefix]
          company_name = entry_match[:company_name]

          # This actually happens three times in the current dataset!
          unless @prefix_cache[mac_prefix].nil?
            #puts "MAC PREFIX COLLISION: #{mac_prefix}"
            #puts "CURRENT = #{@prefix_cache[mac_prefix].inspect}"
            #puts "NEW = #{{:name => company_name, :address => company_address}.inspect}"
            #raise "MAC prefix key collision: #{mac_prefix}"
            next
          end
        
          @prefix_cache[mac_prefix] = company_name
        rescue StandardError => e
          Rails.logger.error e
          Rails.logger.error e.backtrace.join("\n")
        end
      end
      @preloaded = true
    end

    def preload_cache_via_local_data
      local_path = '/etc/objects/mac_vendors'
      preload_cache_via_string File.open(local_path).read if File.exists? local_path
    end
  end

  class RbMacVendor
    @vendors = Redborder::MacVendor.new

    def self.lookup(mac=nil)
      (mac and @vendors.lookup(mac)) || 'Unknown'
    end
  end
end
