#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  module Rule

		private

    def set_query

      return if request.format.text?

      if @policy
        @is_query = {rules: {default_enabled: @policy.default_enabled_rules? ? true : [true, false], rule_db_version_id: @policy.rule_db_versions.map(&:id)}}
      else
        @is_query = {rules: {rule_db_version_id: @rule_db_version.id}}
      end  

      @not_query = []
      @query_tables = {}

      session[:rule][:filter].each do |k, v|

        v.each do |key, value|

          if k.to_sym == :factory_policy_id
            
            (@query_tables[:rules] ||= []) << :factory_policy_rules

            if value[:operator] == "is"
              @is_query[:factory_policy_rules] ||= {}
              @is_query[:factory_policy_rules].has_key?(:factory_policy_id) ? @is_query[:factory_policy_rules][:factory_policy_id] << key : @is_query[:factory_policy_rules][:factory_policy_id] = [key]
            else
              @not_query << {factory_policy_rules: {factory_policy_id: key}} 
            end

          elsif [:flowbit_id, :flowbit_state_id].include?(k.to_sym)

            (@query_tables[:rules] ||= []) << :flowbit_rules

            if value[:operator] == "is"
              @is_query[:flowbit_rules] ||= {}
              @is_query[:flowbit_rules].has_key?(k.to_sym) ? @is_query[:flowbit_rules][k.to_sym] << key : @is_query[:flowbit_rules][k.to_sym] = [key]
            else
              @not_query << {flowbit_rules: {k.to_sym => key}}
            end

          elsif [:category_id, :group_id, :family_id, :protocol, :rule_sid, :rule_gid].include?(k.to_sym)

            if value[:operator] == "is"
              @is_query[:rules] ||= {}
              @is_query[:rules].has_key?(k.to_sym) ? @is_query[:rules][k.to_sym] << key : @is_query[:rules][k.to_sym] = [key]
            else
              @not_query << {rules: {k.to_sym => key}}
            end

          elsif k.to_sym == :class_type_name

            (@query_tables[:rules] ||= []) << :class_type

            if value[:operator] == "is"
              @is_query[:class_types] ||= {}
              @is_query[:class_types].has_key?(:name) ? @is_query[:class_types][:name] << key : @is_query[:class_types][:name] = [key]
            else
              @not_query << {class_types: {name: key}}
            end            

          elsif k.to_sym == :severity

            (@query_tables[:rules] ||= []) << {class_type: [:severity]}

            if value[:operator] == "is"
              @is_query[:severities] ||= {}
              @is_query[:severities].has_key?(:id) ? @is_query[:severities][:id] << key : @is_query[:severities][:id] = [key]
            else
              @not_query << {severities: {id: key}}
            end

          elsif k.to_sym == :rule_source_id

            (@query_tables[:rules] ||= []) << :rule_db_version

            if value[:operator] == "is"
              @is_query[:rule_db_versions] ||= {}
              @is_query[:rule_db_versions].has_key?(:rule_source_id) ? @is_query[:rule_db_versions][:rule_source_id] << key : @is_query[:rule_db_versions][:rule_source_id] = [key]
            else
              @not_query << {rule_db_versions: {rule_source_id: key}}
            end

          elsif k.to_sym == :action_id

            (@query_tables[:rules] ||= []) << :policy_rules

            if value[:operator] == "is"
              @is_query[:policy_rules] ||= {}
              @is_query[:policy_rules].has_key?(k.to_sym) ? @is_query[:policy_rules][k.to_sym] << key : @is_query[:policy_rules][k.to_sym] = [key]
            else
               @not_query << {policy_rules: {k.to_sym => key}}
            end

          elsif k.to_sym == :reference
            
            search_text = case value[:operator].to_sym
            when :contains, :contains_not
              "%#{key.downcase.split.join('%')}%"
            when :starts_with
              "#{key.downcase.split.join('%')}%"
            when :ends_with
              "%#{key.downcase.split.join('%')}"
            end

            rb_reference_rule_ids = RbReferenceRule.where("LOWER(url) LIKE ?", search_text).map(&:id)

            (@query_tables[:rules] ||= []) << :rb_reference_rules

            if value[:operator] == "contains_not"
              @not_query << {rb_reference_rules: {id: rb_reference_rule_ids}}
            else
              @is_query[:rb_reference_rules] ||= {}
              @is_query[:rb_reference_rules].has_key?(:id) ? @is_query[:rb_reference_rules][:id] += rb_reference_rule_ids : @is_query[:rb_reference_rules][:id] = rb_reference_rule_ids
            end

          elsif k.to_sym == :msg

            search_text = case value[:operator].to_sym
            when :contains, :contains_not
              "%#{key.downcase.split.join('%')}%"
            when :starts_with
              "#{key.downcase.split.join('%')}%"
            when :ends_with
              "%#{key.downcase.split.join('%')}"
            end

            rule_ids = Object::Rule.where("LOWER(msg) LIKE ?", search_text).map(&:id)

            if value[:operator] == "contains_not"
              @not_query << {rules: {id: rule_ids}}
            else
              @is_query[:rules] ||= {}
              @is_query[:rules].has_key?(:id) ? @is_query[:rules][:id] += rule_ids : @is_query[:rules][:id] = rule_ids
            end

          elsif [:is_new, :is_modified, :is_deprecated, :default_enabled].include?(k.to_sym)

            key = key == "0" ? false : true

            if value[:operator] == "is"
              @is_query[:rules] ||= {}
              if k.to_sym == :default_enabled
                @is_query[:rules][k.to_sym] = [key]
              else
                @is_query[:rules].has_key?(k.to_sym) ? @is_query[:rules][k.to_sym] << key : @is_query[:rules][k.to_sym] = [key]
              end
            else
              @not_query << {rules: {k.to_sym => key}}
            end

          end

        end

      end

      @query_tables = :rules if @query_tables.blank?
      @msg = params[:msg]

    end  
  end
end
