#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder
  #
  # Class to interact with S3
  #
  class S3
    # S3 instance
    attr_reader :s3

    AWS_FILE_PATH = "#{Rails.root}/config/aws.yml"
    LOCAL_STORAGE = "#{Rails.root}/data"

    #
    # Credentials
    #
    def self.credentials
      return @credentials if @credentials
      @s3_credentials =
        YAML.load(File.read(AWS_FILE_PATH))[Rails.env].symbolize_keys!
    end

    def credentials
      self.class.credentials
    end

    #
    # Disable initializer
    #
    def initialize
      # Get s3
      @s3 = AWS::S3.new(credentials)
    end

    # Methods
    # ------------------------------------------
    
    #
    # Check if the URLs are public (Amazon)
    #
    def public?
      credentials[:generate_s3_url] || false
    end

    #
    # Return the URL for an object
    #
    # == Parameters:
    # action::
    #   Symbol with the action of generated URL: get, read...
    # path::
    #   String with the path to the object (Without bucket)
    # expires:
    #   (Optional) Integer with the number of minutes this link will be
    #   available
    #
    # == Returns:
    # String with the URL
    #
    def object_url(action, path, expires = 10)
      s3_object =
        AWS::S3::S3Object.new(AWS::S3.new.buckets[credentials[:bucket]], path)
      s3_object.url_for(
        action,
        expires: expires, secure: true,
        endpoint: credentials[:s3_host_name]).to_s
    end
  end
end
