#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

# rake db:migrate => [rake db:migrate, rake db:migrate:modules]
#Rake::Task['db:migrate'].enhance do
#  Rake::Task['db:migrate:modules'].invoke
#end 

# rake db:seed => [rake db:seed, rake db:seed:modules]
#Rake::Task['db:seed'].enhance do
#  Rake::Task['db:seed:modules'].invoke
#end

namespace :db do
  namespace :migrate do

    # rake db:migrate:custom_database env DB_PATH=config/custom_database.yml
    desc "Run migrations from application and modules with a custon database path"
    task :custom_database => :environment do
      if ENV["DB_PATH"]
        Rails.application.config.paths['config/database'] = [ENV["DB_PATH"]]
      end
      ActiveRecord::Migration.verbose = ENV["VERBOSE"] ? ENV["VERBOSE"] == "true" : true
      ActiveRecord::Migrator.migrate("db/migrate/", ENV["VERSION"] ? ENV["VERSION"].to_i : nil)
      Rake::Task["db:schema:dump"].invoke if ActiveRecord::Base.schema_format == :ruby

      Dir.entries("#{Rails.root}/lib/modules").reject{|entry| entry == "." || entry == ".."}.each do |dir|
        module_migrations_path = "lib/modules/#{dir}/db/migrate"
        ActiveRecord::Migration.verbose = ENV["VERBOSE"] ? ENV["VERBOSE"] == "true" : true
        ActiveRecord::Migrator.migrate(module_migrations_path, ENV["VERSION"] ? ENV["VERSION"].to_i : nil)
        Rake::Task["db:schema:dump"].invoke if ActiveRecord::Base.schema_format == :ruby
      end  
    end

    # rake db:migrate:application
    desc "Run migrations from application"
    task :application => :environment do
      ActiveRecord::Migration.verbose = ENV["VERBOSE"] ? ENV["VERBOSE"] == "true" : true
      ActiveRecord::Migrator.migrate("db/migrate/", ENV["VERSION"] ? ENV["VERSION"].to_i : nil)
      Rake::Task["db:schema:dump"].invoke if ActiveRecord::Base.schema_format == :ruby
    end
        
    # rake db:migrate:modules
    # rake db:migrate:modules MODULE=monitor
    desc "Run migrations from modules"
    task :modules => :environment do
      if ENV["MODULE"]
        module_migrations_path = "lib/modules/#{ENV['MODULE']}/db/migrate"
        ActiveRecord::Migration.verbose = ENV["VERBOSE"] ? ENV["VERBOSE"] == "true" : true
        ActiveRecord::Migrator.migrate(module_migrations_path, ENV["VERSION"] ? ENV["VERSION"].to_i : nil)
        Rake::Task["db:schema:dump"].invoke if ActiveRecord::Base.schema_format == :ruby
      else
        Dir.entries("#{Rails.root}/lib/modules").reject{|entry| entry == "." || entry == ".."}.each do |dir|
          module_migrations_path = "lib/modules/#{dir}/db/migrate"
          ActiveRecord::Migration.verbose = ENV["VERBOSE"] ? ENV["VERBOSE"] == "true" : true
          ActiveRecord::Migrator.migrate(module_migrations_path, ENV["VERSION"] ? ENV["VERSION"].to_i : nil)
          Rake::Task["db:schema:dump"].invoke if ActiveRecord::Base.schema_format == :ruby
        end  
      end
    end
  end

  namespace :seed do

    # rake db:seed:application
    desc "Run seeds from application"
    task :application => :environment do
      puts "Loading default seeds.rb"
      ActiveRecord::Tasks::DatabaseTasks.load_seed
    end
        
    # rake db:seed:modules
    # rake db:seed:modules MODULE=monitor
    desc "Run seeds from modules"
    task :modules => :environment do   
      if ENV["MODULE"]
        seed_file = File.join(Rails.root, 'lib', 'modules', "#{ENV['MODULE']}", 'db', 'seeds.rb')
        if File.exist?(seed_file)
          puts "Loading #{seed_file}" 
          load(seed_file)
        end
      else   
        Dir.entries("#{Rails.root}/lib/modules").reject{|entry| entry == "." || entry == ".."}.each do |dir|
          seed_file = File.join(Rails.root, 'lib', 'modules', "#{dir}", 'db', 'seeds.rb')
          if File.exist?(seed_file)
            puts "Loading #{seed_file}" 
            load(seed_file)
          end
        end  
      end    
    end
  end
end