require 'yaml'
require 'rake/testtask'

#
# Load rake test namespace
#
namespace :test do
  #
  # Load fixtures from modules
  #
  desc 'Load fixtures from modules'
  task :load_fixture_modules do
    dst = 'test/fixtures'
    FileUtils.cp(Dir.glob('lib/modules/**/test/fixtures/*.yml'), dst)
  end

  #
  # Delete fixtures from modules
  #
  desc 'Delete fixtures from modules'
  task :delete_fixture_modules do
    # Iterate over modules fixtures to get filename
    Dir.glob('lib/modules/**/test/fixtures/*.yml').each do |fix|
      name = fix.split('/').last
      FileUtils.rm("test/fixtures/#{name}")
    end
  end

  #
  # Run tests from test/api modules directory
  #
  desc 'Test API modules source'
  Rake::TestTask.new('modules:api') do |t|
    t.libs << 'test'
    t.pattern = 'lib/modules/**/test/api/**/*_test.rb'
    t.verbose = false
  end

  #
  # Run tests from test/modules directory
  #
  desc 'Test modules integration source'
  Rake::TestTask.new('modules:only-integration') do |t|
    t.libs << 'test'
    t.pattern = 'lib/modules/**/test/integration/*_test.rb'
    t.verbose = false
  end

  #
  # Run tests from test/modules directory
  #
  desc 'Test modules source'
  Rake::TestTask.new('modules:non-integration') do |t|
    t.libs << 'test'
    t.pattern = 'lib/modules/**/test/{models,controllers}/*_test.rb'
    t.verbose = false
  end

  # Full API tests
  #
  desc 'Test api source'
  Rake::TestTask.new('api') do |t|
    t.libs << 'test'
    t.pattern = 'test/api/**/*_test.rb'
    t.test_files = FileList['test/api/**/*_test.rb',
                            'lib/modules/**/test/api/**/*_test.rb']
    t.verbose = false
  end

  # Full Integration tests
  #
  desc 'Test integration elements'
  Rake::TestTask.new('only-integration') do |t|
    t.libs << 'test'
    t.test_files = FileList['test/integration/*_test.rb',
                            'lib/modules/**/test/integration/*_test.rb']
    t.verbose = false
  end

  # Full non-integration tests
  #
  desc 'Test non integration elements'
  Rake::TestTask.new('non-integration') do |t|
    t.libs << 'test'
    t.test_files = FileList['test/{models,controllers,lib}/*_test.rb',
                            'lib/modules/**/test/{models,controllers,lib}/*_test.rb']
    t.verbose = false
  end

  # Create private and public keys for license's signature
  #
  desc 'Create private and public keys'
  task :license_keys do
    # Remove existing files
    File.delete("#{Rails.root}/config/license.key") if File.exist?("#{Rails.root}/config/license.key")
    File.delete("#{Rails.root}/config/license.key.pub") if File.exist?("#{Rails.root}/config/license.key.pub")
    pkey = OpenSSL::PKey::RSA.new 4096

    File.open "#{Rails.root}/config/license.key", 'w' do |io|
      io.write pkey.to_pem
    end

    File.open "#{Rails.root}/config/license.key.pub", 'w' do |io|
      io.write pkey.public_key.to_pem
    end
  end

  # Restore the modules.yml
  at_exit do
    bak = "#{Rails.root}/config/modules.bak"
    if File.exist?(bak)
      curr = "#{Rails.root}/config/modules.yml"
      FileUtils.rm curr
      FileUtils.mv bak, curr
    end
  end
end
