require 'test_helper'
require 'json'

class Api::V1::LicensesControllerTest < ActionDispatch::IntegrationTest
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'an admin user' do
    setup do
      @user = users(:admin)
      @auth_token = users(:admin).authentication_token
      # Base api
      @api = 'api/v1'
      # Organization license
      @license = licenses(:four)

      RedBorder.send(:remove_const, 'LICENSING_MODE')
      RedBorder::LICENSING_MODE = 'global'
    end

    #
    # Tested method: licenses
    #
    should 'render all licenses' do
      get "/#{@api}/licenses?auth_token=#{@auth_token}"
      # Check status
      assert(response.response_code == 200,
             "Response code should be 200 : got #{response.response_code}")
      # Check response and number of items
      json = JSON.parse(response.body)
      assert_equal(json.count, 4, 'Wrong number of licenses')
      assert_equal(json.first['cluster_uuid'], @license.cluster_uuid, 'Wrong cluster_uuid license')
    end

    #
    # Tested method: licenses (available by type)
    #
    should 'render licenses by type' do
      get "/#{@api}/licenses?auth_token=#{@auth_token}&available_by_type=flow"
      # Check status
      assert(response.response_code == 200,
             "Response code should be 200 : got #{response.response_code}")
      # Check response and number of items
      json = JSON.parse(response.body)
      assert_equal(json.count, 1, 'Wrong number of licenses')
      assert_equal(json.first['cluster_uuid'], @license.cluster_uuid, 'Wrong cluster_uuid license')
      assert(json.first['sensors']['flow'] > 0, 'There is no available license for this type of sensor')
    end

    #
    # Tested method: show
    #
    should 'render a license' do
      get "/#{@api}/licenses/#{@license.id}?auth_token=#{@auth_token}"
      # Check status
      assert(response.response_code == 200,
             "Response code should be 200 : got #{response.response_code}")
      # Check response and number of items
      json = JSON.parse(response.body)
      assert_equal(json['id'], @license.id, 'Wrong id license')
      assert_equal(json['uuid'], @license.uuid, 'Wrong uuid license')
      assert_equal(json['cluster_uuid'], @license.cluster_uuid, 'Wrong cluster_uuid license')
    end
  end
end
