require 'test_helper'
require 'json'

class Api::V1::OrganizationControllerTest < ActionDispatch::IntegrationTest
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'n2kafka' do
    setup do
      # Base api
      @api = 'api/v1'
      @organization = sensors(:organization)
    end

    should 'notify reach limit (never notified)' do
      put "/#{@api}/organizations/#{@organization.uuid}/reach_bytes_limit"
      json = JSON.parse(response.body)
      assert(json['msg'], 'email sent')
    end

    should 'notify reach limit' do
      # Initialize last time the notification was sent
      @organization.update(last_organization_notification: (Time.now.to_i - 25.hours))
      put "/#{@api}/organizations/#{@organization.uuid}/reach_bytes_limit"
      json = JSON.parse(response.body)
      assert(json['msg'], 'email sent')
    end

    should 'notify reach limit that is already been notified' do
      # Initialize last time the notification was sent
      @organization.update(last_organization_notification: Time.now.utc.beginning_of_day.to_i)
      put "/#{@api}/organizations/#{@organization.uuid}/reach_bytes_limit"
      json = JSON.parse(response.body)
      assert(json['msg'], 'email already sent today')
    end

    should 'not notify reach limit if is a public IP' do
      ActionDispatch::Request.any_instance.stubs(:remote_ip).returns('8.8.8.8')
      put "/#{@api}/organizations/#{@organization.uuid}/reach_bytes_limit"
      json = JSON.parse(response.body)
      assert(json['error'], 'Failed to authenticate')
    end
  end
end
