require 'test_helper'
require 'json'

class Api::V1::RedborderObjectsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'an user requesting objects' do
    setup do
      @user = users(:admin)
      @net_object = @user.net_objects.first
      @net_object_another = users(:admin_domain).net_objects.first
      @token = @user.authentication_token
    end

    should 'can find his own private objects' do
      get :index, value: @net_object.name[0..4], type: 'NetObject',
                  auth_token: @token
      assert_response :success
      # Get body
      json = JSON.parse(response.body)
      assert(json['query'], 'Query must be true')
      assert_equal(1, json['totalResultCount'], 'Return elements must be 1')
      assert_equal(json['objects'].first['name'], @net_object.name,
                   'Name must coincide')
      assert_equal(json['objects'].first['value'], @net_object.value,
                   'Value must coincide')
    end

    should 'can not read another user objects' do
      get :index, value: @net_object_another.name[0..4], type: 'NetObject',
                  auth_token: @token
      assert_response :success
      # Get body
      json = JSON.parse(response.body)
      assert(json['query'], 'Query must be true')
      assert_equal(0, json['totalResultCount'], 'Return elements must be 0')
      assert_equal(0, json['objects'].size, 'Size of elements is empty')
    end
  end
end
