require 'test_helper'

#
# Test alerts controller
#
class AlertsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'non admin user' do
    setup do
      @user = users(:user_domain)
      sign_in @user
      @alert = Alert.first
    end

    %i(edit destroy update).each do |method|
      should "be redirected when trying to #{method} an alert" do
        get method, id: @alert.id
        assert_redirected_to root_path
      end
    end

    should 'be redirected when trying to create an alert' do
      get :new
      assert_redirected_to root_path
    end
  end

  context 'admin user' do
    setup do
      @user = users(:admin)
      sign_in @user
      @alert = Alert.first
    end

    should 'see alert list' do
      get :index
      assert_response :success
    end

    should 'create an alert' do
      get :new
      assert_response :success

      assert_difference('Alert.count') do
        post :create,
             alert: { message: 'New Alert', enabled: true, level: 'info' }
      end
      assert_response :found
      assert flash[:notice].include? "#{I18n.t 'alert_success'}"
    end

    should 'not create an alert without message' do
      assert_no_difference('Alert.count') do
        post :create,
             alert: { enabled: true, level: 'info' }
      end
      assert_response :found
      assert_redirected_to alerts_url
    end

    should 'update an alert with a valid value' do
      get :edit, id: @alert.id
      assert_response :success

      patch :update, id: @alert.id,
            alert: { level: 'warning' }
      assert_equal 'warning', @alert.reload.level
      assert_redirected_to alerts_url
    end

    should 'not update an alert with a wrong value' do
      patch :update, id: @alert.id,
            alert: { message: '' }
      assert_not_equal '', @alert.reload.message
      assert_redirected_to alerts_url
    end

    should 'destroy an alert' do
      assert_difference('Alert.count', -1) do
        delete :destroy, id: @alert.id
      end
      assert_redirected_to alerts_url
    end

    should 'dismiss an alert' do
      xhr :get, :dismiss, id: @alert.id, format: :js
      assert_equal @alert.id.to_s, session[:dismissed_alerts].last
    end

    should 'disable an alert' do
      xhr :get, :disable, id: @alert.id, format: :js
      assert_equal false, @alert.reload.enabled
    end

    should 'enable an alert' do
      xhr :get, :enable, id: @alert.id, format: :js
      assert_equal true, @alert.reload.enabled
    end
  end
end
