require 'test_helper'

#
# Test application_objects controller
#
class ApplicationObjectsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'admin user' do
    setup do
      @user = users(:admin)
      sign_in @user
      @app = application_objects(:one)

      # Mock
      chef = ChefStub.new
      Chef::DataBagItem.stubs(:load).returns(chef)
      Chef::DataBagItem.stubs(:new).returns(chef)
    end

    should 'get new view' do
      get :new
      assert_response :success
    end

    should 'create a new application object' do
      assert_difference('ApplicationObject.count') do
        xhr :post, :create, format: :js, application_object: { name: 'new-app', value: '9:999' }
      end
      assert_response :success
    end

    should 'not create an application object with an existing value' do
      assert_no_difference('ApplicationObject.count') do
        xhr :post, :create, format: :js, application_object: { name: 'new-app', value: @app.value }
      end
    end

    should 'not create an application object with a wrong name' do
      assert_no_difference('ApplicationObject.count') do
        xhr :post, :create, format: :js, application_object: { name: 'new-&;app', value: '7:777' }
      end
    end

    should 'not create an application object with a wrong value' do
      assert_no_difference('ApplicationObject.count') do
        xhr :post, :create, format: :js, application_object: { name: 'new-app', value: 'a:bb' }
      end
    end

    should 'get edit view' do
      get :edit, id: @app
      assert_response :success
    end

    should 'update an application object' do
      xhr :patch, :update, format: :js, id: @app, application_object: { value: '5:555' }
      assert_response :success
    end

    should 'destroy an application object' do
      assert_difference('ApplicationObject.count', -1) do
        xhr :delete, :destroy, format: :js, id: @app
      end
      assert_response :success
    end

    should 'not destroy an application object if it does not exist' do
      assert_no_difference('ApplicationObject.count') do
        xhr :delete, :destroy, format: :js, id: 5000
      end
      assert_response :success
    end

    should 'search an application object' do
      xhr :get, :search, format: :js, search: 'localhost'
      assert_response :success
    end

    should 'export the list of application objects to csv' do
      get :export, tab: 'app'
      assert_response :success
    end

    should 'import a list of application objects from csv' do
      xhr :get, :import, format: :js, tab: 'app'
      assert_response :success

      post :import_file, tab: 'app',
           application_object: { csv_file: fixture_file_upload('files/20161202162739-Applications.csv') }
      assert_redirected_to objects_path

      post :import_file, tab: 'app',
           add_vars: 'true',
           application_object: { csv_file: fixture_file_upload('files/20161202162739-Applications.csv') }
      assert_redirected_to objects_path
    end
  end
end
