require 'test_helper'

#
# Test redborder objects controller
#
class RedborderObjectsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'admin user' do
    setup do
      @user = users(:admin)
      sign_in @user
      @redborder_object = RedborderObject.where(user: @user).first
    end

    should 'see redborder objects list' do
      get :index
      assert_response :success
    end

    # should 'create a new redborder object' do
    #   get :new, class: 'NetObject'
    #   assert_response :success

    #   assert_difference('RedborderObject.count') do
    #     xhr :post, :create, format: :js, object: { name: 'Name',
    #                                                value: '1.2.3.4',
    #                                                type: 'NetObject' }
    #   end
    #   assert_response :success
    # end

    should 'edit a redborder object' do
      get :edit, id: @redborder_object.id, object: { type: @redborder_object.type }
      assert_response :success
    end

    should 'not destroy a redborder object if it dos not exist' do
      assert_no_difference('RedborderObject.count') do
        xhr :delete, :destroy, format: :js, id: 5000, object: { type: 'NetObject' }
      end
      assert_response :success
    end

    # should 'destroy a redborder object' do
    #   assert_difference('RedborderObject.count', -1) do
    #     xhr :delete, :destroy, format: :js, id: @redborder_object.id,
    #       object: { type: @redborder_object.type }
    #   end
    #   assert_response :success
    # end

    should 'change the session tab' do
      xhr :patch, :change_session_tab, format: :js, id: :mac
      assert_equal :mac, session[:object]
      assert_response :success
    end

    # !Deprecated!
    # This test fail because we add filtering by ip_addresses 
    # MacObject html table is different than NetObject
    # should 'search an object by tab' do
    #   xhr :get, :search, format: :js, tab: 'NetObject', search: 'localhost'
    #   assert_response :success
    # end

    should 'fail if search without tab parameter' do
      xhr :get, :search, format: :js, search: 'localhost'
      assert_response :bad_request
    end

    should 'fail if search by wrong tab' do
      xhr :get, :search, format: :js, tab: 'sensors', search: 'localhost'
      assert_response :bad_request
    end

    should 'export the redborder objects from a selected tab' do
      get :export, tab: 'net'
      assert_response :success
    end

    # should 'import objects from a csv file' do
    #   xhr :get, :import, format: :js, tab: 'NetObject'
    #   assert_response :success

    #   post :import_file, tab: 'net',
    #     redborder_object: { csv_file: fixture_file_upload('files/20160617122625-Net-Objects.csv') }
    #   assert_redirected_to objects_path

    #   post :import_file, tab: 'net', add_vars: 'true',
    #     redborder_object: { csv_file: fixture_file_upload('files/20160617122625-Net-Objects.csv') }
    #   assert_redirected_to objects_path
    # end

  end
end
