require 'test_helper'

class ReportsControllerTest < ActionController::TestCase
  context 'A report' do
    setup do
      @report = reports(:report_1)
      @user = users(:admin)
      sign_in @user

      @namespaces = @user.data_sources_ids
      data_sources = @namespaces.map { |x| "rb_flow_#{x}" }
      Redborder::DataSource.stubs(:data_sources).returns(data_sources)
    end

    # should 'request all namespaces even if the user doesnt select them all' do
    #   data_sources = Redborder::DataSource.data_sources(@namespaces, :reports, @user)
    #   @user.active_data_sources = [@user.data_sources_ids.first]

    #   Redborder::Rbdruid.expects(:query).with do |*args|
    #     args[0][:data_sources] == data_sources && args[0][:include_all_namespaces] == true
    #   end.returns([]).at_least_once

    #   get :show, id: @report.id, format: :pdf
    #   assert_response :ok
    # end

    should 'be created with landscape orientation' do
      assert_difference('Report.count', 1) do
        post :create, report: { name: 'Report name', orientation: Report::ORIENTATION_LANDSCAPE }
      end
      assert_equal Report::ORIENTATION_LANDSCAPE, assigns(:report).orientation
    end

    should 'not update the orientation' do
      post :update, id: @report.id, report: { name: 'Name updated', orientation: Report::ORIENTATION_LANDSCAPE }
      assert_equal Report::ORIENTATION_PORTRAIT, assigns(:report).orientation
      assert_equal 'Name updated', assigns(:report).name
    end
  end

  context 'An organization user' do
    setup do
      @org_admin = users(:admin_domain)
      @report = reports(:organization_report)
      sign_in @org_admin
    end

    should 'share its report with a visible user' do
      org_user = users(:user_domain)

      assert_difference '@report.users.count' do
        post :update, id: @report.id, report: { collaborators: [{ 'user_id' => org_user.id, 'can_edit' => 'on' }] }
      end
    end

    should 'not share its report with user from root' do
      super_admin = users(:admin)

      assert_no_difference '@report.users.count' do
        post :update, id: @report.id, report: { collaborators: [{ 'user_id' => super_admin.id, 'can_edit' => 'on' }] }
      end
    end

    should 'share its report with market domain' do
      market = sensors(:market)

      assert_difference '@report.sensors.count' do
        post :update, id: @report.id, report: { sensors_collaborators: {'0' => { 'sensor_id' => market.id, 'can_edit' => 'none' } } }
      end
    end

    should 'not share its report with root domain' do
      root = sensors(:root)

      assert_no_difference '@report.sensors.count' do
        post :update, id: @report.id, report: { sensors_collaborators: {'0' => { 'sensor_id' => root.id, 'can_edit' => 'none' } } }
      end
    end
  end

  # test 'should get index' do
  #   get :index
  #   assert_response :success
  #   assert_not_nil assigns(:reports)
  # end

  # test 'should get new' do
  #   get :new
  #   assert_response :success
  # end

  # test 'should create report' do
  #   assert_difference('Report.count') do
  #     post :create, report: {}
  #   end

  #   assert_redirected_to report_path(assigns(:report))
  # end

  # test 'should show report' do
  #   get :show, id: @report
  #   assert_response :success
  # end

  # test 'should get edit' do
  #   get :edit, id: @report
  #   assert_response :success
  # end

  # test 'should update report' do
  #   patch :update, id: @report, report: {}
  #   assert_redirected_to report_path(assigns(:report))
  # end

  # test 'should destroy report' do
  #   assert_difference('Report.count', -1) do
  #     delete :destroy, id: @report
  #   end

  #   assert_redirected_to reports_path
  # end
end
