require 'test_helper'

#
# Test sensors controller
#
class SensorsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'super admin user' do
    setup do
      @user = users(:admin)
      sign_in @user
      @root = sensors(:root)
      @service_provider = sensors(:service_provider)
      @namespace = sensors(:namespace)
      @organization = sensors(:organization)
    end

    should 'be able to update active modules in an organization' do
      patch 'update', id: @organization.id,
            sensor: { active_modules: 'gg,dd,vv,ee', type: Sensor::ORGANIZATION }
      assert_equal 'gg,dd,vv,ee', @organization.reload.active_modules
      assert_response :found
    end

    should 'be able to create a SP under the root sensor' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 6, parent_id: @root.id, name: 'SP test' }
      end
    end

    should 'be able to create a Namespace under a SP' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 8, parent_id: @service_provider.id, name: 'NS test' }
      end
    end

    should 'be able to create an Organization under a Namespace' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: 1, domain_type: 2, parent_id: @namespace.id, name: 'ORG test' }
      end
    end

    should 'be able to create a Market under an Organization' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 3, parent_id: @organization.id, name: 'MKT test' }
      end
    end

    should 'be able to create a Building under an Organization' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 5, parent_id: @organization.id, name: 'BD test' }
      end
    end

    should 'be able to create a Deployment under an Organization' do
      assert_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 7, parent_id: @organization.id, name: 'DP test' }
      end
    end

    should 'not be able to create a Deployment under a SP' do
      assert_no_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 7, parent_id: @service_provider.id, name: 'DP test wrong' }
      end
    end

    should 'not be able to create an Organization under a SP' do
      assert_no_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: 1, domain_type: 2, parent_id: @service_provider.id, name: 'ORG test' }
      end
    end

    should 'not be able to create an Organization under the root sensor' do
      assert_no_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: 1, domain_type: 2, parent_id: @root.id, name: 'ORG test' }
      end
    end

    should 'not be able to create a Market under a Namespace' do
      assert_no_difference('Sensor.count') do
        xhr :post, :create, format: :js, sensor: { type: '1', domain_type: 3, parent_id: @namespace.id, name: 'MKT test' }
      end
    end
  end

  context 'non super admin user' do
    setup do
      @user = users(:admin_domain)
      @root = sensors(:root)
      sign_in @user
    end

    should 'not be able to update active modules in an organization' do
      @organization = sensors(:organization)
      patch 'update', id: @organization.id,
            sensor: { active_modules: 'gg,dd,vv,ee', type: Sensor::ORGANIZATION }
      assert_not_equal 'gg,dd,vv,ee', @organization.reload.active_modules
      assert_response :found
    end

    should 'not export tree from root' do
      get :export_tree, id: @root.id
      assert_response :not_found
    end

    should 'not get root\'s children' do
      get :children, id: @root.id
      assert_response :not_found
    end
  end
end
