require 'test_helper'

#
# Test widgets controller
#
class WidgetsControllerTest < ActionController::TestCase
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'An image widget' do
    setup do
      @user = users(:admin)
      sign_in @user

      @dashboard = dashboards(:admin_default)
      @widget = {
        "graph_type"=>"image",
        "widget_type"=>"shape",
        "product_type"=>"none",
        "page"=>"1",
        "name"=>"",
        "transparent"=>"1",
        "parent_id"=> @dashboard.id.to_s,
        "click_action"=>"none",
        "click_dashboard"=> @dashboard.id.to_s,
        "keep_aspect_ratio"=>"1"
      }
    end

    should 'fail if the image is not really an image' do
      fake_image = fixture_file_upload('files/fake_image.png', 'image/png')
      widget = @widget.merge(media: fake_image)
      assert_no_difference('Widget.count') do
        post :create, dashboard_id: @dashboard.id, widget: widget
      end

      assert_response :found
    end

    should 'fail if the image type is not allowed' do
      image = fixture_file_upload('files/poc.html', 'text/html')
      widget = @widget.merge(media: image)
      assert_no_difference('Widget.count') do
        post :create, dashboard_id: @dashboard.id, widget: widget
      end

      assert_response :found
    end

    should 'succeed if the image is ok' do
      image = fixture_file_upload('files/image.png', 'image/png')
      widget = @widget.merge(media: image)
      assert_difference('Widget.count', 1) do
        post :create, dashboard_id: @dashboard.id, widget: widget
      end

      assert_redirected_to dashboards_path
    end
  end

  context 'A widget from a view' do
    setup do
      @user = users(:admin)
      sign_in @user

      @dashboard = dashboards(:admin_default)
      @widget = {
        "widget_type"=>"raw",
        "product_type"=>"flow",
        "graph_type"=>"serie",
        "filters"=>"{}",
        "granularity"=>"all",
        "serie_type"=>"",
        "range"=>"last_1_hour",
        "aggregation"=>"bps",
        "category"=>"",
        "click_action"=>"details",
        "transparent"=>false,
        "timezone"=>"auto",
        "analytics"=>[],
        "position_x"=>5,
        "position_y"=>2,
        "position_width"=>2,
        "position_height"=>2
      }
    end

    should 'redirect to dashboard on save & go' do
      assert_difference('Widget.count', 1) do
        post :create, commit: "Save and go to dashboard", f: true, dashboard_id: @dashboard.id, widget: @widget
      end
      assert_redirected_to dashboards_path
    end

    should 'save the widget' do
      assert_difference('Widget.count', 1) do
        post :create, commit: "Save", f: true, dashboard_id: @dashboard.id, widget: @widget, format: :js
      end
      assert_response :ok
    end
  end

  context 'An organization user' do
    setup do
      @user = users(:user_domain)
      @dashboard = dashboards(:admin_col_dashboard)

      sign_in @user
    end

    should 'not delete a widget from a dashboard without privileges' do
      widget = widgets(:admin_col_dashboard_w1)

      assert_no_difference 'Widget.count' do
        delete :destroy, dashboard_id: @dashboard.id, id: widget.id, format: :js
      end
    end
  end
end
