require 'test_helper'

#
# Test breadcrumb generation functionalities.
#
class BreadcrumbHelperTest < ActionView::TestCase
  # General functionality
  context 'Render sensor breadcrumb' do
    setup do
      @user = users(:admin_domain)
      User.current_user = @user
      @domain_sensor = sensors(:deployment)
      # This is a child of domain
      @organization_sensor = sensors(:organization)
      @market_sensor = sensors(:market)
    end

    # TODO: this test needs to be revised
    #should 'change the id on GET request' do
    #  html = sensor_breadcrumb(@market_sensor, 'root_path', true,
    #                           home: { name: 'Sensor', url: sensors_path },
    #                           route_id: 'sensor_id')
    #  assert(html.include?('?sensor_id='),
    #         'Breadcrumb must change the ID of GET request')
    #end

    should 'render home link (first element personalized)' do
      html = sensor_breadcrumb(@market_sensor, 'edit_sensor_path', true,
                               home: { name: 'Sensor', url: sensors_path })
      assert(html.include?('Sensor</a>'),
             'Breadcrumb must render a home link')
    end

    should 'add custom html options to all links' do
      html = sensor_breadcrumb(@market_sensor, 'edit_sensor_path', true,
                               home: { name: 'Sensor', url: sensors_path },
                               html_options: { remote: true })
      assert_equal(1, html.scan(/data-remote="true"/).size,
                   'Breadcrumb must add custom HTML attributes to links')
    end
  end

  # Create breadcrumb for admin
  context 'Superadmin' do
    setup do
      @user = users(:admin)
      User.current_user = @user
      @sensor = sensors(:root)
      @domain_sensor = sensors(:deployment)
      # This is a child of domain
      @organization_sensor = sensors(:organization)
      @market_sensor = sensors(:market)
      # Flow sensor
      @flow_sensor = sensors(:flow)
    end

    should 'don\'t raise any error when render a sensor breadcrumb' do
      assert_nothing_raised do
        sensor_breadcrumb(@sensor)
        sensor_breadcrumb(@domain_sensor, 'edit_sensor_path')
        sensor_breadcrumb(@organization_sensor, 'edit_sensor_path', true)
        sensor_breadcrumb(@market_sensor, 'edit_sensor_path', false,
                          home: { name: 'Sensor', url: sensors_path })
        sensor_breadcrumb(@flow_sensor, 'edit_sensor_path', true,
                          home: { name: 'Sensor', url: sensors_path },
                          route_id: 'sensor_id')
      end
    end
  end

  # Create a breadcrumb with an user with top domain at domain
  context 'Domain user' do
    setup do
      @user = users(:user_domain)
      User.current_user = @user
      @sensor = sensors(:root)
      @domain_sensor = sensors(:deployment)
      # This is a child of domain
      @organization_sensor = sensors(:organization)
      @market_sensor = sensors(:market)
      # Flow sensor
      @flow_sensor = sensors(:flow)
    end

    should 'don\'t raise any error when render a sensor breadcrumb' do
      assert_nothing_raised do
        sensor_breadcrumb(@domain_sensor, 'edit_sensor_path')
        sensor_breadcrumb(@organization_sensor, 'edit_sensor_path', true)
        sensor_breadcrumb(@market_sensor, 'edit_sensor_path', false,
                          home: { name: 'Sensor', url: sensors_path })
        sensor_breadcrumb(@flow_sensor, 'edit_sensor_path', true,
                          home: { name: 'Sensor', url: sensors_path },
                          route_id: 'sensor_id')
      end
    end

    should 'render the correct sensors based on permissions' do
      # He hasn't got permissions on root, so the breadcrumb only must have 1
      # element
      html = sensor_breadcrumb(@domain_sensor)
      # We need to add 1 to number of nodes because </ul> element
      assert_equal(1, html.split('</li>').size,
                   'Domain user only can\'t show root!')
      html = sensor_breadcrumb(@market_sensor, 'edit_sensor_path', false)
      assert_equal(2, html.split('</li>').size,
                   'Domain user only can\'t show root!')
      html = sensor_breadcrumb(@market_sensor, 'edit_sensor_path', false,
                               home: { name: 'Sensor', url: sensors_path })
      assert_equal(3, html.split('</li>').size,
                   'Domain user only can\'t show root, but breadcrumb must'\
                   ' render home!')
    end

    should 'don\'t render links because he hasn\'t got permissions' do
      # He hasn't got permissions on root, so the breadcrumb only must have 1
      # element
      html = sensor_breadcrumb(@market_sensor, 'edit_sensor_path', true)
      # We need to add 1 to number of nodes because </ul> element
      assert_equal(0, html.scan(/<\/a>/).size,
                   'Domain user without permissions can\'t show links!')
    end
  end
end
