require 'test_helper'

# Test for alerts
#
class AlertsTest < ActionDispatch::IntegrationTest
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'an admin user' do
    # Login admin
    setup do
      @user = users(:admin)
      # Login him
      login_as(@user, scope: :user)
    end

    should 'receive platform alerts' do
      visit '/'
      assert(
        page.has_css?('.alert-container.container .alert',
                      count: Alert.active_alerts.size),
                      'Wrong number of platform alerts displayed')
    end

    should 'create new alerts' do
      visit '/alerts'
      page.find('a[data-target="#new_alert_modal"]').click
      wait_for_ajax page, true
      fill_in('alert[message]', with: 'Creation test')
      # Hidden fields dont work with fill_in
      find('input#start_time_selector', visible: false).set(DateTime.now.to_s)
      find('input#end_time_selector', visible: false).set((DateTime.now + 1.day).to_s)

      click_button 'Create Alert', wait: 0.1
      page.has_content?('Creation test')
    end

    should 'edit alerts' do
      visit '/alerts'
      page.find('.btn i.fa.fa-list-alt', match: :first).click
      wait_for_ajax page
      fill_in('alert[message]', with: 'Edition test')

      click_button 'Update Alert', wait: 0.1
      page.has_content?('Edition test')
    end

    should 'delete alerts' do
      visit '/alerts'
      previous_alerts = Alert.all.size
      page.find('.btn i.fa.fa-trash', match: :first).click
      assert_equal(previous_alerts - 1, page.all('table tbody tr').size)
    end
  end

  context 'a regular user' do
    # Login admin
    setup do
      @user = users(:user1)
      # Login him
      login_as(@user, scope: :user)
    end

    should 'receive platform alerts' do
      visit '/'
      assert(
        page.has_css?('.alert-container.container .alert',
                      count: Alert.active_alerts.size),
        'Wrong number of platform alerts displayed')
    end

    should 'not create new alerts' do
      visit '/alerts'
      # assert(page.has_content?('You don\'t have enough privileges'))
      assert_equal('/', current_path)
    end

    should 'not edit alerts' do
      alert = Alert.first
      visit "/alerts/#{alert.id}/edit"
      # assert(page.has_content?('You don\'t have enough privileges'))
      assert_equal('/', current_path)
    end
  end
end
