require 'test_helper'

#
# Check the redBorder objects interface.
#
class RedborderObjectsTest < ActionDispatch::IntegrationTest
  # To login users
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'an user' do
    # Get first user
    setup do
      @user = users(:admin)
      # Login him
      login_as @user
      # Chef stub when update a net object. (See net_object_ips.rb)
      chef = ChefStub.new
      Chef::Node.stubs(:load).returns(chef)
      Chef::Role.stubs(:load).returns(chef)
      Chef::Node.stubs(:from_hash).returns(chef)
      Chef::Role.stubs(:from_hash).returns(chef)
    end

    should 'create a NetObject' do
      visit '/objects'
      click_link 'New Net object'
      wait_for_ajax(page, true)
      within('#new_object') do
        fill_in 'object[name]', with: net_object_params.name
        fill_in 'object[value]', with: net_object_params.value
        fill_in 'object[comment]', with: net_object_params.comment
      end
      click_button 'Create Net object', wait: 0.1
      # Check the object is created
      page.has_css?('#objects #net table td[data-label="Name"]',
                    text: net_object_params)
      assert(@user.net_objects.map(&:name).include?(net_object_params.name),
             'User must see his new object')
    end

    should 'add an error to form when leave name empty' do
      visit '/objects'
      click_link 'New Net object'
      wait_for_ajax(page, true)
      form_with_error('object', '#new_object', 'Create Net object', 'name') do
        fill_in 'object[value]', with: net_object_params.value
        fill_in 'object[comment]', with: net_object_params.comment
      end
    end

    should 'edit an existent object' do
      visit '/objects'
      new_name = 'NewName123sad12'
      obj = @user.net_objects.first
      css_path = "#objects table tbody tr[data-object-id='#{obj.id}'] "\
                 'a.edit-object'
      page.find(css_path).click
      wait_for_ajax(page, true)
      within('#edit_object') do
        fill_in 'object[name]', with: new_name
      end
      click_button 'Update Net object', wait: 0.1
      page.has_content? new_name
    end

    should 'destroy an object' do
      visit '/objects'
      obj = @user.net_objects.last
      # Check object exist
      page.has_css?("#objects table tbody tr[data-object-id='#{obj.id}'] "\
                    'td[data-label="Name"]')
      css_path = "#objects table tbody tr[data-object-id='#{obj.id}'] "\
                 'a.destroy-object'
      page.find(css_path).click
      # Check it's destroy
      wait_for_ajax(page, true)
      page.has_no_css?("#objects table tbody tr[data-object-id='#{obj.id}'] "\
                       'td[data-label="Name"]')
    end

    should 'search objects' do
      visit '/objects'
      search = @user.net_objects.first.name[0..3]
      count = @user.net_objects.where('name LIKE :search', search: search).count
      within '#net .objects-search form' do
        fill_in 'search', with: search
      end
      find('#net .objects-search form')
      page.execute_script("$('#net .objects-search form').submit()")
      wait_for_ajax(page)
      page.has_css?('#net .objects_content table tbody tr', count: count)
    end

    should 'navigate trought tabs' do
      visit '/objects'
      page.find('li#mac a').click
      mac_classes = page.find('li#mac')[:class]
      assert(mac_classes.include?('active'),
             'Mac tab must be active')
    end
  end

  #
  # Return params for a new object
  #
  def net_object_params
    @net_object ||= Struct.new(:name, :value, :comment).new(
      'new_net_object',
      '10.0.10.10',
      'My own object'
    )
  end
end
