require 'test_helper'

#
# Check the sessions.
#
class SessionsTest < ActionDispatch::IntegrationTest
  context 'When an user go to login page' do
    # Get first user
    setup do
      @admin = users(:admin)
      @user = users(:user_domain)
      @password = 'redborder'
    end

    should 'login with email' do
      visit '/users/login'
      within('#new_user') do
        fill_in 'user[username]', with: @admin.email
        fill_in 'user[password]', with: @password
      end
      click_button 'Sign in'
      assert_equal(dashboards_path, page.current_path,
                   'The login must redirect to user dashboard')
    end

    should 'login with username' do
      Capybara.reset_sessions!
      visit '/users/login'
      within('#new_user') do
        fill_in 'user[username]', with: @user.login
        fill_in 'user[password]', with: @password
      end
      click_button 'Sign in'
      assert_equal(dashboards_path, page.current_path,
                   'The login must redirect to user dashboard')
    end

    should 'Remember his password' do
      Capybara.reset_sessions!
      remember_with(@user.email)
      assert_equal(
        new_user_session_path, page.current_path,
        'When remember password the user must be redirected to login')
      page.has_selector?(
        '#flash_notice ul.clear',
        text: 'You will receive an email with instructions about how to'\
              'reset your password in a few minutes.')
    end

    should 'Show an error when remember a password of inexistent user' do
      Capybara.reset_sessions!
      remember_with('qwkjads@asdljkasd.com')
      assert_equal(
        user_password_path, page.current_path,
        "When error remember password the user musn't be redirected")
      page.has_selector?(
        '.alert.alert-danger',
        text: 'Email not found')
    end
  end

  def remember_with(email)
    visit '/users/login'
    click_link('Forgot Password?')
    within('#new_user') do
      fill_in 'user[email]', with: email
    end
    click_button 'Send'
  end
end
