require 'test_helper'

#
# Check the sessions.
#
class DashboardsTest < ActionDispatch::IntegrationTest
  # To login users
  include Warden::Test::Helpers
  Warden.test_mode!

  context 'A granularity all widget' do

    setup do
      @admin = users(:admin)
      # Login him
      login_as(@admin, scope: :user)
      @dashboard = dashboards(:admin_default)
    end

    should 'create a table widget from a module view if granularity is all' do
      visit '/flow/tops?granularity=all&range=last_1_hour'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'table', Widget.last.graph_type
    end

    should 'create a serie widget from a module view if granularity is not all' do
      visit '/flow/tops'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'multiserie', Widget.last.graph_type
    end

    should 'create a stream widget from the raw view if granularity is all' do
      visit '/flow?granularity=all&chart_type=streams'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'stream', Widget.last.graph_type
    end

    should 'create a stream widget from the raw view if granularity is not all' do
      visit '/flow?chart_type=streams'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'stream', Widget.last.graph_type
    end

    should 'create a table widget from the raw view if granularity is all and chart_type not stream' do
      visit '/flow?chart_type=line&granularity=all'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'table_raw', Widget.last.graph_type
    end

    should 'create a serie widget from the raw view if granularity is not all' do
      visit '/flow?chart_type=line'
      n_widgets = Widget.count
      create_widget_on @dashboard

      assert_equal n_widgets + 1, Widget.count
      assert_equal 'serie', Widget.last.graph_type
    end
  end

  def create_widget_on(dashboard)
    # Find filters button
    find('a', text: /Filters: \d+/, match: :first).click
    # Hover on create a widget on and click on a dashboard
    find('a', text: 'Create a widget on', match: :first).hover
    find('.dropdown-menu a', text: dashboard.name, match: :first).click
    # Wait for the modal
    wait_for_ajax(page, true)
    # Confirm widget creation
    find('.modal-dialog input[value="Save"]', match: :first).click
    wait_for_ajax(page, true)
  end
end