require 'test_helper'

#
# Alarm notificator send different notifications to users based on the options
# of the alarm
#
class AlarmNotificatorTest < ActiveSupport::TestCase
  context 'An alarm' do
    setup do
      @alarm = alarms(:one)
      @alarm2 = alarms(:two)
      # Stub a result
      @result = {
        where: :upper,
        value: 1500,
        fired: true,
        end_time: Time.now,
        start_time: Time.now - 1.hour
      }
      @notificator = Redborder::AlarmNotificator.new(@alarm, @result)
    end

    should 'can notify user by email' do
      m = mail(@alarm)
      AlarmMailer.expects(:alarm).with(@alarm, @result).returns(m)
      @notificator.notify_email
    end

    should 'can notify user by Slack' do
      assert @notificator.respond_to? :notify_slack
      @notificator.expects(:notify_slack).once
    end

    should 'can notify user by push' do
      Redborder::MobilePush.expects(:send_notification).once
      @notificator.notify_mobile
    end

    should 'can notify user by syslog' do
      assert @notificator.respond_to? :notify_syslog
    end

    should 'notify user with all methods' do
      m = mail(@alarm)
      AlarmMailer.expects(:alarm).with(@alarm, @result).returns(m)
      Redborder::MobilePush.expects(:send_notification).once
      @notificator.expects(:notify_syslog).once
      # Notify user
      @notificator.notify
    end

    should 'notify only with email and syslog' do
      m = mail(@alarm2)
      AlarmMailer.expects(:alarm).with(@alarm2, @result).returns(m)
      Redborder::MobilePush.expects(:send_notification).never
      notificator = Redborder::AlarmNotificator.new(@alarm2, @result)
      notificator.expects(:notify_syslog).once
      # Notify user
      notificator.notify
    end
  end

  # AlarmMailer mail to send
  def mail(alarm)
    AlarmMailer.alarm(alarm, @result)
  end
end
