require 'test_helper'

#
# As is a library to perform queries to Aerospike. We need to format results.
#
class Redborder::As::CacheTest < ActiveSupport::TestCase
  include Aerospike

  #
  # Order an Aerospike response
  #
  context 'An Aerospike query' do
    setup do
      # Main class
      @as = Redborder::As::Base.new
      @as2 = Redborder::As::Base.new
      @stmt = Aerospike::Statement.new(AerospikeTestQuery::NAMESPACE,
                                       AerospikeTestQuery::SET, [])
      @stmt2 = Aerospike::Statement.new(AerospikeFailTestQuery::NAMESPACE,
                                        AerospikeFailTestQuery::SET, [])
    end

    should 'be cached' do
      Rails.cache.clear
      # Apply the order
      res = @as.query(AerospikeTestQuery).raw
      # Read cache!
      cached = Rails.cache.read("rb-webui:aerospike:#{@as.statement_id(@stmt)}")
      key = Rails.cache.exist?(
        "rb-webui:aerospike:#{@as.statement_id(@stmt)}")

      assert(key, 'Result must be cached')
      assert_equal(res, cached, 'Expected result will be cached')
    end

    should 'not be cached if Aerospike query fails' do
      Rails.cache.clear
      # Update servers
      back_servers = RedBorder::AEROSPIKE_CONFIG[:servers]
      RedBorder::AEROSPIKE_CONFIG[:servers] = ['179.1.2.3:9999']

      # Apply the order
      res = @as2.query(AerospikeFailTestQuery).raw
      # Read cache!
      cached = Rails.cache.exist?(
        "rb-webui:aerospike:#{@as2.statement_id(@stmt2)}")

      assert_equal([], res, 'By default, it returns an empty array when
                            query fails')
      assert(!cached, 'Result must not be cached')

      # Restore servers
      RedBorder::AEROSPIKE_CONFIG[:servers] = back_servers
    end
  end
end
