require 'test_helper'

#
# As is a library to perform queries to Aerospike. We need to order results.
#
class Redborder::As::QueryTest < ActiveSupport::TestCase
  #
  # Order an Aerospike response
  #
  context 'An Aerospike query' do
    setup do
      Rails.cache.clear
      # Main class
      @as = Redborder::As::Base.new
      # This data must be created on a aerospike server in the Runner machine
      @key = 'TestKey'
      @column = 'test_column'
      @value = 'Testing value'
      @value2 = 'Testing value 2'
      # To create a new element
      @new_key = 'TestKey12312'
      @new_value = 'Testing value 12312'
      # Return all elements but get only test_column
      @res_query = [
        { @column => @value },
        { @column => @value2 }
      ]
      # We return all elements with all columns
      @res_query_all = [
        { @column => @value,
          'score' => 1 },
        { @column => @value2,
          'score' => 2 }
      ]
      # In this case, we apply a filter for score [2-10], so first
      # element musn't show
      @res_with_filter = [
        { @column => @value2,
          'score' => 2 }
      ]
    end

    should 'get a value from a key' do
      # Get from key
      result = @as.get(AerospikeTestQuery, @key).raw
      assert_equal(@value, result[@column],
                   "The value from Aerospike doesn't coincide.")
    end

    # This test is commented because a bug on Aerospike gem.
    #
    # should 'query a value with only one column' do
    #   # Get from a query with only test_column dimension
    #   result = @as.query(AerospikeTestQuery, ['test_column']).raw
    #   assert_equal(@res_query, result,
    #                "The value from Aerospike doesn't coincide.")
    # end

    should 'query a value with all columns' do
      # Get from a query
      result = @as.query(AerospikeTestQuery).raw
      assert_equal(@res_query_all, result,
                   "The value from Aerospike doesn't coincide.")
    end

    should 'apply a filter' do
      # Apply a filter to only return testing value 2
      filter = Aerospike::Filter.Range('score', 2, 10)
      result = @as.query(AerospikeTestQuery, [], [filter]).raw
      assert_equal(@res_with_filter, result,
                   'We must return only second element because we'\
                   'applied a filter')
    end

    should 'check if the key exists' do
      assert_respond_to(@as, :exists?)
      assert(@as.exists?(AerospikeTestQuery, @key),
             "#{@key} key exists in Aerospike!")
    end

    should 'put a new key -> value in a set and delete it' do
      assert_respond_to(@as, :put)
      assert_nothing_raised do
        @as.put(AerospikeTestQuery, @new_key, { @column => @new_value })
      end
      assert(@as.exists?(AerospikeTestQuery, @new_key),
             "#{@new_key} key must exists in Aerospike!")
      assert_respond_to(@as, :delete)
      assert_nothing_raised { @as.delete(AerospikeTestQuery, @new_key) }
      assert_not(@as.exists?(AerospikeTestQuery, @new_key),
                  "#{@new_key} key has been deleted in Aerospike!")
    end
  end
end
