require 'test_helper'

#
# DataSources are the models to retrieve the data from Druid. All DataSource
# must implements an interface methods defined on DruidSource. We must test it.
#
class ChefRoleBuilderTest < ActiveSupport::TestCase
  context 'Creating a sensor' do
    setup do
      @org = sensors(:organization)
    end

    should 'initialize override_attributes by default' do
      builder = Redborder::Chef::Role::Builder.new(@org, ['role[my-sensor]'])
      assert builder.override_attributes.present?
      assert builder.override_attributes.keys.include? 'redborder'
    end

    should 'add domain info to override_attributes' do
      builder = Redborder::Chef::Role::Builder.new(@org, ['role[my-sensor]'], domain: true)
      # Check keys are added
      d = @org.domain_type_name to_key: true
      assert builder.override_attributes['redborder'].keys.include?(d.to_s)
      assert_equal @org.name, builder.override_attributes['redborder'][d.to_s]
      assert builder.override_attributes['redborder'].keys.include?("#{d}_uuid")
      assert_equal @org.uuid, builder.override_attributes['redborder']["#{d}_uuid"]
    end

    should 'create chef role' do
      chef = ChefStub.new
      Chef::Role.stubs(:from_hash).returns(chef)
      builder = Redborder::Chef::Role::Builder.new(@org, ['role[my-sensor]'], domain: true)
      # Check create
      assert builder.create
    end

    should 'Override default attributes' do
      chef = ChefStub.new
      Chef::Role.stubs(:from_hash).returns(chef)
      my_attributes = { 'redborder' => { 'role' => 'my_role' } }
      builder = Redborder::Chef::Role::Builder.new(
        @org, ['role[my-sensor]'], override_attributes: my_attributes)
      # Check attributes is added
      assert_equal(
        my_attributes.merge!('redborder' => { 'sensor_uuid' => @org.uuid }),
        builder.override_attributes
      )
    end

    should 'ensure redborder key is always present' do
      chef = ChefStub.new
      Chef::Role.stubs(:from_hash).returns(chef)
      my_attributes = { 'another' => {} }
      builder = Redborder::Chef::Role::Builder.new(
        @org, ['role[my-sensor]'], override_attributes: my_attributes)
      # Check attributes is added
      assert(
        builder.override_attributes.keys.include?('redborder'),
        'redborder key must be always present'
      )
    end
  end
end
