require 'test_helper'

#
# DataSources are the models to retrieve the data from Druid. All DataSource
# must implements an interface methods defined on DruidSource. We must test it.
#
class DruidSourceTest < ActiveSupport::TestCase
  #
  # All dataSources must implements the interface methods.
  #
  should 'Implements the interface methods' do
    # Test every module with a DataSource
     
    # Modules that fails
    modules_f = []

    Redborder::Module.registered_modules.each do |_, m|
      next if m[:data_source].nil?
      source = m[:data_source].constantize

      # Check all methods. This methods will throw an exception if not defined
      begin
        source.controller
        source.id
        source.default_aggregation
        source.sensors
      rescue NotImplementedError
        modules_f << m[:data_source]
      end
    end

    # If some module fails, assert false
    assert(modules_f.empty?, "The DataSource #{modules_f.join(', ')} must " \
                             'implement DruidSource interface.')
  end
end
