require 'json'

#
# Generate random data
#
module Randomizer
  # Load signatures
  def load_signatures
    @signatures = JSON.parse(
      File.read(
        Rails.root.join('test', 'mocha', 'druid', 'data', 'signatures.json')
      )
    )
  end

  #
  # Return a random integer between [1000000, 1150000]
  #
  def random_integer
    1_000_000 + rand(150_000)
  end

  #
  # Return a random float
  #
  def random_float
    100_000.to_f / rand(20...50)
  end

  #
  # Return a random string with 6-10 length
  #
  def random_string
    length = 6 + rand(4)
    [*('A'..'Z')].sample(length).join
  end

  #
  # Return a random IP
  #
  def random_ip
    Array.new(4) { rand(256) }.join('.')
  end

  #
  # Return a random MAC address
  #
  def random_mac
    (1..6).map { "%0.2X"%rand(256) }.join(':')
  end

  #
  # Random signature for an IPS event
  #
  def random_signature
    load_signatures if @signatures.nil?
    @signatures[rand(@signatures.size) - 1]
  end
end
