require 'test_helper'

#
# Tier class tests
#
class TierTest < ActiveSupport::TestCase
  should validate_presence_of(:name)
  should validate_uniqueness_of(:name)
  should validate_presence_of(:user_id)

  context Tier do
    setup do
      @user = users(:user_domain)
      @user_admin = users(:admin_domain)
      @super_admin = users(:admin)
      @default = tiers(:default)
      @hot = tiers(:hot)
      @default_real = tiers(:default_real)

      chef = ChefStub.new
      Chef::Node.stubs(:load).returns(chef)
    end

    should 'check user is super admin when create a Tier' do
      tier = Tier.new(name: 'NewTier', user: @user)
      assert_not tier.valid?, 'A normal user can\'t create a Tier'
      tier.user = @user_admin
      assert_not(tier.valid?,
                 "An admin user (not super admin) can't create a Tier")
    end

    should "can't destroy default Tier" do
      assert_not(@default.destroy, 'Default Tier can not be destroyed')
    end

    should 'check the type' do
      assert_equal(true, @default.default?)
      assert_equal(true, @hot.default_hot?)
      assert_equal(true, @default.protected?)
      assert_equal(true, @hot.protected?)
    end

    should 'get tier historicals' do
      assert_not_nil(@default.historicals)
    end

    should 'get default tier name' do
      assert_equal(Tier::DEFAULT_TIER_NAME, @default_real.default_tier_name)
      assert_equal(@default.name, @default.default_tier_name)
    end

    should 'get chef tier nodes' do
      assert_instance_of(Array, @default.chef_tier_nodes)
    end

    should 'set a default tier name' do
      assert_equal(Tier::DEFAULT_TIER_NAME, Tier.default_tier_name(Tier::REAL_DEFAULT_TIER_NAME))
      assert_equal('NewName', Tier.default_tier_name('NewName'))
    end

    should 'get chef historical tiers and its names' do
      assert_not_nil(Tier.chef_historical_tiers)
      assert_not_nil(Tier.chef_historical_tiers_names)
    end

    should 'update nodes after update itself' do
      @default.update(name: 'Test')
      assert_equal('Test', @default.reload.name)
    end
  end

  context 'super admin user' do
    should 'create a new tier' do
      assert(
        Tier.create(name: 'newtier', user: @super_admin),
        'A super admin can create new tiers')
    end
  end
end
