#
# Added some helpers to work with API tests
#
module ApiSupport
  #
  # Parse the JSON of the response. This method check if the JSON is
  # wellformed and if query value is present and it's true or false.
  #
  # == Paremeters:
  # body::
  #   String with the repsonse body to parse as JSON
  # status::
  #   Boolean indicating the query status
  #
  # == Returns:
  # Hash of parsed JSON
  #
  def api_json(body, status)
    # Check if the current JSON is valid
    check_json(body)
    # Check query value
    json = JSON.parse(body)
    assert(json.keys.include?('query'), 'Query must be always in the response')
    if status
      assert(json['query'], 'Query value must be true')
    else
      assert_not(json['query'], 'Query value must be false')
    end
    # Return initialized JSON
    json
  end

  #
  # This method is an alias for api_json that initialize status to true
  #
  # == Paremeters:
  # body::
  #   String with the repsonse body to parse as JSON
  #
  # == Returns:
  # Hash of parsed JSON
  #
  def true_api_json(body)
    api_json(body, true)
  end

  #
  # This method is an alias for api_json that initialize status to false
  #
  # == Paremeters:
  # body::
  #   String with the repsonse body to parse as JSON
  #
  # == Returns:
  # Hash of parsed JSON
  #
  def false_api_json(body)
    api_json(body, false)
  end

  #
  # Check if body is a valid string to be parsed as JSON
  #
  # == Parameters:
  # body::
  #   String to be parsed
  #
  def check_json(body)
    assert_nothing_raised JSON::ParserError do
      JSON.parse(body)
    end
  end
end
