#
# Helper methods to create licenses
#
module LicensesSupport
  RedBorder::LICENSE_PRIVATE_KEY_PATH = "#{Rails.root}/config/license.key".freeze

  #
  # Create a license content
  #
  # == Parameter:
  # uuid::
  #   License identification
  # cluster_uuid::
  #   Cluster's UUID
  # organization_uuid::
  #   Organization's UUID. If empty, the content is a global license.
  # expire_at::
  #   Timestamp representing the license's expiration.
  # limit_bytes::
  #   Integer representing the limitation of bytes received.
  #
  # == Returns:
  # License content.
  #
  def create_license(uuid: SecureRandom.uuid, cluster_uuid: Setting.cluster_uuid,
                     organization_uuid: nil, expire_at: (Time.now + 1.year).to_i,
                     limit_bytes: 1_000_000)
    key = OpenSSL::PKey::RSA.new(File.read(RedBorder::LICENSE_PRIVATE_KEY_PATH))
    digest = OpenSSL::Digest::SHA256.new

    info = {
      uuid: uuid,
      cluster_uuid: cluster_uuid,
      expire_at: expire_at,
      limit_bytes: limit_bytes
    }

    organization_uuid && info[:organization_uuid] = organization_uuid

    encoded_info = Base64.urlsafe_encode64(info.to_json)

    license = {
      encoded_info: encoded_info,
      signature: Base64.urlsafe_encode64(key.sign(digest, encoded_info))
    }

    Base64.urlsafe_encode64(license.to_json)
  end
end
