# SimpleCov configuration
require 'simplecov'
SimpleCov.start 'rails'

require "minitest/reporters"
Minitest::Reporters.use! [Minitest::Reporters::SpecReporter.new]

# Base initialization
ENV['RAILS_ENV'] = 'test'
ENV['DITTOCODE_ENV'] ||= 'ENTERPRISE'
require File.expand_path('../../config/environment', __FILE__)
# Require support files
Dir["#{File.dirname(__FILE__)}/support/*.rb"].each { |file| require file }

require 'rails/test_help'
require 'pry'
require 'capybara/rails'
require 'capybara/poltergeist'

# Require all mocks
require 'test/unit'
require 'minitest/autorun'
require 'mocha/minitest'

Shoulda::Matchers.configure do |config|
  config.integrate do |with|
    with.test_framework :minitest
    with.library :rails
  end
end

Dir["#{File.dirname(__FILE__)}/mocha/**/*.rb"].each { |file| require file }

phantomjs_options =
  %w(--load-images=no --ignore-ssl-errors=yes --ssl-protocol=any
     --proxy-type=none)

# Define driver
Capybara.server = :webrick
Capybara.register_driver :poltergeist do |app|
  Capybara::Poltergeist::Driver.new(app,
                                    timeout: 3.minutes,
                                    window_size: [1280, 768],
                                    js_errors: false,
                                    phantomjs_options: phantomjs_options)
end

# Mobile driver
Capybara.register_driver :mobile do |app|
  Capybara::Poltergeist::Driver.new(app,
                                    timeout: 3.minutes,
                                    window_size: [480, 768],
                                    js_errors: false)
end

Capybara.default_driver = :poltergeist
Capybara.current_driver = :poltergeist
Capybara.javascript_driver = :poltergeist
# Increase wait time for capybara (some tests fail)
Capybara.default_max_wait_time = 10

class ActiveSupport::TestCase
  ActiveRecord::Migration.check_pending!
  include Capybara::DSL
  include ModelsSupport
  include LicensesSupport

  # Setup all fixtures in test/fixtures/*.yml for all tests
  # in alphabetical order. Note: You'll currently still have to
  # declare fixtures explicitly in integration tests
  # -- they do not yet inherit this setting
  fixtures :all

  def setup
    Setting.stubs(:cluster_uuid).returns(SecureRandom.uuid)
    chef = ChefStub.new
    Chef::Node.stubs(:load).returns(chef)
    Chef::Role.stubs(:load).returns(chef)
    Chef::ApiClient.stubs(:load).returns(chef)
    Chef::Node.stubs(:from_hash).returns(chef)
    Chef::Role.stubs(:from_hash).returns(chef)
    Chef::ApiClient.stubs(:from_hash).returns(chef)
    data_bag = DataBagStub.new('licenses' => {}, 'sensors' => {})
    Chef::DataBagItem.stubs(:load).returns(data_bag)
  end

  # Add more helper methods to be used by all tests here...
  def teardown
    Capybara.reset_sessions!
    page.driver.reset!
    Capybara.use_default_driver
  end
end

class ActionController::TestCase
  include Devise::TestHelpers
  include LicensesSupport

  def setup
    Setting.stubs(:cluster_uuid).returns(SecureRandom.uuid)
    chef = ChefStub.new
    Chef::Node.stubs(:load).returns(chef)
    Chef::Role.stubs(:load).returns(chef)
    Chef::ApiClient.stubs(:load).returns(chef)
    Chef::Node.stubs(:from_hash).returns(chef)
    Chef::Role.stubs(:from_hash).returns(chef)
    Chef::ApiClient.stubs(:from_hash).returns(chef)
    data_bag = DataBagStub.new('licenses' => {}, 'sensors' => {})
    Chef::DataBagItem.stubs(:load).returns(data_bag)
  end
end

class ActionDispatch::IntegrationTest
  # Make the Capybara DSL available in all integration tests
  include Capybara::DSL
  include IntegrationSupport
  include ApiSupport

  def setup
    # Mock
    chef = ChefStub.new
    Chef::Node.stubs(:load).returns(chef)
    Chef::Role.stubs(:load).returns(chef)
    Chef::ApiClient.stubs(:load).returns(chef)
    Chef::Node.stubs(:from_hash).returns(chef)
    Chef::Role.stubs(:from_hash).returns(chef)
    Chef::ApiClient.stubs(:from_hash).returns(chef)
    data_bag = DataBagStub.new('licenses' => {}, 'sensors' => {})
    Chef::DataBagItem.stubs(:load).returns(data_bag)
  end

  Mocha::Mockery.setup

  Setting.stubs(:cluster_uuid).returns(SecureRandom.uuid)

  # Load all fixtures for integration tests
  fixtures :all
end
