#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

# Load Default Classfications
Classification.find_or_create_by(name: "Unauthorized Root Access", description: 'Unauthorized Root Access', hotkey: 1, locked: true)
Classification.find_or_create_by(name: "Unauthorized User Access", description: 'Unauthorized User Access', hotkey: 2, locked: true)
Classification.find_or_create_by(name: "Attempted Unauthorized Access", description: 'Attempted Unauthorized Access', hotkey: 3, locked: true)
Classification.find_or_create_by(name: "Denial of Service Attack", description: 'Denial of Service Attack', hotkey: 4, locked: true)
Classification.find_or_create_by(name: "Policy Violation", description: 'Policy Violation', hotkey: 5, locked: true)
Classification.find_or_create_by(name: "Reconnaissance", description: 'Reconnaissance', hotkey: 6, locked: true)
Classification.find_or_create_by(name: "Virus Infection", description: 'Virus Infection', hotkey: 7, locked: true)
Classification.find_or_create_by(name: "False Positive", description: 'False Positive', hotkey: 8, locked: true)
Classification.find_or_create_by(name: "Exploit", description: 'Exploit', locked: true)
Classification.find_or_create_by(name: "Malware", description: 'Malware', locked: true)

# Load Default Action Rules
Action.where(name: 'pass').first_or_create(description: 'Ignore the packet', selectable: true)
Action.where(name: 'alert').first_or_create(description: 'Generate an alert using the selected alert method, and then log the packet', selectable: true)
Action.where(name: 'drop').first_or_create(description: 'Block and log the packet', selectable: true)
Action.where(name: 'sdrop').first_or_create(description: 'Block the packet but do not log it', selectable: true)
Action.where(name: 'clear').first_or_create(description: 'This rule will be removed.', selectable: true)
Action.where(name: 'should_drop').first_or_create(description: 'Should be dropped.', selectable: false)
Action.where(name: 'cant_drop').first_or_create(description: 'Cannot be dropped.', selectable: false)

if Flowbit.all.blank?
  Flowbit.create(name: 'set', description: 'Sets the specified state for the current flow and unsets all the other flowbits in a group when a GROUP_NAME is specified.')
  Flowbit.create(name: 'unset', description: 'Unsets the specified state for the current flow.')
  Flowbit.create(name: 'toggle', description: 'Sets the specified state if the state is unset and unsets all the other flowbits in a group when a GROUP_NAME is specified, otherwise unsets the state if the state is set.')
  Flowbit.create(name: 'isset', description: 'Checks if the specified state is set.')
  Flowbit.create(name: 'isnotset', description: 'Checks if the specified state is not set.')
  Flowbit.create(name: 'noalert', description: 'Cause the rule to not generate an alert, regardless of the rest of the detection options.')
  Flowbit.create(name: 'reset', description: 'Reset all states on a given flow.')
end

if FactoryPolicy.all.blank?
  FactoryPolicy.create(name: 'connectivity-ips', description: "This is a speedy policy for people that insist on blocking only the really known bad with no false positives.")
  FactoryPolicy.create(name: 'balanced-ips'    , description: "This is a good starter policy for everyone. It's quick, has a good base coverage level, and covers the latest threats of the day. This policy contains everything that is in the connectivity-ips policy")
  FactoryPolicy.create(name: 'security-ips'    , description: "This is a stringent policy that everyone should strive to get to through tuning. It's quick, but has some policy-type rules in it. This policy contains everything that is in the security-ips policy.")
end

if Category.all.blank?
  Category.create(name: 'attacks', description: 'Attack Execution or Backtrace Rules')
  Category.create(name: 'files', description: 'Vulnerabilities present inside of files')
  Category.create(name: 'malware', description: 'Rules that are malware related')
  Category.create(name: 'os', description: 'Vulnerabilities in OS')
  Category.create(name: 'policy', description: 'Enterprise Policy Violation Rules')
  Category.create(name: 'preprocessors', description: "Preprocessor (Snort Internal's) Rules")
  Category.create(name: 'servers', description: 'Vulnerabilities in or attacks against servers')
  Category.create(name: 'services', description: 'Other Services Rules')
  Category.create(name: 'special', description: 'External Resources Rules')
  Category.create(name: 'web', description: 'Web related rules')
  Category.create(name: 'file_capture', description: 'File Capture Rules')
  Category.create(name: 'others', description: 'Other Rules')
  #Category.create(name: 'dos', description: 'Denial Of Service Rules')
  #Category.create(name: 'email', description: 'Email Service Rules')
  #Category.create(name: 'sql', description: 'Database Service Rules')
end

new_vrt_rule = false
vrt = RuleSource.where(name: "vrt").first_or_create(description: "Sourcefire Vulnerability Research Team",
                                              protected: true, 
                                              tgzurl: "https://repo.redborder.com/rules/vrt.tgz", 
                                              md5url: "https://repo.redborder.com/rules/vrt.tgz.md5", 
                                              search: ["rules", "preproc_rules", "so_rules"], 
                                              so_rules: ["so_rules/precompiled/RHEL-6-0/x86-64"], 
                                              has_code: false, 
                                              enable: true, 
                                              sensor: Sensor.root) do |obj|
                                                new_vrt_rule = true
                                              end
                                              
unless new_vrt_rule
  vrt.update({description: 'Sourcefire Vulnerability Research Team', has_code: false, tgzurl: "https://repo.redborder.com/rules/vrt.tgz", md5url: "https://repo.redborder.com/rules/vrt.tgz.md5"})
end

RuleSource.where(name: "etpro").first_or_create(description: "Emerging Threats PRO rules", protected: true, tgzurl: "http://rules.emergingthreats.net/$$code$$/snort-2.9.8/etpro.rules.tar.gz", md5url: "http://rules.emergingthreats.net/$$code$$/snort-2.9.8/etpro.rules.tar.gz.md5", regexp_family_ignore: "^et |^gpl |^etpro ", search: ["rules"], so_rules: [], has_code: true, enable: false, sensor: Sensor.root)
RuleSource.where(name: "emergingthreats").first_or_create(description: "Emerging Threats community rules", protected: true, tgzurl: "http://rules.emergingthreats.net/open/snort-2.9.8/emerging.rules.tar.gz", md5url: "http://rules.emergingthreats.net/open/snort-2.9.8/emerging.rules.tar.gz.md5", regexp_group_ignore: "emerging-", regexp_family_ignore: "^et |^gpl ", search: ["rules"], so_rules: [], enable: true, sensor: Sensor.root)

#References
RbReference.where(name: 'bugtraq').first_or_create(humanize: "Bugtraq", prefix: 'http://www.securityfocus.com/bid/')
RbReference.where(name: 'cve').first_or_create(humanize: "CVE", prefix: 'http://cve.mitre.org/cgi-bin/cvename.cgi?name=')
RbReference.where(name: 'nessus').first_or_create(humanize: "Nessus", prefix: 'http://cgi.nessus.org/plugins/dump.php3?id=')
RbReference.where(name: 'url').first_or_create(humanize: "URL", prefix: 'http://')
RbReference.where(name: 'arachnids').first_or_create(humanize: "arachNIDS", prefix: 'http://www.whitehats.com/info/IDS')
RbReference.where(name: 'mcafee').first_or_create(humanize: "McAfee", prefix: 'http://vil.nai.com/vil/content/v_')
RbReference.where(name: 'osvdb').first_or_create(humanize: "OSVDB", prefix: 'http://osvdb.org/show/osvdb/')
RbReference.where(name: 'bid').first_or_create(humanize: "BID", prefix: 'http://www.securityfocus.com/bid/')
RbReference.where(name: 'md5').first_or_create(humanize: "MD5", prefix: 'http://www.threatexpert.com/report.aspx?md5=')
RbReference.where(name: 'secunia').first_or_create(humanize: "Secunia", prefix: 'http://www.secunia.com/advisories/')
RbReference.where(name: 'msb').first_or_create(humanize: "Microsoft Security Bulletin", prefix: 'http://technet.microsoft.com/en-us/security/bulletin/')
RbReference.where(name: 'et').first_or_create(humanize: "ET", prefix: 'http://doc.emergingthreats.net/')
RbReference.where(name: 'etpro').first_or_create(humanize: "ETPro", prefix: 'http://doc.emergingthreatspro.com/')
RbReference.where(name: 'telus').first_or_create(humanize: "Telus", prefix: 'http://')
RbReference.where(name: 'threatexpert').first_or_create(humanize: "ThreatExpert", prefix: 'http://www.threatexpert.com/report.aspx?md5=')
RbReference.where(name: 'exploitdb').first_or_create(humanize: "The Exploit Database", prefix: 'http://www.exploit-db.com/exploits/')
RbReference.where(name: 'openpacket').first_or_create(humanize: "OpenPacket", prefix: 'https://www.openpacket.org/capture/grab/')
RbReference.where(name: 'securitytracker').first_or_create(humanize: "SecurityTracker", prefix: 'http://securitytracker.com/id?')
RbReference.where(name: 'xforce').first_or_create(humanize: "X-Force", prefix: 'http://xforce.iss.net/xforce/xfdb/')
RbReference.where(name: 'msft').first_or_create(humanize: "Microsoft TechNet", prefix: 'http://technet.microsoft.com/security/bulletin/')

# Default settings
Setting.find_or_create_by(name: 'signature_lookup', value: 'http://www.snort.org/rule_docs/$$sid$$')
Setting.find_or_create_by(name: 'rule_update_periodicity', value: 'daily')
Setting.find_or_create_by(name: 'rule_update_time', value: '00:00')

# Default alarms
user = User.find_by(login: 'admin', admin: true) or User.first
Alarm.find_or_create_by(name: "Security: Darklist Category not clean detected on Intrussion event", condition: "{\"0\":{\"column\":\"darklist_category\",\"operator\":\"is_not\",\"value\":\"clean\",\"enabled\":true},\"1\":{\"column\":\"darklist_category\",\"operator\":\"not_empty\",\"value\":\"\",\"enabled\":true}}", enabled: true, interval: 3600, product_type: "ips", upper_limit: 1, time_window: 300, upper_limit_enabled: true, user_id: user.id, timezone: "UTC", aggregation: "events", multiplier_upper: 0, severity_id: 6, notify_to_syslog: false, notify_to_email: true, notify_to_slack: false, email_list: user.email)
Alarm.find_or_create_by(name: "Security: Intrusion event classified as Trojan detected", condition: "{\"0\":{\"column\":\"classification\",\"operator\":\"contains\",\"value\":\"Trojan\",\"enabled\":true}}", enabled: true, interval: 3600, product_type: "ips", upper_limit: 1, time_window: 300, upper_limit_enabled: true, user_id: user.id, timezone: "UTC", aggregation: "events", multiplier_upper: 0, severity_id: 5, notify_to_syslog: false, notify_to_email: true, notify_to_slack: false, email_list: user.email)


# not_activated = %w(
#   ips_darklist_score_name ips_darklist_category
#   ips_darklist_protocol ips_darklist_direction)

not_activated = %w( )

Redborder::Module.get(:ips, :columns).keys.each do |col|
  col = "ips_#{col}"
  value = not_activated.include?(col) ? '0' : '1'
  Setting.where(name: col).first_or_create(name: col, value: value)
end

Redborder::Module.get(:ips, :overview_widgets).each do |w|
  setting_name = "overview_#{w[:partial]}"
  Setting.where(name: setting_name).first_or_create(name: setting_name, value: '1')
end
