#######################################################################
# Copyright (c) 2014 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

module Redborder

  module Jobs

    class RbVulnerabilitiesUpdateDBJob < Struct.new(:user_id, :verbose)

      include Redborder::Jobs::JobHelper
      attr_accessor :content

      # Main job execution method
      # This method performs the task of updating the vulnerabilities database.
      # It initializes content with the hostname, executes the update script,
      # checks if the database has been created, and handles any exceptions that occur.
      def perform
        mongodb_service = %x(ping -c 1 mongodb.service | awk '{print $3}' | head -n 1 | sed 's/[()]//g').strip
        node_name = %x(consul members | grep #{mongodb_service} | awk '{print $1}').strip
        # Initialize content with the hostname of the node executing the job
        content = "Node executing this job: #{node_name} \n\n"
        # Execute the vulnerability update script and append the output to content
        update_script_command = "/usr/lib/redborder/bin/rb_manager_utils.sh -e -n #{mongodb_service} -s /usr/lib/redborder/scripts/rb_vulnerability_load_cvedb.rb"
        content += %x(#{update_script_command})

        # Check if the CVE database has been created
        response = %x(/usr/lib/redborder/bin/rb_manager_utils.sh -e -n #{mongodb_service} -s "[ -f /tmp/rb_vulnerability_load_cvedb_last_update ] && printf true || printf false")
        check_last_update = "true" == response

        # Assign the first user's ID if user_id is nil
        user_id = User.first.id if user_id.nil? rescue nil

        # Determine if the job is ready to proceed
        is_job_ready = check_last_update && !user_id.nil?
        unless is_job_ready
          raise I18n.t('lib_redborder.vulnerabilities_update_failed')
        end
      rescue Exception => e
        # Handle any exceptions, log the error, and set the job as not ready
        is_job_ready = false
        e.backtrace.each do |line|
          content += "#{line}\n"
        end
        e.set_backtrace([])
        logit "#{e.class}: #{e.message}" if verbose
        raise e
      ensure
        StoredDelayedJob.create(job: "RbVulnerabilitiesUpdateDBJob", user_id: user_id, content: content)
      end

      def reschedule_at(current_time, attempts)
        if Setting.vulnerability_update_periodicity == "weekly"
          hour, minute = Setting.vulnerability_update_time.split(":").map(&:to_i)
          current_time.beginning_of_week.advance(weeks: 1, hours: hour, minutes: minute)
        elsif Setting.vulnerability_update_periodicity == "daily"
          hour, minute = Setting.vulnerability_update_time.split(":").map(&:to_i)
          current_time.beginning_of_day.advance(days: 1, hours: hour, minutes: minute)
        else
          current_time.advance(hours: 1)
        end
      end
    end
  end
end
